/* CONFIDENTIAL -- Copyright 20011 Intuit Inc. This material contains certain  *
 * trade secrets and confidential and proprietary information of Intuit Inc.  *
 * Use, reproduction, disclosure and distribution by any means are prohibited,*
 * except pursuant to a written license from Intuit Inc. Use of copyright     *
 * notice is precautionary and does not imply publication or disclosure.      *
 */
package com.intuit.sb.cdm.util;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.xml.bind.DatatypeConverter;

/**
 * Adapter for JAXB to bind xs:date type to java.util.Date rather than the
 * default XMLGregorianCalendar.
 */
public class IDSDateAdapter 
{
    private final static String INVALIDDATE = "9999-12-31";

    /**
     * Unmarshal a Date.
     *
     * @param value String from which to unmarshal.
     * @return the unmarshalled Date object
     */
    public static Date parseStringToDate( String value )
    {
        String datePattern =
            "(19|20)\\d\\d([- /.])(0[1-9]|1[012])\\2(0[1-9]|[12][0-9]|3[01])";
        if ( value != null )
        {
            // 16 is the format for 1999-11-11-8:00, it is JAXB format for the
            // date.
            if ( value.length() >= 10 )
            {
                // To get 1999-11-11 to validate the date format.
				/* Always Returning JAXB binding for DateTime because the new IDSDate can handle both date + time
              // value = value.substring( 0, 10 );
               // boolean isMatch = value.matches( datePattern );
               // if ( isMatch )
             //  {
            //        DateAdapter dateAdapter = new DateAdapter();
              //      return dateAdapter.unmarshal(value);
             //   }
            //    else
             //   {
			 */
                    DateTimeAdapter dateAdapter = new DateTimeAdapter();
                    return dateAdapter.unmarshal(value);
             //   }
            }
            else
            {
                return DatatypeConverter.parseDate( INVALIDDATE ).getTime();
            }
        }
        else
        {
            return null;
        }
    }

    /**
     * Marshal a Date to a String for use by JAXB.
     *
     * @param value Date object to marshal.
     * @return String form of date.
     */
    public static String printDateToString( Date value )
    {
        Calendar cal = new GregorianCalendar();
        cal.setTime( value );
        return DatatypeConverter.printDateTime( cal );
    }

}