package com.intuit.sb.payroll.capability.ca.workerpayment;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import junit.framework.Assert;

import org.junit.Test;

import com.intuit.sb.payroll.ca.workerpayment.CanadaLawID;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QBOEPaycheckCalculatorBean;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.Worker;

public class OvertimeTest extends BasePaycheckTest {

	@Test
	public void test_OntarioHourlyWithOvertime()
	{
		BigDecimal wages = new BigDecimal("2000.00");
		BigDecimal overtime = new BigDecimal("375.00");
		
		BigDecimal amount = wages.add(overtime);
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), wages, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.Overtime.getLawId(), overtime, new BigDecimal("0.00"), new BigDecimal("0.00")),//Overtime
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("110.90"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("41.09"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("337.22"), amount, amount),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("169.17"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("25.00"), 
				80.0, new BigDecimal("37.50"), 10.0, 
				PayrollFrequencyEnum.BIWEEKLY, DateUtil.createDate(2009, Calendar.JANUARY, 5), 
				CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), new BigDecimal("8881.00"), lineItems); 
	}
	
	private void doTest(Date birthDate, String province, BigDecimal hourlyRate, 
			double hoursWorked, BigDecimal overtimeRate, double overtimeHoursWorked, 
			PayrollFrequencyEnum payFrequency, Date payDate, 
			CanadaLawID provincialTaxLaw, BigDecimal federalTD1Claim, 
			BigDecimal provincialTD1Claim, LineItemAmount[] lineItems) {
		
		Worker worker = createEmployee(birthDate, provincialTaxLaw, province);
		
		setupHourlyCompensationDetailForWorker(worker, hourlyRate, hoursWorked);
		
		setupHourlyOvertimeCompensationDetailForWorker(worker, overtimeRate, overtimeHoursWorked);
		
		Employer employer = new Employer();	
	    employer.setFirstPayrollDate(birthDate);
	    
	    int daysPerPeriod = frequencyToDaysPerPeriod.get(payFrequency);
		
	    Date payPeriodStartDate = DateUtil.getNextDate(payDate, -1 * daysPerPeriod);
	    
	    DateRange payPeriod = new DateRange(payPeriodStartDate, payDate);
	    
	    Paycheck paycheck = createPaycheck(payDate, worker, payPeriod,
				provincialTaxLaw, federalTD1Claim, provincialTD1Claim);
        
        PaycheckCalculator paycheckCalculator = new QBOEPaycheckCalculatorBean();
        
	    try {
			Paycheck calculated = paycheckCalculator.calculatePaycheck(worker, employer, paycheck, payFrequency);
			
			// verify paycheck line items
			Assert.assertTrue("Expected lineItem value mismatch!", 
					PaycheckUtils.verifyLineItems(calculated.getPaycheckLineItems(),
                    lineItems, "paycheck line items"));
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());			
		}
	}
}
