package com.intuit.sb.payroll.capability.ca.workerpayment;


import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ca.workerpayment.CanadaLawID;
import com.intuit.sb.payroll.ca.workerpayment.CanadaWorkerPaymentService;
import com.intuit.sb.payroll.ca.workerpayment.impl.CanadaWorkerPaymentEngine;
import com.intuit.sb.payroll.capability.ca.client.dto.PayrollScheduleFrequency;
import com.intuit.sb.payroll.capability.ca.client.mappers.PayrollFrequencyMapper;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QBOEPaycheckCalculatorBean;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ca.ServiceLocator;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.ca.model.TD1Detail;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocTypeEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.LawEntityTypeEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.TaxTypeEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

@Test(groups = "serviceittests")
@SuppressWarnings({"unused"})
public class CalculateHourlyPaycheckNoDeductionsTest
{
	private static Logger logger = LogManager.getLogger(CalculateHourlyPaycheckNoDeductionsTest.class);
	
	private static final Map<PayrollFrequencyEnum, Integer> frequencyToDaysPerPeriod = new HashMap<PayrollFrequencyEnum, Integer>();
	
	static
	{
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.ANNUALLY, 365);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.SEMIANNUALLY, 365 / 2);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.SEMIMONTHLY, 365 / 13);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.BIWEEKLY, 365 / 26);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.MONTHLY, 365 / 12);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.QUARTERLY, 365 / 4);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.WEEKLY, 7);
		frequencyToDaysPerPeriod.put(PayrollFrequencyEnum.DAILY, 1);
	}
	
	@Test
	public void test_OntarioHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("134.86"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), new BigDecimal("25.00"), 80.0, 
				PayrollFrequencyEnum.BIWEEKLY, DateUtil.createDate(2009, Calendar.JANUARY, 5), 
				CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), new BigDecimal("8881.00"), 
				lineItems); 
	}
	
	@Test
	public void test_OntarioHourlyWithSmallPay()
	{
		BigDecimal amount = new BigDecimal("100.00");
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), amount, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("0.00"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("1.73"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("0.00"), amount, amount),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("0.00"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), new BigDecimal("1.00"), 100.0, 
				PayrollFrequencyEnum.BIWEEKLY, DateUtil.createDate(2009, Calendar.JANUARY, 5), 
				CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), new BigDecimal("8881.00"), 
				lineItems); 
	}
	
	@Test
	public void test_AlbertaHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_AB.getLawId(), new BigDecimal("124.52"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), new BigDecimal("25.00"), 80.0, 
				PayrollFrequencyEnum.BIWEEKLY, DateUtil.createDate(2009, Calendar.JANUARY, 5), 
				CanadaLawID.PROV_TAX_AB, new BigDecimal("10100.00"), new BigDecimal("16775.00"), 
				lineItems); 
	}
	
	@Test
	public void test_SaskatchewanHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		CanadaLawID provLaw = CanadaLawID.PROV_TAX_SK;
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(provLaw.getLawId(), new BigDecimal("160.96"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), new BigDecimal("25.00"), 80.0, 
				PayrollFrequencyEnum.BIWEEKLY, DateUtil.createDate(2009, Calendar.JANUARY, 5), 
				provLaw, new BigDecimal("10100.00"), new BigDecimal("13269.00"), 
				lineItems); 
	}
	
	@Test
	public void test_ManitobaHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		CanadaLawID provLaw = CanadaLawID.PROV_TAX_MB;
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(provLaw.getLawId(), new BigDecimal("186.10"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), new BigDecimal("25.00"), 80.0, 
				PayrollFrequencyEnum.BIWEEKLY, DateUtil.createDate(2009, Calendar.JANUARY, 5), 
				provLaw, new BigDecimal("10100.00"), new BigDecimal("8134.00"), 
				lineItems); 
	}
	
	@Test
	public void test_BCHourlyWithoutDeductions()
	{
		BigDecimal amount = new BigDecimal("2000.00");
		
		CanadaLawID provLaw = CanadaLawID.PROV_TAX_BC;
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Hourly.getLawId(), new BigDecimal("2000.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), new BigDecimal("92.34"), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), new BigDecimal("34.60"), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("254.72"), amount, amount),//Fed Tax
				new LineItemAmount(provLaw.getLawId(), new BigDecimal("93.94"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), new BigDecimal("25.00"), 80.0, 
				PayrollFrequencyEnum.BIWEEKLY, DateUtil.createDate(2009, Calendar.JANUARY, 5), 
				provLaw, new BigDecimal("10100.00"), new BigDecimal("9373.00"), 
				lineItems); 
	}

	private void doTest(Date birthDate, BigDecimal hourlyRate, double hoursWorked, 
			PayrollFrequencyEnum payFrequency, Date payDate, CanadaLawID provincialTaxLaw, 
			BigDecimal federalTD1Claim, BigDecimal provincialTD1Claim, 
			LineItemAmount[] lineItems) {
		Worker worker = new Employee();
		PersonName personName = new PersonName();
		personName.setFirstName("Joe");
		personName.setLastName("ThePlumber");
		personName.setTitle("Dr.");
		
		worker.setPersonName(personName);
		worker.setDateOfBirth(DateUtil.createDate(1975, Calendar.JULY, 19));
		
		int daysPerPeriod = frequencyToDaysPerPeriod.get(payFrequency);
		
	    Date payPeriodStartDate = DateUtil.getNextDate(payDate, -1 * daysPerPeriod);
	    
	    DateRange payPeriod = new DateRange(payPeriodStartDate, payDate);	    
	    
	    Employer employer = new Employer();	
	    employer.setFirstPayrollDate(birthDate);
	    
	    Paycheck paycheck = new Paycheck();	    
	    paycheck.setPayPeriod(payPeriod);
        paycheck.setPayDate(payDate);	    
		
        setupCompensationDetailForWorker(worker, hourlyRate, hoursWorked);
        setupTaxDetailForWorker(worker, provincialTaxLaw);
        
		paycheck.setWorker(worker); 
		
		setupCompensationDetailsInPaycheck(paycheck, worker.getCompensationDtls(), payPeriod);
		
        setupTD1ForWorker(worker, federalTD1Claim, provincialTD1Claim, provincialTaxLaw);
        
        setupWorkLocationForWorker(worker);
        
        CanadaWorkerPaymentService paycheckCalculator = lookupService();
        
	    try {
	    	PayrollScheduleFrequency payscheduleFreuqency = PayrollFrequencyMapper.mapFromDEntityToEI(payFrequency);
			Paycheck calculated = paycheckCalculator.calculatePaycheck(worker, employer, paycheck, payscheduleFreuqency);
			
			// verify paycheck line items
			Assert.assertTrue(PaycheckUtils.verifyLineItems(calculated.getPaycheckLineItems(),
                    lineItems, "paycheck line items"), "Expected lineItem value mismatch!");
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());			
		}
	}
	
	
	private void setupTaxDetailForWorker(Worker worker, CanadaLawID provincialTaxLaw) {
		TaxDtl federalTax = new TaxDtl();
		Tax tax = new Tax();
		tax.setSourceEntity(LawEntityTypeEnum.WORKER);
		tax.setLawId(CanadaLawID.FED_TAX.getLawId());
		tax.setTaxType(TaxTypeEnum.FEDERALTAX);
		federalTax.setPayrollItem(tax);
		worker.addTaxDtl(federalTax);
		
		TaxDtl provincialTax = new TaxDtl();
		tax = new Tax();
		tax.setSourceEntity(LawEntityTypeEnum.WORKER);
		tax.setLawId(provincialTaxLaw.getLawId());
		tax.setTaxType(TaxTypeEnum.STATETAX);
		provincialTax.setPayrollItem(tax);
		worker.addTaxDtl(provincialTax);
		
		TaxDtl eiTax = new TaxDtl();
		tax = new Tax();
		tax.setSourceEntity(LawEntityTypeEnum.WORKER);
		tax.setLawId(CanadaLawID.EI_EE.getLawId());
		tax.setTaxType(TaxTypeEnum.FEDERALTAX);
		eiTax.setPayrollItem(tax);
		worker.addTaxDtl(eiTax);
		
		TaxDtl cppTax = new TaxDtl();
		tax = new Tax();
		tax.setSourceEntity(LawEntityTypeEnum.WORKER);
		tax.setLawId(CanadaLawID.CPP_EE.getLawId());
		tax.setTaxType(TaxTypeEnum.FEDERALTAX);
		cppTax.setPayrollItem(tax);
		worker.addTaxDtl(cppTax);
	}


	private void setupTD1ForWorker(Worker worker, BigDecimal federalClaim, BigDecimal provincialClaim, CanadaLawID provincialLawId) 
	{		
		createTD1(worker, federalClaim, BigDecimal.ZERO, 
				CanadaLawID.FED_TAX, ComplianceDocTypeEnum.TD1_CRA);
		
		createTD1(worker, provincialClaim, BigDecimal.ZERO, 
				provincialLawId, ComplianceDocTypeEnum.TD1_PROV);
	}


	private void createTD1(Worker worker, BigDecimal totalClaimValue, BigDecimal additionalAmountValue, CanadaLawID law, ComplianceDocTypeEnum complianceDocType) {
		Currency CAD = Currency.getInstance(Locale.CANADA); 
        TD1Detail td1 = new TD1Detail();       
        td1.setLawId(law.getLawId());
        td1.setComplianceDocType(complianceDocType);
        
        Money totalClaimAmt = new Money();
        totalClaimAmt.setCurrencyCode(CAD);       
        totalClaimAmt.setAmount(totalClaimValue);
        td1.setTotalClaimAmount(totalClaimAmt);       
        
        Money additionalTax = new Money();
        additionalTax.setCurrencyCode(CAD);
        additionalTax.setAmount(additionalAmountValue);
        td1.setAdditionalAmount(additionalTax);
        worker.addWorkerComplianceDocDtl(td1);
	}


 
	private void setupCompensationDetailsInPaycheck(Paycheck paycheck,
			Set<CompensationDtl> compensationDetails, DateRange payPeriod) 
	{
        for(CompensationDtl compendationDtl : compensationDetails)
        {
            PaycheckLineItem lineItem = new PaycheckLineItem();
            lineItem.setSourcePayrollItemDtl(compendationDtl);
            lineItem.setLawId(compendationDtl.getPayrollItem().getLawId());
            lineItem.setName(compendationDtl.getDescription());
            lineItem.setPayPeriod(payPeriod);
            
    		PayrollItemDtl linkedPayrollItemDtl = compendationDtl.getLinkedPayrollItemDtl();
    		
    		if (linkedPayrollItemDtl != null) {
    			CompensationDtl linkedCompensationDtl = (CompensationDtl)linkedPayrollItemDtl;
    			Datum linkedPIRate = linkedCompensationDtl.getRate();
    			
    			BigDecimal linkedRate = new BigDecimal(linkedPIRate.getValue1AsCADMoneyAmount().toString());
    			Double inputRate = compendationDtl.getRate().getValue1AsQuantity();
    			BigDecimal moneyValue = linkedRate.multiply(new BigDecimal(inputRate));    			
    			
    			Datum rate = new Datum();
    			rate.setValue1AsCADMoneyAmount(moneyValue);
    		    rate.setType2Value2(linkedPIRate.getType2(), linkedPIRate.getValue2());
    		    
    		    lineItem.setRate(rate); 
    		} else {
    		    lineItem.setRate(compendationDtl.getRate());     
    		}
    		
    		//Quantity
    		lineItem.setQuantity(compendationDtl.getQuantity());
    		
    		//Add the Line Item to Paycheck
    		paycheck.addPaycheckLineItem(lineItem);
        }
	}


	private void setupCompensationDetailForWorker(Worker worker, BigDecimal rate, double quantity)
	{
		CompensationDtl hourlyCompensation = new CompensationDtl();
		hourlyCompensation.setCompensationType(CompensationTypeEnum.HOURLYREG);
		hourlyCompensation.setName("Regular Hourly Wages");
		
		Compensation hComp = new Compensation();
		hComp.setLawId(CanadaLawID.Regular_Hourly.getLawId());
		hComp.setCompensationType(CompensationTypeEnum.HOURLYREG);
		hourlyCompensation.setPayrollItem(hComp);
	
		//Set the Rate
		Datum hourlyRate = new Datum();
		hourlyRate.setValue1AsCADMoneyAmount(rate);
		hourlyRate.setType2(DatumType2Enum.HOUR);
		hourlyRate.setValue2(new Double(1));		
		hourlyCompensation.setRate(hourlyRate);
		
		//Now set the Actual Hours
		Datum quantityDatum = new Datum();
	    quantityDatum.setValue1AsQuantity(quantity);
	    quantityDatum.setType2(DatumType2Enum.NOTAPPLICABLE);	    
	    hourlyCompensation.setQuantity(quantityDatum);	    
	    worker.addCompensationDtl(hourlyCompensation);
		
	}
	
	private void setupWorkLocationForWorker(Worker worker)
	{
		WorkLocation workLocation = new WorkLocation();
		
		GlobalPostalAddress taxAddress = new GlobalPostalAddress();
		taxAddress.setCountryCode(Locale.CANADA.getCountry());
		taxAddress.setCountrySubDivisionCode("ON");
		
		workLocation.setTaxResidenceAddress(taxAddress);
		
		worker.setWorkLocation(workLocation);		
		worker.setTaxResidenceAddress(taxAddress);
	}
	
	private CanadaWorkerPaymentService lookupService()
	{
		ServiceLocator serviceLocator = ServiceLocator.getInstance();
		CanadaWorkerPaymentService paycheckCalculator =
			serviceLocator.getService(CanadaWorkerPaymentEngine.JNDI_NAME);
		return paycheckCalculator;

	}
}
