package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.Date;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;

/**
 * Creates PayrollRun instances based on various inputs.
 *
 * $Author: thobson $ 
 * $Revision: #8 $
 * $DateTime: 2009/05/14 14:08:09 $
 */
public class PayrollRunFactory {

	static private PayrollScheduleComponent sPayrollScheduleComponent =
						ServiceLocator.getInstance().getService(PayrollScheduleComponent.JNDI_NAME);
	
	/**
	 * Factory method to create a PayrollRun from a PayrollRunInfo.
	 * 
	 * @param nextPayRunInfo
	 * @param payrollRun
	 */
public static PayrollRun createPayrollRunFromPayrollRunInfo(PayrollRunInfo nextPayRunInfo) throws ServiceFailureException, 
		ValidationException {
		PayrollRun payrollRun = new PayrollRun();
		payrollRun.setWorkflowStatus(PayrollRunWorkflowStatusEnum.DRAFT);
		if ((nextPayRunInfo.getPayrollRunType() == PayrollRunTypeEnum.TERMINATION) &&
			(nextPayRunInfo.getPayPeriod() == null))
		{
			PayrollSchedule payrollSchedule = sPayrollScheduleComponent.getDefaultPayrollSchedule();
			PayrollRunInfo upcomingPayRunInfo = sPayrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
			
			Date todaysDate = DateUtil.getNowDate();
			DateRange payPeriod = new DateRange();
			Date startDate = upcomingPayRunInfo.getPayPeriod().getStartDate();
			if (startDate.after(todaysDate))
			{
				payPeriod.setStartDate(todaysDate);
			}
			else
			{
				payPeriod.setStartDate(startDate);
			}
			payPeriod.setEndDate(todaysDate);
			payrollRun.setPayPeriod(payPeriod);
			payrollRun.setPayDate(nextPayRunInfo.getPayDate());
			payrollRun.setPayrollRunDate(todaysDate);
			payrollRun.setPayrollFrequency(upcomingPayRunInfo.getFrequency());
		}
		else
		{
			payrollRun.setPayPeriod(nextPayRunInfo.getPayPeriod());
			payrollRun.setPayDate(nextPayRunInfo.getPayDate());
			payrollRun.setPayrollRunDate(nextPayRunInfo.getPayrollRunDate());
			payrollRun.setPayrollFrequency(nextPayRunInfo.getFrequency());
		}
		payrollRun.setTypeOf(nextPayRunInfo.getPayrollRunType());
		payrollRun.setPartialPeriod(nextPayRunInfo.isPartialPayPeriod());
		return payrollRun;
	}
}
