package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.intuit.payroll.j.AdjustmentIf;
import com.intuit.payroll.j.PayrollDatumIf;
import com.intuit.payroll.j.PayrollTransactionLineIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerAdjLineItem;
import com.intuit.sb.payroll.entity.model.WorkerAdjustment;

public class EmployeeAdjustmentMapper {

	public static List<WorkerAdjustment> mapEmployeeAdjustments(Collection<AdjustmentIf> employeeAdjustments,
			AdjustmentContext adjustmentContext) {
		List<WorkerAdjustment> results = new ArrayList<WorkerAdjustment>(employeeAdjustments.size());
		Map<Integer, AdjustmentIf> adjustmentsMap = new HashMap<Integer, AdjustmentIf>();
		for (AdjustmentIf adjustmentIf : employeeAdjustments) {
			adjustmentsMap.put(adjustmentIf.getEmployeeUniqueID(), adjustmentIf);
		}
		
		List<Worker> affectedWorkers = adjustmentContext.getAffectedWorkers();
		for (Worker worker : affectedWorkers) {
			Long workerId = new Long(worker.getId());
			AdjustmentIf adjustmentIf = adjustmentsMap.get(new Integer(workerId.intValue()));
			WorkerAdjustment workerAdjustment = mapAdjustmentToWorkerAdjustment(adjustmentIf, adjustmentContext, worker);
			results.add(workerAdjustment);
		}
		return results;
	}

	private static WorkerAdjustment mapAdjustmentToWorkerAdjustment(AdjustmentIf adjustmentIf, AdjustmentContext adjustmentContext, Worker worker) {
		WorkerAdjustment workerAdjustment = new WorkerAdjustment();
		workerAdjustment.setEffectiveDate(adjustmentIf.getPayrollDate().toUtilDate());
		Collection<PayrollTransactionLineIf> payrollTransactionLines = adjustmentIf.getPayrollTransactionLines();
		for (PayrollTransactionLineIf payrollTransactionLineIf : payrollTransactionLines) {
			WorkerAdjLineItem lineItem = new WorkerAdjLineItem();
			String lawId = new Integer(payrollTransactionLineIf.getPayrollItem().getDetailType().toInt()).toString();
			lineItem.getLawId().setValue(lawId);
			PayrollItemDtl payrollItemDtl = findPayrollItemDtlByLawIdOnWorker(worker, lawId);
			if (payrollItemDtl == null) {
				throw new IllegalStateException("Adjustment created for a Law we don't have on this worker.");
			}
			lineItem.setPayrollItemDtl(payrollItemDtl);
			PayrollDatumIf amountDatum = payrollTransactionLineIf.getDatum(DatumID.AMOUNT);
			Money amount = new Money(amountDatum.getValue().toString());
			lineItem.setAdjustmentAmount(amount);
		}
		
		return workerAdjustment;
	}

	private static PayrollItemDtl findPayrollItemDtlByLawIdOnWorker(Worker worker, String lawId) {
		PayrollItemDtl result = null;
		Set<PayrollItemDtl> payrollItemDtls = worker.getPayrollItemDtls();
		for (PayrollItemDtl payrollItemDtl : payrollItemDtls) {
			String lawIdValue = payrollItemDtl.getPayrollItem().getLawId().getValue();
			if (lawId.equals(lawIdValue)) {
				return payrollItemDtl;
			}
		}
		return result;
	}

}
