package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import com.intuit.dataTypes.Money;
import com.intuit.dataTypes.QBDate;
import com.intuit.payroll.j.AdjustmentControllerIf;
import com.intuit.payroll.j.AdjustmentDatesChooserIf;
import com.intuit.payroll.j.AdjustmentIf;
import com.intuit.payroll.j.CompanyAdjustmentCreatorIf;
import com.intuit.payroll.j.CompanyIf;
import com.intuit.payroll.j.PayrollFactory;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.j.PayrollItemListInfoIf;
import com.intuit.payroll.j.PayrollItemTotalsCollectionIf;
import com.intuit.payroll.j.PayrollItemTotalsIf;
import com.intuit.payroll.j.PayrollTxSummaryIf;
import com.intuit.payroll.j.TaxJurisdictionIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.payroll.types.enumtype.PayrollState;
import com.intuit.payroll.types.enumtype.TotalsTransactionType;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollItemDtlPtdDatum;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.workerpayments.CompanyAdjustmentTotals;
import com.intuit.sb.payroll.entity.workerpayments.WorkerTotals;

/**
 * Handles the workflow of interacting with the Payroll Engine to calculate liability adjustments.
 * 
 * $Author: $ $Revision: $ $DateTime: $
 * 
 */
public class EmployerAdjustmentController {

	private static Logger sLogger = PayrollLoggerFactory.makeLogger("-=:|AdjustmentsIntegration|:=-");

	/**
	 * This is a brutal algorithm to follow but it works. The challenge is that we have to work in several dimensions.
	 * 
	 * We need to sum up all the employee WageBase, ISST and Total Amounts for each PayItem (law). Law and and PayItem
	 * are used in a 1:1 relationship here. Since that alone is too easy, we have to break out these totals by Quarter
	 * within the DateRange specified. And since that's too easy we have to further break them out by Live/ Work
	 * location pairing. Sounds like fun, so let's do it!
	 * 
	 * FIXME: there is no support for live/work history at the moment so that dimension is currently moot and hard-
	 * coded to California.
	 * 
	 * @param adjustmentContext
	 * @return
	 */
	public static Collection<AdjustmentIf> createCompanyRoundingAdjustments(AdjustmentContext adjustmentContext) {

		final CompanyIf company = adjustmentContext.getQboCompany();
		//Add company payroll items
		WorkerTotals workerTotals = adjustmentContext.getWorkerTotals();
		Map<String, Map<String, PayrollItemDtlPtdDatum>> qtrTotalsByLawId = workerTotals.getQtrTotalsByLawId();
		Set<Entry<String, Map<String, PayrollItemDtlPtdDatum>>> entrySet = qtrTotalsByLawId.entrySet();
		for (Entry<String, Map<String, PayrollItemDtlPtdDatum>> entry : entrySet) {
			adjustmentContext.getPayrollItemForLaw(entry.getKey());			
		}
		

		//Once the payroll items are on the company, we should get a controller back due to self-adjusting taxes being
		//present.
		AdjustmentControllerIf adjustmentController = company.getRoundingAdjustmentController(null, null);

		Collection<AdjustmentIf> results = null;
		if (adjustmentController != null) {
			
			if (adjustmentController.needsCompanyAdjustments()) {
				// Items for which PE needs qtr totals in order to calculate adjustments - NOT USED YET, WE HAVE NO
				// PAYMENTS
				Collection<PayrollItemListInfoIf> dbItems = adjustmentController.getNeededPayrollItems();

				// FIXME Tell PE the earliest and latest date for which payroll data exists in the company.
				adjustmentController.setPayrollBeginEndDates(new QBDate(1, 1, 2008), new QBDate(12, 31, 2008));

				// Begin and exclusive end date for which PE needs qtr totals in order to calculate adjustments.
				// Note that these will be QTR boundaries, not random dates. This allows us to bucket the data
				// accordingly.
				QBDate dbBegin = adjustmentController.getTotalsBeginEffectiveDate();
				QBDate dbExclusiveEnd = adjustmentController.getTotalsEndEffectiveDate();
				final DateRange dateRange = new DateRange(dbBegin.toUtilDate(), dbExclusiveEnd.toUtilDate());
				PayrollItemTotalsCollectionIf employerTotals = mapToQboEmployerTotals(adjustmentContext);
				PayrollItemTotalsCollectionIf employeeTotals = mapToQboEmployeeTotals(adjustmentContext);
				PayrollTxSummaryIf uiPayments = getPayrollTxSummary(dateRange, dbItems);
				CompanyAdjustmentCreatorIf creator = adjustmentController.createCompanyAdjustmentCreator(
						employeeTotals, employerTotals, uiPayments);
				AdjustmentDatesChooserIf datesChooser = new AdjustmentDatesChooser();
				results = creator.createAdjustments(datesChooser);
				//FIXME set to debug
				if (sLogger.isInfoEnabled()) {
					sLogger.info("Employer Totals:" + employerTotals.toString());
					sLogger.info("Employee Totals:" + employeeTotals.toString());
					sLogger.info("Resulting Adjustments: " + results.toString());
				}
			}
		}
		return results;
	}


	static PayrollItemTotalsCollectionIf mapToQboEmployeeTotals(AdjustmentContext adjustmentContext) {
		WorkerTotals workerTotals = adjustmentContext.getWorkerTotals();
		PayrollItemTotalsCollectionIf wageTotalsCollection = PayrollFactory.createTotalsCollection(null);
		Map<String, Map<String, PayrollItemDtlPtdDatum>> qtrTotalsByLawId = workerTotals.getQtrTotalsByLawId();

		Set<Entry<String, Map<String, PayrollItemDtlPtdDatum>>> entrySet = qtrTotalsByLawId.entrySet();
		for (Entry<String, Map<String, PayrollItemDtlPtdDatum>> totalsByLawIdEntry : entrySet) {
			
			// Create totals for the law in QBO.
			PayrollItemIf payrollItemForLaw = adjustmentContext.getPayrollItemForLaw(totalsByLawIdEntry.getKey());
			PayrollItemTotalsIf payrollItemTotals = wageTotalsCollection.createTotals(payrollItemForLaw);

			// For each law we will have different Qtrs, populate the totals for each one. The QBO container
			// supports entries for each qtr.
			// FIXME we need to further break down the totals by Live/Work as well as QTR and LawID
			TaxJurisdictionIf CA_JUR = PayrollFactory.createTaxJurisdiction(PayrollState.CA, null);
			Map<String, PayrollItemDtlPtdDatum> qtrTotalsMap = totalsByLawIdEntry.getValue();
			Set<Entry<String, PayrollItemDtlPtdDatum>> qtrTotalsEntrySet = qtrTotalsMap.entrySet();
			for (Entry<String, PayrollItemDtlPtdDatum> qtr : qtrTotalsEntrySet) {
				PayrollItemTotalsMapper.mapPayrollItemDtlPtdDatumToPayrollItemTotals(CA_JUR, CA_JUR, payrollItemTotals,
						qtr.getValue());				
			}

		}

		return wageTotalsCollection;
	}

	/**
	 * Returns a summary of payments made over the date range.
	 * 
	 * NOT USED AT THIS TIME WHILE WE USE ASSISTED. This is a variation point in the logic that we need to consider -
	 * when the company is on assisted, we can't provide this data.
	 * 
	 * @param dateRange
	 * @param dbItems
	 * @return
	 */
	static PayrollTxSummaryIf getPayrollTxSummary(DateRange dateRange, Collection<PayrollItemListInfoIf> dbItems) {
		// We don't have any payments to report.
		PayrollTxSummaryIf txSummary = PayrollFactory.createPayrollTxSummary();
		return txSummary;
	}

	/**
	 * 
	 * @param company
	 * @param dateRange
	 * @param payrollItemMap
	 * @return
	 */
	static PayrollItemTotalsCollectionIf mapToQboEmployerTotals(AdjustmentContext adjustmentContext) {
		final PayrollItemTotalsCollectionIf companyTotalsCollection = PayrollFactory.createTotalsCollection(null);
		CompanyAdjustmentTotals companyTotals = adjustmentContext.getCompanyTotals();
		// If there are no prior adjustments this will be null.
		if (null == companyTotals) {
			return companyTotalsCollection;
		} else {
			Map<String, Set<PtdDatumType>> totalsByLawIdMap = companyTotals.getTotalsByLawIdMap();

			// Running totals have been summed up inside the PayItemDtlPtdDatums in the map, now turn them into QBO
			// totals.
			Set<Entry<String, Set<PtdDatumType>>> totalsByLawIdSet = totalsByLawIdMap.entrySet();
			for (Entry<String, Set<PtdDatumType>> totalByLawIdEntry : totalsByLawIdSet) {
				PayrollItemIf payrollItemForLaw = adjustmentContext.getPayrollItemForLaw(totalByLawIdEntry.getKey());
				PayrollItemTotalsIf payrollItemTotals = companyTotalsCollection.createTotals(payrollItemForLaw);
				populateAdjustmentTotals(payrollItemTotals, totalByLawIdEntry.getValue());
			}
		}
		return companyTotalsCollection;
	}

	/**
	 * Populates an existing QBO totals with the totals from a particular pay item's adjustments.
	 * 
	 * @param totals
	 * @param adjustmentTotals
	 * @param dateRange
	 */
	private static void populateAdjustmentTotals(PayrollItemTotalsIf totals, Set<PtdDatumType> employerTotalsByPeriod) {
		for (PtdDatumType entry : employerTotalsByPeriod) {
			QBDate periodStart = new QBDate(entry.getPeriod().getStartDate());
			QBDate periodEnd = new QBDate(entry.getPeriod().getEndDate());
			Money totalValue = new Money(entry.getMoneyValue().getAmount().toString());
			totals.setTotalFromDB(DatumID.TOTAL_AMOUNT, TotalsTransactionType.ALL, totalValue, periodStart, periodEnd,
					null, null);
		}
	}

}
