package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.Date;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import com.intuit.dataTypes.QBDate;
import com.intuit.payroll.j.PaycheckIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;

/**
 * Hydrates a QBO Paycheck from one of ours.
 * 
 * $Author: $ 
 * $Revision: $
 * $DateTime: $
 *
 */
public class PaycheckFactory {

	private static Logger sLogger = LogManager.getLogger("PaycheckCalculatorBean");
	
	static void createQboPaycheck(PaycheckCalculationContext context, boolean partialPayPeriod) {
		PaycheckIf qboePaycheck = context.getQboeEmployee().createPaycheck(new QBDate(context.getPaycheck().getPayDate()));
		context.setQboePaycheck(qboePaycheck);

		final DateRange payPeriod = context.getPaycheck().getPayPeriod();
		final DateRange employmentPeriod = context.getWorker().getEmploymentPeriod();
		if (employmentPeriod != null) {
			final Date startDate = employmentPeriod.getStartDate();
			final Date endDate = employmentPeriod.getEndDate();
		    // Set PARTIAL_PAY_PERIOD to have Payroll Engine calculate paycheck based on PAY_PERIOD_BEGIN/END
		    // dates instead of PAY_FREQUENCY (eg partial pay period due to hire or termination within the period)
			boolean startInPeriod = false;
			boolean endInPeriod = false;
			if (startDate != null) {
				startInPeriod = isInPeriod(startDate, payPeriod);
				// If the start date is the pay period begin date, it is not considered a partial paycheck
				if (startInPeriod) {
					startInPeriod = !isInPeriod(startDate, new DateRange(payPeriod.getStartDate(), payPeriod.getStartDate()));
				}
			}
			if (endDate != null) {
				endInPeriod = isInPeriod(endDate, payPeriod);	
				// If the end date is the pay period end date, it is not considered a partial paycheck
				if (endInPeriod) {
    				endInPeriod = !isInPeriod(endDate, new DateRange(payPeriod.getEndDate(), payPeriod.getEndDate()));
				}
			}
			// Apply period truncations
			if (startInPeriod && endInPeriod) {
				qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_BEGIN, new QBDate(startDate));
				qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_END, new QBDate(endDate));
				context.getQboePaycheck().setDatumValue(DatumID.PARTIAL_PAY_PERIOD, Boolean.TRUE);			
			} else if (startInPeriod) {
				qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_BEGIN, new QBDate(startDate));
				qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_END, new QBDate(payPeriod.getEndDate()));
				context.getQboePaycheck().setDatumValue(DatumID.PARTIAL_PAY_PERIOD, Boolean.TRUE);
			} else if (endInPeriod) {
				qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_BEGIN, new QBDate(payPeriod.getStartDate()));
				qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_END, new QBDate(endDate));
				context.getQboePaycheck().setDatumValue(DatumID.PARTIAL_PAY_PERIOD, Boolean.TRUE);
			} else {
				qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_BEGIN, new QBDate(payPeriod.getStartDate()));
				qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_END, new QBDate(payPeriod.getEndDate()));
				if (partialPayPeriod) {
					context.getQboePaycheck().setDatumValue(DatumID.PARTIAL_PAY_PERIOD, Boolean.TRUE);
				}
			}
		} else {
			qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_BEGIN, new QBDate(payPeriod.getStartDate()));
			qboePaycheck.setDatumValue(DatumID.PAY_PERIOD_END, new QBDate(payPeriod.getEndDate()));
			if (partialPayPeriod) {
				context.getQboePaycheck().setDatumValue(DatumID.PARTIAL_PAY_PERIOD, Boolean.TRUE);
			}
		}

		calculateQboPaycheckImpl(context);
	}

	private static boolean isInPeriod(Date date, DateRange period) {
		final Date start = period.getStartDate();
		final Date end = period.getEndDate();
		return DateUtil.isInPeriod(date, start, end);
	}
	
	private static void calculateQboPaycheckImpl(PaycheckCalculationContext context) {

		final PayrollItemHandlerChain handlerChain = new PayrollItemHandlerChain();
		final PayrollItemHandlerChainStrategyBuilder strategy = new PayrollItemHandlerChainStrategyBuilder();
		handlerChain.addHandlers(strategy.getHandlers());
		handlerChain.execute(context);
		
		//FIXME Hack to work around taxability issues.
		context.getPayrollItemInfo().restoreDefaults();

		// freeze the paycheck from further updates
		context.getQboePaycheck().setDataBeforeValidate();
		
		sLogger.info(context.getQboePaycheck());

		// run calcs
		context.getQboeEmployee().getTotalsCollection().update(context.getQboePaycheck());
	}
	
	

}
