package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.math.RandomUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import com.intuit.payroll.j.CompanyIf;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;

/**
 * Common interface for context that is passed around PayrollEngine integration.
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public abstract class PayrollEngineContext {
	private static Logger sLogger = LogManager.getLogger("PayrollEngineContext");
	private CompanyIf qboCompany;
	private Employer employer;
	private Map<String, PayrollItemIf> payrollItemMap = new HashMap<String, PayrollItemIf>();
	private PayrollItemInfo payrollItemInfo = new PayrollItemInfo();
	
	public CompanyIf getQboCompany() {
		return qboCompany;
	}

	public void setQboCompany(CompanyIf qboCompany) {
		this.qboCompany = qboCompany;
	}
	
	public void setLawToPayrollItemMap(Map<String, PayrollItemIf> payrollItemMap) {
		this.payrollItemMap = payrollItemMap;
	}

	public Map<String, PayrollItemIf> getLawToPayrollItemMap() {
		return payrollItemMap;
	}
	
	public void setPayrollItemInfo(PayrollItemInfo payrollItemInfo) {
		this.payrollItemInfo = payrollItemInfo;
	}

	public PayrollItemInfo getPayrollItemInfo() {
		return payrollItemInfo;
	}

	public void setEmployer(Employer employer) {
		this.employer = employer;
	}

	public Employer getEmployer() {
		return employer;
	}
	
	/**
	 * Create a QBO {@link PayrollItemIf} based on a LawId.
	 * 
	 * @param company
	 * @param lawId
	 * @param payrollItemMap
	 * @return
	 */
	public PayrollItemIf getPayrollItemForLaw(String lawId) {
		// The contract is that if you create a payroll item, you put in the map for re-use. This is critical
		// since the PE depends on uniquely identifiable items.
		PayrollItemIf qboPayrollItem = this.getLawToPayrollItemMap().get(lawId);
		if (null == qboPayrollItem) {
			qboPayrollItem = createCompanyPayrollItem(lawId);
		}
		return qboPayrollItem;
	}
	
	/*
	 * Singleton Payroll Items
	 * FIXME: creating a random ID is only useful with employee items.
	 */
	public PayrollItemIf createCompanyPayrollItem(String lawId) {
		final PayrollItemDetailType payrollItemDetailType = PayrollItemDetailType.parseInt(Integer.valueOf(lawId));
		PayrollItemIf qboPayrollItem = this.getQboCompany().createPayrollItem(payrollItemDetailType);
		// Singleton pay items (taxes) should be in context, wages may need to be made up for this purpose.
		int pItemUniqueId = RandomUtils.nextInt();
		this.getPayrollItemInfo().setPayrollItem(pItemUniqueId, qboPayrollItem);
		qboPayrollItem.finishInit(pItemUniqueId, false);
		this.getLawToPayrollItemMap().put(lawId, qboPayrollItem);
		sLogger.info("Creating QBO Payroll Item for " + qboPayrollItem.toString());
		return qboPayrollItem;
	}
	
	/**
	 * Creates an Employee Payroll Item based on the PayrollItemDtl.
	 * Does not set rates. The caller should be able to do that.
	 * 
	 * @param qboEmployee
	 * @param payrollItemDtl
	 * @return
	 */
	public EmployeePayrollItemIf createEmployeePayrollItem(EmployeeIf qboEmployee, PayrollItemDtl payrollItemDtl) {
		String lawId = payrollItemDtl.getPayrollItem().getLawId().getValue();
		final PayrollItemIf qboePayrollItem = this.createCompanyPayrollItem(lawId);
		EmployeePayrollItemIf employeePI = qboEmployee.createEmployeePayrollItem(qboePayrollItem);
		return employeePI;
	}
}
