package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import com.intuit.payroll.j.PayrollItemCallbackIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.j.PayrollItemListCallbackIf;
import com.intuit.payroll.j.PayrollItemListInfoIf;
import com.intuit.payroll.j.UserTaxabilityIf;
import com.intuit.payroll.types.PayrollException;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.sb.payroll.capability.workerpayments.CalculationEngineException;

/**
 * The intent of this class is to provide Taxability information related to each payroll item 
 * we use.
 * @author thobson
 *
 */
public class PayrollItemInfo implements PayrollItemCallbackIf, PayrollItemListCallbackIf {
	private UserTaxabilityIf taxability;
	private Map<Integer, PayrollItemListInfoIf> idMap = new TreeMap<Integer, PayrollItemListInfoIf>();

	public PayrollItemIf getPayrollItem(PayrollItemDetailType itemDetailType) {
		for(PayrollItemListInfoIf item : idMap.values()) {
			PayrollItemIf payrollItem = (PayrollItemIf)item;
			
			PayrollItemDetailType detailType = payrollItem.getDetailType();
			
			if (null != detailType && detailType.equals(itemDetailType)) {
				return payrollItem;
			}
		}
		
		return null;
	}

	public PayrollItemIf getPayrollItem(Integer itemId) {
		return (PayrollItemIf) idMap.get(itemId);
	}

	public void setPayrollItem(Integer itemId, PayrollItemListInfoIf payrollItem) {
		idMap.put(itemId, payrollItem);
	}
	
	public UserTaxabilityIf getUserTaxability() {
		return taxability;
	}

	public void setUserTaxability(UserTaxabilityIf taxability) {
		this.taxability = taxability;
	}

	public Iterator<PayrollItemListInfoIf> getFullListIterator() {
		return idMap.values().iterator();
	}
	
	public void restoreDefaults() {
		try {
			taxability.testRestoreDefaults();
		} catch (PayrollException e) {
			throw new CalculationEngineException(e);
		}
	}
}
