package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.intuit.dataTypes.Rate;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.entity.model.EmployerComplianceDocDtl;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

public abstract class QBOEMappedDeductionBase {

	public static class MappedWithholding {
		
	    private DatumID datumID;
	    private Object value;
	    
	    public MappedWithholding(DatumID datumID, Object value) {
	        this.datumID = datumID;
	        this.value = value;
	    }
	    
	    public DatumID getDatumID() {
	        return datumID;
	    }
	    public Object getValue() {
	    	return value;
	    }
	}

	protected Integer lawId;
	protected Rate qboeRate;
	protected Integer filingStatus = null;
	private Set<MappedWithholding> mappedWithholdings = new HashSet<MappedWithholding>();
	private PayrollItemDtl payrollItemDtl;
	
	public QBOEMappedDeductionBase() {
		super();
	}
	
	/**
     * Constructs a mapped payroll item wrapper from the payroll engine payroll item.
     * @param payrollItemDtl payroll engine representation of a payroll item.
     */
    protected QBOEMappedDeductionBase(PayrollItemDtl payrollItemDtl) {
        init(payrollItemDtl);
    }

	public QBOEMappedDeductionBase(TaxDtl taxDtl,
    		Map<String, EmployerComplianceDocDtl> employerComplianceDocDtlMap,
    		Map<String, WorkerComplianceDocDtl> workerComplianceDocDtlMap)
    {
    	init(taxDtl, employerComplianceDocDtlMap, workerComplianceDocDtlMap);
    }
	
	private void init(PayrollItemDtl payrollItemDtl) {
		lawId = Integer.valueOf(payrollItemDtl.getPayrollItem().getLawId().getValue());
        this.setPayrollItemDtl(payrollItemDtl);
	}
    
    public void init(TaxDtl taxDtl,
    		Map<String, EmployerComplianceDocDtl> employerComplianceDocDtlMap,
    		Map<String, WorkerComplianceDocDtl> workerComplianceDocDtlMap) {

    	init(taxDtl);
        
    	if (employerComplianceDocDtlMap.get(lawId.toString()) != null) {
        	mapEmployerDocDtls(employerComplianceDocDtlMap);
    	} else if (workerComplianceDocDtlMap.get(lawId.toString()) != null) {
    		mapWorkerDocDtls(workerComplianceDocDtlMap);
    	} // else it's a tax table rate so we don't need to provide any data
	}
    
    protected abstract void mapEmployerDocDtls(Map<String, EmployerComplianceDocDtl> employerComplianceDocDtlMap);

	protected abstract void mapWorkerDocDtls(Map<String, WorkerComplianceDocDtl> workerComplianceDocDtlMap);

	public Integer getLawId() {
	    return lawId;
	}

	public Rate getRate() {
	    return qboeRate;
	}

	/**
	 * Gets the withholding filing status for this payroll item. Only applies if this
	 * is an income tax item. The filing status is actually a "law id" that uniquely
	 * enumerates the filing status.
	 * @return the filing status for this income tax payroll item.
	 */
	public Integer getFilingStatus() {
	    return filingStatus;
	}

	public Set<MappedWithholding> getMappedWithholdings() {
	    return mappedWithholdings;
	}

	/**
	 * Adds a withholding info to the collection of withholdings for an income tax payroll item.
	 * @param mappedWithholding the withholding to add.
	 */
	protected void addMappedWithholding(MappedWithholding mappedWithholding) {
	    mappedWithholdings.add(mappedWithholding);
	}

	public void setPayrollItemDtl(PayrollItemDtl payrollItemDtl) {
		this.payrollItemDtl = payrollItemDtl;
	}

	public PayrollItemDtl getPayrollItemDtl() {
		return payrollItemDtl;
	}

}