package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import com.intuit.payroll.j.AdjustmentIf;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.types.PayrollException;
import com.intuit.payroll.util.PayrollLog;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.AdjustmentRequest;
import com.intuit.sb.payroll.capability.workerpayments.AdjustmentResponse;
import com.intuit.sb.payroll.capability.workerpayments.AdjustmentTrigger;
import com.intuit.sb.payroll.capability.workerpayments.CalculationEngineException;
import com.intuit.sb.payroll.capability.workerpayments.CalculationInputs;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckInputs;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInputs;
import com.intuit.sb.payroll.capability.workerpayments.impl.PayrollRunFactory;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.DeductionDtl;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerAdjustment;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PtoBaseRateTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerAdjustment;
import com.intuit.sb.payroll.entity.utils.LinkedCompensationUtils;
import com.intuit.sb.payroll.entity.workerpayments.CompanyAdjustmentTotals;
import com.intuit.sb.payroll.entity.workerpayments.WorkerTotals;

/**
 * Moved the implementation of {@link QBOEPaycheckCalculatorBean} to here because
 * Canada needs a non-authenticated version of the bean. US can continue to use
 * the same bean because it supports authentication.
 * 
 *
 */
public class QBOEPaycheckCalculatorBeanImpl implements PaycheckCalculator {
	static {
		// Set our log adapter into the QBOE logger so it logs via our adapter
		PayrollLog.setLogger(new QBOELog());
	}
	
	private Logger sLogger = LogManager.getLogger("PaycheckCalculatorBean");

	public QBOEPaycheckCalculatorBeanImpl() {}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator#calculatePayrollRun(com.intuit.sb.payroll.capability.workerpayments.PayrollRunInputs, java.util.Set)
	 */
	public PayrollRun calculatePayrollRun(PayrollRunInputs payRunInputs, Set<PaycheckInputs> paycheckInputsCollection)
		throws ValidationException, ServiceFailureException {

		PayrollRun payrollRun = PayrollRunFactory.createPayrollRunFromPayrollRunInfo(payRunInputs.getPayrollRunInfo());
		Employer employer = payRunInputs.getEmployer();
		
		for (PaycheckInputs paycheckInputs : paycheckInputsCollection) {
            final Worker worker = paycheckInputs.getWorker();
            
			final Map<PaycheckLineIf, PaycheckLineItem> paycheckLineItemMap = new HashMap<PaycheckLineIf, PaycheckLineItem>();
    		
			final PaycheckCalculationContext context = new PaycheckCalculationContext(employer, worker, paycheckLineItemMap);
			sLogger.info("Creating Context and Company for Payroll Run - PaycheckCalc");
			CompanyFactory.createCompany(context);
			
			//Create Employee
			final EmployeeIf qboEmployee = EmployeeFactory.createQboEmployee(context, worker, payRunInputs.getPayrollRunInfo().getFrequency());
			context.setQboeEmployee(qboEmployee);

			//Create the Domain Paycheck.
	        final Paycheck paycheck = createDomainPaycheck(payRunInputs, paycheckInputs, worker);
	        context.setPaycheck(paycheck);
    		
			PaycheckFactory.createQboPaycheck(context, paycheckInputs.isPartialPayPeriod());
			
			//Map QBO paycheck to Domain paycheck.
			PaycheckMapper.mapPaycheckResults(context, true);
			//Add Domain Paycheck to Domain PayrollRun
			payrollRun.addPaycheck(context.getPaycheck()); // sets both ends of the bi-di relationship
		}

		return payrollRun;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator#calculatePaycheck(com.intuit.sb.payroll.capability.workerpayments.PayrollRunInputs, com.intuit.sb.payroll.capability.workerpayments.PaycheckInputs)
	 */
	public Paycheck calculatePaycheck(PayrollRunInputs payRunInputs, PaycheckInputs paycheckInputs) throws ValidationException, ServiceFailureException {
		try {
            Worker worker = paycheckInputs.getWorker();
			Employer employer = payRunInputs.getEmployer();
			final Map<PaycheckLineIf, PaycheckLineItem> paycheckLineItemMap = new HashMap<PaycheckLineIf, PaycheckLineItem>();
			
			final PaycheckCalculationContext context = new PaycheckCalculationContext(employer, worker, paycheckLineItemMap);
			CompanyFactory.createCompany(context);
			
			EmployeeIf qboeEmployee = EmployeeFactory.createQboEmployee(context, worker, payRunInputs.getPayrollRunInfo().getFrequency());
			context.setQboeEmployee(qboeEmployee);
		
			//Create the Domain Paycheck.
	        final Paycheck paycheck = createDomainPaycheck(payRunInputs, paycheckInputs, worker);
			context.setPaycheck(paycheck);
			
			PaycheckFactory.createQboPaycheck(context, payRunInputs.getPayrollRunInfo().isPartialPayPeriod());

			PaycheckMapper.mapPaycheckResults(context, false);
			return context.getPaycheck();
		} catch (PayrollException e) {
			throw new CalculationEngineException(e);
		}
	}
	
	public AdjustmentResponse calculateLiabilityAdjustments(AdjustmentRequest request) throws ServiceFailureException {
		//Need to build up this context with the payrollItem maps
		Employer employer = request.getEmployer();
		//Data required is the Period Totals by Employee by Pay Item by WL/LL
		WorkerTotals workerTotals = request.getWorkerTotals();
    	CompanyAdjustmentTotals priorAdjustments = request.getCompanyTotals();

		sLogger.info("Creating Context and Company for Liability Adjustments.");

    	AdjustmentContext adjustmentContext = new AdjustmentContext(employer);
		CompanyFactory.createCompany(adjustmentContext);
		CompanyFactory.applyTaxRatesToCompany(adjustmentContext);
    	adjustmentContext.setWorkerTotals(workerTotals);
		adjustmentContext.setCompanyTotals(priorAdjustments);

		PayrollRun payrollRun = request.getPayrollRun();
		AdjustmentResponse response = new AdjustmentResponse();
		//In the case of Void (which triggers potential employee adjustments, run the employee first, then company.
		if (request.getAdjustmentTrigger() == AdjustmentTrigger.PAYCHECK_VOID) {
			//Find the voided check and affected employee
			
			final Collection<AdjustmentIf> employeeAdjustments = EmployeeAdjustmentController.createEmployeeAdjustments(adjustmentContext);
			if (employeeAdjustments != null) {
				List<WorkerAdjustment> adjustments = EmployeeAdjustmentMapper.mapEmployeeAdjustments(employeeAdjustments, adjustmentContext);
				for (WorkerAdjustment workerAdjustment : adjustments) {
					workerAdjustment.setEffectiveDate(DateUtil.getNowDate());
					workerAdjustment.setPayrollPeriod(payrollRun.getPayPeriod());
					workerAdjustment.setTxnDate(DateUtil.getNowDate());
					//FIXME relate to worker.
				}
			}
		}
		
		//Always run company adjustments.
		final Collection<AdjustmentIf> companyAdjustments = 
			EmployerAdjustmentController.createCompanyRoundingAdjustments(adjustmentContext);
		if (companyAdjustments != null) {
			EmployerAdjustment employerAdjustment = EmployerAdjustmentMapper.mapCompanyAdjustments(companyAdjustments, adjustmentContext, request.getPayrollItemMap());
			if (employerAdjustment != null) {
				//Note: this is the current policy, may change.
				employerAdjustment.setEffectiveDate(payrollRun.getPayDate());
				employerAdjustment.setPayrollPeriod(payrollRun.getPayPeriod());
				//FIXME use System Date
				employerAdjustment.setTxnDate(DateUtil.getNowDate());
				//Sets the bi-di relationship.
				payrollRun.addEmployerAdjustment(employerAdjustment);
				response.getEmployerAdjustments().add(employerAdjustment);
			}
		}
		
		return response;
	}

	private Paycheck createDomainPaycheck(PayrollRunInputs payRunInputs, PaycheckInputs paycheckInputs,
			final Worker worker) {
		final Paycheck paycheck = new Paycheck();
		paycheck.setPayPeriod(payRunInputs.getPayrollRunInfo().getPayPeriod());
		paycheck.setPayDate(payRunInputs.getPayrollRunInfo().getPayDate());
		
		//FIXME try to use the System Date PayrollService.getInstance().getSystemDate().getTime()
		paycheck.setTxnDate(DateUtil.getNowDate());
		paycheck.setDistributionMethod(paycheckInputs.getDistributionMethod());
		paycheck.setWorker(worker); // Sets one end of the relationship - set both when persisting.
		
		//Note: it's ok to not have inputs, we use the worker defaults.  Hence the lack of an else.
		if (paycheckInputs.getCalculationInputs() != null) {
		    for (CalculationInputs calculationInputs : paycheckInputs.getCalculationInputs()) {
				PaycheckLineItem compensationItem = createPaycheckLineItemFromInputs(paycheck, calculationInputs, worker);
		        if (compensationItem != null) {
		        	paycheck.addPaycheckLineItem(compensationItem);
		        }
		    }
		}
		return paycheck;
	}

	
	/**
	 * FIXME [th] this is an atrociously complex method, factor out discrete functions.
	 * @param paycheck
	 * @param calculationInputs
	 * @param worker
	 * @return
	 */
	private PaycheckLineItem createPaycheckLineItemFromInputs(Paycheck paycheck,
			CalculationInputs calculationInputs, Worker worker) {

		if (calculationInputs != null) {
			long compensationId = calculationInputs.getSourcePayrollItemDtlId();
			PayrollItemDtl payrollItemDtl = worker.findPayrollItemById(compensationId);
			if (null == payrollItemDtl) {
				throw new IllegalStateException("The payrollItemId does not correspond to a payroll item on the worker");
			}
			PaycheckLineItem lineItem = new PaycheckLineItem();
			lineItem.setSourcePayrollItemDtl(payrollItemDtl);
			lineItem.setLawId(payrollItemDtl.getPayrollItem().getLawId());
			lineItem.setName(payrollItemDtl.getDescription());
			lineItem.setPayPeriod(paycheck.getPayPeriod());
			//Quantity (may be null)
			lineItem.setQuantity(calculationInputs.getQuantity());
			lineItem.setDeleted(payrollItemDtl.isDeleted());
			
			if (payrollItemDtl instanceof HealthBenefitDtl) {
				//Typically a fixed rate but may have been overridden
				Datum healthRate = calculationInputs.getRate();
				healthRate.setType2(DatumType2Enum.PERPAYCHECK);
				lineItem.setRate(healthRate);
			} else if (payrollItemDtl instanceof DeductionDtl) {
			//FIXME not sure if we need something here.
			} else if (payrollItemDtl instanceof CompensationDtl) {
				CompensationDtl compensationDtl = (CompensationDtl) payrollItemDtl;
				//FIXME [LinkedCompensation]
				PayrollItemDtl linkedPayrollItemDtl = compensationDtl.getLinkedPayrollItemDtl();
				//Some links can be deductions, like cash tips
				if (linkedPayrollItemDtl != null && linkedPayrollItemDtl instanceof CompensationDtl) {
					Datum rate = LinkedCompensationUtils.getMoneyRateFromLinkedPayrollItemDtl(compensationDtl, linkedPayrollItemDtl);
				    lineItem.setRate(rate);
				} else {
					if (compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYREG) {
					    Datum hourlyRate = calculationInputs.getRate();
						// rate is in $ / 1 hour
						hourlyRate.setType2(DatumType2Enum.HOUR);
						hourlyRate.setValue2(new Double(1));
						lineItem.setRate(hourlyRate);
					} else if (compensationDtl.getCompensationType() == CompensationTypeEnum.SICKTAKEN) {
						//Find the base rate.
						CompensationDtl baseCompensationForSick = worker.findPtoBaseRateByType(PtoBaseRateTypeEnum.SICK);
						if (null == baseCompensationForSick) {
							throw new AssertionError("Base rate has not been set for Sick Time Taken");
						}
						// If it's salary, don't bother setting a rate.
						if (baseCompensationForSick.getCompensationType() != CompensationTypeEnum.SALARYREG) {
						    Datum sickRate = baseCompensationForSick.getRate();
							// rate is in $ / 1 hour
						    sickRate.setType2(DatumType2Enum.HOUR);
						    sickRate.setValue2(new Double(1));
							lineItem.setRate(sickRate);
						}
					} else if (compensationDtl.getCompensationType() == CompensationTypeEnum.VACTAKEN) {
						CompensationDtl baseCompensationForVac = worker.findPtoBaseRateByType(PtoBaseRateTypeEnum.VACATION);
						if (null == baseCompensationForVac) {
							throw new AssertionError("Base rate has not been set for Vacation Time Taken");
						}
						// If it's salary, don't bother setting a rate.
						if (baseCompensationForVac.getCompensationType() != CompensationTypeEnum.SALARYREG) {
						    Datum vacRate = baseCompensationForVac.getRate();
							// rate is in $ / 1 hour
						    vacRate.setType2(DatumType2Enum.HOUR);
						    vacRate.setValue2(new Double(1));
							lineItem.setRate(vacRate);
						}
					} else {
						Datum otherRate = calculationInputs.getRate();
						otherRate.setType2(DatumType2Enum.NOTAPPLICABLE);
						lineItem.setRate(otherRate);
					}
				}
			}
			return lineItem;
		}
		return null;
	}
}