package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine;

import com.intuit.payroll.j.HasPayrollItemIf;
import com.intuit.payroll.j.PayrollItemListInfoIf;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.payroll.types.enumtype.PaycheckSetupType;

/**
 * Filter for a {@link QBOEFilteredIterator} over a collection objects
 * that support the {@link PayrollItemListInfoIf} or {@link HasPayrollItemIf} interfaces
 * <p>
 * For performance reasons uses {@link PayrollItemListInfoIf} (if available), without
 * invoking {@link PayrollItemListInfoIf#getPayrollItem}
 */
public class QBOEPayrollItemFilter implements QBOEFilter
{
	private PaycheckSetupType paycheckSetupType;

	// No public construction allowed
	private QBOEPayrollItemFilter ()
	{
	}

	static public QBOEFilter create(PaycheckSetupType paycheckSetupType)
	{
		QBOEPayrollItemFilter payrollItemFilter = new QBOEPayrollItemFilter();
		payrollItemFilter.paycheckSetupType = paycheckSetupType;
		return payrollItemFilter;
	}

	public boolean include(Object o)
	{
		PayrollItemListInfoIf piLite = null;
		if (o instanceof PayrollItemListInfoIf) {
			// PayrollItemIf also come in here.
			piLite = (PayrollItemListInfoIf) o;
		} else if (o instanceof HasPayrollItemIf) {
			// call getPayrollItem() only as a last resort because it is inefficient for QBOE SimplePayrollItem (PayrollItemListInfoIf)
			piLite = ((HasPayrollItemIf)o).getPayrollItem();
		} else {
			String err = o + " is not a valid object for PayrollItemFilter.include()";
			throw new IllegalStateException(err);
		}
		PayrollItemDetailType thatType = piLite.getDetailType();
		return (paycheckSetupType != null && paycheckSetupType == thatType.getPaycheckSetupType());
	}

}
