package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import org.apache.commons.lang.math.RandomUtils;

import com.intuit.dataTypes.Money;
import com.intuit.dataTypes.TimeDuration;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.TimeDurationMapper;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.Worker;

public abstract class AbstractPTOHandler extends AbstractCompensationHandler {

	protected void createImpl(PayrollEngineContext context, Worker worker, EmployeeIf employee, PayrollItemDtl compensationDtl) {
		final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(employee, compensationDtl);
		
		if (compensationDtl.getRate() != null && compensationDtl.getRate().getValue1AsMoneyAmount() != null) {
			Money ptoRateAmount = new Money(compensationDtl.getRate().getValue1AsMoneyAmount().toString());
			qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(ptoRateAmount));
		}

		// YTD
		createYTDTotals(context, worker, compensationDtl, qboeEmployeePayrollItem);
		
	}
	
	protected void handleImpl(PaycheckCalculationContext context, CompensationDtl compensationDtl) {
		final PayrollItemDetailType payrollItemDetailType = PayrollItemDetailType.parseInt(new Integer(compensationDtl
				.getPayrollItem().getLawId().getValue()));
		final PayrollItemIf qboePayrollItem = context.getQboCompany().createPayrollItem(payrollItemDetailType);
		qboePayrollItem.setDatumValue(DatumID.NAME, compensationDtl.getDescription());
		int pItemUniqueId = RandomUtils.nextInt();
		context.getPayrollItemInfo().setPayrollItem(pItemUniqueId, qboePayrollItem);
		qboePayrollItem.finishInit(pItemUniqueId, false);
		final EmployeePayrollItemIf qboeEmployeePayrollItem = context.getQboeEmployee().createEmployeePayrollItem(
				qboePayrollItem);
		PaycheckLineItem paycheckLineItem = findPaycheckLineItemForCompensation(context, compensationDtl);

		final Datum lineItemRate = paycheckLineItem.getRate();
		// The rate for this should have been set based on the appropriate compensation that serves as base.
		// Note: if there is a salary, QBO assumes that is the rate, no need to set it.
		if (lineItemRate != null && lineItemRate.getValue1AsMoneyAmount() != null) {
			Money ptoRateAmount = new Money(lineItemRate.getValue1AsMoneyAmount().toString());
			qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(ptoRateAmount));
		}

		PaycheckLineIf qboePaycheckLine = context.getQboePaycheck().createLine(qboeEmployeePayrollItem);
		Datum ptoQuantity = paycheckLineItem.getQuantity();
		if (ptoQuantity == null || ptoQuantity.getValue1AsQuantity() == null) {
			ptoQuantity = new Datum();
			ptoQuantity.setValue1AsQuantity(new Double(0));
		}
		final TimeDuration timeTaken = TimeDurationMapper.getTimeDuration(ptoQuantity.getValue1AsQuantity());
		qboePaycheckLine.setDatumValue(DatumID.HOURS, timeTaken);
		// YTD
		createYTDTotals(context, context.getWorker(), compensationDtl, qboeEmployeePayrollItem);

		context.getPaycheckLineItemMap().put(qboePaycheckLine, paycheckLineItem);
	}

}
