/**
 * 
 */
package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.util.Set;

import org.apache.log4j.Logger;

import com.intuit.dataTypes.Money;
import com.intuit.dataTypes.QBDate;
import com.intuit.dataTypes.TimeDuration;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.j.PayrollFactory;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.j.PayrollItemTotalsCollectionIf;
import com.intuit.payroll.j.PayrollItemTotalsIf;
import com.intuit.payroll.j.TaxJurisdictionIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.payroll.types.enumtype.TotalsTransactionType;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.NationalRegionMapper;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollItemHandler;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QBOEPaycheckCalculatorBean;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.i18n.LocaleObjectFactory;
import com.intuit.sb.payroll.entity.model.DatumType1Enum;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PayrollItemDtlPtdDatum;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public abstract class AbstractPayrollItemHandler implements PayrollItemHandler {

	protected static Logger sLogger = PayrollLoggerFactory.makeLogger(QBOEPaycheckCalculatorBean.class);
	
	
	/**
	 * This determines the line item that relates to the PayrollItemDtl.
	 * There's some extra hacky code in there because it relies on Persisted
	 * compensations (to use their unique IDs). Some of the tests don't persist
	 * the payroll items, so we stretched the checking a bit.
	 * 
	 * @param context
	 * @param payrollItemDtl
	 * @return
	 */
	protected PaycheckLineItem findPaycheckLineItemForCompensation(PaycheckCalculationContext context,
			PayrollItemDtl payrollItemDtl) {
		
		PaycheckLineItem paycheckLineItemToReturn = null;
		
		for (PaycheckLineItem paycheckLineItem: context.getPaycheck().getPaycheckLineItems()) {
			PayrollItemDtl sourcePayrollItemDtl = paycheckLineItem.getSourcePayrollItemDtl();
			if (null == sourcePayrollItemDtl) {
				sLogger.warn("No SourcePayrollItem for PaycheckLineItem with lawId " + paycheckLineItem.getLawId());
			} else if (payrollItemDtl.getId() != 0L) {
				if (sourcePayrollItemDtl.getId() == payrollItemDtl.getId()) {
					paycheckLineItemToReturn = paycheckLineItem;
					break;
				}
			} else if (sourcePayrollItemDtl.getId() == payrollItemDtl.getId()
					&& sourcePayrollItemDtl.getPayrollItem().getLawId().getValue().equals(
							payrollItemDtl.getPayrollItem().getLawId().getValue())
					&& sourcePayrollItemDtl.getRate().equals(payrollItemDtl.getRate())) {
				paycheckLineItemToReturn = paycheckLineItem;
				break;
			}
		}
		
		if(payrollItemDtl.isDeleted()) {
			EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(context.getQboeEmployee(), payrollItemDtl);
			//payrollItemDtl is deleted. Create a History line and add the PaycheckLineItem to the entity paycheck
			@SuppressWarnings("unused")
			PaycheckLineIf qboePaycheckLine = context.getQboePaycheck().createHistoryLine(qboeEmployeePayrollItem);
			PaycheckLineItem paycheckLineItem = createLineItemFromPayrollItemDtl(context, payrollItemDtl);
			// Add to the context's Paycheck.
			//context.getPaycheck().addPaycheckLineItem(paycheckLineItem);
			paycheckLineItemToReturn = paycheckLineItem;

		}
		if(paycheckLineItemToReturn == null) {
			// salary, fixed commissions, bonuses etc. don't require an override
			PaycheckLineItem paycheckLineItem = createLineItemFromPayrollItemDtl(context, payrollItemDtl);
			// Add to the context's Paycheck.
			context.getPaycheck().addPaycheckLineItem(paycheckLineItem);
			paycheckLineItemToReturn = paycheckLineItem;
		}
		return paycheckLineItemToReturn; 
	}
	
	protected PaycheckLineItem createLineItemFromPayrollItemDtl(PaycheckCalculationContext context,
			final PayrollItemDtl payrollItemDtl) {
		final PaycheckLineItem paycheckLineItem = new PaycheckLineItem();
		paycheckLineItem.setSourcePayrollItemDtl(payrollItemDtl);
		paycheckLineItem.setLawId(payrollItemDtl.getPayrollItem().getLawId());
		paycheckLineItem.setName(payrollItemDtl.getDescription());
		paycheckLineItem.setPayPeriod(context.getPaycheck().getPayPeriod());
		paycheckLineItem.setDeleted(payrollItemDtl.isDeleted());

		return paycheckLineItem;
	}
	
	protected void createYTDTotals(PayrollEngineContext context, Worker worker, PayrollItemDtl payrollItemDtl, EmployeePayrollItemIf qboeEmployeePayrollItem) {
		
		PayrollItemTotalsCollectionIf totals = qboeEmployeePayrollItem.getEmployee().getTotalsCollection();
		PayrollItemIf qboePayrollItem = qboeEmployeePayrollItem.getPayrollItem();
		PayrollItemTotalsIf itemTotals = totals.getTotals(qboePayrollItem);
		if (null == itemTotals) {
			itemTotals = totals.createTotals(qboePayrollItem);
		}
		
		NationalRegionMapper regionMapper = LocaleObjectFactory.getInstance().newInstance(NationalRegionMapper.class);
		
		final GlobalPostalAddress workAddress = worker.getWorkLocation().getTaxResidenceAddress();
		final TaxJurisdictionIf workJur = PayrollFactory.createTaxJurisdiction(regionMapper.mapToNationalRegion(workAddress), null);
		final GlobalPostalAddress liveAddress = worker.getTaxResidenceAddress();
		final TaxJurisdictionIf liveJur = PayrollFactory.createTaxJurisdiction(regionMapper.mapToNationalRegion(liveAddress), null);
		Set<PayrollItemDtlPtdDatum> ptdData = payrollItemDtl.getPtdDatums();
		
		for(PayrollItemDtlPtdDatum ptdDatum : ptdData) {
			PtdDatumType ptdDatumType = ptdDatum.getDatum();
			
			QBDate periodStart = null;
			QBDate periodEnd = null;
			
			if (null != ptdDatumType.getPeriod() && null != ptdDatumType.getPeriod().getStartDate()) {
				periodStart = new QBDate(ptdDatumType.getPeriod().getStartDate());
				periodEnd = new QBDate(ptdDatumType.getPeriod().getEndDate());
			} else {
				throw new AssertionError("There should be dates on the PTD Datums");
			}
			
			if (ptdDatumType.getDatumType() != DatumType1Enum.MONEY) {
				sLogger.warn("Don't know how to detail with non-Money PTD values");
				
				continue;
			}
			
			Money totalValue = new Money(ptdDatumType.getMoneyValue().getAmount().toString());
			Money pckTotalValue = new Money(ptdDatumType.getMoneyValue().getAmount().toString());
			itemTotals.setTotalFromDB(DatumID.TOTAL_AMOUNT, TotalsTransactionType.ALL, 
					totalValue, 
					periodStart, periodEnd, liveJur, workJur);
			itemTotals.setTotalFromDB(DatumID.TOTAL_AMOUNT, TotalsTransactionType.PAYCHECKS_YTDS, 
					pckTotalValue, 
					periodStart, periodEnd, liveJur, workJur);
			
			Money totalWageBase = new Money(ptdDatumType.getWageBaseValue().getAmount().toString());
			Money pckTotalWageBase = new Money(ptdDatumType.getWageBaseValue().getAmount().toString());
			itemTotals.setTotalFromDB(DatumID.TOTAL_WAGE_BASE, TotalsTransactionType.ALL, 
					totalWageBase, 
					periodStart, periodEnd, liveJur, workJur);
			itemTotals.setTotalFromDB(DatumID.TOTAL_WAGE_BASE, TotalsTransactionType.PAYCHECKS_YTDS, 
					pckTotalWageBase, 
					periodStart, periodEnd, liveJur, workJur);
			
			Money totalIsst = new Money(ptdDatumType.getIncomeSubjectToTaxValue().getAmount().toString());
			Money pckTotalIsst = new Money(ptdDatumType.getIncomeSubjectToTaxValue().getAmount().toString());
			itemTotals.setTotalFromDB(DatumID.TOTAL_INCOME_SUBJECT_TO_TAX, TotalsTransactionType.ALL, 
					totalIsst, 
					periodStart, periodEnd, liveJur, workJur);
			itemTotals.setTotalFromDB(DatumID.TOTAL_INCOME_SUBJECT_TO_TAX, TotalsTransactionType.PAYCHECKS_YTDS, 
					pckTotalIsst, 
					periodStart, periodEnd, liveJur, workJur);
			
			Double totalHoursDbl = ptdDatumType.getTotalHoursValue();
			Double pckTotalHoursDbl = ptdDatumType.getTotalHoursValue();
			
			TimeDuration totalHours = new TimeDuration((int)(totalHoursDbl * DateUtil.SECONDS_PER_HOUR));
			itemTotals.setTotalFromDB(DatumID.TOTAL_HOURS, TotalsTransactionType.ALL, 
					totalHours, 
					periodStart, periodEnd, liveJur, workJur);
			itemTotals.setTotalFromDB(DatumID.TOTAL_HOURS, TotalsTransactionType.PAYCHECKS_YTDS, 
					pckTotalHoursDbl, 
					periodStart, periodEnd, liveJur, workJur);
		}
	}
}