package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.math.BigDecimal;

import com.intuit.dataTypes.Money;
import com.intuit.dataTypes.Quantity;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Handles default behaviour for Quantity rates (like Piecework).
 * 
 * $Author: $ 
 * $Revision: $
 * $DateTime: $
 *
 */
public abstract class AbstractQuantityBasedCompensationHandler extends AbstractCompensationHandler {

	protected void createImpl(PayrollEngineContext context, Worker worker, EmployeeIf employee, PayrollItemDtl compensationDtl) {
		final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(employee, compensationDtl);
		final BigDecimal unitRate = compensationDtl.getRate().getValue1AsMoneyAmount();
		qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(unitRate.toString()));
		// YTD
		createYTDTotals(context, worker, compensationDtl, qboeEmployeePayrollItem);
		
	}
	
	protected void handleImpl(PaycheckCalculationContext context, final CompensationDtl compensationDtl) {
		final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(context.getQboeEmployee(), compensationDtl);
		final PaycheckLineItem paycheckLineItem = findPaycheckLineItemForCompensation(context, compensationDtl);
	
		final BigDecimal unitRate = compensationDtl.getRate().getValue1AsMoneyAmount();
		qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, new Money(unitRate.toString()));
		PaycheckLineIf qboePaycheckLine = context.getQboePaycheck().createLine(qboeEmployeePayrollItem);

		final Datum unitQuantity = paycheckLineItem.getQuantity();
		if (unitQuantity != null) {
			Double unitsProduced = unitQuantity.getValue1AsQuantity();
			qboePaycheckLine.setDatumValue(DatumID.QUANTITY, new Quantity(unitsProduced));
		} else {
			qboePaycheckLine.setDatumValue(DatumID.QUANTITY, new Quantity(0));
		}

		// YTD
		createYTDTotals(context, context.getWorker(), compensationDtl, qboeEmployeePayrollItem);
		
		context.getPaycheckLineItemMap().put(qboePaycheckLine, paycheckLineItem);
	}

}
