package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import org.apache.commons.lang.math.RandomUtils;

import com.intuit.dataTypes.Rate;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.payroll.types.enumtype.CalcBasedOn;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.Worker;

public class CAVacationPayCompensationHandler extends AbstractCompensationHandler {

	private static List<Locale> localeList = new ArrayList<Locale>();
	
	static {
		localeList.add(Locale.CANADA);
	}
	
	/**
	 * FIXME get the law id from CanadaLawId
	 */
	public String getLawId() {
		return "";
	}

	public List<Locale> getLocaleList() {
		return localeList;
	}
	
	public void handle(PaycheckCalculationContext context) {
		Set<CompensationDtl> workerCompensations = context.getWorker().getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.VACATIONPAY) {
				final PayrollItemDetailType payrollItemDetailType = 
					PayrollItemDetailType.parseInt(new Integer(compensationDtl.getPayrollItem().getLawId().getValue()));
				final PayrollItemIf qboePayrollItem = context.getQboCompany().createPayrollItem(payrollItemDetailType);
				qboePayrollItem.setDatumValue(DatumID.NAME, compensationDtl.getDescription());
				int pItemUniqueId = RandomUtils.nextInt();
				context.getPayrollItemInfo().setPayrollItem(pItemUniqueId, qboePayrollItem);
				qboePayrollItem.finishInit(pItemUniqueId, false);
				final EmployeePayrollItemIf qboeEmployeePayrollItem = context.getQboeEmployee().createEmployeePayrollItem(
						qboePayrollItem);
				PaycheckLineItem paycheckLineItem = findPaycheckLineItemForCompensation(context, compensationDtl);
				final Datum lineItemRate = paycheckLineItem.getRate();
				// Vacation Pay is a Canada-specific concept wherein a percentage of the employee's
				// gross wages is added to the total gross in lieu of accrued vacation hours.
				qboePayrollItem.setDatumValue(DatumID.CALC_BASED_ON, CalcBasedOn.GROSS);
				PaycheckLineIf qboePaycheckLine = context.getQboePaycheck().createLine(qboeEmployeePayrollItem);
				if (lineItemRate != null && lineItemRate.getValue1AsPercentage() != null) {
					Rate vacationPayRate = new Rate(lineItemRate.getValue1AsPercentage().toString() + "%");
					qboePaycheckLine.setDatumValue(DatumID.RATE, vacationPayRate);
				}
				// YTD
				createYTDTotals(context, context.getWorker(), compensationDtl, qboeEmployeePayrollItem);

				context.getPaycheckLineItemMap().put(qboePaycheckLine, paycheckLineItem);
			}
		}
	}

	public void createQboPayrollItemsFromPayrollItemDtls(PayrollEngineContext context, EmployeeIf employee,
			Worker worker) {
		Set<CompensationDtl> workerCompensations = worker.getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.VACATIONPAY) {
				final PayrollItemDetailType payrollItemDetailType = 
					PayrollItemDetailType.parseInt(new Integer(compensationDtl.getPayrollItem().getLawId().getValue()));
				final PayrollItemIf qboePayrollItem = context.getQboCompany().createPayrollItem(payrollItemDetailType);
				qboePayrollItem.setDatumValue(DatumID.NAME, compensationDtl.getDescription());
				int pItemUniqueId = RandomUtils.nextInt();
				context.getPayrollItemInfo().setPayrollItem(pItemUniqueId, qboePayrollItem);
				qboePayrollItem.finishInit(pItemUniqueId, false);
				final EmployeePayrollItemIf qboeEmployeePayrollItem = employee.createEmployeePayrollItem(
						qboePayrollItem);
				
				final Datum lineItemRate = compensationDtl.getRate();
				// Vacation Pay is a Canada-specific concept wherein a percentage of the employee's
				// gross wages is added to the total gross in lieu of accrued vacation hours.
				qboePayrollItem.setDatumValue(DatumID.CALC_BASED_ON, CalcBasedOn.GROSS);
				if (lineItemRate != null && lineItemRate.getValue1AsPercentage() != null) {
					Rate vacationPayRate = new Rate(lineItemRate.getValue1AsPercentage().toString() + "%");
					qboePayrollItem.setDatumValue(DatumID.RATE, vacationPayRate);
				}
				// YTD
				createYTDTotals(context, worker, compensationDtl, qboeEmployeePayrollItem);

			}
		}
		
	}

}
