package com.intuit.sb.payroll.common;

import java.util.ArrayList;
import java.util.List;

import com.intuit.sb.payroll.TaskDesc;
import com.intuit.sb.payroll.WorkflowException;

/**
 * Generic utililties for handling exceptions.
 */
public class ExceptionUtils
{
	@SuppressWarnings("nls")
    private static final String COLON_STR = ": ";

	/**
     * Returns a string containing the throwable message or, if it's null,
     * the string representation of the throwable (i.e. the exception
     * class name).
     * @param t the throwable
     * @return a non-null message from the throwable
     */
    public static String throwableMessage(Throwable t)
    {
        return (t.getMessage() != null) ? t.getMessage() : t.toString();
    }

    /**
     * Returns a string containing all the nested messages in the throwable or, if it's null,
     * the string representation of the throwable (i.e. the exception class name).
     * @param t the throwable
     * @return a non-null message from the throwable
     */
    public static String throwableMessageChain(Throwable t)
    {
        StringBuffer mesg = new StringBuffer(throwableMessage(t));
        for (Throwable chainedThrowable = t.getCause(); chainedThrowable != null; chainedThrowable = chainedThrowable.getCause()) {
        	mesg.append(COLON_STR).append(throwableMessage(chainedThrowable));
        }
    	return mesg.toString();
    }

	public static void generateWorkflowExceptionFromRequiredData(List<RequiredData> requiredData) throws WorkflowException {
		StringBuilder builder = new StringBuilder();
		List<TaskDesc> tasksToComplete = new ArrayList<TaskDesc>();
		for (RequiredData data : requiredData) {
			builder.append(" [Required Data: " + data.getDescription() + "]");
			tasksToComplete.add(new TaskDesc(data.getKey(), data.getDescription()));
		}
		if (tasksToComplete.size() == 0) {
			throw new IllegalStateException("Required data is there, worker has probably not been re-evaluated.");
		}
		throw new WorkflowException("Complete the following data: " + builder.toString(), tasksToComplete);
	}

}