package com.intuit.sb.payroll.common.validation;

import java.math.BigDecimal;
import java.util.List;

import com.intuit.sb.payroll.ValidationException;

/**
 * Base class for Entity Validation classes.
 * 
 * Provides an interface and helper methods for classes that seek to validate one or many entities.
 *
 * $Author: jzhong $
 * $Revision: #1 $
 * $DateTime: 2008/08/15 17:27:47 $
 *
 * @param <E>
 */
public abstract class AbstractEntityDataValidator<E>
{
	
	/**
	 * Validates a collection of Entities.
	 * 
	 * @param entityCollection
	 * @throws PayrollValidationException
	 */
	public abstract void validateEntityCollection(List<E> entityCollection) throws ValidationException;
	
	/**
	 * Validates one Entity.
	 * 
	 * @param entity
	 * @throws ValidationException
	 */
	public abstract void validateEntity(E entity) throws ValidationException;
	
	/**
	 * Validate a BigDecimal is not null, and is >= zero.
	 * 
	 * @throws IllegalArgumentException
	 */
	protected void checkIsNonNegative(BigDecimal variable, String variableName) throws ValidationException
	{
		PayrollParamValidator.checkIsNotNull(variable, variableName);
		PayrollParamValidator.checkIsNonNegative(variable, variableName);
	}
}
