package com.intuit.sb.payroll;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertNull;
import static org.testng.AssertJUnit.assertTrue;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.testng.annotations.Test;

public class ExceptionTest
{

    @Test
    public void testServiceFailureException() {
    	ServiceFailureException exception = new ServiceFailureException();
        assertTrue(exception instanceof Exception);
        assertNull(exception.getMessage());
        assertNull(exception.getCause());

        exception = new ServiceFailureException("test message");
        assertEquals("test message", exception.getMessage());
        assertNull(exception.getCause());

        Throwable throwable = new Throwable();
        exception = new ServiceFailureException(throwable); // message defaults to Throwable class name
        assertEquals("java.lang.Throwable", exception.getMessage());
        assertEquals(throwable, exception.getCause());

        exception = new ServiceFailureException("test message", throwable);
        assertEquals("test message", exception.getMessage());
        assertEquals(throwable, exception.getCause());
    }
	
    @Test
	public void testTask() {
		TaskDesc task = new TaskDesc();
		assertTrue(task instanceof Serializable);
		assertNull(task.getName());
		assertNull(task.getKey());
		
		task.setName("NAME");
		assertEquals("NAME", task.getName());
		
		task.setKey("KEY");
		assertEquals("KEY", task.getKey());
		
		task = new TaskDesc("key", "name");
		assertEquals("name", task.getName());
		assertEquals("key", task.getKey());
	}
	
	@Test
	public void testWorkflowException() {
		WorkflowException exception = new WorkflowException();
		assertTrue(exception instanceof Exception);
		assertNull(exception.getMessage());
		assertNull(exception.getCause());

		exception = new WorkflowException("test message");
		assertEquals("test message", exception.getMessage());
		assertNull(exception.getCause());

		List<TaskDesc> tasksToComplete = new ArrayList<TaskDesc>();
		tasksToComplete.add(new TaskDesc());
		exception = new WorkflowException("test message", tasksToComplete);
		assertEquals("test message", exception.getMessage());
		assertNull(exception.getCause());
		assertEquals(1, exception.getTasksToComplete().size());
		assertEquals(tasksToComplete, exception.getTasksToComplete());

		Throwable throwable = new Throwable();
		exception = new WorkflowException(throwable); // message defaults to Throwable class name
		assertEquals("java.lang.Throwable", exception.getMessage());
		assertEquals(throwable, exception.getCause());

		exception = new WorkflowException("test message", throwable);
		assertEquals("test message", exception.getMessage());
		assertEquals(throwable, exception.getCause());
		
		assertEquals(0, exception.getTasksToComplete().size());
		exception.setTasksToComplete(tasksToComplete);
		assertEquals(1, exception.getTasksToComplete().size());
		assertEquals(tasksToComplete, exception.getTasksToComplete());
	}
	@Test (expectedExceptions=IllegalArgumentException.class)
	public void testWorkflowException2() {
		WorkflowException exception = new WorkflowException();
		exception.setTasksToComplete(null);
	}
	
	@Test
	public void testValidationException() {
		ValidationException exception = new ValidationException();
		assertTrue(exception instanceof Exception);
		assertNull(exception.getMessage());
		assertNull(exception.getCause());

		exception = new ValidationException("test message");
		assertEquals("test message", exception.getMessage());
		assertNull(exception.getCause());

		Throwable throwable = new Throwable();
		exception = new ValidationException(throwable); // message defaults to Throwable class name
		assertEquals("java.lang.Throwable", exception.getMessage());
		assertEquals(throwable, exception.getCause());

		exception = new ValidationException("test message", throwable);
		assertEquals("test message", exception.getMessage());
		assertEquals(throwable, exception.getCause());

	}
}
