package com.intuit.sb.payroll.common;


import java.io.FileOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;

import org.testng.ITestContext;
import org.testng.ITestResult;
import org.testng.TestListenerAdapter;

public class CustomTestListener extends TestListenerAdapter {

  static private final String outputFileName = "target" + File.separator + "TestSuite-output.txt";
  static private PrintStream out;
  private PrintStream console = System.out;
  @SuppressWarnings("unchecked")
  private Class prevTestClass = null;
  @SuppressWarnings("unchecked")
  private Class currentTestClass;
  private int successCount;
  private int failureCount;
  private int skipCount;
  private String summary;
  
  public CustomTestListener()
  {
	  try {
		  out = new PrintStream(new FileOutputStream(outputFileName));
	  }
	  catch (IOException ex)
	  {
		  throw new RuntimeException("can not write to " + outputFileName, ex);
	  }
	  System.setOut(out);
	  System.setErr(out);
  }
  
  @SuppressWarnings("unchecked")
  @Override
  public void onTestStart(ITestResult result)
  {
	  currentTestClass = result.getTestClass().getRealClass();
	  System.setOut(console);
	  //if currentTestClass does not equal to prevTestClass, tests in a new Class are starting
	  if (  currentTestClass != prevTestClass )
	  {
		  if ( successCount + skipCount + failureCount != 0 )
		  {
			  System.out.println(summary);
		  }
		  System.out.println("Running tests in " + currentTestClass.getCanonicalName());
		  prevTestClass = currentTestClass;
		  successCount = 0;
		  failureCount = 0;
		  skipCount = 0;
	  }
	  
	  
	  System.setOut(out);
	  System.setErr(out);
	  
  }
  
  @Override
  public void onTestFailure(ITestResult tr) {
	  failureCount++;
	  logSummary();
  }

  @Override
  public void onTestSkipped(ITestResult tr) {
	  skipCount++;
	  logSummary();
  }

  public void onTestSuccess(ITestResult tr)  {
	  successCount++;
	  logSummary();
  }
  
  @Override
  public void onFinish(ITestContext testContext) {
	  
	  System.setOut(console);
	  System.out.println(summary);

	  System.setOut(out);
	  System.setErr(out);
	  
  }

  private void logSummary() { 
	  summary = successCount + " tests passed " + failureCount + " failed " + skipCount + " skipped"; 
  }
} 