package com.intuit.sb.payroll.common.i18n;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.InfrastructureException;

public class LocaleObjectFactoryTest 
{
	@BeforeMethod
	public void beforeTest()
	{
		LocaleObjectFactory.clear();
	}
	
	@Test(expectedExceptions=InfrastructureException.class)
	public void testNoConfigSpecified() throws Exception
	{
		LocaleObjectFactory.getInstance();
	}
	
	@Test(expectedExceptions=InfrastructureException.class)
	public void testMissingConfigResource() throws Exception
	{
		System.clearProperty(LocaleObjectFactory.CONFIG_PROPERTY);
		
		LocaleObjectFactory.getInstance();
	}
	
	@Test(expectedExceptions=InfrastructureException.class)
	public void testAbstractClassNotFound() throws Exception
	{
		System.setProperty(LocaleObjectFactory.CONFIG_PROPERTY, "missingclass-objfactory-test-config.properties");
		
		LocaleObjectFactory.getInstance();
	}
	
	@Test(expectedExceptions=InfrastructureException.class)
	public void testClassesNotAssignable() throws Exception
	{
		System.setProperty(LocaleObjectFactory.CONFIG_PROPERTY, "nonassignable-objfactory-test-config.properties");
		
		LocaleObjectFactory.getInstance();
	}
	
	@Test(expectedExceptions=InfrastructureException.class)
	public void testClassesNotInstantiatable() throws Exception
	{
		System.setProperty(LocaleObjectFactory.CONFIG_PROPERTY, "noninstantiable-objfactory-test-config.properties");
		
		LocaleObjectFactory.getInstance().newInstance(AbstractClass.class);
	}
	
	@Test
	public void testGetClass() throws Exception
	{
		System.setProperty(LocaleObjectFactory.CONFIG_PROPERTY, "valid-objfactory-test-config.properties");
		
		LocaleObjectFactory objectFactory = LocaleObjectFactory.getInstance();
		
		Class<?> clazz = objectFactory.getClass(AbstractClass.class);
		
		AssertJUnit.assertNotNull(clazz);
		
		AssertJUnit.assertTrue(AbstractClass.class.isAssignableFrom(clazz));
	}
	
	@Test
	public void testNewInstance() throws Exception
	{
		System.setProperty(LocaleObjectFactory.CONFIG_PROPERTY, "valid-objfactory-test-config.properties");
		
		LocaleObjectFactory objectFactory = LocaleObjectFactory.getInstance();
		
		AbstractClass obj = objectFactory.newInstance(AbstractClass.class);
		
		AssertJUnit.assertNotNull(obj);
	}
}
