package com.intuit.sb.payroll.entity.model;

import java.math.BigDecimal;
import java.text.MessageFormat;

import javax.persistence.Enumerated;
import javax.persistence.Transient;

import com.intuit.sb.cdm.basetypes.CurrencyCode;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;

@javax.persistence.Embeddable
public class Money extends MoneyBase {

	private static final long serialVersionUID = 1L;
	// FIXME [I18n]
	private static final int TWO_DECIMALS = 2;
	
	public Money() {

		super();

		BigDecimal amount;
		// FIXME [I18n] set the initial value and scale
		amount = new BigDecimal("0.00");
		amount = amount.setScale(TWO_DECIMALS);
		super.setAmount(amount);
		String currencyCode = LocaleGuru.getInstance().getCurrency();
		CurrencyCode cc = CurrencyCode.valueOf(currencyCode);
		super.setCurrencyCode(cc);
	}
	
	/**
     * @see com.intuit.sb.basetypes.Money#getCurrencyCode()
     */
	@Override
    @Enumerated(javax.persistence.EnumType.STRING)
    public CurrencyCode getCurrencyCode()
    {
        return super.getCurrencyCode();
    }

    /**
     * @see com.intuit.sb.basetypes.Money#setCurrencyCode(CurrencyCode)
     */
	@Override
    public void setCurrencyCode(CurrencyCode currencyCode)
    {
       super.setCurrencyCode(currencyCode);
    }

    /**
     * @see com.intuit.sb.basetypes.Money#getAmount()
     */
	@Override
    public BigDecimal getAmount()
    {
		// FIXME intended for debug
		if (TWO_DECIMALS != super.getAmount().scale()) {
			throw new IllegalStateException("Illegal scale value for Money :" + super.getAmount().scale());							
		}
		return super.getAmount();
    }

    /**
     * @see com.intuit.sb.basetypes.Money#setAmount(BigDecimal)
     */
	@Override
    public void setAmount(BigDecimal amount)
    {
		amount = amount.setScale(TWO_DECIMALS);
		// FIXME [I18n] Enforce the scale setting
        super.setAmount(amount);
    }
    
    @Transient
    public java.util.Currency getCurrency() {
    	return java.util.Currency.getInstance(getCurrencyCode().value());
    }

	@Transient
	public com.intuit.sb.cdm.basetypes.Money getMoney() {
		// FIXME intended for debug
		if (TWO_DECIMALS != super.getAmount().scale()) {
			throw new IllegalStateException("Illegal scale value for Money :" + super.getAmount().scale());							
		}
		return this;
	}

	@Transient
	public void setCurrencyCode(java.util.Currency currencyCode) throws IllegalArgumentException {
		CurrencyCode code = CurrencyCode.valueOf(currencyCode.getCurrencyCode());
		super.setCurrencyCode(code);
	}
	
    // ********************** Constructors (Not generated) ********************** //
	
	/**
	 * Constructs a Money object for a given {currency, amount} pair. The amount
	 * must conform to the requirements of {@code java.math.BigDecimal} string values.
	 * @param currencyCode the ISO 4217 currency code for the new money.
	 * @param amount the stringified form of the amount for the new money.
	 */
	public Money(String currencyCode, String strAmount) throws IllegalArgumentException {
		super.setCurrencyCode(CurrencyCode.valueOf(currencyCode));
		// BigDecimal can't handle an empty string, so set it to "0.00"
		if (strAmount.equals("")) {
		    strAmount = "0.00";
		}
		BigDecimal amount = new BigDecimal(strAmount);
		amount = amount.setScale(TWO_DECIMALS);
		super.setAmount(amount);
	}
	
	/**
	 * Constructs a Money for the default currency according the the Locale Settings.
	 * The amount must conform to the requirements of {@code java.math.BigDecimal} string values.
	 * @param strAmount
	 * @throws IllegalArgumentException
	 */
	public Money(String strAmount) throws IllegalArgumentException {
		String currencyCode = LocaleGuru.getInstance().getCurrency();
		CurrencyCode cc = CurrencyCode.valueOf(currencyCode);
		super.setCurrencyCode(cc);
		
		// BigDecimal can't handle an empty string, so set it to "0.00"
		if (strAmount.equals("")) {
		    strAmount = "0.00";
		}
		BigDecimal amount = new BigDecimal(strAmount);
		amount = amount.setScale(TWO_DECIMALS);
		super.setAmount(amount);
	}

	/**
	 * Adds another {@code Money} amount to this {@code Money} and returns a new
	 * {@code Money} containing the total. This method does not update this {@code Money}
	 * object.
	 * @param amountToAdd the {@code Money} to add.
	 * @return the new {@code Money} containing the total.
	 */
	@Transient
	public Money add(Money amountToAdd) throws IllegalArgumentException {
		
		if (!super.getCurrencyCode().equals(amountToAdd.getCurrencyCode())) {
			String msg = MessageFormat.format("Currency conversion not support for addition of {0} to {1}.",
					super.getCurrencyCode().toString(), amountToAdd.getCurrencyCode().toString());
			throw new IllegalArgumentException(msg);
		}
		
		BigDecimal totalAmount = new BigDecimal("0.00");
		totalAmount = super.getAmount();
		totalAmount = totalAmount.add(amountToAdd.getAmount());
		totalAmount = totalAmount.setScale(TWO_DECIMALS);

		Money total = new Money();
		total.setCurrencyCode(super.getCurrencyCode());
		total.setAmount(totalAmount);

		// FIXME intended for debug
		if (TWO_DECIMALS != total.getAmount().scale()) {
			throw new IllegalStateException("Illegal scale value for Money :" + total.getAmount().scale());							
		}
		
		return total;
	}
	
	/**
	 * Subtracts another {@code Money} amount to this {@code Money} and returns a new
	 * {@code Money} containing the total. This method does not update this {@code Money}
	 * object.
	 * @param amountToSubtract the {@code Money} to subtract.
	 * @return the new {@code Money} containing the total.
	 */
	@Transient
	public Money subtract(Money amountToSubtract) throws IllegalArgumentException {
		
		if (!super.getCurrencyCode().equals(amountToSubtract.getCurrencyCode())) {
			String msg = MessageFormat.format("Currency conversion not support for subtraction of {0} to {1}.",
					super.getCurrencyCode().toString(), amountToSubtract.getCurrencyCode().toString());
			throw new IllegalArgumentException(msg);
		}
				
		BigDecimal totalAmount = new BigDecimal("0.00");
		totalAmount = super.getAmount();
		totalAmount = totalAmount.subtract(amountToSubtract.getAmount());
		totalAmount = totalAmount.setScale(TWO_DECIMALS);

		Money total = new Money();
		total.setCurrencyCode(super.getCurrencyCode());
		total.setAmount(totalAmount);

		// FIXME intended for debug
		if (TWO_DECIMALS != total.getAmount().scale()) {
			throw new IllegalStateException("Illegal scale value for Money :" + total.getAmount().scale());							
		}

		return total;
	}


    /**
	 * Creates textual representation of the object content.
	 * The output can be used for debugging, logging, etc., and is meant for internal purposes only.
	 * 
	 * @return string representing object content in textual form
	 */
	@Override
	public String toString() {
		return (super.getCurrencyCode().toString() + super.getAmount().toString());		// USD:50.25
	}

}
