package com.intuit.sb.payroll.entity.model;

import javax.persistence.Transient;


/**
 * PaycheckLineItem is a copy of all the values relevant to a paycheck line item calculation.  It does
 * not point to any other entity, it contains relevant input and calculated values that may have come from
 * other entities.
 */
public class PaycheckLineItem extends PaycheckLineItemBase {

	private static final long serialVersionUID = 1558908053L;
	
	private boolean deleted;
	private boolean historical;

	public PaycheckLineItem() {
		super.setQuantity(null);
		super.setRate(null);
	}
	
	@Transient
	public void setDeleted(boolean deleted) {
		this.deleted = deleted;
	}
	
	@Transient
	public boolean isDeleted() {
		return this.deleted;
	}
	
	@Transient
	public void setHistorical(boolean historical) {
		this.historical = historical;
	}
	
	@Transient
	public boolean isHistorical() {
		return this.historical;
	}

	/**
	 * Returns a handle to the current YTD Datum.  This never returns null, so it can be used
	 * to create the YTD datum and set a value in it for example.
	 * 
	 * @return the YTD Datum.
	 */
	@Transient
	public PaycheckLitemPtdDatum getYtdDatum() {
		PtdTypeEnum type = PtdTypeEnum.YEAR;
		
		return getPtdDatumOfType(type);
	}
	
	/**
	 * Returns a handle to the current Q1 Datum.  This never returns null, so it can be used
	 * to create the Q1 datum and set a value in it for example.
	 * 
	 * @return the Q1 Datum.
	 */
	@Transient
	public PaycheckLitemPtdDatum getQ1Datum() {
		PtdTypeEnum type = PtdTypeEnum.QUARTER1;
		
		return getPtdDatumOfType(type);
	}
	
	@Transient
	public PaycheckLitemPtdDatum getQ2Datum() {
		PtdTypeEnum type = PtdTypeEnum.QUARTER2;
		
		return getPtdDatumOfType(type);
	}
	
	@Transient
	public PaycheckLitemPtdDatum getQ3Datum() {
		PtdTypeEnum type = PtdTypeEnum.QUARTER3;
		
		return getPtdDatumOfType(type);
	}
	
	@Transient
	public PaycheckLitemPtdDatum getQ4Datum() {
		PtdTypeEnum type = PtdTypeEnum.QUARTER4;
		
		return getPtdDatumOfType(type);
	}

	@Transient
	public PaycheckLitemPtdDatum getPtdDatumOfType(PtdTypeEnum type) {
		for (PaycheckLitemPtdDatum ptd : getPtdDatums()) {
			if (null != ptd.getDatum()) {
				if (ptd.getDatum().getPtdType() == type) {
					return ptd;
				}
			}
		}
		
		// Datum did not exist - create it
		PaycheckLitemPtdDatum ptd = new PaycheckLitemPtdDatum();
		PtdDatumType datumType = new PtdDatumType();
		datumType.setPtdType(type);
		ptd.setDatum(datumType);
		addPtdDatum(ptd);
		
		return ptd;
	}
	


}