package com.intuit.sb.payroll.entity.model;

import java.util.Set;

import javax.persistence.Transient;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.ValidationException;



public class PayrollItemDtl extends PayrollItemDtlBase {

	private static final long serialVersionUID = -885328302L;
	
	public PayrollItemDtl() {
		super();
	}

	/**
	 * Returns a handle to the current YTD Datum.  This never returns null, so it can be used
	 * to create the YTD datum and set a value in it for example.
	 * 
	 * @return the YTD Datum.
	 */
	@Transient
	public PtdDatum getYtdDatum() {
		PtdTypeEnum type = PtdTypeEnum.YEAR;
		
		return getPtdDatumOfType(type);
	}
	
	/**
	 * Returns a handle to the current Q1 Datum.  This never returns null, so it can be used
	 * to create the Q1 datum and set a value in it for example.
	 * 
	 * @return the Q1 Datum.
	 */
	@Transient
	public PtdDatum getQ1Datum() {
		PtdTypeEnum type = PtdTypeEnum.QUARTER1;
		
		return getPtdDatumOfType(type);
	}
	
	@Transient
	public PtdDatum getQ2Datum() {
		PtdTypeEnum type = PtdTypeEnum.QUARTER2;
		
		return getPtdDatumOfType(type);
	}
	
	@Transient
	public PtdDatum getQ3Datum() {
		PtdTypeEnum type = PtdTypeEnum.QUARTER3;
		
		return getPtdDatumOfType(type);
	}
	
	@Transient
	public PtdDatum getQ4Datum() {
		PtdTypeEnum type = PtdTypeEnum.QUARTER4;
		
		return getPtdDatumOfType(type);
	}

	@Transient
	public PayrollItemDtlPtdDatum getPtdDatumOfType(PtdTypeEnum type) {
		for (PayrollItemDtlPtdDatum ptd : this.getPtdDatums()) {
			if (null != ptd.getDatum()) {
				if (ptd.getDatum().getPtdType() == type) {
					return ptd;
				}
			}
		}
		
		// Datum did not exist - create it
		PayrollItemDtlPtdDatum ptd = new PayrollItemDtlPtdDatum();
		PtdDatumType datumType = new PtdDatumType();
		datumType.setPtdType(type);
		ptd.setDatum(datumType);
		this.getPtdDatums().add(ptd);
		
		return ptd;
	}
	
	/**
	 * If the DTL has any PTD information (ie, was part of a paycheck) you can't delete it, you
	 * must set it to deleted.
	 * 
	 * @param payrollItemDtl
	 * @throws ValidationException
	 */
	public void validateNoPeriodToDateValues() throws ValidationException {
		Set<PayrollItemDtlPtdDatum> ptdDatums = this.getPtdDatums();
		for (PayrollItemDtlPtdDatum ptdDatum : ptdDatums) {
			if (ptdDatum.getDatum() != null) {
				this.setDeleted(true);
				//throw new ValidationException("You can't remove a pay item if there are PTD values.  Set it as inactive.");
			}
		}
	}
	
	/**
	 * Creates textual representation of the object content.
	 * The output can be used for debugging, logging, etc., and is meant for internal purposes only.
	 * 
	 * @return string representing object content in textual form
	 */
	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.SIMPLE_STYLE);
	}
	
    public String toStringVerbose()
    {
        return toString() +
        "[" + getDescription() + "]";
    }

}