package com.intuit.sb.payroll.entity.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Transient;


/**
 * Worker is an abstract class which defines attributes common to both {@link Employee} and {@link NonEmployee}.
 * A worker is anyone who engages in an employment relationship with an {@link Employer}. 
 */
public class Worker extends WorkerBase {
	
	private static final long serialVersionUID = 263154412L;
	
	private Collection<WorkerComplianceDocDtl> withholdingDocsForCalc = new ArrayList<WorkerComplianceDocDtl>();

	/**
	 * Default constructor.
	 */
	public Worker() {
	}

	@Transient
	public java.util.Set<OtherBenefitDtl> getOtherBenefitDtls() {
		return getItemsOfType(OtherBenefitDtl.class);
	}

	public void addOtherBenefitDtl(OtherBenefitDtl otherBenefitDtl) {
		getPayrollItemDtls().add(otherBenefitDtl);
	}

	public void removeOtherBenefitDtl(OtherBenefitDtl otherBenefitDtl) {
		getPayrollItemDtls().remove(otherBenefitDtl);
	}

	@Transient
	public java.util.Set<HealthBenefitDtl> getHealthBenefitDtls() {
		return getItemsOfType(HealthBenefitDtl.class);
	}

	public void addHealthBenefitDtl(HealthBenefitDtl healthBenefitDtl) {
		getPayrollItemDtls().add(healthBenefitDtl);
	}

	public void removeHealthBenefitDtl(HealthBenefitDtl healthBenefitDtl) {
		getPayrollItemDtls().remove(healthBenefitDtl);
	}

	@Transient
	public java.util.Set<RetirementBenefitDtl> getRetirementBenefitDtls() {
		return getItemsOfType(RetirementBenefitDtl.class);
	}

	public void addRetirementBenefitDtl(RetirementBenefitDtl retirementBenefitDtl) {
		getPayrollItemDtls().add(retirementBenefitDtl);
	}

	public void removeRetirementBenefitDtl(RetirementBenefitDtl retirementBenefitDtl) {
		getPayrollItemDtls().remove(retirementBenefitDtl);
	}

	@Transient
	public java.util.Set<AdditionDtl> getAdditionDtls() {
		return getItemsOfType(AdditionDtl.class);
	}

	public void addAdditionDtl(AdditionDtl additionDtl) {
		getPayrollItemDtls().add(additionDtl);
	}

	public void removeAdditionDtl(AdditionDtl additionDtl) {
		getPayrollItemDtls().remove(additionDtl);
	}

	@Transient
	public java.util.Set<DeductionDtl> getDeductionDtls() {
		return getItemsOfType(DeductionDtl.class);
	}

	public void addDeductionDtl(DeductionDtl deductionDtl) {
		getPayrollItemDtls().add(deductionDtl);
	}

	public void removeDeductionDtl(DeductionDtl deductionDtl) {
		getPayrollItemDtls().remove(deductionDtl);
	}

	@Transient
	public java.util.Set<PaidTimeOffDtl> getPaidTimeOffDtls() {
		return getItemsOfType(PaidTimeOffDtl.class);
	}
	
	public void addPaidTimeOffDtl(PaidTimeOffDtl paidTimeOffDtl) {
		getPayrollItemDtls().add(paidTimeOffDtl);
	}
	
	public void removePaidTimeOffDtl(PaidTimeOffDtl paidTimeOffDtl) {
		getPayrollItemDtls().remove(paidTimeOffDtl);
	}
	
	@Transient
	public java.util.Set<TaxDtl> getTaxDtls() {
		return getItemsOfType(TaxDtl.class);
	}

	public void addTaxDtl(TaxDtl taxDtl) {
		getPayrollItemDtls().add(taxDtl);
	}

	public void removeTaxDtl(TaxDtl taxDtl) {
		getPayrollItemDtls().remove(taxDtl);
	}

	@Transient
	public java.util.Set<CompensationDtl> getCompensationDtls() {
		return getItemsOfType(CompensationDtl.class);
	}

	public void addCompensationDtl(CompensationDtl compensationDtl) {
		getPayrollItemDtls().add(compensationDtl);
	}

	public void removeCompensationDtl(CompensationDtl compensationDtl) {
		getPayrollItemDtls().remove(compensationDtl);
	}
	
	/**
	 * Transient collection to allow for the Service to add runtime values that do not affect persistence.
	 * FIXME [th] may be US-specific
	 * @return
	 */
	@Transient
	public Collection<WorkerComplianceDocDtl> getWithholdingDocsForCalc() {
		return this.withholdingDocsForCalc;
	}
	
	public void setWithholdingDocsForCalc(Collection<WorkerComplianceDocDtl> withholdingDocs) {
		this.withholdingDocsForCalc = withholdingDocs;
	}
	
//	//TODO AStec: 
//	// Code below is a prototype and should eventually be generated from Anduril.
//	// Boundary conditions for dates need careful examination and will likely require 
//	// code changes. In addition, the code will need to enforce that effective periods 
//	// do not overlap; if they do exception needs to be thrown. 
//	//FIXME always use DateUtil or LocaleGuru for date work = how do we tell Anduril this?.
//	// Assumption: the start date in the non-null employment period is *always* set, else
//	// it is an error (throw exception?). EndDate == null indicates the current period.
//	// For performance the current value may be on the entity and in the history collection!!!
//
//	@Override
//	//@Transient
//    public void setEmploymentPeriod(DateRange employmentPeriod)
//    {
//		// if the current value exists, add it to history and then replace with the new one
//		if (null != super.getEmploymentPeriod()
//				&& null != super.getEmploymentPeriod().getStartDate()) {
//	        DateRange dateRange;
//	        dateRange = super.getEmploymentPeriod().contentCopy();
//			// If necessary, fix the end date for the period being saved
//	        // by subtracting 1 day. 
//			if (null == dateRange.getEndDate()) {
//				Calendar calendar = new GregorianCalendar();
//				calendar.setTime(employmentPeriod.getStartDate());
//				calendar.roll(Calendar.DATE, -1);
//				dateRange.setEndDate(calendar.getTime());
//			}
//			EmploymentPeriod empPeriod = new EmploymentPeriod();
//			empPeriod.setEmploymentPeriod(dateRange);
//			empPeriod.setWorker(this);
//			super.getEmploymentPeriods().add(empPeriod);
//		}
//		// now that we have saved the old value, let's save the new value
//        super.setEmploymentPeriod(employmentPeriod);
//    }
//
//	//TODO AStec 04012009 not tested
//	@Transient
//	public Boolean employedAsOf(Date date) {
//		Boolean found = new Boolean(false);
//		// Check the current value first
//    	if (null == getEmploymentPeriod().getEndDate()
//    			&& (getEmploymentPeriod().getStartDate().before(date)
//    				|| getEmploymentPeriod().getStartDate().equals(date))) {
//    		found = true;
//    	} else if ((getEmploymentPeriod().getStartDate().before(date))
//    				|| (getEmploymentPeriod().getStartDate().equals(date))
//    			&& (date.before(getEmploymentPeriod().getEndDate())
//    				|| date.equals(getEmploymentPeriod().getStartDate()))) {
//    		found = true;
//    	} else {
//    		// get it from history
//			Set<EmploymentPeriod> employmentPeriods = getEmploymentPeriods();
//			if (null != employmentPeriods) {
//				// loop through history entries checking each one
//		        for (EmploymentPeriod period : employmentPeriods) {
//		        	// fyi, the current value may be repeated in the history table
//		        	if (null == period.getEmploymentPeriod().getEndDate()
//		        			&& (period.getEmploymentPeriod().getStartDate().before(date)
//		        				|| period.getEmploymentPeriod().getStartDate().equals(date))) {
//		        		found = true;
//		        		break;
//		        	} else if ((period.getEmploymentPeriod().getStartDate().before(date))
//		        				|| (period.getEmploymentPeriod().getStartDate().equals(date))
//		        			&& (date.before(period.getEmploymentPeriod().getEndDate())
//		        				|| date.equals(getEmploymentPeriod().getStartDate()))) {
//		        		found = true;
//		        		break;  		
//		        	}
//				}
//			}
//    	}
//
//		return found;
//	}	
	
	@SuppressWarnings("unchecked")
	private <T> Set<T> getItemsOfType(Class<T> clazz) {
		Set<T> items = new HashSet<T>();
		for (PayrollItemDtl payrollItemDtl : getPayrollItemDtls()) {
			if (clazz.isInstance(payrollItemDtl)) {
				items.add((T)payrollItemDtl);
			}
		}
		return items;
	}

	public PayrollItemDtl findPayrollItemById(long payrollItemId) {
		Set<PayrollItemDtl> payrollItems = getPayrollItemDtls();
		for (PayrollItemDtl payrollItemDtl : payrollItems) {
			if (payrollItemDtl.getId() == payrollItemId) {
				return payrollItemDtl;
			}
		}
		return null;
	}
	

	public CompensationDtl findPtoBaseRateByType(PtoBaseRateTypeEnum ptoType) {
		Set<CompensationDtl> compensations = getCompensationDtls();
		for (CompensationDtl compensationDtl : compensations) {
			if (compensationDtl.getPtoBaseRateType() == ptoType || 
					compensationDtl.getPtoBaseRateType() == PtoBaseRateTypeEnum.PTO) {
				return compensationDtl;
			}
		}
		return null;
	}

}