package com.intuit.sb.payroll.entity.utils;

import org.apache.commons.lang.StringUtils;

import com.intuit.sb.payroll.entity.model.PersonName;

/**
 * Utility functions to act on the {@link PersonName} entity.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public class PersonNameUtils {

	private static final char SPACE = ' ';
	
	/**
	 * Returns the full name of the person (first, middle, last), ie Alfred E. Newman.
	 * 
	 * Null names are allowed.
	 * 
	 * @param personName
	 * @return a String of the full PersonName.
	 */
	public static String getFullName(PersonName personName) {
		boolean firstExists = false;
		boolean middleExists = false;
		
		StringBuilder strBuilder = new StringBuilder();
		if (!StringUtils.isEmpty(personName.getFirstName())) {
			strBuilder.append(personName.getFirstName());
			firstExists = true;
		}
		
		if (!StringUtils.isEmpty(personName.getMiddleName())) {
			if (firstExists) {
				strBuilder.append(SPACE);
			}
			strBuilder.append(personName.getMiddleName());
			middleExists = true;
		}
		
		if (!StringUtils.isEmpty(personName.getLastName())) {
			if (middleExists || firstExists) {
				strBuilder.append(SPACE);
			}
			strBuilder.append(personName.getLastName());
		}
		return strBuilder.toString();
	}
	
}
