package com.intuit.sb.payroll.message;

import javax.jms.JMSException;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.common.ServiceLocator;


public class PayrollMessageUtil {
	private static final String PAYROLL_MDB_JNDI_LOOKUP = "queue/payrollmdb";
	
	public static void createPayrollMessage(Object object, PayrollMessageEnum payrollMessageEnum) {
		useStateLessBean(object, payrollMessageEnum);
	}
	
	private static void useStateLessBean(Object object, PayrollMessageEnum payrollMessageEnum){
		PayrollMessageDriven payrollMessageDriven = ServiceLocator.getInstance().getService(PayrollMessageDriven.JNDI_NAME);
		PayrollMessage payrollMessage = new PayrollMessage();
		payrollMessage.setEntity(object);
		payrollMessage.setPayrollCommand(payrollMessageEnum);
		payrollMessageDriven.performMessageAction(payrollMessage);
	}
	
	
	@SuppressWarnings("unused") // WIP for Payroll Message Driven Bean
	private void sentMessage(PayrollMessageEnum payrollMessageEnum, Object object){
		try {
			InitialContext ctx = new InitialContext();
			Queue queue = (Queue) ctx.lookup(PAYROLL_MDB_JNDI_LOOKUP);
		    QueueConnectionFactory factory = (QueueConnectionFactory) ctx.lookup("ConnectionFactory");
		    QueueConnection cnn = factory.createQueueConnection();
		    QueueSession session = cnn.createQueueSession(false, QueueSession.AUTO_ACKNOWLEDGE);
		    ObjectMessage message = session.createObjectMessage();
		    PayrollMessage payrollMessage = new PayrollMessage();

			payrollMessage.setEntity(object);

			payrollMessage.setPayrollCommand(payrollMessageEnum);
		    message.setObject(payrollMessage);
		    QueueSender sender = session.createSender(queue);
	        sender.send(message);

		} catch (NamingException e) {
			e.printStackTrace();
			throw new InfrastructureException(e);
		} catch (JMSException e) {
			e.printStackTrace();
			throw new InfrastructureException(e);
		}
		
	}
	
}
