package com.intuit.sb.payroll.rules.impl.drools;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.util.HashMap;
import java.util.LinkedHashMap;

import javax.rules.RuleRuntime;
import javax.rules.RuleServiceProvider;
import javax.rules.RuleServiceProviderManager;
import javax.rules.admin.LocalRuleExecutionSetProvider;
import javax.rules.admin.RuleAdministrator;
import javax.rules.admin.RuleExecutionSet;

import org.drools.jsr94.rules.RuleServiceProviderImpl;

import com.intuit.sb.payroll.rules.RulesException;

public class DroolsJsr94Runtime 
{
	private static DroolsJsr94Runtime instance;
	
    private static final String             RULE_SERVICE_PROVIDER = "http://drools.org/";

    protected RuleAdministrator             ruleAdministrator;

    protected RuleServiceProvider           ruleServiceProvider;

    protected LocalRuleExecutionSetProvider ruleSetProvider;

    protected RuleRuntime                   ruleRuntime;
    
    private boolean initialised = false;
    
   static
    {
    	try
    	{
    		instance = new DroolsJsr94Runtime();
    	}
    	catch(Exception e)
    	{
    		throw new RulesException(e);
    	}
    }
    
    private DroolsJsr94Runtime()throws Exception
    {
    }
    
    public static DroolsJsr94Runtime getInstance()
    {
    	return instance;
    }
    
    public RuleAdministrator getRuleAdministrator() {
		return ruleAdministrator;
	}

	public void setRuleAdministrator(RuleAdministrator ruleAdministrator) {
		this.ruleAdministrator = ruleAdministrator;
	}

	public boolean isInitialised() {
		return initialised;
	}

	public RuleRuntime getRuleRuntime() {
		return ruleRuntime;
	}

	public void setRuleRuntime(RuleRuntime ruleRuntime) {
		this.ruleRuntime = ruleRuntime;
	}

	public RuleServiceProvider getRuleServiceProvider() {
		return ruleServiceProvider;
	}

	public void setRuleServiceProvider(RuleServiceProvider ruleServiceProvider) {
		this.ruleServiceProvider = ruleServiceProvider;
	}

	public LocalRuleExecutionSetProvider getRuleSetProvider() {
		return ruleSetProvider;
	}

	public void setRuleSetProvider(LocalRuleExecutionSetProvider ruleSetProvider) {
		this.ruleSetProvider = ruleSetProvider;
	}

	//method that initializes the rules runtime
	public void Initialize() throws Exception
	{
        RuleServiceProviderManager.registerRuleServiceProvider( DroolsJsr94Runtime.RULE_SERVICE_PROVIDER, RuleServiceProviderImpl.class );
		
		this.ruleServiceProvider = RuleServiceProviderManager.getRuleServiceProvider( DroolsJsr94Runtime.RULE_SERVICE_PROVIDER );
		
		this.ruleAdministrator = this.ruleServiceProvider.getRuleAdministrator();
		
		this.ruleSetProvider = this.ruleAdministrator.getLocalRuleExecutionSetProvider( null );
		
		this.ruleRuntime  = this.ruleServiceProvider.getRuleRuntime();
		
		//initialised completely
		initialised = true;
	}   
	
	//bindUri - unique identifier for this rule set
	//resourceAsStream - the rules resource for this executionSet
    public void addRuleExecutionSet(final String bindUri, final InputStream resourceAsStream) throws Exception
    {
    	if(!initialised)
    	{
    		throw new Exception(" The rules engine is not initialized properly.");
    	}
        final Reader ruleReader = new InputStreamReader( resourceAsStream );

        final RuleExecutionSet ruleExecutionSet = this.ruleSetProvider.createRuleExecutionSet( ruleReader,
                                                                                               null );

        this.ruleAdministrator.registerRuleExecutionSet( bindUri,
                                                         ruleExecutionSet,
                                                         null );
    }
    
    /**
     * Registers a uri with a ruleSet
     * @throws Exception
     */
    public void registerRuleSet(org.drools.rule.Package pkg) throws Exception {
        
	        final RuleExecutionSet res1 =   ruleSetProvider.createRuleExecutionSet(pkg, null);

        	// Register the RuleExecutionSet
            final String rulesetName = res1.getName();
            
            ruleAdministrator.registerRuleExecutionSet(rulesetName, res1, null);        
    }

    /**
     * Registers a uri with a ruleSet
     * @throws Exception
     */
    private void unregisterRuleSet(final String uri) throws Exception 
    {
            ruleAdministrator.deregisterRuleExecutionSet(uri, null);
    }

    /**
     * rules configuration file holds the uri -> drl/dsl mappings
     * reads maps each file in the config file and maps it with the uri and register the ruleset
     */
	public HashMap<String,LinkedHashMap<String,String>> registerRulesets(final URL configUrl) throws Exception
	{
		DroolsHelper helper = new DroolsHelper();
		
        HashMap<String,LinkedHashMap<String,String>> rulesetMap = helper.initializeRulesMap(configUrl);

		for (HashMap<String,String> rulesMap : rulesetMap.values()) //holds the uri to actual drl mapping
		{
			//read every rules mapping from the rulesMap, read the rule set and register it
			for (String url : rulesMap.values())
			{
				try
				{
					org.drools.rule.Package pkg = helper.ReadRule(url);
					registerRuleSet(pkg);
				}
				catch (Exception e)
				{
					throw new RulesException("Unable to register rule [" + url + "]: " + DroolsHelper.throwableMessageChain(e));
				}
			}
		}
		return rulesetMap;
	}
	/**
	 * unregister the all the rulesets specified in a config file
	 */
	public void unRegisterRulesets(final URL configUrl) throws Exception
	{
		DroolsHelper helper = new DroolsHelper();
		
        HashMap<String,LinkedHashMap<String,String>> rulesetMap = helper.initializeRulesMap(configUrl);

		for (HashMap<String,String> rulesMap : rulesetMap.values()) //holds the uri to actual drl mapping
		{
			//read every rules mapping from the rulesMap, read the rule set and unregister it
			for (String url : rulesMap.values())
			{
				try
				{
					unregisterRuleSet(url);
				}
				catch (Exception e)
				{
					throw new RulesException("Unable to unregister rule [" + url + "]: " + DroolsHelper.throwableMessageChain(e));
				}
			}
		}
	}
}
