package com.intuit.sb.payroll.rules.impl.drools;

import java.rmi.RemoteException;
import java.util.List;
import java.util.Map;

import javax.rules.InvalidRuleSessionException;
import javax.rules.RuleRuntime;
import javax.rules.StatelessRuleSession;

public class DroolsStatelessSession
{
	private RuleRuntime ruleRuntime;
	private StatelessRuleSession session;
	
	
	public DroolsStatelessSession(DroolsJsr94Runtime runtime, String uri) throws Exception
	{
		ruleRuntime = runtime.getRuleRuntime();
		initializeRuleSession(uri);
	}
	
	//executes the relevant rules on the objects supplied on the input list and returns all the objects
	@SuppressWarnings("unchecked")
	public List executeRules(List inputList) throws Exception
	{
		final List output = session.executeRules(inputList);
		return output;
	}

    //returns a statelessRuleSession object associated with the rulesExecutionSet represented by the uri
    private void initializeRuleSession(final String uri) throws Exception
    {
        this.initializeStatelessRuleSession( uri, null );
    }

	@SuppressWarnings("unchecked")
	private void initializeStatelessRuleSession(final String uri, final Map properties) throws Exception 
    {
    	session = (StatelessRuleSession)this.ruleRuntime.createRuleSession( uri, properties, RuleRuntime.STATELESS_SESSION_TYPE );
    }
	
	public void Release()
	{
		try
		{
			session.release();
			session = null;
		}
		catch(InvalidRuleSessionException irse)
		{
			;//ignore the exception
		}
		catch(RemoteException re)
		{
			;//ignore the exception
		}
	}
	
	public void finalize()
	{
		if(session != null)
		{
			Release();
		}
	}
}
