package com.intuit.sb.payroll.capability.benefits.impl;

import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.persistence.OptimisticLockException;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.benefits.BenefitsComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.benefits.HealthBenefitEAO;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.HealthBenefit;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Implementation of the BenefitsComponent interface.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
@Stateless(name = "BenefitsComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class BenefitsComponentBean implements BenefitsComponent {
	protected static final Logger sLogger = PayrollLoggerFactory.makeLogger(BenefitsComponentBean.class);

	@EJB
	private HealthBenefitEAO healthBenefitEAO;

	@EJB
	private WorkerComponent workerComponent;
	
	@EJB
	private EntityService entityService;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.wages.IPayrollWageComponent#createHealthBenefitDtl(com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum)
	 */
	@SuppressWarnings("nls")
	public HealthBenefitDtl createHealthBenefitDtl(HealthBenefitTypeEnum healthBenefitTypeEnum)
			throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(healthBenefitTypeEnum, "healthBenefitTypeEnum");
		
		HealthBenefit healthBenefit = findHealthBenefit(healthBenefitTypeEnum);
		HealthBenefitDtl healthBenefitDtl = new HealthBenefitDtl();
		healthBenefitDtl.setPayrollItem(healthBenefit);
		healthBenefitDtl.setBenefitType(healthBenefitTypeEnum);
		healthBenefitDtl.setName(healthBenefit.getName());
		healthBenefitDtl.setDescription(healthBenefit.getDescription());

		return healthBenefitDtl;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.wages.IPayrollWageComponent#findHealthBenefitDtl(com.intuit.sb.payroll.entity.domain.Identifiable)
	 */
	@SuppressWarnings("nls")
	public HealthBenefitDtl findHealthBenefitDtl(Identifiable identifiable) throws ServiceFailureException, ValidationException {
		PayrollParamValidator.checkIsNotNull(identifiable, "identifiable");
		try {
			HealthBenefitDtl healthBenefitDtl = entityService.findByIdentifiable(HealthBenefitDtl.class, identifiable);
			return healthBenefitDtl;
		} catch (Exception ex) {
			throw new ServiceFailureException(ExceptionUtils.throwableMessage(ex));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.wages.IPayrollWageComponent#deleteHealthBenefitDtl(com.intuit.sb.payroll.entity.model.Worker,
	 * 		com.intuit.sb.payroll.entity.model.HealthBenefitDtl)
	 */
	@SuppressWarnings("nls")
	public Worker deleteHealthBenefitDtl(Worker worker, HealthBenefitDtl healthBenefitDtl) 	throws ValidationException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(healthBenefitDtl, "healthBenefitDtl");
		healthBenefitDtl.validateNoPeriodToDateValues();
		worker.removeHealthBenefitDtl(healthBenefitDtl);
		entityService.makeTransient(healthBenefitDtl);
		return worker;
	}

	public Worker addHealthBenefitDtls(Worker worker, List<HealthBenefitDtl> dtlSet) throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		HealthBenefitValidator.getInstance().validateEntityCollection(dtlSet);
		if (entityService.notPersistent(worker)) {
			worker = entityService.makeManaged(worker);
		}
		
		// For all benefits in the set, add each to the worker
		for (HealthBenefitDtl newItem : dtlSet) {
			worker.addHealthBenefitDtl(newItem);
		}

		//FIXME: the worker doesn't seem to get updated when crossing EJB boundaries
		entityService.flush();

		return worker;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.wages.IPayrollWageComponent# updateHealthBenefitDtls(java.lang.Long,
	 * 		java.util.Set<com.intuit.sb.payroll.entity.model.HealthBenefitDtl>)
	 */
	public Worker updateHealthBenefitDtls(Long workerId, List<HealthBenefitDtl> dtlSet) throws ValidationException, ServiceFailureException {
		HealthBenefitValidator.getInstance().validateEntityCollection(dtlSet);

			Worker worker = workerComponent.findWorker(workerId);

			/*
			 * For each element in the update list find a corresponding element
			 * in the current list (flag error, if not found) and update in
			 * place.
			 */
			for (HealthBenefitDtl newItem : dtlSet) {
				// assume guilty until proven innocent
				Boolean notFound = true;
				Boolean staleVersion = true;

				Set<HealthBenefitDtl> healthBenefitDtls = worker.getHealthBenefitDtls();
				for (HealthBenefitDtl existingItem : healthBenefitDtls) {
					if (existingItem.getId() == newItem.getId()) {
						notFound = false;
						if (existingItem.getVersion() == newItem.getVersion()) {
							staleVersion = false;
							newItem.contentCopy(existingItem);
						}
					}
				}

				if (notFound) {
					// exception
					throw new OptimisticLockException(
							"Updated element not found in the current element list. Must have been deleted by another user.");
				} else if (staleVersion) {
					// exception
					throw new OptimisticLockException("Updated element is stale. Data refresh is needed.");
				}
			}

			return worker;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.wages.IPayrollWageComponent#deleteHealthBenefitDtls(java.lang.Long, java.util.Set<com.intuit.sb.payroll.entity.model.HealthBenefitDtl>)
	 */
	public Worker deleteHealthBenefitDtls(Long workerId, List<HealthBenefitDtl> dtlSet) throws ServiceFailureException, ValidationException {
		
		Worker worker = workerComponent.findWorker(workerId);
		for (HealthBenefitDtl newItem : dtlSet) {
			deleteHealthBenefitDtl(worker, newItem);
		}
		return worker;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.benefits.BenefitsComponent#findAllBenefits()
	 */
	public List<HealthBenefit> findAllBenefits() {
		return healthBenefitEAO.findAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.benefits.BenefitsComponent#findHealthBenefit (com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum)
	 */
	public HealthBenefit findHealthBenefit(HealthBenefitTypeEnum healthBenefitType) throws ServiceFailureException {
		try {
			return healthBenefitEAO.findHealthBenefit(healthBenefitType);
		} catch (Exception ex) {
			throw new ServiceFailureException(ExceptionUtils.throwableMessage(ex));
		}
	}

}
