package com.intuit.sb.payroll.capability.employer.impl;

import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Validates {@link EmployerContactPerson} entities.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
@SuppressWarnings("nls")
public class EmployerContactPersonValidator extends AbstractEntityDataValidator<EmployerContactPerson> {

	private static EmployerContactPersonValidator sInstance = new EmployerContactPersonValidator();

	private EmployerContactPersonValidator() {
	}

	public static EmployerContactPersonValidator getInstance() {
		return sInstance;
	}

	@Override
	public void validateEntityCollection(List<EmployerContactPerson> employerContactPersons) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(employerContactPersons, "Employer Contact Persons");
		for (EmployerContactPerson employerContactPerson : employerContactPersons) {
			validateEntity(employerContactPerson);
		}
	}

	@Override
	public void validateEntity(EmployerContactPerson employerContactPerson) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(employerContactPerson, "Employer Contact Person");
		ContactPersonValidator.getInstance().validateEntity(employerContactPerson);
		// add validation checks specific to the EmployerContactPerson here
		PayrollParamValidator.checkIsNotNullOrEmptyString(employerContactPerson.getPersonName().getFirstName(),
				"First Name");
		PayrollParamValidator.checkIsNotNullOrEmptyString(employerContactPerson.getPersonName().getLastName(),
				"Last Name");
		PayrollParamValidator.checkIsNotNullOrEmptyString(employerContactPerson.getWorkPhone().getFreeFormNumber(),
				"Work TelephoneNumber");
		PayrollParamValidator.checkIsNotNull(employerContactPerson.getMailingAddress(), "Residence");
		USPostalAddress mailingAddress = AddressConvertor.getUSFromGlobal(employerContactPerson.getMailingAddress(),
				new USPostalAddressImpl());
		PayrollParamValidator.checkIsNotNullOrEmptyString(mailingAddress.getAddressLine1(), "Address Line 1");
		PayrollParamValidator.checkIsNotNullOrEmptyString(mailingAddress.getCity(), "City");
		PayrollParamValidator.checkIsNotNullOrEmptyString(mailingAddress.getRegionOrState(), "State Code");
		PayrollParamValidator.checkIsNotNullOrEmptyString(mailingAddress.getZipOrPostalCode(), "Zip Code");
		PayrollParamValidator.checkIsNotNullOrEmptyString(employerContactPerson.getEmailAddress().getAddress(),
				"Email Address");
	}
}
