package com.intuit.sb.payroll.capability.employer.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import com.intuit.sb.payroll.capability.employer.JurisdictionType;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.Agency;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.AgencyIdType;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDoc;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.Deduction;
import com.intuit.sb.payroll.entity.model.DeductionTypeEnum;
import com.intuit.sb.payroll.entity.model.EmployerIdTypeEnum;
import com.intuit.sb.payroll.entity.model.HealthBenefit;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.model.LawEntityTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOff;
import com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.RetirementBenefit;
import com.intuit.sb.payroll.entity.model.RetirementBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.model.TaxTypeEnum;
import com.intuit.sb.payroll.entity.taxauthority.JurisdictionEAO;
import com.intuit.sb.payroll.entity.taxes.AgencyEAO;
import com.intuit.sb.payroll.entity.taxes.AgencyIdTypeEAO;
import com.intuit.sb.payroll.entity.workerpayments.PayrollItemEAO;

public class EmployerSeedData {

	//REGEX patterns for IDs
	//FIXME [th] these should come from Payroll Engine.
	private static final String US_IRS_ID_PATTERN = "^[0-9]{2}[- ]?[0-9]{7}$";
	private static final String CA_EDD_ID_PATTERN = "^[0-9]{3}[- ]?[0-9]{4}[- ]?[0-9]{1}$";
	private static final String TX_TWC_ID_PATTERN = "^[0-9]{2}[- ]?[0-9]{6}[- ]?[0-9]{1}$";

	//private static Logger sLogger = PayrollLoggerFactory.makeLogger(EmployerSeedData.class);
	
	private AgencyIdType fedEinType;
	private AgencyIdType caAgencyId;
	private AgencyIdType txAgencyId;
	private Agency agencyFED;
	private Agency agencyEDD;
	private Agency agencyTWC;

	// EAOs
	private PayrollItemEAO payrollItemEAO = ServiceLocator.getInstance().getService(PayrollItemEAO.JNDI_NAME);
    private JurisdictionEAO jurisdictionEAO = ServiceLocator.getInstance().getService(JurisdictionEAO.JNDI_NAME);
    private AgencyEAO agencyEAO = ServiceLocator.getInstance().getService(AgencyEAO.JNDI_NAME);
    private AgencyIdTypeEAO agencyTypeEAO = ServiceLocator.getInstance().getService(AgencyIdTypeEAO.JNDI_NAME);
   

	public static void setupEmployerDataInSystem()
	{
    	EmployerSeedData dataPopulator = new EmployerSeedData();
		dataPopulator.seedData();
	}

    private void seedData() {
//    	//Determine if (any) jurisdictions exist: if so, the global data has been initialized.
//    	List<Jurisdiction> result = jurisdictionEAO.findAll();
//    	if (0 == result.size()) {
//    		// global data must be set only once
	    	seedAgencies();
	    	seedJurisdictionsAndLaws();
//    	}
        seedCompensationPayrollItems();
        seedBenefitPayrollItems();
        seedPTOPayrollItems();
        seedDeductionItems();
    }
    
    private void seedAgencies() {
    	fedEinType = new AgencyIdType();
		fedEinType.setAgencyIdValidationPattern(US_IRS_ID_PATTERN);
    	fedEinType.setAgencyKey(AgencyEnum.US_IRS);
    	fedEinType.setIdTypeKey(EmployerIdTypeEnum.EIN);
    	fedEinType.setGhostText("xx-xxxxxxx");
    	fedEinType.setMandatory(true);

    	caAgencyId = new AgencyIdType();
		caAgencyId.setAgencyIdValidationPattern(CA_EDD_ID_PATTERN);
    	caAgencyId.setAgencyKey(AgencyEnum.CA_EDD);
    	caAgencyId.setIdTypeKey(EmployerIdTypeEnum.SINGLE_VALUE_AGENCY_SPECIFIC);
    	caAgencyId.setGhostText("xxx-xxxx-x");
    	caAgencyId.setMandatory(false);

    	txAgencyId = new AgencyIdType();
		txAgencyId.setAgencyIdValidationPattern(TX_TWC_ID_PATTERN);
    	txAgencyId.setAgencyKey(AgencyEnum.TX_TWC);
    	txAgencyId.setIdTypeKey(EmployerIdTypeEnum.SINGLE_VALUE_AGENCY_SPECIFIC);
    	txAgencyId.setGhostText("xx-xxxxxx-x");
    	txAgencyId.setMandatory(false);

    	fedEinType = agencyTypeEAO.makePersistent(fedEinType);
    	caAgencyId = agencyTypeEAO.makePersistent(caAgencyId);
    	txAgencyId = agencyTypeEAO.makePersistent(txAgencyId);

		agencyFED = new Agency();
		agencyFED.setAgencyKey(AgencyEnum.US_IRS);
		agencyFED.setName("Federal");
		agencyFED.setShortName("IRS");

		agencyEDD = new Agency();
		agencyEDD.setAgencyKey(AgencyEnum.CA_EDD);
		agencyEDD.setName("California");
		agencyEDD.setShortName("EDD");

		agencyTWC = new Agency();
		agencyTWC.setAgencyKey(AgencyEnum.TX_TWC);
		agencyTWC.setName("Texas");
		agencyTWC.setShortName("TWC");

		agencyFED = agencyEAO.makePersistent(agencyFED);
		agencyEDD = agencyEAO.makePersistent(agencyEDD);
		agencyTWC = agencyEAO.makePersistent(agencyTWC);

	}

    /*

    TAXES:
    ------
       Mnemonic            DTValue                     Description

         US-FED-SST       US-FED-SST DTValue          Social Security Tax (Employer contribution)
         US-FED-SST-EE    US-FED-SST DTValue          Social Security Tax (Employee contribution)
         US-FED-MEDT      US-FED-MEDT DTValue         Medicare Tax (Employer contribution)
         US-FED-MEDT-EE   US-FED-MEDT DTValue         Medicare Tax (Employee contribution)
         US-FED-FIT       US-FED-FIT DTValue          Federal Income Tax
         US-FED-FUI       US-FED-FUI DTValue          Federal Unemployment Insurance
         US-CA-PIT        US-CA-PIT DTValue           CA Personal Income Tax
         US-CA-SDI        US-CA-SDI DTValue           CA State Disability Insurance
         US-CA-SUI        US-CA-SUI DTValue           CA State Unemployment Insurance
         US-CA-ETT        US-CA-ETT DTValue           CA Employment Training Tax
         US-TX-SUI        US-TX-SUI DTValue           TX State Unemployment Insurance
         US-CA-SF-PT      US-CA-SF-PT DTVal           CA SF Payroll Tax PaidByType.ER

    WAGES:
    ------
       US-SALARY        		  	  1000013                     US salary wages
       US-HOURLY        		  	  1000012                     US hourly wages
       US-OVERTIME      		  	  1000026                     US Overtime
       US-DOUBLETIME    		  	  1000003                     US Doubletime
       US-COMMISSION    		  	  1000025                     US Commission
       US-PAYCHECKTIPS  		  	  1000004                     US PaycheckTips
       US-CASHTIPS              	  1000041                     US CashTips
       US-ALLOCATEDTIPS 		      1000048                     US AllocatedTips
       US-BONUS         		      1000005                     US Bonus (flat amount)
       US-PIECEWORK    		      1000015                     US Piecework
       US-REIMBURSEMENT		      1000045					  US Reimbursement          				
    */

    private void seedJurisdictionsAndLaws() {
        /* US Setup */
        Jurisdiction us = new Jurisdiction();
        us.setName(JurisdictionType.UNITED_STATES.getName());
        // TODO: Set the Mapping Key value

        //Employee portion of Social Security Tax
        Law sstLawEmployee = createUSEmployeeLaw();
        sstLawEmployee.setName("US-FED-SST-EE");
        sstLawEmployee.setDescription("Social Security Tax (Employee)");
        sstLawEmployee.setLawId(QboeLawIdConstants.sSST_EELawId);  // Employee portion is 61
        us.addLaw(sstLawEmployee);
        //now create the Tax object and associate the law with it
        Tax sstTaxEmployee = assosciateLawWithTaxPayrollItem(sstLawEmployee);
        sstTaxEmployee.setTaxType(TaxTypeEnum.FEDERALTAX);

        //Employer portion of Social Security Tax
        Law sstLawEmployer = createUSEmployerLaw();
        sstLawEmployer.setName("US-FED-SST");
        sstLawEmployer.setDescription("Social Security Tax (Employer)");
        sstLawEmployer.setLawId(QboeLawIdConstants.sSST_ERLawId);  // employer portion is 62
        us.addLaw(sstLawEmployer);
        //now create the Tax object and associate the law with it
        Tax sstTaxEmployer = assosciateLawWithTaxPayrollItem(sstLawEmployer);
        sstTaxEmployer.setTaxType(TaxTypeEnum.FEDERALTAX);

        //Employee portion of Medicare
        Law medLawEmployee = createUSEmployeeLaw();
        medLawEmployee.setName("US-FED-MEDT-EE");
        medLawEmployee.setDescription("Medicare Tax (Employee)");
        medLawEmployee.setLawId(QboeLawIdConstants.sMED_EELawId);  // Employee portion is 63
        us.addLaw(medLawEmployee);
        //now create the Tax object and associate the law with it
        Tax medTaxEmployee = assosciateLawWithTaxPayrollItem(medLawEmployee);
        medTaxEmployee.setTaxType(TaxTypeEnum.FEDERALTAX);

        //Employer portion of Medicare
        Law medLawEmployer = createUSEmployerLaw();
        medLawEmployer.setName("US-FED-MEDT");
        medLawEmployer.setDescription("Medicare Tax (Employer)");
        medLawEmployer.setLawId(QboeLawIdConstants.sMED_ERLawId);  //employer portion is 64
        us.addLaw(medLawEmployer);
        //now create the Tax object and associate the law with it
        Tax medTaxEmployer = assosciateLawWithTaxPayrollItem(medLawEmployer);
        medTaxEmployer.setTaxType(TaxTypeEnum.FEDERALTAX);

        //FIT
        Law fitLaw = createUSEmployeeLaw();
        fitLaw.setName("US-FED-FIT");
        fitLaw.setDescription("Federal Income Tax");
        fitLaw.setLawId(QboeLawIdConstants.sFICALawId);
        ComplianceDoc w4Form = new ComplianceDoc();
        w4Form.setName("US-FED-W4");
        w4Form.setDocId(ComplianceDocIdEnum.W4IRS);
        w4Form.setLaw(fitLaw);
        fitLaw.addComplianceDoc(w4Form);
        us.addLaw(fitLaw);
        //now create the Tax object and associate the law with it
        Tax fitTax = assosciateLawWithTaxPayrollItem(fitLaw);
        fitTax.setTaxType(TaxTypeEnum.FEDERALTAX);

        //FUI
        Law fuiLaw = createUSEmployerLaw();
        fuiLaw.setName("US-FED-FUI");
        fuiLaw.setDescription("Federal Unemployment Insurance");
        fuiLaw.setLawId(QboeLawIdConstants.sFUTALawId);  // For states with SUI; for states without SUI, the value is 65
        us.addLaw(fuiLaw);
        //now create the Tax object and associate the law with it
        Tax fuiTax = assosciateLawWithTaxPayrollItem(fuiLaw);
        fuiTax.setTaxType(TaxTypeEnum.FEDERALTAX);
        
        // FIXME Not sure where should I belong ? COBRA
        Law cobraLaw = createUSEmployerLaw();
        cobraLaw.setName("US-COBRA");
        cobraLaw.setDescription("Federal Cobra Premium");
        cobraLaw.setLawId(QboeLawIdConstants.sCobraPremiumCredit);  // For states with SUI; for states without SUI, the value is 65
        // us.addLaw(cobraLaw);
        // Just create a Payroll Item
        PayrollItem cobraPayrollItem = new PayrollItem();
        cobraPayrollItem.setName(cobraLaw.getName());
        cobraPayrollItem.setDescription(cobraLaw.getDescription());
        cobraPayrollItem.getLawId().setValue(cobraLaw.getLawId());
        cobraPayrollItem.setSourceEntity(cobraLaw.getSourceEntity());
        cobraPayrollItem.setTargetEntity(cobraLaw.getTargetEntity());
        
        
        

/*
// If AEIC is added, it's automatically added to all employees???    
        //AEIC
        Law aeic = createUSEmployeeCreditLaw();
        aeic.setName("US-FED-AEIC");
        aeic.setDescription("Advance Earned Income Credit");
        aeic.setLawId(QboeLawIdContants.sAeicId);
        us.addLaw(aeic);
        //now create the Tax object and associate the law with it
        Tax aeicTax = assosciateLawWithTaxPayrollItem(aeic);
//FIXME  Not sure what TaxTypeEnum value to use for AEIC.        
        aeicTax.setTaxType(TaxTypeEnum.NOTSET);
*/
        
        /* CA Setup */
        Jurisdiction ca = new Jurisdiction();
        ca.setName(JurisdictionType.CALIFORNIA.getName());
        // TODO: Set the Mapping Key value

        //CAPIT
        Law capitLaw = createUSEmployeeLaw();
        capitLaw.setName("US-CA-PIT");
        capitLaw.setDescription("CA Personal Income Tax");
        capitLaw.setLawId(QboeLawIdConstants.sCAPITLawId);
        ComplianceDoc de4Form = new ComplianceDoc();
        de4Form.setName("US-CA-DE-4");
        de4Form.setDocId(ComplianceDocIdEnum.DE4);
        de4Form.setLaw(capitLaw);
        capitLaw.addComplianceDoc(de4Form);
        ComplianceDoc de545Form = new ComplianceDoc();
        de545Form.setName("US-CA-DE-545");
        de545Form.setDocId(ComplianceDocIdEnum.DE545);
        de545Form.setLaw(capitLaw);
        capitLaw.addComplianceDoc(de545Form);
        ca.addLaw(capitLaw);
        //now create the Tax object and associate the law with it
        Tax capitTax = assosciateLawWithTaxPayrollItem(capitLaw);
        capitTax.setTaxType(TaxTypeEnum.STATETAX);

        //CASDI
        Law casdiLaw = createUSEmployeeLaw();
        casdiLaw.setName("US-CA-SDI");
        casdiLaw.setDescription("CA State Disability Insurance");
        casdiLaw.setLawId(QboeLawIdConstants.sCASDILawId);
        ca.addLaw(casdiLaw);
        //now create the Tax object and associate the law with it
        Tax casdiTax = assosciateLawWithTaxPayrollItem(casdiLaw);
        casdiTax.setTaxType(TaxTypeEnum.STATETAX);

        //CASUI
        Law casuiLaw = createUSEmployerLaw();
        casuiLaw.setName("US-CA-SUI");
        casuiLaw.setDescription("CA State Unemployment Insurance");
        casuiLaw.setLawId(QboeLawIdConstants.sCASUILawId);
        ca.addLaw(casuiLaw);
        //now create the Tax object and associate the law with it
        Tax casuiTax = assosciateLawWithTaxPayrollItem(casuiLaw);
        casuiTax.setTaxType(TaxTypeEnum.STATETAX);

        //CAETT
        Law caettLaw = createUSEmployerLaw();
        caettLaw.setName("US-CA-ETT");
        caettLaw.setDescription("CA Employment Training Tax");
        caettLaw.setLawId(QboeLawIdConstants.sCAETTLawId);
        ca.addLaw(caettLaw);
        //now create the Tax object and associate the law with it
        Tax caettTax = assosciateLawWithTaxPayrollItem(caettLaw);
        caettTax.setTaxType(TaxTypeEnum.STATETAX);

        /* SF Setup */
        Jurisdiction sf = new Jurisdiction();
        sf.setName(JurisdictionType.SAN_FRANCISCO.getName());
        // TODO: Set the Mapping Key value

        //SF local tax
//        Law sfLaw = createUSEmployerLaw();
//        sfLaw.setName("US-CA-SF-PT");
//        sfLaw.setDescription("CA SF Payroll Tax");
//        sfLaw.setLawId(QboeLawIdConstants.sCASFPTLawId);
//        sf.addLaw(sfLaw);
//        //now create the Tax object and associate the law with it
//        Tax sfTax = assosciateLawWithTaxPayrollItem(sfLaw);
//        sfTax.setTaxType(TaxTypeEnum.LOCALTAX);

        /* TX Setup */
        Jurisdiction tx = new Jurisdiction();
        tx.setName(JurisdictionType.TEXAS.getName());
        // TODO: Set the Mapping Key value

        //TXSUI
        Law txSuiLaw = createUSEmployerLaw();
        txSuiLaw.setName("US-TX-SUI");
        txSuiLaw.setDescription("TX State Unemployment Insurance");
        txSuiLaw.setLawId(QboeLawIdConstants.sTXSUILawId);
        tx.addLaw(txSuiLaw);
        //now create the Tax object and associate the law with it
        Tax txSuiTax = assosciateLawWithTaxPayrollItem(txSuiLaw);
        txSuiTax.setTaxType(TaxTypeEnum.STATETAX);

        List<Jurisdiction> listJurisdictions = new ArrayList<Jurisdiction>();
        listJurisdictions.add(us);
        listJurisdictions.add(ca);
        listJurisdictions.add(tx);
        listJurisdictions.add(sf);

        //save the Jurisdictions for the company
        saveJurisdictions(listJurisdictions);

        List<PayrollItem> listPayrollItems = new ArrayList<PayrollItem>();
        listPayrollItems.add(sstTaxEmployee);
        listPayrollItems.add(sstTaxEmployer);
        listPayrollItems.add(medTaxEmployee);
        listPayrollItems.add(medTaxEmployer);
        listPayrollItems.add(fitTax);
        listPayrollItems.add(fuiTax);
        listPayrollItems.add(cobraPayrollItem);
        
//        listPayrollItems.add(aeicTax);
        listPayrollItems.add(capitTax);
        listPayrollItems.add(casdiTax);
        listPayrollItems.add(casuiTax);
        listPayrollItems.add(caettTax);
//        listPayrollItems.add(sfTax);
        listPayrollItems.add(txSuiTax);
        

        us.addAgency(agencyFED);
        ca.addAgency(agencyEDD);
        tx.addAgency(agencyTWC);

        //save the tax payroll items for the company
        savePayrollItems(listPayrollItems);
    }

    private void seedCompensationPayrollItems() {
        //company WAGE setup
        //Salary wages
        Compensation pSalary = createCompensationPayrollItem();
        pSalary.setDescription("Salary");
        pSalary.setName("US-SALARY");
        pSalary.getLawId().setValue(QboeLawIdConstants.sSalaryLawId);
        pSalary.setCompensationType(CompensationTypeEnum.SALARYREG);

        //Hourly wages
        Compensation pHourly = createCompensationPayrollItem();
        pHourly.setDescription("Hourly");
        pHourly.setName("US-HOURLY");
        Datum datum = new Datum();
        datum.setValue1AsUSDMoneyAmount(new BigDecimal("10.0"));
        datum.setType2(DatumType2Enum.HOUR);
        datum.setValue2(1.0);
        pHourly.setRate(datum);
        pHourly.getLawId().setValue(QboeLawIdConstants.sHourlyLawId);
        pHourly.setCompensationType(CompensationTypeEnum.HOURLYREG);

        //Overtime wages
        Compensation pOverTime = createCompensationPayrollItem();
        pOverTime.setDescription("Overtime");
        pOverTime.setName("US-OVERTIME");
        Datum datum2 = new Datum();
        datum2.setValue1AsUSDMoneyAmount(new BigDecimal("15.0"));
        datum2.setType2(DatumType2Enum.HOUR);
        datum2.setValue2(1.0);
        pOverTime.setRate(datum2);
        pOverTime.getLawId().setValue(QboeLawIdConstants.sOvertimeLawId);
        pOverTime.setCompensationType(CompensationTypeEnum.HOURLYOVT);

        //Doubletime wages
        Compensation pDoubleTime = new Compensation();
        pDoubleTime.setDescription("Doubletime");
        pDoubleTime.setName("US-DOUBLETIME");
        pDoubleTime.setSourceEntity(LawEntityTypeEnum.EMPLOYER);
        pDoubleTime.setTargetEntity(LawEntityTypeEnum.WORKER);
        pDoubleTime.getLawId().setValue(QboeLawIdConstants.sDoubleTimeLawId);
        pDoubleTime.setCompensationType(CompensationTypeEnum.HOURLYDBL);

        //Bonus wages
        Compensation pBonus = createCompensationPayrollItem();
        pBonus.setDescription("Bonus");
        pBonus.setName("US-BONUS");
        pBonus.getLawId().setValue(QboeLawIdConstants.sBonusLawId);
        pBonus.setCompensationType(CompensationTypeEnum.BONUS);

        //Commission wages
        Compensation pCommission = createCompensationPayrollItem();
        pCommission.setDescription("Commission");
        pCommission.setName("US-COMMISSION");
        pCommission.getLawId().setValue(QboeLawIdConstants.sCommissionLawId);
        pCommission.setCompensationType(CompensationTypeEnum.COMMISSION);

        //PaycheckTips wages
        Compensation pPaycheckTips = createCompensationPayrollItem();
        pPaycheckTips.setDescription("PaycheckTips");
        pPaycheckTips.setName("US-PAYCHECKTIPS");
        pPaycheckTips.getLawId().setValue(QboeLawIdConstants.sPaycheckTipsLawId);
        pPaycheckTips.setCompensationType(CompensationTypeEnum.TIPSPAYCHECK);

        //CashTips wages
        Compensation pCashTips = createCompensationPayrollItem();
        pCashTips.setDescription("CashTips");
        pCashTips.setName("US-CASHTIPS");
        pCashTips.getLawId().setValue(QboeLawIdConstants.sCashTipsLawId);
        pCashTips.setCompensationType(CompensationTypeEnum.TIPSCASH);

        //AllocatedTips wages
        Compensation pAllocatedTips = createCompensationPayrollItem();
        pAllocatedTips.setDescription("AllocatedTips");
        pAllocatedTips.setName("US-ALLOCATEDTIPS");
        pAllocatedTips.getLawId().setValue(QboeLawIdConstants.sAllocatedTipsLawId);
        pAllocatedTips.setCompensationType(CompensationTypeEnum.TIPSALLOCATED);

        //Piecework wage
        Compensation pPiecework = createCompensationPayrollItem();
        pPiecework.setDescription("Piecework");
        pPiecework.setName("US-PIECEWORK");
        pPiecework.getLawId().setValue(QboeLawIdConstants.sPieceworkLawId);
        pPiecework.setCompensationType(CompensationTypeEnum.PIECEWORK);

        //Sick Hourly Taken wage
        Compensation pSickHourlyTaken = createCompensationPayrollItem();
        pSickHourlyTaken.setDescription("Sick Taken");
        pSickHourlyTaken.setName("US-SICKTAKEN");
        pSickHourlyTaken.getLawId().setValue(QboeLawIdConstants.sSickTimeDisbursementLawId);
        pSickHourlyTaken.setCompensationType(CompensationTypeEnum.SICKTAKEN);

        //Vacation Hourly Taken wage
        Compensation pVacHourlyTaken = createCompensationPayrollItem();
        pVacHourlyTaken.setDescription("Vacation Taken");
        pVacHourlyTaken.setName("US-VACATIONTAKEN");
        pVacHourlyTaken.getLawId().setValue(QboeLawIdConstants.sVacationDisbursementLawId);
        pVacHourlyTaken.setCompensationType(CompensationTypeEnum.VACTAKEN);

        //Reimbursement Taxable Taken wage
        Compensation pReimbursement = createCompensationPayrollItem();
        pReimbursement.setDescription("Reimbursement");
		pReimbursement.setName("US-Reimbursement");
        pReimbursement.getLawId().setValue(QboeLawIdConstants.sOtherReimbursementId);
        pReimbursement.setCompensationType(CompensationTypeEnum.REIMBURSEMENT);

        //add compensations
        List<PayrollItem> listPayrollItems = new ArrayList<PayrollItem>();
        listPayrollItems.add(pSalary);
        listPayrollItems.add(pHourly);
        listPayrollItems.add(pOverTime);
        listPayrollItems.add(pDoubleTime);
        listPayrollItems.add(pCommission);
        listPayrollItems.add(pPaycheckTips);
        listPayrollItems.add(pCashTips);
        listPayrollItems.add(pAllocatedTips);
        listPayrollItems.add(pBonus);
        listPayrollItems.add(pPiecework);
        listPayrollItems.add(pSickHourlyTaken);
        listPayrollItems.add(pVacHourlyTaken);
        listPayrollItems.add(pReimbursement);

        //save the payroll items for the company
        savePayrollItems(listPayrollItems);
    }

    private void seedBenefitPayrollItems() {
        //company Benefits setup
        //Sec125Health
        HealthBenefit sec125Health = new HealthBenefit();
        sec125Health.setBenefitType(HealthBenefitTypeEnum.SEC125HEALTH);
        sec125Health.setDescription("Pre-Tax Employee-Paid Health");
        sec125Health.setName("SEC125HEALTH");
        sec125Health.getLawId().setValue(QboeLawIdConstants.sSec125HealthId);
        sec125Health.setSourceEntity(LawEntityTypeEnum.WORKER);
        sec125Health.setTargetEntity(LawEntityTypeEnum.THIRDPARTY);
        // Set a default rate
        Datum rate = new Datum();
        rate.setValue1AsUSDMoneyAmount(new BigDecimal("100.00"));
        rate.setType2(DatumType2Enum.PERPAYCHECK);
        rate.setValue2(1.0);
        sec125Health.setRate(rate);
        
        //401K
        RetirementBenefit p401K = new RetirementBenefit();
        p401K.setBenefitType(RetirementBenefitTypeEnum.PLAN401K);
        p401K.setDescription("Employee-Paid 401K");
        p401K.setName("401K");
        p401K.getLawId().setValue(QboeLawIdConstants.s401KId);
        p401K.setSourceEntity(LawEntityTypeEnum.WORKER);
        p401K.setTargetEntity(LawEntityTypeEnum.THIRDPARTY);
        // No default rate
        // Set the annual maximum
        Datum annualCap = new Datum();
        annualCap.setValue1AsUSDMoneyAmount(new BigDecimal("15500.00"));
        annualCap.setType2(DatumType2Enum.YEAR);
        annualCap.setValue2(1.0);
        p401K.setMaxContribution(annualCap);

        Datum annualCatchupCap = new Datum();
        annualCatchupCap.setValue1AsUSDMoneyAmount(new BigDecimal("5000.00"));
        annualCatchupCap.setType2(DatumType2Enum.YEAR);
        annualCatchupCap.setValue2(1.0);
        p401K.setMaxCatchupContribution(annualCatchupCap);

        
        //add all benefits
        List<PayrollItem> listPayrollItems = new ArrayList<PayrollItem>();
        listPayrollItems.add(sec125Health);
        listPayrollItems.add(p401K);

        // save the payroll items for the company
        savePayrollItems(listPayrollItems);
    }

    /** Serialize PaidTimeOff. */
    private void seedPTOPayrollItems() {
    	PaidTimeOff hourlyVacation = new PaidTimeOff();
    	hourlyVacation.setPaidTimeOffType(PaidTimeOffTypeEnum.VACATION);
    	hourlyVacation.setName("VACATIONACCRUE");
    	hourlyVacation.setDescription("Vacation Accrual");
    	hourlyVacation.getLawId().setValue(QboeLawIdConstants.sVacationAccrualLawId);
    	
    	PaidTimeOff hourlySickTime = new PaidTimeOff();
    	hourlySickTime.setPaidTimeOffType(PaidTimeOffTypeEnum.SICK);
    	hourlySickTime.setName("SICKACCRUE");
    	hourlySickTime.setDescription("Sick Accrual");
    	hourlySickTime.getLawId().setValue(QboeLawIdConstants.sSickTimeAccrualLawId);
    	
    	List<PayrollItem> listPayrollItems = new ArrayList<PayrollItem>();
    	listPayrollItems.add(hourlyVacation);
    	listPayrollItems.add(hourlySickTime);
    	
    	savePayrollItems(listPayrollItems);
    }

    /** Serialize Deductions. */
    private void seedDeductionItems() {
    	Deduction cashTipsOffset = new Deduction();
    	cashTipsOffset.setDeductionType(DeductionTypeEnum.CASHTIPSOFFSET);
    	cashTipsOffset.setName("CASHTIPSOFFSET");
    	cashTipsOffset.setDescription("Cash Tips Offset");
    	cashTipsOffset.getLawId().setValue(QboeLawIdConstants.sCashTipsOutLawId);
    	
    	List<PayrollItem> listPayrollItems = new ArrayList<PayrollItem>();
    	listPayrollItems.add(cashTipsOffset);
    	
    	savePayrollItems(listPayrollItems);
    }
	
    private void savePayrollItems(List<PayrollItem> pItems) {
        for (PayrollItem elem : pItems) {
            elem = payrollItemEAO.makePersistent(elem);
        }
    }

    private void saveJurisdictions(List<Jurisdiction> pJurisdictions) {
        for (Jurisdiction elem : pJurisdictions) {
        	elem = jurisdictionEAO.makePersistent(elem);
        }
    }
    
    private Law createUSLaw() {
        Law law = new Law();
        return law;
    }

    private Law createUSEmployeeLaw() {
        Law law = createUSLaw();
        law.setSourceEntity(LawEntityTypeEnum.WORKER);
        law.setTargetEntity(LawEntityTypeEnum.GOVAGENCY);
        return law;
    }

    private Law createUSEmployerLaw() {
        Law law = createUSLaw();
        law.setSourceEntity(LawEntityTypeEnum.EMPLOYER);
        law.setTargetEntity(LawEntityTypeEnum.GOVAGENCY);
        return law;
    }

    private Tax assosciateLawWithTaxPayrollItem(Law law) {
        Tax tax = null;

        tax = new Tax();
        tax.setName(law.getName());
        tax.setDescription(law.getDescription());
        tax.getLawId().setValue(law.getLawId());
        tax.setSourceEntity(law.getSourceEntity());
        tax.setTargetEntity(law.getTargetEntity());

        return tax;
    }

    private Compensation createCompensationPayrollItem() {
        Compensation compensation = null;

        compensation = new Compensation();
        compensation.setSourceEntity(LawEntityTypeEnum.EMPLOYER);
        compensation.setTargetEntity(LawEntityTypeEnum.WORKER);

        return compensation;
    }
}
