package com.intuit.sb.payroll.capability.employer.impl;

import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;


import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.config.GeminiConfigurationService;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.EmployerId;
/**
 * Validates EmployerTaxID in the EmployerTaxIdValidator entity 
 * This class does not ensure data integrity or completeness.
 */

public class EmployerTaxIdValidator extends AbstractEntityDataValidator<EmployerId> {
	
	private static EmployerTaxIdValidator sInstance = new EmployerTaxIdValidator();
	private EmployerComponent sEmployerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
	private EmployerTaxIdValidator(){
	}
	
	public static EmployerTaxIdValidator getInstance(){
		return sInstance;
	}
	

	@Override
	public void validateEntityCollection(List<EmployerId> employerCollection) throws ValidationException {
		for (EmployerId employerId : employerCollection) {
			try {
				validateEntityImpl(employerId);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	@Override
	public void validateEntity(EmployerId employerId) throws ValidationException {
		try {
			validateEntityImpl(employerId);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 	<ul>Validation logic used for validating EmployerId in Gemini:
	 * 		<li>EmployerId object must be NON-NULL
	 * 		<li>EmployerId must be in proper format
	 * 		<li>EmployerId must contain the required prefix
	 * 	</ul>
	 * @param employerId - EmployerId entity object
	 * @throws ValidationException
	 * @throws IOException 
	 */
	public void validateEntityImpl(EmployerId employerId) throws ValidationException, IOException {
		boolean isValidPrefix = false;
		String preFixofEmployerId = "";
		if(AgencyEnum.US_IRS.equals(employerId.getAgencyIdType().getAgencyKey())){
			List<String> validprefixesList = null;
			String[] validprefixes = null;
			try {
					validprefixes = sEmployerComponent.getAllValidEinPrefixes();
					validprefixesList = Arrays.asList(validprefixes);
				} catch (ServiceFailureException e) {
					e.printStackTrace();
				}
			String employerIdalue = employerId.getValue();
			if(employerIdalue!= null && employerIdalue.length() != 0 ){
				preFixofEmployerId = employerIdalue.substring(0,2);	
				if(validprefixesList.contains(preFixofEmployerId)){
					isValidPrefix = true;
				}
				if(!isValidPrefix){
					throw new ValidationException("The prefix: "+preFixofEmployerId+"is not permitted for EIN: "+employerId.getValue());
				}
			}
		}
	}
	
	public String[] getValidprefixesforEin() throws IOException
	{
		Properties properties = new Properties();
		String permittedEin_properties_file = "PermittedEINPreFixes.properties";
    	String thisPackage = GeminiConfigurationService.class.getPackage().getName();
		String configFileName = thisPackage.replace('.', '/') + "/" + permittedEin_properties_file;
		InputStream propertyStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(configFileName);
		properties.load(propertyStream);
		String einPrefix = properties.getProperty("EIN_prefixes");
		String einPrefixes[] = null;
		if(einPrefix != null)
		{
			einPrefixes = einPrefix.split(",");
		}
		return einPrefixes;
	}
}