package com.intuit.sb.payroll.capability.employer.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.Agency;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.AgencyIdType;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.DisabilityInsDocDtl;
import com.intuit.sb.payroll.entity.model.EffectivePeriod;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerComplianceDocDtl;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.UnemploymentInsDocDtl;
import com.intuit.sb.payroll.entity.taxes.AgencyIdTypeEAO;

/**
 * Handles the side-effects of changes in WorkLocations.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class WorkLocationWorkFlow {

	private static AgencyIdTypeEAO agencyIdTypeEAO = ServiceLocator.getInstance().getService(AgencyIdTypeEAO.JNDI_NAME);

	public static void addNewJurisdictions(Employer employer, List<Jurisdiction> existingJursidictions,
			List<Jurisdiction> currentJursidictions) {

		// Get the new Jurisdiction
		List<Jurisdiction> newJurisdictions = getNewJurisdictions(existingJursidictions, currentJursidictions);

		// Perform workflow for the new Jurisdiction
		for (Jurisdiction jurisdiction : newJurisdictions) {
			performOperationForNewJurisdiction(jurisdiction, employer);
		}
	}

	public static void updateJurisdictions(Employer employer, List<Jurisdiction> oldJursidictions, List<Jurisdiction> newJursidictions) {

		addNewJurisdictions(employer, oldJursidictions, newJursidictions);

		deleteJurisdictions(employer, oldJursidictions, newJursidictions);

	}

	public static void deleteJurisdictions(Employer employer, List<Jurisdiction> oldJursidictions, List<Jurisdiction> newJursidictions) {
		// Get list of removed Jurisdictions
		List<Jurisdiction> removedJurisdictions = getDeletedJurisdictions(oldJursidictions, newJursidictions);

		// Perform workflow for the removed Jurisdiction
		for (Jurisdiction jurisdiction : removedJurisdictions) {
			performOperationForRemovedJurisdiction(jurisdiction, employer);
		}
	}

	private static void performOperationForRemovedJurisdiction(Jurisdiction jurisdiction, Employer employer) {
		// Remove EmployerId
		// Remove SUI
		// Add EmployerId
		Set<EmployerId> empIdSet = employer.getEmployerIds();
		Set<Agency> agencies = jurisdiction.getAgencies();
		for (Agency agency : agencies) {
			EmployerId employerId = getEmployerIdForAgency(agency, empIdSet);
			if (employerId != null) {
				employer.removeEmployerId(employerId);
			}
			// Add SUI Tax Rates 
			removeStateUnEmploymentInsDtl(agency, employer);
		}

	}

	/**
	 * FIXME this is not going to scale, move Agency/Law resolution out of here.
	 * 
	 * @param agency
	 * @param employer
	 */
	private static void removeStateUnEmploymentInsDtl(Agency agency, Employer employer) {
		String lawId = null;

		if (agency.getAgencyKey().equals(AgencyEnum.CA_EDD)) {
			lawId = QboeLawIdConstants.sCASUILawId;
		} else if (agency.getAgencyKey().equals(AgencyEnum.TX_TWC)) {
			lawId = QboeLawIdConstants.sTXSUILawId;
		}
		if (lawId == null)
			return;
		Set<EmployerComplianceDocDtl> empCompliaanceDtlSet = employer.getEmployerComplianceDocDtls();
		Set<EmployerComplianceDocDtl> newCompDtlSet = new HashSet<EmployerComplianceDocDtl>(empCompliaanceDtlSet);

		for (EmployerComplianceDocDtl employerComplianceDocDtl : newCompDtlSet) {
			if (employerComplianceDocDtl.getLawId().getValue().equals(lawId)) {
				employer.removeEmployerComplianceDocDtl(employerComplianceDocDtl);

			}
		}
	}

	private static void performOperationForNewJurisdiction(Jurisdiction jurisdiction, Employer employer) {

		// Add EmployerId
		Set<EmployerId> empIdSet = employer.getEmployerIds();
		Set<Agency> agencies = jurisdiction.getAgencies();
		for (Agency agency : agencies) {
			if (getEmployerIdForAgency(agency, empIdSet) == null) {
				EmployerId employerId = new EmployerId();
				employerId.setValue("");
				employerId.setAgencyIdType(getAgencyTypeForAgency(agency));
				empIdSet.add(employerId);
			}
			// Add SUI Tax Rates
			//TODO: if it was addStateSpecificTaxes and which takes agency and employer
			addStateUnEmploymentInsDtl(agency, employer);
			addStateDisabilityInsurance(agency, employer);
		}
	}

	private static void addStateDisabilityInsurance(Agency agency, Employer employer) {
		String lawId = null;
		Double percentageRate = null;
		
		if (agency.getAgencyKey().equals(AgencyEnum.CA_EDD)) {
			lawId = QboeLawIdConstants.sCASDILawId;
			//FIXME hardcoded default DI rate for CA
			percentageRate=0.8;
		} 

		if (lawId != null) {
			DisabilityInsDocDtl disabilityInsuranceData = new DisabilityInsDocDtl();
			//FIXME hardcoded default DI rates added
			disabilityInsuranceData.setPercentageRate(percentageRate);
			disabilityInsuranceData.getLawId().setValue(lawId);
			employer.addEmployerComplianceDocDtl(disabilityInsuranceData);
		}
		
	}

	/**
	 * FIXME this is not going to scale, move Agency/Law resolution out of here.
	 * 
	 * @param agency
	 * @param employer
	 */
	private static void addStateUnEmploymentInsDtl(Agency agency, Employer employer) {
		String lawId = null;
		
		Double percentageRate = null;
		EffectivePeriod effectivePeriod  = new EffectivePeriod();
		DateRange dateRange = new DateRange();
		dateRange.setStartDate(DateUtil.getFirstDayOfCurrentYear());
		effectivePeriod.setDateRange(dateRange);
		if (agency.getAgencyKey().equals(AgencyEnum.CA_EDD)) {
			lawId = QboeLawIdConstants.sCASUILawId;
			//FIXME hardcoded default SUI rate for CA
			percentageRate=3.4;
		} else if (agency.getAgencyKey().equals(AgencyEnum.TX_TWC)) {
			lawId = QboeLawIdConstants.sTXSUILawId;
			//FIXME hardcoded default SUI rate for TX
			percentageRate=2.7;
		}

		if (lawId != null) {
			UnemploymentInsDocDtl unemploymentInsDtl = new UnemploymentInsDocDtl();
			//FIXME hardcoded default SUI rates added
			unemploymentInsDtl.setPercentageRate(percentageRate);
			unemploymentInsDtl.setEffectivePeriod(effectivePeriod);
			unemploymentInsDtl.getLawId().setValue(lawId);
			unemploymentInsDtl.setDefaultRate(true);
			employer.addEmployerComplianceDocDtl(unemploymentInsDtl);
		}

	}

	private static EmployerId getEmployerIdForAgency(Agency agency, Set<EmployerId> empIdSet) {
		for (EmployerId employerId : empIdSet) {
			if (employerId.getAgencyIdType().getAgencyKey().equals(agency.getAgencyKey())) {
				return employerId;
			}
		}
		return null;
	}

	private static AgencyIdType getAgencyTypeForAgency(Agency agency) {
		List<AgencyIdType> idTypes = agencyIdTypeEAO.findAgenciesByKey(agency.getAgencyKey());
		if (idTypes == null || idTypes.size() == 0) {
			return null;
		}
		return idTypes.get(0);
	}

	private static List<Jurisdiction> getNewJurisdictions(List<Jurisdiction> existingJursidictions,
			List<Jurisdiction> currentJursidictions) {

		List<Jurisdiction> newJurisdictions = new ArrayList<Jurisdiction>(currentJursidictions);

		for (Jurisdiction exjurisdiction : existingJursidictions) {
			newJurisdictions.remove(exjurisdiction);
		}
		return newJurisdictions;
	}

	private static List<Jurisdiction> getDeletedJurisdictions(List<Jurisdiction> oldJursidictions,
			List<Jurisdiction> newJursidictions) {
		List<Jurisdiction> deletedJurisdictions = new ArrayList<Jurisdiction>(oldJursidictions);

		for (Jurisdiction exjurisdiction : newJursidictions) {
			deletedJurisdictions.remove(exjurisdiction);
		}
		return deletedJurisdictions;
	}

}
