package com.intuit.sb.payroll.capability.ppe.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.payroll.j.CompanyIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.types.PayrollException;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.payroll.types.enumtype.PayrollState;
import com.intuit.sb.payroll.capability.ppe.AgencyView;
import com.intuit.sb.payroll.capability.ppe.PPEUtil;
import com.intuit.sb.payroll.capability.ppe.PayrollProcessingEngine;
import com.intuit.sb.payroll.capability.workerpayments.CalculationEngineException;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Jurisdiction;

@Stateless(name = "PayrollProcessingEngine")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class PayrollProcessingEngineBean implements PayrollProcessingEngine 
{

	public ArrayList<AgencyView> getEmployerAgencyDetails(Employer employer, Jurisdiction[] jurisdictions) 
	{
        try 
        {

            CompanyIf co = PPEUtil.createCompany(employer);

            // Specify that this company is new to payroll (makes the SUI rate notes show up
            // and defaults to inexperience rate)
            co.setDatumValue(DatumID.NEW_COMPANY, Boolean.TRUE);

            // Get federal and CA state required taxes.
            // Use the natural sort order of pitems which puts Federal taxes
            // before state and then, income taxes, unemployment, surcharge, disability etc. 
            Collection<PayrollItemIf> taxes = new TreeSet<PayrollItemIf>();
            for(Jurisdiction jur : jurisdictions)
            {
            	try
            	{
	            	PayrollState state = PPEUtil.mapJurisdiction(jur); //PayrollState == null  means Federal
	            	taxes.addAll(co.getRequiredTaxes(state, co.getSystemDate()));
            	}
            	catch(CalculationEngineException ce)
            	{
            		;//ignore the exception that the jurisdiction is not supported, skip to the next jurisdiction
            	}
            }

            // Use the natural sort order of pitems, but chunk them according to agency
            Map<String, AgencyView> agencyMap = new HashMap<String, AgencyView>();
            ArrayList<AgencyView> orderedAgencies = new ArrayList<AgencyView>();
            for (PayrollItemIf tax : taxes)
            {
                // Key is combination of account number label and default agency name
                String agencyKey = PPEUtil.getAgencyKey (tax);
                AgencyView agencyView = agencyMap.get(agencyKey);
                if (agencyView == null) 
                {
                	PayrollItemDetailType pItemTpe = tax.getDetailType();
                	PayrollItemDetailType.ID jurisdictionId = pItemTpe.getInternalID();
                    agencyView = new AgencyView(jurisdictionId.toString(),
                    							pItemTpe.getVendorDefaultName(), 
                    							tax.getDatum(DatumID.ACCOUNT_NUMBER));
                    orderedAgencies.add(agencyView);
                    agencyMap.put(agencyKey, agencyView);
                }
                agencyView.addTax(tax);
            }
            return orderedAgencies;
        } 
        catch (PayrollException e) 
        {
        	throw new CalculationEngineException(e);
        }
	}

}
