package com.intuit.sb.payroll.capability.pts.adapter;

import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.jboss.ws.WSException;

import com.intuit.sb.payroll.capability.config.GeminiConfigurationUtility;
import com.intuit.sb.payroll.capability.pts.adapter.util.PTSLoggerUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sbd.payroll.psp.adapters.gemini.AdapterResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyAdapter;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyAdapterService;
import com.intuit.sbd.payroll.psp.adapters.gemini.ContactAdapterDTO;

/**
 * PSP Request wrapper. This class will serve as wrapper over TPS webService for all
 * Company related services
 * 
 * @author amalik
 *
 */
public class CompanyRequestWrapper {
	
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(CompanyRequestWrapper.class);
	private static final String COMPANY_ADAPTER_WSDL      = "/gemini-ws/services/CompanyAdapter?wsdl";
	
	public AdapterResult addCompanyinPTS(CompanyAdapterDTO companyDTO) {
		sLogger.debug("Adding Company to PSP....");
		CompanyAdapter companyAdapter = getCompanyAdapter();
        AdapterResult adapterResult = null;
        try {
        	logCompanyAdapterDTO(companyDTO);
        	adapterResult = companyAdapter.addCompany(companyDTO);
        } catch (WSException e) {
			//  Exception saving to PSP - for now we just save it locally and proceed
			//	We log the endpoint exception (thrown when PSP web service is not available
			//	and proceed - in this case AdapterResult.success = false will be set
			sLogger.error("Error saving to PSP - local save successful", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}
	
	/**
	 * update the companyInfo in PTS
	 * @param companyDTO
	 * @return adapterResult
	 */
	public AdapterResult updateCompanyinPTS(CompanyAdapterDTO companyDTO) {
		sLogger.debug("Updating Company to PSP....");
		CompanyAdapter companyAdapter = getCompanyAdapter();
        AdapterResult adapterResult = null;
        try {
        	logCompanyAdapterDTO(companyDTO);
        	adapterResult = companyAdapter.updateCompany(companyDTO);
        } catch (WSException e) {
			//  Exception saving to PSP - for now we just save it locally and proceed
			//	We log the endpoint exception (thrown when PSP web service is not available
			//	and proceed - in this case AdapterResult.success = false will be set
			sLogger.error("Error saving to PSP - local save successful", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
        }
        return adapterResult;
	}
	
	public AdapterResult addJurisdictionToCompany(String sourceCompanyId, String jurisdictionId){
		AdapterResult adapterResult;
		sLogger.debug("Sending Jurisdiction "+ jurisdictionId + "to PSP " + sourceCompanyId);
		try {
			CompanyAdapter companyAdapter = getCompanyAdapter();
			adapterResult = companyAdapter.addCompanyJurisdiction(sourceCompanyId, jurisdictionId);
		} catch (WSException e) {
			sLogger.error("Error saving Jurisdiction to PSP - local save successful", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
		}
		return adapterResult;
	}
	
	public AdapterResult addCompanyTaxAgencyId(String sourceCompanyId, 
					String agencyId, String companyAgencyRegistrationId){
		AdapterResult adapterResult = null;
		
		if (sLogger.isDebugEnabled()){
			StringBuffer sb = new StringBuffer();
			sb.append("Sending Employer ID Info to PSP....").append("\n")
			  .append("\t SourceCompanyId             | "+ sourceCompanyId).append("\n")
			  .append("\t AgencyId                    | "+ agencyId).append("\n")
			  .append("\t companyAgencyRegistrationId | "+ companyAgencyRegistrationId).append("\n");
			sLogger.debug(sb.toString());
		}
		
		

		try {
			CompanyAdapter companyAdapter = getCompanyAdapter();
			adapterResult = companyAdapter.addCompanyTaxAgencyId(
					sourceCompanyId, agencyId, companyAgencyRegistrationId);
		} catch (WSException e) {
			//  Exception saving to PSP - for now we just save it locally and proceed
			//	We log the endpoint exception (thrown when PSP web service is not available
			//	and proceed - in this case AdapterResult.success = false will be set
			sLogger.error("Error Sending Employer ID INfo to PSP.... - local save successful", e);
			adapterResult = new AdapterResult();
			adapterResult.setSuccess(false);
		}
		return adapterResult;
	}
	

	/**
	 * Method to create CompanyAdapter based on the TPS host provided in the Config bean
	 * @return
	 */
	private static CompanyAdapter getCompanyAdapter() {
		String tpsHost = GeminiConfigurationUtility.getTaxPaymentServiceHost();
		CompanyAdapterService companyAdapterService = null;
		
		if ((tpsHost == null) || (tpsHost.length() == 0)){
			// Use the default one used while building this service
			companyAdapterService = new CompanyAdapterService();
		}else{
			String wsdlUrlStr = new StringBuffer(tpsHost).append(COMPANY_ADAPTER_WSDL).toString();
			URL url = null;
	        try {
	            URL baseUrl = com.intuit.sbd.payroll.psp.adapters.gemini.CompanyAdapterService.class.getResource(".");
	            url = new URL(baseUrl, wsdlUrlStr);
	        } catch (MalformedURLException e) {
	        	sLogger.warn("Failed to create URL for the wsdl Location: "+ wsdlUrlStr);
	        	sLogger.warn(e.getMessage());
	        }
	        QName qname = new QName("http://gemini.adapters.psp.payroll.sbd.intuit.com/", "CompanyAdapterService");
	        companyAdapterService = new CompanyAdapterService(url, qname);
		}
		 
		CompanyAdapter companyAdapter = companyAdapterService.getCompanyAdapterPort();
		return companyAdapter;
	}
	
	private void logCompanyAdapterDTO(CompanyAdapterDTO companyDTO) {
		if (sLogger.isDebugEnabled()){
			if (companyDTO == null){
				sLogger.debug("\t CompanyAdapterDTO is <null>");
				return;
			}
			StringBuffer sb = new StringBuffer();
			sb.append("\t SourceCompanyId             | ").append(companyDTO.getSourceCompanyId()).append("\n")
			  .append("\t OfferingId                  | ").append(companyDTO.getOffering()).append("\n")
			  .append("\t TaxExemptExpirationDate     | ").append( companyDTO.getTaxExemptExpirationDate()).append("\n")
	  		  .append("\t PayrollFrequencyCd          | ").append( companyDTO.getPayrollFrequencyCd()).append("\n")
			  .append("\t DoingBusinessAsName         | ").append( companyDTO.getDoingBusinessAsName()).append("\n")
			  .append("\t Legal Name                  | ").append( companyDTO.getLegalName()).append("\n")
			  .append("\t Fein                        | ").append( companyDTO.getFein()).append("\n")

			 .append("\t\t SecondaryContact ").append("\n")
			 .append(getContactAdapterDTOStr(companyDTO.getSecondaryContact())).append("\n")
			
			 .append("\t\t PrimaryContact ").append("\n")
			 .append(getContactAdapterDTOStr(companyDTO.getPrimaryContact())).append("\n")
			
			 .append("\t\t PayrollAdminContact ").append("\n")
			 .append(getContactAdapterDTOStr(companyDTO.getPayrollAdminContact())).append("\n")
			 .append("\t\t MailingAddress ").append("\n")
			 .append(PTSLoggerUtil.logAddressAdapterDTO(companyDTO.getMailingAddress(), "MailingAddress")).append("\n")
			 .append("\t\t LegalAddress ").append("\n")
			 .append(PTSLoggerUtil.logAddressAdapterDTO(companyDTO.getLegalAddress(), "LegalAddress")).append("\n");
			
			sLogger.debug(sb.toString());
		}
	}
	
	private String getContactAdapterDTOStr(ContactAdapterDTO contactAdapterDTO){
		if (contactAdapterDTO == null){
			return "\t ContactAdapterDTO is <null>";
		}
		
		StringBuffer logStrBuffer = new StringBuffer();
		logStrBuffer
			.append("\t SourceContactId             | " + contactAdapterDTO.getSourceContactId()).append("\n")
		    .append("\t AccountSignatory            | " + contactAdapterDTO.isAccountSignatory()).append("\n")
		    .append("\t\t ContactAddress " ).append("\n")
		    .append(PTSLoggerUtil.logAddressAdapterDTO(contactAdapterDTO.getAddress(), "Contact Address")).append("\n")
	    	.append("\t Email                       | " + contactAdapterDTO.getEmail()).append("\n")
	    	.append("\t FaxNumber                   | " + contactAdapterDTO.getFaxNumber()).append("\n")
    		.append("\t FirstName                   | " + contactAdapterDTO.getFirstName()).append("\n")
   			.append("\t JobTitle                    | " + contactAdapterDTO.getJobTitle()).append("\n")
			.append("\t LastName                    | " + contactAdapterDTO.getLastName()).append("\n")
			.append("\t MiddleName                  | " + contactAdapterDTO.getMiddleName()).append("\n")
			.append("\t PhoneNumber                 | " + contactAdapterDTO.getPhoneNumber()).append("\n")
			.append("\t SecondPhoneNumber           | " + contactAdapterDTO.getSecondPhoneNumber()).append("\n")
			.append("\t Title                       | " + contactAdapterDTO.getTitle()).append("\n")
			.append("\t TitleSuffix                 | " + contactAdapterDTO.getTitleSuffix()).append("\n");
		
		return logStrBuffer.toString();
	}
}
