package com.intuit.sb.payroll.capability.pts.adapter.mapper;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompensationTransactionAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.DateAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.DeductionTransactionAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.LiabilityTransactionAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PaycheckAdapterDTO;

public class PaycheckMapper {
	public static PaycheckAdapterDTO mapToPaycheck(Paycheck paycheck) throws ServiceFailureException {
		PaycheckAdapterDTO paycheckAdapterDTO = new PaycheckAdapterDTO();
		
		paycheckAdapterDTO.setPaycheckNetAmount(paycheck.getNetAmount().getAmount());
		DateAdapterDTO startDate = DateMapper.mapDateToDTO(paycheck.getPayPeriod().getStartDate());
		DateAdapterDTO endDate = DateMapper.mapDateToDTO(paycheck.getPayPeriod().getEndDate());
		paycheckAdapterDTO.setPayPeriodBeginDate(startDate);
		paycheckAdapterDTO.setPayPeriodEndDate(endDate);
		paycheckAdapterDTO.setSourceEmployeeId(String.valueOf(paycheck.getWorker().getId()));
		
		// Will need to fix PayrollRunMapper.mapToPTSVoidPayrollResult as it depends on SourcePaycheckId
		// for saving voided paychecks in PPS
		paycheckAdapterDTO.setSourcePaycheckId(String.valueOf(paycheck.getId()));
		
		Set<PaycheckLineItem> paycheckLineItems = paycheck.getPaycheckLineItems();
		List<CompensationTransactionAdapterDTO> compensationTxns = paycheckAdapterDTO.getCompensationTransactions();
		List<DeductionTransactionAdapterDTO> deductionTxns = paycheckAdapterDTO.getDeductionTransactions();
		List<LiabilityTransactionAdapterDTO> liabilityTxns = paycheckAdapterDTO.getLiabilityTransactions();
		
		for (PaycheckLineItem paycheckLineItem : paycheckLineItems) {
			switch(paycheckLineItem.getSortSequence()){
				case WAGES:
				case PAIDTIMEOFF:	
					compensationTxns.add(getCompensationTransactionAdapterDTO(paycheckLineItem));
					break;
				case PRETAXDEDUCTION:
				case POSTTAXDEDUCTION:
					deductionTxns.add(getDeductionTransactionAdapterDTO(paycheckLineItem));
					break;
				case EMPLOYERPAIDTAX:
				case EMPLOYEEPAIDTAX:
					liabilityTxns.add(createLiabilityTransactionAdapterDTO(paycheckLineItem));
			}
		}
		return paycheckAdapterDTO;
	}

	private static LiabilityTransactionAdapterDTO createLiabilityTransactionAdapterDTO(
			PaycheckLineItem paycheckLineItem) {
		LiabilityTransactionAdapterDTO liability = new LiabilityTransactionAdapterDTO();
		liability.setLawId(paycheckLineItem.getLawId().getValue());
		liability.setLiabilityAmount(paycheckLineItem.getValue().getAmount());
		liability.setLiabilityTaxableWagesAmount(paycheckLineItem.getIncomeSubjectToTax().getAmount());
		liability.setLiabilityTotalWagesAmount(paycheckLineItem.getWageBase().getAmount());
		return liability;
	}

	private static CompensationTransactionAdapterDTO getCompensationTransactionAdapterDTO(
			PaycheckLineItem paycheckLineItem) {
		CompensationTransactionAdapterDTO compensationTransactionAdapterDTO = new CompensationTransactionAdapterDTO();
		//TODO: verify is the logic amount is set is right
		compensationTransactionAdapterDTO.setCompensationAmount(paycheckLineItem.getValue().getAmount());
		
		BigDecimal hours;
		if(paycheckLineItem.getQuantity() != null && 
				paycheckLineItem.getQuantity().getValue1AsQuantity() != null) {
			hours = new BigDecimal(paycheckLineItem.getQuantity().getValue1AsQuantity());
		}
		else {
			hours = new BigDecimal("0.00");
		}
		hours = hours.setScale(2, RoundingMode.HALF_UP);
		compensationTransactionAdapterDTO.setCompensationHoursWorked(hours);
		compensationTransactionAdapterDTO.setSourceCompensationId(String.valueOf(paycheckLineItem.getId()));
		return compensationTransactionAdapterDTO;
	}
	
	private static DeductionTransactionAdapterDTO getDeductionTransactionAdapterDTO(
			PaycheckLineItem paycheckLineItem) {
		DeductionTransactionAdapterDTO deductionTransactionAdapterDTO = new DeductionTransactionAdapterDTO();
		deductionTransactionAdapterDTO.setDeductionAmount(paycheckLineItem.getValue().getAmount());
		deductionTransactionAdapterDTO.setSourceDeductionId(String.valueOf(paycheckLineItem.getId()));
		return deductionTransactionAdapterDTO;
	}
}
