/**
 *
 */
package com.intuit.sb.payroll.capability.pts.adapter.mapper;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.pts.LiabilityBalance;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerAdjLineItem;
import com.intuit.sb.payroll.entity.model.EmployerAdjustment;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.pts.entity.PTSCancelledPaycheck;
import com.intuit.sb.payroll.pts.entity.PTSCancelledPaycheckEAO;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSPayrollRun;
import com.intuit.sb.payroll.pts.entity.PTSPayrollRunEAO;
import com.intuit.sb.payroll.pts.entity.PTSVoidPayrollResult;
import com.intuit.sb.payroll.pts.entity.PTSVoidPayrollResultEAO;
import com.intuit.sb.payroll.pts.entity.TaxCredit;
import com.intuit.sb.payroll.pts.entity.TaxCreditEAO;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;
import com.intuit.sbd.payroll.psp.adapters.gemini.CancelledPaycheckDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyBankInfoAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.LiabilityBalanceAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PaycheckAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollRunAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollRunTypeAdapterEnum;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollTaxAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollTaxCreditAmountDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.SubmitPayrollResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.VoidPayrollResult;

/**
 * @author dkulkarni
 *
 */
public class PayrollRunMapper {
	public static PayrollRunAdapterDTO mapToPayrollRunDTO(PTSEmployer ptsEmployer, PayrollRun payrollRun) throws ServiceFailureException{
		Employer employer = ptsEmployer.getEmployer();
		PayrollRunAdapterDTO payrollRunAdapterDTO = new PayrollRunAdapterDTO();

		payrollRunAdapterDTO.setSourceCompanyId(CompanyMapper.namespaceEmployerId(employer));
		payrollRunAdapterDTO.setPaycheckDate(DateMapper.mapDateToDTO(payrollRun.getPayDate()));
		payrollRunAdapterDTO.setPayrollRunType(mapToPayrollRunType(ptsEmployer, payrollRun));
		payrollRunAdapterDTO.setSourcePayrollRunId(String.valueOf(payrollRun.getId()));

		List<PaycheckAdapterDTO> dtoPaychecks = payrollRunAdapterDTO.getPaychecks();
		Set<Paycheck> entityPaychecks = payrollRun.getPaychecks();
		for (Paycheck paycheck : entityPaychecks) {
			if (paycheck.getWorkflowStatus()==PaycheckWorkflowStatusEnum.VOIDED){
				continue;
			}
			PaycheckAdapterDTO dtoPaycheck = PaycheckMapper.mapToPaycheck(paycheck);
			dtoPaychecks.add(dtoPaycheck);
		}

		Set<BankAccount> bankAccounts = employer.getBankAccounts();
		List<CompanyBankInfoAdapterDTO> companyBankInfoAdapterDTOs = payrollRunAdapterDTO.getCompanyBankInfo();

		//There must be only one bank account
		for (BankAccount bankAccount : bankAccounts) {
			CompanyBankInfoAdapterDTO companyBankInfoAdapterDTO = BankAccountMapper.mapToCompanyBankInfoAdapterDTO(bankAccount);
			companyBankInfoAdapterDTOs.add(companyBankInfoAdapterDTO);
		}

		Set<EmployerAdjustment> erAdjustments = payrollRun.getEmployerAdjustments();
		List<PayrollTaxAdapterDTO> payrollTaxAdapterDTOs = payrollRunAdapterDTO.getPayrollTaxes();
		for (EmployerAdjustment employerAdjustment : erAdjustments) {
			List<PayrollTaxAdapterDTO> adjustmentDTOs = mapToPayrollTaxAdapterDTO(employerAdjustment);
			payrollTaxAdapterDTOs.addAll(adjustmentDTOs);
		}
		return payrollRunAdapterDTO;
	}

	public static PTSPayrollRun mapToPTSPayrollRun(SubmitPayrollResult submitPayrollResult, PayrollRun payrollRun) {
		PTSPayrollRunEAO ptsPayrollRunEAO = ServiceLocator.getInstance().getService(PTSPayrollRunEAO.JNDI_NAME);
		PTSPayrollRun ptsPayrollRun = ptsPayrollRunEAO.create();
		ptsPayrollRun.setDdDebitAmount(submitPayrollResult.getEmployerDdDebitAmount());
		ptsPayrollRun.setDebitAmount(submitPayrollResult.getEmployerDebitAmount());
		List<PayrollTaxCreditAmountDTO> taxCredits = submitPayrollResult.getEmployerTaxCreditsByPayroll();
		for (PayrollTaxCreditAmountDTO payrollTaxCreditAmountDTO : taxCredits) {
			TaxCreditEAO taxCreditEAO = ServiceLocator.getInstance().getService(TaxCreditEAO.JNDI_NAME);
			TaxCredit taxCredit = taxCreditEAO.create();
			taxCredit.setTaxCreditAmount(payrollTaxCreditAmountDTO.getTaxCreditAmount());
			taxCredit.setTaxCreditPayrollRunId(new Long(payrollTaxCreditAmountDTO.getSourcePayrollRunId()));
			taxCreditEAO.makePersistent(taxCredit);
			ptsPayrollRun.addTaxCredit(taxCredit);
		}
		ptsPayrollRun.setSettlementDate(DateMapper.mapDateAdapterDTOToDate(submitPayrollResult.getSettlementDate()));
		ptsPayrollRun.setTaxDebitAmount(submitPayrollResult.getEmployerTaxDebitAmount());
		ptsPayrollRun.setPayrollRunId(payrollRun.getId());
		return ptsPayrollRun;
	}

	public static List<PayrollTaxAdapterDTO> mapToPayrollTaxAdapterDTO(EmployerAdjustment employerAdjustment) throws ServiceFailureException {
		Set<EmployerAdjLineItem> adjustmentItems = employerAdjustment.getEmployerAdjLineItems();
		List<PayrollTaxAdapterDTO> payrollTaxAdapterDTOs = new  ArrayList<PayrollTaxAdapterDTO>();
		for (EmployerAdjLineItem employerAdjLineItem : adjustmentItems) {
			PayrollTaxAdapterDTO payrollTaxAdapterDTO = new PayrollTaxAdapterDTO();
			payrollTaxAdapterDTO.setAmount(employerAdjLineItem.getAdjustmentAmount().getAmount());
			payrollTaxAdapterDTO.setLawId(employerAdjLineItem.getLawId().getValue());
			payrollTaxAdapterDTO.setEffectiveDate(DateMapper.mapDateToDTO(employerAdjustment.getEffectiveDate()));
			payrollTaxAdapterDTOs.add(payrollTaxAdapterDTO);
		}
		return payrollTaxAdapterDTOs;
	}

	// TODO revisit for the correct type
	public static PayrollRunTypeAdapterEnum mapToPayrollRunType(PTSEmployer ptsEmployer, PayrollRun payrollRun) {
		TaxServiceStatusEnum tpsStatus = ptsEmployer.getCurrentTaxServiceStatus();
		if (tpsStatus != TaxServiceStatusEnum.ACTIVATION_ACTIVE_ST_ACCEPTED){
			return PayrollRunTypeAdapterEnum.HISTORICAL;
		}
		PayrollRunTypeEnum payrollRunType = payrollRun.getTypeOf();
		if (payrollRunType == null){
			return null;
		}

		switch (payrollRunType){
		case SCHEDULED:
		case SYSTEM:
			return PayrollRunTypeAdapterEnum.REGULAR;
		case ADHOC:
			return PayrollRunTypeAdapterEnum.REGULAR;
		case TERMINATION:
			return PayrollRunTypeAdapterEnum.ADJUSTMENT;
		default:
		    return PayrollRunTypeAdapterEnum.REGULAR;
		}
	}

	public static PTSVoidPayrollResult mapToPTSVoidPayrollResult(VoidPayrollResult voidPayrollResult, PayrollRun payrollRun) {
		if(voidPayrollResult == null){
			return null;
		}
		PTSVoidPayrollResultEAO ptsVoidPayrollResultEAO = ServiceLocator.getInstance().getService(PTSVoidPayrollResultEAO.JNDI_NAME);
		PTSCancelledPaycheckEAO ptsCancelledPaycheckEAO = ServiceLocator.getInstance().getService(PTSCancelledPaycheckEAO.JNDI_NAME);
		PTSVoidPayrollResult ptsVoidPayrollResult = ptsVoidPayrollResultEAO.create();
		
		ptsVoidPayrollResult.setDdDebitAmount(voidPayrollResult.getEmployerDdDebitAmount());
		ptsVoidPayrollResult.setDebitAmount(voidPayrollResult.getEmployerDebitAmount());
		ptsVoidPayrollResult.setSettlementDate(DateMapper.mapDateAdapterDTOToDate(voidPayrollResult.getSettlementDate()));
		ptsVoidPayrollResult.setTaxDebitAmount(voidPayrollResult.getEmployerTaxDebitAmount());
		ptsVoidPayrollResult.setAppliedForwardRefund(voidPayrollResult.getAppliedForwardRefundAmount());
		ptsVoidPayrollResult.setCancelledAmount(voidPayrollResult.getCancelledAmount());
		ptsVoidPayrollResult.setImmediateRefund(voidPayrollResult.getImmediateRefundAmount());
		ptsVoidPayrollResult.setTakeOnReturnRefund(voidPayrollResult.getTakeOnReturnRefundAmount());
		
		List<CancelledPaycheckDTO> cancelledPaychecks = voidPayrollResult.getCancelledPaychecks();
		for (CancelledPaycheckDTO cancelledPaycheckDTO : cancelledPaychecks) {
			PTSCancelledPaycheck ptsCancelledPaycheck = ptsCancelledPaycheckEAO.create();
			ptsCancelledPaycheck.setAmount(cancelledPaycheckDTO.getAmount());
			ptsCancelledPaycheck.setPaycheckId(new Long(cancelledPaycheckDTO.getSourcePaycheckId()));
			ptsCancelledPaycheckEAO.makePersistent(ptsCancelledPaycheck);
			ptsVoidPayrollResult.addPTSCancelledPaycheck(ptsCancelledPaycheck);
		}
		return ptsVoidPayrollResult;
		
	}

	public static List<LiabilityBalance> mapToLiabilityBalances(
			List<LiabilityBalanceAdapterDTO> balances) {
		if(balances == null) {
			return null;
		}
		List<LiabilityBalance> balancesList = new ArrayList<LiabilityBalance>();
		for (LiabilityBalanceAdapterDTO liabilityBalanceAdapterDTO : balances) {
			LiabilityBalance liabilityBalance = new LiabilityBalance();
			liabilityBalance.setLawId(liabilityBalanceAdapterDTO.getLawId());
			liabilityBalance.setLiabilityAmount(liabilityBalanceAdapterDTO.getLiabilityAmount());
			liabilityBalance.setPaymentAmount(liabilityBalanceAdapterDTO.getPaymentAmount());
			liabilityBalance.setBalance(liabilityBalanceAdapterDTO.getBalance());
			balancesList.add(liabilityBalance);
		}
		return balancesList;
	}
}
