package com.intuit.sb.payroll.capability.pts.notification.mapper;

import com.intuit.sb.payroll.capability.pts.adapter.dtos.CompanyActivationStatusEnum;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.pts.entity.ACHEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.ActivationStatusEnum;
import com.intuit.sb.payroll.pts.entity.BankAccountStatusEnum;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.RAAEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.RAFEnrollmentStatusEnum;

public class NotificationStatusMapper {
	
	public static Object mapNotificationStatus(
			PTSNotificationTypeEnum notificationType, 
			CompanyActivationStatusEnum newNotificationStatus) {
		
		
		if (newNotificationStatus == null){
			return null;
		}
		switch (newNotificationStatus){
			/* For ACH Activations */
			case ACH_ENRL_COMPLETE: 
				return ACHEnrollmentStatusEnum.ACH_ENROLLMENT_COMPLETE;
			case ACH_ENRL_IN_PROGRESS: 
				return ACHEnrollmentStatusEnum.ACH_ENROLLMENT_IN_PROGRESS;
			case ACH_ENRL_INCOMPLETE: 
				return ACHEnrollmentStatusEnum.ACH_ENROLLMENT_INCOMPLETE;
			
			/* For Activations */
			case ACTIVATION_ACTIVE: 
			{
				if (PTSNotificationTypeEnum.BANK_ACCOUNT_NOTIFICATION.equals(notificationType)){
					return BankAccountStatusEnum.BANKACCOUNT_VERIFIED_RECEIVED;
				}
				return ActivationStatusEnum.ACTIVATION_ACTIVE_ST_RECEIVED;
			}
			case ACTIVATION_AGENT_REVIEW_APPROVED: 
				// FIXME : PSP due to some issue does not sends ACTIVATION_ACTIVE status.
				// for our testing we will map this status to activation active
				// return ActivationStatusEnum.ACTIVATION_AWAITING_AGENT_REVIEW_ST_RECEIVED;
				return ActivationStatusEnum.ACTIVATION_ACTIVE_ST_RECEIVED;
			case ACTIVATION_AWAITING_AGENT_REVIEW: 
				return ActivationStatusEnum.ACTIVATION_AWAITING_AGENT_REVIEW_ST_RECEIVED;
			case ACTIVATION_CANCELLED:
			{	
				if (PTSNotificationTypeEnum.BANK_ACCOUNT_NOTIFICATION.equals(notificationType)){
					return BankAccountStatusEnum.BANKACCOUNT_VERIFICATION_FAILED_RECEIVED;
				}
				return ActivationStatusEnum.ACTIVATION_CANCELLED_ST_RECEIVED;
			}
			case ACTIVATION_PENDING: 
			{
				if (PTSNotificationTypeEnum.BANK_ACCOUNT_NOTIFICATION.equals(notificationType)){
					return BankAccountStatusEnum.BANKACCOUNT_UNDER_VERIFICATION;
				}
				return ActivationStatusEnum.ACTIVATION_PENDING_RECEIVED;
			}
			/* For EFTPS Enrollment */
			case EFTPS_ENRL_CANCELLED: 
				return EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_CANCELLED;
			case EFTPS_ENRL_COMPLETE: 
				return EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_COMPLETE;
			case EFTPS_ENRL_IN_PROGRESS: 
				return EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_INPROGRESS;
			case EFTPS_ENRL_INCOMPLETE: 
				return EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_INCOMPLETE;
			case EFTPS_ENRL_REJECTED_EIN_MIS_MATCH: 
				return EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH;
			case EFTPS_ENRL_REJECTED_NAME_MIS_MATCH: 
				return EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH;
			case EFTPS_ENRL_REJECTED_AGEDOUT: 
				return EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_AGEDOUT;
			
			/* RAA Enrollment */
			case RAA_ENRL_AWAITING_SIGNED_FORMS: 
				return RAAEnrollmentStatusEnum.RAA_ENROLLMENT_AWAITING_SIGNED_FORMS;
			case RAA_ENRL_COMPLETE: 
				return RAAEnrollmentStatusEnum.RAA_ENROLLMENT_COMPLETE;
			case RAA_ENRL_INCOMPLETE: 
				return RAAEnrollmentStatusEnum.RAA_ENROLLMENT_INCOMPLETE;
			case RAA_ENRL_SIGNED_FORMS_RECEIVED_AWAITING_REVIEW: 
				return RAAEnrollmentStatusEnum.RAA_ENROLLMENT_SIGNED_FORMS_RCVD_AWAITING_REVIEW;
			case RAA_ENRL_SIGNED_FORMS_REJECTED: 
				return RAAEnrollmentStatusEnum.RAA_ENROLLMENT_SIGNED_FORMS_REJECTED;
			
				/* RAF Enrollment */
			case RAF_ENRL_AGENCY_REJECTED: 
				return RAFEnrollmentStatusEnum.RAF_ENROLLMENT_AGENCY_REJECTED;
			case RAF_ENRL_AGENT_REJECTED_FORM: 
				return RAFEnrollmentStatusEnum.RAF_ENROLLMENT_AGENT_REJECTED_FORM;
			case RAF_ENRL_AWAITING_SIGNED_FORM: 
				return RAFEnrollmentStatusEnum.RAF_ENROLLMENT_AWAITING_SIGNED_FORMS;
			case RAF_ENRL_IN_PROGRESS: 
				return RAFEnrollmentStatusEnum.RAF_ENROLLMENT_IN_PROGRESS;
			case RAF_ENRL_INCOMPLETE: 
				return RAFEnrollmentStatusEnum.RAF_ENROLLMENT_INCOMPLETE;
			case RAF_ENRL_RAF_COMPLETE: 
				return RAFEnrollmentStatusEnum.RAF_ENROLLMENT_COMPLETE;
			case RAF_ENRL_READY_FOR_RAF: 
				return RAFEnrollmentStatusEnum.RAF_ENROLLMENT_READY_FOR_RAF;
			case RAF_ENRL_READY_FOR_TAPE: 
				return RAFEnrollmentStatusEnum.RAF_ENROLLMENT_READY_FOR_TAPE;
			default :
				return null;
		}
	}

}
