package com.intuit.sb.payroll.capability.pts.notification.mapper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationAttribute;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.PTSNotificationDTO;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.DateMapper;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.pts.entity.PTSMessage;

public class PTSNotificationRequestMapper {

	private static Logger sLogger = PayrollLoggerFactory.makeLogger(PTSNotificationRequestMapper.class);
	
	/* List of attributes provided in notificationAdapterDTO by PTS */
	public static final String SOURCE_BANK_ACCOUNT_ID = "SourceBankAccountId";
	public static final String AGENCY_ID = "AgencyId";
	public static final String DETAILS = "Details";
	public static final String OLD_VALUE = "OldStringValue";
	public static final String NEW_VALUE = "NewStringValue";
	
	private static List<String> validAttributes = new ArrayList<String>();
	static {
		validAttributes.add(SOURCE_BANK_ACCOUNT_ID);
		validAttributes.add(AGENCY_ID);
		validAttributes.add(DETAILS);
		validAttributes.add(OLD_VALUE);
		validAttributes.add(NEW_VALUE);
	}

	public static PTSNotificationRequest mapPTSNotificationRequest(PTSNotificationDTO ptsNotificationDTO){

		PTSNotificationRequest ptsNotificationRequest = new PTSNotificationRequest();

		PTSNotificationTypeEnum notificationType = PTSNotificationTypeEnumMapper.mapPTSNotficationType(ptsNotificationDTO.getNotificationType());
		ptsNotificationRequest.setNotificationType(notificationType);

		Object notificationStatus = NotificationStatusMapper.mapNotificationStatus(notificationType, ptsNotificationDTO.getNewNotificationStatus());
		ptsNotificationRequest.setNotificationStatus(notificationStatus);

		Object oldNotificationStatus = NotificationStatusMapper.mapNotificationStatus(notificationType, ptsNotificationDTO.getOldNotificationStatus());
		ptsNotificationRequest.setOldNotificationStatus(oldNotificationStatus);


		List<PTSMessage> assistedMessage = PTSMessageMapper.mapPTSMessage(ptsNotificationDTO.getActivationMessage());
		ptsNotificationRequest.setPTSMessage(assistedMessage);

		Date timestamp = DateMapper.mapDatetimeDTOToDate(ptsNotificationDTO.getNotificationChangeTimeStamp());
		ptsNotificationRequest.setTimestamp(timestamp);
		
		List<NotificationAttribute> nAttributes = ptsNotificationDTO.getAttributes();
		for (NotificationAttribute att : nAttributes) {
			if (validAttributes.contains(att.getName())){
				ptsNotificationRequest.addNotificationAttributes(att.getName(), att.getValue());
			}else{
				sLogger.warn("Unknow Attributes received "+ att.getName());
			}
		}

		
		return ptsNotificationRequest;
	}

	
}
