package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Date;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSAgencyMapper;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSWorkflowState;
import com.intuit.sb.payroll.pts.entity.RAAEnrollmentState;
import com.intuit.sb.payroll.pts.entity.RAAEnrollmentStateEAO;
import com.intuit.sb.payroll.pts.entity.RAAEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.TaxServiceState;

public class RAAStatusUpdateHandler extends AbstractStatusUpdateHandler {

	private static RAAStatusUpdateHandler raaStatusUpdateHandler = new RAAStatusUpdateHandler();
	
	private RAAStatusUpdateHandler(){
		registerStatusChangeWorkers();
	}
	
	public void validateNotificationRequest(PTSNotificationRequest ptsNotification) throws PTSNotificationException {
		if (!ptsNotification.getNotificationType().equals(PTSNotificationTypeEnum.RAA_NOTIFICATION)){
			throw new PTSNotificationException("Wrong Notification being handler here");
		}
	}


	public static StatusUpdateHandler getInstance() {
		return raaStatusUpdateHandler;
	}
	
	// Inner classes for Individual state handlers

	static class GenericRAAStatusChangeWorker extends StatusChangeWorker{

		@Override
		public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
			try {
				updateRAAEnrollmentStatus(ptsEmployer, ptsNotification);
			} catch (ValidationException e) {
				throw new PTSNotificationException(e.getLocalizedMessage());
			} catch (ServiceFailureException e) {
				throw new PTSNotificationException(e.getLocalizedMessage());
			}

		}

	}
	
	/**
	 * Generic Post processing event after RAA Enrollment update has been received from PTS
	 */
	@SuppressWarnings("unused")
    private static class CommonStatusChangeWorker extends GenericRAAStatusChangeWorker{
		public CommonStatusChangeWorker() {
			super();
			statusTypeChange.add(RAAEnrollmentStatusEnum.RAA_ENROLLMENT_INCOMPLETE);
			statusTypeChange.add(RAAEnrollmentStatusEnum.RAA_ENROLLMENT_AWAITING_SIGNED_FORMS);
			statusTypeChange.add(RAAEnrollmentStatusEnum.RAA_ENROLLMENT_SIGNED_FORMS_RCVD_AWAITING_REVIEW);
			statusTypeChange.add(RAAEnrollmentStatusEnum.RAA_ENROLLMENT_COMPLETE);
			statusTypeChange.add(RAAEnrollmentStatusEnum.RAA_ENROLLMENT_SIGNED_FORMS_REJECTED);
			statusTypeChange.add(RAAEnrollmentStatusEnum.RAA_ENROLLMENT_READY_FOR_RAF);
		}

    	@Override
		public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
    		super.updateStatus(ptsEmployer, ptsNotification);

		}
    }

	public synchronized  static void updateRAAEnrollmentStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws ValidationException, ServiceFailureException {
		ptsEmployer =  getPTSEmployer();
		RAAEnrollmentStateEAO raaEnrollStateEAO = ServiceLocator.getInstance().getService(RAAEnrollmentStateEAO.JNDI_NAME);
		RAAEnrollmentState raaEnrollmentState = new RAAEnrollmentState();
		raaEnrollmentState.setRaaEnrollmentStatus((RAAEnrollmentStatusEnum) ptsNotification.getNotificationStatus());
		if (ptsNotification.getTimestamp() != null){
			raaEnrollmentState.setStatusDate(ptsNotification.getTimestamp());
		}else{
			raaEnrollmentState.setStatusDate(new Date());
		}

		String agencyId = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.AGENCY_ID);
		if (agencyId != null){
			raaEnrollmentState.setAgency(PTSAgencyMapper.mapAgency(agencyId));
		}
		// FIXME achEnrollmentState.setPTSMessages(ptsNotification.getPTSMessage());
		raaEnrollmentState.setActive(Boolean.TRUE);
		PTSWorkflowState ptsWorkflowState = ptsEmployer.getPTSWorkflowState();
		TaxServiceState taxServiceState = ptsWorkflowState.getTaxServiceState();
		raaEnrollmentState = raaEnrollStateEAO.makePersistent(raaEnrollmentState);
		taxServiceState.addNotificationStatusEntity(raaEnrollmentState);

	}


}
