package com.intuit.sb.payroll.capability.taxes.impl;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.rules.RulesBridge;
import com.intuit.sb.payroll.rules.RulesException;
import com.intuit.sb.payroll.rules.RulesRequest;
import com.intuit.sb.payroll.rules.RulesResult;

public class EmployerTaxSetupRules {
	private static Logger sLogger = PayrollLoggerFactory.makeLogger(EmployerTaxSetupRules.class);
	private static PayrollResourceManager sResMan = PayrollResourceManager.getInstance();

	public static boolean isSSNAllowedForEIN(RulesEmployer employer) throws ServiceFailureException {
		sLogger.trace("Entered isSSNAllowedForEIN");
		RulesBridge rulesBridge = RulesBridge.getInstance();
        RulesResult response = null;
        
        try
        {
            LinkedHashMap<String,String> rulesetMap = rulesBridge.getRulesetMap().get(RulesBridge.EMPLOYER_TAX_SETUP_RULESET_NAME);
            Set<String> uris = rulesetMap.keySet(); //get all the uris in the ruleset
            Iterator<String> iter = uris.iterator();
        	RulesRequest request = createRequest(employer);
            while(iter.hasNext())
            {
            	String uri = iter.next();
            	//FIXME - [rj] We have a problem here
            	//if we have multiple rules files in the rule set
            	//the response object of the previous run should be used 
            	//as the input to the next run within this loop
            	response = rulesBridge.runRules(uri, request);
            	request = createRequest((RulesEmployer)response.getData().get(0)); //FIXME - [rj] position based coding is dangerous, what is a better way?
            }
        }
        catch(Exception e)
        {
            throw new ServiceFailureException(sResMan.getString(ResourceKeyConstants.RULES_EXEC_EXCEPTION), e);
        }
        //get the result from the response
        //FIXME [rjThis is a hack, since updating the original object from rules did not work
        //try updating the original object when rules 4 is integrated
        
        boolean ssnAllowed = false;
        
        List<?> listResults = response.getData();
        Iterator<?> iter = listResults.iterator();
        while(iter.hasNext())
        {
        	Object o = iter.next();
        	if(o instanceof RulesEmployer)
        	{
        		ssnAllowed = ((RulesEmployer)o).isSsnAllowedForEIN();
        		break;
        	}
        }
    	return ssnAllowed;
	}

	private static RulesRequest createRequest(RulesEmployer employer) throws RulesException {
		RulesRequest request = new RulesRequest();
		request.aAddInput(employer);
		return request;
	}
}
