package com.intuit.sb.payroll.capability.taxes.impl;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.rules.RulesBridge;
import com.intuit.sb.payroll.rules.RulesException;
import com.intuit.sb.payroll.rules.RulesRequest;
import com.intuit.sb.payroll.rules.RulesResult;

public class PayrollTaxRules {
	private static Logger sLogger = PayrollLoggerFactory.makeLogger(PayrollTaxRules.class);
	private static PayrollResourceManager sResMan = PayrollResourceManager.getInstance();

	/**
	 * Note : applicableTaxes are modified by this method
	 * 
	 * @param employee
	 * @param employer
	 * @param applicableTaxes
	 * @throws ServiceFailureException when rules execution fails
	 */
	@SuppressWarnings("unchecked")
	public static List<Law> getApplicableTaxesForEmployee(RulesEmployee employee, RulesEmployer employer,
			List<Law> applicableTaxes) throws ServiceFailureException {
		sLogger.trace("Entered getApplicableTaxesForEmployee");
		RulesBridge rulesBridge = RulesBridge.getInstance();
		RulesResult response = null;
		try {
			LinkedHashMap<String, String> rulesetMap = rulesBridge.getRulesetMap().get(
					RulesBridge.EMPLOYEE_TAX_SETUP_RULESET_NAME);
			Set<String> uris = rulesetMap.keySet(); // get all the uris in the ruleset
			RulesRequest request = createRequest(employee, employer, applicableTaxes);
			for (String uri : uris) {
				response = rulesBridge.runRules(uri, request);
				request.initialize(response);
			}
		} catch (Exception e) {
			throw new ServiceFailureException(sResMan.getString(ResourceKeyConstants.RULES_EXEC_EXCEPTION), e);
		}

		sLogger.trace("Exited getApplicableTaxesForEmployee");
		// Look for TaskNameList in the result data
		List<Object> data = response.getData();
		// TODO not right way to locate the output Should hide this from users
		for (Object obj : data) {
			if (obj instanceof List) {
				applicableTaxes = (List<Law>) obj;
				break;
			}
		}
		return applicableTaxes;
	}

	private static RulesRequest createRequest(RulesEmployee employee, RulesEmployer employer, List<Law> applicableTaxes)
			throws RulesException {
		RulesRequest request = new RulesRequest();
		request.aAddInput(employee);
		request.aAddInput(employer);
		request.aAddInput(applicableTaxes);
		return request;
	}

}
