package com.intuit.sb.payroll.capability.taxes.withholding;

import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * Interface to support the management of WorkerComplianceDocs.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
@Local
public interface TaxWithholdingsComponent {
	
	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "TaxWithholdingsComponent/local";
	
	/**
	 * Factory method to create an empty {@link WorkerComplianceDocDtl} of the type specified.
	 * @param docType
	 * @return a transient {@link WorkerComplianceDocDtl}.
	 * @throws ValidationException 
	 */
	public WorkerComplianceDocDtl createWorkerComplianceDocDtl(ComplianceDocIdEnum docType) throws ValidationException;
	
	/** 
	 * Add the WorkerComplianceDocDtls to the given worker.
	 * Throws if the given WorkerComplianceDocDtl already has an id or is a duplicate type.
	 * @param worker
	 * @param workerComplianceDocDtl
	 * @return updated Worker
	 * @throws ServiceFailureException 
	 * @throws WorkflowValidationException 
	 * @throws PayrollValidationException 
	 */
	public Worker addWorkerComplianceDocDtl(Worker worker, WorkerComplianceDocDtl workerComplianceDocDtl) 
		throws WorkflowException, ValidationException, ServiceFailureException;
	
	/**
	 * Update the WorkerComplianceDocDtl.
	 * Throws if the given WorkerComplianceDocDtl does not have an id or is not on the worker.
	 * @param worker
	 * @param workerComplianceDocDtl
	 * @return
	 * @throws ServiceFailureException 
	 * @throws PayrollValidationException 
	 * @throws WorkflowValidationException 
	 */
	public Worker updateWorkerComplianceDocDtl(Worker worker, WorkerComplianceDocDtl workerComplianceDocDtl) 
		throws ValidationException, WorkflowException, ServiceFailureException;

	/**
	 * Delete the WorkerComplianceDocDtl.
	 * Throws if the given WorkerComplianceDocDtl does not have an id or is not on the worker.
	 * @param worker
	 * @param workerComplianceDocDtl
	 * @return
	 * @throws ValidationException
	 * @throws WorkflowException
	 * @throws ServiceFailureException
	 */
	public Worker deleteWorkerComplianceDocDtl(Worker worker, WorkerComplianceDocDtl workerComplianceDocDtl)
		throws ValidationException, WorkflowException, ServiceFailureException;
	
    /**
     * Find and return the WorkerComplianceDocDtl with the given Identifiable.
     * Throws if the the WorkerComplianceDocDtl in the database has a newer version than the identifiable.
     * @param identifiable
     * @return the WorkerComplianceDocDtl
     * @throws PayrollEntityNotFoundException
     */
	public WorkerComplianceDocDtl findWorkerComplianceDocDtl(Identifiable identifiable) throws ServiceFailureException;
	
	/**
	 * Based on the current settings in the {@link Worker}, return the existing and derived (if needed) withholding docs.
	 * 
	 * @param worker
	 * @return
	 * @throws ServiceFailureException
	 * @throws WorkflowException
	 * @throws ValidationException
	 */
	public List<WorkerComplianceDocDtl> getCurrentAndDerivedWithholdings(Worker worker) throws ServiceFailureException, WorkflowException, ValidationException;
	
}