package com.intuit.sb.payroll.capability.taxes.withholding.impl;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.CAWagePlan;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.LawId;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * Creation Factory for the supported {@link ComplianceDocIdEnum} s.  Creates a transient instance with the defaults pre-set.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public class WorkerComplianceDocFactory {

	/**
	 * Creates a {@link WorkerComplianceDocDtl} based on the {@link ComplianceDocIdEnum}.  
	 * 
	 * FIXME [th] introduce pluggable builder pattern to deal with jurisdiction variations.
	 * @param docType
	 * @return
	 * @throws ValidationException 
	 */
	public static WorkerComplianceDocDtl createWorkerComplianceDocDtlFromType(ComplianceDocIdEnum docType) throws ValidationException {
		LawId lawId = new LawId();
		lawId.setValue(getLawIdFromFormType(docType));
		switch (docType) {
			case DE4:
				DE4Detail de4Detail = new DE4Detail();
				de4Detail.setDocId(docType);
				de4Detail.setAdditionalAmount(new Money());
				de4Detail.setDerived(true);
				de4Detail.setTaxExempt(false);
				de4Detail.setNoOfAllowances(0);
				de4Detail.setNoOfDeductions(0);
				de4Detail.setWithholdingStatus(DE4WithholdingStatusEnum.SINGLE);
				de4Detail.setLawId(lawId);
				return de4Detail;

			case W4IRS:
				W4Detail w4Detail = new W4Detail();
				w4Detail.setDocId(docType);
				w4Detail.setAdditionalAmount(new Money());
				w4Detail.setDerived(true);
				w4Detail.setTaxExempt(false);
				w4Detail.setNoOfAllowances(0);
				w4Detail.setWithholdingStatus(W4WithholdingStatusEnum.SINGLE);
				w4Detail.setLawId(lawId);
				return w4Detail;
			
			case DE545:
				CAWagePlan wagePlan = new CAWagePlan();
				wagePlan.setDocId(docType);
				wagePlan.setDerived(true);
				wagePlan.setValue("S");
				return wagePlan;
			default:
				throw new ValidationException("Cannot create ComplianceDoc of type " + docType.toString());
		}
	}
	
	public static String getLawIdFromFormType(ComplianceDocIdEnum docType) throws ValidationException {
		String lawId = null;
		if ((docType == null) || (ComplianceDocIdEnum.NOTSET == docType)) {
			throw new ValidationException("docType must be set");
		}
		switch (docType) {
			case W4IRS:
				lawId = QboeLawIdConstants.sFICALawId; // US-FED-FIT
				break;
			case DE4:
				lawId = QboeLawIdConstants.sCAPITLawId; // US-CA-FIT
				break;
			case DE545:
				lawId = "";
				break;
			default:
				throw new ValidationException("docType must be set");
		}
		return lawId;
	}

}
