package com.intuit.sb.payroll.capability.taxes.withholding.impl;

import com.intuit.sb.payroll.capability.taxes.withholding.impl.validators.DE4WithholdingValidator;
import com.intuit.sb.payroll.capability.taxes.withholding.impl.validators.DE545WagePlanValidator;
import com.intuit.sb.payroll.capability.taxes.withholding.impl.validators.W4WithholdingValidator;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * Different {@link WorkerComplianceDocDtl} types require specific validators.  This Factory provides a validator for each type of doc that
 * we support.
 * 
 * $Author: $ 
 * $Revision: $
 * $DateTime: $
 *
 */
public class WorkerComplianceDocValidatorFactory {

	/**
	 * Returns a validator for the designated DocType.
	 * 
	 * @param complianceDocType
	 * @return the Validator.
	 * @throws UnsupportedOperationException if the DocType is not supported.
	 */
	public static WorkerComplianceDocValidator getValidatorForDocType(ComplianceDocIdEnum complianceDocType) {
		switch (complianceDocType) {
			case W4IRS:
				return new W4WithholdingValidator();
				
			case DE4:
				return new DE4WithholdingValidator();
			
			case DE545:
				return new DE545WagePlanValidator();

			default:
				throw new UnsupportedOperationException("No Validator for Document type:" + complianceDocType.toString());
		}
	}

}
