package com.intuit.sb.payroll.capability.taxes.withholding.impl.validators;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerComplianceDocValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * Validates W4 Docs.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public class W4WithholdingValidator implements WorkerComplianceDocValidator {

	public W4WithholdingValidator() {
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerComplianceDocValidator#validateWorkerComplianceDocDtl(com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl)
	 */
	public void validateWorkerComplianceDocDtl(WorkerComplianceDocDtl workerComplianceDoc) throws ValidationException {
		W4Detail w4Detail = (W4Detail) workerComplianceDoc;
		//If you are exempt, nothing applies.
		if (!w4Detail.isTaxExempt()) {
			PayrollParamValidator.checkIsNotNull(w4Detail.getWithholdingStatus(), "withholdingStatus");
			if (w4Detail.getWithholdingStatus() == W4WithholdingStatusEnum.NOTSET) {
				throw new ValidationException("Withholding Status is not set.");
			}
			PayrollParamValidator.checkIsNonNegative(w4Detail.getNoOfAllowances(), "Allowance");
			Money w4AdditionalAmount = w4Detail.getAdditionalAmount();
			if (w4AdditionalAmount != null) {
				PayrollParamValidator.checkIsNonNegative(w4AdditionalAmount.getAmount(), "Extra Withholding");
			}
		}
	}

}
