package com.intuit.sb.payroll.capability.worker.impl;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.worker.impl.evaluator.WorkerEvaluator;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;

public class WorkerEvaluatorFactory 
{
	private static Logger logger = PayrollLoggerFactory.makeLogger(WorkerEvaluatorFactory.class);	
	private static final Map<Locale, String> evalConfig = new HashMap<Locale, String>(); 
	static{
		//FIXME: Load from Config/LocaleObjectFactory wherever
		evalConfig.put(Locale.US, "com.intuit.sb.payroll.capability.worker.impl.evaluator.USWorkerEvaluator");
	}	
	private static final Map <Locale, WorkerEvaluator> evaluators = new HashMap<Locale, WorkerEvaluator>();
	
	public static WorkerEvaluator getWorkerEvaluator(Locale locale) throws ServiceFailureException
	{
		if(locale == null)
		{
			throw new IllegalArgumentException("Locale Cannot be null");
		}
		WorkerEvaluator evaluator = null;

		synchronized (evaluators) 
		{
			evaluator = evaluators.get(locale);
			if(evaluator == null)
			{
				String className = evalConfig.get(locale);
				if(className == null)
				{
					throw new IllegalStateException("No Evaluators configured for Locale:" + locale.toString());
				}				
				//Go and create our evaluator	
				evaluator = createEvaluator(className);
				if(evaluator == null)
				{
					throw new ServiceFailureException("Failed to find Suitable Evaluator for Locale:" + locale.toString());
				}
				
				//Put the evaluator so that we don't have to do this again
				evaluators.put(locale, evaluator);
			}						
		}
		
		return evaluator;
		
	}

	private static WorkerEvaluator createEvaluator(String className) 
	{
		WorkerEvaluator evaluator = null;
		try {
			Class<?> clazz = Class.forName(className);
			Object obj = clazz.newInstance();
			evaluator = (WorkerEvaluator) obj;
		} catch (Exception e) 
		{
			logger.fatal("Failed to instantiate class" + className, e);
		}
		return evaluator;
	}

}
