package com.intuit.sb.payroll.capability.worker.impl.evaluator;

import java.util.Set;

import com.intuit.sb.payroll.common.AbstractPayrollEntityDataEvaluator;
import com.intuit.sb.payroll.common.PayrollEntityDataState;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Evaluates the Worker's Compensations to assess it's completeness.
 * 
 * $Author: AStec $
 * $Revision: #1 $
 * $DateTime: 2008/05/19 13:00:31 $
 *
 */
public class WorkerCompensationEvaluator extends AbstractPayrollEntityDataEvaluator<Worker>
{
	private static WorkerCompensationEvaluator sInstance = new WorkerCompensationEvaluator();

	private WorkerCompensationEvaluator() {}
	
	public static WorkerCompensationEvaluator getInstance()
	{
		return sInstance ;
	}
	
	/**
	 * Workers are considered complete when there is a liveLocation, workLocation and a hire date.
     * @param worker the Worker for evaluating the work location.
	 */
	public PayrollEntityDataState evaluateState(Worker worker)
	{
		Set<CompensationDtl> compensations = worker.getCompensationDtls();
	    // One compensation is required
		if (compensations != null && compensations.size() > 0)
		{
			return PayrollEntityDataState.Complete;
			
		} else
		{
			return PayrollEntityDataState.Incomplete;
		}
	}
}
