package com.intuit.sb.payroll.capability.worker.impl.evaluator;

import java.util.Set;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.AbstractPayrollEntityDataEvaluator;
import com.intuit.sb.payroll.common.PayrollEntityDataEvaluator;
import com.intuit.sb.payroll.common.PayrollEntityDataState;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;


public class WorkerFederalWithholdingsEvaluator extends AbstractPayrollEntityDataEvaluator<Worker>
{
    private static WorkerFederalWithholdingsEvaluator sInstance = new WorkerFederalWithholdingsEvaluator();
    
    private WorkerFederalWithholdingsEvaluator(){}
    
    public static PayrollEntityDataEvaluator<Worker> getInstance() {
        return sInstance ;
    }

    /**
     * Checks that a worker has a complete W4 and any other
     * compliance docs on the worker are complete.
     * 
     * @param the worker.
     * @return false if the W4 is missing or any of the worker's compliance documents are incomplete.
     */
    @Override
    public PayrollEntityDataState evaluateState(Worker entity) throws ValidationException
    {
        PayrollParamValidator.checkIsNotNull(entity, "Worker entity");
        Set<WorkerComplianceDocDtl> compDataColl = entity.getWorkerComplianceDocDtls();
        if (compDataColl.size() == 0)
        {
            return PayrollEntityDataState.Incomplete; 
        }
        
        for (WorkerComplianceDocDtl employeeWithholdingData : compDataColl) {
    		if (employeeWithholdingData instanceof W4Detail) {
    			W4Detail w4Detail = (W4Detail) employeeWithholdingData;
    			if ((null != w4Detail.getNoOfAllowances()) && (w4Detail.getNoOfAllowances() >= 0)) {
    				return PayrollEntityDataState.Complete;
    			}
    		}
        }
        return PayrollEntityDataState.Incomplete;
    }
    
 
}
