package com.intuit.sb.payroll.capability.worker.impl.evaluator;

import java.util.Date;

import com.intuit.sb.payroll.common.AbstractPayrollEntityDataEvaluator;
import com.intuit.sb.payroll.common.PayrollEntityDataEvaluator;
import com.intuit.sb.payroll.common.PayrollEntityDataState;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Evaluates the Hire Date to assess it's completeness.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
public class WorkerHireDateEvaluator extends AbstractPayrollEntityDataEvaluator<Worker> {
	private static WorkerHireDateEvaluator sInstance = new WorkerHireDateEvaluator();

	private WorkerHireDateEvaluator() {
	}

	public static PayrollEntityDataEvaluator<Worker> getInstance() {
		return sInstance;
	}

	/**
	 * Hire dates are considered complete if they are not null.
	 * 
	 * @param worker the Worker for evaluating the hire date.
	 */
	public PayrollEntityDataState evaluateState(Worker worker) {
		DateRange employmentPeriod = worker.getEmploymentPeriod();
		if (null != employmentPeriod) {
			Date hireDate = employmentPeriod.getStartDate();
			if (hireDate != null) {
				return PayrollEntityDataState.Complete;
			}
		}

		return PayrollEntityDataState.Incomplete;
	}
}
