package com.intuit.sb.payroll.capability.worker.impl.evaluator;

import com.intuit.sb.payroll.common.AbstractPayrollEntityDataEvaluator;
import com.intuit.sb.payroll.common.PayrollEntityDataState;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Evaluates the LiveLocation to assess it's completeness.
 * 
 * $Author: AStec $
 * $Revision: #1 $
 * $DateTime: 2008/05/19 13:00:31 $
 *
 */
public class WorkerLiveLocationEvaluator extends AbstractPayrollEntityDataEvaluator<Worker>
{
	private static WorkerLiveLocationEvaluator sInstance = new WorkerLiveLocationEvaluator();

	private WorkerLiveLocationEvaluator() {}
	
	public static WorkerLiveLocationEvaluator getInstance()
	{
		return sInstance ;
	}
	
	/**
	 * Workers are considered complete when there is a liveLocation, workLocation and a hire date.
     * @param worker the Worker for evaluating the work location.
	 */
	public PayrollEntityDataState evaluateState(Worker worker)
	{
	    GlobalPostalAddress liveLocation = worker.getTaxResidenceAddress();
	    // The state is required for the US
		if (liveLocation != null && liveLocation.getCountrySubDivisionCode() != null &&
			liveLocation.getCountrySubDivisionCode().length() > 0)
		{
			return PayrollEntityDataState.Complete;
			
		} else
		{
			return PayrollEntityDataState.Incomplete;
		}
	}
}
