package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;

import com.intuit.dataTypes.EnumTypeIf;
import com.intuit.dataTypes.EnumTypeParserIf;
import com.intuit.dataTypes.Money;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PayrollDatumIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerComplianceDocFactory;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollItemHandler;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.LawId;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

public class USFederalWithholdingHandler implements PayrollItemHandler{
	
	private static List<Locale> localeList = new ArrayList<Locale>();

	static {
		localeList.add(Locale.US);
	}
	
	public String getLawId() {
		// TODO Auto-generated method stub
		return null;
	}

	public List<Locale> getLocaleList() {
		return localeList;
	}
	
	public void createQboPayrollItemsFromPayrollItemDtls(PayrollEngineContext context, EmployeeIf employee,
			Worker worker) {
		// TODO Auto-generated method stub
		
	}


	public void handle(PaycheckCalculationContext context) {
		Worker worker = context.getWorker();
		Collection<WorkerComplianceDocDtl> withholdingDocsForCalc = worker.getWithholdingDocsForCalc();
		for (WorkerComplianceDocDtl workerComplianceDocDtl : withholdingDocsForCalc) {
			//FIXME we need to honour this === EffectivePeriod effectivePeriod  = workerComplianceDocDtl.getEffectivePeriod();
			ComplianceDocIdEnum docId = workerComplianceDocDtl.getDocId();
			if (docId == ComplianceDocIdEnum.W4IRS) {
				W4Detail w4Detail = (W4Detail)workerComplianceDocDtl;
				createQboWithholding(w4Detail, context);
			}
		}
	}
	
    private void createQboWithholding(W4Detail employeeWithholdingData, PaycheckCalculationContext context) {
    	
    	String lawId = null;
        
    	LawId lawIdContainer = employeeWithholdingData.getLawId();
    	if (lawIdContainer != null) {
    		lawId = lawIdContainer.getValue();
    	}
		
		//Hack for tests that are not using the create API in WH Component.
		if (null == lawId) {
			try {
				lawId = WorkerComplianceDocFactory.getLawIdFromFormType(employeeWithholdingData.getDocId());
			} catch (ValidationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		EmployeePayrollItemIf qboeEmployeePayrollItem = null;
		
		Collection<EmployeePayrollItemIf> employeePayrollItems = context.getQboeEmployee().getEmployeePayrollItems();
		for (EmployeePayrollItemIf employeePayrollItemIf : employeePayrollItems) {
			String ePILawId = new Integer(employeePayrollItemIf.getPayrollItem().getDetailType().toInt()).toString();
			if (lawId.equals(ePILawId)) {
				//It's already on the employee
				qboeEmployeePayrollItem = employeePayrollItemIf;
				break;
			}
		}
		
		//If it was not found, create it.
		if (qboeEmployeePayrollItem == null) {
			PayrollItemIf qboePayrollItem = context.getPayrollItemForLaw(lawId);
			
			qboeEmployeePayrollItem = context.getQboeEmployee().createEmployeePayrollItem(
					qboePayrollItem);
		}
		
		qboeEmployeePayrollItem.setDatumValue(DatumID.ALLOWANCES, employeeWithholdingData.getNoOfAllowances());
        
        com.intuit.sb.payroll.entity.model.Money additionalAmount = employeeWithholdingData.getAdditionalAmount();
        if (additionalAmount != null)
        {
            Money value = new Money(additionalAmount.getAmount().toString());
            qboeEmployeePayrollItem.setDatumValue(DatumID.EXTRA_WITHHOLDING, value);
        }

        int filingStatus;
        if (employeeWithholdingData.isTaxExempt()) {
        	filingStatus = 2048; // US_DON_T_WITHHOLD
        }  else {
        	switch (employeeWithholdingData.getWithholdingStatus()) {
        	case MARRIED:
        		filingStatus = 256; // US_MARRIED
        		break;
        	case MARRIEDSINGLERATE:
        		filingStatus = 64; // US_MARRIED_USING_SINGLE
        		break;
        	case SINGLE:
        	default:
        		filingStatus = 512; // US_SINGLE
        		break;
        	}
        }
        
		PayrollDatumIf qboeFilingStatus = qboeEmployeePayrollItem.getDatum(DatumID.FILING_STATUS);
		EnumTypeParserIf<?> parser = qboeFilingStatus.getEnumParser();
		EnumTypeIf value = parser.parseInt(filingStatus);
		qboeFilingStatus.setValue(value);

    }

}
