package com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.handlers;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import org.apache.commons.lang.math.RandomUtils;

import com.intuit.dataTypes.Money;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.payroll.types.enumtype.CalcBasedOn;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PaycheckCalculationContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DeductionDtl;
import com.intuit.sb.payroll.entity.model.DeductionTypeEnum;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.Worker;

public class USTipsCashCompensationHandler extends AbstractFlatRateCompensationHandler {

	private static List<Locale> localeList = new ArrayList<Locale>();

	static {
		localeList.add(Locale.US);
	}

	public String getLawId() {
		return QboeLawIdConstants.sCashTipsLawId;
	}

	public List<Locale> getLocaleList() {
		return localeList;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollItemHandler#createQboPayrollItemsFromPayrollItemDtls(com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollEngineContext, com.intuit.payroll.j.EmployeeIf, com.intuit.sb.payroll.entity.model.Worker)
	 */
	public void createQboPayrollItemsFromPayrollItemDtls(PayrollEngineContext context, EmployeeIf employee,	Worker worker) {
		Set<CompensationDtl> workerCompensations = worker.getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.TIPSCASH) {
				createImpl(context, worker, employee, compensationDtl);
				//Find the related offset
				final DeductionDtl deductionDtl = (DeductionDtl) compensationDtl.getLinkedPayrollItemDtl();
				if (deductionDtl != null && deductionDtl.getDeductionType() == DeductionTypeEnum.CASHTIPSOFFSET) {
					createImpl(context, worker, employee, deductionDtl);
				} else {
					throw new IllegalStateException("Cash Tips must have a linked Deduction of type CashTipsOffset");
				}
			}
		}
	}

	public void handle(PaycheckCalculationContext context) {
		Set<CompensationDtl> workerCompensations = context.getWorker().getCompensationDtls();
		for (final CompensationDtl compensationDtl : workerCompensations) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.TIPSCASH) {
				final EmployeePayrollItemIf qboeEmployeePayrollItem = context.createEmployeePayrollItem(context.getQboeEmployee(), compensationDtl);
				final PaycheckLineItem paycheckLineItem = findPaycheckLineItemForCompensation(context, compensationDtl);
				context.getPaycheck().addPaycheckLineItem(paycheckLineItem);

				qboeEmployeePayrollItem.getPayrollItem().setDatumValue(DatumID.CALC_BASED_ON, CalcBasedOn.FLAT_AMOUNT);
				final Datum lineItemRate = paycheckLineItem.getRate();
				Money fixedCommisionMoneyRate = null;
				if (compensationDtl.isFixedRate() && lineItemRate == null) {
					//Fixed with no override.
					paycheckLineItem.setRate(compensationDtl.getRate().clone());
					final BigDecimal fixedCommissionRate = compensationDtl.getRate().getValue1AsMoneyAmount();
					fixedCommisionMoneyRate = new Money(fixedCommissionRate.toString());
				} else {
					if (lineItemRate != null && lineItemRate.getValue1AsMoneyAmount() != null) {
						fixedCommisionMoneyRate = new Money(lineItemRate.getValue1AsMoneyAmount().toString());
					} else {
						fixedCommisionMoneyRate = new Money(0);
					}
				}
				qboeEmployeePayrollItem.setDatumValue(DatumID.RATE, fixedCommisionMoneyRate);
				final PaycheckLineIf  qboePaycheckLine = context.getQboePaycheck().createLine(qboeEmployeePayrollItem);

				// YTD
				createYTDTotals(context, context.getWorker(), compensationDtl, qboeEmployeePayrollItem);
				
				context.getPaycheckLineItemMap().put(qboePaycheckLine, paycheckLineItem);
				
				//Find the related offset
				final DeductionDtl deductionDtl = (DeductionDtl) compensationDtl.getLinkedPayrollItemDtl();
				if (deductionDtl != null && deductionDtl.getDeductionType() == DeductionTypeEnum.CASHTIPSOFFSET) {
					//************** This particular pay item has an evil twin.
					PayrollItemDetailType tipsCashOutPayItem = qboeEmployeePayrollItem.getPayrollItem().getDetailType().getTwin();
					final PayrollItemIf qboeTipsOutPayrollItem = context.getQboCompany().createPayrollItem(tipsCashOutPayItem);
					qboeTipsOutPayrollItem.setDatumValue(DatumID.NAME, "Tips Cash Offset");
					int pItemUniqueId = RandomUtils.nextInt();
					context.getPayrollItemInfo().setPayrollItem(pItemUniqueId, qboeTipsOutPayrollItem);
					qboeTipsOutPayrollItem.finishInit(pItemUniqueId, false);
					EmployeePayrollItemIf qboeEmployeeTwinPayrollItem = context.getQboeEmployee().createEmployeePayrollItem(
							qboeTipsOutPayrollItem);
					final PaycheckLineItem tipsOutpaycheckLineItem = findPaycheckLineItemForCompensation(context, deductionDtl);
					context.getPaycheck().addPaycheckLineItem(tipsOutpaycheckLineItem);

					final PaycheckLineIf  qboeTipsOutPaycheckLine = context.getQboePaycheck().createLine(qboeEmployeeTwinPayrollItem);

					// YTD
					createYTDTotals(context, context.getWorker(), deductionDtl, qboeEmployeeTwinPayrollItem);
					
					context.getPaycheckLineItemMap().put(qboeTipsOutPaycheckLine, tipsOutpaycheckLineItem);
					
					//******************** end evil twin handling
				} else {
					throw new IllegalStateException("Cash Tips must have a linked Deduction of type CashTipsOffset");
				}
			}
		}
	}
}
