package com.intuit.sb.payroll.message;

import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;

/**
 * The PayrollMessageService provides decoupling between components in the Payroll Domain Services. 
 * It achieves this by acting as a proxy for a real Message Bus and MDBs in each component.  
 * 
 * Events are published to this Service (as they would to a queue) through methods that mimic queues.
 * 
 * The methods will dispatch the events to "simulated" subscribers (no subscription is explicitly done).
 * 
 * Eventually this could actually do the right thing and put messages into Queues.
 * 
 * $Author: $ 
 * $Revision: $
 * $DateTime: $
 *
 */
@Local
public interface PayrollMessageService {
	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "PayrollMessageService/local";

	public void dispatchEmployerEvent(WorkflowEventType workflowEventType, Employer employer) throws ServiceFailureException;

	public void dispatchWorkerEvent(WorkflowEventType workflowEventType, Worker worker) throws ServiceFailureException;
	
	public void dispatchPayrollScheduleEvent(WorkflowEventType workflowEventType, PayrollSchedule payrollSchedule) throws ServiceFailureException, ValidationException;
	
	public void dispatchPayrollRunEvent(WorkflowEventType workflowEventType, PayrollRun payrollRun) throws ServiceFailureException, ValidationException;

	public void dispatchAssistedSetupEvent(WorkflowEventType workflowEventType, PTSEmployer employer) throws ServiceFailureException;
	
	public void dispatchEmployerBankAccountEvent(WorkflowEventType workflowEventType, List<BankAccount> bankAccounts) throws ServiceFailureException, ValidationException ;

	public void dispatchPreAssistedActivationEvent(WorkflowEventType workflowEventType, PTSEmployer ptsEmployer)
			throws ServiceFailureException;
	
}
