package com.intuit.sb.payroll.message.impl;

import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.workerpayments.PayrollPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.message.PayrollMessageService;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;

@Stateless(name = "PayrollMessageService")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class PayrollMessageServiceBean implements PayrollMessageService {

	//FIXME - no DI here because we have a circular dependency.  
	//@EJB
	WorkerComponent workerComponent = ServiceLocator.getInstance().getService(WorkerComponent.JNDI_NAME);
	
	//@EJB
	EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
	
	//@EJB
	WorkerPaymentsComponent workerPaymentsComponent = ServiceLocator.getInstance().getService(WorkerPaymentsComponent.JNDI_NAME);
	
	//@EJB
	EmployerPTSWorkflowComponent ptsWorkflowComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
	
	WorkerPTSWorkflowComponent workerPTSComponent = ServiceLocator.getInstance().getService(WorkerPTSWorkflowComponent.JNDI_NAME);
	
	PayrollPTSWorkflowComponent payrollPTSWorkflowComponent = ServiceLocator.getInstance().getService(PayrollPTSWorkflowComponent.JNDI_NAME);
	
	public void dispatchEmployerEvent(WorkflowEventType workflowEventType, Employer employer) throws ServiceFailureException {
		workerComponent.handleEmployerEvent(workflowEventType, employer);
		ptsWorkflowComponent.handleEmployerEvent(workflowEventType, employer);
	}
	
	public void dispatchWorkerEvent(WorkflowEventType workflowEventType, Worker worker) throws ServiceFailureException {
		workerPaymentsComponent.handleWorkerEvent(workflowEventType, worker);
		workerPTSComponent.handleWorkerEvent(workflowEventType, worker);
	}

	public void dispatchPayrollScheduleEvent(WorkflowEventType workflowEventType, PayrollSchedule payrollSchedule) throws ServiceFailureException, ValidationException {
		employerComponent.handlePayrollScheduleEvent(workflowEventType);
		workerPaymentsComponent.handlePayrollScheduleEvent(workflowEventType, payrollSchedule);
		workerComponent.handlePayrollScheduleEvent(workflowEventType);
	}
	
	public void dispatchPayrollRunEvent(WorkflowEventType workflowEventType, PayrollRun payrollRun) throws ServiceFailureException, ValidationException {
		payrollPTSWorkflowComponent.handlePayrollRunEvent(workflowEventType, payrollRun);
		workerComponent.handlePayrollRunEvent(workflowEventType);
	}

	public void dispatchAssistedSetupEvent(WorkflowEventType workflowEventType, PTSEmployer ptsEmployer) throws ServiceFailureException {
		ptsWorkflowComponent.handleAssistedSetupEvent(workflowEventType, ptsEmployer);
		workerPTSComponent.handleAssistedSetupEvent(workflowEventType, ptsEmployer);
		payrollPTSWorkflowComponent.handleAssistedSetupEvent(workflowEventType, ptsEmployer);
	}
	
	public void dispatchPreAssistedActivationEvent(WorkflowEventType workflowEventType, PTSEmployer ptsEmployer) throws ServiceFailureException {
		payrollPTSWorkflowComponent.handleAssistedSetupEvent(workflowEventType, ptsEmployer);
	}
	
	public void dispatchEmployerBankAccountEvent(WorkflowEventType workflowEventType, List<BankAccount> bankAccounts) throws ServiceFailureException, ValidationException {
		ptsWorkflowComponent.handleEmployerBankAccountEvent(workflowEventType, bankAccounts);
	}
}