package com.intuit.sb.payroll.security;

import java.security.Principal;
import java.util.Properties;
import java.util.Set;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.Subject;
import javax.security.jacc.PolicyContext;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.spc.bhp.commons.ContextFactory;
import com.intuit.spc.bhp.service.authentication.BHPPrincipal;
import com.intuit.spc.bhp.serviceapi.authentication.IBHPPrincipal;
import com.intuit.spc.bhp.serviceapi.authorization.IUserManager;
import com.intuit.spc.bhp.serviceapi.bridge.profile.ICompany;
import com.intuit.spc.bhp.serviceapi.bridge.profile.IProfile;
import com.intuit.spc.bhp.serviceapi.bridge.profile.IProfileService;
import com.intuit.spc.bhp.serviceapi.entity.IIdentity;

/**
 * Utility class for security related actions
 *
 */
public class SecurityHelper {

    private static Logger logger = PayrollLoggerFactory.makeLogger(SecurityHelper.class);
    protected static final String SecuritySubjectContextKey = "javax.security.auth.Subject.container";

    // FIXME TODO Move to Service locator and make it secure
    //THIS IS VERY VERY TEMPORARY AND WILL GO AWAY ....
    /**
     *  Authenticated EJB lookup
     */
    @SuppressWarnings("unchecked")
    public static <T> T getServiceUsingAuthentication(String serviceName) throws ServiceFailureException {

        Properties env = new Properties();
        env.setProperty(Context.SECURITY_PRINCIPAL, "ptstestuserone");
        env.setProperty(Context.SECURITY_CREDENTIALS, "intuitptstest");
        env.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.security.jndi.JndiLoginInitialContextFactory");

        try {
            Context jndiContext = new InitialContext(env);
            Object o = jndiContext.lookup("PayrollService/" + serviceName);
            T service = (T) o;
            return service;
        } catch (NamingException e) {
            logger.fatal(e);
            throw new ServiceFailureException("Unable to retrieve service " + serviceName +
                    " from JNDI Service...");
        }
    }
    
    /**
     * @return The Realm ID (External) for the current BHPPrincipal
     * @throws ServiceFailureException if there is an exception when retreiving the Realm Id from <br>
     * the current principal
     */
    public static String getCurrentRealmId() throws ServiceFailureException {
		IBHPPrincipal bhpPrincipal = null;
		String err = "Unable to get Principal for current context";
		String realmId = "";
        try {
			bhpPrincipal = (IBHPPrincipal) BHPPrincipal.getCurrentPrincipal();
			if (bhpPrincipal != null) {
				realmId = bhpPrincipal.getIdentity().getCurrentRealmId();
			}
		} catch (Exception e) {
			throw new ServiceFailureException(err, e);
		}
		
		return realmId;
    }
    
    public static String getCurrentAuthId() throws ServiceFailureException{
    	IBHPPrincipal bhpPrincipal = null;
		String err = "Unable to get AuthId for current context";
		String authId = "";
        try {
			bhpPrincipal = (IBHPPrincipal) BHPPrincipal.getCurrentPrincipal();
			if (bhpPrincipal != null) {
				authId = bhpPrincipal.getIdentity().getAuthId();
			}
		} catch (Exception e) {
			throw new ServiceFailureException(err, e);
		}
		return authId;
    }
    
    public static void registerEmployerWithGeminiAdmin() {
		if (!ServiceLocator.getInstance().isEmbedded()){ // TODO once we get rid of incontainer testing without security
			IUserManager userManager = (IUserManager)ContextFactory.getBean("UserManager");
			try {
				IBHPPrincipal bhpPrincipal = (IBHPPrincipal) BHPPrincipal.getCurrentPrincipal();
				if (bhpPrincipal != null){
					IIdentity identity = bhpPrincipal.getIdentity();
					logger.debug(" Registering RealmId with Admin "+ identity.getCurrentRealmId() );
					userManager.assignRealmAdminRoleToUser(identity.getCurrentRealmId(),identity.getAuthId());
				}else{
					throw new ServiceFailureException("Not able to get BHPPrincipal");
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
    
	
	
	public static  String getCompanyNameFromIAM() throws Exception {
		String companyId = getCurrentRealmIdFromFromPolicyContext();		
		IProfileService profileService = (IProfileService)ContextFactory.getBean("ProfileServiceManager");
		ICompany company = profileService.getCompanyById(companyId);
		return company.getCompanyName();
	}
	
	public static String getUserNameromFromIAM() throws Exception {
		IProfileService profileService = (IProfileService)ContextFactory.getBean("ProfileServiceManager");
		IProfile userProfile = profileService.getUserProfile(getCurrentAuthId());
		return userProfile.getAddress().getName().getFirstName();		
	}

	private static  String getCurrentRealmIdFromFromPolicyContext() throws Exception {
		IBHPPrincipal principal = getPrincipalFromPolicyContext();
		if (principal != null) {
			IIdentity identity = principal.getIdentity();
			return identity.getCurrentRealmId();        
		}
    	return null;
    }

	private static  IBHPPrincipal getPrincipalFromPolicyContext() throws Exception {
        Subject subject = (Subject)PolicyContext.getContext(SecuritySubjectContextKey);
        if (subject != null) {
            Set<Principal> principalSet = subject.getPrincipals();
            if (principalSet != null) {
            	for (Principal principal : principalSet) {
                    if (principal instanceof IBHPPrincipal) {
                        return (IBHPPrincipal)principal;
                    }
                }
            }
        }
        
        return null;
    }

}
