package com.intuit.sb.payroll.calcengine;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Currency;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.lang.SystemUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;

import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.workerpayments.CalculationInputs;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.HealthBenefit;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PayrollItemDtlPtdDatum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;


/**
 *  <P>The <CODE>PaycheckCalcTestBase</CODE> class contains helper methods for paycheck calculation</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class PaycheckCalcTestBase extends BasicTestBase {
	private static Logger logger = LogManager.getLogger(PaycheckCalcTestBase.class);
    private static final String newline = SystemUtils.LINE_SEPARATOR;
    private static final String tab = "    ";


    protected static Calendar sCompanyStartDate = null;
    protected static Calendar sPeriodStartDate = null;
    protected static Calendar sDailyPeriodEndDate = null;
    protected static Calendar sWeeklyPeriodEndDate = null;
    protected static Calendar sBiweeklyPeriodEndDate = null;
    protected static Calendar sSemimonthlyPeriodEndDate = null;
    protected static Calendar sMonthlyPeriodEndDate = null;
    protected static Calendar sQuarterlyPeriodEndDate = null;
    protected static Calendar sAnnuallyPeriodEndDate = null;
    
    protected static Currency sCurrency;
    protected static BigDecimal sZeroAmount;
    protected static Money sZeroMoney;
    protected static final boolean NOTEXEMPT = false;
    protected static final boolean EXEMPT = true;

    protected Calendar mPaycheckDate = null;
    protected Calendar mPeriodEndDate = null;
    protected Employer mEmployer = null;
    //protected PaycheckCalculator mCalcEngine = null;
    protected String mCALocation = "CA";
    protected String mTXLocation = "TX";
    // USFUI
    protected TaxDtl mUSFUI = null;
    // USFIT
    protected TaxDtl mUSFIT = null;
    // USSSTE employee paid
    protected TaxDtl mUSSSTE = null;
    // USSSTER employer paid
    protected TaxDtl mUSSSTER = null;
    // USMEDTE Employee paid
    protected TaxDtl mUSMEDTE = null;
    // USMEDTER Employer paid
    protected TaxDtl mUSMEDTER = null;
    // USAEIC
    protected TaxDtl mUSAEIC = null;
    // CAETT
    protected TaxDtl mCAETT = null;
    // CASUI
    protected TaxDtl mCASUI = null;
    // CAPIT
    protected TaxDtl mCAPIT = null;
    // CASDI
    protected TaxDtl mCASDI = null;
    // TXSUI
    protected TaxDtl mTXSUI = null;
    
    protected static final int sTaxYear = 2008;
    
    @BeforeClass
    public static void setUpClass() {
        setUpDates();
        
        sCurrency = Currency.getInstance("USD");
        sZeroAmount = new BigDecimal("0.00");
        sZeroMoney = new Money();
        sZeroMoney.setAmount(sZeroAmount);
        sZeroMoney.setCurrencyCode(sCurrency);
    }
    
    @BeforeMethod
    public void setUp() throws Exception {
    	super.setUp();
    	setupTaxes();
    }
    
    protected static void setUpDates() {
        sCompanyStartDate = DateUtil.createCalendar(sTaxYear, Calendar.JANUARY, 1);
        sPeriodStartDate = (Calendar)sCompanyStartDate.clone();
        sDailyPeriodEndDate = (Calendar)sPeriodStartDate.clone();
        sWeeklyPeriodEndDate = (Calendar)sPeriodStartDate.clone();
        sWeeklyPeriodEndDate.add(Calendar.DAY_OF_MONTH,6);
        sBiweeklyPeriodEndDate = (Calendar)sPeriodStartDate.clone();
        sBiweeklyPeriodEndDate.add(Calendar.DAY_OF_MONTH,13);
        sSemimonthlyPeriodEndDate = (Calendar)sPeriodStartDate.clone();
        sSemimonthlyPeriodEndDate.add(Calendar.DAY_OF_MONTH,14);
        sMonthlyPeriodEndDate = (Calendar)sPeriodStartDate.clone();
        sMonthlyPeriodEndDate.add(Calendar.MONTH, 1);
        sMonthlyPeriodEndDate.add(Calendar.DAY_OF_MONTH, -1);
        sQuarterlyPeriodEndDate = (Calendar)sPeriodStartDate.clone();
        sQuarterlyPeriodEndDate.add(Calendar.MONTH, 3);
        sQuarterlyPeriodEndDate.add(Calendar.DAY_OF_MONTH, -1);
        sAnnuallyPeriodEndDate = (Calendar)sPeriodStartDate.clone();
        sAnnuallyPeriodEndDate.add(Calendar.YEAR, 1);
        sAnnuallyPeriodEndDate.add(Calendar.DAY_OF_MONTH, -1);
    }
    
    protected void setupTaxes() {
        // USFUI
        mUSFUI = createTaxItem(PayrollItemType.USFUI.getLawId());
        // USFIT
        mUSFIT = createTaxItem(PayrollItemType.USFIT.getLawId());
        // USSSTE employee paid
        mUSSSTE = createTaxItem(PayrollItemType.USSSTE.getLawId());
        // USSSTER employer paid
        mUSSSTER = createTaxItem(PayrollItemType.USSSTER.getLawId());
        // USMEDTE Employee paid
        mUSMEDTE = createTaxItem(PayrollItemType.USMEDTE.getLawId());
        // USMEDTER Employer paid
        mUSMEDTER = createTaxItem(PayrollItemType.USMEDTER.getLawId());
        // USAEIC
        mUSAEIC = createTaxItem(PayrollItemType.USAEIC.getLawId());
        // CAETT
        mCAETT = createTaxItem(PayrollItemType.CAETT.getLawId());
        // CASUI
        mCASUI = createTaxItem(PayrollItemType.CASUI.getLawId());
        // CAPIT
        mCAPIT = createTaxItem(PayrollItemType.CAPIT.getLawId());
        // CASDI
        mCASDI = createTaxItem(PayrollItemType.CASDI.getLawId());
        // TXSUI
        mTXSUI = createTaxItem(PayrollItemType.TXSUI.getLawId());
    }
    
    protected CalculationInputs createCalculationInputsFromCompensationDtl(CompensationDtl compensationDtl) {
        CalculationInputs calcInput = new CalculationInputs();
        calcInput.setQuantity(compensationDtl.getQuantity());
        calcInput.setRate(compensationDtl.getRate());
        calcInput.setSourcePayrollItemDtlId(compensationDtl.getId());
        return calcInput;
    }
    
	protected HealthBenefitDtl createHealthBenefitDtl(HealthBenefitTypeEnum benefitType, BigDecimal rate,
			DatumType2Enum datumType) {
		HealthBenefitDtl hb = new HealthBenefitDtl();
		hb.setBenefitType(benefitType);
		HealthBenefit healthBenefit = healthBenefitEAO.findHealthBenefit(benefitType);
		hb.setPayrollItem(healthBenefit);
		Datum rateDatum = new Datum();
        rateDatum.setValue1AsUSDMoneyAmount(rate);
        rateDatum.setType2(datumType);
        hb.setRate(rateDatum);
		return hb;
	}
	
	public static CompensationDtl createCommonCompensationDtl(CompensationTypeEnum compensationType) {
        CompensationDtl compensationDtl = new CompensationDtl();
        compensationDtl.setCompensationType(compensationType);
        Compensation compensation = compensationEAO.findCompensation(compensationType);
        compensationDtl.setPayrollItem(compensation);
        compensationDtl.setName(compensation.getName());
        compensationDtl.setDescription(compensation.getDescription());
        return compensationDtl;
	}
	
    public static CompensationDtl createRateBasedWageItemDtl(CompensationTypeEnum compensationType, 
    		BigDecimal rate, Double hours) {
        CompensationDtl compensationDtl = createCommonCompensationDtl(compensationType);
        Datum rateDatum = new Datum();
        rateDatum.setValue1AsUSDMoneyAmount(rate);
        rateDatum.setType2(DatumType2Enum.HOUR);
        rateDatum.setValue2(new Double(1));
        compensationDtl.setRate(rateDatum);
        Datum quantityDatum = new Datum();
        quantityDatum.setValue1AsQuantity(hours);
        quantityDatum.setType2(DatumType2Enum.NOTAPPLICABLE);
        compensationDtl.setQuantity(quantityDatum);
        return compensationDtl;
    }
    
    protected CompensationDtl createOvertimeWageItemDtl(CompensationTypeEnum compensationType,
			CompensationDtl linkedCompensationDtl, double rate) {
        CompensationDtl compensationDtl = createCommonCompensationDtl(compensationType);
        compensationDtl.setLinkedPayrollItemDtl(linkedCompensationDtl);
        Datum rateDatum = new Datum();
        rateDatum.setValue1AsQuantity(rate);
        rateDatum.setType2(DatumType2Enum.PERUNIT);
        compensationDtl.setRate(rateDatum);
        return compensationDtl;
	}
    
    protected CompensationDtl createAmountBasedWageItemDtl(CompensationTypeEnum compensationType, 
    		BigDecimal rate, DatumType2Enum type2) {
        CompensationDtl compensationDtl = createCommonCompensationDtl(compensationType);
        Datum datum = new Datum();
        datum.setValue1AsUSDMoneyAmount(rate);
        datum.setType2(type2);
        datum.setValue2(1.0);
        compensationDtl.setRate(datum);
        return compensationDtl;
    }
    
    /**
     * Utility method for creating a Piecework compensationDtl using the specified values
     * 
     * @param rate The per unit rate (e.g. $2.00 per 5 units)
     * @param units The number of units needed for the per unit rate (e.g. $2.00 per 5 units)
     * @param pieces The total number of units (e.g. 100 units @ $2.00 per 5 units)
     * @return
     */
    public static CompensationDtl createPieceworkWageItemDtl(BigDecimal rate, Double units, Double pieces) {
        CompensationDtl compensationDtl = createCommonCompensationDtl(CompensationTypeEnum.PIECEWORK);
        Datum datum = new Datum();
        datum.setValue1AsUSDMoneyAmount(rate);
        datum.setType2(DatumType2Enum.PERUNIT);
        datum.setValue2(units);
        compensationDtl.setRate(datum);
        Datum quantityDatum = new Datum();
        quantityDatum.setValue1AsQuantity(pieces);
        quantityDatum.setType2(DatumType2Enum.NOTAPPLICABLE);
        compensationDtl.setQuantity(quantityDatum);
        return compensationDtl;
    }
    
	protected PaycheckLineItem createLineItemFromCompensation(CompensationDtl compensationDtl, DateRange payPeriod) {
		PaycheckLineItem lineItem = new PaycheckLineItem();
        lineItem.setLawId(compensationDtl.getPayrollItem().getLawId());
        lineItem.setName(compensationDtl.getName());
        lineItem.setPayPeriod(payPeriod);
		PayrollItemDtl linkedPayrollItemDtl = compensationDtl.getLinkedPayrollItemDtl();
		if (linkedPayrollItemDtl != null) {
			CompensationDtl linkedCompensationDtl = (CompensationDtl)linkedPayrollItemDtl;
			Datum rate = new Datum();
			Datum linkedPIRate = linkedCompensationDtl.getRate();
			BigDecimal moneyValue = linkedPIRate.getValue1AsUSDMoneyAmount().multiply(
					new BigDecimal(compensationDtl.getRate().getValue1AsQuantity()));
			rate.setValue1AsUSDMoneyAmount(moneyValue);
		    rate.setType2Value2(linkedPIRate.getType2(), linkedPIRate.getValue2());
		    lineItem.setRate(rate);
		} else {
		    lineItem.setRate(compensationDtl.getRate());
		    lineItem.setQuantity(compensationDtl.getQuantity());
		}
        
		// Copy all the PtdDatums from the compensation detail to the paycheck line
		for(PayrollItemDtlPtdDatum ptdDatum : compensationDtl.getPtdDatums()) {
			lineItem.getPtdDatumOfType(ptdDatum.getDatum().getPtdType()).setDatum(ptdDatum.getDatum());
		}
		
		lineItem.setSourcePayrollItemDtl(compensationDtl);
		return lineItem;
	}
	
    /**
     * This utility method will initialize the specified compensationDtl using the specified values
     * 
     * @param compensationDtl The compensationDtl to set the values
     * @param compensationRate The Datum to set the rate values
     * @param compensationQuantity The Datum to set the quantity values
	 * @param hourlyRate The hourly rate
	 * @param hours The number of regular hours worked
	 * @param overtimeHours The number of overtime hours worked
	 * @param bonusAmount The bonus amount
     */
	public static void setCompensationRateAndQuantity(CompensationDtl compensationDtl, Datum compensationRate,
					Datum compensationQuantity, BigDecimal hourlyRate, Double hours, Double overtimeHours,
					BigDecimal bonusAmount) {
		if (CompensationTypeEnum.HOURLYREG.equals(compensationDtl.getCompensationType())) {
			compensationRate.setValue1AsUSDMoneyAmount(hourlyRate);
			// rate is in $ / 1 hour
			compensationRate.setType2(DatumType2Enum.HOUR);
			compensationRate.setValue2(new Double(1));
			compensationQuantity.setValue1AsQuantity(hours);	
		} else if (CompensationTypeEnum.HOURLYOVT.equals(compensationDtl.getCompensationType()) ||
						CompensationTypeEnum.HOURLYDBL.equals(compensationDtl.getCompensationType())) {
			// caller will set linked compensation item
			// quantity is simply multiplier (e.g. 1.5 or 2)
			compensationRate.setValue1AsQuantity(compensationDtl.getRate().getValue1AsQuantity());
			compensationQuantity.setValue1AsQuantity(overtimeHours);	
		} else {
		// This probably doesn't cover all the cases...
			compensationRate.setValue1AsUSDMoneyAmount(bonusAmount);			
			compensationRate.setType2(DatumType2Enum.NOTAPPLICABLE);
		}
	}
    /**
     * FIXME: This method is not called by anyone in the workspace. But we need this method when
     * we have the infrastructure for retirement benefits. We also need to fix this method as per Alex's email
     * regarding deprecating the setters and getters for LawID in PayrollItemDtl.
     * @param lawId
     * @return
     */
                                              
/*    protected RetirementBenefitDtl createPercentageBasedRetirementDeductionDtl(String lawId, Double percentage) {
    	RetirementBenefitDtl retirementDtl = new RetirementBenefitDtl();
    	retirementDtl.setLawId(lawId);
        retirementDtl.setBenefitType(RetirementBenefitTypeEnum.PLAN401K);
       
        Datum rate = new Datum();
        rate.setValue1AsPercentage(percentage);
        rate.setType2(DatumType2Enum.PERPAYCHECK);
        rate.setValue2(1.0);
        retirementDtl.setRate(rate);

        Datum annualCap = new Datum();
        annualCap.setValue1AsUSDMoneyAmount(new BigDecimal("15500.00"));
        annualCap.setType2(DatumType2Enum.NOTAPPLICABLE);
        retirementDtl.setMaxContribution(annualCap);

        Datum annualCatchupCap = new Datum();
        annualCatchupCap.setValue1AsUSDMoneyAmount(new BigDecimal("5000.00"));
        annualCatchupCap.setType2(DatumType2Enum.NOTAPPLICABLE);
        retirementDtl.setMaxCatchupContribution(annualCatchupCap);

        return retirementDtl;
    }*/
    
    protected TaxDtl createTaxItem(String lawId) {
        TaxDtl taxDtl = new TaxDtl();
        Tax tax = taxEAO.findTaxByLawId(lawId);
        taxDtl.setPayrollItem(tax);
        return taxDtl;
    }
    
    protected void addWorkerFITComplianceInfo(Worker emp, W4WithholdingStatusEnum withholdStatus, Integer allowance,
                                              boolean isExempt) {
        W4Detail w4Detail = new W4Detail();
        w4Detail.setTaxExempt(isExempt);
        w4Detail.setWithholdingStatus(withholdStatus);
        w4Detail.setNoOfAllowances(allowance);
        w4Detail.setLawId(mUSFIT.getPayrollItem().getLawId());
        w4Detail.setDocId(ComplianceDocIdEnum.W4IRS);
        emp.getWithholdingDocsForCalc().add(w4Detail);
    }
    
    protected void addWorkerFITComplianceInfo(Worker emp, W4WithholdingStatusEnum withholdStatus, Integer allowance,
                                              Money additionalAmount, boolean isExempt) {
        W4Detail w4Detail = new W4Detail();
        w4Detail.setTaxExempt(isExempt);
        w4Detail.setWithholdingStatus(withholdStatus);
        w4Detail.setNoOfAllowances(allowance);
        w4Detail.setLawId(mUSFIT.getPayrollItem().getLawId());
        w4Detail.setDocId(ComplianceDocIdEnum.W4IRS);
        w4Detail.setAdditionalAmount(additionalAmount);
        emp.getWithholdingDocsForCalc().add(w4Detail);
    }
    
    protected void addWorkerCAPITComplianceInfo(Worker emp, DE4WithholdingStatusEnum withholdStatus, Integer allowance,
                                                Money additionalAmount, Integer deductions, boolean isExempt) {
        DE4Detail de4Detail = new DE4Detail();
        de4Detail.setTaxExempt(isExempt);
        de4Detail.setWithholdingStatus(withholdStatus);
        de4Detail.setNoOfAllowances(allowance);
        de4Detail.setDocId(ComplianceDocIdEnum.DE4);
        de4Detail.setAdditionalAmount(additionalAmount);
        de4Detail.setNoOfDeductions(deductions);
        emp.getWithholdingDocsForCalc().add(de4Detail);
    }
    
    protected void setupPaycheckDates(PayrollFrequencyEnum payFrequency) {
        if (payFrequency == PayrollFrequencyEnum.DAILY)
        {
            mPeriodEndDate = (Calendar)sDailyPeriodEndDate.clone();
        } else if (payFrequency == PayrollFrequencyEnum.WEEKLY) {
            mPeriodEndDate = (Calendar)sWeeklyPeriodEndDate.clone();
        } else if (payFrequency == PayrollFrequencyEnum.BIWEEKLY) {
            mPeriodEndDate = (Calendar)sBiweeklyPeriodEndDate.clone();
        } else if (payFrequency == PayrollFrequencyEnum.SEMIMONTHLY) {
            mPeriodEndDate = (Calendar)sSemimonthlyPeriodEndDate.clone();
        } else if (payFrequency == PayrollFrequencyEnum.MONTHLY) {
            mPeriodEndDate = (Calendar)sMonthlyPeriodEndDate.clone();
        } else if (payFrequency == PayrollFrequencyEnum.QUARTERLY) {
            mPeriodEndDate = (Calendar)sQuarterlyPeriodEndDate.clone();
        } else if (payFrequency == PayrollFrequencyEnum.ANNUALLY) {
            mPeriodEndDate = (Calendar)sAnnuallyPeriodEndDate.clone();
        }
        mPaycheckDate = (Calendar)mPeriodEndDate.clone();
    }
    
    protected Worker createEmployee(String liveLocationState, String workLocationState) {
    	
        Worker emp = new Employee();
        USPostalAddress workLocationAddress = new USPostalAddressImpl();

        workLocationAddress.setRegionOrState(workLocationState);
        
        WorkLocation workLocation = new WorkLocation();
        workLocation.setTaxResidenceAddress
            (AddressConvertor.getGlobalFromUS(workLocationAddress, new GlobalPostalAddress()));
        USPostalAddress taxResidenceAddress = new USPostalAddressImpl();
        taxResidenceAddress.setRegionOrState(liveLocationState);
        emp.setTaxResidenceAddress(
        		AddressConvertor.getGlobalFromUS(taxResidenceAddress, new GlobalPostalAddress()));
        emp.setWorkLocation(workLocation);
        return emp;
    }
    
    /**
     * FIXME tget rid of this and just embed it in the paycheck
     * @param checkDate
     * @param startDate
     * @param endDate
     * @return
     */
    protected PayrollRun createPayrollRun(Calendar checkDate, Calendar startDate, Calendar endDate) {
        PayrollRun run = new PayrollRun();
        DateRange dateRange = new DateRange();
        dateRange.setStartDate(startDate.getTime());
        dateRange.setEndDate(endDate.getTime());
        // Use the same date for both PayrollRun and paycheck date
        run.setPayDate(checkDate.getTime());
        run.setPayrollRunDate(checkDate.getTime());
        run.setPayPeriod(dateRange);
        run.setPartialPeriod(false);
        return run;
    }
    
    /**
     * Sets the YTD amount for the specified PayrollItemDtl
     * 
     * @param itemDtl The PayrollItemDtl to set the YTD amount
     * @param payDate TODO
     * @param ytdAmount The YTD amount
     * @param ytdIncomeSubjectToTax 
     * @param ytdWageBase 
     * @param ytdHours 
     */
    protected void setYtdAmount(PayrollItemDtl itemDtl, Date payDate, BigDecimal ytdAmount, BigDecimal ytdIncomeSubjectToTax, BigDecimal ytdWageBase, Double ytdHours) {
    	PayrollItemDtlPtdDatum ptdDatum = new PayrollItemDtlPtdDatum();
    	
    	PtdDatumType ptdDatumType = PtdUtils.newPtdDatumType(PtdTypeEnum.QUARTER1, payDate, 
    			new Money(sCurrency.toString(), ytdAmount.toString()), 
    			new Money(sCurrency.toString(), ytdIncomeSubjectToTax.toString()), 
    			new Money(sCurrency.toString(), ytdWageBase.toString()), 
    			ytdHours);
    	
    	ptdDatum.setDatum(ptdDatumType);
    	
    	itemDtl.getPtdDatums().add(ptdDatum);
    }
    
    /**
     * This utility routine returns a new Money object containing the specified amount
     * 
     * @param amount The amount to initialize the Money object with
     * @return A Money object
     */
    protected Money newMoney(String amount) {
    	return new Money(sCurrency.toString(), amount);
    }
    
    protected class LineItemAmount {
        private String itemId;
        private BigDecimal amount;
        private BigDecimal ytdAmount;
        private BigDecimal isttAmount;
        private BigDecimal wagebaseAmount;
        
        public LineItemAmount(String id, BigDecimal amt) {
            this(id, amt, amt, new BigDecimal("0.00"), new BigDecimal("0.00"));
        }
        
        public LineItemAmount(String id, BigDecimal amt, BigDecimal incomeSubjectToTaxAmt, BigDecimal wageBaseAmt) {
            this(id, amt, amt, incomeSubjectToTaxAmt, wageBaseAmt);
        }
        
        public LineItemAmount(String id, BigDecimal amt, BigDecimal ytdAmt) {
            this(id, amt, ytdAmt, new BigDecimal("0.00"), new BigDecimal("0.00"));
        }
        
        public LineItemAmount(String id, BigDecimal amt, BigDecimal ytdAmt, BigDecimal incomeSubjectToTaxAmt, BigDecimal wageBaseAmt) {
            itemId = id;
            amount = amt;
            ytdAmount = ytdAmt;
            isttAmount = incomeSubjectToTaxAmt;
            wagebaseAmount = wageBaseAmt;
        }
        
        public String getItemId() {
            return itemId;
        }
        
        public BigDecimal getAmount() {
            return amount;
        }
        
        public BigDecimal getYtdAmount() {
            return ytdAmount;
        }
        
        public BigDecimal getIsttAmount() {
            return isttAmount;
        }
        
        public BigDecimal getWagebaseAmount() {
            return wagebaseAmount;
        }
        
        public boolean equals(Object other) {
            if (other == this) {
                return true;            
            }
                    
            if(!(other instanceof LineItemAmount)) {
                return false;
            }
            //If we reach here, other is not null
            LineItemAmount otherLineItemAmount = (LineItemAmount) other;
            //Useful for debugging that's why temporary variables
            boolean idEquals = otherLineItemAmount.getItemId().equals(this.itemId);
            boolean amountEquals = (0 == otherLineItemAmount.getAmount().compareTo(this.amount));
            boolean ytdAmountEquals = (0 == otherLineItemAmount.getYtdAmount().compareTo(this.ytdAmount));
            return  idEquals && amountEquals && ytdAmountEquals;
        }
        
        //If We overrode equals method then we must override hashCode   
        @Override
        public int hashCode() {
            int hash = 1;
            if (this.itemId != null) {
                hash = hash * 31 + this.itemId.hashCode();
            }
            if (this.amount != null) 
            {
                hash = hash * 31 + this.amount.hashCode();
            }
            if (this.ytdAmount != null)
            {
                hash = hash * 31 + this.ytdAmount.hashCode();
            }
            if (this.isttAmount != null)
            {
                hash = hash * 31 + this.isttAmount.hashCode();
            }
            if (this.wagebaseAmount != null)
            {
                hash = hash * 31 + this.wagebaseAmount.hashCode();
            }
            return hash;        
        }

    }
    
    protected boolean verifyLineItems(Set<PaycheckLineItem> computedItems, LineItemAmount [] expectedItems,
                                      String lineItemTypeStr) {
        boolean compareResult = (computedItems.size() == expectedItems.length);
        if (true == compareResult) {
            HashSet<LineItemAmount> computedLineItemSet = new HashSet<LineItemAmount>();
            Iterator<PaycheckLineItem> iterator = computedItems.iterator();
            while (iterator.hasNext()) {
                PaycheckLineItem paycheckLineItem = iterator.next();
                Money lineItemValue = paycheckLineItem.getValue();
                if (lineItemValue != null) {
					LineItemAmount lineItemAmount = new LineItemAmount(paycheckLineItem.getLawId().getValue(),
	                                                    lineItemValue.getAmount(), 
	                                                    getYtdAmount(paycheckLineItem),
	                                                    getIsttAmount(paycheckLineItem),
	                                                    getWagebaseAmount(paycheckLineItem));
	                computedLineItemSet.add(lineItemAmount);
                }
            } 
            HashSet<LineItemAmount> expectedLineItemSet = new HashSet<LineItemAmount>();
            for(int i=0; i < expectedItems.length; i++) {
                expectedLineItemSet.add(expectedItems[i]);
            }
            compareResult = computedLineItemSet.equals(expectedLineItemSet);
        }

        if (false == compareResult) {
        	logger.error("Computed " + lineItemTypeStr);            
            Iterator<PaycheckLineItem> iterator = computedItems.iterator();
            while (iterator.hasNext()) {
                PaycheckLineItem paycheckLineItem = iterator.next();
                Money lineItemValue = paycheckLineItem.getValue();
                if (lineItemValue != null) {
				logger.error(getLawLabel(paycheckLineItem) + " -  amount=" +
                                   lineItemValue.getAmount() + ",  YTD Amount=" + 
                                   getYtdAmount(paycheckLineItem) + ", ISTT Amount=" +
                                   getIsttAmount(paycheckLineItem) + ", WageBase Amount=" +
                                   getWagebaseAmount(paycheckLineItem));  
                }
            }            
            logger.error("Expected " + lineItemTypeStr);            
            for(int i=0; i < expectedItems.length; i++) {
            	logger.error(PayrollItemType.parseLawId(expectedItems[i].getItemId()).getLawLabel() +
                                   " -  amount=" + expectedItems[i].getAmount() + ",  YTD Amount=" +
                                   expectedItems[i].getYtdAmount() + "  ISTT Amount=" +
                                   expectedItems[i].getIsttAmount() + " WageBase Amount=" +
                                   expectedItems[i].getWagebaseAmount());            
            }            
        }
        
        return compareResult;
    }
    
    private BigDecimal getYtdAmount(PaycheckLineItem paycheckLineItem) {
        PtdDatumType ptdDatumType = paycheckLineItem.getYtdDatum().getDatum();
        return ptdDatumType.getMoneyValue().getAmount();
    }
    
    private BigDecimal getIsttAmount(PaycheckLineItem paycheckLineItem) {
        Money isttAmt = paycheckLineItem.getIncomeSubjectToTax();
        return isttAmt.getAmount();
    }
    
    private BigDecimal getWagebaseAmount(PaycheckLineItem paycheckLineItem) {
        Money wagebasetAmt = paycheckLineItem.getWageBase();
        return wagebasetAmt.getAmount();
    }
    
    protected void dumpPaycheck(PayrollRun payrollRunData, Paycheck paycheck) {
        logger.debug("Pay Date=" + payrollRunData.getPayDate());
        logger.debug("Pay Period Start Date=" + payrollRunData.getPayPeriod().getStartDate());
        logger.debug("Pay Period End Date=" + payrollRunData.getPayPeriod().getEndDate());
        logger.debug("-----------------------------------------------------------------");
        logger.debug(formatPaycheckLineItems(paycheck));
    }
    
    private String formatPaycheckLineItems(Paycheck paycheck) {
        StringBuffer sb = new StringBuffer();
        for(PaycheckLineItem paycheckLine : paycheck.getPaycheckLineItems()) {
            sb.append("Paycheck line items").append(newline);
            appendPaycheckLineString(sb, paycheckLine); 
            sb.append(newline);
        }
        return sb.toString();        
    }

    private static void appendPaycheckLineString(StringBuffer sb, PaycheckLineItem paycheckLine) {
        sb.append("Label=" + getLawLabel(paycheckLine)).append(tab);
        sb.append("ID=" + paycheckLine.getLawId()).append(tab);
        sb.append("Value=" + paycheckLine.getValue()).append(tab);
        sb.append("YTD Amount=" + paycheckLine.getYtdDatum().getDatum().getMoneyValue().getAmount()).append(tab);
    }
    
    private static String getLawLabel(PaycheckLineItem paycheckLineItem) {
    	return PayrollItemType.parseLawId(paycheckLineItem.getLawId().getValue()).toString();
    }
}
