package com.intuit.sb.payroll.capability.common;

import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.Gender;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.cdm.basetypes.WebSiteAddress;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.employer.EmployerEAO;
import com.intuit.sb.payroll.entity.model.AgencyIdType;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;
import com.intuit.sb.payroll.entity.model.ContactPerson;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.EmployerIdTypeEnum;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.NonEmployee;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.PersonTaxId;
import com.intuit.sb.payroll.entity.model.PersonTaxIdEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * The purpose of this class is to centralize the utility methods to manage and condition test data for HR Entities in
 * the database.  These methods will allow Worker entities and their associated entities to be created, committed
 * to the database and deleted.  This will allow tests to create the necessary entities in the database, and
 * clean up after execution.
 *
 * The intent is that functionality is added as the need arises for additional test data.
 *
 * $Author: smadduri $
 * $Revision: #7 $
 * $DateTime: 2008/07/14 13:22:36 $
 *
 */
public class WorkLocationAndWorkerDataPopulator extends PayrollTestBase {

	private static int MAX_NUM_OF_WORKERS = 3;
	private int numOfWorkLocations = 3;
	private int sNumOfWorkers = MAX_NUM_OF_WORKERS;
	private WorkLocation [] workLocations = new WorkLocation[numOfWorkLocations];
	private ContactPerson [] contactPersons = null;
	private Worker[] workersArray;
	private USPostalAddress [] addresses = null;
	private static String companySeedXml = "..//composite//test//seeddata//peter.xml";
	
	/**
	 * default constructor
	 */
	public WorkLocationAndWorkerDataPopulator()
    {

    }

    /**
	 * Constructor that takes an integer, specifying the number of Workers to create.
	 *
	 * @param noOfWorkers
	 */
    public WorkLocationAndWorkerDataPopulator(int noOfWorkers)
    {
        if (noOfWorkers <= MAX_NUM_OF_WORKERS && noOfWorkers > 0) {
            sNumOfWorkers = noOfWorkers;
        }
    }

	/**
	 * Public method for this class that will create the WorkLocations, and Workers, and persist them
	 * to the database.  Consumers of this class will only have to call this method to populate the database with
	 * Workers
	 *
	 */
	public void populateWorkLocAndWorkers()
	{
		populateWorkLocs();
		populateWorkers();
		//TODO [Test] createContractors
	}

	public void populateWorkLocs()
	{
		prepare();
		populateOrUpdateEmployerWithWorkLocs();
	}

	/**
	 * retrieve or populate employer, and associate work locations with Employer
	 * @return employer
	 */
	public Employer populateOrUpdateEmployerWithWorkLocs()
	{
		return populateOrUpdateEmployerWithWorkLocs(companySeedXml);
	}

	/**
	 * this will be called from deployment script
	 * @param xmlFilePath the company seed xml file
	 * @return Company The Company object that was just saved
	 */
	public Employer populateOrUpdateEmployerWithWorkLocs(String xmlFilePath)
	{
		EmployerEAO employerEAO = qaBootstrap.getEjbFromClass(EmployerEAO.class);
		EmployerComponent employerComponent = qaBootstrap.getEjbFromClass(EmployerComponent.class);
		Employer employer = employerEAO.findCurrentEmployer();

		//if the employer is already populated by SeedDataPopulator, return the employer
		//NOTE: findCurrentEmployer() will return an employer object with a version value of 0 after persisting an
		//        empty (blank) Employer object into the database
		//      So, we still need to populate the employer information if the version value is 0

        populateWorkLocations();
        
        // Using capability service to kick the worklocation add workflow
        try {
        	employerComponent.addWorkLocations(Arrays.asList(workLocations));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//Always add the work locations to the employer
		//employer.setWorkLocation(new HashSet<WorkLocation>(Arrays.asList(workLocations)));

		if (employer.getVersion() > 0)
		{
			return employer;
		}

		//For deployment script, the employer is just created by EmployerEAO's findCurrentEmployer(), but it's not in DB yet
		//employer.setAdditionDtl(additionDtl);
		BankAccount ownerBankAccount = createBankAccount("owner", "50424");
		Set<BankAccount> ownerAccounts = new HashSet<BankAccount>();
		ownerAccounts.add(ownerBankAccount);
		employer.setBankAccounts(ownerAccounts);
		//employer.setCompensationDtl(compensationDtl);
//		Set<EmployerContactPerson> contacts = new HashSet<EmployerContactPerson>();
//		for ( int i=0; i < workLocations.length; i++)
//		{
//			contacts.add(employerContactPersons[i]);
//		}
//
//		employer.setEmployerContactPerson(contacts);
		employer.setCompanyName("Bogus");
		employer.setDbaName("Bogus Inc");
		//employer.setDeductionDtl(deductionDtl);
		//employer.setEmployerComplianceDtl(compliances);
		employer.setFirstPayrollDate(new Date(0));
		employer.setIndustryType(IndustryTypeEnum.OTHER);
		employer.setNonProfit(false);
		employer.setLegalAndBusAddressSame(false);
		employer.setLegalName("Mom's shop");
		employer.setLegalStructure(LegalStructureEnum.LLC);
        USPostalAddress employerAddress = new USPostalAddressImpl();
        employerAddress.setAddressLine1("456 Third St");
        employerAddress.setCity("Mountain View");
        employerAddress.setGeoCode("US");
        employerAddress.setIsNotToBeStandardized(true);
        employerAddress.setIsStandardized(false);
        employerAddress.setIsStreetAddress(true);
        employerAddress.setRegionOrState("CA");
        employerAddress.setZipOrPostalCode("94043");
        
        GlobalPostalAddress globalAddr = 
        	AddressConvertor.getGlobalFromUS(employerAddress, new GlobalPostalAddress());
        
        employer.setLegalAddress(globalAddr);
        TelephoneNumber phone = new TelephoneNumber();
        phone.setFreeFormNumber("650-555-1212");
        phone.setExtension("2222");
        employer.setLegalPhone(phone);

		// Employer Id
		EmployerId employerIdElement = new EmployerId();
		List<AgencyIdType>  agencyIdtypes = BasicTestBase.getEntityService().findAll(AgencyIdType.class);
		for (AgencyIdType agencyIdType : agencyIdtypes)
		{
			if(EmployerIdTypeEnum.EIN.equals(agencyIdType.getIdTypeKey()))
			{
				employerIdElement.setAgencyIdType(agencyIdType);
				break;
			}
		}
		employerIdElement.setValue("991111110");
		employer.addEmployerId(employerIdElement);

/*		OwnerPerson owner = new OwnerPerson();
        EmailAddress emailAddress = new EmailAddress();
        emailAddress.setLastVerifiedDate(new Date());
        emailAddress.setValue("SomeOne@BogusInc.com");
        PostalAddress address = new PostalAddress();
        address.setAddressLine1("123 First St");
        address.setCity("Mountain View");
        address.setGeoCode("US");
        address.setIsNotToBeStandardized(true);
        address.setIsStandardized(false);
        address.setIsStreetAddress(true);
        address.setRegionOrState("CA");
        address.setZipOrPostalCode("94043");
        TelephoneNumber homePhone = new TelephoneNumber();
        homePhone.setPhoneNumber("650-944-8000");
        PersonName personName = new PersonName();
        personName.setFirstName("Some");
        personName.setLastName("One");
        personName.setMiddleName("");
        personName.setSuffix("");
        personName.setTitle("");
        owner.setEmailAddress(emailAddress);
        owner.setHomelAddress(address);
        owner.setHomePhone(homePhone);
        owner.setMailingAddress(address);
        owner.setPersonName(personName);

		PersonTaxId personTaxId = new PersonTaxId();
		personTaxId.setType(PersonTaxIdEnum.SSN);
		personTaxId.setValue("123-45-6789");
		personTaxId.setLastVerifiedDate(new Date(0));
		owner.setPersonTaxId(personTaxId);

		owner.setWorkAddress(address);
		owner.setWorkPhone(homePhone);

		Set<OwnerPerson> owners = new HashSet<OwnerPerson>();
		owners.add(owner);
		employer.setOwnerPerson(owners);
*/
		//employer.setPayrollItem(payrollItem);
		//employer.setPayrollSchedule(payrollSchedule);
		//employer.setTaxDtl(taxDtl);

		WebSiteAddress webAddress = new WebSiteAddress();
		webAddress.setURI("http://www.bogus.com");
		employer.setWebAddress(webAddress);

//		employer = employerEAO.makePersistent(employer);

		return employer;

	}


	/**
	 * Create and save 3 work locations: SF, San Jose, and Texas
	 */
	private void populateWorkLocations() {

		String [] locationNames = new String[]{"San Francisco, CA", "Mountain View, CA", "Mesquite, TX"};

 		for (int i = 0; i < workLocations.length; i++)
		{

			WorkLocation workLocation = new WorkLocation();

			workLocation.setName(locationNames[i]);

//			EmailAddress emailAddress = new EmailAddress();
//			emailAddress.setLastVerifiedDate(new Date());
//			emailAddress.setValue("payroll@intuit.com");

            TelephoneNumber mainPhone = new TelephoneNumber();
            mainPhone.setFreeFormNumber("650-944-500"+i);
            workLocation.setMainPhone(mainPhone);
//		    workLocation.setContactPerson(employerContactPersons[i]);
//          workLocation.setEmailAddress(emailAddress);

            TelephoneNumber faxPhone = new TelephoneNumber();
            faxPhone.setFreeFormNumber("650-944-600"+i);
            workLocation.setFaxPhone(faxPhone);

			// Make the first WorkLocation default
			workLocation.setDefaultt(i==0);
			USPostalAddress mailingAddress = addresses[i];
            workLocation.setMailingAddress(
            		AddressConvertor.getGlobalFromUS(mailingAddress, new GlobalPostalAddress()));
 

            //set tax address to be the same as mailing address
            workLocation.setTaxResidenceAddress(
            		AddressConvertor.getGlobalFromUS(mailingAddress, new GlobalPostalAddress()));

            WebSiteAddress webAddress = new WebSiteAddress();
            webAddress.setURI("http://www.intuit.com");
            workLocation.setWebAddress(webAddress);

            //Persist the workLocation
            workLocation = BasicTestBase.getEntityService().makePersistent(workLocation);

            workLocations[i] = workLocation;

//            employerContactPersons[i] = workLocation.getContactPerson();

		}
	}

	/**
	 * create up to 2 Employee and 1 nonEmployee, associate each with an WorkLocation, EmployerContactPerson, etc
	 *
	 */
	private void populateWorkers()
	{
		  WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
		  workersArray = new Worker[sNumOfWorkers];

		  for (int i = 0; i < workersArray.length; i++)
		  {
			  Worker worker = null;
			  String firstName = contactPersons[i].getPersonName().getFirstName();
			  String lastName = contactPersons[i].getPersonName().getLastName();
			  String suffix = contactPersons[i].getPersonName().getSuffix();

			  if (isEmployee(i))
			  {
				  worker = new Employee();
			  }
			  else
			  {
				  worker = new NonEmployee();
			  }
			  /******************************
				  * Properties in Employee
			  ******************************/
			  PersonTaxId personTaxId = new PersonTaxId();
			  personTaxId.setType(PersonTaxIdEnum.SSN);
			  personTaxId.setValue("999-99-999"+i);
			  personTaxId.setLastVerifiedDate(new Date(0));
			  worker.setPersonTaxId(personTaxId);

			  Calendar cal = DateUtil.createCalendar(1950+i*20, 0, 1);
			  Date dateOfBirth = new Date(cal.getTimeInMillis());
			  worker.setDateOfBirth(dateOfBirth);
			  worker.setActive(true);
			  worker.setJobPositionNote("Worker Number"+(i+1));
			  worker.setWorkerId((new Integer(i+1)).toString());
			  
			  

              worker.setTaxResidenceAddress(
            		  AddressConvertor.getGlobalFromUS(addresses[i], new GlobalPostalAddress()));

              worker.setEmailAddress(contactPersons[i].getEmailAddress());
              worker.setGender(Gender.Male);

              DateRange dateRange = new DateRange();
              dateRange.setStartDate(DateUtil.createDate(2006, 0, 1));
              worker.setEmploymentPeriod(dateRange);

              worker.setPersonName(contactPersons[i].getPersonName());
              worker.setHomePhone(contactPersons[i].getHomePhone());
              worker.setMobilePhone(contactPersons[i].getMobilePhone());

			  /******************************
			  * Relations in Worker
			  ******************************/
			  //Set relation "ContactPerson"
              Set<ContactPerson> contacts = new HashSet<ContactPerson>();
              contacts.add(contactPersons[i]);
              worker.setContactPersons(contacts);

              // Initialize the RelationshipToOwner field
              worker.setRelationshipToOwner(RelationshipToOwnerEnum.UNRELATED);

              //Set relation WorkLocation
              worker.setWorkLocation(workLocations[i%workLocations.length]);

              //Set relation BankAccount
              BankAccount bankAccount = createBankAccount(firstName + " " + lastName + " " + suffix,
                                                          "123456789"+i);
              Set<BankAccount> bankAccounts = new HashSet<BankAccount>();
              bankAccounts.add(bankAccount);
              worker.setBankAccounts(bankAccounts);

              //persist worker using worker component to kick the Task workflow
			  try {
				worker = workerComponent.addWorker(worker);
			} catch (Exception e) {
				e.printStackTrace();
			}
			 workersArray[i] = worker;
		  }
			 
	}

    private boolean isEmployee(int i)
    {
    	if ((i%2)== 0)
    		return true;
    	else
    		return false;
	}

    private void createUSPhysicalAddress()
    {
    	addresses = new USPostalAddress[numOfWorkLocations];

        String [] addressLines = {"1231 22nd Ave", "2698 Marine Way", "2701 N Mesquite Dr"};
        String [] cityNames = {"San Francisco", "Mountain View", "Mesquite"};
        String [] stateNames = {"CA", "CA", "TX"};
        String [] zipCodes = {"94122", "94043", "75150"};

        for (int i=0; i<this.workLocations.length; i++)
        {
	        USPostalAddress address = new USPostalAddressImpl();
	        address.setAddressLine1(addressLines[i]);
	        address.setCity(cityNames[i]);
	        address.setCountry("United States");
	        address.setIsNotToBeStandardized(true);
	        address.setIsStandardized(true);
	        address.setIsStreetAddress(true);
	        address.setRegionOrState(stateNames[i]);
	        address.setZipOrPostalCode(zipCodes[i]);
	        address.setCountryISOCode("US");

	        addresses[i] = address;
        }

    }

	/**
	 * create a contact Person for each work location
	 * @param i the index of workLocation
	 * @return
	 */
	private void createContactPerson()
	{
		contactPersons = new ContactPerson[numOfWorkLocations];
		String firstName = "John";
		String lastName = "Doe";
		String suffix = "";

		for ( int i = 0; i < numOfWorkLocations; i++)
		{
            suffix += "I";
			EmailAddress emailAddress = new EmailAddress();
			emailAddress.setDateLastVerified(new Date());
			emailAddress.setAddress(firstName+"_"+lastName+"_"+suffix+"@intuit.com");
			USPostalAddress mailingAddress = addresses[i];
	        TelephoneNumber mainPhone = new TelephoneNumber();
	        mainPhone.setFreeFormNumber("650-944-777"+i);

			ContactPerson contactPerson = new ContactPerson();
			contactPerson.setEmailAddress(emailAddress);
			TelephoneNumber homePhone = new TelephoneNumber();
			homePhone.setFreeFormNumber("650-944-123"+i);
			contactPerson.setHomePhone(homePhone);
			contactPerson.setMailingAddress(
					AddressConvertor.getGlobalFromUS(mailingAddress, new GlobalPostalAddress()));
			TelephoneNumber mobilePhone = new TelephoneNumber();
			mobilePhone.setFreeFormNumber("650-944-222"+i);
			contactPerson.setMobilePhone(mobilePhone);
			PersonName personName = new PersonName();
			personName.setFirstName("John");
			personName.setLastName("Doe");
			personName.setMiddleName("");
			personName.setSuffix(suffix);
			personName.setTitle("Worker");
			contactPerson.setPersonName(personName);
			contactPerson.setWorkPhone(mainPhone);

			contactPersons[i] = contactPerson;
		}
	}

    /**
     * pre populate data such as contact persons in memory for work locations and workers
     */
    private void prepare()
    {
    	createUSPhysicalAddress();
		createContactPerson();
    }

    /**
     * create a bank account instance
     * @param name is the name of the account holder
     * @param accountNo is the account No
     * @return an bank account instance
     */
    private BankAccount createBankAccount(String name, String accountNo)
    {
    	BankAccount bankAccount = new BankAccount();
    	bankAccount.setName(name);
    	bankAccount.setAccountNo(accountNo);
    	bankAccount.setRoutingNo("124000012");
    	bankAccount.setBankName("Bank of America");
    	bankAccount.setAccountType(BankAccountTypeEnum.CHECKING);

        USPostalAddress bankAddress = new USPostalAddressImpl();
        bankAddress.setAddressLine1("384 San Antonio rd");
        bankAddress.setCity("Mountain View");
        bankAddress.setGeoCode("US");
        bankAddress.setIsNotToBeStandardized(true);
        bankAddress.setIsStandardized(true);
        bankAddress.setIsStreetAddress(true);
        bankAddress.setRegionOrState("CA");
        bankAddress.setZipOrPostalCode("94040");       
    	bankAccount.setBankAddress(
    			AddressConvertor.getGlobalFromUS(bankAddress, new GlobalPostalAddress()));

	    Calendar cal = DateUtil.createCalendar(1970, 0, 1);
    	bankAccount.setDateLastVerified(new Date(cal.getTimeInMillis()));

    	return bankAccount;
    }
}
