package com.intuit.sb.payroll.capability.employer;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.impl.EmployerContactPersonValidator;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.PersonName;

public class EmployerContactsTest extends EmployerComponentTestBase {

    

    @Test
    public void contactPersonValidatorTest()  throws Exception {
        String firstName = "CreateContactPerson";
        String lastName = "Test";
        String title = "CreateContactPersonsTest";
        String phone = "408-555-1234";
        EmployerContactPerson person = createEmployerContactPerson (firstName, lastName, title, phone);

        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        employerContactPersons.add(person);
        EmployerContactPersonValidator.getInstance().validateEntityCollection(employerContactPersons);
    }

    @Test(expectedExceptions=ValidationException.class)
    public void nullContactPersonValidatorTest() throws Exception {
        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        employerContactPersons.add(null);
        EmployerContactPersonValidator.getInstance().validateEntityCollection(employerContactPersons);
    }

    @Test(expectedExceptions=ValidationException.class)
    public void nullContactPersonsValidatorTest() throws Exception {
        String firstName = "NullContactPerson";
        String lastName = "Test";
        String title = "NullContactPersonsTest";
        String phone = "408-555-1234";
        EmployerContactPerson person = createEmployerContactPerson (firstName, lastName, title, phone);

        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        employerContactPersons.add(person);
        employerContactPersons.add(null);
        EmployerContactPersonValidator.getInstance().validateEntityCollection(employerContactPersons);
    }

    @Test(expectedExceptions=ValidationException.class)
    public void missingNameContactPersonValidatorTest() throws Exception {
        String title = "MissingNameContactPersonTest";
        String phone = "408-555-1234";
        EmployerContactPerson person = createEmployerContactPerson (null, null, title, phone);

        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        employerContactPersons.add(person);
        EmployerContactPersonValidator.getInstance().validateEntityCollection(employerContactPersons);
    }



    @Test
    public void getContactPersonsTest() throws Exception {
        Set<EmployerContactPerson> allEmployerContactPersons = sEmployerComponent.getEmployerContactPersons();
        for (EmployerContactPerson employerContactPerson : allEmployerContactPersons) {
            PersonName personName = employerContactPerson.getPersonName();
            if (personName != null) {
                sLogger.info(personName.getFirstName() + " " + personName.getLastName());
            }
        }
    }

    @Test(expectedExceptions=ValidationException.class)
    public void addContactPersonsTest() throws Exception {
        String firstName = "AddContactPerson";
        String lastName = "Test";
        String title = "AddContactPersonsTest";
        String phone = "408-555-1212";
        EmployerContactPerson person = createEmployerContactPerson (firstName, lastName, title, phone);

        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        employerContactPersons.add(person);
        sEmployerComponent.addEmployerContactPersons(employerContactPersons);

        boolean found= false;
        EmployerContactPerson newPerson = null;
        Set<EmployerContactPerson> employerContactPersons2 = sEmployerComponent.getEmployerContactPersons();
        for (EmployerContactPerson person2 : employerContactPersons2) {
            PersonName personName = person2.getPersonName();
            if (personName != null) {
                if (personName.getTitle().equals(title)) {
                    found = true;
                    newPerson = person2;
                    break;
                }
            }
        }
        AssertJUnit.assertTrue("Did not find the EmployerContactPerson named AddContactPersonsTest", (newPerson != null));

        // Clean up
        if (found) {
            employerContactPersons.clear();
            employerContactPersons.add(newPerson);
            sEmployerComponent.deleteEmployerContactPersons(employerContactPersons);
        }
    }

    @Test(expectedExceptions=ValidationException.class)
    public void addNullContactPersonsTest() throws Exception {
        sEmployerComponent.addEmployerContactPersons(null);
    }

    @Test
    public void addEmptyContactPersonsTest() throws Exception {
        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        sEmployerComponent.addEmployerContactPersons(employerContactPersons);
    }

    @Test(expectedExceptions=ValidationException.class)
    public void updateContactPersonsTest() throws Exception {
        String firstName = "UpdateContactPerson";
        String lastName = "Test";
        String title = "UpdateContactPersonsTest";
        String phone = "408-555-2222";
        EmployerContactPerson person = createEmployerContactPerson (firstName, lastName, title, phone);

        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        employerContactPersons.add(person);
        sEmployerComponent.addEmployerContactPersons(employerContactPersons);

        boolean found= false;
        EmployerContactPerson newPerson = null;
        String newTitle = "Modified UpdateContactPersonsTest";
        List<EmployerContactPerson> employerContactPersons2 = new ArrayList<EmployerContactPerson>(sEmployerComponent.getEmployerContactPersons());
        for (EmployerContactPerson person2 : employerContactPersons2) {
            PersonName personName = person2.getPersonName();
            if (personName != null) {
                if (personName.getTitle().equals(title)) {
                    found = true;
                    newPerson = person2;

                    personName.setTitle(newTitle);
                    person2.setPersonName(personName);
                    break;
                }
            }
        }
        AssertJUnit.assertTrue("Did not find the EmployerContactPerson named UpdateContactPersonsTest", (newPerson != null));

        sEmployerComponent.updateEmployerContactPersons(employerContactPersons2);

        found= false;
        List<EmployerContactPerson> employerContactPersons3 = new ArrayList<EmployerContactPerson>(sEmployerComponent.getEmployerContactPersons());
        for (EmployerContactPerson person3 : employerContactPersons3) {
            PersonName personName = person3.getPersonName();
            if (personName != null) {
                if (personName.getTitle().equals(newTitle)) {
                    newPerson = person3;
                    found = true;
                    break;
                }
            }
        }
        AssertJUnit.assertTrue("Did not find the updated EmployerContactPerson named Modified UpdateContactPersonsTest", found);


        // Clean up
        if (found) {
            employerContactPersons3.clear();
            employerContactPersons3.add(newPerson);
            sEmployerComponent.deleteEmployerContactPersons(employerContactPersons3);
        }
    }

    @Test(expectedExceptions=ValidationException.class)
    public void updateNullContactPersonsTest() throws Exception {
        sEmployerComponent.updateEmployerContactPersons(null);
    }

    @Test
    public void updateEmptyContactPersonsTest() throws Exception {
        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        sEmployerComponent.updateEmployerContactPersons(employerContactPersons);
    }

    @Test
    public void deleteContactPersonsTest() throws Exception {
        String firstName = "DeleteContactPerson";
        String lastName = "Test";
        String title = "DeleteContactPersonsTest";
        String phone = "415-555-1212";
        EmployerContactPerson person = createEmployerContactPerson (firstName, lastName, title, phone);
        EmployerContactPerson person2 = createEmployerContactPerson (firstName+"2", lastName, title+"2", phone);

        // Add 2 contact persons
        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        employerContactPersons.add(person);
        employerContactPersons.add(person2);
        sEmployerComponent.addEmployerContactPersons(employerContactPersons);

        boolean found= false;
        EmployerContactPerson newPerson = null;
        List<EmployerContactPerson> employerContactPersons2 = new ArrayList<EmployerContactPerson>(sEmployerComponent.getEmployerContactPersons());
        for (EmployerContactPerson person3 : employerContactPersons2) {
            PersonName personName = person3.getPersonName();
            if (personName != null) {
                if (personName.getTitle().equals(title)) {
                    found = true;
                    newPerson = person3;
                    break;
                }
            }
        }
        AssertJUnit.assertTrue("Add employer contact person failed: Did not find the EmployerContactPerson named DeleteContactPersonsTest", (newPerson != null));

        // Delete one employerContactPerson (leaving one)
        if (found) {
            employerContactPersons2.clear();
            employerContactPersons2.add(newPerson);
            sEmployerComponent.deleteEmployerContactPersons(employerContactPersons2);
        }

        // Make sure it was deleted
        found= false;
        newPerson = null;
        Set<EmployerContactPerson> employerContactPersons3 = sEmployerComponent.getEmployerContactPersons();
        for (EmployerContactPerson person4 : employerContactPersons3) {
            PersonName personName = person4.getPersonName();
            if (personName != null) {
                if (personName.getTitle().equals(title)) {
                    found = true;
                    newPerson = person4;
                    break;
                }
            }
        }
        AssertJUnit.assertFalse("Delete employer contact person failed - still found the EmployerContactPerson named DeleteContactPersonsTest", found);

        // Clean up
        sEmployerComponent.deleteAllContactPersons();
}

    @Test(expectedExceptions=ValidationException.class)
    public void deleteNullContactPersonsTest() throws Exception {
        sEmployerComponent.deleteEmployerContactPersons(null);
    }

    @Test
    public void deleteEmptyContactPersonsTest() throws Exception {
        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        sEmployerComponent.deleteEmployerContactPersons(employerContactPersons);
    }

    @Test(expectedExceptions=ValidationException.class)
    public void deleteLastContactPersonTest() throws Exception {
        String firstName = "DeleteLastContactPerson";
        String lastName = "Test";
        String title = "DeleteLastContactPersonsTest";
        String phone = "650-555-1212";
        EmployerContactPerson person = createEmployerContactPerson (firstName, lastName, title, phone);

        // Add 1 employer contact person
        List<EmployerContactPerson> employerContactPersons = new ArrayList<EmployerContactPerson>();
        employerContactPersons.add(person);
        sEmployerComponent.addEmployerContactPersons(employerContactPersons);

        boolean found= false;
        EmployerContactPerson newPerson = null;
        List<EmployerContactPerson> employerContactPersons2 = new ArrayList<EmployerContactPerson>(sEmployerComponent.getEmployerContactPersons());
        for (EmployerContactPerson person3 : employerContactPersons2) {
            PersonName personName = person3.getPersonName();
            if (personName != null) {
                if (personName.getTitle().equals(title)) {
                    found = true;
                    newPerson = person3;
                    break;
                }
            }
        }
        AssertJUnit.assertTrue("Add employer contact person failed: Did not find the EmployerContactPerson named DeleteLastContactPersonsTest", (newPerson != null));

        // Delete one/last employerContactPerson
        if (found) {
            employerContactPersons2.clear();
            employerContactPersons2.add(newPerson);
            sEmployerComponent.deleteEmployerContactPersons(employerContactPersons2);
        }
        AssertJUnit.assertTrue("Delete last employer contact person failed ", found);
    }

 
}