package com.intuit.sb.payroll.capability.employer;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Set;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentLineItem;
import com.intuit.sb.payroll.entity.model.PaymentType;

public class EmployerPaymentTest extends EmployerComponentTestBase {
	
	@Test
	public void testGetEmptyPayments() throws ValidationException{
		Employer employer = sEmployerComponent.getEmployer();
		Set<Payment> empPayments = employer.getPayments();
		
		DateRange dateRange = new DateRange();
		Calendar previousYear = DateUtil.getNowCalendar();
		previousYear.set(previousYear.get(Calendar.YEAR)-1,previousYear.get(Calendar.MONDAY)
				, previousYear.get(Calendar.DAY_OF_MONTH));
		
		dateRange.setStartDate(previousYear.getTime());
		dateRange.setEndDate(DateUtil.getNowDate());
		// Get Payments from the Bean
		List<Payment> payments = sEmployerComponent.getPayments(dateRange);
		
		Assert.assertEquals(payments.size(), empPayments.size(), "Size of Payments returned are not same");
	}
	
	@Test
	public void testAddNewPayments() throws ValidationException{
		Employer employer = sEmployerComponent.getEmployer();
		Set<Payment> empPayments = employer.getPayments();
		int initPaymentSize = empPayments.size();
		List<Payment> newPayments = new ArrayList<Payment>();
		
		Payment payment = EmployerComponentTestUtils.createPaymentInMemory(PaymentType.CAUI, QboeLawIdConstants.sCASUILawId, new BigDecimal("100.02"));
		newPayments.add(payment);
		
		List<Payment> payments = sEmployerComponent.updatePayments(newPayments);
		Assert.assertEquals(1, payments.size());
		
		Set<Payment> updatedEmpPayments = employer.getPayments();
		
		Assert.assertEquals(updatedEmpPayments.size(), initPaymentSize+1);
		
		
		DateRange dateRange = new DateRange();
		Calendar previousYear = DateUtil.getNowCalendar();
		previousYear.set(previousYear.get(Calendar.YEAR)-1,previousYear.get(Calendar.MONDAY)
				, previousYear.get(Calendar.DAY_OF_MONTH));
		
		dateRange.setStartDate(previousYear.getTime());
		dateRange.setEndDate(DateUtil.createCalendar(2009, 2, 15).getTime());
		// Get Payments from the Bean
		List<Payment> capPayments = sEmployerComponent.getPayments(dateRange);
		Assert.assertEquals(1, capPayments.size());
	}
	
	
	@Test
	public void testUpdatePayments() throws ValidationException{
		
		Employer employer = sEmployerComponent.getEmployer();
		Set<Payment> empPayments = employer.getPayments();
		int initPaymentSize = empPayments.size();
		
		List<Payment> newPayments = new ArrayList<Payment>();
		Payment payment = EmployerComponentTestUtils.createPaymentInMemory(PaymentType.CAUI, QboeLawIdConstants.sCASUILawId, new BigDecimal("100.02"));
		newPayments.add(payment);
		List<Payment> payments = sEmployerComponent.updatePayments(newPayments);
		Payment savedPayment = payments.get(0);
		Set<PaymentLineItem> paylineItems = savedPayment.getPaymentLineItems();
		Money newMoney = new Money();
		newMoney.setAmount(new BigDecimal("99.09"));
		for (PaymentLineItem payline: paylineItems){
			payline.setAmount(newMoney);
		}
		
		List<Payment> updatedPayments = new ArrayList<Payment>();
		updatedPayments.add(savedPayment);

		List<Payment> updatedNewpayments = sEmployerComponent.updatePayments(updatedPayments);
		paylineItems = updatedNewpayments.get(0).getPaymentLineItems();
		for (PaymentLineItem payline: paylineItems){
			Assert.assertEquals(payline.getAmount(),newMoney);
		}
		
		
		
		DateRange dateRange = new DateRange();
		Calendar previousYear = DateUtil.getNowCalendar();
		previousYear.set(previousYear.get(Calendar.YEAR)-1,previousYear.get(Calendar.MONDAY)
				, previousYear.get(Calendar.DAY_OF_MONTH));
		
		dateRange.setStartDate(previousYear.getTime());
		dateRange.setEndDate(DateUtil.createCalendar(2009, 2, 15).getTime());
		// Get Payments from the Bean
		List<Payment> capPayments = sEmployerComponent.getPayments(dateRange);
		Assert.assertEquals(capPayments.size(),initPaymentSize+1);
	
	}
	
	@Test
	public void deleteAllPayments() throws ValidationException, ServiceFailureException{
		// Setup start
		Employer employer = sEmployerComponent.getEmployer();
		int initPaymentSize = employer.getPayments().size();
		
		List<Payment> newPayments = new ArrayList<Payment>();
		Payment paymentCASUI = EmployerComponentTestUtils.createPaymentInMemory(PaymentType.CAUI, QboeLawIdConstants.sCASUILawId, new BigDecimal("101.02"));
		Payment paymentFED = EmployerComponentTestUtils.createPaymentInMemory(PaymentType.FORM94x, QboeLawIdConstants.sFICALawId, new BigDecimal("101.02"));
		newPayments.add(paymentCASUI);
		newPayments.add(paymentFED);
		sEmployerComponent.updatePayments(newPayments);
		int paymentSizePostAdd = employer.getPayments().size();
		Assert.assertTrue(paymentSizePostAdd == initPaymentSize+2);
		
		// Setup over
		DateRange dateRange = new DateRange();
		dateRange.setStartDate(DateUtil.createCalendar(2009, 0, 15).getTime());
		dateRange.setEndDate(DateUtil.createCalendar(2009, 2, 15).getTime());
		sEmployerComponent.removePayments(dateRange);
		int paymentSizePostDel = employer.getPayments().size();
		Assert.assertTrue(paymentSizePostDel == initPaymentSize);
		
	}
	
	
	
	
	
	

}
