package com.intuit.sb.payroll.capability.employer;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.transaction.NotSupportedException;
import javax.transaction.SystemException;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerComplianceDocDtl;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

public class EmployerWorkLocationsTest extends EmployerComponentTestBase {

	@Test
	public void getWorkLocationsTest() throws Exception {
		List<WorkLocation> allWorkLocations = sEmployerComponent.findAllWorkLocations();
		for (WorkLocation workLocation : allWorkLocations) {
			sLogger.info(workLocation.getWorkerCount());
		}
	}
	
	@Test
	public void addWorkLocationsTest() throws Exception {
		List<WorkLocation> workLocationsToAdd = new ArrayList<WorkLocation>();
		String newWorkLocationName = "NewWorkLocation";
		WorkLocation newWorkLocation = createWorkLocation(newWorkLocationName);
		workLocationsToAdd.add(newWorkLocation);
		List<WorkLocation> allWorkLocations = sEmployerComponent.addWorkLocations(workLocationsToAdd);
		
		//Find our dude
		boolean found = false;
		for (WorkLocation workLocation : allWorkLocations) {
			if (workLocation.getName().equals(newWorkLocationName)) {
				found = true;
				newWorkLocation = workLocation;
				break;
			}
		}
		
		AssertJUnit.assertTrue("Did not find the WorkLocation named NewWorkLocation", found);
		
		// Ensure that this tests gets added to the Employer also
		Employer employer = sEmployerEAO.findCurrentEmployer();
		Set<WorkLocation> wrkLocSet = employer.getWorkLocations();
		found = false;
		for (WorkLocation empworkLoc : wrkLocSet) {
			if (empworkLoc.getName().equals(newWorkLocationName)) {
				found = true;
				break;
			}
		}
		AssertJUnit.assertTrue("Did not find the WorkLocation associated with the employer", found);
		
    	
    	// Clean up
		List<WorkLocation> locationsToDelete = new ArrayList<WorkLocation>();
		locationsToDelete.add(newWorkLocation);
    	sEmployerComponent.deleteWorkLocations(locationsToDelete);
	}
	
	@Test
	public void updateWorkLocationsTest() throws Exception {
		List<WorkLocation> workLocationsToAdd = new ArrayList<WorkLocation>();
		String newWorkLocationName = "NewWorkLocation";
		WorkLocation newWorkLocation = createWorkLocation(newWorkLocationName);
		workLocationsToAdd.add(newWorkLocation);
		List<WorkLocation> allWorkLocations = sEmployerComponent.addWorkLocations(workLocationsToAdd);
		
		//Find our dude
		boolean found = false;
		for (WorkLocation workLocation : allWorkLocations) {
			if (workLocation.getName().equals(newWorkLocationName)) {
				found = true;
				newWorkLocation = workLocation;
				break;
			}
		}
    	
		AssertJUnit.assertTrue("Did not find the WorkLocation named NewWorkLocation", found);
		
		List<WorkLocation> workLocationsToUpdate = new ArrayList<WorkLocation>();
		String updatedWorkLocationName = "UpdatedWorkLocation";
		newWorkLocation.setName(updatedWorkLocationName );
		workLocationsToUpdate.add(newWorkLocation);
		
		allWorkLocations = sEmployerComponent.updateWorkLocations(workLocationsToUpdate);
		
		//Find our dude
		boolean updated = false;
		for (WorkLocation workLocation : allWorkLocations) {
			if (workLocation.getName().equals(updatedWorkLocationName)) {
				updated = true;
				newWorkLocation = workLocation;
				break;
			}
		}
    	
		AssertJUnit.assertTrue("Did not find the WorkLocation named " + updatedWorkLocationName, updated);
		
    	// Clean up
		List<WorkLocation> locationsToDelete = new ArrayList<WorkLocation>();
		locationsToDelete.add(newWorkLocation);
    	sEmployerComponent.deleteWorkLocations(locationsToDelete);
	}

	@Test
	public void deleteWorkLocationsTest() throws Exception {
		List<WorkLocation> workLocationsToAdd = new ArrayList<WorkLocation>();
		String newWorkLocationName = "NewWorkLocation";
		WorkLocation newWorkLocation = createWorkLocation(newWorkLocationName);
		workLocationsToAdd.add(newWorkLocation);
		List<WorkLocation> allWorkLocations = sEmployerComponent.addWorkLocations(workLocationsToAdd);
		
		//Find our dude
		boolean found = false;
		for (WorkLocation workLocation : allWorkLocations) {
			if (workLocation.getName().equals(newWorkLocationName)) {
				found = true;
				newWorkLocation = workLocation;
				break;
			}
		}
    	
		AssertJUnit.assertTrue("Did not find the WorkLocation named NewWorkLocation", found);
		
		List<WorkLocation> workLocationsToDelete = new ArrayList<WorkLocation>();
		workLocationsToDelete.add(newWorkLocation);
		
		sEmployerComponent.deleteWorkLocations(workLocationsToDelete);
		
		allWorkLocations = sEmployerComponent.findAllWorkLocations();
		
		//Dude should not be in there
		boolean deleted = true;
		for (WorkLocation workLocation : allWorkLocations) {
			if (workLocation.getName().equals(newWorkLocationName)) {
				deleted = false;
				break;
			}
		}
    	
		AssertJUnit.assertTrue("WorkLocation named was not deleted " + newWorkLocationName, deleted);
	}
	
	/**
	 * Test to ensure the working of worklocation add and remove workflow
	 * 1. The test will remove all the worklocation making jurisdiction as empty and 
	 * ensure that EmployerId and SUI Taxs are empty 
	 * @throws SystemException 
	 * @throws NotSupportedException 
	 * @throws Exception
	 */
	@Test
	public void checkWorkLocationRemoveWorkflow() throws NotSupportedException, SystemException{
		List<WorkLocation> allOrgWorkLocations = sEmployerComponent.findAllWorkLocations();
		List<Worker>  workerList = entityService.findAll(Worker.class);
		
		
		List<WorkLocation> allWorkLocations = new ArrayList<WorkLocation>();
		for (WorkLocation workLocation : allOrgWorkLocations) {
			if(workLocation.getName().equals(sMesquiteLocation))  		// can't remove all worklocation, so we will keep Tx one
				continue;
			allWorkLocations.add(workLocation);
			for (Worker worker : workerList) {
				if ((worker.getWorkLocation() != null) && 
					(worker.getWorkLocation().getName().equals(workLocation.getName()))){
					worker.setWorkLocation(null);
				}
			}
		}
		
		try {
			Employer employer = sEmployerEAO.findCurrentEmployer();
			int orgEmployerIdSize = employer.getEmployerIds().size();
			// Remove them
			sEmployerComponent.deleteWorkLocations(allWorkLocations);
			
			AssertJUnit.assertTrue("EmployerId should be reduced", orgEmployerIdSize > employer.getEmployerIds().size());
			
			Set<EmployerComplianceDocDtl> empComplianceSet = employer.getEmployerComplianceDocDtls();
			boolean found = false;
			for (EmployerComplianceDocDtl employerComplianceDocDtl : empComplianceSet) {
				if (employerComplianceDocDtl.getLawId().getValue() == QboeLawIdConstants.sCASUILawId){
					found = true;
				}
			}
			if (found){
				AssertJUnit.fail("State SUI rates not removed when worklocations were removed");
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			AssertJUnit.fail(e.getLocalizedMessage());
		}
	}
	
	
	@Test(expectedExceptions=ValidationException.class)
	public void addNullWorkLocationsTest() throws Exception {
		sEmployerComponent.addWorkLocations(null);
	}
	
	@Test(expectedExceptions=ValidationException.class)
	public void updateNullWorkLocationsTest() throws Exception {
		sEmployerComponent.updateWorkLocations(null);
	}

	@Test(expectedExceptions=ValidationException.class)
	public void deleteNullWorkLocationsTest() throws Exception {
		sEmployerComponent.deleteWorkLocations(null);
	}

	@Test(expectedExceptions=ValidationException.class)
	public void addMissingTaxResidenceWorkLocationsTest() throws Exception {
		WorkLocation workLocation = this.createWorkLocation("EmptyTaxAddress");
		workLocation.setTaxResidenceAddress(null);
		List<WorkLocation> workLocationsToAdd = new ArrayList<WorkLocation>();
		workLocationsToAdd.add(workLocation);
		@SuppressWarnings("unused")
		List<WorkLocation> resultsToIgnore = sEmployerComponent.addWorkLocations(workLocationsToAdd);
	}
	
	@Test(expectedExceptions=ValidationException.class)
	public void addMissingTaxResidenceStateWorkLocationsTest() throws Exception {
		WorkLocation workLocation = this.createWorkLocation("EmptyState");
		//This is ugly
		//workLocation.getTaxResidenceAddress().setRegionOrState(null);
		//Get Global 
		//Convert Global to US
		//Set Property on US
		//Convert from US to Global
		//Set the Global on workLocation
		
		GlobalPostalAddress globalAddr = workLocation.getTaxResidenceAddress();
		USPostalAddress usAddr = AddressConvertor.getUSFromGlobal(
				globalAddr, new USPostalAddressImpl());
		usAddr.setRegionOrState(null);
		globalAddr = AddressConvertor.getGlobalFromUS(usAddr, globalAddr);
		workLocation.setTaxResidenceAddress(globalAddr);		
		List<WorkLocation> workLocationsToAdd = new ArrayList<WorkLocation>();
		workLocationsToAdd.add(workLocation);
		@SuppressWarnings("unused")
		List<WorkLocation> resultsToIgnore = sEmployerComponent.addWorkLocations(workLocationsToAdd);
	}

	@Test(expectedExceptions=ValidationException.class)
	public void addMissingNameWorkLocationsTest() throws Exception {
		WorkLocation workLocation = this.createWorkLocation("EmptyName");
		workLocation.setName(null);
		List<WorkLocation> workLocationsToAdd = new ArrayList<WorkLocation>();
		workLocationsToAdd.add(workLocation);
		@SuppressWarnings("unused")
		List<WorkLocation> resultsToIgnore = sEmployerComponent.addWorkLocations(workLocationsToAdd);
	}
	
	@Test(expectedExceptions=ValidationException.class)
	public void deleteLastWorkLocationTest() throws Exception {
		List<WorkLocation> currentWorkLocations = sEmployerComponent.findAllWorkLocations();
		List<WorkLocation> workLocationsToDelete = new ArrayList<WorkLocation>();
		for (WorkLocation workLocation : currentWorkLocations) {
			workLocationsToDelete.add(workLocation);			
		}
		
		sEmployerComponent.deleteWorkLocations(workLocationsToDelete);
	}

	/**
	 * Creates a basic WorkLocation that should pass muster with the validators.
	 * @param name the Name to assign to this WL.
	 * @return the populated WorkLocation.
	 */
	private WorkLocation createWorkLocation(String name) {
		WorkLocation workLocation = new WorkLocation();
		workLocation.setName(name);
		USPostalAddress taxAddress = WorkerTestUtils.createUSPhysicalAddress();
		workLocation.setTaxResidenceAddress(
				AddressConvertor.getGlobalFromUS(taxAddress, new GlobalPostalAddress()));
		return workLocation;
	}
}
