package com.intuit.sb.payroll.capability.pto;

import java.util.Calendar;
import java.util.Date;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pto.impl.PaidTimeOffValidator;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualStartTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;
import com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum;
import com.intuit.sb.payroll.entity.pto.PtoDatumHelper;

public class PaidTimeOffValidatorTest {

	// @Test(expectedExceptions=ValidationException.class)
	// public void testNullName() throws Exception {
	// PTOParamValidator.validatePtoName(null);
	// }
	//
	// @Test(expectedExceptions=ValidationException.class)
	// public void testEmptyName() throws Exception {
	// PTOParamValidator.validatePtoName("");
	// }
	//
	// @Test(expectedExceptions=ValidationException.class)
	// public void testSpaceName() throws Exception {
	// PTOParamValidator.validatePtoName(" ");
	// }
	//
	// @Test
	// public void testValidName() throws ValidationException {
	// PTOParamValidator.validatePtoName("a");
	// PTOParamValidator.validatePtoName("#");
	// PTOParamValidator.validatePtoName("1");
	// PTOParamValidator.validatePtoName(" _");
	// }

	@Test
	public void testValidPaidTimeOffDtl() throws Exception {
		PaidTimeOffDtl info = createValidPaidTimeOffDtl();
		PaidTimeOffValidator.getInstance().validateEntity(info);
		info.setName("#");
		PaidTimeOffValidator.getInstance().validateEntity(info);
		info.setName(" _ ");
		PaidTimeOffValidator.getInstance().validateEntity(info);

		info.setRate(PtoDatumHelper.createRateDatum(1.0, info.getAccrualType()));
		info.setAccrualType(PaidTimeOffAccrualTypeEnum.HOURLY);
		PaidTimeOffValidator.getInstance().validateEntity(info);
		info.setRate(PtoDatumHelper.createRateDatum(31 * 24.0, info.getAccrualType()));
		info.setAccrualType(PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
		PaidTimeOffValidator.getInstance().validateEntity(info);

		Calendar now = DateUtil.getNowCalendar();
		Date accrualStartDate = DateUtil.createDate(now.get(Calendar.YEAR), now.get(Calendar.MONTH), now
				.get(Calendar.DAY_OF_MONTH));
		info.setAccrualStartDate(accrualStartDate);
		info.setAccrualStartType(PaidTimeOffAccrualStartTypeEnum.ON_START_DATE);
		PaidTimeOffValidator.getInstance().validateEntity(info);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testInvalidStartDate() throws Exception {
		PaidTimeOffDtl info = createValidPaidTimeOffDtl();
		info.setAccrualStartDate(null);
		info.setAccrualStartType(PaidTimeOffAccrualStartTypeEnum.ON_START_DATE);
		PaidTimeOffValidator.getInstance().validateEntity(info);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testInvalidHourlyRate() throws Exception {
		PaidTimeOffDtl info = createValidPaidTimeOffDtl();
		info.setRate(PtoDatumHelper.createRateDatum(1.000001, info.getAccrualType()));
		info.setAccrualType(PaidTimeOffAccrualTypeEnum.HOURLY);
		PaidTimeOffValidator.getInstance().validateEntity(info);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testInvalidPerPaycheckRate() throws Exception {
		PaidTimeOffDtl info = createValidPaidTimeOffDtl();
		info.setRate(PtoDatumHelper.createRateDatum(31 * 24.0 + 0.00001, info.getAccrualType()));
		info.setAccrualType(PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
		PaidTimeOffValidator.getInstance().validateEntity(info);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testInvalidOnJanuary1stRate() throws Exception {
		PaidTimeOffDtl info = createValidPaidTimeOffDtl();
		info.setRate(PtoDatumHelper.createRateDatum(999.00001, info.getAccrualType()));
		info.setAccrualType(PaidTimeOffAccrualTypeEnum.PER_YEAR_START_OF_YEAR);
		PaidTimeOffValidator.getInstance().validateEntity(info);
	}

	private PaidTimeOffDtl createValidPaidTimeOffDtl() {
		PaidTimeOffDtl info = new PaidTimeOffDtl();
		info.setPaidTimeOffType(PaidTimeOffTypeEnum.SICK);
		info.setAccrualType(PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
		info.setAccrualStartType(PaidTimeOffAccrualStartTypeEnum.IMMEDIATELY);
		info.setName("1");
		info.setRate(PtoDatumHelper.createRateDatum(1.0, info.getAccrualType()));
		return info;
	}
}
