package com.intuit.sb.payroll.capability.pts.adapter.mapper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sbd.payroll.psp.adapters.gemini.TaxPaymentAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.TaxPaymentsAdapterDTO;

public class PaymentsDTOMapperTest extends BasicTestBase {

	@Test
	public void testPaymentsDTOMapper() throws ServiceFailureException{
		// Setup Create Sample Payments
		List<Payment> payments = new ArrayList<Payment>();
		Map<String,BigDecimal> dataMap = new HashMap<String,BigDecimal>();
		
		dataMap.put(QboeLawIdConstants.sFICALawId, new BigDecimal("120.02"));
		dataMap.put(QboeLawIdConstants.sFUTALawId, new BigDecimal("110.02"));
		dataMap.put(QboeLawIdConstants.sCobraPremiumCredit, new BigDecimal("12.02"));
		dataMap.put(QboeLawIdConstants.sCAETTLawId, new BigDecimal("20.02"));
		dataMap.put(QboeLawIdConstants.sCAPITLawId, new BigDecimal("11.12"));
		dataMap.put(QboeLawIdConstants.sCASDILawId, new BigDecimal("5.42"));
		
		
		payments.add(EmployerComponentTestUtils.createPaymentInMemory(PaymentType.FORM94x, 
				    QboeLawIdConstants.sFICALawId, dataMap.get(QboeLawIdConstants.sFICALawId)));
		payments.add(EmployerComponentTestUtils.createPaymentInMemory(PaymentType.FORM940FUTA, 
			    QboeLawIdConstants.sFUTALawId, dataMap.get(QboeLawIdConstants.sFUTALawId)));
		payments.add(EmployerComponentTestUtils.createPaymentInMemory(PaymentType.COBRA, 
			    QboeLawIdConstants.sCobraPremiumCredit, dataMap.get(QboeLawIdConstants.sCobraPremiumCredit)));
		payments.add(EmployerComponentTestUtils.createPaymentInMemory(PaymentType.CAETT, 
			    QboeLawIdConstants.sCAETTLawId, dataMap.get(QboeLawIdConstants.sCAETTLawId)));
		payments.add(EmployerComponentTestUtils.createPaymentInMemory(PaymentType.CAPIT, 
			    QboeLawIdConstants.sCAPITLawId, dataMap.get(QboeLawIdConstants.sCAPITLawId)));
		payments.add(EmployerComponentTestUtils.createPaymentInMemory(PaymentType.CASDI, 
			    QboeLawIdConstants.sCASDILawId, dataMap.get(QboeLawIdConstants.sCASDILawId)));
		
		EmployerPTSWorkflowComponent employerPTSComponent = qaBootstrap.getEjbFromClass(EmployerPTSWorkflowComponent.class);
		TaxPaymentsAdapterDTO taxPayments = TaxPaymentsMapper.mapToTaxPaymentsAdapterDTO(employerPTSComponent.getPTSEmployer(), payments);
		
		Assert.assertNotNull(taxPayments.getSourceCompanyId(), "source companyId is null");
		Assert.assertEquals(taxPayments.getPayments().size(), payments.size());
		
		List<TaxPaymentAdapterDTO> paymentsDTO = taxPayments.getPayments();
		
		
		for(TaxPaymentAdapterDTO taxPaymentDTO: paymentsDTO){
			Assert.assertNotNull(taxPaymentDTO.getAmount());
			Assert.assertNotNull(taxPaymentDTO.getLawId());
			Assert.assertNotNull(taxPaymentDTO.getDate());
			Assert.assertEquals(taxPaymentDTO.getAmount(), dataMap.get(taxPaymentDTO.getLawId()));
		}
	}
}
