package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Date;
import java.util.List;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.pts.entity.ActivationState;
import com.intuit.sb.payroll.pts.entity.ActivationStatusEnum;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;

public class ActivationStatusUpdateHandlerTest extends StatusUpdateHandlerTestBase {
	
	/**
	 * Create ACTIVATION_ACTIVE_ST_RECEIVED Notification and ensure that it is available in the
	 * persistent state
	 * @throws PTSNotificationException
	 */
	@Test(enabled=false, groups={"broken"})
	public void testRegistration() throws PTSNotificationException{

		PTSNotificationRequest ptsNotification = createPTSNotificationRequest(
				PTSNotificationTypeEnum.ACTIVATION_NOTIFICATION, ActivationStatusEnum.ACTIVATION_ACTIVE_ST_RECEIVED);
		Date date = ptsNotification.getTimestamp();
		updateNotificationRequest(ptsNotification);

		List<ActivationState> states = activationStateEAO.findAllInSortedOrder();
		ActivationState newState = null;
		for (ActivationState state : states) {
			if (state.getActivationStatus().equals(ActivationStatusEnum.ACTIVATION_ACTIVE_ST_RECEIVED) &&
					state.getStatusDate().equals(date)) {
				newState = state;
				break;
			}
		}
		
		Assert.assertNotNull(newState);

		/*
		 * Test the task created for ACTIVATION_ACTIVE_ST_RECEIVED 
		 * There should be ONE AND ONLY ONE task that is of status NOTSTARTED
		 */
		TaskManager tManager = qaBootstrap.getEjbFromClass(TaskManager.class);
		List<Task> tasks = tManager.findTasksByClass(EmployerPTSWorkflowTasks.TAX_PAYMENTS_TASKS_CLASS);
		int count = 0;
		for (Task task : tasks) {
			if (task.getName().equals(EmployerPTSWorkflowTasks.ACTIVATION_ACTIVE_TASK) && 
					task.getTaskClass().equals(EmployerPTSWorkflowTasks.TAX_PAYMENTS_TASKS_CLASS) &&
					task.getStatus().equals(TaskStatusEnum.NOTSTARTED)) {
				count++;
			}
		}
		Assert.assertEquals(count, 1, "There should be one ONLY ONE ACTIVATION_ACTIVE_TASK");
}
	
	/**
	 * Simulates the case of Employer ACCEPTING ACTIVATION_ACTIVE notification from PTS<br>
	 * by clicking the OK on the dialog presented.
	 * This will result in:
	 *  -- Sending PTS FDoS notificaiton (not implemented)
	 *  -- Closing ACTIVATION_ACTIVE TO*DO task
	 *  -- Creating ACTIVATION_ACTIVE_ACCEPTED TO*DO task
	 * @throws Exception
	 */
	@Test (enabled=false, dependsOnMethods = {"testRegistration"})
	public void testActivationAcceptedWorkflow() throws Exception {

		BankAccount[] bankAccts = sEmployerComponent.getBankAccounts();
		if (bankAccts.length == 0){
			BankAccount ba = new BankAccount();
			ba.setAccountNo("1239089");
			ba.setRoutingNo("110000000");
			ba.setAccountType(BankAccountTypeEnum.SAVINGS);
			ba.setName("Test Account");
			ba.setBankName("Test bank");
			sEmployerComponent.addBankAccounts(new BankAccount[]{ba});
		}
		
		
		employerPTSComponent.updateEmployerTaxServiceState(TaxServiceStatusEnum.ACTIVATION_ACTIVE_ST_ACCEPTED);
		
		/*
		 * Test the task created for ACTIVATION_ACTIVE_ACCEPTED_TASK 
		 * There should be ONE AND ONLY ONE task that is of status NOTSTARTED
		 */
		TaskManager tManager = qaBootstrap.getEjbFromClass(TaskManager.class);
		List<Task> tasks = tManager.findTasksByClass(EmployerPTSWorkflowTasks.TAX_PAYMENTS_TASKS_CLASS);
		int count = 0;
		for (Task task : tasks) {
			if (task.getName().equals(EmployerPTSWorkflowTasks.ACTIVATION_ACTIVE_ACCEPTED_TASK) && 
					task.getStatus().equals(TaskStatusEnum.NOTSTARTED)) {
				count++;
			}
		}
		
		Assert.assertEquals(count, 1, "There should be one ONLY ONE ACTIVATION_ACTIVE_ACCEPTED_TASK");
		
		
		/*
		 * Test the task created for ACTIVATION_ACTIVE_ST_RECEIVED to be CLOSED
		 * There should be NO task that is of status NOTSTARTED
		 */
		tasks = tManager.findTasksByClass(EmployerPTSWorkflowTasks.TAX_PAYMENTS_TASKS_CLASS);
		count = 0;
		for (Task task : tasks) {
			if (task.getName().equals(EmployerPTSWorkflowTasks.ACTIVATION_ACTIVE_TASK) && 
					task.getStatus().equals(TaskStatusEnum.NOTSTARTED)) {
				count++;
			}
		}
		
		Assert.assertEquals(count, 0, "There should be one NO ACTIVATION_ACTIVE_TASK");
	}
	
	/**
	 * This test will ensure that we have status change worker for all the defined status
	 */
	@Test(enabled=false, groups={"broken"})
	public void testActivationStatusHandler(){
		ActivationStatusEnum[] status = ActivationStatusEnum.values();
		for (ActivationStatusEnum activationStatusEnum : status) {
			if (activationStatusEnum.equals(ActivationStatusEnum.NOTSET)){
				// skip
				continue;
			}
			PTSNotificationRequest ptsNotificationRequest = createPTSNotificationRequest(
									PTSNotificationTypeEnum.ACTIVATION_NOTIFICATION, activationStatusEnum);
			try {
				   updateNotificationRequest(ptsNotificationRequest);
				} catch (PTSNotificationException e) {
				Assert.fail(e.getLocalizedMessage(), e);
			}
		}
		
	}
}
