package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.InteractionEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;



/**
 * Tests for handling EFPTSEnrollmentRejection notifications (ex. EIN mismatch rejection,LegalName mismatch rejection)
 * @author vbudania
 *
 */
public class EFPTSEnrollmentRejectionHandlerTest extends StatusUpdateHandlerTestBase{

    //Test if company already exists if not add the company
    @Test(enabled=true)
    public void testCompanyAlreadyAdded() throws ValidationException, ServiceFailureException
    {
        PTSEmployer ptsEmployer = employerPTSComponent.getPTSEmployer();
        employerPTSComponent.updatePPS_PTS_InteractionState(ptsEmployer.getEmployer(), InteractionEnum.ADDED);
        AssertJUnit.assertEquals("Company already added",true,
        		StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer()));
    }

    @Test(enabled=true)
    public void testEINMismatchRejectionReceived() throws ValidationException , ServiceFailureException, PTSNotificationException {
        PTSEmployer  ptsEmployer = employerPTSComponent.getPTSEmployer();
        PTSNotificationRequest ptsNotificationRequest  = createPTSNotificationRequest(PTSNotificationTypeEnum.EFTPS_ENROLLMENT_NOTIFICATION,
                EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH);

        //Send the EIN Mismatch Rejection Notice
        updateNotificationRequest(ptsNotificationRequest);

        EFTPSEnrollmentStatusEnum eftpsEnrollmentStatusEnum =
            StatusUpdateHandlerUtil.getRecentlyReceivedEFTPSRejection().getEftpsEnrollmentStatus();
        AssertJUnit.assertEquals("EIN Mismamatch Rejection not Recevied",
                EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH, eftpsEnrollmentStatusEnum);

        //Test if EINMismatch task got created
        List<Task> rejectedEINMismatch = taskEAO.findTaskByTaskNameAndPayLoad(EmployerPTSWorkflowTasks.REJECTED_EIN_MISMATCH_TASK, ptsEmployer.getEmployer().getId());
        AssertJUnit.assertEquals("EIN Mismatch Rejection not created",
                EmployerPTSWorkflowTasks.REJECTED_EIN_MISMATCH_TASK, rejectedEINMismatch.get(0).getName());

        //Check the status of the task
        AssertJUnit.assertEquals("Task was already closed",TaskStatusEnum.NOTSTARTED, rejectedEINMismatch.get(0).getStatus());

        //Update the EmployerId
        
        
       	List<EmployerId> employerIdList = new ArrayList<EmployerId>();
       	Set<EmployerId> empIdSet = ptsEmployer.getEmployer().getEmployerIds();
       	for(EmployerId empId: empIdSet){
       		employerIdList.add(empId);
       	}
       	
		empTaxSettingComponent.updateEmployerIds(employerIdList);

        //Test if EINMismatch task got closed
        AssertJUnit.assertEquals("Task didnot get closed",TaskStatusEnum.COMPLETED,rejectedEINMismatch.get(0).getStatus());

        //Test if PrintandResubmitForms Task got created
        List<Task> printandResubmitTask = taskEAO.findTaskByTaskNameAndPayLoad(EmployerPTSWorkflowTasks.PRINT_AND_RESUBMIT_FORMS, ptsEmployer.getEmployer().getId());
        AssertJUnit.assertEquals("Print and Resubmit Forms not created",EmployerPTSWorkflowTasks.PRINT_AND_RESUBMIT_FORMS, printandResubmitTask.get(0).getName());

        //Check the status of the task
        AssertJUnit.assertEquals("Task was already closed",TaskStatusEnum.NOTSTARTED,printandResubmitTask.get(0).getStatus());
    }
}