package com.intuit.sb.payroll.capability.taxes;

import java.util.ArrayList;

import org.testng.AssertJUnit;

import org.apache.log4j.Logger;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.common.PayrollTestBase;
import com.intuit.sb.payroll.capability.taxes.impl.PayrollTaxRules;
import com.intuit.sb.payroll.capability.taxes.impl.RulesEmployee;
import com.intuit.sb.payroll.capability.taxes.impl.RulesEmployer;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;

public class ApplicableTaxesTest extends PayrollTestBase
{
    private  static final Logger sLogger = PayrollLoggerFactory.makeLogger(ApplicableTaxesTest.class);

	@Test
    public void testgetApplicableTaxesForEmployeeJurisdiction() throws Exception {
        RulesEmployee employee = new RulesEmployee();
        employee.setWorkCountry("US");
        employee.setWorkState("CA");
        employee.setAgeInYears(18);
        employee.setRelationToOwner(RelationshipToOwnerEnum.CHILD);
        RulesEmployer employer = new RulesEmployer();
        employer.setIsForProfit(false);
        employer.setLegalStructure(LegalStructureEnum.STATUS501C3);      
        ArrayList<Law> applicableTaxes = createCaliforniaWorkerTaxes(); 
        sLogger.info("Laws size before Entering rules=" + applicableTaxes.size());
        AssertJUnit.assertTrue("US-FED-FUI law not found in input", containsLaw("US-FED-FUI", applicableTaxes));        
        applicableTaxes = (ArrayList<Law>)PayrollTaxRules.getApplicableTaxesForEmployee(employee, employer, applicableTaxes);
        AssertJUnit.assertFalse("US-FED-FUI law found in output", containsLaw("US-FED-FUI", applicableTaxes));           
        sLogger.info("Laws size after applying rules=" + applicableTaxes.size());
    }
    
	@Test
    public void testgetApplicableTaxesForEmployerNonProfit() throws Exception {
        RulesEmployee employee = new RulesEmployee();
        employee.setWorkCountry("US");
        employee.setWorkState("CA");
        employee.setAgeInYears(22);
        RulesEmployer employer = new RulesEmployer();
        employer.setIsForProfit(false);
        employer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        ArrayList<Law> applicableTaxes = createCaliforniaWorkerTaxes(); 
        sLogger.info("Laws size before Entering rules=" + applicableTaxes.size());
        AssertJUnit.assertTrue("US-FED-FUI law not found in input", containsLaw("US-FED-FUI", applicableTaxes));        
        applicableTaxes = (ArrayList<Law>)PayrollTaxRules.getApplicableTaxesForEmployee(employee, employer, applicableTaxes);
        AssertJUnit.assertFalse("US-FED-FUI law found in output", containsLaw("US-FED-FUI", applicableTaxes));           
        sLogger.info("Laws size after applying rules=" + applicableTaxes.size());
        
        for(Law tax:applicableTaxes)
        {
        	System.out.println(tax.getName());
        }
    }

	
	//TODO: Laws must implement an equals method?
    private boolean containsLaw(String name, ArrayList<Law> laws)
    {
        boolean found = false;
        for(int i=0; i < laws.size(); i++)
        {
            Law myLaw = laws.get(i);
            if(name.equals(myLaw.getName()))
            {
                found = true;
                break;
            }            
        }
        return found;        
    }

    private ArrayList<Law> createCaliforniaWorkerTaxes()
    {
        ArrayList<Law> applicableTaxes = new ArrayList<Law>();        
        
        Law usSST = new Law();
        usSST.setName("US-FED-SST");
        applicableTaxes.add(usSST);
        
        Law usSSTee = new Law();
        usSSTee.setName("US-FED-SST-EE");
        applicableTaxes.add(usSSTee);
        
        Law usMEDT = new Law();
        usMEDT.setName("US-FED-MEDT");
        applicableTaxes.add(usMEDT);
        
        Law usMEDTee = new Law();
        usMEDTee.setName("US-FED-MEDT-EE");
        applicableTaxes.add(usMEDTee);

        Law usFIT = new Law();
        usFIT.setName("US-FED-FIT");
        applicableTaxes.add(usFIT);
        
        Law usFUI = new Law();
        usFUI.setName("US-FED-FUI");
        applicableTaxes.add(usFUI);

        Law caPIT = new Law();
        caPIT.setName("US-CA-PIT");
        applicableTaxes.add(caPIT);
        
        Law caSDI = new Law();
        caSDI.setName("US-CA-SDI");
        applicableTaxes.add(caSDI);
                
        Law caSUI = new Law();
        caSUI.setName("US-CA-SUI");

        applicableTaxes.add(caSUI);
               
        Law caETT = new Law();
        caETT.setName("US-CA-ETT");
        applicableTaxes.add(caETT);
        return applicableTaxes;
    }
}
