package com.intuit.sb.payroll.capability.taxes.withholding;

import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * <P>
 * The <CODE>AddWorkerComplianceDocDtlTest</CODE> class contains tests for testing the
 * addWorkerComplianceDocDtl() in TaxWithholdingsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class AddWorkerComplianceDocDtlTest extends TaxWithholdingsComponentTestBase {
	private static Worker sEmployee;

    @BeforeMethod
    public static void initializeBeforeTest() {
    	sEmployee = WorkerTestUtils.createEmployee();
    }
    
    @AfterMethod
    public static void cleanUpAfterTest() {
    	sEmployee = null;
    }

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() to add a W4Detail</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the TaxWithholdingsComponent.addWorkerComplianceDocDtl() to add a W4Detail</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a W4Detail is added to the worker
	 * </UL>
	 */
	@Test
	public void testAddW4() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		W4Detail w4Detail = (W4Detail)workerComplianceDocDtl;
		w4Detail.setDerived(false);
		w4Detail.setTaxExempt(false);
		w4Detail.setWithholdingStatus(W4WithholdingStatusEnum.SINGLE);
		w4Detail.setNoOfAllowances(1);
    	// Exercise
		Worker worker = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
		// Verify
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = worker.getWorkerComplianceDocDtls();
		AssertJUnit.assertEquals("Number of ComplianceDocDtls mismatch!", 1, workerComplianceDocDtls.size());
		W4Detail actual = (W4Detail)workerComplianceDocDtls.iterator().next();
		AssertJUnit.assertTrue("ComplianceDocDtl mismatch!", verifyW4Detail(w4Detail, actual));
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() to add an Exempt W4Detail</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the TaxWithholdingsComponent.addWorkerComplianceDocDtl() to add an Exempt W4Detail</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a W4Detail is added to the worker
	 * </UL>
	 */
	@Test
	public void testAddExemptW4() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		W4Detail w4Detail = (W4Detail)workerComplianceDocDtl;
		w4Detail.setDerived(false);
		w4Detail.setTaxExempt(true);
		w4Detail.setWithholdingStatus(null);
		w4Detail.setNoOfAllowances(null);
    	// Exercise
		Worker worker = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
		// Verify
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = worker.getWorkerComplianceDocDtls();
		AssertJUnit.assertEquals("Number of ComplianceDocDtls mismatch!", 1, workerComplianceDocDtls.size());
		W4Detail actual = (W4Detail)workerComplianceDocDtls.iterator().next();
		AssertJUnit.assertTrue("ComplianceDocDtl mismatch!", verifyW4Detail(w4Detail, actual));
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() to add a DE4Detail</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the TaxWithholdingsComponent.addWorkerComplianceDocDtl() to add a DE4Detail</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a DE4Detail is added to the worker
	 * </UL>
	 */
	@Test
	public void testAddDE4() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		DE4Detail de4Detail = (DE4Detail)workerComplianceDocDtl;
		de4Detail.setDerived(false);
		de4Detail.setTaxExempt(false);
		de4Detail.setWithholdingStatus(DE4WithholdingStatusEnum.SINGLE);
		de4Detail.setNoOfAllowances(1);
    	// Exercise
		Worker worker = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, de4Detail);
		// Verify
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = worker.getWorkerComplianceDocDtls();
		AssertJUnit.assertEquals("Number of ComplianceDocDtls mismatch!", 1, workerComplianceDocDtls.size());
		DE4Detail actual = (DE4Detail)workerComplianceDocDtls.iterator().next();
		AssertJUnit.assertTrue("ComplianceDocDtl mismatch!", verifyDE4Detail(de4Detail, actual));
	}

}
