package com.intuit.sb.payroll.capability.taxes.withholding;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerTaxWithholdingRules;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

public class DeriveStateWithholdingFromFederalWithholdingTest {
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(DeriveStateWithholdingFromFederalWithholdingTest.class);


	/**
	 * Two tests: 1. Run with fedWithholding set to true 2. Run with fedWithholding set to false
	 * 
	 */
	@Test
	public void testDeriveDE4FromW4ForSingleInCA() throws Exception {
		/**
		 * Steps 1. Create Federal Withholding Data object 2. Create State Withholding Data object 3. Invoke rules
		 * engine 4. Verify that rules were fired
		 */

		// Step 1
		W4Detail fedWithholding = new W4Detail();
		fedWithholding.setDerived(false);
		fedWithholding.setWithholdingStatus(W4WithholdingStatusEnum.SINGLE);
		Money money = new Money();
		money.setAmount(new BigDecimal("100"));
		fedWithholding.setAdditionalAmount(money);
		fedWithholding.setTaxExempt(false);
		fedWithholding.setNoOfAllowances(Integer.valueOf(2));
		fedWithholding.setDocId(ComplianceDocIdEnum.W4IRS);

		// Step 2
		DE4Detail stateWithholding = new DE4Detail();
		stateWithholding.setDocId(ComplianceDocIdEnum.DE4);
		stateWithholding.setDerived(true);
		
		
		Set<WorkerComplianceDocDtl> compDataCollection = new HashSet<WorkerComplianceDocDtl>();
		compDataCollection.add(fedWithholding);
		compDataCollection.add(stateWithholding);

		// Step 3

		Collection<WorkerComplianceDocDtl> results = WorkerTaxWithholdingRules
				.deriveStateWithholdingFromFederalWithholding(compDataCollection);

		// Get the withHolding from the result
		for (WorkerComplianceDocDtl workerComplianceDocDtl : results) {
			if (workerComplianceDocDtl instanceof DE4Detail) {
				stateWithholding = (DE4Detail) workerComplianceDocDtl;
			} else if (workerComplianceDocDtl instanceof W4Detail) {
				fedWithholding = (W4Detail) workerComplianceDocDtl;
			}
		}

		// Step 4
		dumpWithholdingData("StateWithholding", stateWithholding);
		dumpWithholdingData("FedWithholding", fedWithholding);
		assertRulesFired(fedWithholding, stateWithholding);
	}

	private void assertRulesFired(W4Detail fedWithholdingData, DE4Detail stateWithholdingData) {
		AssertJUnit.assertTrue("Not same WithholdingStatus : ", fedWithholdingData.getWithholdingStatus().toString().equals(
				stateWithholdingData.getWithholdingStatus().toString()));
		AssertJUnit.assertTrue("Not same Additional Amount : ", fedWithholdingData.getAdditionalAmount().equals(
				stateWithholdingData.getAdditionalAmount()));
		// AssertJUnit.assertTrue("Not same ComplianceEffectiveDate : ",fedWithholdingData.getComplianceEffectiveDate().equals(stateWithholdingData.getComplianceEffectiveDate()));
		// AssertJUnit.assertTrue("Not same ComplianceExpirationDate : ",fedWithholdingData.getComplianceExpirationDate().equals(stateWithholdingData.getComplianceExpirationDate()));
		AssertJUnit.assertTrue("Not same isTaxExempt : ",
				fedWithholdingData.isTaxExempt() == (stateWithholdingData.isTaxExempt()));
		AssertJUnit.assertTrue("Not same NumberOfAllowances : ",
				fedWithholdingData.getNoOfAllowances() == (stateWithholdingData.getNoOfAllowances()));
	}

	private void dumpWithholdingData(String name, DE4Detail withholdingData) {
		sLogger.info(name);
		sLogger.info("    WithholdingStatus : " + withholdingData.getWithholdingStatus());
		sLogger.info("    IsDerived : " + withholdingData.isDerived());
		sLogger.info("    Additional Amount : " + withholdingData.getAdditionalAmount());
		// sLogger.info("    ComplianceEffectiveDate : "+withholdingData.getComplianceEffectiveDate());
		// sLogger.info("    ComplianceExpirationDate : "+withholdingData.getComplianceExpirationDate());
		sLogger.info("    isTaxExempt : " + withholdingData.isTaxExempt());
		sLogger.info("    NumberOfAllowances : " + withholdingData.getNoOfAllowances());
		sLogger.info("    NumberOfDeductions : " + withholdingData.getNoOfDeductions());
	}

	private void dumpWithholdingData(String name, W4Detail withholdingData) {
		sLogger.info(name);
		sLogger.info("    WithholdingStatus : " + withholdingData.getWithholdingStatus());
		sLogger.info("    IsDerived : " + withholdingData.isDerived());
		sLogger.info("    Additional Amount : " + withholdingData.getAdditionalAmount());
		// sLogger.info("    ComplianceEffectiveDate : "+withholdingData.getComplianceEffectiveDate());
		// sLogger.info("    ComplianceExpirationDate : "+withholdingData.getComplianceExpirationDate());
		sLogger.info("    isTaxExempt : " + withholdingData.isTaxExempt());
		sLogger.info("    NumberOfAllowances : " + withholdingData.getNoOfAllowances());
	}
}
