package com.intuit.sb.payroll.capability.worker;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 *  <P>The <CODE>DeleteWorkerExceptionsTest</CODE> class contains tests for testing exceptions that can be thrown
 *     by deleteWorker() in WorkerComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class DeleteWorkerExceptionsTest extends WorkerComponentTestBase
{

    /**
     * <P>Test calling deleteWorker() with null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.deleteWorker() with null</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.deleteWorker"}, expectedExceptions=ValidationException.class)
    public void testDeleteWorkerWithNull() throws Exception {
    	// Exercise
    	sWorkerComponent.deleteWorker(null);
    }

    /**
     * <P>Test calling deleteWorker() with new empty Worker object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new empty Worker object</LI>
     * <LI>Call WorkerComponent.deleteWorker() with empty Worker object</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.deleteWorker"}, expectedExceptions=ValidationException.class)
    public void testDeleteWorkerWithNewEmptyWorkerObject() throws Exception {
    	// Setup
    	Worker worker = new Worker();
    	// Exercise
    	sWorkerComponent.deleteWorker(worker);
    }

    /**
     * <P>Test calling deleteWorker() whom has a paycheck</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker</LI>
     * <LI>Add a paycheck to the Worker</LI>
     * <LI>Call WorkerComponent.deleteWorker() with the Worker object</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.deleteWorker"}, expectedExceptions=ValidationException.class)
    public void testDeleteWorkerWithPaychecks() throws Exception {
    	// Setup
    	Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Joe", sMountainViewLocation);
    	worker.addPaycheck(new Paycheck());
    	// Exercise
    	sWorkerComponent.deleteWorker(worker);
    }

}
