package com.intuit.sb.payroll.capability.worker;

import java.util.Date;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.compensation.CompensationComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * EmployeeCrud tests.
 * FIXME these suck.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class EmployeeCRUDTest extends WorkerComponentTestBase
{

    /**
     * <P>Tests Employee Validation through CRUD</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a new Employee</LI>
     * <LI>Set the First Name to be empty (a violation of the validation rules)</LI>
     * <LI>Save the updated employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that the employee does not save, and that the proper exception comes back.</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testEmployeeValidation() throws Exception	{
    	Employee employee = createNewEmployee();
    	employee.getPersonName().setFirstName("");
    	
    	//not valid, firstname is nullable
    	sWorkerComponent.addWorker(employee);
        AssertJUnit.fail("Should not have worked");
	}
	
	@Test(expectedExceptions=ValidationException.class)
	public void testEmployeeHireDateValidation() throws Exception	{
    	Employee employee = createNewEmployee();
    	employee.getEmploymentPeriod().setStartDate(sInvalidHireDate);
    	
    	//not valid, hire date is > 1940 and < 1yr + current date
    	sWorkerComponent.addWorker(employee);
        AssertJUnit.fail("Should not have worked");
	}

	@Test
	public void testAddWorker() throws Exception {
		//Hire them
    	Employee employee = createNewEmployee();
    	employee = (Employee) sWorkerComponent.addWorker(employee);
    	AssertJUnit.assertNotNull(employee);
    	AssertJUnit.assertTrue(employee.getId() > 0);
	}

	@Test
	public void testRehireWorker() throws Exception {
		//Hire them
    	Employee employee = createNewEmployee();
    	employee = (Employee) sWorkerComponent.addWorker(employee);
    	Date terminationDate = DateUtil.getNowDate();
    	//Fire them (this is tested in another method).
		employee = (Employee) sWorkerComponent.terminateWorker(employee, terminationDate);
		//Re-hire them
		employee = (Employee) sWorkerComponent.hireExistingWorker(employee.getId(), terminationDate);
		
    	AssertJUnit.assertTrue("Employee should have been marked active", employee.isActive());
    	AssertJUnit.assertNotNull("Current EmploymentPeriod should have a start date", employee.getEmploymentPeriod().getStartDate());
    	AssertJUnit.assertNull("Current EmploymentPeriod should not have an end date", employee.getEmploymentPeriod().getEndDate());
    	AssertJUnit.assertEquals("Hire date is not the same as that set", sHireDate, employee.getEmploymentPeriod().getStartDate());
    	AssertJUnit.assertEquals("Employee was not set to EMPLOYED", EmploymentStatusEnum.EMPLOYED, employee.getEmploymentStatus());
    }
	
	@Test()
	public void testDeleteSimpleWorker() throws Exception {
		//Hire them
    	Employee employee = createNewEmployee();
    	employee = (Employee) sWorkerComponent.addWorker(employee);
    	Long workerId = employee.getId();
		sWorkerComponent.deleteWorker(employee);
		//Should not find it.
		Worker worker = entityService.findById(Worker.class, workerId);
		AssertJUnit.assertNull(worker);
	}
	
	public void testDeleteModeratelyComplexWorker() throws Exception {
		//Hire them
    	Employee employee = createNewEmployee();
    	employee = (Employee) sWorkerComponent.addWorker(employee);
    	CompensationComponent compensationComponent = qaBootstrap.getEjbFromClass(CompensationComponent.class);
    	CompensationDtl compDtl = compensationComponent.createCompensationDtl(CompensationTypeEnum.SALARYREG);
    	compDtl.setName("DeleteMe");
    	employee = (Employee) compensationComponent.addCompensationToWorker(compDtl, employee);
		// Create an address
		GlobalPostalAddress taxResidenceGlobalAddress = new GlobalPostalAddress();
		taxResidenceGlobalAddress.setCountrySubDivisionCode("CA");
		employee.setTaxResidenceAddress(taxResidenceGlobalAddress);

		// Add a work location
		WorkLocation workLocation = new WorkLocation();
		workLocation.setName("DeleteMe");
		workLocation.setTaxResidenceAddress(taxResidenceGlobalAddress);
		employee.setWorkLocation(workLocation);
        // This adds a default withholding
        employee = (Employee) sWorkerComponent.updateWorker(employee);
    	Long workerId = employee.getId();
		sWorkerComponent.deleteWorker(employee);
		//Should not find it.
		Worker worker = entityService.findById(Worker.class, workerId);
		AssertJUnit.assertNull(worker);
	}
	
	@Test(expectedExceptions={ValidationException.class})
	public void testDeleteWorkerWithPaycheck() throws Exception {
		//Hire them
    	Employee employee = createNewEmployee();
    	employee = (Employee) sWorkerComponent.addWorker(employee);
    	employee.addPaycheck(new Paycheck());
    	Long workerId = employee.getId();
		sWorkerComponent.deleteWorker(employee);
		//Should not find it.
		Worker worker = entityService.findById(Worker.class, workerId);
		AssertJUnit.assertNull(worker);
	}
}
