package com.intuit.sb.payroll.capability.worker;

import java.util.Calendar;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.impl.WorkerValidator;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.PersonName;

public class EmployeeValidationsTest {
	
	/**
	 * Creates an employee with minimum required attributes.
	 * Tests that a null employee throws a ValidationException.
	 * @throws Exception
	 */
	@Test(expectedExceptions=ValidationException.class)
	public void testNullEmployeeValidation() throws Exception {
		Employee employee = new Employee();	
		employee = null;
		WorkerValidator.getInstance().validateEntity(employee);
	}

	/**
	 * No exceptions expected.
	 * @throws Exception
	 */
	@Test
	public void testEmployeeValidation() throws Exception {
		Employee employee = setRequiredAttributesOfEmployee();
		WorkerValidator.getInstance().validateEntity(employee);
	}
	
	/**
	 * Creates an employee with minimum required attributes.
	 * Tests that a null employee person name throws a ValidationException.
	 * @throws Exception
	 */
	@Test(expectedExceptions=ValidationException.class)
	public void testNullEmployeePersonNameValidation() throws Exception {
		Employee employee = setRequiredAttributesOfEmployee();
		employee.setPersonName(null);
		WorkerValidator.getInstance().validateEntity(employee);
	}
	
	/**
	 * Creates an employee with minimum required attributes.
	 * Tests that a blank employee first name throws a ValidationException.
	 * @throws Exception
	 */
	@Test(expectedExceptions=ValidationException.class)
	public void testBlankEmployeeFirstNameValidation() throws Exception {
		Employee employee = setRequiredAttributesOfEmployee();
		employee.getPersonName().setFirstName("");
		WorkerValidator.getInstance().validateEntity(employee);
	}
	
	/**
	 * Creates an employee with minimum required attributes.
	 * Tests that an invalid hire date throws a ValidationException.
	 * @throws Exception
	 */
	@Test(expectedExceptions=ValidationException.class)
	public void testInvalidEmployeeHireDateValidation() throws Exception {
		Employee employee = setRequiredAttributesOfEmployee();
		employee.getEmploymentPeriod().setStartDate(DateUtil.createDate(1940, Calendar.DECEMBER, 31));
		WorkerValidator.getInstance().validateEntity(employee);
	}
	
	/**
	 * Create an employee with all the minimum required attributes. 
	 * @return 
	 */
	private Employee setRequiredAttributesOfEmployee() {
		Employee employee = new Employee();	
		PersonName personName = new PersonName();
		personName.setLastName("Employee");
		personName.setFirstName("Real");
		employee.setPersonName(personName);
		DateRange employmentPeriod = new DateRange();
		employmentPeriod.setStartDate(DateUtil.createDate(1941, Calendar.DECEMBER, 31));
		employmentPeriod.setEndDate(null);
		employee.setEmploymentPeriod(employmentPeriod);
		return employee;		
	}
}
