package com.intuit.sb.payroll.capability.worker;

import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.WorkLocation;

public class WorkerWorkLocationsTest extends WorkerComponentTestBase {

	@Test
	public void assignWorkLocationToWorkerWithNoWorkLocationNoTaxes() throws Exception {
		Employee employee = this.createNewEmployee();
		
		String workLocationName = "WorkerAssignmentLocation";
		WorkLocation workLocation = WorkerTestUtils.createWorkLocation(workLocationName);
		employee = entityService.makePersistent(employee);

		workLocation = entityService.makePersistent(workLocation);
		employee = (Employee) sWorkerComponent.assignWorkLocationToWorker(workLocation, employee);
		AssertJUnit.assertNotNull("Worker WorkLocation is null", employee.getWorkLocation());
		AssertJUnit.assertTrue("Worklocation was not the same as assigned", employee.getWorkLocation().getName().equals(workLocationName));
		Set<TaxDtl> taxes = employee.getTaxDtls();
		AssertJUnit.assertNotNull("Worker Taxes are null", taxes);
		AssertJUnit.assertTrue("Expected more than 0 taxes to be set", taxes.size() > 0);
	}
	
	@Test
	public void assignWorkLocationToWorkerWithWorkLocation() throws Exception {
		//Create and save a workLocation.
		String originalWorkLocationName = "OriginalWorkerAssignmentLocation";
		WorkLocation originalWorkLocation = WorkerTestUtils.createWorkLocation(originalWorkLocationName);
		originalWorkLocation = entityService.makePersistent(originalWorkLocation);

		//Create and save a workLocation.
		String newWorkLocationName = "NewWorkerAssignmentLocation";
		WorkLocation newWorkLocation = WorkerTestUtils.createWorkLocation(newWorkLocationName);
		newWorkLocation = entityService.makePersistent(newWorkLocation);
		
		//Create and save an Employee
		Employee employee = this.createNewEmployee();
		employee.setWorkLocation(originalWorkLocation);
		employee = (Employee) sWorkerComponent.addWorker(employee);


		employee = (Employee) sWorkerComponent.assignWorkLocationToWorker(newWorkLocation, employee);
		
		//AssertJUnit!
		AssertJUnit.assertNotNull("Worker WorkLocation is null", employee.getWorkLocation());
		AssertJUnit.assertTrue("Worklocation was not the same as assigned", employee.getWorkLocation().getName().equals(newWorkLocationName));
		Set<TaxDtl> taxes = employee.getTaxDtls();
		AssertJUnit.assertNotNull("Worker Taxes are null", taxes);
		AssertJUnit.assertTrue("Expected more than 0 taxes to be set", taxes.size() > 0);
	}

	@Test
	public void assignNullWorkLocationToWorker() throws Exception {
		//The Worker is not ready for tax in the first place, so no taxes should exist.
		Employee employee = this.createNewEmployee();
		employee = (Employee) sWorkerComponent.addWorker(employee);

		employee = (Employee) sWorkerComponent.assignWorkLocationToWorker(null, employee);
		AssertJUnit.assertNull("Worker WorkLocation is not null", employee.getWorkLocation());
		Set<TaxDtl> taxes = employee.getTaxDtls();
		AssertJUnit.assertNotNull("Worker Taxes are null", taxes);
		AssertJUnit.assertTrue("Expected 0 taxes to be set", taxes.size() == 0);		
	}

	@Test(expectedExceptions=ValidationException.class)
	public void assignWorkLocationToNullWorker() throws Exception {
		sWorkerComponent.assignWorkLocationToWorker(null, null);
	}

	@Test
	public void assignWorkLocationToWorkerNotReadyForTax() throws Exception {

	}
}
