package com.intuit.sb.payroll.capability.worker.evaluator;

import java.math.BigDecimal;
import java.util.Date;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.PersonTaxId;
import com.intuit.sb.payroll.entity.model.PersonTaxIdEnum;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

public class EvaluatorTestBase extends BasicTestBase
{
    
    
    public Worker createWorkerWithHireDateAndWorkLocation() 
    {

        Employee employee = new Employee();
        Worker worker = (Worker) employee;
        
        DateRange dates = new DateRange();
        dates.setStartDate(new Date());
        dates.setEndDate(null);
        worker.setEmploymentPeriod(dates);
        
        WorkLocation worklocation = new WorkLocation();
        worklocation.setName("A Name to Remember");
        worklocation.setEmailAddress(WorkerTestUtils.createEmailAddress());
        USPostalAddress address = WorkerTestUtils.createUSPhysicalAddress();
        worklocation.setTaxResidenceAddress(
        		AddressConvertor.getGlobalFromUS(address, new GlobalPostalAddress()));
        worklocation.setMainPhone(WorkerTestUtils.CreatePhone());
        
        worklocation = entityService.makePersistent(worklocation);
        worker.setWorkLocation(worklocation);
        
        associateplatformEntitiesWithWorker(worker);        

        return worker;
    }
    
    //FIXME [Test] Ensure we are bypassing the component business logic on purpose.
    protected Worker saveWorker(Worker worker) throws Exception
    {        
        worker = entityService.makePersistent(worker);
        return worker;
    }
    
    protected Worker setWorkerWithHolding(Worker worker) throws ServiceFailureException
    {
        W4Detail employeeWithholdingData = new W4Detail();
        employeeWithholdingData.setDocId(ComplianceDocIdEnum.W4IRS);
        employeeWithholdingData.setTaxExempt(false);
        employeeWithholdingData.setNoOfAllowances(3);
        Money amount = new Money();
        amount.setAmount(new BigDecimal("500.0"));
        employeeWithholdingData.setAdditionalAmount(amount);
        employeeWithholdingData.setWithholdingStatus(W4WithholdingStatusEnum.MARRIED);
        employeeWithholdingData.setDocId(ComplianceDocIdEnum.W4IRS);
        worker.addWorkerComplianceDocDtl(employeeWithholdingData);

        if (worker.getPersonTaxId() != null)
        {
            worker.getPersonTaxId().setValue("655-567-4587");
        } 
        else
        {
            PersonTaxId personTaxId = new PersonTaxId();
            personTaxId.setValue("655-567-4587");
            personTaxId.setType(PersonTaxIdEnum.SSN);
            worker.setPersonTaxId(personTaxId);
        }
        return worker;
    }
    
    protected Worker setDE4ForWorker(Worker worker)
    {
        DE4Detail de4Detail = new DE4Detail();
        de4Detail.setDocId(ComplianceDocIdEnum.DE4);
        //States are derived
        de4Detail.setDerived(true);
        de4Detail.setTaxExempt(false);
        de4Detail.setNoOfAllowances(0);
        // Purposely don't set the deductions field
        worker.addWorkerComplianceDocDtl(de4Detail);
        return worker;
    }
    
    protected Worker setEmptyDE4ForWorker(Worker worker)
    {
        DE4Detail de4Detail = new DE4Detail();
        de4Detail.setDocId(ComplianceDocIdEnum.DE4);
        //States are derived
        de4Detail.setDerived(false);
        de4Detail.setTaxExempt(false);
        de4Detail.setNoOfAllowances(null);
        // Purposely don't set the deductions field
        worker.addWorkerComplianceDocDtl(de4Detail);
        return worker;
    }
    
    private void associateplatformEntitiesWithWorker(Worker worker)
    {
        //set the personal details
    	PersonName personName = new PersonName();
    	personName.setFirstName("The Dude");
    	personName.setMiddleName("Junior");
    	personName.setLastName("Last Name");
    	worker.setPersonName(personName);
        worker.setDateOfBirth(DateUtil.createDate(1935, 1, 8)); //Elvis' birthday       

        USPostalAddress homeAddress = new USPostalAddressImpl();
        homeAddress.setAddressLine1("721 Old county Road");
        homeAddress.setAddressLine2("Apt K");
        homeAddress.setCity("Belmont");
        homeAddress.setZipOrPostalCode("94002");
        homeAddress.setRegionOrState("CA");
        homeAddress.setCountry("USA");
        homeAddress.setCountryISOCode("US");
        worker.setTaxResidenceAddress(
        		AddressConvertor.getGlobalFromUS(homeAddress, new GlobalPostalAddress()));

        TelephoneNumber homeTelephone = new TelephoneNumber();
        homeTelephone.setFreeFormNumber("650333444");
        worker.setHomePhone(homeTelephone);
           
        EmailAddress email = new EmailAddress();
        email.setAddress("blah@blah.com");
        worker.setEmailAddress(email);
    }
}
