package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;

/**
 * <P>
 * The <CODE>FindPayrollRunTest</CODE> class contains tests for testing the
 * findPayrollRun() in WorkerPaymentsComponent
 * </P>
 * 
 * $Author: pszeto $ $Revision: #13 $ $DateTime: 2009/05/06 16:58:06 $
 * 
 */
public class FindRecentPayrollRunsForYearTest extends WorkerPaymentsComponentTestBase {

	@BeforeClass
	public static void setUpFindPayrollRunTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpFindPayrollRunTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownFindPayrollRunTest() {
		try {
			getUserTransaction().begin();
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownFindPayrollRunTest(): " + t.getMessage());
		}
	}

	@Test
	public void testFindWithNoPayRuns() throws Exception {
     	//Exercise
    	List<PayrollRun> savedPayrollRuns = sWorkerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.SCHEDULED, 2008);

    	//Verify
    	AssertJUnit.assertEquals("expected pay runs returned", 0, savedPayrollRuns.size());
	}
	@Test(expectedExceptions=ValidationException.class)
	public void testFindZeroPayRuns() throws Exception {
       	//Exercise
      	sWorkerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.SCHEDULED, 0);
	}
	@Test
	public void testFindWithOnePayRun() throws Exception {
    	List<PayrollRun> payrollRuns = new ArrayList<PayrollRun>();
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);

        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
         	
        	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	
         	//Exercise
        	List<PayrollRun> savedPayrollRuns = sWorkerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.SCHEDULED, 2008);

        	//Verify
        	AssertJUnit.assertEquals("expected pay runs returned", 1, savedPayrollRuns.size());
    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().begin();
            }
        	//Tear down
    		for (PayrollRun payrollRun : payrollRuns) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}
	@Test
	public void testFindWithOneSchedulePayRunOfTwoTotalRuns() throws Exception {
    	List<PayrollRun> payrollRuns = new ArrayList<PayrollRun>();
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);

        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.ADHOC));
         	
        	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	
         	//Exercise
        	List<PayrollRun> savedPayrollRuns = sWorkerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.SCHEDULED, 2008);

        	//Verify
        	AssertJUnit.assertEquals("expected pay runs returned", 1, savedPayrollRuns.size());
        	PayrollRun savedPayrollRun = savedPayrollRuns.get(0);
        	AssertJUnit.assertEquals("run is scheduled", PayrollRunTypeEnum.SCHEDULED, savedPayrollRun.getTypeOf());
    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().begin();
            }
        	//Tear down
    		for (PayrollRun payrollRun : payrollRuns) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}
	@Test
	public void testFindWithOneSchedulePayRunOfTwoScheduledRuns() throws Exception {
    	List<PayrollRun> payrollRuns = new ArrayList<PayrollRun>();
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);

        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
         	
        	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	
         	//Exercise
        	List<PayrollRun> savedPayrollRuns = sWorkerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.SCHEDULED, 2008);

        	//Verify
        	AssertJUnit.assertEquals("expected pay runs returned", 2, savedPayrollRuns.size());
    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().begin();
            }
        	//Tear down
    		for (PayrollRun payrollRun : payrollRuns) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}
	@Test
	public void testFindWithMultiplePayRuns() throws Exception {
    	List<PayrollRun> payrollRuns = new ArrayList<PayrollRun>();
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);

        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
         	
        	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	
         	//Exercise
        	List<PayrollRun> savedPayrollRuns = sWorkerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.SCHEDULED, 2008);

        	//Verify
        	AssertJUnit.assertEquals("expected pay runs returned", 2, savedPayrollRuns.size());
        	AssertJUnit.assertTrue("runs are ordered correctly", savedPayrollRuns.get(0).getPayDate().after(savedPayrollRuns.get(1).getPayDate()));
    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().begin();
            }
        	//Tear down
    		for (PayrollRun payrollRun : payrollRuns) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}
	@Test (enabled = false)
	public void testLimitedFindWithMultipleMixedPayRuns() throws Exception {
    	List<PayrollRun> payrollRuns = new ArrayList<PayrollRun>();
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);

        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.ADHOC));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.ADHOC));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.ADHOC));
         	
        	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	
         	//Exercise
        	List<PayrollRun> savedPayrollRuns = sWorkerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.SCHEDULED, 2008);

        	//Verify
        	AssertJUnit.assertEquals("expected pay runs returned", 3, savedPayrollRuns.size());
        	AssertJUnit.assertEquals("run 0 is scheduled", PayrollRunTypeEnum.SCHEDULED, savedPayrollRuns.get(0).getTypeOf());
        	AssertJUnit.assertEquals("run 1 is scheduled", PayrollRunTypeEnum.SCHEDULED, savedPayrollRuns.get(1).getTypeOf());
        	AssertJUnit.assertEquals("run 2 is scheduled", PayrollRunTypeEnum.SCHEDULED, savedPayrollRuns.get(2).getTypeOf());
        	AssertJUnit.assertTrue("runs 0 & 1 are ordered correctly", savedPayrollRuns.get(0).getPayDate().after(savedPayrollRuns.get(1).getPayDate()));
        	AssertJUnit.assertTrue("runs 1 & 2 are ordered correctly", savedPayrollRuns.get(1).getPayDate().after(savedPayrollRuns.get(2).getPayDate()));
    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().begin();
            }
        	//Tear down
    		for (PayrollRun payrollRun : payrollRuns) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}
	@Test
	public void testFindWithMultipleMixedPayRuns() throws Exception {
    	List<PayrollRun> payrollRuns = new ArrayList<PayrollRun>();
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);

        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.ADHOC));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.ADHOC));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.ADHOC));
        	payrollRuns.add(createPayrollRun(PayrollRunTypeEnum.SCHEDULED));
         	
        	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	
         	//Exercise
        	List<PayrollRun> savedPayrollRuns = sWorkerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.SCHEDULED, 2008);

        	//Verify
        	AssertJUnit.assertEquals("expected pay runs returned", 2, savedPayrollRuns.size());
        	AssertJUnit.assertEquals("run 0 is scheduled", PayrollRunTypeEnum.SCHEDULED, savedPayrollRuns.get(0).getTypeOf());
        	AssertJUnit.assertEquals("run 1 is scheduled", PayrollRunTypeEnum.SCHEDULED, savedPayrollRuns.get(1).getTypeOf());
//        	AssertJUnit.assertEquals("run 2 is scheduled", PayrollRunTypeEnum.SCHEDULED, savedPayrollRuns.get(2).getTypeOf());
//        	AssertJUnit.assertEquals("run 3 is scheduled", PayrollRunTypeEnum.SCHEDULED, savedPayrollRuns.get(3).getTypeOf());
//        	AssertJUnit.assertEquals("run 4 is scheduled", PayrollRunTypeEnum.SCHEDULED, savedPayrollRuns.get(4).getTypeOf());
    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().begin();
            }
        	//Tear down
    		for (PayrollRun payrollRun : payrollRuns) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}
	
	private PayrollRun createPayrollRun(PayrollRunTypeEnum runType) throws Exception {
        PayrollRunInfo payrollRunInfo = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
        payrollRunInfo.setPayrollRunType(runType);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
    	return sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}
}
