package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Calendar;
import java.util.Date;

import org.testng.annotations.AfterClass;
import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;


/**
 * The <CODE>GetNextScheduledPayrollRunInfoExistingPayrollRunMonthlyTest</CODE> class contains tests for testing the
 * getNextScheduledPayrollRunInfo() in PayrollScheduleComponent using a monthly schedule with existing PayrollRuns
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetNextScheduledPayrollRunInfoExistingPayrollRunMonthlyTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private static Calendar periodEndDate = null;
	private static PayrollRun payrollRun = null;
	private static PayrollSchedule aSchedule = null;
	
    /**
     * <P>One time setup for this test class</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Remember some environment setting so that it can be restored later</LI>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Create a monthly PayrollSchedule</LI>
     * <LI>Create 1 payrollRun</LI>
     * </OL>
     *
     */
	@BeforeClass
	public static void setUpGetNextScheduledPayrollRunInfoExistingPayrollRunMonthlyTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 31);
	    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
	    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
    		payrollRun = createOnePayrollRun(sReadyForPayWorkerId, aSchedule);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpGetNextScheduledPayrollRunInfoExistingPayrollRunMonthlyTest(): " +
			            t.getMessage());
		}
	}

    /**
     * <P>One time tearDown for this test class</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Delete the payrollRun that was created in setUp</LI>
     * <LI>Delete the payroll ready worker that was created in setUp</LI>
     * <LI>Restore the environment settings</LI>
     * </OL>
     *
     */
	@AfterClass
	public static void tearDownGetNextScheduledPayrollRunInfoExistingPayrollRunMonthlyTest() {
		try {
			getUserTransaction().begin();
			if (null != payrollRun) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
			}
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownGetNextScheduledPayrollRunInfoExistingPayrollRunMonthlyTest(): " +
			            t.getMessage());
		}
	}

    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo() throws Exception {
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 4),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 2),
		                                             PayrollFrequencyEnum.MONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, after changing PayrollRunDate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingPayRunDate() throws Exception {
		// Setup
		PayrollSchedule schedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                 sDEFAULT_PAYROLL_RUN_OFFSET+1, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 4),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 3),
		                                             PayrollFrequencyEnum.MONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, after changing PayDayDate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingPayDayDate() throws Exception {
		// Setup
		PayrollSchedule schedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                 sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET-1);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 3),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 2),
		                                             PayrollFrequencyEnum.MONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, NextPayPeriodEndDate in the
     * PayrollSchedule is set before the PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testNextPayPeriodEndDate_Before_PayrollRun() throws Exception {
		// Setup
		Calendar nextPayPeriodEndDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 31);
		PayrollSchedule schedule = createMonthlySchedule(nextPayPeriodEndDate.getTime(),
		                                                 PayrollSchedule.LAST_DAY_OF_MONTH, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	             sDEFAULT_PAY_DAY_OFFSET);
		// Exercise

		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 4),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 2),
		                                             PayrollFrequencyEnum.MONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, NextPayPeriodEndDate in the
     * PayrollSchedule is set to 2 months after the PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testNextPayPeriodEndDate_After_PayrollRun() throws Exception {
		// Setup
		Calendar nextPayPeriodEndDate = DateUtil.createCalendar(2009, Calendar.MARCH, 31);
		PayrollSchedule schedule = createMonthlySchedule(nextPayPeriodEndDate.getTime(),
		                                                 PayrollSchedule.LAST_DAY_OF_MONTH, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	             sDEFAULT_PAY_DAY_OFFSET);
		// Exercise

		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 4),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 2),
		                                             PayrollFrequencyEnum.MONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, NextPayPeriodEndDate in the
     * PayrollSchedule is set to 2 months after the PayrollRun, and also changing the PayrollRunDate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testNextPayPeriodEndDate_After_PayrollRun_And_ChangePayrollRunDate() throws Exception {
		// Setup
		Calendar nextPayPeriodEndDate = DateUtil.createCalendar(2009, Calendar.MARCH, 31);
		PayrollSchedule schedule = createMonthlySchedule(nextPayPeriodEndDate.getTime(),
		                                                 PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                 sDEFAULT_PAYROLL_RUN_OFFSET+1, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise

		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 4),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 3),
		                                             PayrollFrequencyEnum.MONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing monthly PayrollRun after changing pay
     *  frequency to a weekly schedule, next pay period end date is exactly 1 week from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a weekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingToWeekly_with_Exact_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 7);
		PayrollSchedule schedule = createWeeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	            sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 7)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 11),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 9),
		                                             PayrollFrequencyEnum.WEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing monthly PayrollRun after changing pay
     *  frequency to a weekly schedule, next pay period end date is less than 1 week from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a weekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToWeekly_with_Shorter_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 6);
		PayrollSchedule schedule = createWeeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	            sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 6)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 11),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 9),
		                                             PayrollFrequencyEnum.WEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing monthly PayrollRun after changing pay
     *  frequency to a weekly schedule, next pay period end date is more than 1 week from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a weekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToWeekly_with_Longer_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 8);
		PayrollSchedule schedule = createWeeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	            sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 8)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 11),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 9),
		                                             PayrollFrequencyEnum.WEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing monthly PayrollRun after changing pay
     *  frequency to a biweekly schedule, next pay period end date is exactly 2 weeks from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a biweekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingToBiweekly_with_Exact_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 14);
		PayrollSchedule schedule = createBiweeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	              sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 14)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
		                                             PayrollFrequencyEnum.BIWEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing monthly PayrollRun after changing pay
     *  frequency to a biweekly schedule, next pay period end date is less than 2 weeks from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a biweekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToBiweekly_with_Shorter_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 13);
		PayrollSchedule schedule = createBiweeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	              sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 13)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
		                                             PayrollFrequencyEnum.BIWEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing monthly PayrollRun after changing pay
     *  frequency to a biweekly schedule, next pay period end date is more than 2 weeks from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a biweekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToBiweekly_with_Longer_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 16);
		PayrollSchedule schedule = createBiweeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	              sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 16)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
		                                             PayrollFrequencyEnum.BIWEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing monthly PayrollRun after changing pay
     *  frequency to a semimonthly schedule; Pay period ends on the 15th and the last day of the month, next pay period
     *  end date is exactly a full pay period</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a semimonthly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingToSemimonthly_with_Exact_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 15);
		PayrollSchedule schedule = createSemimonthlySchedule(nextPeriodEndDate, 15, PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                     sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 15)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
		                                             PayrollFrequencyEnum.SEMIMONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing monthly PayrollRun after changing pay
     *  frequency to a semimonthly schedule; Pay period ends on the 12th and the 27th of the month, next pay period
     *  end date is less than a full pay period</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a semimonthly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToSemimonthly_with_Shorter_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 12);
		PayrollSchedule schedule = createSemimonthlySchedule(nextPeriodEndDate, 12, 27,
		                                                     sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 12)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 18),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 13),
		                                             PayrollFrequencyEnum.SEMIMONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing monthly PayrollRun after changing pay
     *  frequency to a semimonthly schedule; Pay period ends on the 3rd and the 18th of the month, next pay period
     *  end date is more than a full pay period</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a semimonthly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToSemimonthly_with_Longer_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 18);
		PayrollSchedule schedule = createSemimonthlySchedule(nextPeriodEndDate, 3, 18,
		                                                     sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 18)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 23),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
		                                             PayrollFrequencyEnum.SEMIMONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
}
