package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>GetPaycheckByIdTest</CODE> class contains tests for testing the
 * getPaycheckById() in WorkerPaymentsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class GetPaycheckByIdTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private BigDecimal mHourlyRate = new BigDecimal("10.0");
	private BigDecimal mBonusAmount = new BigDecimal("200.0");

	@BeforeClass
	public static void setUpGetPaycheckByIdTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpGetPaycheckByIdTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownGetPaycheckByIdTest() {
		try {
			getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownGetPaycheckByIdTest(): " + t.getMessage());
		}
	}

	private PayrollRun createAPayrollRun() throws Exception {
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
      	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
      	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
      					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
      					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
      	paycheckCalculationRequests.add(calcRequest);
      	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
      	PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
      	return payrollRun;
	}
	
	/**
	 * <P>Test calling getPaycheckById() with a null PaycheckId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.getPaycheckById() with a null PaycheckId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPaycheckId() throws Exception {
    	//Exercise
		@SuppressWarnings("unused")
		Paycheck paycheck = sWorkerPaymentsComponent.getPaycheckById(null);
	}

	/**
	 * <P>Test calling getPaycheckById() with a non-existent PaycheckId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.getPaycheckById() with a non-existent PaycheckId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions=ServiceFailureException.class)
	public void testGetPaycheckByIdWithNonExistentId() throws Exception {
    	//Exercise
		@SuppressWarnings("unused")
		Paycheck paycheck = sWorkerPaymentsComponent.getPaycheckById(324983549L);
	}

	/**
	 * <P>Test calling getPaycheckById() with a valid PaycheckId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.getPaycheckById() with a valid PaycheckId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the paycheck that was returned is correct
	 * </UL>
	 */
	@Test
	public void testGetPaycheckById() throws Exception {
		// Setup
		PayrollRun payrollRun = createAPayrollRun();
    	// Exercise
		Paycheck expectedPaycheck = payrollRun.getPaychecks().iterator().next();
		Paycheck paycheck = sWorkerPaymentsComponent.getPaycheckById(expectedPaycheck.getId());
		AssertJUnit.assertTrue("Paycheck mismatch!", verifyPaycheck(expectedPaycheck, paycheck));
	}

}
