package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Calendar;
import java.util.List;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;


/**
 * The <CODE>GetUpcomingScheduledPayrollRunInfoExceptionsTest</CODE> class contains exception tests for testing
 * getUpcomingScheduledPayrollRunInfo() in PayrollScheduleComponent
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetUpcomingScheduledPayrollRunInfoExceptionsTest extends WorkerPaymentsComponentTestBase {
	
    /**
     * <P>Test calling getUpcomingScheduledPayrollRunInfo() with a null PayrollSchedule</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfo() with a null PayrollSchedule</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayrollSchedule() throws Exception {
		// Exercise
		@SuppressWarnings("unused")
		List<PayrollRunInfo> computedList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(null,
				DateUtil.createDate(2008, Calendar.JANUARY, 1));
	}
	
    /**
     * <P>Test calling getUpcomingScheduledPayrollRunInfo() with a null Date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a weekly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfo() with a null Date</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullDate() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 4);
    	aSchedule = createWeeklySchedule(periodEndDate.getTime(), sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		@SuppressWarnings("unused")
		List<PayrollRunInfo> computedList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(aSchedule,
				null);
	}
	
}
