package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;


/**
 * <P>The <CODE>GetUpcomingScheduledPayrollRunInfoMonthlyTest</CODE> class contains tests for testing
 * getUpcomingScheduledPayrollRunInfo() in PayrollScheduleComponent using a monthly schedule with no existing
 * PayrollRuns.</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetUpcomingScheduledPayrollRunInfoMonthlyTest extends WorkerPaymentsComponentTestBase {
	
    /**
     * <P>Test getting the up-coming PayrollRunInfo, Pay period end date is on the 1st of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfos() to get the up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate1() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 1);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 1, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		List<PayrollRunInfo> computedList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(aSchedule,
				DateUtil.createDate(2009, Calendar.JANUARY, 1));
		// Verify
		List<PayrollRunInfo> expectedList = new ArrayList<PayrollRunInfo>();
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 2),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 1)),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 6),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 2),
		                                    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 2),
		                                                  DateUtil.createDate(2009, Calendar.FEBRUARY, 1)),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                                  DateUtil.createDate(2009, Calendar.MARCH, 1)),
		                                    DateUtil.createDate(2009, Calendar.MARCH, 4),
		                                    DateUtil.createDate(2009, Calendar.MARCH, 2),
		                                    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 2),
										                  DateUtil.createDate(2009, Calendar.APRIL, 1)),
										    DateUtil.createDate(2009, Calendar.APRIL, 6),
										    DateUtil.createDate(2009, Calendar.APRIL, 2),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 2),
										                  DateUtil.createDate(2009, Calendar.MAY, 1)),
										    DateUtil.createDate(2009, Calendar.MAY, 6),
										    DateUtil.createDate(2009, Calendar.MAY, 4),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 2),
										                  DateUtil.createDate(2009, Calendar.JUNE, 1)),
										    DateUtil.createDate(2009, Calendar.JUNE, 4),
										    DateUtil.createDate(2009, Calendar.JUNE, 2),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 2),
										                  DateUtil.createDate(2009, Calendar.JULY, 1)),
										    DateUtil.createDate(2009, Calendar.JULY, 6),
										    DateUtil.createDate(2009, Calendar.JULY, 2),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 2),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 1)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 5),
										    DateUtil.createDate(2009, Calendar.AUGUST, 3),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 2),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 1)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 4),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 2),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 2),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 1)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 6),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 2),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 2),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 1)),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 4),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 2),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 2),
										                  DateUtil.createDate(2009, Calendar.DECEMBER, 1)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 4),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 2),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfoList(expectedList, computedList));
	}
	
    /**
     * <P>Test getting the up-coming PayrollRunInfo, Pay period end date is on the last day of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfos() to get the up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_December() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 31);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    			 						  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		List<PayrollRunInfo> computedList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(aSchedule,
				DateUtil.createDate(2009, Calendar.JANUARY, 1));
		// Verify
		List<PayrollRunInfo> expectedList = new ArrayList<PayrollRunInfo>();
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 1),
		                                        		  DateUtil.createDate(2008, Calendar.DECEMBER, 31)),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 6),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 2),
		                                    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 1),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                    DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                  DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
		                                    DateUtil.createDate(2009, Calendar.MARCH, 4),
		                                    DateUtil.createDate(2009, Calendar.MARCH, 2),
		                                    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 1),
										                  DateUtil.createDate(2009, Calendar.MARCH, 31)),
										    DateUtil.createDate(2009, Calendar.APRIL, 3),
										    DateUtil.createDate(2009, Calendar.APRIL, 1),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 1),
										                  DateUtil.createDate(2009, Calendar.APRIL, 30)),
										    DateUtil.createDate(2009, Calendar.MAY, 5),
										    DateUtil.createDate(2009, Calendar.MAY, 1),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 1),
										                  DateUtil.createDate(2009, Calendar.MAY, 31)),
										    DateUtil.createDate(2009, Calendar.JUNE, 3),
										    DateUtil.createDate(2009, Calendar.JUNE, 1),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 1),
										                  DateUtil.createDate(2009, Calendar.JUNE, 30)),
										    DateUtil.createDate(2009, Calendar.JULY, 3),
										    DateUtil.createDate(2009, Calendar.JULY, 1),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 1),
										                  DateUtil.createDate(2009, Calendar.JULY, 31)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 5),
										    DateUtil.createDate(2009, Calendar.AUGUST, 3),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 1),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 31)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 3),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 1),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 1),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 30)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 5),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 1),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 1),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 31)),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 4),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 2),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 1),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 30)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 3),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 1),
										    PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED));
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfoList(expectedList, computedList));
	}
	
}
