package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;


/**
 * The <CODE>GetUpcomingScheduledPayrollRunInfosExistingPayrollRunWeeklyTest</CODE> class contains tests for testing the
 * getUpcomingScheduledPayrollRunInfos() in PayrollScheduleComponent using a weekly schedule with existing PayrollRuns
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetUpcomingScheduledPayrollRunInfosExistingPayrollRunWeeklyTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private static Calendar periodEndDate = null;
	private static PayrollRun payrollRun = null;
	private static PayrollSchedule aSchedule = null;
	
    /**
     * <P>One time setup for this test class</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Remember some environment setting so that it can be restored later</LI>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Create a weekly PayrollSchedule</LI>
     * <LI>Create 1 payrollRun</LI>
     * </OL>
     *
     */
	@BeforeClass
	public static void setUpGetUpcomingScheduledPayrollRunInfosExistingPayrollRunWeeklyTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 4);
	    	aSchedule = createWeeklySchedule(periodEndDate.getTime(), sDEFAULT_PAYROLL_RUN_OFFSET,
	    	                                 sDEFAULT_PAY_DAY_OFFSET);
    		payrollRun = createOnePayrollRun(sReadyForPayWorkerId, aSchedule);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpGetUpcomingScheduledPayrollRunInfosExistingPayrollRunWeeklyTest(): " +
			            t.getMessage());
		}
	}

    /**
     * <P>One time tearDown for this test class</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Delete the payrollRun that was created in setUp</LI>
     * <LI>Delete the payroll ready worker that was created in setUp</LI>
     * <LI>Restore the environment settings</LI>
     * </OL>
     *
     */
	@AfterClass
	public static void tearDownGetUpcomingScheduledPayrollRunInfosExistingPayrollRunWeeklyTest() {
		try {
			getUserTransaction().begin();
			if (null != payrollRun) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
			}
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownGetUpcomingScheduledPayrollRunInfosExistingPayrollRunWeeklyTest(): " +
			            t.getMessage());
		}
	}

    /**
     * <P>Test getting the up-coming PayrollRunInfos with 1 existing PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getUpcomingScheduledPayrollRunInfos() to get the up-coming PayrollRunInfos</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcoming2009PayrollRunInfos() throws Exception {
		// Exercise
		List<PayrollRunInfo> payrollRunInfoList = pScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(aSchedule,
				DateUtil.createDate(2009, Calendar.JANUARY, 1));
		// Verify
		List<PayrollRunInfo> expectedList = new ArrayList<PayrollRunInfo>();
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 5),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 11)),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 14),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 12),
		                                    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 12),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 18)),
										                  // 1/19/2009 is a holiday
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 22),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 20),
		                                    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 19),
		                                                  DateUtil.createDate(2009, Calendar.JANUARY, 25)),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 28),
		                                    DateUtil.createDate(2009, Calendar.JANUARY, 26),
		                                    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 26),
										                  DateUtil.createDate(2009, Calendar.FEBRUARY, 1)),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
										                  DateUtil.createDate(2009, Calendar.FEBRUARY, 8)),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 11),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 9),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 9),
										                  DateUtil.createDate(2009, Calendar.FEBRUARY, 15)),
										                  // 2/16/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 16),
										                  DateUtil.createDate(2009, Calendar.FEBRUARY, 22)),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 25),
										    DateUtil.createDate(2009, Calendar.FEBRUARY, 23),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 23),
										                  DateUtil.createDate(2009, Calendar.MARCH, 1)),
										    DateUtil.createDate(2009, Calendar.MARCH, 4),
										    DateUtil.createDate(2009, Calendar.MARCH, 2),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 2),
										                  DateUtil.createDate(2009, Calendar.MARCH, 8)),
										    DateUtil.createDate(2009, Calendar.MARCH, 11),
										    DateUtil.createDate(2009, Calendar.MARCH, 9),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 9),
										                  DateUtil.createDate(2009, Calendar.MARCH, 15)),
										    DateUtil.createDate(2009, Calendar.MARCH, 18),
										    DateUtil.createDate(2009, Calendar.MARCH, 16),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 16),
										                  DateUtil.createDate(2009, Calendar.MARCH, 22)),
										    DateUtil.createDate(2009, Calendar.MARCH, 25),
										    DateUtil.createDate(2009, Calendar.MARCH, 23),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 23),
										                  DateUtil.createDate(2009, Calendar.MARCH, 29)),
										    DateUtil.createDate(2009, Calendar.APRIL, 1),
										    DateUtil.createDate(2009, Calendar.MARCH, 30),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 30),
										                  DateUtil.createDate(2009, Calendar.APRIL, 5)),
										    DateUtil.createDate(2009, Calendar.APRIL, 8),
										    DateUtil.createDate(2009, Calendar.APRIL, 6),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 6),
										                  DateUtil.createDate(2009, Calendar.APRIL, 12)),
										    DateUtil.createDate(2009, Calendar.APRIL, 15),
										    DateUtil.createDate(2009, Calendar.APRIL, 13),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 13),
										                  DateUtil.createDate(2009, Calendar.APRIL, 19)),
										    DateUtil.createDate(2009, Calendar.APRIL, 22),
										    DateUtil.createDate(2009, Calendar.APRIL, 20),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 20),
										                  DateUtil.createDate(2009, Calendar.APRIL, 26)),
										    DateUtil.createDate(2009, Calendar.APRIL, 29),
										    DateUtil.createDate(2009, Calendar.APRIL, 27),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 27),
										                  DateUtil.createDate(2009, Calendar.MAY, 3)),
										    DateUtil.createDate(2009, Calendar.MAY, 6),
										    DateUtil.createDate(2009, Calendar.MAY, 4),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 4),
										                  DateUtil.createDate(2009, Calendar.MAY, 10)),
										    DateUtil.createDate(2009, Calendar.MAY, 13),
										    DateUtil.createDate(2009, Calendar.MAY, 11),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 11),
										                  DateUtil.createDate(2009, Calendar.MAY, 17)),
										    DateUtil.createDate(2009, Calendar.MAY, 20),
										    DateUtil.createDate(2009, Calendar.MAY, 18),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 18),
										                  DateUtil.createDate(2009, Calendar.MAY, 24)),
										                  // 5/25/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.MAY, 28),
										    DateUtil.createDate(2009, Calendar.MAY, 26),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 25),
										                  DateUtil.createDate(2009, Calendar.MAY, 31)),
										    DateUtil.createDate(2009, Calendar.JUNE, 3),
										    DateUtil.createDate(2009, Calendar.JUNE, 1),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 1),
										                  DateUtil.createDate(2009, Calendar.JUNE, 7)),
										    DateUtil.createDate(2009, Calendar.JUNE, 10),
										    DateUtil.createDate(2009, Calendar.JUNE, 8),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 8),
										                  DateUtil.createDate(2009, Calendar.JUNE, 14)),
										    DateUtil.createDate(2009, Calendar.JUNE, 17),
										    DateUtil.createDate(2009, Calendar.JUNE, 15),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 15),
										                  DateUtil.createDate(2009, Calendar.JUNE, 21)),
										    DateUtil.createDate(2009, Calendar.JUNE, 24),
										    DateUtil.createDate(2009, Calendar.JUNE, 22),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 22),
										                  DateUtil.createDate(2009, Calendar.JUNE, 28)),
										    DateUtil.createDate(2009, Calendar.JULY, 1),
										    DateUtil.createDate(2009, Calendar.JUNE, 29),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 29),
										                  DateUtil.createDate(2009, Calendar.JULY, 5)),
										    DateUtil.createDate(2009, Calendar.JULY, 8),
										    DateUtil.createDate(2009, Calendar.JULY, 6),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 6),
										                  DateUtil.createDate(2009, Calendar.JULY, 12)),
										    DateUtil.createDate(2009, Calendar.JULY, 15),
										    DateUtil.createDate(2009, Calendar.JULY, 13),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 13),
										                  DateUtil.createDate(2009, Calendar.JULY, 19)),
										    DateUtil.createDate(2009, Calendar.JULY, 22),
										    DateUtil.createDate(2009, Calendar.JULY, 20),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 20),
										                  DateUtil.createDate(2009, Calendar.JULY, 26)),
										    DateUtil.createDate(2009, Calendar.JULY, 29),
										    DateUtil.createDate(2009, Calendar.JULY, 27),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 27),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 2)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 5),
										    DateUtil.createDate(2009, Calendar.AUGUST, 3),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 3),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 9)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 12),
										    DateUtil.createDate(2009, Calendar.AUGUST, 10),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 10),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 16)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 19),
										    DateUtil.createDate(2009, Calendar.AUGUST, 17),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 17),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 23)),
										    DateUtil.createDate(2009, Calendar.AUGUST, 26),
										    DateUtil.createDate(2009, Calendar.AUGUST, 24),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 24),
										                  DateUtil.createDate(2009, Calendar.AUGUST, 30)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 2),
										    DateUtil.createDate(2009, Calendar.AUGUST, 31),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 31),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 6)),
										                  // 9/7/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 10),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 8),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 7),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 13)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 16),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 14),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 14),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 20)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 23),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 21),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 21),
										                  DateUtil.createDate(2009, Calendar.SEPTEMBER, 27)),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 30),
										    DateUtil.createDate(2009, Calendar.SEPTEMBER, 28),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 28),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 4)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 7),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 5),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 5),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 11)),
										                  // 10/12/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.OCTOBER, 15),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 13),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 12),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 18)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 21),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 19),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 19),
										                  DateUtil.createDate(2009, Calendar.OCTOBER, 25)),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 28),
										    DateUtil.createDate(2009, Calendar.OCTOBER, 26),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 26),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 1)),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 4),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 2),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 2),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 8)),
										                  // 11/11/2009 is a holiday
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 12),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 9),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 9),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 15)),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 18),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 16),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 16),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 22)),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 25),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 23),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 23),
										                  DateUtil.createDate(2009, Calendar.NOVEMBER, 29)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 2),
										    DateUtil.createDate(2009, Calendar.NOVEMBER, 30),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 30),
										                  DateUtil.createDate(2009, Calendar.DECEMBER, 6)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 9),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 7),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.DECEMBER, 7),
										                  DateUtil.createDate(2009, Calendar.DECEMBER, 13)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 16),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 14),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.DECEMBER, 14),
										                  DateUtil.createDate(2009, Calendar.DECEMBER, 20)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 23),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 21),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		expectedList.add(new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.DECEMBER, 21),
										                  DateUtil.createDate(2009, Calendar.DECEMBER, 27)),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 30),
										    DateUtil.createDate(2009, Calendar.DECEMBER, 28),
										    PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		AssertJUnit.assertTrue("Expected PayrollRunInfo list mismatch!", verifyPayrollRunInfoList(expectedList,
		                                                                                     payrollRunInfoList));
	}
	
}
