package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.workerpayments.impl.PayrollScheduleDateUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;

public class PayrollScheduleTestBase extends BasicTestBase
{
    protected static PayrollScheduleComponent payrollScheduleComponent;
   
    @BeforeClass
    public static void initializePayrollScheduleTestBase()
    {
    	payrollScheduleComponent = qaBootstrap.getEjbFromClass(PayrollScheduleComponent.class);
    }
    
    public void validatePayrollScheduleList(List<com.intuit.sb.payroll.entity.model.PayrollSchedule> expectedScheduleList,
                                            List<com.intuit.sb.payroll.entity.model.PayrollSchedule> scheduleList)
    {
        AssertJUnit.assertTrue("PayrollSchedule List returned is not expected",
                          scheduleList.size() == expectedScheduleList.size());
        boolean match = false;
        for (com.intuit.sb.payroll.entity.model.PayrollSchedule exPayrollSchedule : expectedScheduleList)
        {
            for (com.intuit.sb.payroll.entity.model.PayrollSchedule pSchedule : scheduleList)
            {

                match = false;
                if (exPayrollSchedule.getId() == pSchedule.getId())
                {
                    match = true;
                    // Validate
                    validatePayrollSchedule(pSchedule, exPayrollSchedule, false);
                    break;
                }
            }
            AssertJUnit.assertTrue("No match found for the PayrollSchedule", match);
        }

    }

    public void validatePayrollSchedule(com.intuit.sb.payroll.entity.model.PayrollSchedule pSchedule, com.intuit.sb.payroll.entity.model.PayrollSchedule exPayrollSchedule, boolean checkId)
    {
        if (checkId)
        {
            AssertJUnit.assertEquals("PayrollSchdule ID does not match, expected ", exPayrollSchedule.getId(), pSchedule.getId());
        }

        AssertJUnit.assertEquals("Parameter does not match, expected " + exPayrollSchedule.getFrequency() + " against "
            + pSchedule.getFrequency(), exPayrollSchedule.getFrequency(), pSchedule.getFrequency());

        if (pSchedule.getFrequency() != PayrollFrequencyEnum.MONTHLY
            && pSchedule.getFrequency() != PayrollFrequencyEnum.SEMIMONTHLY)
        {
	        	Calendar cal1 = DateUtil.getNowCalendar();
	        	cal1.setTime(pSchedule.getNextPayPeriodEndDate());
	        	Calendar cal2 = DateUtil.getNowCalendar();
	        	cal2.setTime(exPayrollSchedule.getNextPayPeriodEndDate());
	            if (!DateUtil.isSameDay(cal1, cal2))
	            {
	                AssertJUnit.assertTrue("Parameter does not match, expected " + exPayrollSchedule.getNextPayPeriodEndDate()
	                    + " against " + pSchedule.getNextPayPeriodEndDate(), false);
        	}
        }

        AssertJUnit.assertEquals("Payroll Schedule Memo does not match, expected " + exPayrollSchedule.getMemo()
            + " against " + pSchedule.getMemo(), pSchedule.getMemo(), exPayrollSchedule.getMemo());

        AssertJUnit.assertEquals("Parameter does not match, expected " + exPayrollSchedule.getPayDayOffset() + " against "
            + pSchedule.getPayDayOffset(), exPayrollSchedule.getPayDayOffset(), pSchedule.getPayDayOffset());

        AssertJUnit.assertEquals("Parameter does not match, expected " + exPayrollSchedule.getPayrollRunDayOffset()
            + " against " + pSchedule.getPayrollRunDayOffset(), exPayrollSchedule.getPayrollRunDayOffset(), pSchedule
                        .getPayrollRunDayOffset());

    }
    
    public List<PayrollSchedule> createPayrollSchedules(PayrollFrequencyEnum[] payFrequency) throws Exception
    {
        Calendar cal = DateUtil.getNowCalendar();
        List<com.intuit.sb.payroll.entity.model.PayrollSchedule> createPScheduleList = new ArrayList<com.intuit.sb.payroll.entity.model.PayrollSchedule>();
    	PayrollScheduleComponent psComponent = qaBootstrap.getEjbFromClass(PayrollScheduleComponent.class);
        try
        {
            for (int i = 0; i < payFrequency.length; i++)
            {
                PayrollFrequencyEnum frequency = payFrequency[i];

                // Keep the i below 15
                cal.set(2008, 03, 15 + i);
               PayrollSchedule pSchedule = new PayrollSchedule();
                if ((frequency == PayrollFrequencyEnum.MONTHLY) || (frequency == PayrollFrequencyEnum.SEMIMONTHLY))
                {
                    pSchedule.setPayPeriodEndDayofMonth1(PayrollSchedule.LAST_DAY_OF_MONTH);
                }
                else
                {
                    Calendar nextPayDate = DateUtil.getNextWeekDayOfWeek(cal, Calendar.FRIDAY);
                    pSchedule.setNextPayPeriodEndDate(nextPayDate.getTime());
                }
                pSchedule.setFrequency(frequency);

                pSchedule.setMemo(frequency.toString());
                pSchedule.setPayDayOffset(3);
                pSchedule.setPayrollRunDayOffset(3);
                pSchedule.setDefaultt(true);
                createPScheduleList.add(psComponent.createPayrollSchedule(pSchedule));
            }
        }
        catch (Exception e)
        {
        	sLogger.error("Something failed when creating schedules", e);
        	throw e;
        }
       
        return createPScheduleList;

    }

    public List<PayrollSchedule> getPayrollSchedules()
    {
    	return entityService.findAll(PayrollSchedule.class);
    }

    @SuppressWarnings("deprecation")
    public static boolean compareDatesIgnoringTime(Date date1, Date date2)
    {
        if (date1.getDate() == date2.getDate() && date1.getMonth() == date2.getMonth()
            && date1.getYear() == date2.getYear())
        {
            return true;
        }

        return false;

    }

    /**
     * FIXME [test] this is pretty much a duplicate method from TestUtils.  Need to consolidate.
     * @param frequency
     * @param periodStart
     * @param payrollRunDateOffset
     * @param payrollPayDateThreshold
     * @return
     * @throws Exception
     */
    protected PayrollSchedule createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum frequency, Calendar periodStart,
            int payrollRunDateOffset, int payrollPayDateThreshold) throws Exception {
    	PayrollSchedule payrollSchedule = new PayrollSchedule();
    	payrollSchedule.setDefaultt(true);
    	payrollSchedule.setFrequency(frequency);
    	payrollSchedule.setPayDayOffset(payrollPayDateThreshold);
    	payrollSchedule.setPayrollRunDayOffset(payrollRunDateOffset);
    	payrollSchedule.setPeriodStartDate(periodStart.getTime());

        if (frequency == PayrollFrequencyEnum.MONTHLY)
        {
            payrollSchedule.setPayPeriodEndDayofMonth1(28);
        }
        else if (frequency == PayrollFrequencyEnum.SEMIMONTHLY)
        {
            payrollSchedule.setPayPeriodEndDayofMonth1(15);
            payrollSchedule.setPayPeriodEndDayofMonth2(PayrollSchedule.LAST_DAY_OF_MONTH);
        }
        else
        {
        	Calendar periodEndCal = PayrollScheduleDateUtils.getPayPeriodEndDate(periodStart, frequency);
        	payrollSchedule.setNextPayPeriodEndDate(periodEndCal.getTime());
        }

		payrollSchedule = payrollScheduleComponent.createPayrollSchedule(payrollSchedule);
		
		return payrollSchedule;
    }
}
