package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.EmployerAdjLineItem;
import com.intuit.sb.payroll.entity.model.EmployerAdjustment;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PaycheckLineItemSortSequenceEnum;
import com.intuit.sb.payroll.entity.model.PaycheckWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerAdjLineItem;
import com.intuit.sb.payroll.entity.model.WorkerAdjustment;

/**
 * <P>
 * The <CODE>VoidPaychecksTest</CODE> class contains tests for testing the
 * voidPaychecks() method in WorkerPaymentsComponent
 * </P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class VoidPaychecksTest extends WorkerPaymentsComponentTestBase {
	final static int numOfPayrollRuns = 13;
	final static int numOfPayrollRuns2 = 2;  //NOTE: Only large enough for 1 adhoc and 1 termination payroll run
	static PayrollRun sPayrollRuns[] = new PayrollRun[numOfPayrollRuns];
	static PayrollRun sPayrollRuns2[] = new PayrollRun[numOfPayrollRuns2];
	private static Long sWorkerIds[] = {0L, 0L, 0L};
	private static BigDecimal mHourlyRate = new BigDecimal("10.0");
	private static BigDecimal mBonusAmount = new BigDecimal("25000.0");

	/* The following paychecks will be created.
	 * Paycheck dated 07/17/2008 (for Jill): Gross:2538.46, Net:1748.48, Company Taxes:303.35, Employee Taxes:789.98
	 * Paycheck dated 07/31/2008 (for Jill): Gross:2538.46, Net:1748.47, Company Taxes:303.36, Employee Taxes:789.99
	 * Paycheck dated 07/31/2008 (for Bill): Gross:25950.00, Net:13258.47, Company Taxes:2286.18, Employee Taxes:12691.53
	 * Paycheck dated 08/14/2008 (for Jill): Gross:2538.46, Net:1748.50, Company Taxes:276.86, Employee Taxes:789.96
	 * Paycheck dated 08/28/2008 (for Jill): Gross:2538.46, Net:1748.47, Company Taxes:194.20, Employee Taxes:789.99
	 * Paycheck dated 09/11/2008 (for Jill): Gross:2538.46, Net:1748.48, Company Taxes:194.19, Employee Taxes:789.98
	 * Paycheck dated 09/25/2008 (for Jill): Gross:2538.46, Net:1748.47, Company Taxes:194.20, Employee Taxes:789.99
	 * Paycheck dated 09/25/2008 (for Phil): Gross:2538.46, Net:1748.48, Company Taxes:303.35, Employee Taxes:789.98
	 * Paycheck dated 10/09/2008 (for Jill): Gross:2538.46, Net:1748.50, Company Taxes:194.18, Employee Taxes:789.96
	 * Paycheck dated 10/23/2008 (for Jill): Gross:2538.46, Net:1748.47, Company Taxes:194.20, Employee Taxes:789.99
	 * Paycheck dated 11/06/2008 (for Jill): Gross:2538.46, Net:1748.48, Company Taxes:194.19, Employee Taxes:789.98
	 * Paycheck dated 11/20/2008 (for Jill): Gross:2538.46, Net:1748.47, Company Taxes:194.20, Employee Taxes:789.99
	 * Paycheck dated 12/04/2008 (for Jill): Gross:2538.46, Net:1748.50, Company Taxes:194.18, Employee Taxes:789.96
	 * Paycheck dated 12/04/2008 (for Phil): Gross:2538.46, Net:1748.47, Company Taxes:303.36, Employee Taxes:789.99
	 * Paycheck dated 12/18/2008 (for Jill): Gross:2538.46, Net:1748.48, Company Taxes:194.19, Employee Taxes:789.98
	 * Paycheck dated 12/18/2008 (for Phil): Gross:2538.46, Net:1748.50, Company Taxes:276.86, Employee Taxes:789.96
	 * Paycheck dated 01/01/2009 (for Jill): Gross:2538.46, Net:1746.92, Company Taxes:303.35, Employee Taxes:791.54
	 */
	@BeforeClass
	public static void setUpVoidPaychecksTest() {
		PayrollSchedule aSchedule = null;
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			// Create 3 worker
			sWorkerIds[0] = WorkerTestUtils.createSalaryPayrollReadyWorker("Jill", sMountainViewLocation);
			sWorkerIds[1] = WorkerTestUtils.createSalaryPayrollReadyWorker("Phil", sMountainViewLocation);
			sWorkerIds[2] = WorkerTestUtils.createPayrollReadyWorker("Bill", sMountainViewLocation);
			getUserTransaction().commit();
			// Create 12 scheduled payroll runs (for year 2008)
			// Create 1 schedule payroll run (for year 2009)
			PayrollRunInfo payrollRunInfo = null;
			// There are a total of 12 biweekly payroll runs for the year 2008, and 1 in 2009:
			//    1st paycheck - pay period 07/01/2008 - 07/14/2008, paycheck date of 07/17/2008
			//    2nd paycheck - pay period 07/15/2008 - 07/28/2008, paycheck date of 07/31/2008
			//    3rd paycheck - pay period 07/29/2008 - 08/11/2008, paycheck date of 08/14/2008
			//    4th paycheck - pay period 08/12/2008 - 08/25/2008, paycheck date of 08/28/2008
			//    5th paycheck - pay period 08/26/2008 - 09/08/2008, paycheck date of 09/11/2008
			//    6th paycheck - pay period 09/09/2008 - 09/22/2008, paycheck date of 09/25/2008
			//    7th paycheck - pay period 09/23/2008 - 10/06/2008, paycheck date of 10/09/2008
			//    8th paycheck - pay period 10/07/2008 - 10/20/2008, paycheck date of 10/23/2008
			//    9th paycheck - pay period 10/21/2008 - 11/03/2008, paycheck date of 11/06/2008
			//   10th paycheck - pay period 11/04/2008 - 11/17/2008, paycheck date of 11/20/2008
			//   11th paycheck - pay period 11/18/2008 - 12/01/2008, paycheck date of 12/04/2008
			//   12th paycheck - pay period 12/02/2008 - 12/15/2008, paycheck date of 12/18/2008
			//   13th paycheck - pay period 12/16/2008 - 12/29/2008, paycheck date of 01/01/2009
			int k = 0;
			Date nextPayPeriodEndDate = DateUtil.createDate(2008, Calendar.JUNE, 30);
			int checkNumber = 1;
			for (int j = 0; j < numOfPayrollRuns; j++) {
				// Calculate the next pay period end date
				nextPayPeriodEndDate = DateUtil.getNextDate(nextPayPeriodEndDate, 14);
		    	aSchedule = createBiweeklySchedule(nextPayPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		    			sDEFAULT_PAY_DAY_OFFSET);
		    	payrollRunInfo = createAPayrollRunInfo(aSchedule);
		    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
		    	PaycheckCalculationRequest calcRequest = createSalaryPaycheckCalculationRequest(sWorkerIds[0],
		    					payrollRunInfo, PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
		    	paycheckCalculationRequests.add(calcRequest);
		    	// For the paycheck date of 07/31/2008, create a paycheck for the sWorkerIds[2] worker as well.
		    	// This worker's paycheck is larger than the combined paychecks in that quarter.
		    	if (j == 1) {
					getUserTransaction().begin();
		        	PaycheckCalculationRequest calcRequest3 = createPaycheckCalculationRequest(sWorkerIds[2],
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
					getUserTransaction().commit();
			    	paycheckCalculationRequests.add(calcRequest3);
		    	}
		    	// For the paycheck date of 12/04/2008, create a paycheck for the sWorkerIds[1] worker as well
		    	if (j == 10) {
			    	PaycheckCalculationRequest calcRequest3 = createSalaryPaycheckCalculationRequest(sWorkerIds[1],
	    					payrollRunInfo, PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
			    	paycheckCalculationRequests.add(calcRequest3);
		    	}
		    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
		        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
		        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
		
		        // Set the Payroll system date to the payroll run date
		        setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
		        sPayrollRuns[j] = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
		        
		        // Update the Paycheck(s) and PayrollRun status
	    		Set<Paycheck> paychecks = sPayrollRuns[j].getPaychecks();
	    		Iterator<Paycheck> iterator = paychecks.iterator();
	    		PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
	    		payrollRunUpdateRequest.setPayrollRunId(sPayrollRuns[j].getId());
	    		List<PaycheckUpdateRequest> paycheckUpdateRequests = new ArrayList<PaycheckUpdateRequest>();
	    		while (iterator.hasNext()) {
		    		Paycheck paycheck = iterator.next();
		    		PaycheckUpdateRequest paycheckUpdateRequest = new PaycheckUpdateRequest();
		    		paycheckUpdateRequest.setCheckNumber(""+checkNumber++);
		    		paycheckUpdateRequest.setMemo("Test paycheck");
		    		paycheckUpdateRequest.setPaycheckId(paycheck.getId());
		    		paycheckUpdateRequest.setPaycheckStatus(PaycheckWorkflowStatusEnum.DISTRIBUTED);
		    		paycheckUpdateRequests.add(paycheckUpdateRequest);
	    		}
	    		payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
	        	sWorkerPaymentsComponent.updatePayrollRun(payrollRunUpdateRequest);

	        	// Create 1 adhoc paycheck and 1 termination paycheck (all using the second worker)
				if ((j == 5) || (j == 11)) {
					PayrollRunInfo payrollRunInfo2 = new PayrollRunInfo(payrollRunInfo.getPayPeriod(),
							payrollRunInfo.getPayDate(), payrollRunInfo.getPayrollRunDate(), payrollRunInfo.getFrequency(),
							payrollRunInfo.getPayrollRunType());
					if (j == 11) {
	    				payrollRunInfo2.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
					} else {
	    				payrollRunInfo2.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
					}
			    	List<PaycheckCalculationRequest> paycheckCalculationRequests2 = new ArrayList<PaycheckCalculationRequest>();
			    	PaycheckCalculationRequest calcRequest2 = createSalaryPaycheckCalculationRequest(sWorkerIds[1],
			    			payrollRunInfo2, PayrollDistributionMethodEnum.HANDWRITTENPAYCHECK);
			    	paycheckCalculationRequests2.add(calcRequest2);
			    	
			    	PayrollRunCommitRequest payrollRunRequest2 = new PayrollRunCommitRequest();
			        payrollRunRequest2.setPayrollRunInfo(payrollRunInfo2);
			        payrollRunRequest2.setCalculationRequests(paycheckCalculationRequests2);
			
			        // Set the Payroll system date to the payroll run date
			        setPayrollSystemDate(payrollRunInfo2.getPayrollRunDate());
			        sPayrollRuns2[k++] = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest2);
			        
			        // Update the Paycheck(s) and PayrollRun status
		    		paychecks = sPayrollRuns2[k-1].getPaychecks();
		    		iterator = paychecks.iterator();
		    		payrollRunUpdateRequest = new PayrollRunUpdateRequest();
		    		payrollRunUpdateRequest.setPayrollRunId(sPayrollRuns2[k-1].getId());
		    		paycheckUpdateRequests = new ArrayList<PaycheckUpdateRequest>();
		    		while (iterator.hasNext()) {
			    		Paycheck paycheck = iterator.next();
			    		PaycheckUpdateRequest paycheckUpdateRequest = new PaycheckUpdateRequest();
			    		paycheckUpdateRequest.setCheckNumber(""+checkNumber++);
			    		paycheckUpdateRequest.setMemo("Test paycheck");
			    		paycheckUpdateRequest.setPaycheckId(paycheck.getId());
			    		paycheckUpdateRequest.setPaycheckStatus(PaycheckWorkflowStatusEnum.DISTRIBUTED);
			    		paycheckUpdateRequests.add(paycheckUpdateRequest);
		    		}
		    		payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
		        	sWorkerPaymentsComponent.updatePayrollRun(payrollRunUpdateRequest);
				}
		     	// Restore the Payroll system date to the current date
		     	resetPayrollSystemDate();
			}    	
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpVoidPaychecksTest(): " + t.getMessage());
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
		}
	}

	@AfterClass
	public static void tearDownVoidPaychecksTest() {
		try {
			getUserTransaction().begin();
			for (int j = 0; j < numOfPayrollRuns; j++) {
	    		if (sPayrollRuns[j] != null) {
	            	Long payrollRunId = sPayrollRuns[j].getId();
	            	cleanUpPayrollRun(payrollRunId);
	    		}
			}
			for (int j = 0; j < numOfPayrollRuns2; j++) {
				if (sPayrollRuns2[j] != null) {
		        	Long payrollRunId = sPayrollRuns2[j].getId();
		        	cleanUpPayrollRun(payrollRunId);
				}
			}
			for (int i = 0; i < sWorkerIds.length; i++) {
				if (0 != sWorkerIds[i]) {
					Worker worker = entityService.findById(Worker.class, sWorkerIds[i]);
					WorkerTestUtils.cleanUpWorker(worker);
					sWorkerIds[i] = 0L;
				}
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownVoidPaychecksTest(): " + t.getMessage());
		}
	}

	/************************************** Void paycheck in current quarter **************************************/

    /**
     * <P>Test voiding the first paycheck in 2009 (paycheck date of 01/01/2009) on 01/05/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the list of Paycheck ids</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck for the PayrollRun is updated</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testVoiding1PaycheckIn1PaycheckPayrollRun() throws Exception {
    	PayrollRun payrollRun = sPayrollRuns[12];
    	try {
	        // Set the Payroll system date to the date that the paycheck is being voided
	        setPayrollSystemDate(DateUtil.createDate(2009, Calendar.JANUARY, 5));
	    	// Exercise
	     	List<Long> paycheckIds = new ArrayList<Long>();
			Set<Paycheck> paychecks = payrollRun.getPaychecks();
			Paycheck paycheck = paychecks.iterator().next();
			paycheckIds.add(paycheck.getId());
	    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	
	    	// Verify
	    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, updatedPayrollRun.getPaychecks().size());
	    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
	    					updatedPayrollRun.getWorkflowStatus());
	    	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "0.00"), updatedPayrollRun.getTotalNetPay());
	    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdTaxes());
	    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdTaxes());
	    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdBenefits());
	    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdBenefits());
	    	
			Set<Paycheck> updatedPaychecks = updatedPayrollRun.getPaychecks();
			Paycheck updatedPaycheck = updatedPaychecks.iterator().next();
	    	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.VOIDED,
	    	                    updatedPaycheck.getWorkflowStatus());
	    	//TODO  Make sure that the following validation logic is correct/complete, then enable the verification
//	    	HashSet<WorkerAdjustment> expectedWorkerAdjustments = new HashSet<WorkerAdjustment>();
//	    	WorkerAdjustment workerAdjustment = new WorkerAdjustment();
//	    	workerAdjustment.setEffectiveDate(paycheck.getPayDate());
//	    	workerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//	    	workerAdjustment.setPayrollRun(updatedPayrollRun);
//	    	workerAdjustment.setWorkerAdjLineItems(createWorkerAdjLineItemSet(paycheck));
//	    	expectedWorkerAdjustments.add(workerAdjustment);
//	    	AssertJUnit.assertTrue("WorkerAdjustments mismatch!", verifyWorkerAdjustments(expectedWorkerAdjustments,
//	    			updatedPayrollRun.getWorkerAdjustments()));
	    	// Include the EmployerAdjustment from the existing PayrollRun
//	    	Set<EmployerAdjustment> expectedEmployerAdjustments = payrollRun.getEmployerAdjustments();
//	    	EmployerAdjustment employerAdjustment = new EmployerAdjustment();
//	    	employerAdjustment.setEffectiveDate(paycheck.getPayDate());
//	    	employerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//	    	employerAdjustment.setPayrollRun(updatedPayrollRun);
//	    	employerAdjustment.setEmployerAdjLineItems(createEmployerAdjLineItemSet(paycheck));
//	    	expectedEmployerAdjustments.add(employerAdjustment);
//	    	AssertJUnit.assertTrue("EmployerAdjustments mismatch!", verifyEmployerAdjustments(expectedEmployerAdjustments,
//	    			updatedPayrollRun.getEmployerAdjustments()));
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
    	}
	}

    /**
     * <P>Test voiding all paychecks in a payroll run (paycheck date of 12/04/2008) on 12/08/2008</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the list of Paycheck ids</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck for the PayrollRun is updated</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testVoidingAllPaycheckInMultiplePaycheckPayrollRun() throws Exception {
    	PayrollRun payrollRun = sPayrollRuns[10];
    	try {
	        // Set the Payroll system date to the date that the paycheck is being voided
	        setPayrollSystemDate(DateUtil.createDate(2008, Calendar.DECEMBER, 8));
	    	// Exercise
	     	List<Long> paycheckIds = new ArrayList<Long>();
			Set<Paycheck> paychecks = payrollRun.getPaychecks();
			Iterator<Paycheck> iterator = paychecks.iterator();
			while (iterator.hasNext()) {
				Paycheck paycheck = iterator.next();
				paycheckIds.add(paycheck.getId());
			}
	    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	
	    	// Verify
	    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 2, updatedPayrollRun.getPaychecks().size());
	    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
	    					updatedPayrollRun.getWorkflowStatus());
	    	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "0.00"), updatedPayrollRun.getTotalNetPay());
	    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdTaxes());
	    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdTaxes());
	    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdBenefits());
	    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdBenefits());
	    	
			Set<Paycheck> updatedPaychecks = updatedPayrollRun.getPaychecks();
			iterator = updatedPaychecks.iterator();
			while (iterator.hasNext()) {
				Paycheck updatedPaycheck = iterator.next();
		    	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.VOIDED,
		    	                    updatedPaycheck.getWorkflowStatus());
			}
	    	//TODO  Make sure that the following validation logic is correct/complete, then enable the verification
//	    	HashSet<WorkerAdjustment> expectedWorkerAdjustments = new HashSet<WorkerAdjustment>();
//			iterator = paychecks.iterator();
//			while (iterator.hasNext()) {
//				Paycheck paycheck = iterator.next();
//		    	WorkerAdjustment workerAdjustment = new WorkerAdjustment();
//		    	workerAdjustment.setEffectiveDate(paycheck.getPayDate());
//		    	workerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//		    	workerAdjustment.setPayrollRun(updatedPayrollRun);
//		    	workerAdjustment.setWorkerAdjLineItems(createWorkerAdjLineItemSet(paycheck));
//		    	expectedWorkerAdjustments.add(workerAdjustment);
//			}
//	    	AssertJUnit.assertTrue("WorkerAdjustments mismatch!", verifyWorkerAdjustments(expectedWorkerAdjustments,
//	    			updatedPayrollRun.getWorkerAdjustments()));
	    	//TODO verify the EmployerAdjustment
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
    	}
	}

    /**
     * <P>Test voiding all paychecks in a payroll run (paycheck date of 12/04/2008) on 12/08/2008 by voiding 1 paycheck
     * per void operation</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the list of Paycheck ids to contain the first paycheck</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the first paycheck</LI>
     * <LI>Set up the list of Paycheck ids to contain the second paycheck</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the second paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck for the PayrollRun is updated</LI>
     * </UL>
     */
	// This test is for verifying IP001386
	@Test(enabled=false, groups={"broken"})
	public void testVoidingAllPaycheckOneAtATimeInMultiplePaycheckPayrollRun() throws Exception {
    	PayrollRun payrollRun = sPayrollRuns[10];
    	try {
	        // Set the Payroll system date to the date that the paycheck is being voided
	        setPayrollSystemDate(DateUtil.createDate(2008, Calendar.DECEMBER, 8));
	    	// Exercise
	     	List<Long> paycheckIds = new ArrayList<Long>();
			Set<Paycheck> paychecks = payrollRun.getPaychecks();
			Iterator<Paycheck> iterator = paychecks.iterator();
			// Void the first paycheck
			Paycheck paycheck = iterator.next();
			paycheckIds.add(paycheck.getId());
			@SuppressWarnings("unused")
	    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	
	    	// Now void the second paycheck
	    	paycheck = iterator.next();
	    	paycheckIds.clear();
	    	paycheckIds.add(paycheck.getId());
	    	PayrollRun updatedPayrollRun2 = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	    	
	    	// Verify
	    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 2, updatedPayrollRun2.getPaychecks().size());
	    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
	    					updatedPayrollRun2.getWorkflowStatus());
	    	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "0.00"), updatedPayrollRun2.getTotalNetPay());
	    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun2.getTotalWorkerPdTaxes());
	    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun2.getTotalEmployerPdTaxes());
	    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun2.getTotalWorkerPdBenefits());
	    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun2.getTotalEmployerPdBenefits());
	    	
			Set<Paycheck> updatedPaychecks = updatedPayrollRun2.getPaychecks();
			iterator = updatedPaychecks.iterator();
			while (iterator.hasNext()) {
				Paycheck updatedPaycheck = iterator.next();
		    	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.VOIDED,
		    	                    updatedPaycheck.getWorkflowStatus());
			}
	    	//TODO  Make sure that the following validation logic is correct/complete, then enable the verification
//	    	HashSet<WorkerAdjustment> expectedWorkerAdjustments = new HashSet<WorkerAdjustment>();
//			iterator = paychecks.iterator();
//			while (iterator.hasNext()) {
//				Paycheck paycheck = iterator.next();
//		    	WorkerAdjustment workerAdjustment = new WorkerAdjustment();
//		    	workerAdjustment.setEffectiveDate(paycheck.getPayDate());
//		    	workerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//		    	workerAdjustment.setPayrollRun(updatedPayrollRun);
//		    	workerAdjustment.setWorkerAdjLineItems(createWorkerAdjLineItemSet(paycheck));
//		    	expectedWorkerAdjustments.add(workerAdjustment);
//			}
//	    	AssertJUnit.assertTrue("WorkerAdjustments mismatch!", verifyWorkerAdjustments(expectedWorkerAdjustments,
//	    			updatedPayrollRun.getWorkerAdjustments()));
	    	//TODO verify the EmployerAdjustment
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
    	}
	}

    /**
     * <P>Test voiding 1 paycheck in a payroll run containing multiple paychecks (paycheck date of 12/04/2008) on
     *  12/08/2008</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the list of Paycheck ids</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck for the PayrollRun is updated</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testVoiding1PaycheckInMultiplePaycheckPayrollRun() throws Exception {
    	PayrollRun payrollRun = sPayrollRuns[10];
    	try {
	        // Set the Payroll system date to the date that the paycheck is being voided
	        setPayrollSystemDate(DateUtil.createDate(2008, Calendar.DECEMBER, 8));
	    	// Exercise
	     	List<Long> paycheckIds = new ArrayList<Long>();
			Set<Paycheck> paychecks = payrollRun.getPaychecks();
			Paycheck paycheck = null;
			// Voiding the paycheck for sWorkerIds[0], this paycheck's net amount is $1748.50, workerPdTaxes is $789.96,
			//   and employerPdTaxes is $194.18.
			Iterator<Paycheck> iterator = paychecks.iterator();
			while (iterator.hasNext()) {
				paycheck = iterator.next();
				if (paycheck.getWorker().getId() == sWorkerIds[0]) {
					paycheckIds.add(paycheck.getId());
					break;
				}
			}
	    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	
	    	// Verify
	    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 2, updatedPayrollRun.getPaychecks().size());
	    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
	    					updatedPayrollRun.getWorkflowStatus());
	    	// Remaining paycheck is for sWorkerIds[1], this paycheck's net amount is $1748.47, workerPdTaxes is $789.99,
			//   and employerPdTaxes is $303.36.
	    	//TODO  May need to update the expected values after adjusting for penny rounding
	    	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "1748.47"), updatedPayrollRun.getTotalNetPay());
	    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "789.99"),
	    	                updatedPayrollRun.getTotalWorkerPdTaxes());
	    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "303.36"),
	    	                updatedPayrollRun.getTotalEmployerPdTaxes());
	    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdBenefits());
	    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdBenefits());
	    	
			Set<Paycheck> updatedPaychecks = updatedPayrollRun.getPaychecks();
			Paycheck updatedPaycheck = null;
			iterator = updatedPaychecks.iterator();
			while (iterator.hasNext()) {
				updatedPaycheck = iterator.next();
				if (updatedPaycheck.getId() == paycheck.getId()) {
			    	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.VOIDED,
		                    updatedPaycheck.getWorkflowStatus());
				} else {
			    	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.DISTRIBUTED,
		                    updatedPaycheck.getWorkflowStatus());
				}
			}
	    	//TODO  Make sure that the following validation logic is correct/complete, then enable the verification
//	    	HashSet<WorkerAdjustment> expectedWorkerAdjustments = new HashSet<WorkerAdjustment>();
//	    	WorkerAdjustment workerAdjustment = new WorkerAdjustment();
//	    	workerAdjustment.setEffectiveDate(paycheck.getPayDate());
//	    	workerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//	    	workerAdjustment.setPayrollRun(updatedPayrollRun);
//	    	workerAdjustment.setWorkerAdjLineItems(createWorkerAdjLineItemSet(paycheck));
//	    	expectedWorkerAdjustments.add(workerAdjustment);
//	    	AssertJUnit.assertTrue("WorkerAdjustments mismatch!", verifyWorkerAdjustments(expectedWorkerAdjustments,
//	    			updatedPayrollRun.getWorkerAdjustments()));
	    	// Include the EmployerAdjustment from the existing PayrollRun
//	    	Set<EmployerAdjustment> expectedEmployerAdjustments = payrollRun.getEmployerAdjustments();
//	    	EmployerAdjustment employerAdjustment = new EmployerAdjustment();
//	    	employerAdjustment.setEffectiveDate(paycheck.getPayDate());
//	    	employerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//	    	employerAdjustment.setPayrollRun(updatedPayrollRun);
//	    	employerAdjustment.setEmployerAdjLineItems(createEmployerAdjLineItemSet(paycheck));
//	    	expectedEmployerAdjustments.add(employerAdjustment);
//	    	AssertJUnit.assertTrue("EmployerAdjustments mismatch!", verifyEmployerAdjustments(expectedEmployerAdjustments,
//	    			updatedPayrollRun.getEmployerAdjustments()));
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
    	}
	}
	
	/************************************** Void paycheck in last quarter **************************************/

    /**
     * <P>Test voiding the first paycheck in 2008 (paycheck date of 07/17/2008) on 10/06/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the list of Paycheck ids</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck for the PayrollRun is updated</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testVoiding1PaycheckForLastQuarterIn1PaycheckPayrollRun() throws Exception {
    	PayrollRun payrollRun = sPayrollRuns[0];
    	try {
	        // Set the Payroll system date to the date that the paycheck is being voided
	        setPayrollSystemDate(DateUtil.createDate(2008, Calendar.OCTOBER, 6));
	    	// Exercise
	     	List<Long> paycheckIds = new ArrayList<Long>();
			Set<Paycheck> paychecks = payrollRun.getPaychecks();
			Paycheck paycheck = paychecks.iterator().next();
			paycheckIds.add(paycheck.getId());
	    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	
	    	// Verify
	    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, updatedPayrollRun.getPaychecks().size());
	    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
	    					updatedPayrollRun.getWorkflowStatus());
	    	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "0.00"), updatedPayrollRun.getTotalNetPay());
	    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdTaxes());
	    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdTaxes());
	    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdBenefits());
	    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdBenefits());
	    	
			Set<Paycheck> updatedPaychecks = updatedPayrollRun.getPaychecks();
			Paycheck updatedPaycheck = updatedPaychecks.iterator().next();
	    	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.VOIDED,
	    	                    updatedPaycheck.getWorkflowStatus());
	    	//TODO  Make sure that the following validation logic is correct/complete, then enable the verification
//	    	HashSet<WorkerAdjustment> expectedWorkerAdjustments = new HashSet<WorkerAdjustment>();
//	    	WorkerAdjustment workerAdjustment = new WorkerAdjustment();
//	    	workerAdjustment.setEffectiveDate(paycheck.getPayDate());
//	    	workerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//	    	workerAdjustment.setPayrollRun(updatedPayrollRun);
//	    	workerAdjustment.setWorkerAdjLineItems(createWorkerAdjLineItemSet(paycheck));
//	    	expectedWorkerAdjustments.add(workerAdjustment);
//	    	AssertJUnit.assertTrue("WorkerAdjustments mismatch!", verifyWorkerAdjustments(expectedWorkerAdjustments,
//	    			updatedPayrollRun.getWorkerAdjustments()));
	    	// Include the EmployerAdjustment from the existing PayrollRun
//	    	Set<EmployerAdjustment> expectedEmployerAdjustments = payrollRun.getEmployerAdjustments();
//	    	EmployerAdjustment employerAdjustment = new EmployerAdjustment();
//	    	employerAdjustment.setEffectiveDate(paycheck.getPayDate());
//	    	employerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//	    	employerAdjustment.setPayrollRun(updatedPayrollRun);
//	    	employerAdjustment.setEmployerAdjLineItems(createEmployerAdjLineItemSet(paycheck));
//	    	expectedEmployerAdjustments.add(employerAdjustment);
//	    	AssertJUnit.assertTrue("EmployerAdjustments mismatch!", verifyEmployerAdjustments(expectedEmployerAdjustments,
//	    			updatedPayrollRun.getEmployerAdjustments()));
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
    	}
	}

    /**
     * <P>Test voiding all paychecks in a payroll run (paycheck date of 12/04/2008) on 01/05/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the list of Paycheck ids</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck for the PayrollRun is updated</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testVoidingAllPaycheckForLastQuarterInMultiplePaycheckPayrollRun() throws Exception {
    	PayrollRun payrollRun = sPayrollRuns[10];
    	try {
	        // Set the Payroll system date to the date that the paycheck is being voided
	        setPayrollSystemDate(DateUtil.createDate(2009, Calendar.JANUARY, 5));
	    	// Exercise
	     	List<Long> paycheckIds = new ArrayList<Long>();
			Set<Paycheck> paychecks = payrollRun.getPaychecks();
			Iterator<Paycheck> iterator = paychecks.iterator();
			while (iterator.hasNext()) {
				Paycheck paycheck = iterator.next();
				paycheckIds.add(paycheck.getId());
			}
	    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	
	    	// Verify
	    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 2, updatedPayrollRun.getPaychecks().size());
	    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
	    					updatedPayrollRun.getWorkflowStatus());
	    	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "0.00"), updatedPayrollRun.getTotalNetPay());
	    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdTaxes());
	    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdTaxes());
	    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdBenefits());
	    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdBenefits());
	    	
			Set<Paycheck> updatedPaychecks = updatedPayrollRun.getPaychecks();
			iterator = updatedPaychecks.iterator();
			while (iterator.hasNext()) {
				Paycheck updatedPaycheck = iterator.next();
		    	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.VOIDED,
		    	                    updatedPaycheck.getWorkflowStatus());
			}
	    	//TODO  Make sure that the following validation logic is correct/complete, then enable the verification
//	    	HashSet<WorkerAdjustment> expectedWorkerAdjustments = new HashSet<WorkerAdjustment>();
//			iterator = paychecks.iterator();
//			while (iterator.hasNext()) {
//				Paycheck paycheck = iterator.next();
//		    	WorkerAdjustment workerAdjustment = new WorkerAdjustment();
//		    	workerAdjustment.setEffectiveDate(paycheck.getPayDate());
//		    	workerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//		    	workerAdjustment.setPayrollRun(updatedPayrollRun);
//		    	workerAdjustment.setWorkerAdjLineItems(createWorkerAdjLineItemSet(paycheck));
//		    	expectedWorkerAdjustments.add(workerAdjustment);
//			}
//	    	AssertJUnit.assertTrue("WorkerAdjustments mismatch!", verifyWorkerAdjustments(expectedWorkerAdjustments,
//	    			updatedPayrollRun.getWorkerAdjustments()));
	    	//TODO verify the EmployerAdjustment
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
    	}
	}

    /**
     * <P>Test voiding a huge paycheck in 2008 (paycheck date of 07/31/2008) on 10/06/2009</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the list of Paycheck ids</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck for the PayrollRun is updated</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testVoidingHugePaycheckForLastQuarter() throws Exception {
    	PayrollRun payrollRun = sPayrollRuns[1];
    	try {
	        // Set the Payroll system date to the date that the paycheck is being voided
	        setPayrollSystemDate(DateUtil.createDate(2008, Calendar.OCTOBER, 6));
	    	// Exercise
	     	List<Long> paycheckIds = new ArrayList<Long>();			
			Set<Paycheck> paychecks = payrollRun.getPaychecks();
			Paycheck paycheck = null;
			// Voiding the paycheck for sWorkerIds[2], this paycheck's net amount is $13258.47, workerPdTaxes is $12691.53,
			//   and employerPdTaxes is $2286.18.
			Iterator<Paycheck> iterator = paychecks.iterator();
			while (iterator.hasNext()) {
				paycheck = iterator.next();
				if (paycheck.getWorker().getId() == sWorkerIds[2]) {
					paycheckIds.add(paycheck.getId());
					break;
				}
			}
	    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	
	    	// Verify
	    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 2, updatedPayrollRun.getPaychecks().size());
	    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
	    					updatedPayrollRun.getWorkflowStatus());
	    	// Remaining paycheck is for sWorkerIds[0], this paycheck's net amount is $1748.47, workerPdTaxes is $789.99,
			//   and employerPdTaxes is $303.36.
	    	//TODO  May need to update the expected values after adjusting for penny rounding
	    	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "1748.47"), updatedPayrollRun.getTotalNetPay());
	    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "789.99"),
	    	                updatedPayrollRun.getTotalWorkerPdTaxes());
	    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "303.36"),
	    	                updatedPayrollRun.getTotalEmployerPdTaxes());
	    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalWorkerPdBenefits());
	    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
	    	                updatedPayrollRun.getTotalEmployerPdBenefits());
	    	
			Set<Paycheck> updatedPaychecks = updatedPayrollRun.getPaychecks();
			Paycheck updatedPaycheck = null;
			iterator = updatedPaychecks.iterator();
			while (iterator.hasNext()) {
				updatedPaycheck = iterator.next();
				if (updatedPaycheck.getId() == paycheck.getId()) {
			    	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.VOIDED,
		                    updatedPaycheck.getWorkflowStatus());
				} else {
			    	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.DISTRIBUTED,
		                    updatedPaycheck.getWorkflowStatus());
				}
			}
	    	//TODO  Make sure that the following validation logic is correct/complete, then enable the verification
//	    	HashSet<WorkerAdjustment> expectedWorkerAdjustments = new HashSet<WorkerAdjustment>();
//	    	WorkerAdjustment workerAdjustment = new WorkerAdjustment();
//	    	workerAdjustment.setEffectiveDate(paycheck.getPayDate());
//	    	workerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//	    	workerAdjustment.setPayrollRun(updatedPayrollRun);
//	    	workerAdjustment.setWorkerAdjLineItems(createWorkerAdjLineItemSet(paycheck));
//	    	expectedWorkerAdjustments.add(workerAdjustment);
//	    	AssertJUnit.assertTrue("WorkerAdjustments mismatch!", verifyWorkerAdjustments(expectedWorkerAdjustments,
//	    			updatedPayrollRun.getWorkerAdjustments()));
	    	// Include the EmployerAdjustment from the existing PayrollRun
//	    	Set<EmployerAdjustment> expectedEmployerAdjustments = payrollRun.getEmployerAdjustments();
//	    	EmployerAdjustment employerAdjustment = new EmployerAdjustment();
//	    	employerAdjustment.setEffectiveDate(paycheck.getPayDate());
//	    	employerAdjustment.setPayrollPeriod(paycheck.getPayPeriod());
//	    	employerAdjustment.setPayrollRun(updatedPayrollRun);
//	    	employerAdjustment.setEmployerAdjLineItems(createEmployerAdjLineItemSet(paycheck));
//	    	expectedEmployerAdjustments.add(employerAdjustment);
//	    	AssertJUnit.assertTrue("EmployerAdjustments mismatch!", verifyEmployerAdjustments(expectedEmployerAdjustments,
//	    			updatedPayrollRun.getEmployerAdjustments()));
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
    	}
	}

	/**************************************** Exception test(s) ****************************************/

	/**
	 * <P>Test calling voidPaychecks() with paycheckIds from different PayrollRuns</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.voidPaychecks() with paycheckIds from different PayrollRuns</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testVoidingPaychecksFromDifferentPayrollRuns() throws Exception {
    	// Exercise
     	List<Long> paycheckIds = new ArrayList<Long>();
     	// Add a paycheck from the 1st PayrollRun
		Set<Paycheck> paychecks = sPayrollRuns[0].getPaychecks();
		Paycheck paycheck = paychecks.iterator().next();
		paycheckIds.add(paycheck.getId());
     	// Add a paycheck from the 2nd PayrollRun
		paychecks = sPayrollRuns[1].getPaychecks();
		paycheck = paychecks.iterator().next();
		paycheckIds.add(paycheck.getId());
		@SuppressWarnings("unused")
    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	}

    /**
     * <P>Test voiding an already voided paycheck</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the list of Paycheck ids</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the paycheck</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the same paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testDoubleVoidPaycheck() throws Exception {
    	PayrollRun payrollRun = sPayrollRuns[12];
    	try {
	        // Set the Payroll system date to the date that the paycheck is being voided
	        setPayrollSystemDate(DateUtil.createDate(2009, Calendar.JANUARY, 5));
	    	// Exercise
	     	List<Long> paycheckIds = new ArrayList<Long>();
			Set<Paycheck> paychecks = payrollRun.getPaychecks();
			Paycheck paycheck = paychecks.iterator().next();
			paycheckIds.add(paycheck.getId());
			@SuppressWarnings("unused")
	    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
	
	    	// Now re-void the same check
	    	updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
    	}
	}

    /**
     * <P>Test voiding same paycheck in the same void paycheck operation</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the list of Paycheck ids with the same paycheck id</LI>
     * <LI>Call WorkerPaymentsComponent.voidPaychecks() to void the paycheck</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions = ValidationException.class)
	public void testVoidSamePaycheckInListOfPaycheckIds() throws Exception {
    	PayrollRun payrollRun = sPayrollRuns[12];
    	try {
	        // Set the Payroll system date to the date that the paycheck is being voided
	        setPayrollSystemDate(DateUtil.createDate(2009, Calendar.JANUARY, 5));
	        // Setup
	     	List<Long> paycheckIds = new ArrayList<Long>();
			Set<Paycheck> paychecks = payrollRun.getPaychecks();
			Paycheck paycheck = paychecks.iterator().next();
			paycheckIds.add(paycheck.getId());
			// Add the same paycheck id again
			paycheckIds.add(paycheck.getId());
	    	// Exercise
			@SuppressWarnings("unused")
	    	PayrollRun updatedPayrollRun = sWorkerPaymentsComponent.voidPaychecks(paycheckIds);
		} finally {
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
    	}
	}

	/**************************************** utility routines ****************************************/

	boolean verifyWorkerAdjustments(HashSet<WorkerAdjustment> expected, Set<WorkerAdjustment> computed) {
		boolean result = expected.size() == computed.size();
		if (result) {
			for (WorkerAdjustment workerAdjustment1 : expected) {
				boolean found = false;
				for (WorkerAdjustment workerAdjustment2 : computed) {
					//FIXME  How can we figure out which WorkerAdjustment correspond to which voided paycheck?
					if (workerAdjustment1.getPayrollRun().getId() == workerAdjustment2.getPayrollRun().getId()) {
						found = true;
						result = result && (workerAdjustment1.isEquivalent(workerAdjustment2));
						// Now compare the WorkerAdjLineItems
						if (result) {
							result = verifyWorkerAdjLineItems(workerAdjustment1.getWorkerAdjLineItems(),
									workerAdjustment2.getWorkerAdjLineItems());
						}
						break;
					}
				}
				if (!result || !found) {
					break;
				}
			}
		}
		if (!result) {
			sLogger.error("Expected WorkerAdjustment values:");
			for (WorkerAdjustment workerAdjustment : expected) {
				sLogger.error(workerAdjustment.toString());
			}
			sLogger.error("Computed WorkerAdjustment values:");
			for (WorkerAdjustment workerAdjustment : computed) {
				sLogger.error(workerAdjustment.toString());
			}
		}
		return result;
	}
	
	Set<WorkerAdjLineItem> createWorkerAdjLineItemSet(Paycheck paycheck) {
		Set<WorkerAdjLineItem> workerAdjLineItemSet = new HashSet<WorkerAdjLineItem>();
		Set<PaycheckLineItem> paycheckLineItems = paycheck.getPaycheckLineItems();
		for (PaycheckLineItem paycheckLineItem : paycheckLineItems) {
			//Non-wage items are reversed.
			if (PaycheckLineItemSortSequenceEnum.WAGES != paycheckLineItem.getSortSequence()&&
					PaycheckLineItemSortSequenceEnum.EMPLOYERPAIDTAX != paycheckLineItem.getSortSequence()) {
				WorkerAdjLineItem workerAdjLineItem = new WorkerAdjLineItem();
				workerAdjLineItem.setAdjustmentAmount(paycheckLineItem.getValue());
				workerAdjLineItem.setLawId(paycheckLineItem.getLawId());
				workerAdjLineItem.setPayrollItemDtl(paycheckLineItem.getSourcePayrollItemDtl());
				//TODO PtdDatums???
				workerAdjLineItemSet.add(workerAdjLineItem);
			}
		}
		return workerAdjLineItemSet;
	}
	
	boolean verifyWorkerAdjLineItems(Set<WorkerAdjLineItem> expected, Set<WorkerAdjLineItem>computed) {
		boolean result = expected.size() == computed.size();
		if (result) {
			for (WorkerAdjLineItem workerAdjLineItem1 : expected) {
				boolean found = false;
				for (WorkerAdjLineItem workerAdjLineItem2 : computed) {
					if (workerAdjLineItem1.getLawId().getValue().equals(workerAdjLineItem2.getLawId().getValue())) {
						found = true;
						result = result && workerAdjLineItem1.isEquivalent(workerAdjLineItem2);
						break;
					}
					if (!result) {
						break;
					}
				}
				if (!found) {
					result = false;
					break;
				}
			}
		}
		if (!result) {
			sLogger.error("Expected WorkerAdjLineItem values:");
			for (WorkerAdjLineItem workerAdjLineItem : expected) {
				sLogger.error(workerAdjLineItem.toString());
			}
			sLogger.error("Computed WorkerAdjLineItem values:");
			for (WorkerAdjLineItem workerAdjLineItem : computed) {
				sLogger.error(workerAdjLineItem.toString());
			}
		}
		return result;
	}
	
	Set<EmployerAdjLineItem> createEmployerAdjLineItemSet(Paycheck paycheck) {
		Set<EmployerAdjLineItem> employerAdjLineItemSet = new HashSet<EmployerAdjLineItem>();
		Set<PaycheckLineItem> paycheckLineItems = paycheck.getPaycheckLineItems();
		for (PaycheckLineItem paycheckLineItem : paycheckLineItems) {
			//Employer taxes are reversed.
			if (PaycheckLineItemSortSequenceEnum.EMPLOYERPAIDTAX == paycheckLineItem.getSortSequence()) {
				EmployerAdjLineItem employerAdjLineItem = new EmployerAdjLineItem();
				employerAdjLineItem.setAdjustmentAmount(paycheckLineItem.getValue());
				employerAdjLineItem.setLawId(paycheckLineItem.getLawId());
				employerAdjLineItemSet.add(employerAdjLineItem);
			}
		}
		return employerAdjLineItemSet;
	}
	
	boolean verifyEmployerAdjustments(Set<EmployerAdjustment> expected, Set<EmployerAdjustment> computed) {
		boolean result = expected.size() == computed.size();
		if (result) {
			for (EmployerAdjustment adjustment1 : expected) {
				boolean found = false;
				for (EmployerAdjustment adjustment2 : computed) {
					//FIXME  How can we figure out which EmployerAdjustment correspond to which voided paycheck?
					if (adjustment1.getPayrollRun().getId() == adjustment2.getPayrollRun().getId() &&
						adjustment1.getEmployerAdjLineItems().size() == adjustment2.getEmployerAdjLineItems().size()) {
						found = true;
						//FIXME  The isEquivalent() method fails due to differences in millisecond value when comparing
						//          Dates.
						result = result && (adjustment1.isEquivalent(adjustment2));
						// Now compare the EmployerAdjLineItems
						if (result) {
							result = verifyEmployerAdjLineItems(adjustment1.getEmployerAdjLineItems(),
									adjustment2.getEmployerAdjLineItems());
						}
						break;
					}
					if (!result) {
						break;
					}
				}
				if (!found) {
					result = false;
					break;
				}
			}
		}
		if (!result) {
			sLogger.error("Expected EmployerAdjustment values:");
			for (EmployerAdjustment employerAdjustment : expected) {
				sLogger.error(employerAdjustment.toString());
			}
			sLogger.error("Computed EmployerAdjustment values:");
			for (EmployerAdjustment employerAdjustment : computed) {
				sLogger.error(employerAdjustment.toString());
			}
		}
		return result;
	}
	
	boolean verifyEmployerAdjLineItems(Set<EmployerAdjLineItem> expected, Set<EmployerAdjLineItem>computed) {
		boolean result = expected.size() == computed.size();
		if (result) {
			for (EmployerAdjLineItem employerAdjLineItem1 : expected) {
				boolean found = false;
				for (EmployerAdjLineItem employerAdjLineItem2 : computed) {
					if (employerAdjLineItem1.getLawId().getValue().equals(employerAdjLineItem2.getLawId().getValue())) {
						found = true;
						result = result && employerAdjLineItem1.isEquivalent(employerAdjLineItem2);
						break;
					}
					if (!result || !found) {
						break;
					}
				}
			}
		}
		if (!result) {
			sLogger.error("Expected EmployerAdjLineItem values:");
			for (EmployerAdjLineItem employerAdjLineItem : expected) {
				sLogger.error(employerAdjLineItem.toString());
			}
			sLogger.error("Computed EmployerAdjLineItem values:");
			for (EmployerAdjLineItem employerAdjLineItem : computed) {
				sLogger.error(employerAdjLineItem.toString());
			}
		}
		return result;
	}
	
}
