package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;

import com.intuit.sb.cdm.basetypes.Money;
import com.intuit.sb.payroll.calcengine.PaycheckCalcTestBase;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.EmployerAdjLineItem;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>WorkerPaymentsComponentTestBase</CODE> class contains utility methods for testing WorkerPaymentsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class WorkerPaymentsComponentTestBase extends PaycheckCalcTestBase {
	protected static WorkerPaymentsComponent sWorkerPaymentsComponent;
	PayrollScheduleComponent pScheduleComponent;
	protected PayrollSchedule aSchedule = null;
	protected static int sDEFAULT_PAY_DAY_OFFSET = 3;
	protected static int sDEFAULT_PAYROLL_RUN_OFFSET = 1;
	private static WorkerComponent workerComponent;
	
	final static int numOfPayrollRuns = 26;
	final static int numOfPayrollRuns2 = 4;  //NOTE: Only large enough for 3 adhoc and 1 termination payroll run
	static PayrollRun payrollRuns[] = new PayrollRun[numOfPayrollRuns];
	static PayrollRun payrollRuns2[] = new PayrollRun[numOfPayrollRuns2];

	@BeforeMethod
	public void initializeWorkerPaymentsComponentTestCase() {
		pScheduleComponent = qaBootstrap.getEjbFromClass(PayrollScheduleComponent.class);
	}

	@BeforeClass
	public static void initializeWorkerPaymentsComponent() {
		sWorkerPaymentsComponent = qaBootstrap.getEjbFromClass(WorkerPaymentsComponent.class);
		workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
	}

	/**
	 * This utility method will return a Date object with a calculated date using the specified values
	 * 
	 * @param date
	 *            A Date object containing the starting date value
	 * @param field
	 *            The field number to apply the amount value to
	 * @param amount
	 *            The value to adjust
	 * @return A Date object containing the newly calculated date
	 */
	protected static Date addDate(Date date, int field, int amount) {
		Calendar computedDate = DateUtil.getNowCalendar();
		computedDate.setTime(date);
		computedDate.add(field, amount);
		return computedDate.getTime();
	}

	/**
	 * Calculates a new date given a starting date and the number of days to adjust
	 * 
	 * @param date
	 *            The starting date to use for the Date calculation
	 * @param amount
	 *            The number of days to add to or subtract from the starting date
	 * @return The calculated date
	 */
	protected static Date addDays(Date date, int amount) {
		return addDate(date, Calendar.DAY_OF_MONTH, amount);
	}

	/**
	 * Calculates a pay period DateRange given a payroll schedule
	 * 
	 * @param schedule
	 *            The payroll schedule
	 * @return The calculated pay period DateRange
	 */
	private static DateRange calculatePayPeriod(PayrollSchedule schedule) {
		DateRange payPeriod = new DateRange();
		Calendar periodEndDate = null;
		if (null != schedule.getNextPayPeriodEndDate()) {
			periodEndDate = DateUtil.dateToCalendar(schedule.getNextPayPeriodEndDate());
		} else {
			// If the pay period end date is not supplied, use the current date
			periodEndDate = DateUtil.getNowCalendar();
		}
		Calendar periodStartDate = DateUtil.getNowCalendar();
		// Set the pay period start date to be the same as the end date for now
		periodStartDate.setTime(periodEndDate.getTime());
		// Now calculate the pay period start date
		switch (schedule.getFrequency()) {
			case WEEKLY:
			case BIWEEKLY:
				int days = (PayrollFrequencyEnum.WEEKLY == schedule.getFrequency() ? -6 : -13);
				periodStartDate.add(Calendar.DAY_OF_MONTH, days);
				break;
			case MONTHLY:
				int day = schedule.getPayPeriodEndDayofMonth1();
				if (PayrollSchedule.LAST_DAY_OF_MONTH == day) {
					// For last day of the month, the start date is the 1st of the month
					periodStartDate.set(Calendar.DAY_OF_MONTH, 1);
					// And the last day is the actual last day of the month
					periodEndDate.set(Calendar.DAY_OF_MONTH, DateUtil.getLastDayOfMonth(periodEndDate));
				} else {
					periodEndDate.set(Calendar.DAY_OF_MONTH, day);
					periodStartDate.setTime(periodEndDate.getTime());
					periodStartDate.add(Calendar.MONTH, -1);
					periodStartDate.add(Calendar.DAY_OF_MONTH, 1);
				}
				break;
			case SEMIMONTHLY:
				int day1 = schedule.getPayPeriodEndDayofMonth1();
				int day2 = schedule.getPayPeriodEndDayofMonth2();
				int dayOfMonth = periodEndDate.get(Calendar.DAY_OF_MONTH);
				if (PayrollSchedule.LAST_DAY_OF_MONTH == day2) {
					if (dayOfMonth >= 28) {
						periodStartDate.set(Calendar.DAY_OF_MONTH, 16);
						// And the last day is the actual last day of the month
						periodEndDate.set(Calendar.DAY_OF_MONTH, DateUtil.getLastDayOfMonth(periodEndDate));
					} else {
						periodStartDate.set(Calendar.DAY_OF_MONTH, 1);
					}
				} else {
					if (dayOfMonth == day2) {
						periodStartDate.set(Calendar.DAY_OF_MONTH, day1 + 1);
					} else {
						periodStartDate.add(Calendar.MONTH, -1);
						periodStartDate.set(Calendar.DAY_OF_MONTH, day2 + 1);
					}
				}
				break;
			default:
				AssertJUnit.fail("Unsupported frequency detected in calculatePeriodStartDate()!");
		}
		payPeriod.setStartDate(periodStartDate.getTime());
		payPeriod.setEndDate(periodEndDate.getTime());
		return payPeriod;
	}

	/**
	 * This utility method will create a PayrollRunInfo object using the specified values
	 * 
	 * @param schedule
	 *            The PayrollSchedule
	 * @param payrollRunType
	 *            The payroll run type
	 * @return A PayrollRunInfo object with initialized values
	 */
	protected static PayrollRunInfo createAPayrollRunInfo(PayrollSchedule schedule, PayrollRunTypeEnum payrollRunType) {
		PayrollRunInfo payrollRunInfo = new PayrollRunInfo();
		payrollRunInfo.setPayPeriod(calculatePayPeriod(schedule));
		payrollRunInfo
				.setPayrollRunDate(addDays(schedule.getNextPayPeriodEndDate(), schedule.getPayrollRunDayOffset()));
		payrollRunInfo.setPayDate(addDays(schedule.getNextPayPeriodEndDate(), schedule.getPayDayOffset()));
		payrollRunInfo.setFrequency(schedule.getFrequency());
		payrollRunInfo.setPayrollRunType(payrollRunType);
		return payrollRunInfo;
	}

	/**
	 * This utility method will create a PayrollRunInfo object with a scheduled PayrollRunType, using values specified
	 * in the schedule
	 * 
	 * @param schedule
	 *            The PayrollSchedule
	 * @return A PayrollRunInfo object with initialized values
	 */
	protected static PayrollRunInfo createAPayrollRunInfo(PayrollSchedule schedule) {
		return createAPayrollRunInfo(schedule, PayrollRunTypeEnum.SCHEDULED);
	}

	/**
	 * This utility method will create a PaycheckCalculationRequest object using the specified inputs
	 * 
	 * @param workerId
	 *            The id of the worker
	 * @param payrollRunInfo
	 *            An initialized PayrollRunInfo object containing the payroll run info
	 * @param hourlyRate
	 *            The hourly rate
	 * @param hours
	 *            The number of hours for the hourly compensation item
	 * @param overtimeHours
	 *            The number of overtime hours
	 * @param bonusAmount
	 *            The amount for the bonus
	 * @param distributionMethod
	 *            The paycheck distribution method
	 * @return A PaycheckCalculationRequest initialized with the specified values
	 * @throws Exception
	 */
	public static PaycheckCalculationRequest createPaycheckCalculationRequest(Long workerId,
			PayrollRunInfo payrollRunInfo, BigDecimal hourlyRate, Double hours, Double overtimeHours,
			BigDecimal bonusAmount, PayrollDistributionMethodEnum distributionMethod) throws Exception {
		PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
		Worker worker = workerComponent.findWorker(workerId);
		Set<CompensationDtl> compensationDtls = worker.getCompensationDtls();
		List<CalculationInputs> calculationInputs = new ArrayList<CalculationInputs>();
		for (CompensationDtl compensationDtl : compensationDtls) {
			// Copy the datums to their transient siblings for calculation purposes.
			CalculationInputs calcInputs = new CalculationInputs();
			Datum calcRate = new Datum();
			calcInputs.setRate(calcRate);
			Datum calcQty = new Datum();
			calcInputs.setQuantity(calcQty);
			setCompensationRateAndQuantity(compensationDtl, calcRate, calcQty, hourlyRate, hours, overtimeHours,
					bonusAmount);
			calcInputs.setSourcePayrollItemDtlId(compensationDtl.getId());
			calculationInputs.add(calcInputs);
		}
		// Need to clone the dates here, otherwise changing values in PayrollRunInfo will affect the dates in the
		// PaycheckCalculationRequest
		paycheckCalculationRequest.setPaycheckDate((Date) payrollRunInfo.getPayDate().clone());
		DateRange period = payrollRunInfo.getPayPeriod();
		DateRange payPeriod = new DateRange(period.getStartDate(), period.getEndDate());
		paycheckCalculationRequest.setPayPeriod(payPeriod);
		paycheckCalculationRequest.setCalculationInputs(calculationInputs);
		paycheckCalculationRequest.setWorkerId(workerId);
		paycheckCalculationRequest.setFrequency(payrollRunInfo.getFrequency());
		paycheckCalculationRequest.setDistributionMethod(distributionMethod);

		return paycheckCalculationRequest;
	}

	/**
	 * Creates a {@link PaycheckCalculationRequest} based on the provided inputs.
	 * 
	 * @param worker
	 * @param frequency
	 * @param payPeriod
	 * @param calculationInputs
	 * @return
	 */
	protected PaycheckCalculationRequest createPaycheckRequest(Worker worker, PayrollFrequencyEnum frequency,
			DateRange payPeriod, List<CalculationInputs> calculationInputs) {
		PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
		paycheckCalculationRequest.setPaycheckDate(mPaycheckDate.getTime());
		paycheckCalculationRequest.setPayPeriod(payPeriod);
		paycheckCalculationRequest.setCalculationInputs(calculationInputs);
		paycheckCalculationRequest.setWorkerId(worker.getId());
		paycheckCalculationRequest.setFrequency(frequency);
		return paycheckCalculationRequest;
	}

	/**
	 * From a {@link Set} of {@link CompensationDtl}, create a {@link List} of {@link CalculationInputs}.
	 * 
	 * @param compensationDtls
	 * @return the {@link CalculationInputs}
	 */
	protected List<CalculationInputs> createCalcInputsFromCompensationDtls(Set<CompensationDtl> compensationDtls) {
		List<CalculationInputs> calculationInputs = new ArrayList<CalculationInputs>();
		for (CompensationDtl compensationDtl : compensationDtls) {
			CalculationInputs calcInputs = createCalcInputsFromCompensationDtl(compensationDtl);
			calculationInputs.add(calcInputs);
		}
		return calculationInputs;
	}

	/**
	 * From a CompensationDtl, take the rate and quantity (if available) and create a CalculationInputs instance.
	 * 
	 * @param compensationDtl
	 * @return a {@link CalculationInputs}
	 */
	protected CalculationInputs createCalcInputsFromCompensationDtl(CompensationDtl compensationDtl) {
		CalculationInputs calcInputs = new CalculationInputs();
		if (null != compensationDtl.getRate()) {
			calcInputs.setRate(compensationDtl.getRate().clone());
		}
		if (null != compensationDtl.getQuantity()) {
			calcInputs.setQuantity(compensationDtl.getQuantity().clone());
		}
		calcInputs.setSourcePayrollItemDtlId(compensationDtl.getId());
		return calcInputs;
	}

	private CompensationDtl findCompensationDtl(Set<CompensationDtl> compensationDtls, long id) {
		for (CompensationDtl compensationDtl : compensationDtls) {
			if (id == compensationDtl.getId()) {
				return compensationDtl;
			}
		}
		return null;
	}

	private void setPtoDisburseHours(Set<CompensationDtl> compensationDtls, List<CalculationInputs> calculationInputs,
			CompensationTypeEnum type, Double hours) throws Exception {
		boolean found = false;
		for (CalculationInputs calculationInput : calculationInputs) {
			CompensationDtl compensationDtl = findCompensationDtl(compensationDtls, calculationInput
					.getSourcePayrollItemDtlId());
			if ((null != compensationDtl) && (type == compensationDtl.getCompensationType())) {
				found = true;
				Datum quantityDatum = new Datum();
				quantityDatum.setValue1AsQuantity(hours);
				calculationInput.setQuantity(quantityDatum);
				break;
			}
		}
		AssertJUnit.assertTrue("Cannot find compensation " + type.toString() + " on the CalculationInputs!", found);
	}

	/**
	 * This utility routine will set the number of hours taken for the Sick compensation item
	 * 
	 * @param compensationDtls
	 *            The set of compensationDtl containing the PTO item
	 * @param calculationInputs
	 *            The list of CalculationInputs to set the vacation hours taken
	 * @param hours
	 *            The number of sick hours taken
	 * 
	 * @throws Exception
	 */
	protected void setSickDisburseHours(Set<CompensationDtl> compensationDtls,
			List<CalculationInputs> calculationInputs, Double hours) throws Exception {
		setPtoDisburseHours(compensationDtls, calculationInputs, CompensationTypeEnum.SICKTAKEN, hours);
	}

	/**
	 * This utility routine will set the number of hours taken for the Vacation compensation item
	 * 
	 * @param compensationDtls
	 *            The set of compensationDtl containing the PTO item
	 * @param calculationInputs
	 *            The list of CalculationInputs to set the vacation hours taken
	 * @param hours
	 *            The number of vacation hours taken
	 * 
	 * @throws Exception
	 */
	protected void setVacationDisburseHours(Set<CompensationDtl> compensationDtls,
			List<CalculationInputs> calculationInputs, Double hours) throws Exception {
		setPtoDisburseHours(compensationDtls, calculationInputs, CompensationTypeEnum.VACTAKEN, hours);
	}

	/**
	 * This utility method will initialize the specified compensationDtl using the specified values
	 * 
	 * @param compensationDtl
	 *            The compensationDtl to set the values
	 * @param compensationRate
	 *            The Datum to set the rate values
	 * @param compensationQuantity
	 *            The Datum to set the quantity values
	 * @param hours
	 *            The number of regular hours worked
	 * @param overtimeHours
	 *            The number of overtime hours worked
	 * @param bonusAmount
	 *            The bonus amount
	 */
	// protected void setSalaryCompensationQuantity(CompensationDtl compensationDtl, Datum compensationRate,
	// Datum compensationQuantity, Double hours, Double overtimeHours, BigDecimal bonusAmount) {
	// if (CompensationTypeEnum.SALARYREG.equals(compensationDtl.getCompensationType())) {
	// // Shouldn't need to set the rate, only need to set the hours
	// compensationQuantity.setValue1AsQuantity(hours);
	// } else if (CompensationTypeEnum.SALARYOVT.equals(compensationDtl.getCompensationType()) ||
	// CompensationTypeEnum.SALARYDBL.equals(compensationDtl.getCompensationType())) {
	// // caller will set linked compensation item
	// // quantity is simply multiplier (e.g. 1.5 or 2)
	// compensationRate.setValue1AsQuantity(compensationDtl.getRate().getValue1AsQuantity());
	// compensationQuantity.setValue1AsQuantity(overtimeHours);
	// } else {
	// // This probably doesn't cover all the cases...
	// compensationRate.setValue1AsUSDMoneyAmount(bonusAmount);
	// compensationRate.setType2(DatumType2Enum.NOTAPPLICABLE);
	// }
	// }
	/**
	 * This utility method will create a PaycheckCalculationRequest object with no rate overrides
	 * 
	 * @param workerId
	 *            The id of the worker
	 * @param payrollRunInfo
	 *            An initialized PayrollRunInfo object containing the payroll run info
	 * @param hours
	 *            The number of hours for the hourly compensation item
	 * @param overtimeHours
	 *            The number of overtime hours
	 * @param distributionMethod
	 *            The paycheck distribution method
	 * @return A PaycheckCalculationRequest initialized with the specified values
	 * @throws Exception
	 */
	protected static PaycheckCalculationRequest createSalaryPaycheckCalculationRequest(Long workerId,
			PayrollRunInfo payrollRunInfo, PayrollDistributionMethodEnum distributionMethod) throws Exception {
		PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
		paycheckCalculationRequest.setPaycheckDate(payrollRunInfo.getPayDate());
		paycheckCalculationRequest.setPayPeriod(payrollRunInfo.getPayPeriod());
		paycheckCalculationRequest.setWorkerId(workerId);
		paycheckCalculationRequest.setFrequency(payrollRunInfo.getFrequency());
		paycheckCalculationRequest.setDistributionMethod(distributionMethod);

		return paycheckCalculationRequest;
	}

	/**
	 * This utility method will clean up the specified PayrollRun from the database
	 * 
	 * @param payrollRunId
	 *            The id of the PayrollRun to cleanup
	 */
	protected static void cleanUpPayrollRun(Long payrollRunId) throws Exception {
		sWorkerPaymentsComponent.deletePayrollRun(payrollRunId);
		TaskUtils.removeAllWorkerPaymentTasks();
	}

	/**
	 * This utility method will validate the expected PayrollRunInfo values in the specified PayrollRun using the values
	 * specified in the PayrollRunRequest.
	 * 
	 * @param payrollRun
	 *            The PayrollRun to validate
	 * @param payrollRunRequest
	 *            The PayrollRunRequest containing values to validate against
	 * @return true if values match expected values, false otherwise
	 */
	protected boolean validatePayrollRun(PayrollRun payrollRun, PayrollRunCommitRequest payrollRunRequest) {
		PayrollRunInfo payrollRunInfo = payrollRunRequest.getPayrollRunInfo();
		// Check if the dates match
		Calendar cal1 = DateUtil.getNowCalendar();
		cal1.setTime(payrollRunInfo.getPayDate());
		Calendar cal2 = DateUtil.getNowCalendar();
		cal2.setTime(payrollRun.getPayDate());
		if (!DateUtil.isSameDay(cal1, cal2)) {
			return false;
		}
		cal1 = DateUtil.getNowCalendar();
		cal1.setTime(payrollRunInfo.getPayPeriod().getStartDate());
		cal2 = DateUtil.getNowCalendar();
		cal2.setTime(payrollRun.getPayPeriod().getStartDate());
		if (!DateUtil.isSameDay(cal1, cal2)) {
			return false;
		}
		cal1 = DateUtil.getNowCalendar();
		cal1.setTime(payrollRunInfo.getPayPeriod().getEndDate());
		cal2 = DateUtil.getNowCalendar();
		cal2.setTime(payrollRun.getPayPeriod().getEndDate());
		if (!DateUtil.isSameDay(cal1, cal2)) {
			return false;
		}
		cal1 = DateUtil.getNowCalendar();
		cal1.setTime(payrollRunInfo.getPayrollRunDate());
		cal2 = DateUtil.getNowCalendar();
		cal2.setTime(payrollRun.getPayrollRunDate());
		if (!DateUtil.isSameDay(cal1, cal2)) {
			return false;
		}
		// Check the Pay frequency
		if (payrollRunInfo.getFrequency() != payrollRun.getPayrollFrequency()) {
			return false;
		}
		// TODO Add other validation here

		// Everything is correct
		return true;
	}

	/**
	 * This utility method will validate the expected Paycheck values using the values specified in the reference
	 * Paycheck.
	 * 
	 * @param paycheck
	 *            The Paycheck to validate
	 * @param refPaycheck
	 *            The Paycheck containing values to validate against
	 * @return true if values match expected values, false otherwise
	 */
	protected boolean validatePaycheck(Paycheck paycheck, Paycheck refPaycheck) {
		if (!paycheck.getCreationDate().equals(refPaycheck.getCreationDate()))
			return false;
		if (!paycheck.getDistributionMethod().equals(refPaycheck.getDistributionMethod()))
			return false;
		if (!paycheck.getGrossAmount().equals(refPaycheck.getGrossAmount()))
			return false;
		if (!paycheck.getNetAmount().equals(refPaycheck.getNetAmount()))
			return false;
		if (!paycheck.getPayDate().equals(refPaycheck.getPayDate()))
			return false;
		if (!paycheck.getTxnDate().equals(refPaycheck.getTxnDate()))
			return false;
		if (!paycheck.getWorker().equals(refPaycheck.getWorker()))
			return false;
		if (!paycheck.getWorkerPdTaxes().equals(refPaycheck.getWorkerPdTaxes()))
			return false;

		// Everything is correct
		return true;
	}

	/**
	 * This utility method will create 1 PayrollRun for the specified worker using the specified PayrollSchedule
	 * 
	 * @param workerId
	 *            The worker to be used for the PayrollRun
	 * @param schedule
	 *            The payroll schedule
	 * @return A PayrollRun object
	 * @throws Exception
	 */
	protected static PayrollRun createOnePayrollRun(Long workerId, PayrollSchedule schedule) throws Exception {
		PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(schedule);
		List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
		PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(workerId, payrollRunInfo,
				new BigDecimal("10.0"), new Double(80), new Double(10), new BigDecimal("200.0"),
				PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
		paycheckCalculationRequests.add(calcRequest);
		PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
		payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
		payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
		// Set the Payroll system date to the payroll run date
		setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
		// Restore the Payroll system date to the current date
		resetPayrollSystemDate();
		return payrollRun;
	}

	/**
	 * This utility method will compare 2 Set of PaycheckLineItem for equivalence
	 * 
	 * @param expected
	 *            The Set of PaycheckLineItem to verify against
	 * @param computed
	 *            The Set of PaycheckLineItem to verify
	 * @return true if the 2 Set of PaycheckLineItem are equal
	 */
	protected boolean verifyPaycheckLineItems(Set<PaycheckLineItem> expected, Set<PaycheckLineItem> computed) {
		boolean compareResult = (expected.size() == computed.size());
		if (true == compareResult) {
			Set<PaycheckLineItem> computedLineItemSet = new HashSet<PaycheckLineItem>();
			for (PaycheckLineItem computedItem : computed) {
				computedLineItemSet.add(computedItem);
			}

			Set<PaycheckLineItem> expectedLineItemSet = new HashSet<PaycheckLineItem>();
			for (PaycheckLineItem expectedItem : expected) {
				expectedLineItemSet.add(expectedItem);
			}
			compareResult = computedLineItemSet.equals(expectedLineItemSet);
		}

		if (false == compareResult) {
			sLogger.error("Computed PaycheckLineItems:");
			for (PaycheckLineItem paycheckLineItem : computed) {
				sLogger.error(paycheckLineItem.toString());
			}
			sLogger.error("Expected PaycheckLineItems:");
			for (PaycheckLineItem paycheckLineItem : expected) {
				sLogger.error(paycheckLineItem.toString());
			}
		}

		return compareResult;
	}

	/**
	 * This utility method will compare 2 Paycheck for equivalence
	 * 
	 * @param expected
	 *            The Paycheck to verify against
	 * @param computed
	 *            The Paycheck to verify
	 * @return true if the 2 Paycheck are equal
	 */
	protected boolean verifyPaycheck(Paycheck expected, Paycheck computed) {
		boolean result = true;
		result = result && expected.getDeductionAmount().equals(computed.getDeductionAmount());
		result = result && expected.getDistributionMethod().equals(computed.getDistributionMethod());
		result = result && expected.getEmployerPdBenefits().equals(computed.getEmployerPdBenefits());
		result = result && expected.getEmployerPdTaxes().equals(computed.getEmployerPdTaxes());
		result = result && expected.getGrossAmount().equals(computed.getGrossAmount());
		if (null == expected.getMemo()) {
			result = result && (null == computed.getMemo());
		} else {
			result = result && expected.getMemo().equals(computed.getMemo());
		}
		result = result && expected.getNetAmount().equals(computed.getNetAmount());
		result = result && verifyPaycheckLineItems(expected.getPaycheckLineItems(), computed.getPaycheckLineItems());
		result = result && expected.getPayDate().equals(computed.getPayDate());
		result = result && expected.getPayPeriod().equals(computed.getPayPeriod());
		if (null == expected.getTxnDate()) {
			result = result && (null == computed.getTxnDate());
		} else {
			result = result && expected.getTxnDate().equals(computed.getTxnDate());
		}
		if (null == expected.getTxnNo()) {
			result = result && (null == computed.getTxnNo());
		} else {
			result = result && expected.getTxnNo().equals(computed.getTxnNo());
		}
		result = result && expected.getWorker().equals(computed.getWorker());
		result = result && expected.getWorkerPdBenefits().equals(computed.getWorkerPdBenefits());
		result = result && expected.getWorkerPdTaxes().equals(computed.getWorkerPdTaxes());
		if (null == expected.getWorkflowStatus()) {
			result = result && (null == computed.getWorkflowStatus());
		} else {
			result = result && expected.getWorkflowStatus().equals(computed.getWorkflowStatus());
		}
		return result;
	}

	/**
	 * This utility method will compare 2 PayrollRunInfo lists for equality
	 * 
	 * @param expected
	 *            The expected list of PayrollRunInfo to compare against
	 * @param computed
	 *            The list of PayrollRunInfo to compare
	 * @return true if the 2 lists are equal, false otherwise
	 */
	protected boolean verifyPayrollRunInfoList(List<PayrollRunInfo> expected, List<PayrollRunInfo> computed) {
		boolean result = false;
		result = expected.equals(computed);
		if (false == result) {
			sLogger.error("Computed PayrollRunInfo:");
			for (PayrollRunInfo payrollRunInfo : computed) {
				sLogger.error(payrollRunInfo);
			}
			sLogger.error("Expected PayrollRunInfo:");
			for (PayrollRunInfo payrollRunInfo : expected) {
				sLogger.error(payrollRunInfo);
			}
		}
		return result;
	}

	/**
	 * This utility method will compare 2 PayrollRunInfo for equality
	 * 
	 * @param expected
	 *            The expected PayrollRunInfo to compare against
	 * @param computed
	 *            The PayrollRunInfo to compare
	 * @return true if the 2 PayrollRunInfos are equal, false otherwise
	 */
	protected boolean verifyPayrollRunInfo(PayrollRunInfo expected, PayrollRunInfo computed) {
		boolean result = false;
		result = expected.equals(computed);
		if (false == result) {
			sLogger.error("Computed PayrollRunInfo:");
			sLogger.error(computed);
			sLogger.error("Expected PayrollRunInfo:");
			sLogger.error(expected);
		}
		return result;
	}

	/**
	 * This utility method will create a Weekly PayrollSchedule using the specified values
	 * 
	 * @param nextPayPeriodEndDate
	 *            The next pay period end date
	 * @param payrollRunDayOffset
	 *            The payrollRun day offset
	 * @param payDayOffset
	 *            The payDay offset
	 * @return A Weekly PayrollSchedule
	 */
	protected static PayrollSchedule createWeeklySchedule(Date nextPayPeriodEndDate, int payrollRunDayOffset,
			int payDayOffset) {
		PayrollSchedule payrollSchedule = new PayrollSchedule();
		payrollSchedule.setNextPayPeriodEndDate(nextPayPeriodEndDate);
		payrollSchedule.setFrequency(PayrollFrequencyEnum.WEEKLY);
		payrollSchedule.setPayrollRunDayOffset(payrollRunDayOffset);
		payrollSchedule.setPayDayOffset(payDayOffset);
		return payrollSchedule;
	}

	/**
	 * This utility method will create a Biweekly PayrollSchedule using the specified values
	 * 
	 * @param nextPayPeriodEndDate
	 *            The next pay period end date
	 * @param payrollRunDayOffset
	 *            The payrollRun day offset
	 * @param payDayOffset
	 *            The payDay offset
	 * @return A Biweekly PayrollSchedule
	 */
	protected static PayrollSchedule createBiweeklySchedule(Date nextPayPeriodEndDate, int payrollRunDayOffset,
			int payDayOffset) {
		PayrollSchedule payrollSchedule = new PayrollSchedule();
		payrollSchedule.setNextPayPeriodEndDate(nextPayPeriodEndDate);
		payrollSchedule.setFrequency(PayrollFrequencyEnum.BIWEEKLY);
		payrollSchedule.setPayrollRunDayOffset(payrollRunDayOffset);
		payrollSchedule.setPayDayOffset(payDayOffset);
		return payrollSchedule;
	}

	/**
	 * This utility method will create a Monthly PayrollSchedule using the specified values
	 * 
	 * @param nextPayPeriodEndDate
	 *            The next pay period end date
	 * @param payPeriodEndDayofMonth1
	 *            The end day of the pay period
	 * @param payrollRunDayOffset
	 *            The payrollRun day offset
	 * @param payDayOffset
	 *            The payDay offset
	 * @return A Biweekly PayrollSchedule
	 */
	protected static PayrollSchedule createMonthlySchedule(Date nextPayPeriodEndDate, int payPeriodEndDayofMonth1,
			int payrollRunDayOffset, int payDayOffset) {
		PayrollSchedule payrollSchedule = new PayrollSchedule();
		payrollSchedule.setNextPayPeriodEndDate(nextPayPeriodEndDate);
		payrollSchedule.setPayPeriodEndDayofMonth1(payPeriodEndDayofMonth1);
		payrollSchedule.setFrequency(PayrollFrequencyEnum.MONTHLY);
		payrollSchedule.setPayrollRunDayOffset(payrollRunDayOffset);
		payrollSchedule.setPayDayOffset(payDayOffset);
		return payrollSchedule;
	}

	/**
	 * This utility method will create a semimonthly PayrollSchedule using the specified values
	 * 
	 * @param nextPayPeriodEndDate
	 *            The next pay period end date
	 * @param payPeriodEndDayofMonth1
	 *            The end day of the first pay period
	 * @param payPeriodEndDayofMonth2
	 *            The end day of the second pay period
	 * @param payrollRunDayOffset
	 *            The payrollRun day offset
	 * @param payDayOffset
	 *            The payDay offset
	 * @return A Semimonthly PayrollSchedule
	 */
	protected static PayrollSchedule createSemimonthlySchedule(Date nextPayPeriodEndDate, int payPeriodEndDayofMonth1,
			int payPeriodEndDayofMonth2, int payrollRunDayOffset, int payDayOffset) {
		PayrollSchedule payrollSchedule = new PayrollSchedule();
		payrollSchedule.setNextPayPeriodEndDate(nextPayPeriodEndDate);
		payrollSchedule.setPayPeriodEndDayofMonth1(payPeriodEndDayofMonth1);
		payrollSchedule.setPayPeriodEndDayofMonth2(payPeriodEndDayofMonth2);
		payrollSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
		payrollSchedule.setPayrollRunDayOffset(payrollRunDayOffset);
		payrollSchedule.setPayDayOffset(payDayOffset);
		return payrollSchedule;
	}

	protected class AdjLineItem {
		private String lawId;
		private Money adjustmentAmount = new Money();
		private PayrollItem payrollItem;

		public AdjLineItem(String lawId, Money adjustmentAmount, PayrollItem payrollItem) {
			this.lawId = lawId;
			this.adjustmentAmount = adjustmentAmount;
			this.payrollItem = payrollItem;
		}

		public String getLawId() {
			return lawId;
		}

		public void setLawId(String lawId) {
			this.lawId = lawId;
		}

		public Money getAdjustmentAmount() {
			return adjustmentAmount;
		}

		public void setAdjustmentAmount(Money adjustmentAmount) {
			this.adjustmentAmount = adjustmentAmount;
		}

		public PayrollItem getPayrollItem() {
			return payrollItem;
		}

		public void setPayrollItem(PayrollItem payrollItem) {
			this.payrollItem = payrollItem;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + getOuterType().hashCode();
			result = prime * result + ((adjustmentAmount == null) ? 0 : adjustmentAmount.hashCode());
			result = prime * result + ((lawId == null) ? 0 : lawId.hashCode());
			result = prime * result + ((payrollItem == null) ? 0 : payrollItem.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			AdjLineItem other = (AdjLineItem) obj;
			if (!getOuterType().equals(other.getOuterType()))
				return false;
			if (adjustmentAmount == null) {
				if (other.adjustmentAmount != null)
					return false;
			} else if (!adjustmentAmount.equals(other.adjustmentAmount))
				return false;
			if (lawId == null) {
				if (other.lawId != null)
					return false;
			} else if (!lawId.equals(other.lawId))
				return false;
			if (payrollItem == null) {
				if (other.payrollItem != null)
					return false;
			} else if (!payrollItem.equals(other.payrollItem))
				return false;
			return true;
		}

		private WorkerPaymentsComponentTestBase getOuterType() {
			return WorkerPaymentsComponentTestBase.this;
		}

	}

	/**
	 * This utility method will only verify the lawId, adjustmentAmount and PayrollItem values in the calculated set of
	 * EmployerAdjLineItem against an expected set of AdjLineItem values to see if they are equal.
	 * 
	 * @param expected
	 *            A hash set of AdjLineItem containing the expected values
	 * @param calculated
	 *            The set of EmployerAdjLineItem to verify the values against the expected set
	 * @return true if the values are equal, false otherwise
	 */
	protected boolean verifyEmployerAdjustment(HashSet<AdjLineItem> expected, Set<EmployerAdjLineItem> calculated) {
		boolean result = (expected.size() == calculated.size());
		if (true == result) {
			HashSet<AdjLineItem> computedSet = new HashSet<AdjLineItem>();
			Iterator<EmployerAdjLineItem> iterator = calculated.iterator();
			while (iterator.hasNext()) {
				EmployerAdjLineItem employerAdjLineItem = iterator.next();
				AdjLineItem adjLineItem = new AdjLineItem(employerAdjLineItem.getLawId().getValue(),
						employerAdjLineItem.getAdjustmentAmount(), employerAdjLineItem.getPayrollItem());
				computedSet.add(adjLineItem);
			}
			result = expected.equals(computedSet);
		}
		if (false == result) {
			sLogger.error("Expected AdjLineItem values:");
			for (AdjLineItem adjLineItem : expected) {
				sLogger.error("LawId: " + adjLineItem.getLawId());
				sLogger.error("adjustment amount: " + adjLineItem.getAdjustmentAmount().toString());
				sLogger.error("PayrollItem: " + adjLineItem.getPayrollItem().toString());
			}
			sLogger.error("Computed EmployerAdjLineItem values:");
			for (EmployerAdjLineItem employerAdjLineItem : calculated) {
				sLogger.error("LawId: " + employerAdjLineItem.getLawId().getValue());
				sLogger.error("adjustment amount: " + employerAdjLineItem.getAdjustmentAmount().toString());
				sLogger.error("PayrollItem: " + employerAdjLineItem.getPayrollItem().toString());
			}
		}
		return result;
	}
	
	/**
	 * 
	 * This utility method will do the following for the 2 workers passed in:
	 * 		Create 25 scheduled payroll runs (for year 2008)
	 * 		Create 1 scheduled payroll run (for year 2009)
	 **/
	public static void createScheduledPayrollRuns(Long sWorkerIds[]) throws Exception {
		
		// Create 25 scheduled payroll runs (for year 2008)
		// Create 1 schedule payroll run (for year 2009)
		PayrollRunInfo payrollRunInfo = null;
		// There are a total of 25 biweekly payroll runs for the year 2008, and 1 in 2009:
		//    1st paycheck - pay period 01/01/2008 - 01/14/2008, paycheck date of 01/17/2008
		//    2nd paycheck - pay period 01/15/2008 - 01/28/2008, paycheck date of 01/31/2008
		//    3rd paycheck - pay period 01/29/2008 - 02/11/2008, paycheck date of 02/14/2008
		//    4th paycheck - pay period 02/12/2008 - 02/25/2008, paycheck date of 02/28/2008
		//    5th paycheck - pay period 02/26/2008 - 03/10/2008, paycheck date of 03/13/2008
		//    6th paycheck - pay period 03/11/2008 - 03/24/2008, paycheck date of 03/27/2008
		//    7th paycheck - pay period 03/25/2008 - 04/07/2008, paycheck date of 04/10/2008
		//    8th paycheck - pay period 04/08/2008 - 04/21/2008, paycheck date of 04/27/2008
		//    9th paycheck - pay period 04/22/2008 - 05/05/2008, paycheck date of 05/08/2008
		//   10th paycheck - pay period 05/06/2008 - 05/19/2008, paycheck date of 05/22/2008
		//   11th paycheck - pay period 05/20/2008 - 06/02/2008, paycheck date of 06/05/2008
		//   12th paycheck - pay period 06/03/2008 - 06/16/2008, paycheck date of 06/19/2008
		//   13th paycheck - pay period 06/17/2008 - 06/30/2008, paycheck date of 07/03/2008
		//   14th paycheck - pay period 07/01/2008 - 07/14/2008, paycheck date of 07/17/2008
		//   15th paycheck - pay period 07/15/2008 - 07/28/2008, paycheck date of 07/31/2008
		//   16th paycheck - pay period 07/29/2008 - 08/11/2008, paycheck date of 08/14/2008
		//   17th paycheck - pay period 08/12/2008 - 08/25/2008, paycheck date of 08/28/2008
		//   18th paycheck - pay period 08/26/2008 - 09/08/2008, paycheck date of 09/11/2008
		//   19th paycheck - pay period 09/09/2008 - 09/22/2008, paycheck date of 09/25/2008
		//   20th paycheck - pay period 09/23/2008 - 10/06/2008, paycheck date of 10/09/2008
		//   21th paycheck - pay period 10/07/2008 - 10/20/2008, paycheck date of 10/23/2008
		//   22th paycheck - pay period 10/21/2008 - 11/03/2008, paycheck date of 11/06/2008
		//   23th paycheck - pay period 11/04/2008 - 11/17/2008, paycheck date of 11/20/2008
		//   24th paycheck - pay period 11/18/2008 - 12/01/2008, paycheck date of 12/04/2008
		//   25th paycheck - pay period 12/02/2008 - 12/15/2008, paycheck date of 12/18/2008
		//   26th paycheck - pay period 12/16/2008 - 12/29/2008, paycheck date of 01/01/2009
		PayrollSchedule aSchedule = null;
		int k = 0;
		Date nextPayPeriodEndDate = DateUtil.createDate(2007, Calendar.DECEMBER, 31);
		for (int j = 0; j < numOfPayrollRuns; j++) {
			// Calculate the next pay period end date
			nextPayPeriodEndDate = DateUtil.getNextDate(nextPayPeriodEndDate, 14);
	    	aSchedule = createBiweeklySchedule(nextPayPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
	    			sDEFAULT_PAY_DAY_OFFSET);
	    	payrollRunInfo = createAPayrollRunInfo(aSchedule);
	    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
	    	PaycheckCalculationRequest calcRequest = createSalaryPaycheckCalculationRequest(sWorkerIds[0],
	    					payrollRunInfo, PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
	    	paycheckCalculationRequests.add(calcRequest);
	    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
	        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
	        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
	
	        // Set the Payroll system date to the payroll run date
	        setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
	        payrollRuns[j] = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	        
    		// Create some adhoc paychecks and 1 termination paycheck (all using the second worker)
			if ((j == 5) || (j == 11) || (j == 18) || (j == 24)) {
				PayrollRunInfo payrollRunInfo2 = new PayrollRunInfo(payrollRunInfo.getPayPeriod(),
						payrollRunInfo.getPayDate(), payrollRunInfo.getPayrollRunDate(), payrollRunInfo.getFrequency(),
						payrollRunInfo.getPayrollRunType());
				if (j == 24) {
    				payrollRunInfo2.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
				} else {
    				payrollRunInfo2.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
				}
		    	List<PaycheckCalculationRequest> paycheckCalculationRequests2 = new ArrayList<PaycheckCalculationRequest>();
		    	PaycheckCalculationRequest calcRequest2 = createSalaryPaycheckCalculationRequest(sWorkerIds[1],
		    			payrollRunInfo2, PayrollDistributionMethodEnum.HANDWRITTENPAYCHECK);
		    	paycheckCalculationRequests2.add(calcRequest2);
		    	
		    	PayrollRunCommitRequest payrollRunRequest2 = new PayrollRunCommitRequest();
		        payrollRunRequest2.setPayrollRunInfo(payrollRunInfo2);
		        payrollRunRequest2.setCalculationRequests(paycheckCalculationRequests2);
		
		        // Set the Payroll system date to the payroll run date
		        setPayrollSystemDate(payrollRunInfo2.getPayrollRunDate());
		        payrollRuns2[k++] = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest2);
			}
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
		}
		
	}
	
	public static void deleteCreatedScheduledPayrollRuns(Long sWorkerIds[]) throws Exception {
		for (int j = 0; j < numOfPayrollRuns; j++) {
    		if (payrollRuns[j] != null) {
            	Long payrollRunId = payrollRuns[j].getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
		}
		for (int j = 0; j < numOfPayrollRuns2; j++) {
			if (payrollRuns2[j] != null) {
	        	Long payrollRunId = payrollRuns2[j].getId();
	        	cleanUpPayrollRun(payrollRunId);
			}
		}
		for (int i = 0; i < sWorkerIds.length; i++) {
			if (0 != sWorkerIds[i]) {
				Worker worker = entityService.findById(Worker.class, sWorkerIds[i]);
				WorkerTestUtils.cleanUpWorker(worker);
				sWorkerIds[i] = 0L;
			}
		}
	}

}
