package com.intuit.sb.payroll.reports.impl;

import java.io.StringWriter;
import java.io.Writer;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class Rs2Xml {

	/**
	 * @param JDBC
	 *            ResultSet to be transformed into a DOM instance
	 * @return DOM instance conforming to W3C's Document interface
	 * @throws SQLException
	 */
	public static String PaymentReportRs2Xml(ResultSet rs, String periodStr) throws SQLException {
		
		ResultSetMetaData rsmd = rs.getMetaData();

		Double rowTotal[] = new Double[rsmd.getColumnCount()+1];
		ArrayList<ArrayList<String>> dataGrid = new ArrayList<ArrayList<String>>();
		
		// Create a dynamic matrix for gridRows
		
		// Add the col label row (row 0)
		dataGrid.add(new ArrayList<String>());
		
		// add the "data" rows
		for (int i=1; i < rsmd.getColumnCount(); i++ ) {
			dataGrid.add(new ArrayList<String>());
		}

		// Add the col totals row (last row)
		dataGrid.add(new ArrayList<String>());
		
		for (int i=0; i < rsmd.getColumnCount() + 1; i++ ) {
			rowTotal[i] = new Double(0.0);
		}
		
		for (int gridCol = 0; rs.next() == true; gridCol++) {
			
			// The first data grid row contains labels only
			
			if (gridCol == 0) {
				int gridRow = 0;
				// One time initialization of the leftmost column
				((ArrayList<String>)dataGrid.get(gridRow++)).add(periodStr);
				
				for (int rsCol = 2; rsCol < rsmd.getColumnCount() + 1; rsCol++, gridRow++) {
					((ArrayList<String>)dataGrid.get(gridRow)).add(rsmd.getColumnName(rsCol));
				}
				((ArrayList<String>)dataGrid.get(dataGrid.size()-1)).add("TOTALS");
				gridCol++;
			} 
			
			Double colTotal = 0.00;
			int gridRow = 0;
			for (int rsCol = 1; rsCol < rsmd.getColumnCount() + 1; rsCol++, gridRow++) {
				if (rsCol == 1) {
					((ArrayList<String>)dataGrid.get(gridRow)).add("PAYDATE " + rs.getString(rsCol));
				} else {
					((ArrayList<String>)dataGrid.get(gridRow)).add(rs.getString(rsCol));
					colTotal += rs.getDouble(rsCol);
					rowTotal[gridRow] += rs.getDouble(rsCol);
				}
			}
			((ArrayList<String>)dataGrid.get(dataGrid.size()-1)).add(colTotal.toString());
		}
		
		Double gTotal = 0.0;
		for (int gridRow = 1; gridRow < rsmd.getColumnCount() + 1; gridRow++) {
			((ArrayList<String>)dataGrid.get(gridRow)).add(rowTotal[gridRow].toString());
			gTotal += rowTotal[gridRow];
		}
		((ArrayList<String>)dataGrid.get(dataGrid.size()-1)).add(gTotal.toString());
		
		// emit the XML doc using matrix data
		Document myDocument = null;
		String xmlString = null;
		
		try {
			myDocument =
				((DocumentBuilderFactory.newInstance()).newDocumentBuilder())
					.newDocument();
		} catch (ParserConfigurationException pce) {
			
		}
		
		Element root = myDocument.createElement("ROWSET");
		myDocument.appendChild(root);

		Element element, row;

		for (int i = 0; i < dataGrid.size(); i++) {   
			row = myDocument.createElement("ROW");
			root.appendChild(row);
	    	for (int j = 0; j < ((ArrayList<String>)dataGrid.get(i)).size(); j++) {
	    		element = myDocument.createElement("COL");
				element.appendChild(
						myDocument.createTextNode((String)((ArrayList<String>)dataGrid.get(i)).get(j)));
				row.appendChild(element);
	    	}
	    }   
	
		try {
			new StreamResult();
			Writer outWriter = new StringWriter();   
			StreamResult result = new StreamResult( outWriter );   
			Transformer myTransformer =
				(TransformerFactory.newInstance()).newTransformer();
			myTransformer.transform(new DOMSource(myDocument), result ); 
			xmlString = outWriter.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return xmlString;
	}
}
