package com.intuit.sb.payroll.services.benefits;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.benefits.HealthBenefit;

/**
 * This interface contains apis for managing or retrieving Benefit specific information
 *  
 * 
 * $Author: dyee $ 
 * $Revision: #13 $
 * $DateTime: 2008/10/16 15:43:18 $
 * 
 */
@Remote
public interface BenefitsService
{
	public static final String JNDI_NAME = "BenefitsService/remote";

	/**
	 * Find a Health Benefit template for the employer.
     * @param HealthBenefitType type
     * @return a HealthBenefit template
	 * @throws ServiceFailureException if there is a problem with the database etc...
	 */
	public HealthBenefitTemplate findEmployerHealthBenefit(HealthBenefitType type) throws ServiceFailureException, ValidationException;

	/**
	 * Provides a listing of all the Health Benefits for the employer.
     * @param 
     * @return an array containing the HealthBenefit templates
	 * @throws ServiceFailureException if there is a problem with the database etc...
	 */
	public HealthBenefitTemplate[] findEmployerHealthBenefits() throws ServiceFailureException, ValidationException;

	/**
	 * Provides a listing of all the Health Benefits for the given worker.
     * @param workerHandle worker handle
     * @return an array containing the added HealthBenefits
	 * @throws ServiceFailureException if there is a problem with the database etc...
	 */
	public HealthBenefit[] findAllHealthBenefits(ObjectHandle workerHandle) throws ServiceFailureException, ValidationException;

    /**
     * Add the given HealthBenefits to the given worker.
     * @param workerHandle worker handle
     * @param healthBenefits array
     * @return an response dto containing the worker and their HealthBenefits
    */
	public HealthBenefitResponse addHealthBenefits(ObjectHandle workerHandle, HealthBenefit[] healthBenefits) throws ServiceFailureException, ValidationException; 
    
    /**
     * Update the given HealthBenefits for a worker.  
     * @param workerHandle worker handle
     * @param healthBenefits array
     * @return an response dto containing the worker and their HealthBenefits
     */
    public HealthBenefitResponse updateHealthBenefits(ObjectHandle workerHandle, HealthBenefit[] healthBenefits) throws ServiceFailureException, ValidationException;
    
    /**
     * Delete the given HealthBenefits for a worker.  
     * @param workerHandle worker handle
     * @param healthBenefits array
     * @return an response dto containing the worker and their HealthBenefits
     */
    public HealthBenefitResponse deleteHealthBenefits(ObjectHandle workerHandle, HealthBenefit[] healthBenefits) throws ServiceFailureException, ValidationException;
    
}
