package com.intuit.sb.payroll.services.common;

import java.io.Serializable;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;


/**
 * A handle to an object. This encapsulates all the properties required to obtain the model
 * entity that was used to populate the enclosing object.
 * <p>
 * The usage pattern of this class is that the server will populate the handle with the
 * "key" properties of a model entity when sending properties of that entity to a remote
 * client for an "edit" dialog. The remote client must pass this handle back, unmodified,
 * with the updates so the server can merge the updates into the original model entity.
 * <p>
 * Remote clients should consider this class an opaque object. Its properties can change
 * in future releases.
 * <p>
 * Note that the handle does not contain the type of model entity the properties apply to.
 * It is required that the server knows the appropriate model EAO to use.
 */
public class ObjectHandle implements Serializable {

	private static final long serialVersionUID = -3617696955673056962L;
	
	private long Id;
	private long version;

	/**
	 * Default constructor required for LCDS. Do not use directly.
	 * @deprecated
	 */
	public ObjectHandle()
	{
	}
	
	/**
	 * Constructs a fully populated instance. 
	 */
	public ObjectHandle(long Id, long version) {
		this.Id = Id;
		this.version = version;
	}
	
	/**
	 * Gets the Id of the containing object.
	 * @return the Id
	 */
	public long getId() {
		return Id;
	}
	/**
	 * Sets the Id of the containing object.
	 * Only provided for use by LCDS.
	 * @param Id the Id to set
	 * @deprecated
	 */
	public void setId(long Id) {
		this.Id = Id;
	}
	/**
	 * Gets the modification version of the containing object.
	 * @return the version
	 */
	public long getVersion() {
		return version;
	}
	/**
	 * Sets the modification version of the containing object.
	 * Only provided for use by LCDS.
	 * @param version the version to set
	 * @deprecated
	 */
	public void setVersion(long version) {
		this.version = version;
	}
	
	@Override
	public boolean equals(Object other) {
        if (other == null) return false;
        if (this == other) return true;
        if (!(other instanceof ObjectHandle)) return false;

        final ObjectHandle that = (ObjectHandle)other;

        if (this.Id != that.getId()) return false;
        if (this.version != that.getVersion()) return false;

        return true;
	}

	@Override
	public int hashCode() {
        int result;
        Long tempId = new Long(this.Id);
        Long tempVer = new Long(this.version);
        result = tempId.hashCode() * 27 + tempVer.hashCode();
        return result;
	}
	
	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.SIMPLE_STYLE);
	}
}
