package com.intuit.sb.payroll.services.common.impl;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.domain.EntityId;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.services.common.ObjectHandle;

/**
 * Maps a model entity's key properties to an ObjectHandle and 
 * vice versa.
 */
public class ObjectHandleMapper {
	// No instances allowed
	private ObjectHandleMapper() {}
	
	public static ObjectHandle mapToObjectHandle(Identifiable identifiable) {
		return new ObjectHandle(identifiable.getId(), identifiable.getVersion());
	}

	public static Identifiable mapToIdentifiable(ObjectHandle objectHandle) throws ValidationException {
		validateId(objectHandle.getId());
		validateVersion(objectHandle.getVersion());
		return new EntityId(objectHandle.getId(), objectHandle.getVersion());
	}
	private static void validateId(Long id) throws ValidationException {
		if (id == null) {
			throw new ValidationException("Invalid ObjectHandle - missing ID");
		}
	}
	private static void validateVersion(Long version) throws ValidationException {
		if (version == null) {
			throw new ValidationException("Invalid ObjectHandle - missing version");
		}
	}

}
