package com.intuit.sb.payroll.services.common.impl;


import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.PhysicalAddress;

public class PhysicalAddressMapper {

	private PhysicalAddressMapper() {}
	
	/**
	 * Creates a DTO address from an Entity address.
	 * 
	 * @param entityAddress
	 * @return a PhysicalAddress instance.
	 */
	public static PhysicalAddress mapAddress(USPostalAddress entityAddress) {

		if (null != entityAddress) {
			PhysicalAddress dtoAddress = new PhysicalAddress();
			dtoAddress.setAddressLine1(entityAddress.getAddressLine1());
			dtoAddress.setAddressLine2(entityAddress.getAddressLine2());
			dtoAddress.setCity(entityAddress.getCity());
			dtoAddress.setCountry(entityAddress.getCountry());
			dtoAddress.setStateOrRegion(entityAddress.getRegionOrState());
			dtoAddress.setZipOrPostalCode(entityAddress.getZipOrPostalCode());
			return dtoAddress;
		} else {
			return null;
		}
	}
	
	/**
	 * Maps from a DTO address to an Entity address.  Requires the caller to send in an existing
	 * instance of the Entity address, since there are more fields supported in the entity than
	 * the DTO and we don't want to lose them if they have been set (such as standardization/County resolution).
	 * 
	 * @param dtoAddress
	 * @param entityAddress
	 */
	public static void mapAddress(PhysicalAddress dtoAddress, USPostalAddress entityAddress) {

		if (null != dtoAddress) {
			entityAddress.setAddressLine1(dtoAddress.getAddressLine1());
			entityAddress.setAddressLine2(dtoAddress.getAddressLine2());
			entityAddress.setCity(dtoAddress.getCity());
			entityAddress.setCountry(dtoAddress.getCountry());
			//FIXME [I18n] we require this to be able to resolve jurisdictions, not if it's
			// fair to have the client pass it through. - [th]
			entityAddress.setCountryISOCode("US");
			entityAddress.setRegionOrState(dtoAddress.getStateOrRegion());
			entityAddress.setZipOrPostalCode(dtoAddress.getZipOrPostalCode());
		}
	}
}
