package com.intuit.sb.payroll.services.employer.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.takehome.entity.TakeHomeCompanyYTDChoices;
import com.intuit.sb.payroll.takehome.entity.TakehomeCompany;
import com.intuit.sb.payroll.takehome.entity.TakehomeCompanyEAO;

/**
 * Implements the Employer Info aspects of the EmployerService interface.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 */
public class EmployerInfoImpl {

    public static EmployerInfo getEmployerInfo() throws ServiceFailureException, ValidationException    {
        EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
        Employer employer = employerComponent.getEmployer();

        //Map from the Model info to the DTO
        EmployerInfo eInfo = EmployerInfoMapper.mapEmployerInfo(employer);

        //return to client.
        return eInfo;

    }

    /**
     * Adds the passed in Employer business address into the system and returns the just
     * added Employer information
     *
     * @param EmployerInfo
     * @return void
     * @throws ServiceFailureException
     * @throws ValidationException
     */
    public static EmployerInfo saveEmployerInfo(EmployerInfo employerInfo) throws ServiceFailureException, ValidationException {
        validateEmployerInfo(employerInfo);
        //Map the DTO to the Model
        Employer employer = EmployerInfoMapper.mapEmployerInfo(employerInfo);

        EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
        employer = employerComponent.saveEmployer(employer);
               
        // Map and save the application domain entity
		ObjectHandle prEmployerObjectHandle = new ObjectHandle(employer.getId(), employer.getVersion());
		saveTakeHomeCompany(employerInfo, prEmployerObjectHandle);
		
		//Map from the Model info to the DTO
        EmployerInfo eInfo = EmployerInfoMapper.mapEmployerInfo(employer);
 
        //return to client.
        return eInfo;
    }

    private static void validateEmployerInfo(EmployerInfo eInfo) throws ValidationException {
        if (null == eInfo) {
            throw new ValidationException("EmployerInfo cannot be null");
        }
    }
    
    private static TakehomeCompany saveTakeHomeCompany(EmployerInfo employerInfo, ObjectHandle employerObjectHandle) 
    throws ServiceFailureException, ValidationException {
		if (employerInfo == null) { // nothing to map
			return null;
		}

		TakehomeCompanyEAO takehomeCompanyEAO = ServiceLocator.getInstance().getService(TakehomeCompanyEAO.JNDI_NAME);
		
		TakehomeCompany companyEntity = takehomeCompanyEAO.findByPayrollEmployerId(employerObjectHandle.getId());
		if (null == companyEntity) {
			companyEntity = new TakehomeCompany();
			companyEntity.setPayrollEmployerId(employerObjectHandle.getId());
			companyEntity.setYTDPaymentChoices(new TakeHomeCompanyYTDChoices());
			companyEntity = takehomeCompanyEAO.makePersistent(companyEntity);
		} 
				
		companyEntity.setPrintPreferences(EmployerInfoMapper.mapPrintPreferencesFromDTO(employerInfo.getPrintPreferences()));
		companyEntity.setCompanyLogo(employerInfo.getCompanyLogo());
		if(employerInfo.getTakeHomeCompanyYTDChoices() != null){
			companyEntity.setYTDPaymentChoices(EmployerInfoMapper.mapTakeHomeCompanyYTDChoicesFromDTO(employerInfo.getTakeHomeCompanyYTDChoices()));
		}
		takehomeCompanyEAO.flush();
		return companyEntity;
	}
}
