package com.intuit.sb.payroll.services.pto;

import java.io.Serializable;
import java.util.Date;

import com.intuit.sb.payroll.services.common.ObjectHandle;

public class PTOInfo implements Serializable {
	private static final long serialVersionUID = 2612174081395478768L;
	
	private ObjectHandle objectHandle;
	private Long workerId;
	private String name;
	private PTOType ptoType;
	/**
	 * accrualRate is hours earned per year unless the ptoAccrualType is 
	 * HOURLY, in which case it is hours earned per hour
	 */
	private Double accrualRate;
	private PTOAccrualType ptoAccrualType;
	private boolean limitAccrual;
	private boolean allowCarryOver;
	private Double maximumAccrual;
	private Double accruedHours;
	private Double disbursedHoursYTD;
	private AccrualStartType accrualStartType;
	private Date accrualStartDate;
	
	/**
	 * Default Constructor
	 */
	public PTOInfo() {
		super();
	}
	
	/**
	 * @param workerId
	 * @param name
	 * @param ptoType
	 * @param accrualRate
	 * @param ptoAccrualType
	 * @param limitAccrual
	 * @param allowCarryOver
	 * @param maximumAccrual
	 * @param accruedHours
	 * @param disbursedHoursYTD
	 * @param accrualStartType
	 * @param accrualStartDate
	 */
	public PTOInfo(ObjectHandle objectHandle, Long workerId, String name, 
			PTOType ptoType, Double accrualRate, PTOAccrualType ptoAccrualType,
			boolean limitAccrual, boolean allowCarryOver,
			Double maximumAccrual, Double accruedHours, 
			Double disbursedHoursYTD, AccrualStartType accrualStartType, 
			Date accrualStartDate) {
		super();
		this.workerId = workerId;
		this.name = name;
		this.ptoType = ptoType;
		this.accrualRate = accrualRate;
		this.ptoAccrualType = ptoAccrualType;
		this.limitAccrual = limitAccrual;
		this.allowCarryOver = allowCarryOver;
		this.maximumAccrual = maximumAccrual;
		this.accruedHours = accruedHours;
		this.disbursedHoursYTD = disbursedHoursYTD;
		this.accrualStartType = accrualStartType;
		this.accrualStartDate = accrualStartDate;
	}
	
	/**
	 * @param objectHandle
	 */
	public void setObjectHandle(ObjectHandle objectHandle) {
		this.objectHandle = objectHandle;
	}

	/**
	 * @return the ObjectHandle
	 */
	public ObjectHandle getObjectHandle() {
		return objectHandle;
	}

	/**
	 * @return the workerId
	 */
	public Long getWorkerId() {
		return workerId;
	}
	/**
	 * @param workerId the workerId to set
	 */
	public void setWorkerId(Long workerId) {
		this.workerId = workerId;
	}
	/**
	 * @return the ptoType
	 */
	public PTOType getPtoType() {
		return ptoType;
	}
	/**
	 * @param ptoType the ptoType to set
	 */
	public void setPtoType(PTOType ptoType) {
		this.ptoType = ptoType;
	}
	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * Get the accrual rate, which is measured in hours/year except for hourly
	 * workers, for whom it is measured in hours/hour worked.
	 * @return the accrualRate
	 */
	public Double getAccrualRate() {
		return accrualRate;
	}
	/**
	 * Set the accrual rate, which is measured in hours/year except for hourly
	 * workers, for whom it is measured in hours/hour worked.
	 * @param accrualRate the accrualRate to set
	 */
	public void setAccrualRate(Double accrualRate) {
		this.accrualRate = accrualRate;
	}
	/**
	 * @return the ptoAccrualType
	 */
	public PTOAccrualType getPtoAccrualType() {
		return ptoAccrualType;
	}
	/**
	 * @param ptoAccrualType the ptoAccrualType to set
	 */
	public void setPtoAccrualType(PTOAccrualType ptoAccrualType) {
		this.ptoAccrualType = ptoAccrualType;
	}
	/**
	 * @return the limitAccrual
	 */
	public boolean isLimitAccrual() {
		return limitAccrual;
	}
	/**
	 * @param limitAccrual the limitAccrual to set
	 */
	public void setLimitAccrual(boolean limitAccrual) {
		this.limitAccrual = limitAccrual;
	}
	/**
	 * @return the allowCarryOver
	 */
	public boolean isAllowCarryOver() {
		return allowCarryOver;
	}
	/**
	 * @param allowCarryOver the allowCarryOver to set
	 */
	public void setAllowCarryOver(boolean allowCarryOver) {
		this.allowCarryOver = allowCarryOver;
	}
	/**
	 * @return the maximumAccrual
	 */
	public Double getMaximumAccrual() {
		return maximumAccrual;
	}
	/**
	 * @param maximumAccrual the maximumAccrual to set
	 */
	public void setMaximumAccrual(Double maximumAccrual) {
		this.maximumAccrual = maximumAccrual;
	}
	/**
	 * @return the accruedHours
	 */
	public Double getAccruedHours() {
		return accruedHours;
	}
	/**
	 * @param accruedHours the accruedHours to set
	 */
	public void setAccruedHours(Double accruedHours) {
		this.accruedHours = accruedHours;
	}
	/**
	 * @return the disbursedHoursYTD
	 */
	public Double getDisbursedHoursYTD() {
		return disbursedHoursYTD;
	}
	/**
	 * @param disbursedHoursYTD the disbursedHoursYTD to set
	 */
	public void setDisbursedHoursYTD(Double disbursedHoursYTD) {
		this.disbursedHoursYTD = disbursedHoursYTD;
	}	
	/**
	 * @return the accrualStartDate
	 */
	public Date getAccrualStartDate() {
		return accrualStartDate;
	}
	/**
	 * @param accrualStartDate the accrualStartDate to set
	 */
	public void setAccrualStartDate(Date accrualStartDate) {
		this.accrualStartDate = accrualStartDate;
	}	
	/**
	 * @return the accrualStartType
	 */
	public AccrualStartType getAccrualStartType() {
		return accrualStartType;
	}
	/**
	 * @param accrualStartType the accrualStartType to set
	 */
	public void setAccrualStartType(AccrualStartType accrualStartType) {
		this.accrualStartType = accrualStartType;
	}
}
