package com.intuit.sb.payroll.services.pto.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.pto.PTOInfo;
import com.intuit.sb.payroll.services.pto.PTOType;

/**
 * Validators for PTO service API parameters.
 */
public class PTOParamValidator {


	/**
	 * Validates a ptoType to ensure it has a value.
	 * @param ptoType the type to validate.
	 * @throws ValidationException if any of the validation requirements aren't met.
	 */
	static void validatePtoType(PTOType ptoType) throws ValidationException
	{
		if (ptoType == null) {
			//FIXME [I18N] resource translation
			throw new ValidationException("PTO type is null");
		}
	}
	
	/**
	 * Validates a PTOInfo to ensure it confirms to minimum requirements:
	 * <ul>
	 * <li>the PTOInfo itself is non-null</li>
	 * <li>if there is a non-null workerId it must be valid</li>
	 * <li>all enums must be non-null</li>
	 * <li>name must have a value</li>
	 * </ul>
	 * Note that it is NOT a requirement that hoursAvailable be <= maxHours. This allows
	 * payroll admins to force a workers available hours above their configured maximum
	 * if they want to.
	 * @param ptoInfo the ptoInfo to validate.
	 * @throws ValidationException if any of the validation requirements aren't met.
	 */
	static void validatePto(PTOInfo ptoInfo) throws ValidationException, ServiceFailureException
	{
		if (ptoInfo == null) {
			//FIXME [I18N] resource translation
			throw new ValidationException("PtoInfo is null");
		}

		if (ptoInfo.getWorkerId() != null) {
			CommonValidators.validateId(ptoInfo.getWorkerId());
		}
		
		if (ptoInfo.getAccrualStartType() == null) {
			//FIXME [I18N] resource translation
			throw new ValidationException("Accrual start type is null");
		}
		if (ptoInfo.getPtoAccrualType() == null) {
			//FIXME [I18N] resource translation
			throw new ValidationException("Accrual type is null");
		}
		
		if (ptoInfo.getPtoType() == null) {
			//FIXME [I18N] resource translation
			throw new ValidationException("PTO type is null");
		}

	}
}
