package com.intuit.sb.payroll.services.taxes.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.EffectivePeriod;
import com.intuit.sb.payroll.entity.model.EffectivePeriodType;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.UnemploymentInsDocDtl;
import com.intuit.sb.payroll.services.common.DatePeriod;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.taxes.EmployerUnemploymentIns;
import com.intuit.sb.payroll.services.taxes.SuiSetting;

public class SuiTaxMapper {

	public static SuiSetting mapToSuiSetting(UnemploymentInsDocDtl suiTax,
			Jurisdiction jurisdiction) {
		if (suiTax == null || jurisdiction == null) {
			return null;
		}

		SuiSetting suiSetting = new SuiSetting();
		if (suiTax.getPercentageRate() != null) {
			suiSetting.setCurrentRate(suiTax.getPercentageRate().toString());
		} else {
			suiSetting.setCurrentRate("");
		}

		if (suiTax.getPercentageFutureRate() != null) {
			suiSetting.setFutureRate(suiTax.getPercentageFutureRate()
					.toString());
		} else {
			suiSetting.setFutureRate("");
		}

		suiSetting.setFutureRateStartDate(suiTax.getFutureRateStartDate());
		suiSetting.setRateChangedByIntuit(suiTax.isRateChangedByIntuit());
		suiSetting.setState(jurisdiction.getName());
		suiSetting.setUniqueId(suiTax.getId());
		suiSetting.setDefaultRate(suiTax.isDefaultRate());
		suiSetting.setMaxRate(suiTax.getMaxRate());
		suiSetting.setMinRate(suiTax.getMinRate());
		return suiSetting;
	}
	
	public static EmployerUnemploymentIns mapToEmployerUnemploymentIns(UnemploymentInsDocDtl suiTax,
			Jurisdiction jurisdiction) {
		if (suiTax == null || jurisdiction == null) {
			return null;
		}

		EmployerUnemploymentIns employerUnemploymentIns = new EmployerUnemploymentIns();
		if (suiTax.getPercentageRate() != null) {
			employerUnemploymentIns.setEffectiveRate(suiTax.getPercentageRate().toString());
		} else {
			employerUnemploymentIns.setEffectiveRate("");
		}

		
		employerUnemploymentIns.setRateChangedByIntuit(suiTax.isRateChangedByIntuit());
		employerUnemploymentIns.setState(jurisdiction.getName());
		employerUnemploymentIns.setObjectHandle(new ObjectHandle(suiTax.getId(), suiTax.getVersion()));
		employerUnemploymentIns.setDefaultRate(suiTax.isDefaultRate());
		employerUnemploymentIns.setMaxRate(suiTax.getMaxRate());
		employerUnemploymentIns.setMinRate(suiTax.getMinRate());
		employerUnemploymentIns.setEffectivePeriod(maptoDatePeriod(suiTax.getEffectivePeriod().getDateRange()));
		return employerUnemploymentIns;
	}
	
	public static UnemploymentInsDocDtl mapToUnemploymentInsDocDtl(EmployerUnemploymentIns employerUnemployment) throws ServiceFailureException{
		UnemploymentInsDocDtl unemploymentInsDocDtl = null;
		TaxComponent taxComponent = ServiceLocator.getInstance().getService(TaxComponent.JNDI_NAME);
		if (employerUnemployment.getObjectHandle() != null){
				 unemploymentInsDocDtl = taxComponent.getEmployerUnemploymentInsurance(employerUnemployment
						.getObjectHandle().getId());
		}else{
			unemploymentInsDocDtl = new UnemploymentInsDocDtl();
			EffectivePeriod effectivePeriod = new EffectivePeriod();
			effectivePeriod.setDateRange(new DateRange());
			effectivePeriod.setTypeOf(EffectivePeriodType.QUARTER);
			unemploymentInsDocDtl.setEffectivePeriod(effectivePeriod);

		}
		
		// Fill in the data
		unemploymentInsDocDtl.setPercentageRate(Double.valueOf(employerUnemployment.getEffectiveRate()));
		unemploymentInsDocDtl.setRateChangedByIntuit(employerUnemployment.isRateChangedByIntuit());
		unemploymentInsDocDtl.getEffectivePeriod().getDateRange().setEndDate(employerUnemployment.getEffectivePeriod().getPeriodEndDate());
		unemploymentInsDocDtl.getEffectivePeriod().getDateRange().setStartDate(employerUnemployment.getEffectivePeriod().getPeriodStartDate());
		return unemploymentInsDocDtl;
	}
	
	
	private static DatePeriod maptoDatePeriod(DateRange dateRange){
		DatePeriod datePeriod = new DatePeriod();
		datePeriod.setPeriodStartDate(dateRange.getStartDate());
		datePeriod.setPeriodEndDate(dateRange.getEndDate());
		return datePeriod;
	}
	
	

	
}
