package com.intuit.sb.payroll.services.taxes.impl;

import java.util.Collection;

import com.intuit.sb.payroll.capability.ppe.AgencyView;
import com.intuit.sb.payroll.services.taxes.TaxDetails;
import com.intuit.sb.payroll.services.taxes.TaxSummary;

/**
 * This class holds all the helper functions that are needed for mapping TaxDetails
 * to and from various object modes like the PPE and the core payroll model
 * @author rjohn2
 *
 */
public class TaxDetailsMapper 
{
	/**
	 * This function maps the AgencyView to TaxDetails 
	 * @param agencyView of type AgencyView
	 * @return array of TaxDetails
	 */
	public static TaxDetails mapToTaxDetails(AgencyView agencyView)
	{
		TaxDetails details = new TaxDetails();
		
		//map agency name
		String agencyName = agencyView.getDefaultAgencyName();
		int endIndex = agencyName.indexOf('#') + 1; //strip off the unwanted text if any after # send fromPPE
		agencyName.substring(0, endIndex);
		details.setAgencyName(agencyName);
		
		//map Jurisdiction
		String jurisdictionName = agencyView.getJusrisdiction();
		details.setJurisdictionName(jurisdictionName);
		
		//map TINs
		String accountNumber = agencyView.getTinNumber();
		if(accountNumber == null)
		{
			accountNumber = ""; //set as empty string for UI
		}
		details.setAgencyTIN(accountNumber);
		
		//map agency Urls
		String agencyUrl = agencyView.getAgencyUrl();
		details.setAgencyUrl(agencyUrl);
		
		// Tax Mappings
		Collection<AgencyView.TaxRowView> taxRows = agencyView.getOrderedTaxRows();
		
		//initialize he array to hold the taxSummary
		TaxSummary[] taxes = new TaxSummary[taxRows.size()];
		
		//now iterate through the taxes
		int rowNum = 0;
		for (AgencyView.TaxRowView row : taxRows) 
        {
			TaxSummary summary = new TaxSummary();
			taxes[rowNum] = summary;
			
			String taxName = row.getTaxName();
        	taxes[rowNum].setShortDescription(taxName);

        	//map GUIDs
        	long uniqueId = row.getErTaxGuid();
        	taxes[rowNum].setUniqueId(uniqueId);
        	
        	String companyRate = row.getCompanyRate();
        	if(companyRate.compareTo("") == 0) //TOD [rj] -this hard coding needs to be removed as well
        	{
        		companyRate = "None";
        	}
        	taxes[rowNum].setCompanyTaxRate(companyRate);
        	
        	String employeeRate = row.getEmployeeRate();
        	if(employeeRate.compareTo("Table") == 0) //TOD [rj] -this hard coding needs to be removed as well
        	{
        		employeeRate = "Varies by employee";
        	}
        	else if(employeeRate.compareTo("") == 0) //TOD [rj] -this hard coding needs to be removed as well
        	{
        		companyRate = "None";
        	}
        	taxes[rowNum].setEmployeeRate(employeeRate);
        	
        	String wageLimit = row.getWageLimit();
        	taxes[rowNum].setWageLimit(wageLimit);
        	
        	boolean editable = row.isEditable();
        	taxes[rowNum].setEditable(editable);
        	
            rowNum++;
        }
		details.setTaxSummaryCollection(taxes);

		return details;
	}
}
