package com.intuit.sb.payroll.services.taxpayments;

import java.io.Serializable;
import java.util.Date;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.services.common.ObjectHandle;

/**
 * Service class to store information on a single YTD Tax Payment made by the user.<br>
 * The Tax Payment consists of the the paymentDate - indicating when the payment was made<br>
 * and an Array of Tax Payment Details - {@code TaxPaymentLineItem}.  Each detail item <br>
 * gives information on the type of payment and the amount of payment {@code BigDecimal}.
 * <br><br>
 * The actual complete detail of TaxPaymentLineItem for the Employer will consist of a number of {@code TaxPaymentLineItem} objects<br>
 * since there may be multiple payments made over a year as well as the liabilities incurred by the Employer 
 *
 */
public class TaxPayments implements Serializable {

	private static final long serialVersionUID = -7433264751660612796L;

	protected Date taxPaymentDate;
	
	protected TaxPaymentLineItem[] taxPaymentLineItems = {};	//	always have empty array (never null)

	protected ObjectHandle objectHandle;
	
	private boolean sentToPTS;

	public Date getTaxPaymentDate() {
		return taxPaymentDate;
	}

	public void setTaxPaymentDate(Date taxPaymentDate) {
		this.taxPaymentDate = taxPaymentDate;
	}

	public TaxPaymentLineItem[] getTaxPaymentLineItems() {
		return taxPaymentLineItems;
	}

	public void setTaxPaymentLineItems(TaxPaymentLineItem[] taxPayments) {
		if (taxPayments == null) {
			taxPayments = new TaxPaymentLineItem[]{};	//	empty array
		}
		this.taxPaymentLineItems = taxPayments;
	}
	
	public void addTaxPayments(TaxPaymentLineItem taxPayment) {
		taxPaymentLineItems[taxPaymentLineItems.length] = taxPayment;
	}

	public ObjectHandle getObjectHandle() {
		return objectHandle;
	}

	public void setObjectHandle(ObjectHandle objectHandle) {
		this.objectHandle = objectHandle;
	}	

	public boolean isSentToPTS() {
		return sentToPTS;
	}

	public void setSentToPTS(boolean sentToPTS) {
		this.sentToPTS = sentToPTS;
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}	
}
