package com.intuit.sb.payroll.services.wages;

import java.io.Serializable;
import java.math.BigDecimal;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * Each CompensationItem is sent from the client as a set of values (such as rate
 * per hour and number of hours). The server calculates the total amount
 * of money earned for the CompensationItem and returns it as a CompensationItemTotal.
 *
 */
public class CompensationItemTotal implements Serializable
{

	private static final long serialVersionUID = -7064210391930184124L;
	private BigDecimal total;
    private CompensationItem   compensationItem;

    /**
     * Default constructor
     * @deprecated
     */
    public CompensationItemTotal()
    {
    }
    
    /**
     * Constructs compensationItem total.
     * @param total  total as BigDecimal.
     * @param compensationItem  CompensationItem associated with the total.
     * @deprecated
     */
    public CompensationItemTotal(BigDecimal total, CompensationItem compensationItem)
    {
    	this.total = total;
    	this.compensationItem = compensationItem;
    }

	/**
	 * @return the total, the computed compensationItem values.
     * @deprecated
	 */
	public BigDecimal getTotal() {
		return total;
	}

	/**
	 * @param total the total to set.
     * @deprecated
	 */
	public void setTotal(BigDecimal total) {
		this.total = total;
	}

	/**
	 * @return the CompensationItem.
     * @deprecated
	 */
	public CompensationItem getCompensationItem() {
		return compensationItem;
	}

	/**
	 * @param compensationItem the CompensationItem to set.
     * @deprecated
	 */
	public void setCompensationItem(CompensationItem compensationItem) {
		this.compensationItem = compensationItem;
	}
	
    @Override
    public boolean equals(Object other)
    {
        return EqualsBuilder.reflectionEquals(this, other);
    }
    
    @Override
    public int hashCode()
    {
       return HashCodeBuilder.reflectionHashCode(this);
    }
    
	@Override
	public String toString()
	{
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}    
}
