/**
 * 
 */
package com.intuit.sb.payroll.services.wages;

import com.intuit.sb.payroll.services.common.EnumBase;

/**
 * CompensationType represents the list of built in CompensationTypes
 * Currently only few wage types are hard coded, we need to have the complete list
 * This should be auto generated from the CompensationTypes
 */
public class CompensationType extends EnumBase 
{
	private static final long serialVersionUID = 2848924813257791430L;

	public final static CompensationType Salary           = new CompensationType("Salary",0,CompensationTypeQualifier.RATE_BASED,"1000013");
    public final static CompensationType Hourly           = new CompensationType("Hourly",1,CompensationTypeQualifier.RATE_BASED,"1000012");
    public final static CompensationType Overtime         = new CompensationType("Overtime",2,CompensationTypeQualifier.RATE_BASED,"1000026");
    public final static CompensationType Doubletime       = new CompensationType("Doubletime",3,CompensationTypeQualifier.RATE_BASED,"1000003");
    public final static CompensationType Bonus            = new CompensationType("Bonus",4,CompensationTypeQualifier.AMOUNT_BASED,"1000005");
    public final static CompensationType Commission       = new CompensationType("Commission",5,CompensationTypeQualifier.AMOUNT_BASED,"1000025");
    public final static CompensationType TipsPaycheck    = new CompensationType("TipsPaycheck",6,CompensationTypeQualifier.AMOUNT_BASED,"1000004");
    public final static CompensationType TipsAllocated   = new CompensationType("TipsAllocated",7,CompensationTypeQualifier.AMOUNT_BASED,"1000048");
    public final static CompensationType TipsCash        = new CompensationType("TipsCash",8,CompensationTypeQualifier.AMOUNT_BASED,"1000041");
    public final static CompensationType Piecework       = new CompensationType("Piecework",9,CompensationTypeQualifier.RATE_BASED,"1000015");
    public final static CompensationType SickTaken 		= new CompensationType("SickTaken",10,CompensationTypeQualifier.RATE_BASED,"1000021");
    public final static CompensationType VacationTaken 	= new CompensationType("VacationTaken",11,CompensationTypeQualifier.RATE_BASED,"1000019");
    public final static CompensationType Reimbursement 	= new CompensationType("Reimbursement",14,CompensationTypeQualifier.AMOUNT_BASED,"1000045");
    
    
    private CompensationTypeQualifier qualifier;  
    private String  lawId;           // Specifies the Law corresponding to the CompensationType

    private CompensationType(String name, int ordinal, CompensationTypeQualifier qualifier, String lawId)
    {
    	super(name,ordinal);
    	this.qualifier      = qualifier;
    	this.lawId          = lawId;
    }
    
	/** 
	 * Public no-arg constructor is required by LCDS.  Without it, you get an exception at
	 * runtime with the following message: "Types must have a public, no arguments constructor".
	 * This should never be used by hand-written code.
	 * @deprecated
	 */
	public CompensationType()
	{
		super();
	}

	/**
	 * @return the Qualifier associated with the CompensationType 
	 */
	public CompensationTypeQualifier getQualifier() {
		return qualifier;
	}

	/**
	 * set the Qualifier of the CompensationType
	 */
	public void setQualifier(CompensationTypeQualifier qualifier) {
		this.qualifier = qualifier;
	}


	/**
	 * @return the lawId of the CompensationType
	 */
	public String getLawId() {
		return lawId;
	}

	/**
	 * set the lawId of the CompensationType
	 */
	public void setLawId(String lawId) {
		this.lawId = lawId;
	}

}
