package com.intuit.sb.payroll.services.wages;

import java.io.Serializable;

/**
 * The input request for a saveCompensationItems call.
 */
public class SaveCompensationItemRequest implements Serializable {

	private static final long serialVersionUID = 7547867171659658491L;

	private CompensationItem compensationItem;
	private SaveCompensationItemOperation compensationItemOperation;

	
	/**
	 * No args constructor required for LCDS.
	 */
	public SaveCompensationItemRequest() {
		super();
	}

	/**
	 * Convenience constructor to instantiate a complete instance of this class.
	 * @param compensationItem The contained CompensationItem for a request.
	 * @param compensationItemOperation The operation to perform upon the CompensationItem.
	 */
	public SaveCompensationItemRequest(CompensationItem compensationItem,
			SaveCompensationItemOperation compensationItemOperation) {
		super();
		this.compensationItem = compensationItem;
		this.compensationItemOperation = compensationItemOperation;
	}

	/**
	 * Gets the CompensationItem contained in this request.
	 * @return the CompensationItem.
	 */
	public CompensationItem getCompensationItem() {
		return compensationItem;
	}

	/**
	 * Sets the CompensationItem for this request.
	 * @param compensationItem the CompensationItem.
	 */
	public void setCompensationItem(CompensationItem compensationItem) {
		this.compensationItem = compensationItem;
	}

	/**
	 * Gets the operation for the contained CompensationItem.
	 * @return the operation.
	 */
	public SaveCompensationItemOperation getCompensationItemOperation() {
		return compensationItemOperation;
	}

	/**
	 * Sets the operation for the contained CompensationItem. 
	 * @param compensationItemOperation the operation.
	 */
	public void setCompensationItemOperation(
			SaveCompensationItemOperation compensationItemOperation) {
		this.compensationItemOperation = compensationItemOperation;
	}
	
}
