package com.intuit.sb.payroll.services.worker;

import java.io.Serializable;
import java.util.Date;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.services.benefits.HealthBenefit;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.pto.PTOInfo;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;


public class Worker implements Serializable, Comparable<Worker> {

	private static final long serialVersionUID = 3819316073004725187L;
	
	// properties from the payroll domain
	private ObjectHandle objectHandle;
	private String firstName;
	private String middleName;
	private String lastName;
	private String jobTitle;
	private Date hireDate;
	private Date terminationDate;
	private EmploymentStatusEnum employmentStatus;
	private Date birthDate;
	private WorkerType workerType;
	private String phoneNumber;
	private String emailAddress;
	private PhysicalAddress taxResidenceAddress;
	private WorkLocation workLocation;
	private boolean isReadyToPay;
	private boolean isAlreadyPaid;
	private boolean active;
	private String[] missingReadyToPayProperties;
	private PaycheckDistributionMethod paycheckDistributionMethod;
	private HealthBenefit[] healthBenefits;
	private PTOInfo[] ptoInfos;
	private CompensationItem[] compensationItems;
	private String workerTaxIdNumber;
	private String wagePlanCode;
	
	
	// Properties from the application domain
	private ObjectHandle takehomeWorkerObjectHandle;
	private String avatar;
	private byte[] workerImage;

	public Worker() {
		super();
		
	}
	
	public Worker(ObjectHandle objectHandle, String firstName, String middleName,
			String lastName, String jobTitle, Date hireDate, Date birthDate,
			WorkerType workerType,
			PhysicalAddress taxResidenceAddress, WorkLocation workLocation, PaycheckDistributionMethod paycheckDistributionMethod,
			ObjectHandle takehomeWorkerObjectHandle, String avatar) {
		this();
		this.objectHandle = objectHandle;
		this.firstName = firstName;
		this.middleName = middleName;
		this.lastName = lastName;
		this.jobTitle = jobTitle;
		this.hireDate = hireDate;
		this.birthDate = birthDate;
		this.workerType = workerType;
		this.taxResidenceAddress = taxResidenceAddress;
		this.workLocation = workLocation;
		this.paycheckDistributionMethod = paycheckDistributionMethod;
		this.takehomeWorkerObjectHandle = takehomeWorkerObjectHandle;
		this.avatar = avatar;
	}
	

	public ObjectHandle getObjectHandle() {
		return objectHandle;
	}

	public void setObjectHandle(ObjectHandle objectHandle) {
		this.objectHandle = objectHandle;
	}

	public String getFirstName() {
		return firstName;
	}
	
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}
	
	public String getMiddleName() {
		return middleName;
	}
	
	public void setMiddleName(String middleName) {
		this.middleName = middleName;
	}
	
	public String getLastName() {
		return lastName;
	}
	
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}
	
	public String getJobTitle() {
		return jobTitle;
	}
	
	public void setJobTitle(String jobTitle) {
		this.jobTitle = jobTitle;
	}
	
	public Date getHireDate() {
		return hireDate;
	}
	
	public void setHireDate(Date hireDate) {
		this.hireDate = hireDate;
	}
	
	public Date getTerminationDate() {
		return terminationDate;
	}

	public void setTerminationDate(Date terminationDate) {
		this.terminationDate = terminationDate;
	}

	public EmploymentStatusEnum getEmploymentStatus() {
		return employmentStatus;
	}

	public void setEmploymentStatus(EmploymentStatusEnum employmentStatus) {
		this.employmentStatus = employmentStatus;
	}

	public WorkerType getWorkerType() {
		return workerType;
	}
	
	public void setWorkerType(WorkerType workerType) {
		this.workerType = workerType;
	}
	
	
	public WorkLocation getWorkLocation() {
		return workLocation;
	}

	public void setWorkLocation(WorkLocation workLocation) {
		this.workLocation = workLocation;
	}
    
    public Date getBirthDate() {
		return birthDate;
	}

	public void setBirthDate(Date birthDate) {
		this.birthDate = birthDate;
	}

	public boolean isReadyToPay()
	{
		return isReadyToPay;
	}

	public void setReadyToPay(boolean isReadyToPay)
	{
		this.isReadyToPay = isReadyToPay;
	}

	public boolean isAlreadyPaid()
	{
		return isAlreadyPaid;
	}

	public void setAlreadyPaid(boolean isAlreadyPaid)
	{
		this.isAlreadyPaid = isAlreadyPaid;
	}

	public String[] getMissingReadyToPayProperties() {
		return missingReadyToPayProperties;
	}
	                                                  
	public void setMissingReadyToPayProperties(String[] missingReadyToPayProperties) {
		this.missingReadyToPayProperties = missingReadyToPayProperties;
	}
	
	public String getPhoneNumber() {
		return phoneNumber;
	}

	public void setPhoneNumber(String phoneNumber) {
		this.phoneNumber = phoneNumber;
	}

	public String getEmailAddress() {
		return emailAddress;
	}

	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	public PhysicalAddress getTaxResidenceAddress() {
		return taxResidenceAddress;
	}

	public void setTaxResidenceAddress(PhysicalAddress taxResidenceAddress) {
		this.taxResidenceAddress = taxResidenceAddress;
	}

	public ObjectHandle getTakehomeWorkerObjectHandle() {
		return takehomeWorkerObjectHandle;
	}

	public void setTakehomeWorkerObjectHandle(ObjectHandle takehomeWorkerObjectHandle) {
		this.takehomeWorkerObjectHandle = takehomeWorkerObjectHandle;
	}
	
	public PaycheckDistributionMethod getPaycheckDistributionMethod(){
		return paycheckDistributionMethod;
	}

	public void setPaycheckDistributionMethod(PaycheckDistributionMethod paycheckDistributionMethod) {
		this.paycheckDistributionMethod = paycheckDistributionMethod;
	}
	
	/**
	 * Always null. The worker's HealthBenefits are never returned form the server
	 * as part of the Worker DTO.
	 * @return the healthBenefits
	 */
	public HealthBenefit[] getHealthBenefits() {
		return healthBenefits;
	}

	/**
	 * Sets the HealthBenefits for the worker.
	 * If the parameter is null the worker's current HealthBenefits
	 * are not set unchanged. 
	 *   
	 * @param healthBenefits the HealthBenefits to set.
	 */
	public void setHealthBenefits(HealthBenefit[] healthBenefits) {
		this.healthBenefits = healthBenefits;
	}

	/**
	 * Always null. The worker's PTOInfos are never returned form the server
	 * as part of the Worker DTO.
	 * @return the ptoInfos
	 */
	public PTOInfo[] getPtoInfos() {
		return ptoInfos;
	}

	/**
	 * Sets the PTOInfos for the worker.
	 * If the parameter is null the worker's current PTOInfos
	 * are not set unchanged. 
	 *   
	 * @param ptoInfos the PTOInfos to set.
	 */
	public void setPtoInfos(PTOInfo[] ptoInfos) {
		this.ptoInfos = ptoInfos;
	}

	/**
	 * Always null. The worker's CompensationItems are never returned form the server
	 * as part of the Worker DTO.
	 * @return the compensationItems
	 */
	public CompensationItem[] getCompensationItems() {
		return compensationItems;
	}

	/**
	 * Sets the CompensationItems for the worker.
	 * If the parameter is null the worker's current CompensationItems
	 * are not set unchanged. 
	 *   
	 * @param compensationItems the CompensationItems to set.
	 */
	public void setCompensationItems(CompensationItem[] compensationItems) {
		this.compensationItems = compensationItems;
	}

	public String getAvatar() {
		return avatar;
	}

	public void setAvatar(String avatar) {
		this.avatar = avatar;
	}
	
	public byte[] getWorkerImage() {
		return workerImage;
	}

	public void setWorkerImage(byte[] workerImage) {
		this.workerImage = workerImage;
	}

	public void setActive(boolean active) {
		this.active = active;
	}

	public boolean isActive() {
		return active;
	}

	public void setWorkerTaxIdNumber(String workerTaxIdNumber) {
		this.workerTaxIdNumber = workerTaxIdNumber;
	}

	public String getWorkerTaxIdNumber() {
		return workerTaxIdNumber;
	}

	public void setWagePlanCode(String wagePlanCode) {
		this.wagePlanCode = wagePlanCode;
	}

	public String getWagePlanCode() {
		return wagePlanCode;
	}

	/**
     * @param other the other object to compare against
     */
    public boolean equals(Object other)
    {
        return EqualsBuilder.reflectionEquals(this, other);
    }

    @Override
    public int hashCode()
    {
       return HashCodeBuilder.reflectionHashCode(this);
    }
    
    public int compareTo(Worker worker) {
        return this.firstName.compareTo(worker.firstName);
    }
    
	@Override
	public String toString()
	{
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}
}
