package com.intuit.sb.payroll.services.worker.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.common.impl.ObjectHandleMapper;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.takehome.entity.TakehomeWorker;
import com.intuit.sb.payroll.takehome.entity.TakehomeWorkerEAO;

/**
 * Implementation of the SaveWorker contract as described in IWorkerService.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
class SaveWorkerImpl {

	/**
	 * Public . Currently also doubles as the Controller and WorkflowManager.
	 * 
	 * @param saveWorkerRequest
	 * @return
	 * @throws ValidationException
	 * @throws WorkflowException
	 * @throws ServiceFailureException
	 */
	
	static Worker saveWorker(WorkerComponent workerComponent, Worker worker) throws ValidationException, WorkflowException,
			ServiceFailureException {

		if (worker != null) {
			//Map and save the Domain Service Entity
			com.intuit.sb.payroll.entity.model.Worker workerEntity = null;
			
			boolean newWorker = false;
			
			ObjectHandle workerUniqueId = worker.getObjectHandle();
			if (workerUniqueId == null || workerUniqueId.getId() == 0L) {
				workerEntity = new Employee();
				newWorker = true;
			} else {
				CommonValidators.validateObjectHandle(workerUniqueId);
				Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
				workerEntity = workerComponent.findWorkerByIdentifiable(identifiable);
			}

			workerEntity = setEntityWorkerProperties(worker, workerEntity);
			workerEntity = setComplianceDocs(worker, workerEntity);
			//FIXME [th] this is circumventing the WageService logic and may not work.
			workerEntity = WorkerMapper.mapFromPojoPayrollItemsToWorker(worker, workerEntity);
			
			if (newWorker) {
				workerEntity = workerComponent.addWorker(workerEntity);
			} else {
				workerEntity = workerComponent.updateWorker(workerEntity);
			}
			

			// Map and save the application domain entity
			ObjectHandle prWorkerObjectHandle = new ObjectHandle(workerEntity.getId(), workerEntity.getVersion());
			saveTakeHomeWorker(worker, prWorkerObjectHandle);
			
			//Map back to the DTOs
			Worker savedWorker = WorkerMapper.mapFromEntityToPojo(workerEntity);
			
			return savedWorker;
		} else {
			throw new ServiceFailureException("worker cannot be null");
		}
	}


	private static TakehomeWorker saveTakeHomeWorker(Worker worker, ObjectHandle payrollWorkerObjectHandle)
			throws ValidationException, ServiceFailureException {

		TakehomeWorker takehomeWorker = null;
		if (worker == null) { // nothing to map
			return null;
		}

		TakehomeWorkerEAO workerEAO = ServiceLocator.getInstance().getService(TakehomeWorkerEAO.JNDI_NAME);
		
		ObjectHandle workerUniqueId = worker.getTakehomeWorkerObjectHandle();
		if (workerUniqueId == null || workerUniqueId.getId() == 0L) {
			takehomeWorker = new TakehomeWorker();
			takehomeWorker.setPayrollWorkerId(payrollWorkerObjectHandle.getId()); // only set on construction
			takehomeWorker = workerEAO.makePersistent(takehomeWorker);
		} else {
			CommonValidators.validateObjectHandle(workerUniqueId);
			Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
			takehomeWorker = workerEAO.findByIdentifiable(identifiable);
		}

		takehomeWorker.setAvatar(worker.getAvatar());
		takehomeWorker.setWorkerImage(worker.getWorkerImage());
		workerEAO.flush();
		return takehomeWorker;
	}

	static com.intuit.sb.payroll.entity.model.Worker setEntityWorkerProperties(Worker worker, com.intuit.sb.payroll.entity.model.Worker prWorker)
			throws ValidationException, WorkflowException, ServiceFailureException {
		// Convert to Entity
		prWorker = WorkerMapper.mapFromPojoToWorker(worker, prWorker);

		// Apply Taxes
		TaxComponent taxComponent = ServiceLocator.getInstance().getService(TaxComponent.JNDI_NAME);
		prWorker = taxComponent.applyApplicableTaxes(prWorker);
		return prWorker;
	}

	/**
	 * For CA workers, maps the wage plan code to the compliance doc. Either the code is new and a new
	 * compliance doc is created or there's already a doc and it's updated with the current code value.
	 * There is no support for removing a wage plan compliance doc.
	 * 
	 * @param worker
	 * @param prWorker
	 * @return the updated prWorker with a wage plan doc if appropriate.
	 * @throws ValidationException
	 * @throws WorkflowException
	 * @throws ServiceFailureException
	 */
	private static com.intuit.sb.payroll.entity.model.Worker setComplianceDocs(Worker worker, com.intuit.sb.payroll.entity.model.Worker prWorker)
		throws ValidationException, WorkflowException, ServiceFailureException {
		
//		String wagePlanCode = worker.getWagePlanCode();
//		if (wagePlanCode != null && wagePlanCode.length() != 0) {
//			WorkerComplianceDocDtl workerComplianceDocDtl = null;
//			TaxWithholdingsComponent taxComponent = ServiceLocator.getInstance().getService(TaxWithholdingsComponent.JNDI_NAME);
//			prWorker = taxComponent.addWorkerComplianceDocDtl(prWorker, workerComplianceDocDtl);
//		}
		
		return prWorker;
	}
}
