package com.intuit.sb.payroll.services.workerpayments;

import java.io.Serializable;
import java.math.BigDecimal;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.services.worker.WorkerType;

/**
 * Summary data on a single calculated paycheck. This may or may not represent a
 * committed and dispersed payment.
 */
public class PaycheckSummary implements Serializable {

	private static final long serialVersionUID = 7823219325105197149L;
	private Long paycheckId;
	private Long workerId;
	private String firstName;
	private String lastName;
	private String hours;
	private WorkerType workerType;
	private BigDecimal grossPay;
	private BigDecimal netPay;
	private BigDecimal employeePaidTaxes;
	private BigDecimal employeePaidBenefits;
	private PaycheckDistributionMethod distributionMethod;
	private String checkNumber;
	private String memo;
	private PaycheckStatus paycheckStatus;

	/**
	 * @return the distributionMethod
	 */
	public PaycheckDistributionMethod getDistributionMethod() {
		return distributionMethod;
	}

	/**
	 * @param distributionMethod
	 *            the distributionMethod to set
	 */
	public void setDistributionMethod(PaycheckDistributionMethod distributionMethod) {
		this.distributionMethod = distributionMethod;
	}

	/**
	 * @return the grossPay
	 */
	public BigDecimal getGrossPay() {
		return grossPay;
	}

	/**
	 * @param grossPay
	 *            the grossPay to set
	 */
	public void setGrossPay(BigDecimal grossPay) {
		this.grossPay = grossPay;
	}

	/**
	 * @return the netPay
	 */
	public BigDecimal getNetPay() {
		return netPay;
	}

	/**
	 * @param netPay
	 *            the netPay to set
	 */
	public void setNetPay(BigDecimal netPay) {
		this.netPay = netPay;
	}

	/**
	 * @return the paycheckId
	 */
	public Long getPaycheckId() {
		return paycheckId;
	}

	/**
	 * @param paycheckId
	 *            the paycheckId to set
	 */
	public void setPaycheckId(Long paycheckId) {
		this.paycheckId = paycheckId;
	}

	/**
	 * @return FirstName
	 */
	public String getFirstName() {
		return firstName;
	}

	/**
	 * @param firstName
	 *            FirstName to set
	 */
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	/**
	 * @return LastName
	 */
	public String getLastName() {
		return lastName;
	}

	/**
	 * @param lastName
	 *            LastName to set
	 */
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	/**
	 * @param workerType
	 *            workerType to set
	 */
	public void setWorkerType(WorkerType workerType) {
		this.workerType = workerType;
	}

	/**
	 * @return workerType
	 */
	public WorkerType getWorkerType() {
		return workerType;
	}

	/**
	 * @param workerId
	 *            Id of worker
	 */
	public void setWorkerId(Long workerId) {
		this.workerId = workerId;
	}

	/**
	 * @return Id of worker
	 */
	public Long getWorkerId() {
		return workerId;
	}

	public String getHours() {
		return hours;
	}

	public void setHours(String hours) {
		this.hours = hours;
	}

	public BigDecimal getEmployeePaidTaxes() {
		return employeePaidTaxes;
	}

	public void setEmployeePaidTaxes(BigDecimal employeePaidTaxes) {
		this.employeePaidTaxes = employeePaidTaxes;
	}

	public BigDecimal getEmployeePaidBenefits() {
		return employeePaidBenefits;
	}

	public void setEmployeePaidBenefits(BigDecimal employeePaidBenefits) {
		this.employeePaidBenefits = employeePaidBenefits;
	}

	public String getCheckNumber() {
		return checkNumber;
	}

	public void setCheckNumber(String checkNumber) {
		this.checkNumber = checkNumber;
	}

	public String getMemo() {
		return memo;
	}

	public void setMemo(String memo) {
		this.memo = memo;
	}

	public PaycheckStatus getPaycheckStatus() {
		return paycheckStatus;
	}

	public void setPaycheckStatus(PaycheckStatus paycheckStatus) {
		this.paycheckStatus = paycheckStatus;
	}

	@Override
	public boolean equals(Object other) {
		return EqualsBuilder.reflectionEquals(this, other);
	}

	@Override
	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}
}
