package com.intuit.sb.payroll.services.workerpayments;

import java.io.Serializable;
import java.util.Date;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * This class define a parameters of a Payroll Run time period like the 
 * start date for which the worker will be paid, or the Period end Date, 
 * the date on which the actual Pay will be made
 * 
 * $Author: dkulkarni $ 
 * $Revision: #3 $ 
 * $DateTime: 2008/05/29 06:00:33 $
 * 
 */
public class PayrollRunPeriod implements Serializable
{

	private static final long serialVersionUID = -4892069223097532560L;
	private Date periodStartDate;
	private Date periodEndDate;
	private Date payDate;
	private Date payrollRunDate;
	private PayrollScheduleFrequency frequency;
	private boolean partialPeriod = false;
	
	public Date getPayrollRunDate() {
		return payrollRunDate;
	}
	public void setPayrollRunDate(Date payrollRunDate) {
		this.payrollRunDate = payrollRunDate;
	}
	/**
	 * @return the payDate
	 */
	public Date getPayDate() {
		return payDate;
	}
	/**
	 * @param payDate the payDate to set
	 */
	public void setPayDate(Date payDate) {
		this.payDate = payDate;
	}
	/**
	 * @return the periodEndDate
	 */
	public Date getPeriodEndDate() {
		return periodEndDate;
	}
	/**
	 * @param periodEndDate the periodEndDate to set
	 */
	public void setPeriodEndDate(Date periodEndDate) {
		this.periodEndDate = periodEndDate;
	}
	/**
	 * @return the periodStartDate
	 */
	public Date getPeriodStartDate() {
		return periodStartDate;
	}
	/**
	 * @param periodStartDate the periodStartDate to set
	 */
	public void setPeriodStartDate(Date periodStartDate) {
		this.periodStartDate = periodStartDate;
	}
	
	/**
	 * @return  Frequency component
	 */
	public PayrollScheduleFrequency getFrequency()
	{
		return this.frequency;
	}

	/**
	 * @param frequency  frequency of the payment
	 */
	public void setFrequency(PayrollScheduleFrequency frequency)
	{
		this.frequency = frequency;
	}	
	
	public void setPartialPeriod(boolean partialPeriod) {
		this.partialPeriod = partialPeriod;
	}
	
	public boolean isPartialPeriod() {
		return partialPeriod;
	}
	
    @Override
    public boolean equals(Object other)
    {
        return EqualsBuilder.reflectionEquals(this, other);
    }
    
    @Override
    public int hashCode()
    {
       return HashCodeBuilder.reflectionHashCode(this);
    }
    
	@Override
	public String toString()
	{
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}


}
