package com.intuit.sb.payroll.services.workerpayments;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;

/**
 * Summary information on a payroll run. This is designed to be fairly lightweight and convey the bare minimum overview
 * of the payroll run. It gives the details regarding the total Amount paid to Employees by this PayrollRun Total Amount
 * of Taxes Paid State Payroll Run is in either draft or committed
 */
public class PayrollRunSummary implements Serializable {

	private static final long serialVersionUID = 1597987888166220554L;
	private Long payrollRunId;
	private PayrollRunPeriod payrollPeriod;
	private PayrollRunState payrollRunState;
	private PayrollRunTypeEnum payrollRunType;
	private PaycheckSummary[] paycheckSummaries;
	private BigDecimal totalEmployeeNet;
	private BigDecimal totalEmployeeGross;
	private BigDecimal totalEmployeePaidTax;
	private BigDecimal totalEmployerPaidTax;
	private BigDecimal totalEmployeePaidBenefits;
	private BigDecimal totalEmployerPaidBenefits;
	private BigDecimal taxPaymentAmount;
	private Date taxPaymentWithdrawalDate;
	private String description;

	/**
	 * @return payrollRunId
	 */
	public Long getPayrollRunId() {
		return payrollRunId;
	}

	/**
	 * @param payrollRunId
	 *            the payrollRunId
	 */
	public void setPayrollRunId(Long payrollRunId) {
		this.payrollRunId = payrollRunId;
	}

	/**
	 * @return PaycheckSummary of each Employee
	 */
	public PaycheckSummary[] getPaycheckSummaries() {
		return paycheckSummaries;
	}

	/**
	 * 
	 * @param paycheckSummaries
	 *            list of paycheck summaries of employees
	 */
	public void setPaycheckSummaries(PaycheckSummary[] paycheckSummaries) {
		this.paycheckSummaries = paycheckSummaries;
	}

	/**
	 * @return the payrollPeriod
	 */
	public PayrollRunPeriod getPayrollPeriod() {
		return payrollPeriod;
	}

	/**
	 * @param payrollPeriod
	 *            the payrollPeriod to set
	 */
	public void setPayrollPeriod(PayrollRunPeriod payrollPeriod) {
		this.payrollPeriod = payrollPeriod;
	}

	/**
	 * @return the totalEmployeePaid
	 */
	public BigDecimal getTotalEmployeeGross() {
		return totalEmployeeGross;
	}

	/**
	 * @param totalEmployeePaid
	 *            the totalEmployeePaid to set
	 */
	public void setTotalEmployeeGross(BigDecimal amount) {
		this.totalEmployeeGross = amount;
	}

	/**
	 * @return the totalTaxesPaid
	 */
	public BigDecimal getTotalEmployeePaidTax() {
		return totalEmployeePaidTax;
	}

	/**
	 * @param totalTaxesPaid
	 *            the totalTaxesPaid to set
	 */
	public void setTotalEmployeePaidTax(BigDecimal totalTaxesPaid) {
		this.totalEmployeePaidTax = totalTaxesPaid;
	}

	/**
	 * @return the payrollRunState
	 */
	public PayrollRunState getPayrollRunState() {
		return payrollRunState;
	}

	/**
	 * @param payrollRunState
	 *            the payrollRunState to set
	 */
	public void setPayrollRunState(PayrollRunState payrollRunState) {
		this.payrollRunState = payrollRunState;
	}

	/**
	 * @return the employerPaidTax
	 */
	public BigDecimal getTotalEmployerPaidTax() {
		return totalEmployerPaidTax;
	}

	/**
	 * @param employerPaidTax
	 *            the employerPaidTax to set
	 */
	public void setTotalEmployerPaidTax(BigDecimal employerPaidTax) {
		this.totalEmployerPaidTax = employerPaidTax;
	}

	public BigDecimal getTotalEmployeeNet() {
		return totalEmployeeNet;
	}

	public void setTotalEmployeeNet(BigDecimal totalEmployeeNet) {
		this.totalEmployeeNet = totalEmployeeNet;
	}

	public BigDecimal getTotalEmployeePaidBenefits() {
		return totalEmployeePaidBenefits;
	}

	public void setTotalEmployeePaidBenefits(BigDecimal totalEmployeePaidBenefits) {
		this.totalEmployeePaidBenefits = totalEmployeePaidBenefits;
	}

	public BigDecimal getTotalEmployerPaidBenefits() {
		return totalEmployerPaidBenefits;
	}

	public void setTotalEmployerPaidBenefits(BigDecimal totalEmployerPaidBenefits) {
		this.totalEmployerPaidBenefits = totalEmployerPaidBenefits;
	}

	public void setPayrollRunType(PayrollRunTypeEnum payrollRunType) {
		this.payrollRunType = payrollRunType;
	}

	public PayrollRunTypeEnum getPayrollRunType() {
		return payrollRunType;
	}

	/**
	 * @return the taxPaymentAmount
	 */
	public BigDecimal getTaxPaymentAmount() {
		return taxPaymentAmount;
	}

	/**
	 * @param taxPaymentAmount the taxPaymentAmount to set
	 */
	public void setTaxPaymentAmount(BigDecimal taxPaymentAmount) {
		this.taxPaymentAmount = taxPaymentAmount;
	}

	/**
	 * @param taxPaymentWithdrawalDate the taxPaymentWithdrawalDate to set
	 */
	public void setTaxPaymentWithdrawalDate(Date taxPaymentWithdrawalDate) {
		this.taxPaymentWithdrawalDate = taxPaymentWithdrawalDate;
	}

	/**
	 * @return the taxPaymentWithdrawalDate
	 */
	public Date getTaxPaymentWithdrawalDate() {
		return taxPaymentWithdrawalDate;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getDescription() {
		return description;
	}

	@Override
	public boolean equals(Object other) {
		return EqualsBuilder.reflectionEquals(this, other);
	}

	@Override
	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}
}
