package com.intuit.sb.payroll.services.workerpayments.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculationRequest;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckUpdateRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunCommitRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunUpdateRequest;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PaycheckWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.pts.entity.PTSPayrollRun;
import com.intuit.sb.payroll.pts.entity.PTSPayrollRunEAO;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PaycheckStatus;
import com.intuit.sb.payroll.services.workerpayments.PaycheckSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunCreateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunResponse;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunState;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;

/**
 * Utility class for mapping payroll run to service level PayrollRunResponse and PayrollRunSummary
 *
 * $Author: oblackburn $
 * $Revision: #2 $
 * $DateTime: 2008/05/27 16:28:11 $
 */
public class PayrollRunMapper {
    
    /**
     * Converts Component PayrollRun to Service PayrollRunResponse
     *
     * @param payrollRun for which a PayrollRunResponse is required
     * @return
     */
    static PayrollRunResponse mapPayrollRunToPayrollRunResponse(PayrollRun payrollRun) {
        PayrollRunResponse compositeResponse = new PayrollRunResponse();

        compositeResponse.setPayrollRunId(payrollRun.getId());

        compositeResponse.setPayrollRunPeriod(createPayrollRunPeriodFromPayrollRun(payrollRun));

        Iterator<Paycheck> iter = payrollRun.getPaychecks().iterator();
        PaycheckDetails[] paycheckDetails = new PaycheckDetails[payrollRun.getPaychecks().size()];

        int i = 0;
        while (iter.hasNext()) {
            Paycheck entityPaycheck = iter.next();
            PaycheckDetails eiPaycheckDetails = PaycheckMapper.mapPaycheckToPaycheckDetails(entityPaycheck);
            paycheckDetails[i++] = eiPaycheckDetails;
        }
        compositeResponse.setPaycheckDetails(paycheckDetails);
        compositeResponse.setPayrollRunState(getPayrollRunState(payrollRun.getWorkflowStatus()));
        compositeResponse.setPayrollRunType(payrollRun.getTypeOf());
        compositeResponse.setDescription(payrollRun.getDescription());

        return compositeResponse;
    }

    /**
     * Maps PayrollRun entity to PayrollRunSummary POJO object
     *
     * @param payrollRun entity object, assumption is this will not be null
     * @return PayrollRunSummary
     */
    public static PayrollRunSummary mapPayrollRunToPayrollRunSummary(PayrollRun payrollRun)
    throws ValidationException, ServiceFailureException {
        int i = 0;

        PayrollRunSummary summary = new PayrollRunSummary();

        // Go through all paychecks and get the worker name, gross pay, net pay and distribution method
        Set<Paycheck> payChecks = payrollRun.getPaychecks();
        PaycheckSummary[] paycheckSummary = new PaycheckSummary[payrollRun.getPaychecks().size()];

        for (Paycheck paycheck : payChecks) {
            PaycheckSummary pcSummary = createPaycheckSummaryFromPaycheck(paycheck);
            paycheckSummary[i++] = pcSummary;
        }
        summary.setPaycheckSummaries(paycheckSummary);
        summary.setTotalEmployeeGross(payrollRun.getTotalGrossPay().getAmount());
        summary.setTotalEmployeeNet(payrollRun.getTotalNetPay().getAmount());
        summary.setTotalEmployeePaidTax(payrollRun.getTotalWorkerPdTaxes().getAmount());
        summary.setTotalEmployerPaidTax(payrollRun.getTotalEmployerPdTaxes().getAmount());
        summary.setTotalEmployeePaidBenefits(payrollRun.getTotalWorkerPdBenefits().getAmount());
        summary.setTotalEmployerPaidBenefits(payrollRun.getTotalEmployerPdBenefits().getAmount());
        summary.setPayrollRunId(payrollRun.getId());
        summary.setPayrollPeriod(createPayrollRunPeriodFromPayrollRun(payrollRun));
        summary.setPayrollRunState(getPayrollRunState(payrollRun.getWorkflowStatus()));
        summary.setPayrollRunType(payrollRun.getTypeOf());
        summary.setDescription(payrollRun.getDescription());

        PTSPayrollRunEAO payrollRunEAO = ServiceLocator.getInstance().getService(PTSPayrollRunEAO.JNDI_NAME);
        PTSPayrollRun ptsPayrollRun = payrollRunEAO.findPTSPayrollRunByPayrollRunId(payrollRun.getId());
        if(ptsPayrollRun != null) {
        	summary.setTaxPaymentAmount(ptsPayrollRun.getTaxDebitAmount());
        	summary.setTaxPaymentWithdrawalDate(ptsPayrollRun.getSettlementDate());
        }
        return summary;
    }

    private static PaycheckSummary createPaycheckSummaryFromPaycheck(Paycheck paycheck) 
    throws ValidationException, ServiceFailureException 
    {
        PaycheckSummary pcSummary = new PaycheckSummary();
        pcSummary.setPaycheckId(paycheck.getId());

        Worker worker = paycheck.getWorker();
        com.intuit.sb.payroll.services.worker.Worker workerDTO = WorkerMapper.mapFromEntityToPojo(worker);

        pcSummary.setFirstName(workerDTO.getFirstName());
        pcSummary.setLastName(workerDTO.getLastName());
        pcSummary.setWorkerId(workerDTO.getObjectHandle().getId());
        pcSummary.setWorkerType(workerDTO.getWorkerType());
        pcSummary.setDistributionMethod(PayrollDistributionMethodMapper.mapEntityToInterface(paycheck.getDistributionMethod()));
        pcSummary.setGrossPay(paycheck.getGrossAmount().getAmount());
        pcSummary.setNetPay(paycheck.getNetAmount().getAmount());
        pcSummary.setEmployeePaidBenefits(paycheck.getWorkerPdBenefits().getAmount());
        pcSummary.setEmployeePaidTaxes(paycheck.getWorkerPdTaxes().getAmount());
        pcSummary.setCheckNumber(paycheck.getTxnNo());
        pcSummary.setMemo(paycheck.getMemo());
        pcSummary.setPaycheckStatus(mapPaycheckWorkflowStatus(paycheck.getWorkflowStatus()));
        pcSummary.setHours(getHoursFromPaycheck(paycheck));
        return pcSummary;
    }

    private static String getHoursFromPaycheck(Paycheck paycheck) {
    	double hours = 0;
		String result = "NOTSET";
		Set<PaycheckLineItem> paycheckLineItems = paycheck.getPaycheckLineItems();
		for (PaycheckLineItem paycheckLineItem : paycheckLineItems) {
			if (paycheckLineItem.getSourcePayrollItemDtl() instanceof CompensationDtl) {
				CompensationDtl compensation = (CompensationDtl) paycheckLineItem.getSourcePayrollItemDtl();
				switch (compensation.getCompensationType()) {
				case HOURLYDBL:
				case HOURLYOVT:
				case HOURLYREG:
				case SICKTAKEN:
				case VACTAKEN:
					hours = hours + paycheckLineItem.getQuantity().getValue1AsQuantity();
				default:
					if (result.equals("NOTSET")) {
						result = compensation.getDescription();
					} else {
						result = "Other";
					}
				}
			}
		}
		
		if (hours > 0) {
			result = String.valueOf(hours);
		}
		return result;
	}

    private static PayrollRunPeriod createPayrollRunPeriodFromPayrollRun(PayrollRun payrollRun) {
        PayrollRunPeriod period = new PayrollRunPeriod();
        period.setPeriodStartDate(payrollRun.getPayPeriod().getStartDate());
        period.setPeriodEndDate(payrollRun.getPayPeriod().getEndDate());
        period.setPayDate(payrollRun.getPayDate());
        period.setPayrollRunDate(payrollRun.getPayrollRunDate());
        period.setFrequency(PayrollFrequencyMapper.mapFromDEntityToEI(payrollRun.getPayrollFrequency()));
        period.setPartialPeriod(payrollRun.isPartialPeriod());
        return period;
    }

    public static PayrollRunPeriod createPayrollRunPeriodFromPayrollRunInfo(com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo payrollRunInfo) {
        PayrollRunPeriod period = new PayrollRunPeriod();
        period.setPeriodStartDate(payrollRunInfo.getPayPeriod().getStartDate());
        period.setPeriodEndDate(payrollRunInfo.getPayPeriod().getEndDate());
        period.setPayDate(payrollRunInfo.getPayDate());
        period.setPayrollRunDate(payrollRunInfo.getPayrollRunDate());
        period.setFrequency(PayrollFrequencyMapper.mapFromDEntityToEI(payrollRunInfo.getFrequency()));
        period.setPartialPeriod(payrollRunInfo.isPartialPayPeriod());
        return period;
    }
    
    private static PayrollRunState getPayrollRunState(PayrollRunWorkflowStatusEnum entityStatus) {
        PayrollRunState status = null;
        switch (entityStatus)
        {
            case DRAFT:
                status = PayrollRunState.DRAFT;
                break;
            case COMMITTED:
                status = PayrollRunState.COMPLETE;
                break;
            case CANCELED:
                status = PayrollRunState.CANCELLED;
                break;
            case NOTSET:
                status = PayrollRunState.NOT_SET;
                break;
            default:
            	status = PayrollRunState.NOT_SET;
            	break;
        }
        return status;
    }

	public static PayrollRunCommitRequest mapPayrollRunSaveRequestToPayrollRunRequest(PayrollRunSaveRequest payrollRunRequest) throws ValidationException {
		
		PayrollRunCommitRequest capabilityRequest = new PayrollRunCommitRequest();
		PayrollRunInfo info = mapPayrollRunPeriodToPayrollRunInfo(payrollRunRequest.getPayrollRunPeriod());
		info.setPayrollRunType(payrollRunRequest.getPayrollRunType());
		info.setDescription(payrollRunRequest.getDescription());
		PaycheckRequest[] paycheckRequests = payrollRunRequest.getPaycheckRequests();
		List<PaycheckCalculationRequest> capabilityPaycheckRequests = new ArrayList<PaycheckCalculationRequest>(paycheckRequests.length);
		for (PaycheckRequest paycheckRequest : paycheckRequests) {
			PaycheckCalculationRequest capabilityPaycheckRequest = PaycheckMapper.mapPaycheckRequestToPaycheckCalculationRequest(paycheckRequest);
			capabilityPaycheckRequests.add(capabilityPaycheckRequest);
		}
		
		capabilityRequest.setPayrollRunId(payrollRunRequest.getPayrollRunId());
		capabilityRequest.setPayrollRunInfo(info);
		capabilityRequest.setCalculationRequests(capabilityPaycheckRequests);
		return capabilityRequest;
	}
	
	public static com.intuit.sb.payroll.capability.workerpayments.PayrollRunCreateRequest mapPayrollRunCreateRequest(PayrollRunCreateRequest createRequest)
	{
		com.intuit.sb.payroll.capability.workerpayments.PayrollRunCreateRequest capabilityRequest = new com.intuit.sb.payroll.capability.workerpayments.PayrollRunCreateRequest();
		PayrollRunInfo payrollRunInfo = null;
		if (createRequest.getPeriod() != null)
		{
			payrollRunInfo = mapPayrollRunPeriodToPayrollRunInfo(createRequest.getPeriod());
		}
		else
		{
			payrollRunInfo = new PayrollRunInfo();
		}
		payrollRunInfo.setPayrollRunType(createRequest.getPayrollRunType());
		
		List<Long> workerIds = new ArrayList<Long>();
		for(Long workerId : createRequest.getWorkerIds())
		{
			workerIds.add(new Long(workerId));
		}
		capabilityRequest.setWorkerIdList(workerIds);
		capabilityRequest.setPayrollRunInfo(payrollRunInfo);
		
		return capabilityRequest;
	}

	private static PayrollRunInfo mapPayrollRunPeriodToPayrollRunInfo(PayrollRunPeriod payrollRunPeriod) {
		PayrollRunInfo info = new PayrollRunInfo();
		
		DateRange dateRange = new DateRange();
		dateRange.setStartDate(payrollRunPeriod.getPeriodStartDate());
		dateRange.setEndDate(payrollRunPeriod.getPeriodEndDate());
		info.setPayPeriod(dateRange);
		info.setPayrollRunDate(payrollRunPeriod.getPayrollRunDate());
		info.setPayDate(payrollRunPeriod.getPayDate());
		info.setFrequency(PayrollFrequencyMapper.mapFromEIToDEntity(payrollRunPeriod.getFrequency()));
		info.setPartialPayPeriod(payrollRunPeriod.isPartialPeriod());
		return info;
	}

	public static PayrollRunUpdateRequest mapPayrollRunUpdateRequest(
			com.intuit.sb.payroll.services.workerpayments.PayrollRunUpdateRequest updateRequest) {
		PayrollRunUpdateRequest capabilityRequest = new PayrollRunUpdateRequest();
		capabilityRequest.setPayrollRunId(updateRequest.getPayrollRunId());
		capabilityRequest.setPaycheckUpdateRequests(mapPaycheckUpdateRequests(updateRequest.getPaycheckUpdateRequests()));
		return capabilityRequest;
	}

	private static List<PaycheckUpdateRequest> mapPaycheckUpdateRequests(
			com.intuit.sb.payroll.services.workerpayments.PaycheckUpdateRequest[] paycheckUpdateRequests) {
		List<PaycheckUpdateRequest> results = new ArrayList<PaycheckUpdateRequest>(paycheckUpdateRequests.length);
		for (com.intuit.sb.payroll.services.workerpayments.PaycheckUpdateRequest paycheckUpdateRequest : paycheckUpdateRequests) {
			PaycheckUpdateRequest newRequest = new PaycheckUpdateRequest();
			newRequest.setCheckNumber(paycheckUpdateRequest.getCheckNumber());
			newRequest.setMemo(paycheckUpdateRequest.getMemo());
			newRequest.setPaycheckId(paycheckUpdateRequest.getPaycheckId());
			newRequest.setPaycheckStatus(mapPaycheckWorkflowStatus(paycheckUpdateRequest.getPaycheckStatus()));
			results.add(newRequest);
		}
		return results;
	}

	public static List<Long> mapVoidPaychecksRequest(Long[] paycheckIdArray) {
		
		List<Long> paycheckIdList = new ArrayList<Long>(paycheckIdArray.length);
		for (Long paycheckId :paycheckIdArray) {
			paycheckIdList.add(paycheckId);
		}
		
		return paycheckIdList;
	}
	
	/**
	 * FIXME [Mapping] as this evolves, add more enums.
	 * @param paycheckStatus 
	 * @return
	 */
	private static PaycheckWorkflowStatusEnum mapPaycheckWorkflowStatus(PaycheckStatus paycheckStatus) {
		if (PaycheckStatus.COMPLETE == paycheckStatus) {
			return PaycheckWorkflowStatusEnum.DISTRIBUTED;
		} else if (PaycheckStatus.COMMITTED == paycheckStatus) {
				return PaycheckWorkflowStatusEnum.COMMITTED;
		} else if (PaycheckStatus.VOIDED == paycheckStatus) {
			return PaycheckWorkflowStatusEnum.VOIDED;
		} else {
			return PaycheckWorkflowStatusEnum.NOTSET;
		}
	}

	/**
	 * FIXME [Mapping] as this evolves, add more enums.
	 * @param workflowStatus
	 * @return
	 */
	public static PaycheckStatus mapPaycheckWorkflowStatus(PaycheckWorkflowStatusEnum workflowStatus) {
		if (PaycheckWorkflowStatusEnum.DISTRIBUTED.equals(workflowStatus)) {
			return PaycheckStatus.COMPLETE;
		} else if (PaycheckWorkflowStatusEnum.COMMITTED.equals(workflowStatus)) {
			return PaycheckStatus.COMMITTED;
		} else if (PaycheckWorkflowStatusEnum.VOIDED.equals(workflowStatus)) {
			return PaycheckStatus.VOIDED;
		} else {
			return PaycheckStatus.NOT_SET;
		}
	}

	public static com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo mapPayrollRunToPayrollRunInfo(PayrollRun payrollRun) {
		com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo compositeResponse = new com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo();

        compositeResponse.setPayrollRunId(payrollRun.getId());
        compositeResponse.setPayrollRunPeriod(createPayrollRunPeriodFromPayrollRun(payrollRun));
        compositeResponse.setPayrollRunState(getPayrollRunState(payrollRun.getWorkflowStatus()));
        compositeResponse.setPayrollRunType(payrollRun.getTypeOf());

        return compositeResponse;
	}

	public static com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo mapPayrollRunInfo(
			PayrollRunInfo nextScheduledPayrollRunInfo) {
		com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo compositeResponse = new com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo();

        compositeResponse.setPayrollRunId(null);
        compositeResponse.setPayrollRunType(nextScheduledPayrollRunInfo.getPayrollRunType());
        compositeResponse.setPayrollRunPeriod(createPayrollRunPeriodFromPayrollRunInfo(nextScheduledPayrollRunInfo));
        compositeResponse.setPayrollRunState(PayrollRunState.NOT_SET);

        return compositeResponse;
	}
}
