package com.intuit.sb.payroll.services.workerpayments.impl;

import java.util.Date;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;

/**
 * Utility class for performing PayrollRunSaveRequest validation
 *
 * Validates the PayrollRunSaveRequest passed in by the client
 * If any of the validations fails throws ValidationException
 */

class PayrollRunPeriodValidator
{

	/**
	 * Private constructor to prevent instantiation.
	 */
	private PayrollRunPeriodValidator() {}

    /**
     * <p>Checks if {@link PayrollRunPeriod} is valid.</p>
     *
     * @param period The value validation is being performed on.  A <code>null</code>
     * value is considered invalid.
     * @throws  ValidationException if the validation fails.
     */
    static void validate(PayrollRunPeriod period) throws ValidationException
    {
    	if(period == null)
    	{
    		throw new ValidationException("Invalid PayrollRunPeriod: PayrollRunPeriod can not be null ..");
    	}

    	Date startDate = period.getPeriodStartDate();
    	Date endDate = period.getPeriodEndDate();

    	Date runDate = period.getPayrollRunDate();
    	Date payDate = period.getPayDate();

    	if(startDate == null)
    	{
    		throw new ValidationException("periodStartDate is null");
    	}
    	if(endDate == null)
    	{
    		throw new ValidationException("periodEndDate is null");
    	}
    	if(runDate == null)
    	{
    		throw new ValidationException("runDate is null");
    	}
    	if(payDate == null)
    	{
    		throw new ValidationException("paydate is null");
    	}
    }
}
