package com.intuit.sb.payroll.services.workerpayments.impl;

import java.util.Set;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PayrollRun;

/**
 * Reports!
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public class PayrollRunReports {
	
	private static final char comma = ',';
	private static final String newLine = System.getProperty("line.separator");
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(PayrollRunReports.class);
	
	/**
	 * The Layout of this CSV is to create a list of each line item for each paycheck.
	 * 
	 * @param payrollRun
	 * @return
	 */
	public synchronized static String createCSV(PayrollRun payrollRun) {
		StringBuilder buf = new StringBuilder();
		createCSVHeader(buf);
		Set<Paycheck> paychecks = payrollRun.getPaychecks();
		for (Paycheck paycheck : paychecks) {
			Set<PaycheckLineItem> paycheckLineItems = paycheck.getPaycheckLineItems();
			for (PaycheckLineItem paycheckLineItem : paycheckLineItems) {
				createCSVLine(buf, payrollRun, paycheck, paycheckLineItem);
			}
		}
		final String report = buf.toString();
		sLogger.info(report);
		return report;
	}

	private static void createCSVHeader(StringBuilder buf) {
		//PayrollRun Data
		appendValue(buf, "PayrollRunId");
		appendValue(buf, "PayrollRun Date");
		appendValue(buf, "PayrollFrequency");
		appendValue(buf, "TotalPayRunGrossPay");
		appendValue(buf, "TotalPayRunNetPay");
		appendValue(buf, "TotalPayRunEmployeePaidTaxes");
		appendValue(buf, "TotalPayRunEmployerPaidTaxes");
		//Paycheck Data
		appendValue(buf, "WorkerId");
		appendValue(buf, "WorkerFirstName");
		appendValue(buf, "WorkerLastName");
		appendValue(buf, "TotalPaycheckGrossPay");
		appendValue(buf, "TotalPaycheckNetPay");
		appendValue(buf, "TotalPaycheckEmployeePaidTaxes");
		appendValue(buf, "TotalPaycheckEmployerPaidTaxes");
		//Line Item Data
		appendValue(buf, "PaycheckLineItemId");
		appendValue(buf, "LineItemName");
		appendValue(buf, "LawId");
		appendValue(buf, "PaycheckLineItemAmount");
		appendValue(buf, "ISTT");
		appendValue(buf, "WageBase");
		appendValue(buf, "YearToDate");
		appendValue(buf, "SortSequence");
		appendValue(buf, "SourceEntity");
		appendLastValue(buf, "TargetEntity");

	}

	private static void createCSVLine(StringBuilder buf, PayrollRun payrollRun, Paycheck paycheck,
			PaycheckLineItem paycheckLineItem) {
		//PayrollRun Data
		appendValue(buf, String.valueOf(payrollRun.getId()));
		appendValue(buf, payrollRun.getPayrollRunDate().toString());
		appendValue(buf, payrollRun.getPayrollFrequency().toString());
		appendValue(buf, payrollRun.getTotalGrossPay().getAmount().toString());
		appendValue(buf, payrollRun.getTotalNetPay().getAmount().toString());
		appendValue(buf, payrollRun.getTotalWorkerPdTaxes().getAmount().toString());
		appendValue(buf, payrollRun.getTotalEmployerPdTaxes().getAmount().toString());
		//Paycheck Data
		appendValue(buf, String.valueOf(paycheck.getWorker().getId()));
		appendValue(buf, paycheck.getWorker().getPersonName().getFirstName());
		appendValue(buf, paycheck.getWorker().getPersonName().getLastName());
		appendValue(buf, paycheck.getGrossAmount().getAmount().toString());
		appendValue(buf, paycheck.getNetAmount().getAmount().toString());
		appendValue(buf, paycheck.getWorkerPdTaxes().getAmount().toString());
		appendValue(buf, paycheck.getEmployerPdTaxes().getAmount().toString());
		//Line Item Data
		appendValue(buf, String.valueOf(paycheckLineItem.getId()));
		appendValue(buf, paycheckLineItem.getName());
		appendValue(buf, paycheckLineItem.getLawId().getValue());
		appendValue(buf, String.valueOf(paycheckLineItem.getValue().getAmount()));
		appendValue(buf, String.valueOf(paycheckLineItem.getIncomeSubjectToTax().getAmount()));
		appendValue(buf, String.valueOf(paycheckLineItem.getWageBase().getAmount()));
		appendValue(buf, String.valueOf(paycheckLineItem.getYtdDatum().getDatum().getMoneyValue().getAmount()));
		appendValue(buf, String.valueOf(paycheckLineItem.getSortSequence()));
		appendValue(buf, String.valueOf(paycheckLineItem.getSourceEntity()));
		appendLastValue(buf, String.valueOf(paycheckLineItem.getTargetEntity()));
		
	}

	private static void appendValue(StringBuilder buf, String string) {
		buf.append(string);
		buf.append(comma);
	}
	
	private static void appendLastValue(StringBuilder buf, String string) {
		buf.append(string);
		buf.append(newLine);
	}

}
