package com.intuit.sb.payroll.services.workerpayments.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PaycheckUpdateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunCreateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunUpdateRequest;

/**
 * Utility class for performing PayrollRunSaveRequest validation Validates the PayrollRunSaveRequest
 * passed in by the client If any of the validations fails throws ValidationException
 *
 * $Author: oblackburn $
 * $Revision: #2 $
 * $DateTime: 2008/05/27 16:28:11 $
 */
class PayrollRunSaveRequestValidator {

	/**
	 * Private constructor to prevent instantiation.
	 */
    private PayrollRunSaveRequestValidator() {}
    
    
    static void validate(PayrollRunCreateRequest createRequest) throws ValidationException, ServiceFailureException
    {
    	if(createRequest == null)
    	{
    		throw new ServiceFailureException("PayrollRunCreateRequest cannot be null");
    	}
    	
        // now validate the PayrollRunPeriod
        PayrollRunPeriod period = createRequest.getPeriod();
        if (period != null && !(createRequest.getPayrollRunType() == PayrollRunTypeEnum.TERMINATION)) 
        {
        	PayrollRunPeriodValidator.validate(period);
        }
    }

    /**
     * <p>
     * Checks if {@link PayrollRunSaveRequest} is valid.
     * </p>
     *
     * @param runRequest The value validation is being performed on. A <code>null</code> value is considered invalid.
     * @throws ValidationException if the validation fails.
     */
    static void validate(PayrollRunSaveRequest runRequest) throws ValidationException, ServiceFailureException {
        if (runRequest == null) {
            throw new ServiceFailureException("PayrollRunSaveRequest can not be null ..");
        }

        // now validate the PayrollRunPeriod
        PayrollRunPeriod period = runRequest.getPayrollRunPeriod();

        PayrollRunPeriodValidator.validate(period);

        // now validate the WorkerWageInfo collection passed in
        PaycheckRequest[] paychecks = runRequest.getPaycheckRequests();

        if (paychecks == null) {
            throw new ServiceFailureException("paychecks are null ..");
        }

        for (PaycheckRequest info : paychecks) {
            PaycheckRequestValidator.validate(info);
        }
        
        if (runRequest.getPayrollRunType() == PayrollRunTypeEnum.TERMINATION) {
        	// Must have a termination date
        	if (null == runRequest.getTerminationDate()) {
        		throw new ValidationException("Must have a Termination Date for a Termination Payroll Run.");
        	}
        }
    }

	public static void validate(PayrollRunUpdateRequest updateRequest) throws ValidationException,
						ServiceFailureException {
        if (updateRequest == null) {
            throw new ServiceFailureException("PayrollRunUpdateRequest can not be null ..");
        }

        // now validate the PaycheckUpdateRequest collection passed in
        PaycheckUpdateRequest[] paycheckUpdateRequests = updateRequest.getPaycheckUpdateRequests();

        if (paycheckUpdateRequests == null) {
            throw new ServiceFailureException("paycheckUpdateRequests are null ..");
        }

        for (PaycheckUpdateRequest info : paycheckUpdateRequests) {
            PaycheckRequestValidator.validate(info);
        }
		
	}
}
