package com.intuit.sb.payroll.services.workerpayments.impl;

import java.util.List;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.services.common.ObjectHandle;

/**
 * Utility class to map EntityInterface PayrollSchedule to DataEntiyy PayrollSchedule and vice versa 
 * 
 * $Author: thobson $
 * $Revision: #5 $ 
 * $DateTime: 2008/01/04 11:43:53 $
 */
public class PayrollScheduleMapper
{

    private PayrollScheduleMapper() {}
    
    /**
     * This utility method converts Data Entity PayrollSchedule to EntityInterface PayrollSchedule
     * 
     * @param payrollSchedule Data Entity PayrollSchedule to map from
     * @return EntityInterface PayrollSchedule
     */
    public static com.intuit.sb.payroll.services.workerpayments.PayrollSchedule mapFromDEntityToEI(
                         PayrollSchedule payrollSchedule)
    {
        if (payrollSchedule == null)
            return null;
        
        com.intuit.sb.payroll.services.workerpayments.PayrollSchedule eIPayrollSchedule =
            new com.intuit.sb.payroll.services.workerpayments.PayrollSchedule();

        eIPayrollSchedule.setFrequency(PayrollFrequencyMapper.mapFromDEntityToEI(payrollSchedule.getFrequency()));
        eIPayrollSchedule.setPeriodEndDate(payrollSchedule.getNextPayPeriodEndDate());

        eIPayrollSchedule.setPayrollPayDateOffset(payrollSchedule.getPayDayOffset());
        eIPayrollSchedule.setPayrollRunDateOffset(payrollSchedule.getPayrollRunDayOffset());

        eIPayrollSchedule.setPeriodStartDate(payrollSchedule.getPeriodStartDate());
        ObjectHandle objectHandle = new ObjectHandle(payrollSchedule.getId(), payrollSchedule.getVersion());
        eIPayrollSchedule.setObjectHandle(objectHandle);
        eIPayrollSchedule.setPayPeriodEndDayofMonth1(payrollSchedule.getPayPeriodEndDayofMonth1());
        eIPayrollSchedule.setPayPeriodEndDayofMonth2(payrollSchedule.getPayPeriodEndDayofMonth2());
        eIPayrollSchedule.setPayrollScheduleMemo(payrollSchedule.getMemo());
        //FIXME hardcoded until the app uses this.
        eIPayrollSchedule.setDefaultt(true);
        return eIPayrollSchedule;
    }

    /**
     * This utility method converts Entity Interface PayrollSchedule to Data Entity PayrollSchedule
     * 
     * @param eiPayrollSchedule Entity Interface PayrollSchedule to map from
     * @return Data Entity PayrollSchedule
     * @throws ServiceFailureException 
     */
    public static PayrollSchedule mapFromEIToDEntity(
                       com.intuit.sb.payroll.services.workerpayments.PayrollSchedule eiPayrollSchedule) throws ServiceFailureException {
        if (eiPayrollSchedule == null)
            return null;
        PayrollScheduleComponent component =  ServiceLocator.getInstance().getService(PayrollScheduleComponent.JNDI_NAME);
        
        List<PayrollSchedule> payScheduleDEList = component.getAllPayrollSchedules();
        PayrollSchedule payrollSchedule = null;
        
        // TakeHome application will have only one PayrollSchedule throw exception if thats not the case
        if (payScheduleDEList.size() == 1){
        	payrollSchedule = payScheduleDEList.get(0);
        }else{
        	payrollSchedule = new PayrollSchedule();
        }
    	
        payrollSchedule.setFrequency(PayrollFrequencyMapper.mapFromEIToDEntity(eiPayrollSchedule.getFrequency()));
        payrollSchedule.setNextPayPeriodEndDate(eiPayrollSchedule.getPeriodEndDate());

        payrollSchedule.setPayDayOffset(eiPayrollSchedule.getPayrollPayDateOffset());
        payrollSchedule.setPayrollRunDayOffset(eiPayrollSchedule.getPayrollRunDateOffset());
        
        payrollSchedule.setPayPeriodEndDayofMonth1(eiPayrollSchedule.getPayPeriodEndDayofMonth1());
        payrollSchedule.setPayPeriodEndDayofMonth2(eiPayrollSchedule.getPayPeriodEndDayofMonth2());
        payrollSchedule.setMemo(eiPayrollSchedule.getPayrollScheduleMemo());
        // FIXME Hardcoded until the app uses this
        payrollSchedule.setDefaultt(true);
        
        return payrollSchedule;
    }
}
