package com.intuit.sb.payroll.services.workerpayments.impl;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.PayrollSchedule;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;

/**
 * Utility Class to validate DTO instances of a PayrollSchedule.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
class PayrollScheduleValidator {
	
	/**
	 * Private constructor to prevent instantiation.
	 */
	private PayrollScheduleValidator() {}
	
	/**
	 * Validates the Entity Interface PayrollSchedule
	 * 
	 * @param payrollSchedule
	 * @throws ValidationException if
	 * <pre>
	 *      PayrollSchedule is null
	 *      PayrollScheduleFrequency is null or NotSet
	 * </pre>
	 * 
	 */
	static void validatePayrollSchedule(PayrollSchedule payrollSchedule) throws ValidationException,
			ServiceFailureException {

		if (payrollSchedule == null) {
			throw new ServiceFailureException("PayrollSchedule is null");
		}

		PayrollScheduleFrequency payFrequency = payrollSchedule.getFrequency();
		if (payFrequency == null) {
			// TODO [I18n] Resource Mgr
			throw new ServiceFailureException("Payroll Frequency in the PayrollSchedule is null");
		}
	}
}
