package com.intuit.sb.payroll.task;

import java.io.Serializable;
import java.util.Date;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;

/**
 * Defines a task.
 */
public class Task implements Serializable
{
	private static final long serialVersionUID = 7468167972601378632L;
	
	private String name = null;
	private String description = null;
	
	private String messageResource;
	private TaskType type;
	private Date dueDate;
	private TaskStatus status;
	
	// TODO: This should only exist for "run payroll" tasks.
	//keeping it here so for the time being so taht the PayrollRun does not break
	private PayrollRunPeriod payrollRunPeriod;

	//Task object handle
    private ObjectHandle objectHandle;
	
    // Priority will be used to list the order these Tasks should be displayed
    private int priority;
    
    // Identification of entity for which this task was created
    private long entityId;
	

	/**
	 * Minimal "bean" constructor
	 */
	public Task() {
		super();
	}

	/**
	 * Maximal constructor.
	 * @param name The name of the task.
	 */
	public Task(String name) {
		this();
		setName(name);
	}
	
	/**
	 * Gets the task name.
	 * @return the name of the task. Can be null.
	 */
	public String getName() {
		return name;
	}
	/**
	 * Sets the task name.
	 * @param name the new name for the task. Can be null.
	 */
	public void setName(String name) {
		this.name = name;
	}

	
	public String getDescription() {
		return description;
	}

	/**
	 * Sets a small description about the task.
	 * @param the short description for the task. Can be null.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the messageResourceId
	 */
	public String getMessageResource() {
		return messageResource;
	}

	/**
	 * @param messageResourceId the messageResourceId to set
	 */
	public void setMessageResource(String messageResource) {
		this.messageResource = messageResource;
	}

	/**
	 * @return the type
	 */
	public TaskType getType() {
		return type;
	}

	/**
	 * @param type the type to set
	 */
	public void setType(TaskType type) {
		this.type = type;
	}

	/**
	 * @return the dueDate
	 */
	public Date getDueDate() {
		return dueDate;
	}

	/**
	 * @param dueDate the dueDate to set
	 */
	public void setDueDate(Date dueDate) {
		this.dueDate = dueDate;
	}
	
	/**
	 * @return the status
	 */
	public TaskStatus getstatus() {
		return status;
	}

	/**
	 * @param status the status to set
	 */
	public void setStatus(TaskStatus status) {
		this.status = status;
	}
	
	// TODO: These should only exist for "run payroll" tasks.
	//keeping it for the time being not to break existing PayrollRun
	public PayrollRunPeriod getPayrollRunPeriod() {
		return payrollRunPeriod;
	}
	
	public void setPayrollRunPeriod(PayrollRunPeriod payrollRunPeriod)
	{
		this.payrollRunPeriod = payrollRunPeriod;
	}
	
	public ObjectHandle getObjectHandle() {
		return objectHandle;
	}

	public void setObjectHandle(ObjectHandle objectHandle) {
		this.objectHandle = objectHandle;
	}
	
	public long getEntityId() {
		return entityId;
	}

	public void setEntityId(long workerId) {
		this.entityId = workerId;
	}
	
	@Override
	public String toString()
	{
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}

	public int getPriority() {
		return priority;
	}

	public void setPriority(int priority) {
		this.priority = priority;
	}
}
