package com.intuit.sb.payroll.services.common;


import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.impl.PhysicalAddressMapper;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Tests the PhysicalAddressMapper.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class PhysicalAddressMapperTest {

	
	@Test
	public void testBasicDtoToEntityMapping() throws Exception {
		PhysicalAddress dtoAddress = PojoUtils.createUSCAPhysicalAddress();
		
		USPostalAddress entityAddress = new USPostalAddressImpl();
		
		PhysicalAddressMapper.mapAddress(dtoAddress, entityAddress);
		
		AssertJUnit.assertEquals("Address Line 1 does not match", PojoUtils.EMPLOYEE_ADDRESSLINE1, entityAddress.getAddressLine1());
		AssertJUnit.assertEquals("Address Line 2 does not match", PojoUtils.EMPLOYEE_ADDRESSLINE2, entityAddress.getAddressLine2());
		AssertJUnit.assertEquals("City does not match", PojoUtils.EMPLOYEE_CITY, entityAddress.getCity());
		AssertJUnit.assertEquals("State does not match", PojoUtils.EMPLOYEE_STATE, entityAddress.getRegionOrState());
		AssertJUnit.assertEquals("Country does not match", PojoUtils.EMPLOYEE_COUNTRY, entityAddress.getCountry());
		AssertJUnit.assertEquals("Country ISO does not match", PojoUtils.EMPLOYEE_COUNTRYISOCODE, entityAddress.getCountryISOCode());
		AssertJUnit.assertEquals("Zip Code does not match", PojoUtils.EMPLOYEE_ZIPCODE, entityAddress.getZipOrPostalCode());
	}

	
	@Test
	public void testBasicEntityToDtoMapping() throws Exception {
		USPostalAddress entityAddress = new USPostalAddressImpl();
		entityAddress.setAddressLine1(PojoUtils.EMPLOYEE_ADDRESSLINE1);
		entityAddress.setAddressLine2(PojoUtils.EMPLOYEE_ADDRESSLINE2);
		entityAddress.setCity(PojoUtils.EMPLOYEE_CITY);
		entityAddress.setCountry(PojoUtils.EMPLOYEE_COUNTRY);
		entityAddress.setCountryISOCode(PojoUtils.EMPLOYEE_COUNTRYISOCODE);
		entityAddress.setZipOrPostalCode(PojoUtils.EMPLOYEE_ZIPCODE);
		entityAddress.setRegionOrState(PojoUtils.EMPLOYEE_STATE);
		
		PhysicalAddress dto = PhysicalAddressMapper.mapAddress(entityAddress);
		
		AssertJUnit.assertEquals("Address Line 1 does not match", PojoUtils.EMPLOYEE_ADDRESSLINE1, dto.getAddressLine1());
		AssertJUnit.assertEquals("Address Line 2 does not match", PojoUtils.EMPLOYEE_ADDRESSLINE2, dto.getAddressLine2());
		AssertJUnit.assertEquals("City does not match", PojoUtils.EMPLOYEE_CITY, dto.getCity());
		AssertJUnit.assertEquals("State does not match", PojoUtils.EMPLOYEE_STATE, dto.getStateOrRegion());
		AssertJUnit.assertEquals("Country does not match", PojoUtils.EMPLOYEE_COUNTRY, dto.getCountry());
		AssertJUnit.assertEquals("Zip Code does not match", PojoUtils.EMPLOYEE_ZIPCODE, dto.getZipOrPostalCode());
		
	}
	
}
