package com.intuit.sb.payroll.services.employer;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.entity.model.PayrollSchedule;

/*
 * This class contains a single test to test bank holidays and tax forms due dates
 * Tests to check for past and future payroll runs are part of the
 * GetNextScheduledPayrollRunInfo* and GetUpcomingScheduledPayrollRunInfo* tests
 */
public class CalendarEntriesTest extends EmployerServiceTestBase{

	
	@Test
    public void getCalendarEntriesTestForYear2009() throws Exception {
		
		// Ensure there is no payroll schedule setup
		List<PayrollSchedule> payrollScheduleList= sPayrollScheduleComponent.getAllPayrollSchedules();
		
		// TakeHome application will have only one PayrollSchedule 
		PayrollSchedule payrollSchedule = null;
		
        if (payrollScheduleList.size() == 1){
        	payrollSchedule = payrollScheduleList.get(0);
        	//Delete the payroll schedule
        	sPayrollScheduleComponent.deletePayrollSchedule(payrollSchedule);     	
        }
        
		CalendarEntry[] calendarEntry = sEmployerService.getCalendarEntries(2009);
    	
    	AssertJUnit.assertNotNull("response returned", calendarEntry);
    	
    	
		// Entries 0-9 are bank holidays and 10-15 are tax form due dates
    	int expectedCalendarEntries = 16;
    	AssertJUnit.assertEquals(expectedCalendarEntries, calendarEntry.length);
    	
    	// 0-9 are holidays for 2009
    	// Test all the returned descriptions and some random dates (for e.g. for holidays that do not have a 
    	// specific date, but a specific day instead)
    	
    	// 1/1/2009 - New Year's Day
    	AssertJUnit.assertEquals("New Year's Day", calendarEntry[0].getDescription());
    	
    	// 1/19/2009 - Martin Luther King Day
    	AssertJUnit.assertEquals("Martin Luther King Day", calendarEntry[1].getDescription());
    	AssertJUnit.assertEquals(19, calendarEntry[1].getDay());
    	
    	// 2/16/2009 - President's Day
    	AssertJUnit.assertEquals("President's Day", calendarEntry[2].getDescription());
    	AssertJUnit.assertEquals(16, calendarEntry[2].getDay());
    	
    	// 5/25/2009 - Memorial Day
    	AssertJUnit.assertEquals("Memorial Day", calendarEntry[3].getDescription());
    	AssertJUnit.assertEquals(25, calendarEntry[3].getDay());
    	
    	// 7/4/2009 - Independence Day
    	AssertJUnit.assertEquals("Independence Day", calendarEntry[4].getDescription());
    	
    	// 9/7/2009 - Labor Day
    	AssertJUnit.assertEquals("Labor Day", calendarEntry[5].getDescription());
    	AssertJUnit.assertEquals(7, calendarEntry[5].getDay());
    	
    	// 10/12/2009 - Columbus Day
    	AssertJUnit.assertEquals("Columbus Day", calendarEntry[6].getDescription());
    	AssertJUnit.assertEquals(12, calendarEntry[6].getDay());
    	
    	// 11/11/2009 - Veterans Day
    	AssertJUnit.assertEquals("Veterans Day", calendarEntry[7].getDescription());
    	AssertJUnit.assertEquals(11, calendarEntry[7].getDay());
    	
    	// 11/26/2009 - Thanksgiving Day
    	AssertJUnit.assertEquals("Thanksgiving Day", calendarEntry[8].getDescription());
    	AssertJUnit.assertEquals(26, calendarEntry[8].getDay());
    	
    	// 12/25/2009 - Christmas Day
    	AssertJUnit.assertEquals("Christmas Day", calendarEntry[9].getDescription());
    	
    	// 2/2/2009 - Annual and quarterly forms due. W-2 due to employee
    	AssertJUnit.assertEquals("Annual and quarterly forms due. W-2 due to employee", calendarEntry[10].getDescription());
    	AssertJUnit.assertEquals(2, calendarEntry[10].getDay());
    	
    	// 3/2/2009 - W-2 due for paper filers
    	AssertJUnit.assertEquals("W-2 due for paper filers", calendarEntry[11].getDescription());
    	AssertJUnit.assertEquals(2, calendarEntry[11].getDay());
    	
    	// 3/31/2009 - W-2 due for electronic filers
    	AssertJUnit.assertEquals("W-2 due for electronic filers", calendarEntry[12].getDescription());
    	AssertJUnit.assertEquals(31, calendarEntry[12].getDay());
    	
    	// 4/30/2009 - Quarterly forms due
    	AssertJUnit.assertEquals("Quarterly forms due", calendarEntry[13].getDescription());
    	AssertJUnit.assertEquals(30, calendarEntry[13].getDay());
    	
    	// 7/31/2009 - Quarterly forms due
    	AssertJUnit.assertEquals("Quarterly forms due", calendarEntry[14].getDescription());
    	AssertJUnit.assertEquals(31, calendarEntry[14].getDay());
    	
    	// 11/2/2009 - Quarterly forms due
    	AssertJUnit.assertEquals("Quarterly forms due", calendarEntry[15].getDescription());
    	AssertJUnit.assertEquals(2, calendarEntry[15].getDay());
    	
	}
	
}
