package com.intuit.sb.payroll.services.employer;


import java.util.ArrayList;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;

/**
 * Contains tests for testing EmployerService APIs
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class EmployerServiceTest extends EmployerServiceTestBase
{   
    
    private EmployerContact[] createEmployerContacts()
    {
    	ArrayList<EmployerContact> contactsList = new ArrayList<EmployerContact>();

		contactsList.add(createEmployerContact("Reuben", "John", "209-123-2222", 
													EmployerContactType.PrimaryContact));
		contactsList.add(createEmployerContact("Amit", "Jain", "650-555-1212", 
													EmployerContactType.SecondaryContact));
		
		EmployerContact[] contacts = new EmployerContact[contactsList.size()];
		contacts = contactsList.toArray(contacts);

		return contacts;	
    }
    
    /**
     * Deletes a Employer contact from the company Employer contacts and checks it is deleted
     */
    @Test(enabled=false, groups={"broken"})
    public void deleteEmployerContactTest() throws Exception
    {
    	try
    	{
	    	EmployerContact[] contactsFromDb; 
	    	contactsFromDb = sEmployerService.getEmployerContacts();
	    	if(0 == contactsFromDb.length)
	    	{
		    	EmployerContact[] contacts = createEmployerContacts();
		    	contactsFromDb = sEmployerService.addEmployerContacts(contacts);
	    	}
	    	//now delete a contact from this
	    	
	    	sEmployerService.deleteEmployerContacts(contactsFromDb);
	    	
	    	//now try to get back the primaryContact that was just deleted
	    	contactsFromDb = sEmployerService.getEmployerContacts();
	    	AssertJUnit.assertEquals("Expected no contacts", 0, contactsFromDb.length);
	    	
    	}
    	catch(ValidationException ve)
    	{
    		AssertJUnit.fail("ValidationException not expected: " + ve.getMessage());
    	}
    }
    
    /**
     * Deletes all Employer contact from the company Employer contacts and checks it is deleted
     */
    @Test
    public void deleteAllEmployerContactTest() throws Exception
    {
    	try
    	{
	    	EmployerContact[] contactsFromDb; 
	    	contactsFromDb = sEmployerService.getEmployerContacts();
	    	if(0 == contactsFromDb.length)
	    	{
		    	EmployerContact[] contacts = createEmployerContacts();
		    	contactsFromDb = sEmployerService.addEmployerContacts(contacts);
	    	}

	    	sEmployerService.deleteAllEmployerContacts();
	    	
	    	//now try to get back the primaryContact that was just deleted
	    	contactsFromDb = sEmployerService.getEmployerContacts();
	    	AssertJUnit.assertEquals("Expected no contacts", 0, contactsFromDb.length);
	    	
    	}
    	catch(ValidationException ve)
    	{
    		AssertJUnit.fail("ValidationException not expected: " + ve.getMessage());
    	}
    }

    @Test(enabled=false, groups={"broken"})
    public void addEmployerContactTest() throws Exception
    {
    	try
    	{
	    	//first get all the EmployerContacts for the company
	    	EmployerContact[] contactsFromDb; 
	    	contactsFromDb = sEmployerService.getEmployerContacts();
	    	if(0 != contactsFromDb.length)
	    	{
	    		sEmployerService.deleteAllEmployerContacts();
	    	}
	    	
	    	contactsFromDb = sEmployerService.getEmployerContacts();
	    	AssertJUnit.assertEquals("Could not remove primary contacts.", 0, contactsFromDb.length);
	    	
	    	EmployerContact[] contacts = new EmployerContact[2];
	    	contacts[0] = createEmployerContact("Ram", "Bho", "231-444-1234", 
														EmployerContactType.PrimaryContact);
	    	contacts[1] = createEmployerContact("Bho", "Ram", "231-333-1235", 
														EmployerContactType.PrimaryContact);

	    	contactsFromDb = sEmployerService.addEmployerContacts(contacts);

	    	/*
	    	 *  smart checking needed as elements can be returned in random order
	    	 *  however object ids cannot be used for the purpose as in memory objects
	    	 *  have null ids.
	    	 *  Let's use the phone nr as the unique id.
	    	 */
	    	for (Integer i = 0; i < contacts.length; i++) {
	    		Boolean found = false;
	    		for (Integer k = 0; k < contacts.length; k++) {
			    	if (contacts[i].getPhone().equals(contactsFromDb[k].getPhone())) {
			    		found = true;
			    		AssertJUnit.assertEquals("First name dif", contacts[i].getFirstName(), contactsFromDb[k].getFirstName());	    		
				    	AssertJUnit.assertEquals("Last name dif", contacts[i].getLastName(), contactsFromDb[k].getLastName());	    		
				    	AssertJUnit.assertEquals("TelephoneNumber ext dif", contacts[i].getExtn(), contactsFromDb[k].getExtn());	    		
				    	AssertJUnit.assertEquals("Fax nr dif", contacts[i].getFax(), contactsFromDb[k].getFax());	    		
				    	AssertJUnit.assertEquals("Role title dif", contacts[i].getTitle(), contactsFromDb[k].getTitle());	    		
				    	AssertJUnit.assertEquals("Email dif", contacts[i].getEmail(), contactsFromDb[k].getEmail());
				    	AssertJUnit.assertEquals("PrimaryContact dif", contacts[i].getEmployerContactType(), contactsFromDb[k].getEmployerContactType());
			    	}
		    	}
		    	AssertJUnit.assertEquals("Matching contact object not found.", true, found.booleanValue());
	    	}
    	}
    	catch(ValidationException ve){
    		AssertJUnit.fail("ValidationException not expected: " + ve.getMessage());
    	}
    }
    
    @Test(enabled=false, groups={"broken"})
    public void loadApplicationTest() throws Exception
    {
    	LoadApplicationResponse applicationResponse = sEmployerService.loadApplication();
    	
    	AssertJUnit.assertNotNull("response returned", applicationResponse);
    	AssertJUnit.assertFalse("response not a newCompany", applicationResponse.isNewCompany());
    	AssertJUnit.assertNull("response no buildtime (unsupported)", applicationResponse.getBuildTime());
    	AssertJUnit.assertNull("response no version (unsupported)", applicationResponse.getVersion());

    	AssertJUnit.assertNotNull("response payschdule", applicationResponse.getPayrollSchedule());
    	AssertJUnit.assertEquals("response payschdule.frequency", PayrollScheduleFrequency.BIWEEKLY.ordinal(), applicationResponse.getPayrollSchedule().getFrequency().ordinal());
    	AssertJUnit.assertEquals("response payschdule.payrollPayDateOffset", 3, applicationResponse.getPayrollSchedule().getPayrollPayDateOffset());
   	
    	AssertJUnit.assertNotNull("response employerInfo", applicationResponse.getEmployerInfo());
    	AssertJUnit.assertEquals("response employerInfo.assistedType", AssistedStateType.NOT_ASSISTED, applicationResponse.getEmployerInfo().getAssistedStateType());
    	AssertJUnit.assertEquals("response employerInfo.companyName", "Bogus", applicationResponse.getEmployerInfo().getCompanyName());
    	AssertJUnit.assertEquals("response employerInfo.DBAName", "Bogus Trading", applicationResponse.getEmployerInfo().getDBAName());
    	AssertJUnit.assertEquals("response employerInfo.legalName", "Bogus Inc", applicationResponse.getEmployerInfo().getLegalName());
    	
    	AssertJUnit.assertNotNull("response tasks", applicationResponse.getCurrentTasks());
    	
    	AssertJUnit.assertNotNull("response workers", applicationResponse.getWorkers());
    	AssertJUnit.assertEquals("response number of workers", 3, applicationResponse.getWorkers().length);

    	AssertJUnit.assertNotNull("response calendarEntryYears", applicationResponse.getCalendarEntryYears());
    	AssertJUnit.assertEquals("response number of calendarEntryYears", 3, applicationResponse.getCalendarEntryYears().length);
    }
    
}
