package com.intuit.sb.payroll.services.employer;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.CompanyActivationStatusEnum;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.DatetimeDTO;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.PTSNotificationDTO;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.CompanyMapper;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.services.common.BankAccount;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.taxes.EmployerTaxSettingsService;
import com.intuit.sb.payroll.services.worker.WorkerService;
import com.intuit.sb.payroll.services.worker.WorkerType;


/**
 * <P>The <CODE>EmployerServiceTestBase</CODE> class is the base class for the IEmployerService Interface tests.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class EmployerServiceTestBase extends BasicTestBase {
	
	
    protected static EmployerService sEmployerService;
    protected static WorkerService sWorkerService;
    protected static EmployerTaxSettingsService sEmployerTaxSettingsService;
    
    
    @BeforeClass
    public static void initializeEmployerServiceTestBaseClass()
    {
    	sEmployerService = qaBootstrap.getEjbFromClass(EmployerService.class);
    	sWorkerService = qaBootstrap.getEjbFromClass(WorkerService.class);
    	sEmployerTaxSettingsService = qaBootstrap.getEjbFromClass(EmployerTaxSettingsService.class);
    }
    
    /**
	 * Utility method to add basic Employer Info to an existing Employer Info
	 */
    public static EmployerInfo createBasicEmployerInfo() {
	
    	EmployerInfo employerInfo = new EmployerInfo();
    	employerInfo.setDBAName("Bogus Inc");
    	employerInfo.setCompanyPhone("111-222-3333");
    	employerInfo.setWebAddress("www.mycompany.com");
    	employerInfo.setCompanyAddress(createPhysicalAddress());
    	employerInfo.setLegalAddress(createPhysicalAddress());
    	employerInfo.setIndustryType(IndustryType.NonProfit);
    	employerInfo.setCompanyType(CompanyType.LLC);
    	employerInfo.setLegalPhone("222-111-4444");
    	employerInfo.setLegalName("Legally Bogus");
    	employerInfo.setLegalAddressSameAsBusinessAddress(false);
    	employerInfo.setExtn("1234");
    	employerInfo.setAssistedStateType(AssistedStateType.NOT_ASSISTED);
    	PrintPreferencesDTO printPreferences = new PrintPreferencesDTO();
    	printPreferences.setXalignment(1);
    	printPreferences.setYalignment(2);
    	employerInfo.setPrintPreferences(printPreferences);
    	
 		return employerInfo;
	}
	
	/**
     * Utility method to create a primary contact from the first name and last name passed in
     * @param firstName 
     * @param lastName
     * @return
     */
	public static  EmployerContact createEmployerContact(String firstName, String lastName, String phone,
    		EmployerContactType empContactType) {
    	EmployerContact employerContact = new EmployerContact();
		employerContact.setFirstName(firstName);
		employerContact.setLastName(lastName);
		employerContact.setEmail(firstName+ "." + lastName +"@MyCompany.com");
		employerContact.setPhone(phone);
		employerContact.setExtn("4444");
		employerContact.setOwner(true);
		employerContact.setEmployerContactType(empContactType);
		employerContact.setTitle("Payroll Manager");
		employerContact.setFax("111-222-5555");
		employerContact.setSameAsCompanyAddress(false);
		
		PhysicalAddress address = createPhysicalAddress();
		
		employerContact.setAddress(address);
		
		return employerContact;
    }
    
	/**
	 * Utility method to create a PhysicalAddress // TODO Move this to Util class
	 */
	public static PhysicalAddress createPhysicalAddress() {
		PhysicalAddress physicalAddress = new PhysicalAddress();
		physicalAddress.setAddressLine1("123 Main St.");
		physicalAddress.setCity("Mountain View");
		physicalAddress.setStateOrRegion("CA");
		physicalAddress.setZipOrPostalCode("94043");
		physicalAddress.setCountry("USA");
		return physicalAddress;
	}
	
	/**
	 * Utility method to create a Bank Account
	 */
	public static BankAccount createBankAccount(BankAccountTypeEnum accountType) {
		BankAccount bankAccount = new BankAccount();		
		bankAccount.setBankName("My Bank");
		bankAccount.setAccountType(accountType);
		bankAccount.setAccountNo(EmployerComponentTestUtils.generateBankAccountNumber());
		bankAccount.setRoutingNo("110000000");
		return bankAccount;
	}
	
	public  static PTSNotificationDTO createPTSNotiticationDTO(NotificationTypeEnum activationType,CompanyActivationStatusEnum activationStatus) throws ServiceFailureException{
		
		PTSNotificationDTO ptsdto= new PTSNotificationDTO();
		PTSEmployer ptsEmployer = getPTSEmployer();
		
		ptsdto.setSourceCompanyId(CompanyMapper.namespaceEmployerId(ptsEmployer.getEmployer()));
		ptsdto.setNotificationType(activationType);
		ptsdto.setNewNotificationStatus(activationStatus);
		ptsdto.setNotificationChangeTimeStamp(getDateDTO());
		ptsdto.setOldNotificationStatus(null); // for now its null
		
		return ptsdto;
	}
	
	private static DatetimeDTO getDateDTO(){
		
		Calendar now = Calendar.getInstance();
		DatetimeDTO dateDTO = new DatetimeDTO();
		dateDTO.setDay(now.get(Calendar.DAY_OF_MONTH));
		dateDTO.setMonth(now.get(Calendar.MONTH));
		dateDTO.setYear(now.get(Calendar.MONTH));
		dateDTO.setHour(now.get(Calendar.HOUR));
		dateDTO.setMinute(now.get(Calendar.MINUTE));
		dateDTO.setSecond(now.get(Calendar.SECOND));
		return dateDTO;
		
	}
	
	private static PTSEmployer getPTSEmployer(){
		PTSEmployer ptsEmployer = null ;
		
		List<PTSEmployer> ptsEmployerList = entityService.findAll(PTSEmployer.class);
		if (ptsEmployerList.size() > 0){
			ptsEmployer= ptsEmployerList.get(0);
		}else{
			ptsEmployer= null;
		}
		return ptsEmployer;
	}
	
	public static BankAccount[] addBanksToCurrentEmployer(BankAccount[] banks) throws ServiceFailureException, ValidationException{
		
		BankAccount[] savedBankAccounts = sEmployerService.addBankAccounts(banks);
		return savedBankAccounts;
		
	}
	
	/**
	 * Utility method to create a CompanyLegalTaxFilingInfo// TODO Move this to Util class
	 */
	public static CompanyLegalTaxFilingInfo createCompanyLegalTaxFilingInfo() {
		CompanyLegalTaxFilingInfo companyLegalTaxFileInfo = new CompanyLegalTaxFilingInfo();
		companyLegalTaxFileInfo.setCompanyLegalType(LegalStructureEnum.CCORP);
		companyLegalTaxFileInfo.setIndustryType(IndustryTypeEnum.CHURCH);
		companyLegalTaxFileInfo.setLegalName("Ryan's Widgets");
		companyLegalTaxFileInfo.setLegalPhone("111-222-3333");
		companyLegalTaxFileInfo.setExtn("123");
		companyLegalTaxFileInfo.setLegalAddressSameAsBusinessAddress(false);
		companyLegalTaxFileInfo.setLegalAddress(createPhysicalAddress());
		
		return companyLegalTaxFileInfo;
	}
	
	/**
     * Utility method to create a work Location from the name and worker count passed in
     * @param name
     * @param workerCount
     * @return
     */
	public static WorkLocation createWorkLocation(String name, String phoneNumber) {
    	WorkLocation workLocation = new WorkLocation();
    	
    	workLocation.setName(name);
    	workLocation.setPhoneNumber(phoneNumber);
    	workLocation.setExtn("5440");
    	//set the Physical Address
    	workLocation.setAddress(createPhysicalAddress());
    	
    	return workLocation;
    }
	
	/**
	 * Utility method to remove all workers associated with the given workLocation id
	 * 
	 * @param objectHandle  The worklocation id to remove all workers from
	 */
	public static void removeAllWorkersFromWorkLocation(ObjectHandle objectHandle) {
	    List<Worker>  workerList = entityService.findAll(Worker.class);
	    for (Worker worker : workerList) {
	        if ((null != worker.getWorkLocation()) && (0L != worker.getWorkLocation().getId())) {
    	        if (objectHandle.getId() == worker.getWorkLocation().getId()) {
    	            worker.setWorkLocation(null);
    	        }
	        }
	    }
	}
	
	/**
	 * Utility method to retrieve all workers' workLocation info
	 * 
	 * @return A WorkerLocation array containing a map of the workers' workLocation 
	 */
	public static WorkerLocation[] getWorkerWorkLocationAssociations() {
		List<Worker>  workerList = entityService.findAll(Worker.class);
	    WorkerLocation[] workerWorkLocations = new WorkerLocation[workerList.size()];
		try {
		    int index = 0;
		    for (Worker worker : workerList) {
		        if (null != worker.getWorkLocation()) { 
		        	WorkerLocation workerLocation = new WorkerLocation();
		            workerLocation.setWorkerId(worker.getId());
		            workerLocation.setWorkLocationId(worker.getWorkLocation().getId());
		            workerWorkLocations[index++] = workerLocation;
		        }
		    }
		} catch(Exception e){
	        e.printStackTrace();
	        AssertJUnit.fail("Error in getWorkerWorkLocationAssociations()!");
	    }
		return workerWorkLocations;
	}

	/**
     * <P>Utility method to create a worker that's not ready for Payroll.</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Worker</LI>
     * <LI>Assign ContactDetail</LI>
     * </OL>
     * * 
     * @return String Id of the worker
     */    
	public static Long createWorkerWithWorkLocation(WorkLocation workLocation) throws Exception {
        Long workerId = null;
        com.intuit.sb.payroll.services.worker.Worker aWorker = createEIWorker();
        aWorker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
        aWorker.setWorkLocation(workLocation);
        
        try {
            com.intuit.sb.payroll.services.worker.Worker payrollReadyWorker = sWorkerService.saveWorker(aWorker);
            workerId = payrollReadyWorker.getObjectHandle().getId();
        } catch (ValidationException e) {
            e.printStackTrace();
            AssertJUnit.fail("Error in createWorkerWithWorkLocation()!");
        } catch (WorkflowException e) {
            e.printStackTrace();
            AssertJUnit.fail("Error in createWorkerWithWorkLocation()!");
        } catch (Exception e) {
            e.printStackTrace();
            AssertJUnit.fail("Error in createWorkerWithWorkLocation()!");
        }
        return workerId;
    }
    
    private static com.intuit.sb.payroll.services.worker.Worker createEIWorker() {
        com.intuit.sb.payroll.services.worker.Worker worker = new com.intuit.sb.payroll.services.worker.Worker();
        worker.setFirstName("Tom");
        worker.setLastName("Smith");
        worker.setJobTitle("Temp worker");
        worker.setHireDate(new Date());
        worker.setBirthDate(new Date());
        worker.setWorkerType(WorkerType.EMPLOYEE);
        
        return worker;
    }
      
    public static boolean verifyWorkLocation(WorkLocation expectedLocation, WorkLocation computedLocation) {
        AssertJUnit.assertEquals("address mismatch", expectedLocation.getAddress(), computedLocation.getAddress());
        AssertJUnit.assertEquals("name mismatch", expectedLocation.getName(), computedLocation.getName());
        AssertJUnit.assertEquals("phoneNumber mismatch", expectedLocation.getPhoneNumber(), computedLocation.getPhoneNumber());
        AssertJUnit.assertEquals("extn mismatch", expectedLocation.getExtn(), computedLocation.getExtn());
        AssertJUnit.assertEquals("workerCount mismatch", expectedLocation.getWorkerCount(), computedLocation.getWorkerCount());
        return true;
    }
    
    public static void verifyEmployerInfo(EmployerInfo expected, EmployerInfo actual) {
    	AssertJUnit.assertEquals("company name mismatch", expected.getCompanyName(), actual.getCompanyName());
    	AssertJUnit.assertEquals("DBA name mismatch", expected.getDBAName(), actual.getDBAName());
    	AssertJUnit.assertEquals("web mismatch", expected.getWebAddress(), actual.getWebAddress());
    	AssertJUnit.assertEquals("company type mismatch", expected.getCompanyType(), actual.getCompanyType());
    	AssertJUnit.assertEquals("industry type mismatch", expected.getIndustryType(), actual.getIndustryType());
    	AssertJUnit.assertEquals("legal name mismatch", expected.getLegalName(), actual.getLegalName());
    	AssertJUnit.assertEquals("legal address mismatch", expected.getLegalAddress(), actual.getLegalAddress());
    	AssertJUnit.assertEquals("legal phone mismatch", expected.getLegalPhone(), actual.getLegalPhone());
    	AssertJUnit.assertEquals("legal address same as business mismatch", expected.isLegalAddressSameAsBusinessAddress(), actual.isLegalAddressSameAsBusinessAddress());
    	AssertJUnit.assertEquals("assisted type mismatch", expected.getAssistedStateType(), actual.getAssistedStateType());
    }
    
    public static boolean verifyEmployerContact(EmployerContact expectedContact , EmployerContact actualContacts,boolean isAdd){
    	if(!isAdd){
            AssertJUnit.assertTrue("EmployerContact Id mismatch", expectedContact.getObjectHandle().getId() == actualContacts.getObjectHandle().getId());
    	}
    	//AssertJUnit.assertTrue("Id mismatch",expectedContact.getUniqueId().longValue()== actualContacts.getUniqueId().longValue());
    	AssertJUnit.assertEquals("Address mismatch",expectedContact.getAddress(), actualContacts.getAddress());
    	AssertJUnit.assertEquals("Extn mismatch",expectedContact.getExtn(), actualContacts.getExtn());
    	AssertJUnit.assertEquals("Title mismatch",expectedContact.getTitle(), actualContacts.getTitle());
    	AssertJUnit.assertEquals("Email mismatch",expectedContact.getEmail(), actualContacts.getEmail());
    	AssertJUnit.assertEquals("Employer Contact Type mismatch",expectedContact.getEmployerContactType(), actualContacts.getEmployerContactType());
    	AssertJUnit.assertEquals("Employer First Name mismatch",expectedContact.getFirstName(), actualContacts.getFirstName());
    	AssertJUnit.assertEquals("Employer Last Name mismatch",expectedContact.getLastName(), actualContacts.getLastName());
    	AssertJUnit.assertEquals("Employer TelephoneNumber number mismatch",expectedContact.getPhone(), actualContacts.getPhone());
    	AssertJUnit.assertEquals("Employer Ownership mismatch",expectedContact.isOwner(), actualContacts.isOwner());
    	AssertJUnit.assertEquals("Employer Same BusinessAddress mismatch",expectedContact.isSameAsCompanyAddress(), actualContacts.isSameAsCompanyAddress());
    	return true;
    }

    public static boolean verifyWorkLocations(WorkLocation[] expectedLocation, WorkLocation[] computedLocation) {
        boolean compareResult = (expectedLocation.length == computedLocation.length);
        for (WorkLocation workLocation1 : expectedLocation) {
            boolean found = false;
            for (WorkLocation workLocation2 : computedLocation) {
                if (workLocation1.getName().equals(workLocation2.getName())) {
                    found = true;
                    // Accumulate all the results
                    compareResult |= verifyWorkLocation(workLocation1, workLocation2);
                    break;
                }
            }
            AssertJUnit.assertTrue("Cannot find expected workLocation in computedLocation", found);
        }
        return compareResult;
    }
    
    public static boolean verifyEmployerContacts(EmployerContact[] contacts,EmployerContact[] actualContacts,boolean isAdd) throws ServiceFailureException, ValidationException{
    	
    	
    	boolean compareResult = true;
    	for (EmployerContact contact1 : contacts) {
            boolean found = false;
            for (EmployerContact contact2 : actualContacts) {
                if (contact1.getFirstName().equals(contact2.getFirstName()) && contact1.getLastName().equals(contact2.getLastName()) && contact1.getPhone().equals(contact2.getPhone())) {
                    found = true;
                    // Accumulate all the results
                    compareResult |= verifyEmployerContact(contact1, contact2,isAdd);
                    break;
                }
            }
            AssertJUnit.assertTrue("Cannot find expected EmployerContacts in computed EmployerContacts", found);
        }
    	
    	return compareResult;
    	
    }
    
    public static boolean verifyEmployerContactsDelete(EmployerContact[] employercontactsdeleted) throws ServiceFailureException, ValidationException{
    	
    	EmployerContact[] employercontact = sEmployerService.getEmployerContacts();
    	
    	for(int i=0 ;i< employercontact.length;i++){
    		for(int j = 0 ; j< employercontactsdeleted.length ; j++){
                if(employercontactsdeleted[j].getObjectHandle().getId() == employercontact[j].getObjectHandle().getId()){
    				AssertJUnit.fail("EmployerContact deletion failed");
    			}
    		}
    	}
    	return true;
    }
    
    
    public static BankAccount getBankAccountBasedOnAccountNo(BankAccount[] bankAccounts, String bankAccountNo){
		BankAccount bankAccountRetrieved = null;
		for(BankAccount ba:bankAccounts){
			if(ba.getAccountNo().equals(bankAccountNo))
			{
				bankAccountRetrieved = ba;
				break;
			}
		}
		return bankAccountRetrieved;
	}
	
    public static String getBankIdFromBankAccount(BankAccount bankAccount){
		StringBuilder str = new StringBuilder();
	    str.append(bankAccount.getObjectHandle().getId());		
		String id = str.toString();			
		return id;
	}


}
