package com.intuit.sb.payroll.services.pto.impl;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.HashSet;
import java.util.Set;

import org.testng.AssertJUnit;

import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualStartTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;
import com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtlPtdDatum;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.services.pto.AccrualStartType;
import com.intuit.sb.payroll.services.pto.PTOAccrualType;
import com.intuit.sb.payroll.services.pto.PTOInfo;
import com.intuit.sb.payroll.services.pto.PTOType;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.PtoBaseRateType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;


public class PTOMapperTest extends WorkerServiceTestBase {
	
	private static PaidTimeOffComponent ptoComponent;
	
	@BeforeClass
	public static void initializePTOMapperTest()
	{
		ptoComponent = ServiceLocator.getInstance().getService(
				PaidTimeOffComponent.JNDI_NAME);
	}
	
	private static PTOInfo createPTO(Worker worker, PTOType ptoType) {
		//Create a PTOInfo 
		PTOInfo ptoInfo = new PTOInfo();
		ptoInfo.setWorkerId(worker.getObjectHandle().getId());
		ptoInfo.setName(ptoType.toString());
		ptoInfo.setPtoType(ptoType);
		ptoInfo.setAccrualRate(new Double("2.35"));
		ptoInfo.setAllowCarryOver(true);
		ptoInfo.setPtoAccrualType(PTOAccrualType.PER_YEAR_FROM_HIRE_DATE);
		ptoInfo.setLimitAccrual(true);
		ptoInfo.setMaximumAccrual(new Double("200.00"));
		ptoInfo.setAccruedHours(new Double("5.40"));
		ptoInfo.setDisbursedHoursYTD(new Double("8.00"));
		
		Calendar now = DateUtil.getNowCalendar();
		ptoInfo.setAccrualStartDate(now.getTime());
		ptoInfo.setAccrualStartType(AccrualStartType.IMMEDIATELY);
		
		return ptoInfo;
	}
	
	private static PaidTimeOffDtl createPaidTimeOffDtl(PaidTimeOffTypeEnum paidTimeOffTypeEnum) 
	throws ServiceFailureException {
		PaidTimeOffDtl paidTimeOffDtl = ptoComponent.createPaidTimeOffDtl(paidTimeOffTypeEnum);
		
		Calendar now = DateUtil.getNowCalendar();
		paidTimeOffDtl.setAccrualStartDate(now.getTime());
		paidTimeOffDtl.setAccrualStartType(PaidTimeOffAccrualStartTypeEnum.IMMEDIATELY);
		paidTimeOffDtl.setAccrualType(PaidTimeOffAccrualTypeEnum.PER_YEAR_FROM_HIRE_DATE);
		paidTimeOffDtl.setAccruedHours(new Double("5.40"));
		paidTimeOffDtl.setAllowCarryOver(true);
		paidTimeOffDtl.setDescription(paidTimeOffTypeEnum.toString());
		paidTimeOffDtl.setLimitAccrual(true);
		paidTimeOffDtl.setMaximumAccrual(new Double("200.00"));
		paidTimeOffDtl.setPaidTimeOffType(paidTimeOffTypeEnum);
		// Datum for accrual rate
		Datum rate = new Datum();
		rate.setValue1AsQuantity(new Double("2.35"));
		rate.setType2Value2(DatumType2Enum.YEAR, new Double(1));
		paidTimeOffDtl.setRate(rate);
		// PtdDatum for disbursed hours year-to-date
		PtdDatumType datum = new PtdDatumType();
		datum.setQuantityValue(new Double("8.00"));
		datum.setPtdType(PtdTypeEnum.YEAR);
		PayrollItemDtlPtdDatum ptdDatum = new PayrollItemDtlPtdDatum();
		ptdDatum.setDatum(datum);
		Set<PayrollItemDtlPtdDatum> ptdDatumSet = new HashSet<PayrollItemDtlPtdDatum>();
		ptdDatumSet.add(ptdDatum);
		paidTimeOffDtl.setPtdDatums(ptdDatumSet);
		
		return paidTimeOffDtl;
	}
	
	private static boolean comparePTOInfoToPaidTimeOffDtl(PTOInfo ptoInfo, PaidTimeOffDtl paidTimeOffDtl)
	{
		if (ptoInfo.getName().equals(paidTimeOffDtl.getDescription()) &&
			ptoInfo.getPtoType().equals(PTOMapper.mapToPTOType(paidTimeOffDtl.getPaidTimeOffType())) &&
			ptoInfo.getAccrualRate().equals(paidTimeOffDtl.getRate().getValue1AsQuantity()) &&
			ptoInfo.isAllowCarryOver() == paidTimeOffDtl.isAllowCarryOver() &&
			PTOMapper.mapToAccrualTypeEnum(ptoInfo.getPtoAccrualType()).equals(paidTimeOffDtl.getAccrualType()) && 
			ptoInfo.isLimitAccrual() == paidTimeOffDtl.isLimitAccrual() &&
			ptoInfo.getMaximumAccrual().equals(paidTimeOffDtl.getMaximumAccrual()) &&
			ptoInfo.getAccruedHours().equals(paidTimeOffDtl.getAccruedHours()) &&
//			ptoInfo.getDisbursedHoursYTD().equals(obj) &&
			ptoInfo.getAccrualStartDate().equals(paidTimeOffDtl.getAccrualStartDate()) &&
			PTOMapper.mapToAccrualStartTypeEnum(ptoInfo.getAccrualStartType()) == paidTimeOffDtl.getAccrualStartType()) {
			return true;
		}
			
		return false;
	}
	
	private static Worker newWorker;
	private static Worker savedWorker;
	private static long sWorkerId;
	
	public void setupWorker(PtoBaseRateType ptoBaseRateType) throws Exception {
		// create a new worker
		newWorker = createWorker(); // create a worker with minimum data

		// Test SaveWorker with no compensations
		savedWorker = sWorkerService.saveWorker(newWorker);
		
		sWorkerId = savedWorker.getObjectHandle().getId();
		
		// save salary compensation item
		CompensationItem[] inputCompensations = new CompensationItem[1];
		inputCompensations[0] = new CompensationItem(null, new BigDecimal(10000), 
				0, CompensationType.Salary, "salary",true);
		inputCompensations[0].setPtoBaseRateType(ptoBaseRateType);
        // Test inserting compensations
		sWageService.saveCompensationItems(sWorkerId, 
			mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, inputCompensations));
	}
	
	public void cleanupWorker() throws Exception {
		cleanUpEIWorker(savedWorker.getObjectHandle().getId());
	}
	
	/**
     * <P>Test mapToPaidTimeOffDtl() with  all valid details - Happy Path</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker and save</LI>
     * <LI>Add one salary compensation item and set ptoBaseRateType to SICK</LI> 
     * <LI>Create PTOInfo(in this case, SICK)</LI>
     * <LI>Map the PTOInfo to a PaidTimeOffDtl</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PTOInfo is mapped correctly to the PaidTimeOffDtl</LI>
     * @throws Exception
     * </UL>
     */
	@Test
	public void testMapToPaidTimeOffDtl() throws Exception {
		
		setupWorker(PtoBaseRateType.SICK);
		
		PTOInfo ptoInfo = createPTO(savedWorker, PTOType.SICK_TIME);
		
		PaidTimeOffDtl paidTimeOffDtl = PTOMapper.mapToPaidTimeOffDtl(ptoInfo);
		
		AssertJUnit.assertTrue(comparePTOInfoToPaidTimeOffDtl(ptoInfo, paidTimeOffDtl));
		
		cleanupWorker();
		
	}
	
	/**
     * <P>Test mapToPTOInfo() with  all valid details - Happy Path</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker and save</LI>
     * <LI>Add one salary compensation item and set ptoBaseRateType to SICK</LI> 
     * <LI>Create PaidTimeOffDtl(in this case, SICK)</LI>
     * <LI>Map the PaidTimeOffDtl to a PTOInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PaidTimeOffDtl is mapped correctly to the PTOInfo</LI>
     * @throws Exception
     * </UL>
     */
	@Test
	public void testMapToPTOInfo() throws Exception {
		
		setupWorker(PtoBaseRateType.VACATION);
		
		PaidTimeOffDtl paidTimeOffDtl = createPaidTimeOffDtl(PaidTimeOffTypeEnum.VACATION);
		
		PTOInfo ptoInfo = PTOMapper.mapToPTOInfo(sWorkerId, paidTimeOffDtl);
		
		AssertJUnit.assertTrue(comparePTOInfoToPaidTimeOffDtl(ptoInfo, paidTimeOffDtl));
		
		cleanupWorker();
		
	}

}
