package com.intuit.sb.payroll.services.taxes;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.UnemploymentInsDocDtl;
import com.intuit.sb.payroll.services.common.ObjectHandle;

/**
 *  <P>The <CODE>EmployerTaxSettingsServiceTest</CODE> class contains test methods for Testing the Employer Tax Settings service</P>
 *
 * $Author: rjohn2 $
 * $Revision: #1 $
 * $DateTime: 2008/08/21 14:10:25 $
 *
 */

public class EmployerTaxSettingsServiceTest extends EmployerTaxSettingsServiceTestBase
{
    private EmployerTaxSettingsService mEmployerTaxSettingService;
    private EmployerComponent  mEmployerComponent;

    @BeforeMethod
    public void initializeEmployerTaxSettingsServiceTestClass()
    {
    	mEmployerTaxSettingService = qaBootstrap.getEjbFromClass(EmployerTaxSettingsService.class);
    	mEmployerComponent = qaBootstrap.getEjbFromClass(EmployerComponent.class);
    	
    }
    
    @Test
	public void testgetEmployerTaxIDs() throws Exception
	{
		EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
		AssertJUnit.assertNotNull(taxIdInfos);

		// Should be same as Jurisdictions
		List<Jurisdiction> jurisdictionList = mEmployerComponent.determineAllJurisdictions();
		AssertJUnit.assertEquals("Employer Tax Id should be same as number of Jurisdictions", jurisdictionList.size() -1, taxIdInfos.length);
	}

	@Test
	public void testgetEmployerTaxIDsWithOnlyFederal() throws Exception
	{
		// Should add only if EIN does not exist
		EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
		EmployerTaxIdInfo fedTaxIdInfo = null;
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos)
		{
			if (employerTaxIdInfo.getTaxAgency().name.contains("US_IRS"))
			{
				fedTaxIdInfo = employerTaxIdInfo;
			}
		}
		AssertJUnit.assertNotNull(fedTaxIdInfo);
	}

	@Test
	public void updateOnlyFedralId() throws Exception
	{
		EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos)
		{
			if (employerTaxIdInfo.getTaxAgency().name.contains("US_IRS"))
			{
				employerTaxIdInfo.setTaxId("991111112");
			}
		}
		mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);

		EmployerTaxIdInfo[] taxIdInfos2 = mEmployerTaxSettingService.getEmployerTaxIDs();
		boolean federalIdfound = false;
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos2)
		{
			if (employerTaxIdInfo.getTaxAgency().name.contains("US_IRS"))
			{
				AssertJUnit.assertEquals("991111112", employerTaxIdInfo.getTaxId());
				federalIdfound = true;
			}
		}
		if(!federalIdfound)
		{
			AssertJUnit.fail("Could not find federal Id");
		}
	}

	@Test
	public void updateAllIds() throws Exception
	{
		EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos)
		{
			if (employerTaxIdInfo.getTaxAgency().name.contains("US_IRS"))
			{
				employerTaxIdInfo.setTaxId("99-1111112");
			}
			else if (employerTaxIdInfo.getTaxAgency().name.contains("CA_EDD"))
			{
				employerTaxIdInfo.setTaxId("123-4567-8");
			}
			else if (employerTaxIdInfo.getTaxAgency().name.contains("TX_TWC"))
			{
				employerTaxIdInfo.setTaxId("12-345678-9");
			}
		}

		mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);

		EmployerTaxIdInfo[] taxIdInfos2 = mEmployerTaxSettingService.getEmployerTaxIDs();
		int updateCount = 0;
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos2)
		{
			if (employerTaxIdInfo.getTaxAgency().name.contains("US_IRS"))
			{
				AssertJUnit.assertEquals("991111112", employerTaxIdInfo.getTaxId());
				updateCount++;
			}
			else if (employerTaxIdInfo.getTaxAgency().name.contains("CA_EDD"))
			{
				AssertJUnit.assertEquals("12345678", employerTaxIdInfo.getTaxId());
				updateCount++;
			}
			else if (employerTaxIdInfo.getTaxAgency().name.contains("TX_TWC"))
			{
				AssertJUnit.assertEquals("123456789", employerTaxIdInfo.getTaxId());
				updateCount++;
			}
		}
		AssertJUnit.assertEquals(3, updateCount);
	}


    @Test
    public void getEmployerSUIsettingsTest()
    {
    	EmployerUnemploymentIns[] settings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
    	List<UnemploymentInsDocDtl>  suiTaxes = entityService.findAll(UnemploymentInsDocDtl.class);
    	AssertJUnit.assertTrue("Expected two but got : " + settings.length, settings.length == suiTaxes.size());
    }

    @Test
    public void updateRateTest() throws Exception
    {
    	EmployerUnemploymentIns[] settings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
    	for(EmployerUnemploymentIns setting:settings)
    	{
    		setting.setEffectiveRate("2.8");
    	}
    	//now update in the back end
   		mEmployerTaxSettingService.updateEmployerUnemploymentIns(settings);

    	//now read back the SuiSettings
   		EmployerUnemploymentIns[] updatedSettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
    	for (EmployerUnemploymentIns suiSetting : updatedSettings)
		{
			AssertJUnit.assertEquals("2.8", suiSetting.getEffectiveRate());
		}
    }
    
    @Test(expectedExceptions=ServiceFailureException.class)
    public void updateInvalidSuiTest() throws Exception
    {
    	EmployerUnemploymentIns[] suiSettings = {new EmployerUnemploymentIns()};
    	suiSettings[0].setEffectiveRate("2.7");
    	suiSettings[0].setDefaultRate(true);
    	suiSettings[0].setState("California");
    	suiSettings[0].setObjectHandle(new ObjectHandle(0L,0));
    	mEmployerTaxSettingService.updateEmployerUnemploymentIns(suiSettings);
    }

    @Test
    public void updateFutureRateTest() throws Exception
    {
    	EmployerUnemploymentIns[] settings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
    	for(EmployerUnemploymentIns setting:settings)
    	{
    		Date rateStartDate = setting.getEffectivePeriod().getPeriodStartDate();
    		Calendar cal = LocaleGuru.getInstance().getCalendar();
    		cal.setTime(rateStartDate);
    		cal.add(Calendar.MONTH, 3);
    		setting.setEffectiveRate("3.2");
    		setting.getEffectivePeriod().setPeriodStartDate(cal.getTime());
    	}
    	//now update in the back end
  		mEmployerTaxSettingService.updateEmployerUnemploymentIns(settings);

//    	//now read back the SuiSettings
//  		EmployerUnemploymentIns[] updatedSettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
//    	for (EmployerUnemploymentIns suiSetting : updatedSettings)
//		{
//			AssertJUnit.assertEquals("3.2", suiSetting.getFutureRate());
//			AssertJUnit.assertEquals(tommorrow,suiSetting.getFutureRateStartDate());
//		}
    }

     @Test
    public void getTaxDetailsTest() throws Exception
    {
    	try
    	{
	    	TaxDetails[] taxDetails = mEmployerTaxSettingService.getTaxDetails();
	
	
	    	// Verify the size of the array with the jurisdiction
	
			List<Jurisdiction> jurisdictionList = mEmployerComponent.determineAllJurisdictions();
			AssertJUnit.assertEquals("Employer TaxDetails sections should be same as number of Jurisdictions", jurisdictionList.size() -1, taxDetails.length);

	    	// Verify the TIN etc in the Tax Details
			EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
			int count = 0;
			int taxIdCount = 0;
			for(EmployerTaxIdInfo info: taxIdInfos)
			{
				String taxid = info.getTaxId();
				if(taxid != null)
				{
					taxIdCount++;
				}
				for (TaxDetails taxDetail : taxDetails) 
				{
					String tin = taxDetail.getAgencyTIN();
					if(tin != null && taxid != null)
					{
						if(tin.compareTo(taxid) == 0)
						{
							count++;
							break;
						}
					}
				}
			}
			AssertJUnit.assertEquals("Employer TaxDetails sections should have same number of TaxIds as setup on the employer", taxIdCount, count);
	
	    	// Verify the sui Tax Rates

			// Be happy
    	}
    	catch(ValidationException ve)
    	{
    		AssertJUnit.fail("Did not expect ValidationException");
    	}

    }
}

