package com.intuit.sb.payroll.services.taxes;

import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.entity.employer.WorkLocationEAO;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.services.employer.WorkerLocation;

/**
 *	This class will be the base class for all the tests for EmployerTaxSettingServiceBean
 *  The class extends EmployerServiceTestBase as lot of common methods can be reused
 *
 * $Author: amalik $
 * $Revision: #1 $
 * $DateTime: 2008/08/26 14:20:19 $
 */
public class EmployerTaxSettingsServiceTestBase extends EmployerServiceTestBase {
	protected static EmployerTaxSettingsService mEmployerTaxSettingService;
	protected static EmployerComponent mEmployerComponent;
	protected static WorkLocationEAO sWorkLocationEAO;
	
	@BeforeClass
	public static void initializeEmployerTaxSettingsServiceTestBaseClass()
	{
		mEmployerTaxSettingService = qaBootstrap.getEjbFromClass(EmployerTaxSettingsService.class);
		mEmployerComponent = qaBootstrap.getEjbFromClass(EmployerComponent.class);
		sWorkLocationEAO = qaBootstrap.getEjbFromClass(WorkLocationEAO.class);
	}
	
	public List<WorkLocation> removeSomeWorkLocations(String workLocationName) throws Exception 
	{
		List<WorkLocation> allOrgWorkLocations = mEmployerComponent.findAllWorkLocations();
		List<Worker>  workerList = entityService.findAll(Worker.class);
		List<WorkLocation> allWorkLocations = new ArrayList<WorkLocation>();
		
		for (WorkLocation workLocation : allOrgWorkLocations) {
			if(workLocation.getName().equals(workLocationName))  		// can't remove all worklocation, so we will keep Tx one
				continue;
			allWorkLocations.add(workLocation);
			for (Worker worker : workerList) {
				if ((worker.getWorkLocation() != null) && 
					(worker.getWorkLocation().getName().equals(workLocation.getName()))){
					worker.setWorkLocation(null);
				}
			}
		}
		
		mEmployerComponent.deleteWorkLocations(allWorkLocations);
		return allOrgWorkLocations;
     }
	
	public void addWorkLocation(String name , String phone) throws  ServiceFailureException, ValidationException
	{
			
			com.intuit.sb.payroll.services.common.WorkLocation[] wrkLocation = new com.intuit.sb.payroll.services.common.WorkLocation[]{createWorkLocation(name, phone)};
			sEmployerService.addWorkLocations(wrkLocation);
		
	}
	
	
	/**
	 * Utility method to remove all workers associated with the given workLocation id
	 * 
	 * @param id  The worklocation id to remove all workers from
	 */
	protected void removeAllWorkersFromWorkLocation(Long id){
	    List<Worker>  workerList = entityService.findAll(Worker.class);
	    for (Worker worker : workerList)
	    {
	        if ((null != worker.getWorkLocation()) && (0 != worker.getWorkLocation().getId()))
	        {
	        	Long tempId = new Long(worker.getWorkLocation().getId());
    	        if (id.equals(tempId))
    	        {
    	            worker.setWorkLocation(null);
    	        }
	        }
	    }
	}
	
	public boolean removeWorkLocationWithWorkersXXXX(){
		 boolean isRemoved = false ;
		
		WorkerLocation[] workerWorkLocationAssociations = getWorkerWorkLocationAssociations();
		try{
			for (WorkerLocation mapItem : workerWorkLocationAssociations)
			{
				removeAllWorkersFromWorkLocation(mapItem.getWorkerId());
			}
			isRemoved = true;
		}
		catch(Exception e){
			AssertJUnit.fail(e.getMessage()+"Exception thrown while removing the worklocation , test exited without completion");
		}
		return isRemoved;
	}
}
