package com.intuit.sb.payroll.services.taxes;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;


/**
 *  <P>The <CODE>GetApplicableTaxesWashingtonTest</CODE> class contains test methods for getting applicable taxes in Washington</P>
 *
 *  <DL>
 *  <DT><B>Requirements Document:</B></DT>
 *  <DD><A HREF="http://spitfire.corp.intuit.net/sites/merlinpayroll/Shared%20Documents/Forms/AllItems.aspx?RootFolder=%2fsites%2fmerlinpayroll%2fShared%20Documents%2fRequirements&View=%7b146B0034%2d58AF%2d4E76%2d93B1%2dA00364A96B7E%7d">Gemini requirements</A></DD>
 *  </DL>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetApplicableTaxesWashingtonTest extends TaxesTestBase
{

	@BeforeClass
	public static void setUpGetApplicableTaxesWashington()
	{
        try
        {
            getUserTransaction().begin();
            saveEmployerInfo();
    		setUpWashingtonAddress();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in setUpGetApplicableTaxesWashington(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
	@AfterClass
	public static void tearDownGetApplicableTaxesWashington()
	{
        try
        {
            getUserTransaction().begin();
            restoreEmployerInfo();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in tearDownGetApplicableTaxesWashington(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
    private static void setUpWashingtonAddress()
    {
        sPhysicalAddress.setAddressLine1("4124 Tacoma Mall Blvd");        
        sPhysicalAddress.setCity("Tacoma");
        sPhysicalAddress.setRegionOrState("WA");
        sPhysicalAddress.setZipOrPostalCode("98409");        
    }
    
    /************* Test cases for employee not related to the employer *************/
    
    /**
     * <P>Tests applicable taxes for non profit organization in Tacoma, WA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in Tacoma, WA</LI>
     * <LI>Set up list of expected taxes</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a PayrollException is thrown since WA is not a supported jurisdiction yet
     * </UL>
      */
    @Test
    public void testApplicableTaxesNonProfitInWashingtonAge22()
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up list of taxes, although it is not needed to be correct since an exception is expected to be thrown
        String[] expectedTaxes = {"US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE"};
        
        try {
            performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED, expectedTaxes);
            AssertJUnit.fail("Exception was not thrown!");
        } catch (Exception ex) {
            AssertJUnit.assertTrue((ex.getMessage().contains("Unsupported Jurisdiction:WA") == true));
        }
    }
    
}
