package com.intuit.sb.payroll.services.taxes;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;


/**
 *  <P>The <CODE>GetWithholdingFormsTexasTest</CODE> class contains test methods for getting Withholding forms for Texas</P>
 *
 *  <DL>
 *  <DT><B>Requirements Document:</B></DT>
 *  <DD><A HREF="http://spitfire.corp.intuit.net/sites/merlinpayroll/Shared%20Documents/Forms/AllItems.aspx?RootFolder=%2fsites%2fmerlinpayroll%2fShared%20Documents%2fRequirements&View=%7b146B0034%2d58AF%2d4E76%2d93B1%2dA00364A96B7E%7d">Gemini requirements</A></DD>
 *  </DL>
 *
 * $Author: pszeto $
 * $Revision: #5 $
 * $DateTime: 2008/02/14 14:00:52 $
 *
 */
public class GetWithholdingFormsTexasTest extends TaxesTestBase
{

	@BeforeClass
	public static void setUpGetWithholdingFormsTexas()
	{
        try
        {
            getUserTransaction().begin();
            saveEmployerInfo();
    		setUpTexasAddress();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in setUpGetWithholdingFormsTexas(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
	@AfterClass
	public static void tearDownGetWithholdingFormsTexas()
	{
        try
        {
            getUserTransaction().begin();
            restoreEmployerInfo();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in tearDownGetWithholdingFormsTexas(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
    /************* Test cases for employee not related to the employer *************/
    
    /**
     * <P>Tests applicable withholding forms for non profit organization in Mesquite, TX; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in Mesquite, TX</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testNonProfit501c3InTexasAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                   expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for SoleProprietor in Mesquite, TX; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to SoleProprietor</LI>
     * <LI>Set work location in Mesquite, TX</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testSoleProprietorInTexasForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                   expectedWithholdingForms);
    }
    
}
