package com.intuit.sb.payroll.services.taxpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.taxes.impl.LawUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.HistPaycheck;
import com.intuit.sb.payroll.entity.model.HistPaycheckLineItem;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentLineItem;
import com.intuit.sb.payroll.entity.model.PaymentLineItemSplit;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.services.taxpayments.impl.DateRangeMapper;
import com.intuit.sb.payroll.services.taxpayments.impl.FutaPaymentsAndLiabilitiesMapper;
import com.intuit.sb.payroll.services.taxpayments.impl.PaymentTypeLawMapper;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 * Test FutaPaymentsAndLiabilitiesMapper
 */
public class FutaPaymentsAndLiabilitiesMapperTest extends WorkerPaymentServiceTestBase {

	private static Money sAmountLiability = new Money("35.44");
	private static Money sAmountPayment = new Money("38.44");
	
	@Test
	public void getPaymentEntityFromFutaPaymentsAndLiabilitiesTest() throws Exception {
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = 
			createDummyFutaPaymentsAndLiabilities(PtdTypeEnum.QUARTER2);
		FutaPaymentsAndLiabilities futaPandL = futaPaymentsAndLiabilities.get(0);
		List<Payment> payments = 
			FutaPaymentsAndLiabilitiesMapper.getPaymentEntityFromFutaPaymentsAndLiabilities(futaPaymentsAndLiabilities);
		Assert.assertNotNull(payments);
		Assert.assertEquals(payments.size(), 1);
		Payment payment = payments.get(0);
		Assert.assertNotNull(payment);
		Assert.assertEquals(payment.getPaymentDate(), futaPandL.getQuarterPaymentPeriod().getPeriodStartDate());
		Assert.assertEquals(payment.getPayrollPeriod(), DateRangeMapper.mapPaymentPeriodToDateRange(futaPandL.getQuarterPaymentPeriod()));
		Assert.assertNotNull(payment.getPaymentLineItems());
		Assert.assertEquals(payment.getPaymentLineItems().size(), 1);
		
		PaymentLineItem pli = payment.getPaymentLineItems().iterator().next();
		Assert.assertNotNull(pli);
		Assert.assertNotNull(pli.getPaymentLineItemSplits());
		Assert.assertEquals(pli.getAmount(), sAmountPayment);
		Assert.assertEquals(pli.getPaymentLineItemSplits().size(), 1);
		Assert.assertEquals(pli.getPaymentType(), PaymentType.FORM940FUTA);
		
		PaymentLineItemSplit split = pli.getPaymentLineItemSplits().iterator().next();
		Assert.assertNotNull(split);
		List<PayrollItem> payrollItems = PaymentTypeLawMapper.mapPaymentTypeToPayrollItem(PaymentType.FORM940FUTA);
		PayrollItem pItem = LawUtils.getPayrollItem(payrollItems, QboeLawIdConstants.sFUTALawId);
		Assert.assertEquals(split.getPayrollItem(), pItem);
		Assert.assertEquals(split.getSplitAmount(), sAmountPayment);
		
	}
	
	@Test
	public void getHistPaycheckEntityFromFutaPaymentsAndLiabilitiesTest() throws Exception {
		
	}
	
	@Test
	public void mapPaymentAndHistPaycheckEntityToFutaPaymentsAndLiabilitiesTest() throws Exception {
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = 
			createDummyFutaPaymentsAndLiabilities(PtdTypeEnum.QUARTER2);
		FutaPaymentsAndLiabilities futaPandL = futaPaymentsAndLiabilities.get(0);
		List<HistPaycheck> liabilities = 
			FutaPaymentsAndLiabilitiesMapper.getHistPaycheckEntityFromFutaPaymentsAndLiabilities(futaPaymentsAndLiabilities);
		
		Assert.assertNotNull(liabilities);
		Assert.assertEquals(liabilities.size(), 1);
		Assert.assertNotNull(liabilities.get(0));
		HistPaycheck hpc = liabilities.get(0);
		Assert.assertEquals(hpc.getGrossAmount(), sAmountLiability);
		Assert.assertEquals(hpc.getPayrollPeriod(), DateRangeMapper.mapPaymentPeriodToDateRange(futaPandL.getQuarterPaymentPeriod()));
		Assert.assertEquals(hpc.getEmployer(), sEmployerComponent.getEmployer());
		Assert.assertNotNull(hpc.getHistPaycheckLineItems());
		Assert.assertEquals(hpc.getHistPaycheckLineItems().size(), 1);

		HistPaycheckLineItem hpli = hpc.getHistPaycheckLineItems().iterator().next();
		Assert.assertNotNull(hpli);
		Assert.assertEquals(hpli.getValue(), sAmountLiability);
		Assert.assertEquals(hpli.getLawId().getValue(), QboeLawIdConstants.sFUTALawId);
		List<PayrollItem> payrollItems = PaymentTypeLawMapper.mapPaymentTypeToPayrollItem(PaymentType.FORM940FUTA);
		PayrollItem pItem = LawUtils.getPayrollItem(payrollItems, QboeLawIdConstants.sFUTALawId);
		Assert.assertEquals(hpli.getPayrollItem(), pItem);
	}
	
	private static List<FutaPaymentsAndLiabilities> createDummyFutaPaymentsAndLiabilities(PtdTypeEnum qtr) 
			throws ValidationException {
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();
		FutaPaymentsAndLiabilities fpandl = new FutaPaymentsAndLiabilities();
		fpandl.setFutaPayment(createDummyFutaPaymentLiability(FutaAmountType.FUTAPAYMENT));
		fpandl.setFutaLiability(createDummyFutaPaymentLiability(FutaAmountType.FUTALIABILITY));
	
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		DateRange dr = PtdUtils.getDateRangeForPeriodType(qtr, cal.get(Calendar.YEAR));
		fpandl.setQuarterPaymentPeriod(DateRangeMapper.mapDateRangeToPaymentPeriod(dr));
		futaPaymentsAndLiabilities.add(fpandl);
		return futaPaymentsAndLiabilities;
	}
	
	private static FutaPaymentLiability createDummyFutaPaymentLiability(FutaAmountType type) {
		FutaPaymentLiability fpl = new FutaPaymentLiability();
		fpl.setFutaAmountType(type);
		if (type.equals(FutaAmountType.FUTALIABILITY)) {
			fpl.setAmount(sAmountLiability.getAmount());
		} else {
			fpl.setAmount(sAmountPayment.getAmount());
		}
		return fpl;
	}
}
