package com.intuit.sb.payroll.services.taxpayments;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;


import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentLineItem;
import com.intuit.sb.payroll.entity.model.PaymentLineItemSplit;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.services.taxpayments.impl.PaymentTypeLawMapper;

/** Utility class for {@link TaxPayments} Tests */
public class TaxPaymentsTestUtil {

	public static ToDateTaxPayments createToDateTaxPayments() throws Exception {
		ToDateTaxPayments todate = new ToDateTaxPayments();
		
//		Create 2 TaxPayments - both in Q2
		Calendar cal1 = LocaleGuru.getInstance().getCalendar();	//	now
		cal1.set(Calendar.MONTH, 3);	//	Apr
		cal1.set(Calendar.DAY_OF_MONTH, 10);	//	10th
		
		todate.addTaxPayments(createTaxPaymentsDto(cal1.getTime()));

		Calendar cal2 = LocaleGuru.getInstance().getCalendar();	//	now
		cal2.set(Calendar.MONTH, 4);	//	May
		cal2.set(Calendar.DAY_OF_MONTH, 14);	//	14th
		todate.addTaxPayments(createTaxPaymentsDto(cal2.getTime()));
		
		return todate;
	}
	
	/**
	 * Create {@link TaxPayments TaxPayments} object with date as given
	 * @param date
	 */
	public static TaxPayments createTaxPaymentsDto(Date date) throws Exception {
		TaxPayments tp = new TaxPayments();
		tp.setTaxPaymentDate(date);
		for (int i = 0; i < 3; i++) {
			tp.addTaxPayments(createTaxPaymentsLineItem());
		}
		return tp;
	}
	
	// Create TaxPaymentLineItem with random values
	public static TaxPaymentLineItem createTaxPaymentsLineItem() throws Exception {
		TaxPaymentLineItem tpLineItem = new TaxPaymentLineItem();
		tpLineItem.setAmount(generateRandomAmount());
		tpLineItem.setLawId(getRandomLawId());
		tpLineItem.setPaymentType(getRandomPaymentType().name());

		return tpLineItem;
	}
	
	public static Payment[] createPayments(int numberOfPayments) throws Exception {
		Payment[] payments = new Payment[numberOfPayments];
		int day = 1;	//	initially 1st of the month
		Calendar cal = LocaleGuru.getInstance().getCalendar();	//	now
		cal.set(Calendar.MONTH, 4);	//	May
		for (int i = 0; i < numberOfPayments; i++) {
			cal.set(Calendar.DAY_OF_MONTH, day);	//	generated
			payments[i] = createPayment(cal);
			// increment by 2 days
			day += 2;
		}
		return payments;
	}

	// Create TaxPaymentLineItem with random values
	public static Payment createPayment(Calendar cal) throws Exception {
		Payment payment = new Payment();
		payment.setPaymentDate(cal.getTime());
		Set<PaymentLineItem> set = new HashSet<PaymentLineItem>();
		payment.setPaymentLineItems(set);
		int count = 0;	//	create 3 Payment line items
		PaymentLineItem pli = null;
		while (count <= 2) {
			pli = createPaymentLineItem();
			if (!doesPaymentContainThisLineItemType(pli.getPaymentType(), payment)) {
				set.add(pli);	//	ensure unique set of Payment Line Items
				count++;
			}
		}
		// Q2 for this year
		payment.setPayrollPeriod(
				PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER2, cal.get(Calendar.YEAR)));
		return payment;
	}
	
	private static boolean doesPaymentContainThisLineItemType(PaymentType type, Payment payment) {
		Set<PaymentLineItem> set = payment.getPaymentLineItems();
		for (PaymentLineItem paymentLineItem : set) {
			if (paymentLineItem.getPaymentType().equals(type)) {
				return true;
			}
		}
		return false;
	}

	// Create TaxPaymentLineItem with random values
	public static PaymentLineItem createPaymentLineItem() throws Exception {
		PaymentLineItem pLineItem = new PaymentLineItem();
		pLineItem.setAmount(generateRandomMoney());
		pLineItem.setPaymentType(getRandomPaymentType());
		PaymentLineItemSplit split = createPaymentLineItemSplit(pLineItem);
		Set<PaymentLineItemSplit> set = new HashSet<PaymentLineItemSplit>();
		set.add(split);
		pLineItem.setPaymentLineItemSplits(set);

		return pLineItem;
	}
	
	// Create TaxPaymentLineItem with random values
	public static PaymentLineItem create94xPaymentLineItem(Money amount) throws Exception {
		PaymentLineItem pLineItem = new PaymentLineItem();
		pLineItem.setAmount(amount);
		pLineItem.setPaymentType(PaymentType.FORM94x);
		PaymentLineItemSplit split = createPaymentLineItemSplit(pLineItem);
		Set<PaymentLineItemSplit> set = new HashSet<PaymentLineItemSplit>();
		set.add(split);
		pLineItem.setPaymentLineItemSplits(set);

		return pLineItem;
	}
	
	// Create TaxPaymentLineItem with random values
	public static PaymentLineItemSplit createPaymentLineItemSplit(PaymentLineItem pLineItem) throws Exception {
		PaymentLineItemSplit split = new PaymentLineItemSplit();
		split.setSplitAmount(pLineItem.getAmount());
		// assuming there will be only one LAW ID for our Payment Type
		List<PayrollItem> payrollItems = PaymentTypeLawMapper.mapPaymentTypeToPayrollItem(pLineItem.getPaymentType());
		split.setPayrollItem(payrollItems.get(0));
		return split;
	}
	
	public static List<PaymentType> getPaymentTypes() {
		List<PaymentType> types = new ArrayList<PaymentType>();
		types.add(PaymentType.CAETT);
		types.add(PaymentType.CAPIT);
		types.add(PaymentType.CASDI);
		types.add(PaymentType.CAUI);
		types.add(PaymentType.FORM940FUTA);
		return types;
	}
	
	// Load law ids (name and values) as a map using reflection
	@SuppressWarnings("unchecked")
	public static Map<String, String> getAllLawIdsMap() throws Exception {
		Map<String, String> map = new HashMap<String, String>();
		Class clazz = Class.forName("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants");
		Field[] fields = clazz.getDeclaredFields();
		for (Field field : fields) {
			map.put(field.getName(), (String)field.get(null));
		}
		
		return map;
	}
	
	// Load law ids (values) as a List using reflection
	@SuppressWarnings("unchecked")
	public static List<String> getAllLawIdsList() throws Exception {
		List<String> list = new ArrayList<String>();
		Class clazz = Class.forName("com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants");
		Field[] fields = clazz.getDeclaredFields();
		for (Field field : fields) {
			list.add((String)field.get(null));
		}
		
		return list;
	}
	
	// Load law ids (values) as a List 
	public static List<String> getAllLawIdsList2() throws Exception {
		List<String> list = new ArrayList<String>();
		list.add(QboeLawIdConstants.sSST_ERLawId);
		list.add(QboeLawIdConstants.sSST_EELawId);
		list.add(QboeLawIdConstants.sMED_EELawId);
		list.add(QboeLawIdConstants.sMED_ERLawId);
		list.add(QboeLawIdConstants.sCAETTLawId);
		list.add(QboeLawIdConstants.sCASUILawId);
		list.add(QboeLawIdConstants.sFUTALawId);
		list.add(QboeLawIdConstants.sCAPITLawId);
		list.add(QboeLawIdConstants.sCASDILawId);
		list.add(QboeLawIdConstants.sFICALawId);
		list.add(QboeLawIdConstants.sCobraPremiumCredit);
		


		return list;
	}
	
	// Load law ids (values) as a List 
	public static List<String> getAll94xLawIds() throws Exception {
		List<String> list = new ArrayList<String>();
		list.add(QboeLawIdConstants.sFICALawId);
		list.add(QboeLawIdConstants.sSST_ERLawId);
		list.add(QboeLawIdConstants.sSST_EELawId);
		list.add(QboeLawIdConstants.sMED_ERLawId);
		list.add(QboeLawIdConstants.sMED_EELawId);
		return list;
	}
	
	
	// retrieve random law id from specific list
	public static String getRandomLawId() throws Exception {
		int rand = -1;
		while (true) {
			rand = (int)(Math.random()*10);
			if (rand >= getAllLawIdsList2().size()) {
				continue;
			} else {
				break;
			}
		}
		return getAllLawIdsList2().get(rand);
	}
	
	// retrieve random law id from specific list
	public static PaymentType getRandomPaymentType() throws Exception {
		int rand = -1;
		while (true) {
			rand = (int)(Math.random()*10);
			if (rand >= getPaymentTypes().size()) {
				continue;
			} else {
				break;
			}
		}
		return getPaymentTypes().get(rand);
	}
	
	public static BigDecimal generateRandomAmount() {
		double rand = Math.random();	//	decimal # between 0.0 and 1.0
		rand = rand*100;				//  2 digits before decimal
		BigDecimal bd = new BigDecimal(rand);
		bd = bd.setScale(2, RoundingMode.HALF_UP);	//	 round off to 2 decimals
		
		return bd;
	}
	
	public static Money generateRandomMoney() {
		double rand = Math.random();	//	decimal # between 0.0 and 1.0
		rand = rand*100;				//  2 digits before decimal
		BigDecimal bd = new BigDecimal(rand);
		bd = bd.setScale(2, RoundingMode.HALF_UP);	//	 round off to 2 decimals
		
		Money m = new Money();
		m.setAmount(bd);
		Currency cur = Currency.getInstance(LocaleGuru.getInstance().getCurrency());
		m.setCurrencyCode(cur);
		return m;
	}
	
	/**
	 * Compares two ToDateTaxPayments objects and returns true is same
	 *  
	 * @param (ToDateTaxPayments expected, ToDateTaxPayments actual)
	 * @RETURN Boolean
	 * @throws Exception
	 */
	public static Boolean compareToDateTaxPayments(ToDateTaxPayments expected, ToDateTaxPayments actual){
		Boolean same = true;
		try{
		if(expected.getTaxPayments().length == actual.getTaxPayments().length){
			List<TaxPayments> expectedPaymentsList = Arrays.asList(expected.getTaxPayments());
			List<TaxPayments> actualPaymentsList = Arrays.asList(actual.getTaxPayments());
			
			for(TaxPayments expectedTaxPayment: expectedPaymentsList){
				Date date = expectedTaxPayment.getTaxPaymentDate();
				Boolean dateFound = false;
				
				for(TaxPayments actualTaxPayment: actualPaymentsList){
					if(actualTaxPayment.getTaxPaymentDate().equals(date)){
						dateFound = true;
						List<TaxPaymentLineItem> expectedItemsList = Arrays.asList(expectedTaxPayment.getTaxPaymentLineItems());
						List<TaxPaymentLineItem> actualItemsList = Arrays.asList(actualTaxPayment.getTaxPaymentLineItems());
						if(expectedItemsList.size()!=actualItemsList.size())
							throw new Exception("compare failed");
						
						else{
							for(TaxPaymentLineItem expectedLineItem: expectedItemsList){
								
								BigDecimal amount = expectedLineItem.getAmount();
								amount = amount.setScale(2, RoundingMode.HALF_UP);
								Boolean amountFound = false;
								for(TaxPaymentLineItem actualLineItem: actualItemsList){
									if(actualLineItem.getAmount().equals(amount)){
										amountFound = true;
										break;
									}										
								}
								if(!amountFound)
									throw new Exception("compare failed");																
							}							
						}
					}
				}
				if(!dateFound)
					throw new Exception("compare failed");
			}				
		}
		else
			same = false;
		}catch(Exception e){
			same = false;			
		}
		return same;
	}
	
	public static Payment addPaymentWithDateAnd94xItem(Calendar cal, Money lineItem94xpart) throws Exception{
		Payment payment = TaxPaymentsTestUtil.createPayment(cal);
		PaymentLineItem form94xLineItem = TaxPaymentsTestUtil.create94xPaymentLineItem(lineItem94xpart);
		payment.addPaymentLineItem(form94xLineItem);
		return payment;
	}
	
}
