package com.intuit.sb.payroll.services.wages;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;


/**
 * Tests the CompensationItem operations of the WorkerService for the CompensationItem Reimbursement 
 * @author ss2
 *
 */

public class ReimbursementWageItemTest extends WorkerServiceTestBase{
	
	 protected static List<CompensationItem> compensationItems = null;
	 protected static WageService sWageService;
	 
	 @BeforeClass
	 public static void initializeReimbursementWageItemTestClass()
	 {
		 sWageService = qaBootstrap.getEjbFromClass(WageService.class);
	 }
	 
	 /**
		 * <P>Adds Reimbursement Wage Item with Varied Rate and Checks for successful Addition of it</P>
		 *
		 * <H3>Steps:</H3>
		 * <OL>
		 * <LI>Set up Reimbursement Wage Item with Varied rate</LI>
		 * <LI>Call WageService's saveCompensationItems command</LI>
		 * </OL>
		 *
		 * <UL>
		 * <LI>Verify the list of Wage items returned matches what's expected 
		 * </UL>
		 */	 
	 
	  @Test
	 public void testAddReimbursementWithVariedRate() throws Exception {
	     //Setup
		 List<CompensationItem> compensationItemList = setUpReimbursementWageItem(false);
		 CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		 wageItems = compensationItemList.toArray(wageItems);
		 
		 // Exercise
		 sWageService.saveCompensationItems(sWorkerId,mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, wageItems));
		 
		 //Verify
		 CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		 AssertJUnit.assertTrue("Wage items mismatch!",verifyCompensationItems(updatedWageItems,compensationItemList));				
	}	
	
	  /**
		 * <P>Adds Reimbursement Wage Item with Fixed Rate and Checks for successful Addition of it</P>
		 *
		 * <H3>Steps:</H3>
		 * <OL>
		 * <LI>Set up Reimbursement Wage Item with Fixed rate</LI>
		 * <LI>Call WageService's saveCompensationItems command</LI>
		 * </OL>
		 *
		 * <UL>
		 * <LI>Verify the list of Wage items returned matches what's expected 
		 * </UL>
		 */	 
	 	  
	 @Test
	 public void testAddReimbursementWithFixedRate() throws Exception {
		 //Setup
		 List<CompensationItem> compensationItemList = setUpReimbursementWageItem(true);
		 CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		 wageItems = compensationItemList.toArray(wageItems);
		 
		 // Exercise
		 sWageService.saveCompensationItems(sWorkerId,mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, wageItems));
		 
		 //Verify
		 CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		 AssertJUnit.assertTrue("Wage items mismatch!",verifyCompensationItems(updatedWageItems,compensationItemList));
	}
		

	 
	 @Test(expectedExceptions=ValidationException.class)
	 public void testFixedReimbursementWithNullRate() throws Exception {
		 // Setup
		 CompensationItem compItem = new CompensationItem();
		 compItem.setCompensationType(CompensationType.Reimbursement);
		 compItem.setFixedRate(true);
		 compItem.setName("Reimbursement");
		 compItem.setRate(null);
		 
		 CompensationItem[] wageItems = new CompensationItem[1];
		 wageItems[0]=compItem;
		 // Exercise
		 sWageService.saveCompensationItems(sWorkerId,mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, wageItems));
	 }
		 
	 
	 /**
		 * <P>Test updating Reimbursement Wage Item with Varied rate to Reimbursement wage Item with Fixed rate</P>
		 *
		 * <H3>Steps:</H3>
		 * <OL>
		 * <LI>Set up Reimbursement wage item with Varied rate</LI>
		 * <LI>Call WageService's saveCompensationItems command</LI>
		 * <LI>Retrieve the saved Wage Item</LI>
		 * <LI>Modify the retrieved wage Item to hold the Reimbursement Wage Item With Fixed rate</LI>
		 * <LI>Call WageService's saveCompensationItems command to perform the actual work (UPDATE)</LI>
		 * </OL>
		 *
		 * <UL>
		 * <LI>Verify the list of wage items returned matches what's expected 
		 * </UL>
		 */
	 
	 @Test 
	 public void testUpdateVariedRateReimbursementToFixedRateReimbursement() throws Exception {
		 //Setup
		 List<CompensationItem> compensationItemList = setUpReimbursementWageItem(false);
		 CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		 wageItems = compensationItemList.toArray(wageItems);
		 // Exercise
		 sWageService.saveCompensationItems(sWorkerId,mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, wageItems));
	     CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
			
		 //Update the varied Reimbursement wage to fixed rate 
		 List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		 for (CompensationItem compItem : updatedWageItems) {
			 if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {
				 if (compItem.getRate()!=null) {
					 AssertJUnit.fail("Varied Reimbursement cannot have a rate");
				 }
				 compItem.setFixedRate(true);
				 compItem.setRate(new BigDecimal("111.11"));
				 compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
				 } else {
					 AssertJUnit.fail("Reimbursement Wage Item not saved as expected");
				 }
		 }
			
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
			} else {
				AssertJUnit.fail("No Compensation Items to Update");
			}
			//Verify that Reimbursement has been updated
			CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
			for (CompensationItem cItem : workerWageItemsNew) {
				if (CompensationType.Reimbursement.equals(cItem.getCompensationType())) {
					BigDecimal value = cItem.getRate();
					if (!value.equals(new BigDecimal("111.11"))) {
						AssertJUnit.fail("Varied Reimbursement Wage Item not updated to Fixed Reimbursement Wage Item");
					}
				} else {
					AssertJUnit.fail("Reimbursement Component Item not saved");
				}
			}
		}
		
	 
	 /**
		 * <P>Test updating Reimbursement Wage Item with Fixed rate to Reimbursement wage Item with Varied rate</P>
		 *
		 * <H3>Steps:</H3>
		 * <OL>
		 * <LI>Set up Reimbursement wage item with Fixed rate</LI>
		 * <LI>Call WageService's saveCompensationItems command</LI>
		 * <LI>Retrieve the saved Wage Item</LI>
		 * <LI>Modify the retrieved wage Item to hold the Reimbursement Wage Item With Varied rate</LI>
		 * <LI>Call WageService's saveCompensationItems command to perform the actual work (UPDATE)</LI>
		 * </OL>
		 *
		 * <UL>
		 * <LI>Verify the list of wage items returned matches what's expected 
		 * </UL>
		 */
	 
	 @Test 
	 public void testUpdateFixedRateReimbursementToVariedRateReimbursement() throws Exception {
		 //Setup
		 List<CompensationItem> compensationItemList = setUpReimbursementWageItem(true);
		 CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		 wageItems = compensationItemList.toArray(wageItems);
		 
		 // Exercise
		 sWageService.saveCompensationItems(sWorkerId,mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, wageItems));
	   	 CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		  
		 //Update the fixed Reimbursement wage to varied rate 
		 List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		 for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {
				compItem.setFixedRate(false);
				compItem.setRate(new BigDecimal("111.11"));
				compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else {
						AssertJUnit.fail("Reimbursement Wage Item not saved as expected");
			}
		}
			
		//If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else {
				AssertJUnit.fail("No Compensation Items to Update");
		}

		//Verify that Reimbursement has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Reimbursement.equals(cItem.getCompensationType())) {
				if (cItem.getRate() == null) {
					AssertJUnit.fail("Varied Reimbursement wage item should not be null");
				}
			} else {
					AssertJUnit.fail("Reimbursement Compensation Item not saved");
			}
		}
	}
	 
	 	/**
		 * <P>Test Deleting Reimbursement Wage Item with Fixed Rate</P>
		 *
		 * <H3>Steps:</H3>
		 * <OL>
		 * <LI>Set up Reimbursement Wage Item with Fixed Rate</LI>
		 * <LI>Call WageService's saveCompensationItems command to perform the actual save work</LI>
		 * <LI>Call WageService's getCompensationItems command to return the saved compensations</LI>
		 *  <LI>Call WageService's saveCompensationItems command to delete the Reimbursement wage Item</LI>
		 * </OL>
		 *
		 * <UL>
		 * <LI>Verify the list of wage items returned matches what's expected 
		 * </UL>
		 */
	 @Test
	 public void testDeleteReimbursementFixedRate() throws Exception{
		 	 
		 //Setup
		 List<CompensationItem> compensationItemList = setUpReimbursementWageItem(true);
		 CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		 wageItems = compensationItemList.toArray(wageItems);
		
		 // Exercise
		 sWageService.saveCompensationItems(sWorkerId,mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, wageItems));
 		 CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
 		 List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

 		 for (CompensationItem compItem : updatedWageItems) {
				if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		
		//If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		//Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Reimbursement Wage Item not deleted");
		}
	} 
	 
	 	/**
		 * <P>Test Deleting Reimbursement Wage Item with Varied Rate</P>
		 *
		 * <H3>Steps:</H3>
		 * <OL>
		 * <LI>Set up Reimbursement Wage Item with Varied Rate</LI>
		 * <LI>Call WageService's saveCompensationItems command to perform the actual save work</LI>
		 * <LI>Call WageService's getCompensationItems command to return the saved compensations</LI>
		 *  <LI>Call WageService's saveCompensationItems command to delete the Reimbursement wage Item</LI>
		 * </OL>
		 *
		 * <UL>
		 * <LI>Verify the list of wage items returned matches what's expected 
		 * </UL>
		 */
	 @Test
	 public void testDeleteReimbursementWithVariedRate() throws Exception{
		 	 
		 //Setup
		 List<CompensationItem> compensationItemList = setUpReimbursementWageItem(false);
 		 CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		 wageItems = compensationItemList.toArray(wageItems);
		
		 // Exercise
		 sWageService.saveCompensationItems(sWorkerId,mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, wageItems));
		 CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
 		 List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
 		 
 		 for (CompensationItem compItem : updatedWageItems) {
				if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		
		//If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		//Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Reimbursement Wage Item not deleted");
		}
	}  
	 
	 
	 
	private SaveCompensationItemRequest mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation operation,CompensationItem compensationItem) {
			return new SaveCompensationItemRequest(compensationItem, operation);
	}
	
	/**
	 *  Sets up the CompensationItemList with ReimbursementWage Componsation Item
	 */
	private List<CompensationItem> setUpReimbursementWageItem(boolean fixedRate) {
		//SETUP
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		
		//Create Reimbursement Compensation Item
		CompensationItem reimbursement = new CompensationItem();
		reimbursement.setCompensationType(CompensationType.Reimbursement);
		reimbursement.setFixedRate(fixedRate);
		
		if (fixedRate) {
			reimbursement.setName("Fixed Reimbursement");
			reimbursement.setRate(new BigDecimal("111"));
		} else {
			reimbursement.setName("Varied Reimbursement");
			reimbursement.setRate(null);
		}
		compensationItemList.add(reimbursement);
		return compensationItemList;
	}
}
