package com.intuit.sb.payroll.services.workerpayments;

import java.util.Calendar;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;

public class CreatePayrollRunTest extends WorkerPaymentServiceTestBase{
	
    private static Long sReadyForPayWorkerId = null;

    @BeforeClass
    public static void setUpCreatePayrollRunTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createMVPayrollReadyWorker("Paul");
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCommitPayrollRunTest(): " + t.getMessage());
        }
    }

    @AfterClass
    public static void tearDownCreatePayrollRunTest() {
        try {
            getUserTransaction().begin();
            if (null != sReadyForPayWorkerId) {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCommitPayrollRunTest(): " + t.getMessage());
        }
    }
    
	/**
	 * <P>This test is to create a scheduled payroll run with one payroll ready worker with hourly wages</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add worker ID to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the workers paycheck is not calculated since we passed in a worker who 
	 * doesn't have any fixed wages. Also verify the payroll run created is in DRAFT state</LI>
	 * </UL>
	 * 	
	 */
    @Test
    public void testCreatePayrollRun() throws ValidationException, ServiceFailureException
    {
    	PayrollRunResponse payResponse = null;
    	try
    	{
    		//Create PayrollRunCreateRequest
    		PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
    		Long[] workerIds = new Long[1];
    		workerIds[0] = sReadyForPayWorkerId;

    		//create a pay period
    		PayrollRunPeriod period = createBiweeklyPayrollRunPeriod(DateUtil.createDate(2008, Calendar.DECEMBER, 14));
    		
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(period.getPayrollRunDate());

    		
    		createRequest.setWorkerIds(workerIds);
    		createRequest.setPeriod(period);
    		createRequest.setPayrollRunType(PayrollRunTypeEnum.SCHEDULED);
    		
    		//Execute
    		payResponse = sWorkerPayService.createPayrollRun(createRequest);
    		
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();            

    		
    		//Verify
    		AssertJUnit.assertEquals("PayPeriod mismatch!", period, payResponse.getPayrollRunPeriod());
    		AssertJUnit.assertEquals("PayrollRunType mismatch!", createRequest.getPayrollRunType(), payResponse.getPayrollRunType());
    		AssertJUnit.assertEquals("PayrollRunState mismatch!", PayrollRunState.DRAFT, payResponse.getPayrollRunState());
    		AssertJUnit.assertNotNull(payResponse.getPayrollRunId());
    	}
    	finally
    	{
    		//Nothing to do
    	}
    }
    


}
