package com.intuit.sb.payroll.services.workerpayments;


import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;


/**
 * Contains tests for testing the WorkerPaymentService's GetPayrollRunById() and GetPayrollRunSummaryById() API
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetPayrollRunTest extends WorkerPaymentServiceTestBase
{
    protected static PayrollRunSaveRequest sSaveRequest = null;
    private static Long workerId = null;
        
    @BeforeClass
    public static void setUpGetPayrollRun()
    {
        if (sSaveRequest == null)
        {
            try
            {
                getUserTransaction().begin();
                try
                {
                	workerId = createMVPayrollReadyWorker("CA_DUDE");
                	Long[] workerIds = new Long[] {workerId};
                    sSaveRequest = createPayrollRunSaveRequest(workerIds, null );
                }
                catch (ValidationException ex)
                {
                	ex.printStackTrace();
                    AssertJUnit.fail("Unexpected ValidationException was thrown! " + ex.getMessage());
                }
                getUserTransaction().commit();
            }
            catch (Throwable t)
            {
                sLogger.error(t.getMessage());
                try
                {
                    getUserTransaction().rollback();
                }
                catch (Exception e)
                {
                }
                AssertJUnit.fail("Error in setUpTaxes(): " + t.getMessage());
            }
        }
        
    }
    
    @AfterClass
    public static void tearDownGetPayrollRun()
    {
        try
        {
            getUserTransaction().begin();
            deleteAllPayrollRun();
            if (null != workerId)
            {
                cleanUpEIWorker(workerId);
                workerId = null;
            }
            sSaveRequest = null;
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in setUpTaxes(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
    }
    
    /**
     * Test method for
     * {@link com.intuit.sb.payroll.services.workerpayments.impl.WorkerPaymentServiceBean#getPayrollRunById(java.lang.String)}.
     * This test will try to retrieve a draft Payrollrun by Id in normal scenario
     * <ul>
     *  Steps
     *  <li>Create Payemployee Service </li>
     *  <li>Create a payrollrun as draft </li>
     *  <li>Call getPayrollRunById with the Id just created</li>
     *  <li>AssertJUnit if the retrieved payrollrun is the one saved as draft</li>
     * </ul>
     * @throws ValidationException
     * @throws PayrollRunNotFoundException
     */
    // Please don't ignore this this is not duplicate. Validate routine goes deep inside the
    // repo to validate stuff amalik
    @Test(enabled=false, groups={"broken"})
    public void testGetPayrollRunFromDraftById()
    {
        PayrollRunSummary runSummary;
        PayrollRunResponse response;
        try
        {
            runSummary = sWorkerPayService.saveDraftPayrollRun(sSaveRequest);
            response = sWorkerPayService.getPayrollRunById(runSummary.getPayrollRunId());
        }
        catch (Exception e)
        {
            e.printStackTrace();
            AssertJUnit.assertFalse("Unexcepted Exception thrown", true);
            return;
        }
        
        validatePayrollRunResponse(sSaveRequest, response, false);
    }

    /**
     * Test method for
     * {@link com.intuit.sb.payroll.services.workerpayments.impl.WorkerPaymentServiceBean#getPayrollRunById(java.lang.String)}.
     * This test will try to retrieve a draft Payrollrun by Id in normal scenario twice and check if
     * the second time retrived payroll run is updated with new paycheks
     * <ul>
     *  Steps
     *  <li>Create Payemployee Service </li>
     *  <li>Create a payrollrun as draft </li>
     *  <li>Call getPayrollRunById with the Id just created</li>
     *  <li>Save Current Paycheck details</li>
     *  <li>Call getPayrollRunById with the Id just created again</li>
     *  <li>Verify if the paychecks are updated</li>
     * </ul>
     * @throws ValidationException
     * @throws PayrollRunNotFoundException
     */
    @Test(enabled=false, groups={"broken"})
    public void testGetPayrollRunAfterCalcPaycheckChanged()
    {
        PayrollRunSummary runSummary;
        PayrollRunResponse response;
        try
        {
            runSummary = sWorkerPayService.saveDraftPayrollRun(sSaveRequest);
            response = sWorkerPayService.getPayrollRunById(runSummary.getPayrollRunId());
            PaycheckDetails[] paycheckList = response.getPaycheckDetails();
            Long oldPaycheckId = paycheckList[0].getPaycheckId();
            response = sWorkerPayService.getPayrollRunById(runSummary.getPayrollRunId());
            paycheckList = response.getPaycheckDetails();
            Long newPaycheckId = paycheckList[0].getPaycheckId();
            AssertJUnit.assertNotSame(oldPaycheckId, newPaycheckId);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            AssertJUnit.assertFalse("Unexcepted Exception thrown", true);
            return;
        }

        validatePayrollRunResponse(sSaveRequest, response, false);
    }

    /**
     * Test method for
     * {@link com.intuit.sb.payroll.services.workerpayments.impl.WorkerPaymentServiceBean#getPayrollRunById(java.lang.String)}.
     * This test will try to retrieve a committed Payrollrun by Id in normal scenario
     * <ul>
     *  Steps
     *  <li>Create Payemployee Service </li>
     *  <li>Create a payrollrun as by committing the payrollrun </li>
     *  <li>Call getPayrollRunById with the Id just created</li>
     *  <li>AssertJUnit if the retrieved payrollrun is the one committed</li>
     * </ul>
     * @throws ValidationException
     * @throws PayrollRunNotFoundException
     */
    // Please don't ignore this this is not duplicate. Validate routine goes deep inside the
    // repo to validate stuff  amalik
    @Test(enabled=false, groups={"broken"})
    public void testGetCommittedPayrollRunById()
    {
        PayrollRunSummary runSummary;
        PayrollRunResponse response;
        try
        {
            runSummary = sWorkerPayService.commitPayrollRun(sSaveRequest).getPayrollRunSummary();
            response = sWorkerPayService.getPayrollRunById(runSummary.getPayrollRunId());
        }
        catch (Exception e)
        {
            e.printStackTrace();
            AssertJUnit.assertFalse("Unexcepted Exception thrown", true);
            return;
        }
        validatePayrollRunResponse(sSaveRequest, response, true);
    }

    /**
     * Test method for
     * {@link com.intuit.sb.payroll.services.workerpayments.impl.WorkerPaymentServiceBean#getPayrollRunSummaryById(java.lang.String)}.
     * This test will try to retrieve run summary a draft Payrollrun by Id in normal scenario
     * <ul>
     *  Steps
     *  <li>Create Payemployee Service </li>
     *  <li>Create a payrollrun as draft </li>
     *  <li>Call getPayrollRunSummaryById with the Id just created</li>
     *  <li>AssertJUnit if the retrieved payrollrun summary is that of one saved as draft</li>
     * </ul>
     */
    @Test(enabled=false, groups={"broken"})
    public void testGetDraftPayrollRunSummaryById()
    {
        PayrollRunSummary runSummary;
        try
        {
            runSummary = sWorkerPayService.saveDraftPayrollRun(sSaveRequest);
            runSummary = sWorkerPayService.getPayrollRunSummaryById(runSummary.getPayrollRunId());
        }
        catch (Exception e)
        {
            e.printStackTrace();
            AssertJUnit.assertFalse("Unexcepted Exception thrown", true);
            return;
        }
        validatePayrollRunSummary(sSaveRequest, runSummary, false);
    }

    /**
     * Test method for
     * {@link com.intuit.sb.payroll.services.workerpayments.impl.WorkerPaymentServiceBean#getPayrollRunSummaryById(java.lang.String)}.
     * This test will try to retrieve run summary a committed Payrollrun by Id in normal scenario
     * <ul>
     *  Steps
     *  <li>Create Payemployee Service </li>
     *  <li>Create a payrollrun by commiting it </li>
     *  <li>Call getPayrollRunSummaryById with the Id just created</li>
     *  <li>AssertJUnit if the retrieved payrollrun summary is that of one committed</li>
     * </ul>
     */
    @Test(enabled=false, groups={"broken"})
    public void testGetCommittedPayrollRunSummaryById()
    {
        PayrollRunSummary runSummary;
        try
        {
            runSummary = sWorkerPayService.commitPayrollRun(sSaveRequest).getPayrollRunSummary();
            runSummary = sWorkerPayService.getPayrollRunSummaryById(runSummary.getPayrollRunId());
        }
        catch (Exception e)
        {
            e.printStackTrace();
            AssertJUnit.assertFalse("Unexcepted Exception thrown", true);
            return;
        }
        validatePayrollRunSummary(sSaveRequest, runSummary, true);
    }

    /**
     * Test method for
     * {@link com.intuit.sb.payroll.services.workerpayments.impl.WorkerPaymentServiceBean#getPayrollRunSummaryById(java.lang.String)}.
     * This test will try to retrieve run summary a draft Payrollrun by Id in normal scenario
     * <ul>
     *  Steps
     *  <li>Create Payemployee Service </li>
     *  <li>Create a payrollrun as draft </li>
     *  <li>Call getPayrollRunSummaryById with the Id just created</li>
     *  <li>Save Current Paycheck details</li>
     *  <li>Call getPayrollRunSummaryById with the Id just created again</li>
     *  <li>Verify if the paychecks are updated</li>
     *  <li>AssertJUnit if the retrieved payrollrun summary is that of one saved as draft</li>
     * </ul>
     */
    @Test(enabled=false, groups={"broken"})
    public void testGetPayrollSummaryAfterCalcPaycheckChanged()
    {
        PayrollRunSummary runSummary;
        try
        {
            runSummary = sWorkerPayService.saveDraftPayrollRun(sSaveRequest);
            runSummary = sWorkerPayService.getPayrollRunSummaryById(runSummary.getPayrollRunId());
            PaycheckSummary[] pcSummary = runSummary.getPaycheckSummaries();
            Long oldPaycheckId = pcSummary[0].getPaycheckId();
            runSummary = sWorkerPayService.getPayrollRunSummaryById(runSummary.getPayrollRunId());
            pcSummary = runSummary.getPaycheckSummaries();
            Long newPaycheckId = pcSummary[0].getPaycheckId();
            AssertJUnit.assertNotSame(oldPaycheckId, newPaycheckId);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            AssertJUnit.assertFalse("Unexcepted Exception thrown", true);
            return;
        }
        validatePayrollRunSummary(sSaveRequest, runSummary, false);
    }
}
