package com.intuit.sb.payroll.services.workerpayments;

import java.util.Calendar;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestBase;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.services.workerpayments.impl.PayrollFrequencyMapper;
import com.intuit.sb.payroll.services.workerpayments.impl.PayrollScheduleMapper;

/**
 * @author dkulkarni
 */
public class PayrollScheduleMapperTest extends PayrollScheduleTestBase
{
    protected static WorkerPaymentService sWorkerPaymentService;

    @BeforeClass
    public static void initializePayrollScheduleMapperTestClass()
    {
    	sWorkerPaymentService = qaBootstrap.getEjbFromClass(WorkerPaymentService.class);
    }
    
    @BeforeMethod
    public void testInitialize()
    {
        deleteAllPayrollSchedule();
    }
    
    /**
     * Test method for
     * {@link com.intuit.sb.payroll.services.workerpayments.impl.PayrollScheduleMapper#mapFromDEntityToEI(com.intuit.sb.payroll.component.PayrollSchedule)}.
     */
    @Test
    public void testMapFromNullDEntityToEI()
    {
        PayrollSchedule eiPayrollSchedule = PayrollScheduleMapper.mapFromDEntityToEI(null);
        AssertJUnit.assertNull(eiPayrollSchedule);
    }

    /**
     * Test method for
     * {@link com.intuit.sb.payroll.services.workerpayments.impl.PayrollScheduleMapper#mapFromDEntityToEI(com.intuit.sb.payroll.component.PayrollSchedule)}.
     */
    @Test
    public void testMapFromSimpleDEntityToEI() throws Exception
    {
        // Need to persist this as we need mapping of Unique Id from Component.PayrollSchedule
        createPayrollSchedules(new PayrollFrequencyEnum[] { PayrollFrequencyEnum.WEEKLY });
        List<com.intuit.sb.payroll.entity.model.PayrollSchedule> schedList = getPayrollSchedules();
        com.intuit.sb.payroll.entity.model.PayrollSchedule dePayrollSchedule = schedList.get(0);

        // Updating a Domain Entity PayrollSchedule dePayrollSchedule
        dePayrollSchedule.setFrequency(PayrollFrequencyEnum.MONTHLY);
        dePayrollSchedule.setMemo("Memo Memo Memo Memo Memo");
        dePayrollSchedule.setNextPayPeriodEndDate(DateUtil.createDate(2009, 5, 15));  // June 15, 2009
        dePayrollSchedule.setPeriodStartDate(DateUtil.createDate(2009,4,18)); // May 18, 2009
        dePayrollSchedule.setPayDayOffset(3);
        dePayrollSchedule.setPayrollRunDayOffset(4);
        dePayrollSchedule.setDefaultt(true);

        // Mapping from Domain Entity PayrollSchedule dePayrollSchedule
        // to Entity Service PayrollSchedule eiPayrollSchedule
        PayrollSchedule eiPayrollSchedule = PayrollScheduleMapper.mapFromDEntityToEI(dePayrollSchedule);

        // AssertJUniting fields of eiPayrollSchedule against
        PayrollFrequencyEnum eiFreq = PayrollFrequencyMapper.mapFromEIToDEntity(eiPayrollSchedule.getFrequency());
        AssertJUnit.assertEquals(dePayrollSchedule.getFrequency(), eiFreq);
        AssertJUnit.assertEquals(Integer.valueOf(dePayrollSchedule.getPayDayOffset()), Integer.valueOf(eiPayrollSchedule.getPayrollPayDateOffset()));
        AssertJUnit.assertEquals(dePayrollSchedule.getPayrollRunDayOffset(), Integer.valueOf(eiPayrollSchedule.getPayrollRunDateOffset()));
        AssertJUnit.assertEquals(dePayrollSchedule.getId(), eiPayrollSchedule.getObjectHandle().getId());
        
        Calendar cal1 = DateUtil.getNowCalendar();
        cal1.setTime(eiPayrollSchedule.getPeriodStartDate());
        AssertJUnit.assertTrue("Period StartDate mismatch", DateUtil.isSameDay(cal1, DateUtil.createCalendar(2009,4,18))); // May 18, 2009
         
        cal1.setTime(eiPayrollSchedule.getPeriodEndDate());
        AssertJUnit.assertTrue("Period EndDate mismatch", DateUtil.isSameDay(cal1, DateUtil.createCalendar(2009,5,15))); // June 15, 2009
                        
    }

    /**
     * Test method for
     * {@link com.intuit.sb.payroll.services.workerpayments.impl.PayrollScheduleMapper#mapFromEIToDEntity(com.intuit.sb.payroll.services.workerpayments.PayrollSchedule)}.
     */
    @Test(enabled=false, groups={"broken"})
    public void testMapFromEIToDEntity() throws Exception
    {
        createPayrollSchedules(new PayrollFrequencyEnum[] { PayrollFrequencyEnum.WEEKLY });
        PayrollSchedule eiPayrollSchedule = sWorkerPaymentService.getPayrollSchedule();

        // Updating a Domain Entity PayrollSchedule dePayrollSchedule
        eiPayrollSchedule.setFrequency(PayrollScheduleFrequency.WEEKLY);
        eiPayrollSchedule.setPayrollPayDateOffset(4);
        eiPayrollSchedule.setPeriodStartDate(DateUtil.createDate(2010, 10,7)); // Nov 7, 2010
        eiPayrollSchedule.setPeriodEndDate(DateUtil.createDate(2010, 11,6)); // Dec 6, 2010
        eiPayrollSchedule.setPayrollRunDateOffset(-1); // Dec 5, 2010

        // Mapping from Entity Service PayrollSchedule eiPayrollSchedule
        // to Domain Entity PayrollSchedule dePayrollSchedule
        com.intuit.sb.payroll.entity.model.PayrollSchedule dePayrollSchedule =
            PayrollScheduleMapper.mapFromEIToDEntity(eiPayrollSchedule);

        // AssertJUniting fields of dePayrollSchedule against eiPayrollSchedule
        PayrollFrequencyEnum eiFreq = PayrollFrequencyMapper.mapFromEIToDEntity(eiPayrollSchedule.getFrequency());
        AssertJUnit.assertEquals(dePayrollSchedule.getFrequency(), eiFreq);
        AssertJUnit.assertEquals(Integer.valueOf(dePayrollSchedule.getPayDayOffset()), Integer.valueOf(eiPayrollSchedule.getPayrollPayDateOffset()));
        Calendar cal1 = DateUtil.getNowCalendar();
        cal1.setTime(dePayrollSchedule.getNextPayPeriodEndDate());
        AssertJUnit.assertEquals(cal1, DateUtil.createCalendar(2010, 11,6));
        
        AssertJUnit.assertEquals(dePayrollSchedule.getPayrollRunDayOffset().intValue(), -1);
    }

}
