package com.intuit.sb.payroll.services.workerpayments;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO;
import com.intuit.sb.payroll.services.common.EntityTestBaseHelper;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.taxes.FormType;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingInfoRequest;
import com.intuit.sb.payroll.services.taxes.WithholdingInfoOperation;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.taxes.WithholdingStatus;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.CompensationUtils;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerType;


/**
 *  <P>The <CODE>PayEmployeeServiceTestBase</CODE> class contains helper methods for testing
 *     IPayEmployeeService.calcPaycheckDetails()</P>
 */
public class WorkerPaymentServiceTestBase extends EntityUtilsTestBase
{
    protected static WorkerPaymentService sWorkerPayService;
    protected static WorkerPaymentsComponent workerPaymentsComponent;
    protected static PayrollRunEAO prEAO;

    protected static Date sPeriodStartDate = null;
    protected static Date sWeeklyPeriodEndDate = null;
    protected static Date sBiweeklyPeriodEndDate = null;
    protected static Date sSemimonthlyPeriodEndDate = null;
    protected static Date sMonthlyPeriodEndDate = null;

    protected static String sSalaryWageDescStr = null;
    protected static String sHourlyWageDescStr = null;
    protected static String sOvertimeWageDescStr = null;
    protected static String sDoubletimeWageDescStr = null;
    protected static String sPaycheckTipsWageDescStr = null;
    protected static String sCashTipsWageDescStr = null;
    protected static String sCashTipsOffsetDescStr = null;
    protected static String sAllocatedTipsWageDescStr = null;
    protected static String sBonusWageDescStr = null;
    protected static String sPieceworkWageDescStr = null;
    protected static String sCommissionWageDescStr = null;
    protected static String sFICA_DescStr = null;
    protected static String sSST_EE_DescStr = null;
    protected static String sSST_ER_DescStr = null;
    protected static String sMEDT_EE_DescStr = null;
    protected static String sMEDT_ER_DescStr = null;
    protected static String sFUTA_DescStr = null;
    protected static String sCAPIT_DescStr = null;
    protected static String sCASDI_DescStr = null;
    protected static String sCASUI_DescStr = null;
    protected static String sCAETT_DescStr = null;
    protected static String sCA_SF_PT_DescStr = null;
    protected static String sTXSUI_DescStr = null;
    
    protected static int sNumberOfWageItems = 1;
    protected static int sWorkerWageInfoSize = 1;

    protected Date mPaycheckDate = null;
    protected Date mPeriodEndDate = null;

    protected static boolean sDataInitialized = false;
    protected static PayrollSchedule pSch = null;
    protected static int sDefaultPayrollDayOffset = 3;
    protected static int sDefaultPayrollRunDayOffset = 1;
    protected static int sDefaultPayPeriodEndDayOfMonth1 = 0;
    protected static int sDefaultPayPeriodEndDayOfMonth2 = 0;
    protected static int sDefaultWeekCounter = 2;
	protected static CompensationItem notACompensationItem = null;
    
    private static boolean sDescStrInitialized = false;
    private static final int sTaxYear = 2008;
    
    public static void initializeEAOs()
    {
    	sWorkerPayService = qaBootstrap.getEjbFromClass(WorkerPaymentService.class);
    	prEAO = qaBootstrap.getEjbFromClass(PayrollRunEAO.class);
    	workerPaymentsComponent = qaBootstrap.getEjbFromClass(WorkerPaymentsComponent.class);
    }
    
    @BeforeClass
    public static void setUpWorkerPaymentService()
    {    	
    	initializeEAOs();
        if (!sDataInitialized)
        {
            try
            {
                getUserTransaction().begin();
                setUpDates();
                initPayrollItemDescriptions();
                sDataInitialized = true;
                getUserTransaction().commit();
            }
            catch (Throwable t)
            {
                sLogger.error(t.getMessage());
                try
                {
                    getUserTransaction().rollback();
                }
                catch (Exception e)
                {
                }
                AssertJUnit.fail("Error in setUpWorkerPaymentService(): " + t.getMessage());
            }
        }
    }
    
    private static void initPayrollItemDescriptions()
    {
        if (!sDescStrInitialized)
        {
            List<PayrollItem>  payrollItemList = entityService.findAll(PayrollItem.class);
            for (int i = 0; i < payrollItemList.size(); i++)
            {
            	//FIXME [Test] Use the payrollItem instanceof to find the type, the law id's should be opaque.
                PayrollItem item = payrollItemList.get(i);
                String lawId = item.getLawId().getValue();
                if (lawId.equals(QboeLawIdConstants.sHourlyLawId))
                {
                    sHourlyWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sOvertimeLawId))
                {
                    sOvertimeWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sDoubleTimeLawId))
                {
                    sDoubletimeWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sSalaryLawId))
                {
                    sSalaryWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sPaycheckTipsLawId))
                {
                    sPaycheckTipsWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sCashTipsLawId))
                {
                    sCashTipsWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sCashTipsOutLawId))
                {
                	sCashTipsOffsetDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sAllocatedTipsLawId))
                {
                    sAllocatedTipsWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sBonusLawId))
                {
                    sBonusWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sPieceworkLawId))
                {
                    sPieceworkWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sCommissionLawId))
                {
                    sCommissionWageDescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sFICALawId))
                {
                    sFICA_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sSST_EELawId))
                {
                    sSST_EE_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sSST_ERLawId))
                {
                    sSST_ER_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sMED_EELawId))
                {
                    sMEDT_EE_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sMED_ERLawId))
                {
                    sMEDT_ER_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sFUTALawId))
                {
                    sFUTA_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sCAPITLawId))
                {
                    sCAPIT_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sCASDILawId))
                {
                    sCASDI_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sCASUILawId))
                {
                    sCASUI_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sCAETTLawId))
                {
                    sCAETT_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sCASFPTLawId))
                {
                    sCA_SF_PT_DescStr = item.getDescription();
                }
                else if (lawId.equals(QboeLawIdConstants.sTXSUILawId))
                {
                    sTXSUI_DescStr = item.getDescription();
                }
            }
            sDescStrInitialized = true;
        }
    }
    
    private static void setUpDates()
    {
        Calendar calendar = DateUtil.createCalendar(sTaxYear, 0, 1);
        sPeriodStartDate = calendar.getTime();
        Calendar tempCalendar = (Calendar)calendar.clone();
        tempCalendar.add(Calendar.DAY_OF_MONTH, 6);
        sWeeklyPeriodEndDate = tempCalendar.getTime();
        tempCalendar = (Calendar)calendar.clone();
        tempCalendar.add(Calendar.DAY_OF_MONTH, 13);
        sBiweeklyPeriodEndDate = tempCalendar.getTime();
        tempCalendar = (Calendar)calendar.clone();
        tempCalendar.add(Calendar.DAY_OF_MONTH, 14);
        sSemimonthlyPeriodEndDate = tempCalendar.getTime();
        tempCalendar = (Calendar)calendar.clone();
        tempCalendar.add(Calendar.MONTH, 1);
        tempCalendar.add(Calendar.DAY_OF_MONTH, -1);
        sMonthlyPeriodEndDate = tempCalendar.getTime();
    }
    
    protected void setupPaycheckDates(PayrollScheduleFrequency payFrequency)
    {
        if (PayrollScheduleFrequency.WEEKLY.equals(payFrequency))
        {
            mPeriodEndDate = (Date)sWeeklyPeriodEndDate.clone();
        }
        else if (PayrollScheduleFrequency.BIWEEKLY.equals(payFrequency))
        {
            mPeriodEndDate = (Date)sBiweeklyPeriodEndDate.clone();
        }
        else if (PayrollScheduleFrequency.SEMIMONTHLY.equals(payFrequency))
        {
            mPeriodEndDate = (Date)sSemimonthlyPeriodEndDate.clone();
        }
        else if (PayrollScheduleFrequency.MONTHLY.equals(payFrequency))
        {
            mPeriodEndDate = (Date)sMonthlyPeriodEndDate.clone();
        }
        mPaycheckDate = (Date)mPeriodEndDate.clone();
    }
    
    protected PayrollRunPeriod createPayrollRunPeriod(PayrollScheduleFrequency frequency)
    {
        // Set up paycheck related dates
        setupPaycheckDates(frequency);
        PayrollRunPeriod payrollRunPeriod = new PayrollRunPeriod();
        payrollRunPeriod.setFrequency(frequency);
        payrollRunPeriod.setPeriodStartDate(sPeriodStartDate);
        payrollRunPeriod.setPeriodEndDate(mPeriodEndDate);
        payrollRunPeriod.setPayrollRunDate(mPeriodEndDate);
        payrollRunPeriod.setPayDate(mPaycheckDate);
        return payrollRunPeriod;
    }
    
    /**
     * This utility method will calculate the next PayrollRunPeriod values given a PayrollRunPeriod.  It assumes the
     * following:
     * 		Semimonthly - it's always on the 15th and last day of the month.
     * 		Monthly - It's always the last day of the month.
     * 
     * @param origPayrollRunPeriod The current PayrollRunPeriod to use to calculate the next one
     * @return A PayrollRunPeriod filled with the values for the next PayrollRun Period
     */
    protected PayrollRunPeriod calculateNextPayrollRunPeriod(PayrollRunPeriod origPayrollRunPeriod)
    {
    	int days = 0;
    	PayrollScheduleFrequency frequency = origPayrollRunPeriod.getFrequency();
    	if (PayrollScheduleFrequency.BIWEEKLY.equals(frequency)) {
    		days = 14;
    	} else if (PayrollScheduleFrequency.WEEKLY.equals(frequency)) {
    		days = 7;
    	} else if (PayrollScheduleFrequency.SEMIMONTHLY.equals(frequency)) {
        	Date periodEndDate = origPayrollRunPeriod.getPeriodEndDate();
        	Calendar periodEnd = DateUtil.dateToCalendar(periodEndDate);
        	int day = periodEnd.get(Calendar.DAY_OF_MONTH);
    		if (15 == day) {
    			// Last period was for the first half of the month, so calculate the remaining # of days to end of month
    			days = DateUtil.getLastDayOfMonth(periodEnd) - 15;
    		} else {
    			// Last period was the second half of the month, so just add 15 days
    			days = 15;
    		}
    	} else if (PayrollScheduleFrequency.MONTHLY.equals(frequency)) {
        	Date periodStartDate = origPayrollRunPeriod.getPeriodStartDate();
        	Calendar periodStart = DateUtil.dateToCalendar(periodStartDate);
        	periodStart.add(Calendar.MONTH, 1);
    		days = DateUtil.getLastDayOfMonth(periodStart);
    	}
        PayrollRunPeriod payrollRunPeriod = new PayrollRunPeriod();
        payrollRunPeriod.setFrequency(origPayrollRunPeriod.getFrequency());
        payrollRunPeriod.setPeriodStartDate(DateUtil.getNextDate(origPayrollRunPeriod.getPeriodStartDate(), days));
        payrollRunPeriod.setPeriodEndDate(DateUtil.getNextDate(origPayrollRunPeriod.getPeriodEndDate(), days));
        payrollRunPeriod.setPayrollRunDate(DateUtil.getNextDate(origPayrollRunPeriod.getPayrollRunDate(), days));
        payrollRunPeriod.setPayDate(DateUtil.getNextDate(origPayrollRunPeriod.getPayDate(), days));
        return payrollRunPeriod;
    }
    
    protected PaycheckRequest createPaycheckRequest(Long workerId, CalcInputs[] inputs, PayrollRunPeriod payrollRunPeriod,
                                                    PaycheckDistributionMethod paycheckDistributionMethod)
    {
        PaycheckRequest paycheckRequest = new PaycheckRequest();
        paycheckRequest.setWorkerId(workerId);
        paycheckRequest.setCalcInputs(inputs);
        paycheckRequest.setPayrollRunPeriod(payrollRunPeriod);
        paycheckRequest.setDistributionMethod(paycheckDistributionMethod);
        return paycheckRequest;
    }
    
    protected PaycheckRequest createPaycheckRequestWithPrintMethod(Long workerId, CalcInputs[] inputs,
                                                                   PayrollRunPeriod payrollRunPeriod)
    {
        return createPaycheckRequest(workerId, inputs, payrollRunPeriod, PaycheckDistributionMethod.PRINT);
    }
    
    protected CompensationItem findCompensationItem(CompensationItem[] compensationItems, String name,
    				CompensationType compensationType) {
    	CompensationItem compensationItem = null;
    	for (CompensationItem aCompensationItem : compensationItems) {
    		if (name.equals(aCompensationItem.getName()) && (compensationType.equals(aCompensationItem.getCompensationType()))) {
    			compensationItem = aCompensationItem;
    			break;
    		}
    	}
    	AssertJUnit.assertNotNull("Can't find specified CompensationItem!", compensationItem);
    	return compensationItem;
    }
    
    public static CompensationItem[] setupWageItemsForAllWageTypes(CompensationItem[] compensationItems, String hourlyRate, double hourlyHours,
            String overtimeRate, double overtimeHours, String tipAmount, double doubleHours)
    {
    	CompensationItem[] setupCompensationItems = setupWageItems(compensationItems, hourlyRate, hourlyHours,
            overtimeRate, overtimeHours, tipAmount);
        CompensationItem hourlyCompensationItem = null;
    	for (int i = 0; i < setupCompensationItems.length; i++)
        {
            CompensationType wageType = setupCompensationItems[i].getCompensationType();
            if (CompensationType.Hourly.equals(wageType)) {
                // Remember the hourly item
                hourlyCompensationItem = setupCompensationItems[i];
            } 
            else if (CompensationType.Doubletime.equals(wageType)) {
            	setupCompensationItems[i].setRate(new BigDecimal(2.0));
                setupCompensationItems[i].setUnits(doubleHours);
                // hourly item must have been set up already before overtime can be set up
                AssertJUnit.assertNotNull("hourlyCompensationItem cannot be null!", hourlyCompensationItem);
                setupCompensationItems[i].setLinkedCompensationItem(hourlyCompensationItem);
            }
        }
    	return setupCompensationItems;
    }
    
    static public CompensationItem[] setupWageItems(CompensationItem[] compensationItems, String hourlyRate, double hourlyHours,
                                        String overtimeRate, double overtimeHours, String tipAmount)
    {
        CompensationItem[] setupCompensationItems = new CompensationItem[compensationItems.length];
        CompensationItem hourlyCompensationItem = null;
        for (int i = 0; i < compensationItems.length; i++)
        {
            setupCompensationItems[i] = new CompensationItem();
            CompensationType wageType = compensationItems[i].getCompensationType();
			setupCompensationItems[i].setName(compensationItems[i].getName());
			setupCompensationItems[i].setObjectHandle(compensationItems[i].getObjectHandle());
			setupCompensationItems[i].setCompensationType(wageType);
			setupCompensationItems[i].setFixedRate(compensationItems[i].isFixedRate());
			//set rate and unit to the same as in compensationItems[i]. Rate and unit will be overwritten by parameters passed in
			setupCompensationItems[i].setRate(compensationItems[i].getRate());
			setupCompensationItems[i].setUnits(compensationItems[i].getUnits());
  
            if (CompensationType.Hourly.equals(wageType)) {
            	setupCompensationItems[i].setRate(new BigDecimal(hourlyRate));
                setupCompensationItems[i].setUnits(hourlyHours);
                // Remember the hourly item
                hourlyCompensationItem = setupCompensationItems[i];
            }
            else if (CompensationType.Overtime.equals(wageType)) {
            	setupCompensationItems[i].setRate(new BigDecimal(overtimeRate));
                setupCompensationItems[i].setUnits(overtimeHours);
                // hourly item must have been set up already before overtime can be set up
                AssertJUnit.assertNotNull("hourlyCompensationItem cannot be null!", hourlyCompensationItem);
                setupCompensationItems[i].setLinkedCompensationItem(hourlyCompensationItem);
            }
            else if (CompensationType.Doubletime.equals(wageType)) {
            	// pass thru, let the setupWageItemsForAllWageTypes() method take care of setting it up
            }
            else if (CompensationType.TipsPaycheck.equals(wageType) ||
            		 CompensationType.TipsCash.equals(wageType) ||
            		 CompensationType.TipsAllocated.equals(wageType)) {
            	setupCompensationItems[i].setRate(new BigDecimal(tipAmount));
                setupCompensationItems[i].setUnits(0.0);
            }
            else if (CompensationType.Bonus.equals(wageType)) {
            	setupCompensationItems[i].setRate(new BigDecimal(tipAmount));
                setupCompensationItems[i].setUnits(0.0);
            }
            else if (CompensationType.Commission.equals(wageType)) {
            	setupCompensationItems[i].setRate(new BigDecimal(tipAmount));
                setupCompensationItems[i].setUnits(0.0);
            }
            else if (CompensationType.Piecework.equals(wageType)) {
                setupCompensationItems[i].setUnits(1000.0);
            }
            else if (CompensationType.Salary.equals(wageType)) {
            	// Do nothing
             }
            else {
                AssertJUnit.fail("Unexpected wage type: " + wageType.toString());
            }
        }
        return setupCompensationItems;
    }

 
    /**
     * Uses PaycheckLineItemLite to narrow down the fields it wants to compare.  Right now it's LawId and Amount.
     * 
     * @param computedItems
     * @param expectedItems
     * @param lineItemTypeStr
     * @return true if the collections have the same contents.
     */
    protected boolean verifyPaycheckLineItems(PaycheckLineItem [] computedItems, PaycheckLineItem [] expectedItems,
                                              String lineItemTypeStr)  {
        boolean compareResult = (computedItems.length == expectedItems.length);
        if (true == compareResult)
        {
            Set<PaycheckLineItemLite> computedLineItemSet = new HashSet<PaycheckLineItemLite>();
            for (PaycheckLineItem computedItem : computedItems) {
                computedLineItemSet.add(new PaycheckLineItemLite(computedItem));
            } 
            
            Set<PaycheckLineItemLite> expectedLineItemSet = new HashSet<PaycheckLineItemLite>();
            for(PaycheckLineItem expectedItem: expectedItems)
            {
                expectedLineItemSet.add(new PaycheckLineItemLite(expectedItem));
            }
            compareResult = computedLineItemSet.equals(expectedLineItemSet);
        }

        if (false == compareResult)
        {
            sLogger.error("Computed " + lineItemTypeStr);            
            for(int i=0; i < computedItems.length; i++)
            {
                sLogger.error(computedItems[i].toString());            
            }            
            sLogger.error("Expected " + lineItemTypeStr);            
            for(int i=0; i < expectedItems.length; i++)
            {
                sLogger.error(expectedItems[i].toString());            
            }            
        }
        
        return compareResult;
    }
    
    protected boolean verifyAmount(BigDecimal computedAmount, BigDecimal expectedAmount, String lineItemTypeStr)
    {
        boolean compareResult = (0 == computedAmount.compareTo(expectedAmount));
        
        if (false == compareResult)
        {
            sLogger.error("Computed " + lineItemTypeStr + ": " + computedAmount.toString());            
            sLogger.error("Expected " + lineItemTypeStr + ": " + expectedAmount.toString());            
        }
        
        return compareResult;
    }
    
    /**
     * Creates a PayrollRunPeriod based on the specified periodEndDate and frequency.  The payroll run date
     * is the day after the periodEndDate, and the paycheck date is 2 days after the payrollRunDate.
     * 
     * @param periodEndDate The pay period end date
     * @param frequency The pay frequency (e.g. Weekly, Biweekly, Semimonthly, Monthly)
     * @param payDayOffset The pay day (paycheck date) offset from the pay period end date
     * @param payrollRunOffset The payroll run day offset from the pay period end date
     * @return A PayrollRunPeriod with all the dates properly initialized
     */
    protected static PayrollRunPeriod createAPayrollRunPeriod(Date periodEndDate, PayrollFrequencyEnum frequency,
                                                       int payDayOffset, int payrollRunOffset)
    {
        PayrollRunPeriod payrollRunPeriod = new PayrollRunPeriod();
        payrollRunPeriod.setPeriodEndDate(periodEndDate);
        Date periodStartDate = calculatePeriodStartDate(periodEndDate, frequency);
        payrollRunPeriod.setPeriodStartDate(periodStartDate);
        payrollRunPeriod.setPayrollRunDate(addDays(periodEndDate, payrollRunOffset));
        payrollRunPeriod.setPayDate(addDays(periodEndDate, payDayOffset));
        payrollRunPeriod.setFrequency(mapPayrollFrequency(frequency));
        return payrollRunPeriod;
    }
    
    /**
     * Creates a weekly PayrollRunPeriod using specified pay period end date, payrollRun offset of 1 and
     *   payDay offset of 3
     * @param periodEndDate The pay period end date
     * @return A weekly PayrollRunPeriod
     */
    protected static PayrollRunPeriod createWeeklyPayrollRunPeriod(Date periodEndDate)
    {
        // The paycheck date is 3 days after the pay period end date, or 2 days after the payroll run date
        // The payroll run date is 1 day after the pay period end date
        return createAPayrollRunPeriod(periodEndDate, PayrollFrequencyEnum.WEEKLY, sDefaultPayrollDayOffset, sDefaultPayrollRunDayOffset);
    }
    
    /**
     * Creates a biweekly PayrollRunPeriod using specified pay period end date, payrollRun offset of 1 and
     *   payDay offset of 3
     * @param periodEndDate The pay period end date
     * @return A biweekly PayrollRunPeriod
     */
    protected static PayrollRunPeriod createBiweeklyPayrollRunPeriod(Date periodEndDate)
    {
        // The paycheck date is 3 days after the pay period end date, or 2 days after the payroll run date
        // The payroll run date is 1 day after the pay period end date
        return createAPayrollRunPeriod(periodEndDate, PayrollFrequencyEnum.BIWEEKLY, sDefaultPayrollDayOffset, sDefaultPayrollRunDayOffset);
    }
    
    /**
     * Creates a semimonthly PayrollRunPeriod using specified pay period end date, payrollRun offset of 1 and
     *   payDay offset of 3
     * @param periodEndDate The pay period end date
     * @return A semimonthly PayrollRunPeriod
     */
    protected static PayrollRunPeriod createSemimonthlyPayrollRunPeriod(Date periodEndDate)
    {
        // The paycheck date is 3 days after the pay period end date, or 2 days after the payroll run date
        // The payroll run date is 1 day after the pay period end date
        return createAPayrollRunPeriod(periodEndDate, PayrollFrequencyEnum.SEMIMONTHLY, sDefaultPayrollDayOffset, sDefaultPayrollRunDayOffset);
    }
    
    /**
     * Creates a monthly PayrollRunPeriod using specified pay period end date, payrollRun offset of 1 and
     *   payDay offset of 3
     * @param periodEndDate The pay period end date
     * @return A monthly PayrollRunPeriod
     */
    protected static PayrollRunPeriod createMonthlyPayrollRunPeriod(Date periodEndDate)
    {
        // The paycheck date is 3 days after the pay period end date, or 2 days after the payroll run date
        // The payroll run date is 1 day after the pay period end date
        return createAPayrollRunPeriod(periodEndDate, PayrollFrequencyEnum.MONTHLY, sDefaultPayrollDayOffset, sDefaultPayrollRunDayOffset);
    }
    
    protected static PayrollRunSaveRequest createPayrollRunSaveRequest(Long[] workerIds, Long payrollRunId,
    		PayrollRunTypeEnum payrollRunType, Date periodEndDate) throws Exception
    {
        PayrollRunSaveRequest saveRequest = new PayrollRunSaveRequest();
        PaycheckRequest[] paycheckRequests = new PaycheckRequest[workerIds.length];
        PayrollRunPeriod payrollRunPeriod = createBiweeklyPayrollRunPeriod(periodEndDate);

        for(int count = 0;count < workerIds.length;count++)
        {
            paycheckRequests[count] = new PaycheckRequest();
            CompensationItem[] workerCompensationItems = sWageService.getCompensationItems(workerIds[count]);
            CalcInputs[] inputList = createCalcInputsFromCompensationItems(workerCompensationItems);
			paycheckRequests[count].setCalcInputs(inputList);
            paycheckRequests[count].setWorkerId(workerIds[count]);
            Worker worker = sWorkerService.getWorkerById(workerIds[count]);
            paycheckRequests[count].setDistributionMethod(worker.getPaycheckDistributionMethod());
            paycheckRequests[count].setPayrollRunPeriod(payrollRunPeriod);
        }
        
        if (null != payrollRunId)
        {
            saveRequest.setPayrollRunId(payrollRunId);
        }
        
        saveRequest.setPayrollRunPeriod(payrollRunPeriod);
        saveRequest.setPaycheckRequests(paycheckRequests);
        saveRequest.setPayrollRunType(payrollRunType);
        
        return saveRequest;     
    }

    protected static PayrollRunSaveRequest createPayrollRunSaveRequest(Long[] workerIds, Long payrollRunId,
			   Date periodEndDate) throws Exception
	{
		return createPayrollRunSaveRequest(workerIds, payrollRunId, PayrollRunTypeEnum.SCHEDULED, periodEndDate);     
	}

    public static PayrollRunSaveRequest createPayrollRunSaveRequest(Long[] workerIds, Long payrollRunId)
    					throws Exception {
    	// Use a date of June 1, 2008 for the period end date
    	return createPayrollRunSaveRequest(workerIds, payrollRunId, (DateUtil.createDate(2008, Calendar.JUNE, 1)));
    }
    
    protected static PayrollRunCreateRequest createPayrollRunCreateRequest(Long[] workerIds,
    		PayrollRunTypeEnum payrollRunType, Date periodEndDate) throws Exception
    {
    	PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
        PayrollRunPeriod payrollRunPeriod = createBiweeklyPayrollRunPeriod(periodEndDate);
        createRequest.setPeriod(payrollRunPeriod);
        createRequest.setPayrollRunType(payrollRunType);
        
        return createRequest;     
    }

    protected static PayrollRunCreateRequest createPayrollRunCreateRequest(Long[] workerIds, Date periodEndDate)
    					throws Exception {
		return createPayrollRunCreateRequest(workerIds, PayrollRunTypeEnum.SCHEDULED, periodEndDate);     
	}

    public static PayrollRunCreateRequest createPayrollRunCreateRequest(Long[] workerIds)
    					throws Exception {
    	// Use a date of June 1, 2008 for the period end date
    	return createPayrollRunCreateRequest(workerIds, (DateUtil.createDate(2008, Calendar.JUNE, 1)));
    }
    
    protected static PayrollRunSaveRequest setUpPayrollRunSaveRequest(Long[] workerIds, Date payPeriodEndDate) throws Exception{
		PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
		//create a pay period
		PayrollRunPeriod period = createBiweeklyPayrollRunPeriod(payPeriodEndDate);
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(period.getPayrollRunDate());
		createRequest.setWorkerIds(workerIds);
		createRequest.setPeriod(period);
		createRequest.setPayrollRunType(PayrollRunTypeEnum.SCHEDULED);
		
		PayrollRunResponse payResponse = sWorkerPayService.createPayrollRun(createRequest);
		
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,
    			payResponse.getPayrollRunId(),payPeriodEndDate);
    	// Set the correct values for PayrollRunPeriod and PayrollRunType 
    	payrollRunSaveRequest.setPayrollRunPeriod(period);
    	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.SCHEDULED);    
    	
    	return payrollRunSaveRequest;
	}
    
	protected static CalcInputs[] createCalcInputsFromCompensationItems(CompensationItem[] workerCompensationItems) {
		List<CalcInputs> inputList = new ArrayList<CalcInputs>();
		for (CompensationItem compensationItem : workerCompensationItems) {
			CalcInputs calcInputs = new CalcInputs();
			calcInputs.setUnits(compensationItem.getUnits());
			calcInputs.setRate(compensationItem.getRate());
			calcInputs.setPayItemObjectHandle(compensationItem.getObjectHandle());
			inputList.add(calcInputs);
		}
		return inputList.toArray(new CalcInputs[inputList.size()]);
	}
    
    private void validatePaycheckDetail(PaycheckDetails pcDetails)
    {
            AssertJUnit.assertNotNull("PayCheck detail is null",pcDetails);
            AssertJUnit.assertNotNull("Paycheck Id is null", pcDetails.getPaycheckId());
            AssertJUnit.assertTrue("PaycheckDetails should have a gross total with some valid values",
                              pcDetails.getGrossTotal().doubleValue() > 0);
            AssertJUnit.assertTrue("PaycheckDetails should have a net with some valid values", pcDetails.getNetTotal().abs().doubleValue() > 0.0);
            AssertJUnit.assertNotNull("Paycheck distribution is null",pcDetails.getDistributionMethod());
    }
    
    /**
     * This utility method will verify the 2 PayrollRunSummary objects for equivalence
     * @param expected The PayrollRunSummary object to verify against
     * @param computed The PayrollRunSummary object to verify
     * @return true if the 2 payrollRunSummary are equal
     */
	protected boolean verifyPayrollRunSummary(PayrollRunSummary expected, PayrollRunSummary computed) {
		boolean result = true;
		result = result && expected.getPayrollRunId().equals(computed.getPayrollRunId());
		result = result && expected.getPayrollRunState().equals(computed.getPayrollRunState());
		result = result && expected.getTotalEmployeeNet().equals(computed.getTotalEmployeeNet());
		result = result && expected.getTotalEmployeeGross().equals(computed.getTotalEmployeeGross());
		result = result && expected.getTotalEmployeePaidTax().equals(computed.getTotalEmployeePaidTax());
		result = result && expected.getTotalEmployerPaidTax().equals(computed.getTotalEmployerPaidTax());
		result = result && expected.getTotalEmployeePaidBenefits().equals(computed.getTotalEmployeePaidBenefits());
		result = result && expected.getTotalEmployerPaidBenefits().equals(computed.getTotalEmployerPaidBenefits());
		Set<PaycheckSummary> computedSet = new HashSet<PaycheckSummary>();
		for (PaycheckSummary paycheckSummary : computed.getPaycheckSummaries()) {
			computedSet.add(paycheckSummary);
		}
		Set<PaycheckSummary> expectedSet = new HashSet<PaycheckSummary>();
		for (PaycheckSummary paycheckSummary : expected.getPaycheckSummaries()) {
			expectedSet.add(paycheckSummary);
		}
		result = result && expectedSet.equals(computedSet);
		if (!result) {
			sLogger.error("Expected PayrollRunSummary:");
			sLogger.error(expected.toString());
			sLogger.error("Computed PayrollRunSummary:");
			sLogger.error(computed.toString());
		}
		return result;
	}
	
	/**
	 * This utility will create and initialize a PaycheckSummary object using the passed in parameters
	 * @param workerId The id of the worker
	 * @param hours The total hours worked for the pay period
	 * @param grossPay The gross pay on the paycheck
	 * @param netPay The net pay on the paycheck
	 * @param employeePaidTaxes The total employee paid taxes on the paycheck
	 * @param employeePaidBenefits The total employee paid benefits on the paycheck
	 * @param distributionMethod The distribution method of the paycheck
	 * @param paycheckStatus The status of the paycheck
	 * @param payrollRunSummary A PayrollRunSummary of a payroll run
	 * @return A PaycheckSummary object initialized with the specified values
	 * @throws Exception
	 */
	protected PaycheckSummary createPaycheckSummary(Long workerId, String hours, BigDecimal grossPay, BigDecimal netPay,
					BigDecimal employeePaidTaxes, BigDecimal employeePaidBenefits,
					PaycheckDistributionMethod distributionMethod, PaycheckStatus paycheckStatus,
					PayrollRunSummary payrollRunSummary) throws Exception {
		PaycheckSummary paycheckSummary = new PaycheckSummary();
		for (PaycheckSummary aPaycheckSummary : payrollRunSummary.getPaycheckSummaries()) {
			if (aPaycheckSummary.getWorkerId().equals(workerId)) {
				paycheckSummary.setPaycheckId(aPaycheckSummary.getPaycheckId());
				paycheckSummary.setCheckNumber(aPaycheckSummary.getCheckNumber());
				break;
			}
		}
		paycheckSummary.setWorkerId(workerId);
		Worker worker = sWorkerService.getWorkerById(workerId);
		paycheckSummary.setFirstName(worker.getFirstName());
		paycheckSummary.setLastName(worker.getLastName());
		paycheckSummary.setWorkerType(worker.getWorkerType());
		paycheckSummary.setHours(hours);
		paycheckSummary.setGrossPay(grossPay);
		paycheckSummary.setNetPay(netPay);
		paycheckSummary.setEmployeePaidTaxes(employeePaidTaxes);
		paycheckSummary.setEmployeePaidBenefits(employeePaidBenefits);
		paycheckSummary.setDistributionMethod(distributionMethod);
		paycheckSummary.setCheckNumber(null);
		paycheckSummary.setPaycheckStatus(paycheckStatus);
		return paycheckSummary;
	}
	
    protected void validatePayrollRunSummary(PayrollRunSaveRequest saveRequest, PayrollRunSummary runSummary,
                                             boolean isCommit)
    {
        AssertJUnit.assertNotNull(runSummary);
        AssertJUnit.assertNotNull(runSummary.getPayrollRunId());
        validatePaycheckSummaryArray(saveRequest.getPaycheckRequests(), runSummary.getPaycheckSummaries());
        validatePayrollRunPeriod(saveRequest.getPayrollRunPeriod(), runSummary.getPayrollPeriod());
        validatePayrollRunState(isCommit, runSummary.getPayrollRunState());
        AssertJUnit.assertTrue("Gross has value zero", runSummary.getTotalEmployeeGross().doubleValue() > 0.0);
        AssertJUnit.assertTrue("Total Employee Paid Taxes has value zero", runSummary.getTotalEmployeePaidTax().doubleValue() > 0.0);
        AssertJUnit.assertTrue("Total Employer Paid Taxes has value zero", runSummary.getTotalEmployerPaidTax().doubleValue() > 0.0);
    }

    protected void validatePaycheckNumber(PayrollRunSaveRequest saveRequest, PayrollRunSummary runSummary) throws ValidationException
    {
    	PaycheckRequest[] workerWageInfo = saveRequest.getPaycheckRequests();
        
        PayrollRun payrollRun = entityService.findById(PayrollRun.class, runSummary.getPayrollRunId());
        
        AssertJUnit.assertEquals("Number of paychecks are not equal", workerWageInfo.length,payrollRun.getPaychecks().size());
    }

    protected void validatePayrollRunResponse(PayrollRunSaveRequest saveRequest, PayrollRunResponse response,
                                              boolean isCommit)
    {
        AssertJUnit.assertNotNull("PayrollRunResponse can not be null", response);
        AssertJUnit.assertNotNull("Response should have a valid PayrollRunId ", response.getPayrollRunId());
        AssertJUnit.assertTrue("Response should have a gross total with some valid values",
                          response.getPaycheckDetails().length > 0);
        AssertJUnit.assertTrue("Response should have a gross total with some valid values", response.getPaycheckDetails()[0]
                        .getGrossTotal().doubleValue() > 0.0);

        // Validate
        validateResponsePaycheckDetails(saveRequest.getPaycheckRequests(), response.getPaycheckDetails());
        validatePayrollRunDates(saveRequest.getPayrollRunPeriod(), response.getPayrollRunPeriod());
        validatePayrollRunState(isCommit, response.getPayrollRunState());
    }

    protected void validatePayCheckDetails(PaycheckRequest paycheckRequest, PaycheckDetails payCheckDetails)
    {
        validatePaycheckDetail(payCheckDetails);
    }
    
    private void validateResponsePaycheckDetails(PaycheckRequest[] paycheckRequests, PaycheckDetails[] paycheckDetails)
    {
        for (PaycheckDetails pcDetails : paycheckDetails)
        {
            validatePaycheckDetail(pcDetails);
        }

        for (PaycheckRequest workerWage : paycheckRequests)
        {
            Long workerId = workerWage.getWorkerId();
            List<PaycheckDetails> paycheckDetailList = getPaycheckDetailForWorker(paycheckDetails, workerId);

            AssertJUnit.assertEquals(paycheckDetailList.size(),1);
            PaycheckDetails pcDetail = paycheckDetailList.get(0);
            //FIXME this is not really relevant - need to have a gross amount expected up front, ---validateGrossAmount(workerWage, pcDetail.getGrossTotal());
            validateNetPay(workerWage,pcDetail);
            AssertJUnit.assertEquals("Paycheck distribution method does not match", workerWage.getDistributionMethod(), pcDetail.getDistributionMethod());

        }
    }

    private void validateNetPay(PaycheckRequest paycheckRequest, PaycheckDetails pcDetail)
    {
        PaycheckLineItem[] pcLineItem = pcDetail.getPaycheckLineItemsList();

        BigDecimal totalTax = new BigDecimal(0.0);
        List<PaycheckLineItem> wageItemsList = new ArrayList<PaycheckLineItem>();

        for (PaycheckLineItem item : pcLineItem)
        {
            AssertJUnit.assertNotNull(item.getAmount());
            AssertJUnit.assertNotNull(item.getLawId());
            AssertJUnit.assertNotNull(item.getTitle());
            if ((item.getPayrollItemCategory() != PayrollItemCategoryType.COMPENSATION) && 
            	(item.getPayrollItemCategory() != PayrollItemCategoryType.EMPLOYEE_PAID_TAX))
            {
              totalTax = totalTax.add(item.getAmount());
            }

            if (item.getPayrollItemCategory() != PayrollItemCategoryType.COMPENSATION) 
            {
                wageItemsList.add(item);
            }
        }

        // NetPay + totalTax == Gross Amount

        BigDecimal calcGross =  totalTax.add(pcDetail.getNetTotal());
        AssertJUnit.assertEquals("Pcline items does not validate the net & gross Pay", calcGross.compareTo(pcDetail.getGrossTotal()), 0);
    }

    //FIXME this could be used to calculate an expected gross amount, as long as everything is a compensation item.
//    private BigDecimal calculateExpectedGrossAmount(CompensationItem[] wageItems)
//    {
//        BigDecimal wageGross = new BigDecimal(0);
//        for (int i = 0; i < wageItems.length; i++)
//        {
//            CompensationItem wage = wageItems[i];
//            CompensationType wageType = wage.getCompensationType();
//            BigDecimal rate = wage.getRate();
//            double units = wage.getUnits();
//            if (CompensationType.Bonus.equals(wageType))
//            {
//                wageGross = wageGross.add(rate);
//            }
//            else if (CompensationType.Commission.equals(wageType))
//            {
//                wageGross = wageGross.add(rate);
//            }
//            else if (CompensationType.Doubletime.equals(wageType))
//            {
//                wageGross = wageGross.add(rate.multiply(new BigDecimal(units)));
//            }
//            else if (CompensationType.Hourly.equals(wageType))
//            {
//                wageGross = wageGross.add(rate.multiply(new BigDecimal(units)));
//            }
//            else if (CompensationType.Overtime.equals(wageType))
//            {
//                wageGross = wageGross.add(rate.multiply(new BigDecimal(units)));
//            }
//            else if (CompensationType.Piecework.equals(wageType))
//            {
//                wageGross = wageGross.add(rate.multiply(new BigDecimal(units)));
//            }
//            else if (CompensationType.Salary.equals(wageType))
//            {
//                wageGross = wageGross.add(rate.multiply(new BigDecimal(units)));
//            }
//            else if (CompensationType.TipsAllocated.equals(wageType))
//            {
//                wageGross = wageGross.add(rate);
//            }
//            else if (CompensationType.TipsCash.equals(wageType))
//            {
//                wageGross = wageGross.add(rate);
//            }
//            else if (CompensationType.TipsPaycheck.equals(wageType))
//            {
//                wageGross = wageGross.add(rate);
//            }
//        }
//        return wageGross;
//    }

    private List<PaycheckDetails> getPaycheckDetailForWorker(PaycheckDetails[] paycheckDetails, Long workerId)
    {
        List<PaycheckDetails> pcDetailsList = new ArrayList<PaycheckDetails>();
        for (PaycheckDetails paychkdetail : paycheckDetails)
        {
            AssertJUnit.assertNotNull(paychkdetail.getWorkerId());
            if (paychkdetail.getWorkerId().equals(workerId))
            {
                pcDetailsList.add(paychkdetail);
            }
        }
        return pcDetailsList;
    }

    protected void validatePayrollRunDates(PayrollRunPeriod expectedPayrollRunPeriod, PayrollRunPeriod payrollPeriod)
    {
        validateDate("PayrollRunPeriod PayDate is not same", expectedPayrollRunPeriod.getPayDate(), payrollPeriod
                        .getPayDate());
        validateDate("PayrollRunPeriod PayrollRunDate is not same", expectedPayrollRunPeriod.getPayrollRunDate(),
                     payrollPeriod.getPayrollRunDate());
        validateDate("PayrollRunPeriod PeriodEndDate is not same", expectedPayrollRunPeriod.getPeriodEndDate(),
                     payrollPeriod.getPeriodEndDate());
        validateDate("PayrollRunPeriod Start Date is not same", expectedPayrollRunPeriod.getPeriodStartDate(),
                     payrollPeriod.getPeriodStartDate());
    }

    private void validateDate(String message, Date expectedDate, Date actualDate)
    {
        Calendar expectedCal = DateUtil.getNowCalendar();
        expectedCal.setTime(expectedDate);
        Calendar actualCal = DateUtil.getNowCalendar();
        actualCal.setTime(actualDate);
        AssertJUnit.assertEquals(message, true, DateUtil.isSameDay(expectedCal, actualCal));
    }

    private void validatePaycheckSummaryArray(PaycheckRequest[] paycheckRequests, PaycheckSummary[] payCheckDetails)
    {
        AssertJUnit.assertEquals("Number of paychecks should be same as number of workers", paycheckRequests.length,
                            payCheckDetails.length);

        for (int i = 0; i < paycheckRequests.length; i++)
        {
        	PaycheckRequest paycheckRequest = paycheckRequests[i];
            Long workerId = paycheckRequest.getWorkerId();
            PaycheckSummary paycheckSummary = getPaycheckSummaryForWorker(payCheckDetails, workerId);
            AssertJUnit.assertNotNull("No Paycheck summary found", paycheckSummary);
            validatePaycheckSummary(paycheckRequest, paycheckSummary);
        }
    }

    private void validatePaycheckSummary(PaycheckRequest paycheckRequest, PaycheckSummary paycheckSummary)
    {
        AssertJUnit.assertEquals("Worker id does not match", paycheckRequest.getWorkerId(), paycheckSummary.getWorkerId());
        // Load worker
        Worker worker = null;

        try {
            worker = sWorkerService.getWorkerById(paycheckSummary.getWorkerId());
        } catch (Exception e) {
            e.printStackTrace();
            AssertJUnit.assertTrue("Not able to find Worker", false);
            return;
        }

        AssertJUnit.assertNotNull("Paycheck Id is null", paycheckSummary.getPaycheckId());

        AssertJUnit.assertEquals("Workers first Name does not match", worker.getFirstName(), paycheckSummary.getFirstName());
        AssertJUnit.assertEquals("Workers last name does not match", worker.getLastName(), paycheckSummary.getLastName());
        AssertJUnit.assertEquals("Workers WorkerType does not match", worker.getWorkerType(), paycheckSummary
                        .getWorkerType());
        AssertJUnit.assertEquals("Worker's Paycheck distribution method does not match", paycheckRequest
                        .getDistributionMethod(), paycheckSummary.getDistributionMethod());

        //FIXME this gross amount needs to exist up front ----validateGrossAmount(paycheckRequest, paycheckSummary.getGrossPay());
        //FIXME there are a few more attributes to verify.
        AssertJUnit.assertEquals(paycheckSummary.getNetPay().compareTo(new BigDecimal(-1.0)), 1);
    }
    
    private void validatePayrollRunState(boolean isCommit, PayrollRunState payrollRunState)
    {
        if (isCommit)
        {
            AssertJUnit.assertEquals(payrollRunState, PayrollRunState.COMPLETE);
        }
        else
        {
            AssertJUnit.assertEquals(payrollRunState, PayrollRunState.DRAFT);
        }
    }
    
    protected void validatePayrollRunPeriod(PayrollRunPeriod expectedPayrollRunPeriod, PayrollRunPeriod payrollPeriod)
    {
        AssertJUnit.assertEquals("PayrollRunPeriod frequency is not same", expectedPayrollRunPeriod.getFrequency(),
                                payrollPeriod.getFrequency());

        validateDate("PayrollRunPeriod PayDate is not same", expectedPayrollRunPeriod.getPayDate(), payrollPeriod
                        .getPayDate());

        validateDate("PayrollRunPeriod PayrollRunDate is not same", expectedPayrollRunPeriod.getPayrollRunDate(),
                     payrollPeriod.getPayrollRunDate());

        validateDate("PayrollRunPeriod PeriodEndDate is not same", expectedPayrollRunPeriod.getPeriodEndDate(),
                     payrollPeriod.getPeriodEndDate());

        validateDate("PayrollRunPeriod Start Date is not same", expectedPayrollRunPeriod.getPeriodStartDate(),
                     payrollPeriod.getPeriodStartDate());

    }

    protected PaycheckSummary getPaycheckSummaryForWorker(PaycheckSummary[] payCheckDetails, Long workerId)
    {
        for (PaycheckSummary pcSummary : payCheckDetails)
        {
            if (pcSummary.getWorkerId().equals(workerId))
            {
                return pcSummary;
            }

        }
        return null;
    }
    
    /**
     * This utility method will create a payroll-ready worker with the given first name with a WorkLocation in
     * 		Mountain View.  This worker has the following attributes:
     *         Withholding information - Single 1 for both W4 and DE4.
     *         PaycheckDistributionMethod - HandWritten
     *         Compensation items - Hourly, Overtime, PaycheckTips
     * @param name The worker's first name
     * @return The id of the worker that was created
     * @throws Exception
     */
    protected static Long createMVPayrollReadyWorker(String name) throws Exception {
		PhysicalAddress taxResidence = PojoUtils.createUSCAPhysicalAddress();
		Date birthday  = DateUtil.createDate(1970, 1, 1);
		Date hireDate = DateUtil.createDate(2008, 1, 1);
        Worker worker =
            new Worker(null, name, "", "Ready", "A worker", hireDate, birthday, WorkerType.EMPLOYEE, taxResidence,
                       getSpecifiedWorkLocation(sMountainViewLocation),PaycheckDistributionMethod.HANDWRITTEN, null, "Avatar.gif");

        Worker savedWorker = sWorkerService.saveWorker(worker);

        WithholdingInformation w4 = EntityTestBaseHelper.createEIWithholdingInformation(savedWorker, FormType.US_W4);
        w4.setAllowances(1);
        w4.setDeductions(0);
        w4.setExempt(false);
        w4.setExtraWithholding(new BigDecimal(0));
        w4.setWithholdingStatus(WithholdingStatus.SINGLE);
        final String workerTaxIdNumber = "123-45-6789";
        SaveWithholdingInfoRequest w4Request = new SaveWithholdingInfoRequest();
        w4Request.setWithholdingInfo(w4);
        w4Request.setWorkerId(savedWorker.getObjectHandle().getId());
        w4Request.setWorkerTaxIdNumber(workerTaxIdNumber);
        w4Request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
		sTaxWithholdingsService.saveWithholdingInformation(w4Request);
        
        WithholdingInformation de4 = EntityTestBaseHelper.createEIWithholdingInformation(savedWorker, FormType.CA_DE4);
        de4.setAllowances(1);
        de4.setDeductions(0);
        de4.setExempt(false);
        de4.setExtraWithholding(new BigDecimal(0));
        de4.setWithholdingStatus(WithholdingStatus.SINGLE);
        SaveWithholdingInfoRequest de4request = new SaveWithholdingInfoRequest();
        de4request.setWithholdingInfo(de4);
        de4request.setWorkerId(savedWorker.getObjectHandle().getId());
        de4request.setWorkerTaxIdNumber(workerTaxIdNumber);
        //Add
        de4request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
        sTaxWithholdingsService.saveWithholdingInformation(de4request);
        
        CompensationItem[] wageItems = new CompensationItem[3];
        double units = 0;	// Default units - we currently don't collect them in the app.
        CompensationItem tips = new CompensationItem(null, new BigDecimal(0), units, CompensationType.TipsPaycheck, sPaycheckTipsWageDescStr,true);
        CompensationItem hourly = new CompensationItem(null, new BigDecimal(40), units, CompensationType.Hourly, sHourlyWageDescStr,true);
        CompensationItem overtime = new CompensationItem(null, new BigDecimal(1.5), units, CompensationType.Overtime, sOvertimeWageDescStr, hourly,true);

        wageItems[0] = overtime;
        wageItems[1] = hourly;
        wageItems[2] = tips;

        sWageService.saveCompensationItems(savedWorker.getObjectHandle().getId(),
                         CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD, wageItems));
        
        return savedWorker.getObjectHandle().getId();
    }

    /**
     * This utility method will return a WorkLocation object with the specified name
     * @param name The name of the WorkLocation to retrieve
     * @return A WorkLocation object corresponding to the given name
     * @throws Exception
     */
    protected static WorkLocation getSpecifiedWorkLocation(String name) throws Exception
    {
        WorkLocation caWorkLocation = null;
        WorkLocation[] workLocations = sEmployerService.getWorkLocations();
        for (WorkLocation workLocation : workLocations)
        {
            if (workLocation.getName().contains(name))
            {
                caWorkLocation = workLocation;
                break;
            }
        }
        return caWorkLocation;
    }

    /**
     * This utility method will delete all PayrollRuns from the database
     * @throws ServiceFailureException 
     * @throws ValidationException 
     */
    protected static void deleteAllPayrollRun() throws ValidationException, ServiceFailureException
    {
        List<PayrollRun>  payrollRunList = entityService.findAll(PayrollRun.class);
        for (PayrollRun payrollRun : payrollRunList)
        {
            cleanUpPayrollRunItems(payrollRun.getId());
        }

    }
    
    /**
     * This utility method will delete the given PayrollRun from the database
     * @param payrollRunId The id of the PayrollRun to delete
     * @throws ServiceFailureException 
     * @throws ValidationException 
     */
    protected static void cleanUpPayrollRunItems(Long payrollRunId) throws ValidationException, ServiceFailureException
    {
    	workerPaymentsComponent.deletePayrollRun(payrollRunId);
    }
	
    /**
     * This utility method will map a PayrollRunSummary object into a PayrollRunInfo object
     * 
     * @param payrollRunSummary The PayrollRunSummary object to map the PayrollRunInfo from
     * @return A PayrollRunInfo object initialized with values from the PayrollRunSummary object
     * @throws Exception
     */
	protected PayrollRunInfo mapToPayrollRunInfo(PayrollRunSummary payrollRunSummary) throws Exception {
		PayrollRunInfo payrollRunInfo = new PayrollRunInfo();
		payrollRunInfo.setPayrollRunId(payrollRunSummary.getPayrollRunId());
		payrollRunInfo.setPayrollRunPeriod(payrollRunSummary.getPayrollPeriod());
		payrollRunInfo.setPayrollRunState(payrollRunSummary.getPayrollRunState());
		return payrollRunInfo;
	}
	
    /**
     * This utility method will map a PayrollRunSummary object into a PayrollRunResponse object
     * 
     * @param payrollRunSummary The PayrollRunSummary object to map the PayrollRunResponse from
     * @return A PayrollRunResponse object initialized with values from the PayrollRunSummary object
     * @throws Exception
     */
	protected PayrollRunResponse mapToPayrollRunResponse(PayrollRunSummary payrollRunSummary) throws Exception {
		PayrollRunResponse payrollRunResponse = new PayrollRunResponse();
		payrollRunResponse.setPayrollRunId(payrollRunSummary.getPayrollRunId());
		payrollRunResponse.setPayrollRunPeriod(payrollRunSummary.getPayrollPeriod());
		payrollRunResponse.setPayrollRunState(payrollRunSummary.getPayrollRunState());
		PaycheckSummary[] paycheckSummaries = payrollRunSummary.getPaycheckSummaries();
		PaycheckDetails[] paycheckDetails = new PaycheckDetails[paycheckSummaries.length];
		int i = 0;
		for (PaycheckSummary paycheckSummary : paycheckSummaries) {
			Long paycheckId = paycheckSummary.getPaycheckId();
			paycheckDetails[i++] = sWorkerPayService.getPaycheckById(paycheckId);
		}
		payrollRunResponse.setPaycheckDetails(paycheckDetails);
		return payrollRunResponse;
	}
	
	/**
	 * This utility method will compare 2 PayrollRunResponse object for equivalence
	 * @param expected The PayrollRunResponse object to compare against
	 * @param computed The PayrollRunResponse object to perform the comparison on
	 * @return true if the 2 PayrollRunResponse objects are equal
	 */
	protected boolean verifyPayrollRunResponse(PayrollRunResponse expected, PayrollRunResponse computed) {
		boolean result = true;
		result = result && expected.getPayrollRunId().equals(computed.getPayrollRunId());
		result = result && expected.getPayrollRunPeriod().equals(computed.getPayrollRunPeriod());
		result = result && expected.getPayrollRunState().equals(computed.getPayrollRunState());
		PaycheckDetails[] expectedPaycheckDetails = expected.getPaycheckDetails();
		PaycheckDetails[] computedPaycheckDetails = computed.getPaycheckDetails();
		for (PaycheckDetails expectedPaycheckDetail : expectedPaycheckDetails) {
			boolean found = false;
			for (PaycheckDetails computedPaycheckDetail : computedPaycheckDetails) {
				if (expectedPaycheckDetail.getPaycheckId().equals(computedPaycheckDetail.getPaycheckId())) {
					found = true;
					result = result && verifyPaycheckDetails(expectedPaycheckDetail, computedPaycheckDetail);
					break;
				}
			}
			result = result && found;
			if (!result) {
				break;
			}
		}
		return result;
	}
	
	/**
	 * This utility method will compare 2 PayrollRunResponse[] for equivalence
	 * @param expected The PayrollRunResponse[] to compare against
	 * @param computed The PayrollRunResponse[] to perform the comparison on
	 * @return true if the 2 PayrollRunResponse[] are equal
	 */
	protected boolean verifyPayrollRunResponses(PayrollRunResponse[] expected, PayrollRunResponse[] computed) {
		boolean result = true;
		for (PayrollRunResponse expectedPayrollRunResponse : expected) {
			boolean found = false;
			for (PayrollRunResponse computedPayrollRunResponse : computed) {
				if (expectedPayrollRunResponse.getPayrollRunId().equals(computedPayrollRunResponse.getPayrollRunId())) {
					found = true;
					result = verifyPayrollRunResponse(expectedPayrollRunResponse, computedPayrollRunResponse);
					break;
				}
			}
			result = result && found;
			if (!result) {
				break;
			}
		}
		return result;
	}
	
	/**
	 * This utility method will compare 2 PayrollRunInfo object for equivalence
	 * @param expected The PayrollRunInfo object to compare against
	 * @param computed The PayrollRunInfo object to perform the comparison on
	 * @return true if the 2 PayrollRunInfo objects are equal
	 */
	protected boolean verifyPayrollRunInfo(PayrollRunInfo expected, PayrollRunInfo computed) {
		boolean result = true;
		if (null != expected.getPayrollRunId()) {
			result = result && expected.getPayrollRunId().equals(computed.getPayrollRunId());
		} else {
			result = result && (null == computed.getPayrollRunId());
		}
		if (null != expected.getPayrollRunPeriod()) {
			result = result && expected.getPayrollRunPeriod().equals(computed.getPayrollRunPeriod());
		} else {
			result = result && (null == computed.getPayrollRunPeriod());
		}
		if (null != expected.getPayrollRunState()) {
			result = result && expected.getPayrollRunState().equals(computed.getPayrollRunState());
		} else {
			result = result && (null == computed.getPayrollRunState());
		}
		if (false == result) {
			sLogger.error("Expected:");
			sLogger.error(expected.toString());
			sLogger.error("Computed:");
			sLogger.error(computed.toString());
		}
		return result;
	}
	
	/**
	 * This utility method will compare 2 PayrollRunInfo[] for equivalence
	 * @param expected The PayrollRunInfo[] to compare against
	 * @param computed The PayrollRunInfo[] to perform the comparison on
	 * @return true if the 2 PayrollRunInfo[] are equal
	 */
	protected boolean verifyPayrollRunInfos(PayrollRunInfo[] expected, PayrollRunInfo[] computed) {
		boolean result = true;
		for (PayrollRunInfo expectedPayrollRunInfo : expected) {
			boolean found = false;
			for (PayrollRunInfo computedPayrollRunInfo : computed) {
				if (expectedPayrollRunInfo.getPayrollRunId().equals(computedPayrollRunInfo.getPayrollRunId())) {
					found = true;
					result = verifyPayrollRunInfo(expectedPayrollRunInfo, computedPayrollRunInfo);
					break;
				}
			}
			result = result && found;
			if (!result) {
				break;
			}
		}
		return result;
	}
	
    /**
     * This utility method will calculate the overtime rate given an hourly rate and multiplier
     * @param hourlyRate The hourly rate
     * @param overtimeRate The overtime multiplier
     * @return A BigDecimal object containing the overtime rate
     */
	protected BigDecimal getCalculatedOvertimeRate(String hourlyRate, String overtimeMultiplier) {
		BigDecimal calculatedRate = new BigDecimal(hourlyRate);
		calculatedRate = calculatedRate.multiply(new BigDecimal(overtimeMultiplier));
		//FIXME Should we do rounding at all here?  QBO/IOP allows up to 5 digits for rates
		// Set to 2 decimal places
		calculatedRate = calculatedRate.setScale(2, RoundingMode.HALF_UP);
		return calculatedRate;
	}

	/**
	 * This utility method will verify 2 PaycheckDetails object to see if they are equal
	 * 
	 * @param expected The PaycheckDetails object to verify against
	 * @param computed The PaycheckDetails object to verify
	 * @return true if equal, false otherwise
	 */
	protected boolean verifyPaycheckDetails(PaycheckDetails expected, PaycheckDetails computed) {
		boolean result = true;
		if (null == expected.getCheckNumber()) {
			result = (null == computed.getCheckNumber());
		} else {
			result = result && (expected.getCheckNumber().equals(computed.getCheckNumber()));
		}
		result = result && (expected.getDistributionMethod().equals(computed.getDistributionMethod()));
		result = result && (expected.getGrossTotal().equals(computed.getGrossTotal()));
		result = result && (expected.getNetTotal().equals(computed.getNetTotal()));
		result = result && (expected.getPaycheckId().equals(computed.getPaycheckId()));
		result = result && (expected.getWorkerId().equals(computed.getWorkerId()));
		result = result && (expected.getPaycheckStatus().equals(computed.getPaycheckStatus()));
        Set<PaycheckLineItem> computedLineItemSet = new HashSet<PaycheckLineItem>();
        for (PaycheckLineItem computedItem : computed.getPaycheckLineItemsList()) {
            computedLineItemSet.add(computedItem);
        } 
        
        Set<PaycheckLineItem> expectedLineItemSet = new HashSet<PaycheckLineItem>();
        for(PaycheckLineItem expectedItem: expected.getPaycheckLineItemsList())
        {
            expectedLineItemSet.add(expectedItem);
        }
        result = result && computedLineItemSet.equals(expectedLineItemSet);
		if (false == result) {
            sLogger.error("Computed PaycheckDetails:");            
            sLogger.error(computed.toString());            
            sLogger.error("Expected PaycheckDetails:");            
            sLogger.error(expected.toString());            
		}
		return result;
	}
	
	protected void fixupSourcePayrollItemDetailId(PaycheckLineItem[] items, com.intuit.sb.payroll.entity.model.Worker worker) {
		Set<PayrollItemDtl> details = worker.getPayrollItemDtls();
		
		
		for (PaycheckLineItem item : items)
		{
			boolean found = false;
			for (PayrollItemDtl detail : details)
			{
				if(item.getLawId().equals(detail.getPayrollItem().getLawId().getValue()))
				{
					item.setPayrollItemId(detail.getId());
					found = true;
					break;
				}
			}
			AssertJUnit.assertTrue("Can't find payroll item detail - lawId: " + item.getLawId(), found);
		}	
	}
	
	protected CalcInputs[] removeInputFromArray(long id, CalcInputs[] calcInputs) {
		List<CalcInputs> inputList = new ArrayList<CalcInputs>();
		for (CalcInputs calcInput : calcInputs) {
			if (calcInput.getPayItemObjectHandle().getId() != id) {
				inputList.add(calcInput);
			}
		}
		
		return inputList.toArray(new CalcInputs[inputList.size()]);
	}
}
