package com.intuit.sb.payroll.entity.deductions.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.EntityResources;
import com.intuit.sb.payroll.entity.NamedQueryCatalog;
import com.intuit.sb.payroll.entity.deductions.DeductionEAO;
import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.Deduction;
import com.intuit.sb.payroll.entity.model.DeductionTypeEnum;

@Stateless(name="DeductionEAO")
@TransactionManagement(TransactionManagementType.CONTAINER) // Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY) 
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
@SuppressWarnings("nls")
public final class DeductionEjbEAO extends GenericEjbEAO<Deduction>
	implements DeductionEAO {

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.deductions.DeductionEAO#findDeduction(com.intuit.sb.payroll.entity.model.DeductionTypeEnum)
	 */
	public Deduction findDeduction(DeductionTypeEnum deductionType) {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put(NamedQueryCatalog.DEDUCTION_TYPE_QUERY_PARAM, deductionType);
        
        List<Deduction> results = null;
		try	{
			// named query is in orm.xml
			results = findByNamedQuery(NamedQueryCatalog.FIND_DEDUCTION_BY_TYPE, queryParameters);
		}
		catch(RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		switch (results.size()) {
		case 0:
			String mesg1 = MessageFormat.format(EntityResources.getString("deduction.does.not.exist"), deductionType.toString());
			logger.error(mesg1);
			throw new IllegalStateException(mesg1);
		case 1:
			return results.get(0);
		default:
			String mesg2 = MessageFormat.format(EntityResources.getString("deduction.multiple.for.type"), deductionType.toString());
			logger.error(mesg2);
			throw new IllegalStateException(mesg2);
		}
	}
}
