package com.intuit.sb.payroll.entity.taxes.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.EntityResources;
import com.intuit.sb.payroll.entity.NamedQueryCatalog;
import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.Agency;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.taxes.AgencyEAO;

@Stateless(name = "AgencyEAO")
@TransactionManagement(TransactionManagementType.CONTAINER)
// Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY)
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
@SuppressWarnings("nls")
public class AgencyEjbEAO extends GenericEjbEAO<Agency> implements AgencyEAO {
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.taxes.AgencyEAO#findAgencyByKey(com.intuit.sb.payroll.entity.model.AgencyEnum)
	 */
	public Agency findAgencyByKey(AgencyEnum agencyKey) {
		Map<String, Object> queryParameters = new HashMap<String, Object>();
		queryParameters.put(NamedQueryCatalog.AGENCY_KEY_QUERY_PARAM, agencyKey);
		List<Agency> agencyList = findByNamedQuery(NamedQueryCatalog.FIND_AGENCY_BY_KEY, queryParameters);

		if (agencyList == null) {
			return null;
		}

		switch (agencyList.size()) {
			case 0:
				return null;
			case 1:
				return agencyList.get(0);
			default:
				String mesg = MessageFormat.format(EntityResources.getString("agency.multiple.for.key"), agencyKey.toString());
				throw new IllegalStateException(mesg);
		}
	}
}
