package com.intuit.sb.payroll.entity.taxes.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.taxes.TaxEAO;

/**
 * The EJB3 implementation of the {@link TaxEAO TaxEAO} interface.
 * As required by the superclass, this concrete EAO implementation
 * is an EJB3 SSB. It publishes itself in JNDI under its interface,
 * not implementation, name.
 * 
 * @see Tax, TaxEAO, GenericEAO
 *
 * $Author: oblackburn $
 * $Revision: #2 $
 * $DateTime: 2008/08/19 11:31:02 $
 *
 */
@Stateless(name = "TaxEAO")
@TransactionManagement(TransactionManagementType.CONTAINER) // Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY) 
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public final class TaxEjbEAO extends GenericEjbEAO<Tax> implements TaxEAO {

    /**
     * EJB required no-argument constructor.
     */
    public TaxEjbEAO() {
        super();
    }
    
    // ********************** Business Queries ********************** //

    /* (non-Javadoc)
     * @see com.intuit.sb.payroll.component.taxes.TaxEAO#findTaxTypeByLawId(java.lang.String)
     */
    public Tax findTaxByLawId(String lawId) {   
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("lawId", lawId);
        
        List<Tax> results = null;
		try
		{
			// named query is in orm.xml
			results = findByNamedQuery("findTaxByLawId", queryParameters);
		}
		catch(RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		if (results == null)
		{
		    return null;
		}
        switch (results.size())
        {
            case 0:
                return null;
            case 1:
                return results.get(0);
            default:
                throw new IllegalStateException("Found multiple Tax with lawId '" + lawId);
        }
    }
}
