package com.intuit.sb.payroll.entity.wages.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.entity.EntityResources;
import com.intuit.sb.payroll.entity.NamedQueryCatalog;
import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.wages.CompensationEAO;

@Stateless(name="CompensationEAO")
@TransactionManagement(TransactionManagementType.CONTAINER) // Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY) 
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
@SuppressWarnings("nls")
public final class CompensationEjbEAO extends GenericEjbEAO<Compensation>
	implements CompensationEAO {

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.wages.CompensationEAO#findCompensation(com.intuit.sb.payroll.entity.model.CompensationTypeEnum)
	 */
	public Compensation findCompensation(CompensationTypeEnum compensationType) {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put(NamedQueryCatalog.COMPENSATION_TYPE_QUERY_PARAM, compensationType);
        
        List<Compensation> results = null;
		try	{
			// named query is in orm.xml
			results = findByNamedQuery(NamedQueryCatalog.FIND_COMPENSATION_BY_TYPE, queryParameters);
		}
		catch(RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		switch (results.size()) {
		case 0:
			String mesg1 = MessageFormat.format(EntityResources.getString("compensation.does.not.exist"), compensationType.toString());
			logger.error(mesg1);
			throw new IllegalStateException(mesg1);
		case 1:
			return results.get(0);
		default:
			String mesg2 = MessageFormat.format(EntityResources.getString("compensation.multiple.for.type"), compensationType.toString());
			logger.error(mesg2);
			throw new IllegalStateException(mesg2);
		}
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.component.wages.CompensationEAO#findCompensation(java.lang.String)
	 */
	public Compensation findCompensation(Long uniqueId) {
		Compensation compensation = findById(uniqueId);
		if (compensation == null)
		{
			String mesg = MessageFormat.format(EntityResources.getString("compensation.with.id.does.not.exist"), uniqueId);
			throw new InfrastructureException(mesg);
		}
		return compensation;
	}

}
