package com.intuit.sb.payroll.entity.worker;

import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.entity.eao.GenericEAO;
import com.intuit.sb.payroll.entity.model.Worker;

@Local
public interface WorkerEAO extends GenericEAO<Worker> {

	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "WorkerEAO/local";

	/**
     * Finds all the workers  not ready for payroll 
     * @return the list of worker for the entity
     */
    public List<Worker> findWorkersNotReadyForPayroll();
    
    /**
     * Finds all the workers ready for payroll 
     * @return the list of worker for the entity
     */
    public List<Worker> findWorkersReadyForPayroll();
    
    /**
     * Finds the number of workers in the company
     * @return the number of workers in the system for the user Realm, 
     */
    public int getWorkerCount();
    
    /**
     * Finds all the workers for a given Work Location 
     * @return the list of workers for the entity
     */
    public List<Worker> findWorkersByWorkLocation(Long uniqueId);
    
    /**
     * Finds all workers who have their Active state to true.
     * @return
     */
    public List<Worker> findActiveWorkers();
    
    /**
     * Finds all workers who have their Active state to false;
     * @return
     */
    public List<Worker> findInactiveWorkers();
}
