package com.intuit.sb.payroll.entity.workerpayments.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.workerpayments.PaycheckEAO;

@Stateless(name="PaycheckEAO")
@TransactionManagement(TransactionManagementType.CONTAINER) // Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY) 
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public final class PaycheckEjbEAO extends GenericEjbEAO<Paycheck>
	implements PaycheckEAO {

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.workerpayments.PaycheckEAO#findMostRecentPaycheckForWorker(com.intuit.sb.payroll.entity.model.Worker)
	 */
	public Paycheck findMostRecentPaycheckForWorker(Worker worker) {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("workerId", worker.getId()); // not getWorkerId - that's a company's employee#
        
        List<Paycheck> results = null;
		try	{
			// named query is in orm.xml
			results = findByNamedQuery("findMostRecentPaycheckForWorker", queryParameters);
		}
		catch(RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		switch (results.size()) {
		case 0:
			return null;
		case 1:
			return results.get(0);
		default:
			String mesg2 = MessageFormat.format("Found multiple Paychecks with identical txnDate for Worker ID ''{0}''.", worker.getId());
			logger.error(mesg2);
			throw new IllegalStateException(mesg2);
		}
	}

}
