package com.intuit.sb.payroll.entity.workerpayments.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.NamedQueryCatalog;
import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO;

@Stateless(name="PayrollRunEAO")
@TransactionManagement(TransactionManagementType.CONTAINER)
@TransactionAttribute(TransactionAttributeType.MANDATORY) 
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
@SuppressWarnings("nls")
public final class PayrollRunEjbEAO extends GenericEjbEAO<PayrollRun> implements PayrollRunEAO {

	/**
	 * Experimental query to get payruns from a date on.
	 */
	public List<PayrollRun> findPayrollRunsFromDate(Date fromDate) {
		List<PayrollRun> results = null;

        Map<String, Object> queryParameters = new HashMap<String, Object>();
	    queryParameters.put(NamedQueryCatalog.START_DATE_QUERY_PARAM, fromDate);
	
		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_PAYROLLRUNS_FROM_DATE, queryParameters);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		
		return results;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO#findLastCompletedPayrollRun()
	 */
	public PayrollRun findLastCompletedPayrollRun() {
		List<PayrollRun> results = null;
		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_LAST_COMPLETED_PAYROLLRUN);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		
		if (results.size() > 0) {
			return results.get(0);
		} else {
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO#findRecentPayrollRuns(com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum, int)
	 */
	public List<PayrollRun> findRecentPayrollRuns(PayrollRunTypeEnum payrollRunType, int numberOfRuns)
	{
		List<PayrollRun> results = null;

        Map<String, Object> queryParameters = new HashMap<String, Object>();
	    queryParameters.put(NamedQueryCatalog.PAYROLL_RUN_TYPE_PARAM, payrollRunType);
	
		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_RECENT_PAYROLLRUNS, queryParameters, 0, numberOfRuns);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		
		return results;
	}
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO#findRecentPayrollRuns(com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum, int)
	 */
	public List<PayrollRun> findRecentPayrollRunsForYear(PayrollRunTypeEnum payrollRunType, int year)
	{
		List<PayrollRun> results = null;

        Map<String, Object> queryParameters = new HashMap<String, Object>();
	    queryParameters.put(NamedQueryCatalog.PAYROLL_RUN_TYPE_PARAM, payrollRunType);
	    queryParameters.put(NamedQueryCatalog.YEAR, year);

		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_RECENT_PAYROLLRUNS_FOR_YEAR, queryParameters);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		
		return results;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO#findAllDescending()
	 */
	public List<PayrollRun> findAllDescending() {
		List<PayrollRun> results = null;
		try {
			results = this.findByNamedQuery(NamedQueryCatalog.FIND_ALL_PAYROLLRUNS_DESC);
		} catch (RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		
		return results;
	}

	
}
