package com.intuit.sb.payroll.takehome.entity.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.takehome.entity.TakehomeCompany;
import com.intuit.sb.payroll.takehome.entity.TakehomeCompanyEAO;

/**
 * Implements the generic CRUD data access operations 

 * @param TakehomeCompany the concrete extension of {@code BaseEntity} that implements the entity
 * managed by this EAO. 
 * 
 * @see GenericEjbEAO
 */

@Stateless(name="TakehomeCompanyEAO")
@TransactionManagement(TransactionManagementType.CONTAINER)
@TransactionAttribute(TransactionAttributeType.MANDATORY) 
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public final class TakehomeCompanyEjbEAO extends GenericEjbEAO<TakehomeCompany>
	implements TakehomeCompanyEAO {

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.takehome.entity.TakehomeCompanyEAO#findByPayrollEmployerId(java.lang.Long)
	 */
	public TakehomeCompany findByPayrollEmployerId(Long uniqueId) {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("payrollEmployerId", uniqueId);
        
        List<TakehomeCompany> results = null;
		try	{
			// named query is in orm.xml
			results = findByNamedQuery("findTakehomeCompanyByEmployerId", queryParameters);
		}
		catch(RuntimeException ex) {
			handleExceptionMightBeOLE(ex);
		}
		switch (results.size()) {
		case 0:
			return null;
		case 1:
			return results.get(0);
		default:
			String mesg2 = MessageFormat.format("Found multiple Takehome Companies with Payroll Employer ID ''{0}''.", uniqueId);
			logger.error(mesg2);
			throw new IllegalStateException(mesg2);
		}
	}
}
