package com.intuit.sb.payroll.capability.employer;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.BasicPTSTestBase;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandler;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerFactory;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentState;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;

public class EFTPSRejectionTaskTest extends BasicPTSTestBase{
	
	@Test(enabled=true)
	public void testEINMismatchRejectionHandled() throws ValidationException , ServiceFailureException {
		PTSEmployer ptsEmployer = sEmployerPTSWorkflowComponent.getPTSEmployer();
		PTSNotificationRequest ptsRequest  = new PTSNotificationRequest();
		ptsRequest.setNotificationType(PTSNotificationTypeEnum.EFTPS_ENROLLMENT_NOTIFICATION);
		ptsRequest.setNotificationStatus(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH);
		
		PTSNotificationTypeEnum ptsNotificationType = ptsRequest.getNotificationType();
		
		StatusUpdateHandler statusUpdateHandler = StatusUpdateHandlerFactory.getStatusUpdateHandler(ptsNotificationType);
		try {
			statusUpdateHandler.updateStatus(ptsEmployer, ptsRequest);
		} catch (PTSNotificationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//Send the EIN Mismatch Rejection Notice
		//EFTPSEnrollmentStatusEnum eftpsEnrollmentStatusEnum = StatusUpdateHandlerUtil.getRecentlyReceivedEFTPSRejection(ptsEmployer).getEftpsEnrollmentStatus();
		EFTPSEnrollmentState eftpsEnrollmentState = StatusUpdateHandlerUtil.getRecentlyReceivedEFTPSRejection();
		AssertJUnit.assertEquals("EIN Mismamatch Rejection not Recevied",eftpsEnrollmentState,EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH);
		
		//Test if EINMismatch task got created
		List<Task> rejectedEINMismatch = taskEAO.findTaskByTaskNameAndPayLoad(EmployerPTSWorkflowTasks.REJECTED_EIN_MISMATCH_TASK, ptsEmployer.getEmployer().getId());
		AssertJUnit.assertEquals("EIN Mismatch Rejection not created",EmployerPTSWorkflowTasks.REJECTED_EIN_MISMATCH_TASK, rejectedEINMismatch.get(0).getName());
		
		//Update the EmployerId
		
		sEmployerComponent.updateEmployerIds(ptsEmployer.getEmployer());
		
		//Test if EINMismatch task got closed
		
		AssertJUnit.assertEquals("Task didnot get closed",TaskStatusEnum.COMPLETED, rejectedEINMismatch.get(0).getStatus());
		
		//Test if PrintandResubmitForms Task got created
		List<Task> printandResubmitTask = taskEAO.findTaskByTaskNameAndPayLoad(EmployerPTSWorkflowTasks.PRINT_AND_RESUBMIT_FORMS, ptsEmployer.getEmployer().getId());
		AssertJUnit.assertEquals("Print and Resubmit Forms not created",EmployerPTSWorkflowTasks.PRINT_AND_RESUBMIT_FORMS, printandResubmitTask.get(0).getName());
	}
		
	@Test(enabled=true)
	public void testLegalNameMismatchRejectionHandled() throws ValidationException , ServiceFailureException {
		PTSEmployer ptsEmployer = sEmployerPTSWorkflowComponent.getPTSEmployer();
		PTSNotificationRequest ptsRequest  = new PTSNotificationRequest();
		ptsRequest.setNotificationType(PTSNotificationTypeEnum.EFTPS_ENROLLMENT_NOTIFICATION);
		ptsRequest.setNotificationStatus(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH);
		
		PTSNotificationTypeEnum ptsNotificationType = ptsRequest.getNotificationType();
		
		StatusUpdateHandler statusUpdateHandler = StatusUpdateHandlerFactory.getStatusUpdateHandler(ptsNotificationType);
		try {
			statusUpdateHandler.updateStatus(ptsEmployer, ptsRequest);
		} catch (PTSNotificationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//Send the LegalName Mismatch Rejection Notice
		EFTPSEnrollmentState eftpsEnrollmentState = StatusUpdateHandlerUtil.getRecentlyReceivedEFTPSRejection();
		AssertJUnit.assertEquals("LegalName Mismamatch Rejection not Recevied",eftpsEnrollmentState,EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH);
		
		//Test if LegalNameMismatch task got created
		List<Task> rejectedLegalNameMismatch = taskEAO.findTaskByTaskNameAndPayLoad(EmployerPTSWorkflowTasks.REJECTED_LEGALNAME_MISMATCH_TASK, ptsEmployer.getEmployer().getId());
		AssertJUnit.assertEquals("LegalName Mismatch Rejection not created",EmployerPTSWorkflowTasks.REJECTED_LEGALNAME_MISMATCH_TASK, rejectedLegalNameMismatch.get(0).getName());
		
		//Update the Employer
		
		sEmployerComponent.saveEmployer(ptsEmployer.getEmployer());
		
		//Test if LegalNameMismatch task got closed
		
		AssertJUnit.assertEquals("Rejected Legal Name mismatch Task is not closed",TaskStatusEnum.COMPLETED, rejectedLegalNameMismatch.get(0).getStatus());
		
		//Test if PrintandResubmitForms Task got created
		List<Task> printandResubmitTask = taskEAO.findTaskByTaskNameAndPayLoad(EmployerPTSWorkflowTasks.PRINT_AND_RESUBMIT_FORMS, ptsEmployer.getEmployer().getId());
		AssertJUnit.assertEquals("Print and Resubmit Forms not created",EmployerPTSWorkflowTasks.PRINT_AND_RESUBMIT_FORMS, printandResubmitTask.get(0).getName());		
	}
}
