package com.intuit.sb.payroll.itest.util;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.itest.common.IntegrationTestBase;
import com.intuit.sb.payroll.psp.testws.BatchJobTestWSRequestWraper;
import com.intuit.sb.payroll.psp.testws.CompanyTestWSRequestWrapper;
import com.intuit.sb.payroll.psp.testws.PSPDateTestWSRequestWrapper;
import com.intuit.sbd.payroll.psp.webservices.Exception_Exception;

public class PayrollTestUtil extends IntegrationTestBase{
	
	private static PSPDateTestWSRequestWrapper psDateTestWSRequestWrapper = null;
	private static CompanyTestWSRequestWrapper companyTestWSRequestWrapper = null;
	private static BatchJobTestWSRequestWraper batchJobTestWSRequestWraper = null;
	
	public static void initialize(String targetHost){
		
		psDateTestWSRequestWrapper = new PSPDateTestWSRequestWrapper(targetHost);
		companyTestWSRequestWrapper = new CompanyTestWSRequestWrapper(targetHost);
		batchJobTestWSRequestWraper = new BatchJobTestWSRequestWraper(targetHost);
	
	}
	
	public static  void activateTaxPaymentService(String sourceCompanyID , String sourceCompanyBankID) throws Exception_Exception, InterruptedException{
		
		
		// Run EFTPS batch process
		batchJobTestWSRequestWraper.runEFTPSBatchProcessor();
		
		//Change the PSP date
		psDateTestWSRequestWrapper.adjustPSPDateToOffloadTransaction();	
		
		//Run ACH
		batchJobTestWSRequestWraper.runACH();
		
		//Change the PSP date to 5 days in future
		psDateTestWSRequestWrapper.adjustPSPDateForBankActivation();
		
		//Run ACH Transaction Processor
		batchJobTestWSRequestWraper.runACHTransactionProcessor();
		
		//upload the forms IRS
		companyTestWSRequestWrapper.uploadRAAForms(sourceCompanyID, "IRS", "Federal8655");
		
		//upload the forms CA
		companyTestWSRequestWrapper.uploadRAAForms(sourceCompanyID, "CAEDD", "LPOA");
		
		//set Activation Status 
		companyTestWSRequestWrapper.updateActivationState(sourceCompanyID,"InProgress",CompanyTestWSRequestWrapper.getActivationCheckListItems());
		
		//run EFTPS Enrollment Processor
		batchJobTestWSRequestWraper.runEFPTSEnrollmentBatchProcessor();
		
		// -- set Activation Status to Complete 
		companyTestWSRequestWrapper.updateActivationState(sourceCompanyID,"Complete",null);
		
		// -- Accept the TaxService
		//companyTestWSRequestWrapper.acceptTaxService(sourceCompanyID, sourceCompanyBankID);
		
		
	}
	
	
	  public static String generateRandomEIN() throws ServiceFailureException {

	    	long randomEIN;
	    	long min = 1000000;
	    	long max = 9999999;
	    	String [] validPrefixes = null;
	    	
			validPrefixes = employerTaxSettingsService.getAllValidEinPrefixes();
			
	        randomEIN = (long)Math.floor((Math.random() * (max - min + 1) )) + min;
	        int index = (int)(Math.random()*validPrefixes.length);
	        String prefix = validPrefixes[index];
	        StringBuilder str = new StringBuilder();
	        str.append(prefix).append(randomEIN);        
	        return str.toString();          
	    }




}
