package com.intuit.sb.payroll.psp.testws;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import com.intuit.sbd.payroll.psp.adapters.gemini.AcceptTaxServiceAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollAdapter;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollAdapterService;
import com.intuit.sbd.payroll.psp.webservices.ActivationCheckListItemWSDTO;
import com.intuit.sbd.payroll.psp.webservices.CompanyBankAccountWSDTO;
import com.intuit.sbd.payroll.psp.webservices.CompanyWS;
import com.intuit.sbd.payroll.psp.webservices.CompanyWSDTO;
import com.intuit.sbd.payroll.psp.webservices.CompanyWSService;
import com.intuit.sbd.payroll.psp.webservices.DocumentWSDTO;
import com.intuit.sbd.payroll.psp.webservices.Exception_Exception;

public class CompanyTestWSRequestWrapper {
	
	
	private static final String COMPANY_TESTWS_WSDL  = "/test-ws/services/CompanyWS?wsdl";
	private static final String PAYROLL_ADAPTER_WSDL = "/gemini-ws/services/PayrollAdapter?wsdl";
	private String host = null;
	private CompanyWS companyWS = null;
	
	public CompanyTestWSRequestWrapper(String host){
		this.host = host;
		companyWS = getCompanyWS();
		
	}
	
	public List<CompanyBankAccountWSDTO> queryCompanyBankAccounts(String pspCompanyID) throws Exception_Exception{
		return companyWS.queryCompanyBankAccounts(pspCompanyID);
	}
	
	public List<CompanyWSDTO> queryCompanyByLegalName(String companyLegalName) throws Exception_Exception{
		
		return companyWS.queryCompaniesByLegalName(null, companyLegalName);
	}
	
	public void uploadRAAForms(String sourceCompanyId,String agencyID,String formId) throws Exception_Exception{
		
		
		companyWS.uploadRAAForms("GEMINI", sourceCompanyId, agencyID, getDocumentDTO(formId));
		
	}
	
	public void updateActivationState(String sourceCompanyId , String status ,List <ActivationCheckListItemWSDTO> checklistItems) throws Exception_Exception{
		
		
		companyWS.updateActivationCheckListCore("GEMINI", sourceCompanyId, status, checklistItems);
	}
	
	// move this out
	public void acceptTaxService(String sourceCompanyID , String sourceBankId){
		PayrollAdapter payrollAdapter = getPayrollAdapter();
		payrollAdapter.acceptTaxService(getAcceptTaxServiceAdapterDTO(sourceCompanyID,sourceBankId));
		
	}
	
	
	private static AcceptTaxServiceAdapterDTO getAcceptTaxServiceAdapterDTO(String sourceCompanyID , String sourceBankId){
		
		AcceptTaxServiceAdapterDTO aTaxServiceAdapterDTO = new AcceptTaxServiceAdapterDTO();
		aTaxServiceAdapterDTO.setSourceCompanyId(sourceCompanyID);
		aTaxServiceAdapterDTO.setSourceCompanyBankAccountId(sourceBankId);
		return aTaxServiceAdapterDTO;
	}
	
	public static List <ActivationCheckListItemWSDTO> getActivationCheckListItems(){
	
		List<ActivationCheckListItemWSDTO> acList = new ArrayList<ActivationCheckListItemWSDTO>();
		
		ActivationCheckListItemWSDTO acListItemWSDTO_IRS = new ActivationCheckListItemWSDTO();
		
		acListItemWSDTO_IRS.setAgency("IRS");
		acListItemWSDTO_IRS.setStatus("Complete");
		acListItemWSDTO_IRS.setItemType("RAAEnrollment8655");
		
		ActivationCheckListItemWSDTO acListItemWSDTO_CA = new ActivationCheckListItemWSDTO();
		
		acListItemWSDTO_CA.setAgency("CAEDD");
		acListItemWSDTO_CA.setStatus("Complete");
		acListItemWSDTO_CA.setItemType("RAAEnrollmentLPOA");
		
		acList.add(acListItemWSDTO_IRS);
		acList.add(acListItemWSDTO_CA);
		
		return acList;
		
	}
	
	public static DocumentWSDTO getDocumentDTO(String formId){
		DocumentWSDTO dto = new DocumentWSDTO();
		dto.setMimeType("text/plain");
		dto.setFormId(formId);
		dto.setDocument("test document");
		return dto;
	}
	
	private CompanyWS getCompanyWS(){
		
		CompanyWSService companyWSService = null;
	
		String wsdlUrlStr = host+COMPANY_TESTWS_WSDL;
		
		URL url = null;
        try {
            URL baseUrl = CompanyWSService.class.getResource(".");
            url = new URL(baseUrl, wsdlUrlStr);
        } catch (MalformedURLException e) {
        	
        	
        }
        QName qname = new QName("http://webservices.psp.payroll.sbd.intuit.com/", "CompanyWSService");
        companyWSService = new CompanyWSService(url,qname);
        CompanyWS companyWS = companyWSService.getCompanyWSPort();
        
        return companyWS;

	}
	
	private  PayrollAdapter getPayrollAdapter(){
		
		PayrollAdapterService pAdapterService = null;
		
		String wsdlUrlStr = host+PAYROLL_ADAPTER_WSDL;
		
		URL url = null;
        try {
            URL baseUrl = PayrollAdapterService.class.getResource(".");
            url = new URL(baseUrl, wsdlUrlStr);
        } catch (MalformedURLException e) {
        	
        }
        
        QName qname = new QName("http://gemini.adapters.psp.payroll.sbd.intuit.com/", "PayrollAdapterService");
       
        pAdapterService= new PayrollAdapterService(url,qname);
        
        PayrollAdapter pAdapter = pAdapterService.getPayrollAdapterPort();
        
        return pAdapter;
		
	}


}
