package com.intuit.sb.payroll.qa.services.employee;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import junit.framework.Assert;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.PTSWorkerPaymentServiceTestBase;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.pts.entity.InteractionEnum;
import com.intuit.sb.payroll.pts.entity.PPSPTSInteraction;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;



public class EmployeeManagementTest extends PTSWorkerPaymentServiceTestBase{
	private Worker worker1 = null;
	private com.intuit.sb.payroll.services.worker.Worker worker = null;
	private static Date hireDate = null;
	private static Date terminationDate = null;
	
	@BeforeMethod
	public void SetUp() throws Exception{
		prepareCurrentEmployerForAssistedWithBank();
		worker1 = new Worker();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DAY_OF_YEAR, -4);
		hireDate = cal.getTime();		
		worker1 = createAssistedReadyEmployee(worker1, hireDate);
		terminationDate = DateUtil.getNowDate();
		worker = WorkerMapper.mapFromEntityToPojo(worker1);		
	}
	
	/**
     * <P>Test re hiring a terminated employee existing in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee and terminate</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Re hire the terminated employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Activate Employee call made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Service failure exception:Unable to Submit PayrollRun to PTS
	public void testRehireATerminatedEmployee()throws Exception{		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest); // terminate worker
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
     	
     	worker = WorkerMapper.mapFromEntityToPojo(worker1);
     	
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		List<PPSPTSInteraction> ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker.getObjectHandle().getId());
    	int interactionsBeforeRehire = ppsPTSInteractions.size();
    			
    	//
    	PPSPTSInteraction ppsInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.ADDED);
		Date beforeDate = ppsInteraction.getInteractionDate();
    	//Exercise
		worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId(),terminationDate );
		//Assert
		ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker.getObjectHandle().getId());
    	Assert.assertEquals("Activate not called", interactionsBeforeRehire+1, ppsPTSInteractions.size());			
    	ppsInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.ADDED);
		Assert.assertTrue("Activate call not made to PSP",beforeDate.before(ppsInteraction.getInteractionDate()));
	}
	
	/**
     * <P>Test re hiring a terminated employee existing in PSP on Null Date</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee and terminate</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Re hire the terminated employee on Null Date</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Activate Employee call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Service failure exception:Unable to Submit PayrollRun to PTS
	public void testRehireEmployeeWithNullDate()throws Exception{		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);// terminate worker
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
     	
     	worker = WorkerMapper.mapFromEntityToPojo(worker1);     	
     	
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		//verify no add employee call made to PSP
		List<PPSPTSInteraction> ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker1.getId());
    	int interactionsBeforeTerminate = ppsPTSInteractions.size();		
		
		try{
			worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId(), null);
			Assert.fail("Expected Validation Exception is not thrown");
		}catch(ValidationException ve){
			ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker.getObjectHandle().getId());
        	Assert.assertEquals("DeActivate not called", interactionsBeforeTerminate, ppsPTSInteractions.size());			
    	}		
	}
	
	/**
     * <P>Test re hiring a null Id employee</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee and terminate</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Re hire the employee with null Id</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Activate Employee call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Service failure exception:Unable to Submit PayrollRun to PTS
	public void testRehireEmployeeWithNullId()throws Exception{		
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest); // teminate worker
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
     	worker = WorkerMapper.mapFromEntityToPojo(worker1);   	
     	
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		List<PPSPTSInteraction> ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker1.getId());
    	int interactionsBeforeTerminate = ppsPTSInteractions.size();		
		
		try{
			worker = sWorkerService.hireExistingWorker(null ,terminationDate );
			Assert.fail("Expected Validation Exception is not thrown");
		}catch(ValidationException ve){
			ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker.getObjectHandle().getId());
        	Assert.assertEquals("DeActivate not called", interactionsBeforeTerminate, ppsPTSInteractions.size());			
    	}		
	}
	
	/**
     * <P>Test re hiring a re hired employee existing in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee and terminate</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Re hire the terminated employee</LI>
     * <LI>Re hire the same employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Activate Employee call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)// Service failure exception saying company not active in PSP
	public void testRehireAlreadyRehiredEmployee()throws Exception{		
				
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
				
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		//terminate the employee
		worker = sWorkerService.terminateWorker(worker, terminationDate);
		//re-hire the terminated employee
		worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId(),terminationDate );
//		List<PPSPTSInteraction> ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker1.getId());
//    	int interactionsBeforeTerminate = ppsPTSInteractions.size();		
		
		try{
			worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId() ,terminationDate );
			Assert.fail("Expected Validation Exception is not thrown");
		}catch(ValidationException ve){
			Assert.fail(ve.getMessage());
			//TODO hook to PSP Test Web Service for data validation 			
    	}		
	}
	
	/**
     * <P>Test Updating an employee existing in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Update the employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Update Employee call made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Error during tear down in CompensationItemTestBase- null pointer exception
	public void testUpdateAssistedEmployeeOfEmployerInPTS()throws Exception{

		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		//Exercise
		worker.setWorkerTaxIdNumber("123456789");
		worker = sWorkerService.saveWorker(worker);
		//Assert Update
		
		PPSPTSInteraction ppsPTSInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.UPDATED);
		Assert.assertNotNull("Update Call not made to PSP",ppsPTSInteraction);
	}	
	
	/**
     * <P>Test Updating an Assisted ready employee existing in PSP to Non-Assisted</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Update the employee to make Non Assisted</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Update Employee call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false) // bug??? call made to PSP
	public void testUpdateAssistedEmployeeToNonAssistedForEmployerInPTS()throws Exception{

		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);		
		worker.setWorkerTaxIdNumber(null);
		worker = sWorkerService.saveWorker(worker);
		//Assert Update
		PPSPTSInteraction ppsPTSInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.UPDATED);
		Assert.assertNull("Update Call Should not be made to PSP",ppsPTSInteraction);
	}
	
	/**
     * <P>Test Updating a terminated employee existing in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>terminate this employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Update the terminated employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Update Employee call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Service Failure Exception: Unable to Submit PayrollRun to PTS
	public void testUpdateATerminatedEmployee()throws Exception{
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest); // terminate worker
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
     	
     	worker = WorkerMapper.mapFromEntityToPojo(worker1);
     	
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		//Excercise
		worker.setFirstName("Bravo");
		sWorkerService.saveWorker(worker);
		
		//Assert Update
		PPSPTSInteraction ppsPTSInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.UPDATED);
		Assert.assertNull("Update Call Should not be made to PSP",ppsPTSInteraction);
	}
	
	/**
     * <P>Test Updating and Re hiring a terminated employee existing in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>terminate this employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Update the terminated employee</LI>
     * <LI>Re hire the same employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Activate and Update Employee calls made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Service Failure Exception: Unable to Submit PayrollRun to PTS
	public void testUpdateATerminatedEmployeeAndRehire()throws Exception{
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);// terminate worker
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
     	worker = WorkerMapper.mapFromEntityToPojo(worker1);
     	
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		List<PPSPTSInteraction> ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker.getObjectHandle().getId());
		int interactionsBeforeUpdate = ppsPTSInteractions.size();
		PPSPTSInteraction ppsInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.UPDATED);
		Date beforeDate = ppsInteraction.getInteractionDate();
		
		//Exercise
		worker.setFirstName("Bravo");
		worker = sWorkerService.saveWorker(worker);		
		worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId(),terminationDate );
		
		//Assert that no call for Submit Payroll is made by Gemini
     	ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker.getObjectHandle().getId());
    	Assert.assertEquals("DeActivate not called", interactionsBeforeUpdate+2, ppsPTSInteractions.size());  			
    	ppsInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.ADDED);
		Assert.assertTrue("Activate call not made to PSP",beforeDate.before(ppsInteraction.getInteractionDate()));
		ppsInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.UPDATED);
		Assert.assertNotNull("Update call not made to PSP", ppsInteraction);
	}
	
	/**
     * <P>Test terminating an employee existing in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Terminate the employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that InActivate Employee call made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Service Failure Exception: Unable to Submit PayrollRun to PTS
	public void testTerminateAnEmployeeOfEmployerInPTS()throws Exception{
		
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		PPSPTSInteraction ppsPTSInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.DEACTIVATED);
		Assert.assertNull("Deactivate call should not be made to PSP",ppsPTSInteraction);
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);// terminate worker
    	
		worker = WorkerMapper.mapFromEntityToPojo(worker1);
		
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
				
		//Verify
		ppsPTSInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.DEACTIVATED);
		
		Assert.assertNotNull("Deactivate call not made to PSP",ppsPTSInteraction);
		
	}
	
	/**
     * <P>Test Terminating a Re hired employee existing in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>terminate this employee</LI>
     * <LI>Add company to PSP</LI>    
     * <LI>Re hire the terminated employee</LI>
     * <LI>Terminate the re hired employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that InActivate Employee call made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Service Failure Exception: Unable to Submit PayrollRun to PTS
	public void testTerminateRehiredEmployeeOfEmployerInPTS()throws Exception{
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    	
		worker = WorkerMapper.mapFromEntityToPojo(worker1);
		
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
     	
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		PPSPTSInteraction ppsPTSInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.DEACTIVATED);
		Date beforeDate = ppsPTSInteraction.getInteractionDate();
		//Exercise
		worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId(),terminationDate );
		
		workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
     	
		sWorkerService.terminateWorker(worker, terminationDate);
		
		//Verify
		ppsPTSInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.DEACTIVATED);
		Date afterDate = ppsPTSInteraction.getInteractionDate();
		Assert.assertTrue("Deactivate Call not made to PSP", afterDate.after(beforeDate));
		
	}	
	
	/**
     * <P>Test terminating employee existing in PSP on Null Date</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>terminate the employee on Null Date</LI>
     * </OL>
     * <UL>
     * <LI>Verify that InActivate Employee call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=true)
	public void testTerminateEmployeeWithNullDate()throws Exception{
		
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);		
		    	
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(null);
			    	
    	try{
    		//Exercise
    		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);// terminate worker
        	
    		worker = WorkerMapper.mapFromEntityToPojo(worker1);
    		
        	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
         	
    		Assert.fail("Expected Validation Exception is not thrown");
    	}
    	catch(ValidationException ve){
    		//TODO hook to PSP Test Web Service for data validation    		
    	}		
	}
		
	
	/**
     * <P>Test terminating a terminated employee existing in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee and terminate</LI>
     * <LI>Add company to PSP</LI>
     * <LI>terminate the employee</LI>
     * <LI>terminate the same employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that InActivate Employee call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)// Service failure exception saying company not active in PSP
	public void testTerminateAlreadyTerminatedEmployee()throws Exception{
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();		
		
		worker = sWorkerService.terminateWorker(worker, terminationDate);
		List<PPSPTSInteraction> ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker1.getId());
    	int interactionsBeforeTerminate = ppsPTSInteractions.size();		
    	
    	try{
    		worker = sWorkerService.terminateWorker(worker, hireDate);
    		Assert.fail("Expected Validation Exception is not thrown");
    	}
    	catch(ValidationException ve){
    		
    		ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker.getObjectHandle().getId());
        	Assert.assertEquals("DeActivate Should not be called to PSP", interactionsBeforeTerminate, ppsPTSInteractions.size());  			
    		
    	}		
	}
	
	/**
     * <P>Test deleting an employee existing in PSP who has no Payrolls</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>delete the employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Delete Employee call made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)// Error in tear down of Compensation Item TestBase
	public void testDeleteAnAssistedEmployeeWithNoPayrollsForEmployerInPTS()throws Exception{
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		//Exercise
		sWorkerService.deleteWorker(worker.getObjectHandle().getId());
		
		//Verify
		PPSPTSInteraction ppsInteraction =StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.REMOVED);
		Assert.assertNotNull("Delete Call not made to PSP",ppsInteraction);
	}
	
	/**
     * <P>Test deleting an employee existing in PSP who hasPayrolls</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>terminate this employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>delete the employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Delete Employee call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Service failure exception:Unable to Submit PayrollRun to PTS
	public void testDeleteATerminatedEmployeeOfEmployerInPTS()throws Exception{
		
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest); // terminate worker
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
     	
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		//Exercise
		sWorkerService.deleteWorker(worker.getObjectHandle().getId());
		//Verify
		PPSPTSInteraction ppsInteraction = StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.REMOVED);
		Assert.assertNotNull("Delete Call not made to PSP",ppsInteraction);
	
	}
	
	/**
     * <P>Test deleting an employee existing in PSP who hasPayrolls</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Run a payroll including this employee</LI>
     * <LI>delete the employee</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Delete Employee call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false) //Service failure exception:Unable to Submit PayrollRun to PTS
	public void testDeleteAnAssistedEmployeeWithPayrollsForEmployerInPTS()throws Exception{
		
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, DateUtil.getNowDate());
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.SCHEDULED);
		
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
     	
     	worker = WorkerMapper.mapFromEntityToPojo(worker1);     	
     	
     	workerIds = new Long[1];
		workerIds[0] = worker.getObjectHandle().getId();
		payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, terminationDate);
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(terminationDate);
     	
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    	
		worker = WorkerMapper.mapFromEntityToPojo(worker1);
     	
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		try{
			//Exercise
			sWorkerService.deleteWorker(worker.getObjectHandle().getId());
			Assert.fail("Expected Validation Exception");
		}catch(ValidationException ve){
			//verify
			PPSPTSInteraction ppsInteraction =StatusUpdateHandlerUtil.getPPSPTSInteraction(Worker.class.getName(), worker.getObjectHandle().getId(), InteractionEnum.REMOVED);
			Assert.assertNull("Delete Call made to PSP",ppsInteraction);
		}
				
	}
	
	/**
     * <P>Test Hiring an employee with future date and add company to PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Create an assisted ready employee with hire date as future date</LI>
     * <LI>Add company to PSP</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Add Employee call not made to PSP immediately</LI>
     * </UL>
     */
	@Test(enabled=false)// Dev implementation need to fix this.
	public void testHireAnEmployeeWithFutureDateAndAddCompanyToPSP() throws Exception{
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, 2);
		hireDate = cal.getTime(); 
		worker1.getEmploymentPeriod().setStartDate(hireDate);
		worker1 = sWorkerComponent.updateWorker(worker1);
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		//Assert that add employee call is not made by Gemini
     	List<PPSPTSInteraction> ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker1.getId());
    	Assert.assertEquals("DeActivate not called", 2, ppsPTSInteractions.size());  			
	}
	
	/**
     * <P>Test Hiring an employee with future date to a company in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Create an assisted ready employee with hire date as future date</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Add Employee call not made to PSP immediately</LI>
     * </UL>
     */
	@Test(enabled=false)// Dev implementation need to fix this.
	public void testCompanyInPSPAddsEmployeeWithFutureHireDate() throws Exception{
		
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		worker1 = new Worker();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, 2);
		hireDate = cal.getTime(); 
		worker1 = createAssistedReadyEmployee(worker1, hireDate);			
		//Assert that no call for Submit Payroll is made by Gemini
     	List<PPSPTSInteraction> ppsPTSInteractions = ppsPTSInteractionEAO.findAll(Worker.class.getName(), worker1.getId());
    	Assert.assertEquals("DeActivate not called", 2, ppsPTSInteractions.size());  			
	}

}
