package com.intuit.sb.payroll.qa.services.employer;


import java.util.Calendar;

import org.testng.Assert;
import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.PTSWorkerTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.common.impl.ObjectHandleMapper;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.worker.Worker;


/**
 * /**
 * <P>The <CODE>PTSSendEmployeesTests</CODE> class contains test methods for adding a Existing Employee (Batch send while signing up for assisted)to PTS after adding the company to PTS and ValidationException for various error path
 * scenario 
 * $Author: kvenkatraman $
 * Required Field for Employee to be assisted ready
 * 1. Home address
 * 2. Payment method
 * 3. Work location
 * 4. First name
 * 5. Last name
 * 6. Hire Date
 * 7. Pay item
 * 8. SSN
 *
 */
public class PTSSendEmployeesTest extends PTSWorkerTestBase{
	
	protected static EmployerPTSWorkflowComponent empPTSWfComponent;
	
	
	 @BeforeClass
	 public static void initializePTSSendEmployeesTestsClass()
	 {
		 
		 empPTSWfComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
	 }
	
	@BeforeMethod
	public void setUpMethod() throws Exception{
		EmployerComponentTestUtils.deleteCurrentEmployer();
		createAndSaveAssistedReadyEmployer();
	}
	 /**
	 * <P>
	 * Test adding Assisted ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Compensation Item</LI>
	 * <LI>SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Add Worklocation to Worker</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that Employer is Added to PSP and Employee is added to PSP
	 * </LI>
	 * </OL>
	 */


	@Test (enabled = false)// to be fixed by Dev- SSN not saved with WorkerService.saveworker
	public void testAddAssistedReadyEmployee() throws Exception{

		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		worker = addWorkLocationToWorker(worker, "Nanak Pura", "New Delhi");
		worker.setWorkerTaxIdNumber("484848484");
		worker = sWorkerService.saveWorker(worker);
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
				StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));
		
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
	
		AssertJUnit.assertEquals("Add Employee Call is not sent to PSP", true, 
				StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
		// Assert which employee added by employee id
		AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", true, 
				StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
		
	}
	
	 /**
	 * <P>
	 * Test adding  Employee with SSN Missing for Assisted Ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Compensation Item</LI>
	 * <LI> Null SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Add Work location to Worker</LI>
	 * <LI>Set SSN to Null and Save Employee</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is not added to PSP
	 * <LI>Verify that Employee is not added to PSP
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=false ) //to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddEmployeeNoSSN() throws Exception{
		
		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		worker = addWorkLocationToWorker(worker, "Discovery Drive", "MV");
		worker.setWorkerTaxIdNumber(null);
		worker = sWorkerService.saveWorker(worker);
		
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
			
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));
			// Assert which employee added by employee id
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
			
		}
		

	}
	
	 /**
	 * <P>
	 * Test adding  Employee with Blank SSN Assisted Ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Compensation Item</LI>
	 * <LI> Blank SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Add Worklocation to Worker</LI>
	 * <LI>Set SSN to Null and Save Employee</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is not added to PSP
	 * <LI>Verify that Employee is not added to PSP
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=false ) //to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddEmployeeBlankSSN() throws Exception{
		
		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		worker = addWorkLocationToWorker(worker, "Discovery Drive", "MV");
		worker.setWorkerTaxIdNumber("       ");
		worker = sWorkerService.saveWorker(worker);
		
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));
			// Assert which employee added by employee id
			AssertJUnit.assertEquals("Add Employee Call is sent to PSP", false, 
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
			
		}

	}
	
	 /**
	 * <P>
	 * Test adding  Employee with Formatted SSN for Assisted Ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Compensation Item</LI>
	 * <LI> Null SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Add Worklocation to Worker</LI>
	 * <LI>Set formatted SSN and Save Employee</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is not added to PSP
	 * <LI>Verify that Employee is not added to PSP
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=false ) //to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddEmployeeFormatedSSN() throws Exception{

		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		worker = addWorkLocationToWorker(worker, "Discovery Drive", "MV");
		worker.setWorkerTaxIdNumber("111-11-1111");
		worker = sWorkerService.saveWorker(worker);
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));
			
			// Assert which employee added by employee id
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
			
		}

	}
	
	 /**
	 * <P>
	 * Test adding  Employee with PayItem Missing for Assisted Ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Null Compensation Item</LI>
	 * <LI>SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Add Worklocation to Worker</LI>
	 * <LI>Set Null PayItem</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is not added to PSP
	 * <LI>Verify that Employee is not added to PSP
	 * </LI>
	 * </OL>
	 */
	@Test (enabled=false ) //to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddEmployeeNoPayItem() throws Exception{

		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		worker = addWorkLocationToWorker(worker, "Discovery Drive", "MV");
		worker.setCompensationItems(null);
		worker = sWorkerService.saveWorker(worker);
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));

			// Assert which employee added by employee id
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
		}

}
	
	 /**
	 * <P>
	 * Test adding  Employee with Home Address Missing for Assisted Ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Null Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Compensation Item</LI>
	 * <LI>SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Add Worklocation to Worker</LI>
	 * <LI>Set Null Home Address</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is not added to PSP
	 * <LI>Verify that Employee is not added to PSP
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=false ) //to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddEmployeeNoHomeAddress()throws Exception{
		
		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		worker = addWorkLocationToWorker(worker, "Discovery Drive", "MV");
		worker.setTaxResidenceAddress(null);
		worker = sWorkerService.saveWorker(worker);
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));

			// Assert which employee added by employee id
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
		}

		
	}
	
	 /**
	 * <P>
	 * Test adding  Employee with Worklocation Missing for Assisted Ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Null Compensation Item</LI>
	 * <LI>SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is not added to PSP
	 * <LI>Verify that Employee is not added to PSP
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=false ) //to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddEmployeeNoWorkLocation() throws Exception{
		
		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));

			// Assert which employee added by employee id
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
		}
		
	}
	
	 /**
	 * <P>
	 * Test adding  Employee with HireDate Missing for Assisted Ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>No Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Null Compensation Item</LI>
	 * <LI>SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is not added to PSP
	 * <LI>Verify that Employee is not added to PSP
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=false ) //to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddEmployeeNoStartDate() throws Exception{

		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		worker = addWorkLocationToWorker(worker, "Discovery Drive", "MV");
		worker.setHireDate(null);
		worker = sWorkerService.saveWorker(worker);
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));

			// Assert which employee added by employee id
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
		}

	}
	
	 /**
	 * <P>
	 * Test adding  Employee with Future Hiredate for Assisted Ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Future Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Compensation Item</LI>
	 * <LI>SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is not added to PSP
	 * <LI>Verify that Employee is not added to PSP
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=false ) //to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddEmployeeFutureStartDate() throws Exception{
		
		Calendar now = Calendar.getInstance();
		int year= now.get(Calendar.YEAR);
		now.set(Calendar.YEAR, year+1);
		java.util.Date date =now.getTime();
		
		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		worker = addWorkLocationToWorker(worker, "Discovery Drive", "MV");
		worker.setHireDate(date);
		worker = sWorkerService.saveWorker(worker);
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));

			// Assert which employee added by employee id
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
		}

		
	}
	 /**
	 * <P>
	 * Test adding  Employee with Payment Missing for Assisted Ready Employee 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Null Payment Method</LI>
	 * <LI>Compensation Item</LI>
	 * <LI>SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create a basic employee</LI>
	 * <LI>Add Pay Item to Worker</LI>
	 * <LI>Send Company Info to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is not added to PSP
	 * <LI>Verify that Employee is not added to PSP
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=false )//to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddEmployeeNoPaymentMethod() throws Exception{
		
		Worker worker = createBasicEmployee("Madan", "Mohan");
		worker = addPayItemToWorker(worker);
		worker = addWorkLocationToWorker(worker, "Discovery Drive", "MV");
		worker.setPaycheckDistributionMethod(null);
		worker = sWorkerService.saveWorker(worker);
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));

			// Assert which employee added by employee id
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity));
		}
		
	}
		
	

	@Test(enabled=false,expectedExceptions = ValidationException.class)
	public void testAddSameEmployeeMultipleTimes(){
		// implement this when "adding employees after the activation activation process" is implemented.Current implementation
		// is batch send
		// same employee will be sent to PTS multiple times
	}
	
	@Test(enabled=false)
	public void testAddDuplicateEmployee(){
		// implement this when "adding employees after the activation activation process" is implemented.Current implementation
		// is batch send
		// same employee will be sent to PTS 2nd time
	}
	
	 /**
	 * <P>
	 * Test adding  Multiple Assisted Ready Employees 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Compensation Item</LI>
	 * <LI>SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create Multiple Assisted Ready Employees</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException Exception is thrown
	 * <LI>Verify that Employer is Added to PSP
	 * <LI>Verify that Employee is Added to PSP
	 * </LI>
	 * </OL>
	 */
	@Test(enabled=false)// to be fixed by Dev- SSN not saved with WorkerService.saveworker
	public void testAddMultipleEmployees() throws Exception{
		
		Worker worker1 = createBasicEmployee("Madan", "Mohan");
		worker1 = addPayItemToWorker(worker1);
		worker1 = addWorkLocationToWorker(worker1, "Discovery Drive", "MV");
		worker1.setWorkerTaxIdNumber("484848484");
		worker1 = sWorkerService.saveWorker(worker1);
		
		ObjectHandle workerUniqueId = worker1.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId);
		Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
		com.intuit.sb.payroll.entity.model.Worker workerEntity1 = sWorkerComponent.findWorkerByIdentifiable(identifiable);
		
		
		Worker worker2 = createBasicEmployee("Raja", "Ram");
		worker2 = addPayItemToWorker(worker2);
		worker2 = addWorkLocationToWorker(worker2, "Perl Street", "MV");
		worker2.setWorkerTaxIdNumber("484848484");
		worker2 = sWorkerService.saveWorker(worker2);
		
		ObjectHandle workerUniqueId2 = worker2.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId2);
		Identifiable identifiable2 = ObjectHandleMapper.mapToIdentifiable(workerUniqueId2);
		com.intuit.sb.payroll.entity.model.Worker workerEntity2 = sWorkerComponent.findWorkerByIdentifiable(identifiable2);
		
		
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
				StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));
		AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", true, 
				StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity1));
		AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", true, 
				StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity2));
		
	}
	
	 /**
	 * <P>
	 * Test adding  Multiple Some Assisted Ready and Some Non Assisted Ready Employees 
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through WorkerService
	 * </P>
	 * <OL>
	 * <LI>Home Address</LI>
	 * <LI>Work location</LI>
	 * <LI>Hire Date</LI>
	 * <LI>Payment Method</LI>
	 * <LI>Compensation Item</LI>
	 * <LI>SSN</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Assisted Employer with random EIN - SETUP</LI>
	 * <LI>Create Assisted Ready Employee</LI>
	 * <LI>Create Non Assisted Ready Employee</LI>
	 * <LI>send Information to PSP</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException is thrown
	 * <LI>Verify that Company is not added to PTS
	 * <LI>Verify that Employee is not added to PTS
	 * 
	 * </LI>
	 * </OL>
	 */
	@Test (enabled=false ) //to be fixed by Dev- not an error but From testability perspective ValidationException is to be thrown
	public void testAddMultipleMixedEmployees() throws Exception{
		
		createBasicEmployee("Benjemin", "Button");
		
		Worker worker1  = createBasicEmployee("Madan", "Mohan"); 
		worker1 = addPayItemToWorker(worker1);
		
		ObjectHandle workerUniqueId1 = worker1.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId1);
		Identifiable identifiable1 = ObjectHandleMapper.mapToIdentifiable(workerUniqueId1);
		com.intuit.sb.payroll.entity.model.Worker workerEntity1 = sWorkerComponent.findWorkerByIdentifiable(identifiable1);
				
		Worker worker2  = createBasicEmployee("Ray", "Birklid"); 
		worker2 = addPayItemToWorker(worker2);
		worker2 = addWorkLocationToWorker(worker2, "Discovery Drive", "MV");
		worker2.setWorkerTaxIdNumber("484848484");
		worker2 = sWorkerService.saveWorker(worker2);
		
		ObjectHandle workerUniqueId2 = worker2.getObjectHandle();
		CommonValidators.validateObjectHandle(workerUniqueId2);
		Identifiable identifiable2 = ObjectHandleMapper.mapToIdentifiable(workerUniqueId2);
		com.intuit.sb.payroll.entity.model.Worker workerEntity2 = sWorkerComponent.findWorkerByIdentifiable(identifiable2);
		
		try{
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
			Assert.fail("Expected ValidationException is not thrown");
		}
		catch(ValidationException ve){
			AssertJUnit.assertEquals("Add Company Call is sent to PSP", false,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()));
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false, 
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity1));
			AssertJUnit.assertEquals("Add Employee Call is  sent to PSP", false, 
					StatusUpdateHandlerUtil.isEntityAddedToPTS(workerEntity2));
		}

	}

}
