package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;
import com.intuit.sb.payroll.services.wages.WageService;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;

/**
 * Base class for Compensation Tests.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public abstract class CompensationTestBase extends WorkerServiceTestBase {
	protected static WageService sWageService;

	@BeforeClass
	public static void initializeCompensationTestBaseClass()
	{
		sWageService = qaBootstrap.getEjbFromClass(WageService.class);
	}
	
	protected SaveCompensationItemRequest mapToSaveCompensationItemRequestObject(
			SaveCompensationItemOperation operation, CompensationItem compensationItem) {
		return new SaveCompensationItemRequest(compensationItem, operation);
	}

	protected CompensationItem setUpReimbursementWage(boolean fixedRate, boolean taxable) {

		// Create Tips Compensation Item
		CompensationItem reimbursementCompItem = new CompensationItem();

		reimbursementCompItem.setFixedRate(fixedRate);
		if (taxable) {
			reimbursementCompItem.setCompensationType(CompensationType.Reimbursement);
		} else {
			reimbursementCompItem.setCompensationType(CompensationType.Reimbursement);
		}
		if (fixedRate) {
			reimbursementCompItem.setName("Fixed Reimbursement");
			reimbursementCompItem.setRate(new BigDecimal("122.50"));
		} else {
			reimbursementCompItem.setName("Varied Reimbursement");
			reimbursementCompItem.setRate(null);
		}
		return reimbursementCompItem;

	}

	/**
	 * This function will set up Hourly Wage Item with Double Time and/or
	 * Overtime depending on the value of the String passed
	 * 
	 * @return
	 */
	protected List<CompensationItem> setUpHourlyWageWithExtraTime(String extraTimeType) {
		// SETUP
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		// Create Hourly Compensation Item
		CompensationItem hourlyCompensationItem = new CompensationItem();
		hourlyCompensationItem.setCompensationType(CompensationType.Hourly);
		hourlyCompensationItem.setRate(new BigDecimal("18.00"));
		hourlyCompensationItem.setName("Hourly");
		hourlyCompensationItem.setUnits(new Double(10));
		hourlyCompensationItem.setFixedRate(true);
		// Create Overtime Compensation Item
		CompensationItem overTimeItem = new CompensationItem();
		overTimeItem.setCompensationType(CompensationType.Overtime);
		overTimeItem.setRate(new BigDecimal("1.5"));
		overTimeItem.setName("Overtime");
		overTimeItem.setFixedRate(true);
		// Create Double Compensation Item
		CompensationItem doubleTime = new CompensationItem();
		doubleTime.setCompensationType(CompensationType.Doubletime);
		doubleTime.setRate(new BigDecimal("2"));
		doubleTime.setName("Doubletime");
		doubleTime.setFixedRate(true);
		// Link Overtime and Double Time to CompensationItem()
		// Add Hourly to the List
		compensationItemList.add(hourlyCompensationItem);
		// if extraTimeType = "Both", add both overtime and DoubleTime to the
		// list and link it to Hourly WageItem
		if (extraTimeType.equalsIgnoreCase("Both")) {
			overTimeItem.setLinkedCompensationItem(hourlyCompensationItem);
			doubleTime.setLinkedCompensationItem(hourlyCompensationItem);
			compensationItemList.add(overTimeItem);
			compensationItemList.add(doubleTime);
		}
		// if extraTimeType = "Overtime", add overtime to the list and link it
		// to Hourly WageItem
		if (extraTimeType.equalsIgnoreCase("Overtime")) {
			overTimeItem.setLinkedCompensationItem(hourlyCompensationItem);
			compensationItemList.add(overTimeItem);
		}
		// if extraTimeType = "DoubleTime", add overtime to the list and link it
		// to Hourly WageItem
		if (extraTimeType.equalsIgnoreCase("Doubletime")) {
			doubleTime.setLinkedCompensationItem(hourlyCompensationItem);
			compensationItemList.add(doubleTime);
		}

		return compensationItemList;
	}

	/**
	 * This function will set up ComissionWage Item
	 * 
	 * @return
	 */
	protected CompensationItem setUpComissionWage(boolean fixedRate) {

		// Create Tips Compensation Item
		CompensationItem comissionCompItem = new CompensationItem();

		comissionCompItem.setFixedRate(fixedRate);
		comissionCompItem.setCompensationType(CompensationType.Commission);

		if (fixedRate) {
			comissionCompItem.setRate(new BigDecimal("122.50"));
			comissionCompItem.setName("Fixed Comission");
		} else {
			comissionCompItem.setRate(null);
			comissionCompItem.setName("Varied Comission");
		}
		return comissionCompItem;
	}

	/**
	 * This function will set up Tips Wage Item
	 * 
	 * @return
	 */
	protected CompensationItem setUpTipsWage(boolean cash, boolean fixedRate) {

		// Create Tips Compensation Item
		CompensationItem tipsCompItem = new CompensationItem();

		tipsCompItem.setFixedRate(fixedRate);
		if (cash) {
			tipsCompItem.setCompensationType(CompensationType.TipsCash);
		} else {
			tipsCompItem.setCompensationType(CompensationType.TipsPaycheck);
		}
		if (fixedRate) {
			tipsCompItem.setName("Fixed Tips");
			tipsCompItem.setRate(new BigDecimal("122.50"));
		} else {
			tipsCompItem.setName("Varied Tips");
			tipsCompItem.setRate(null);
		}
		return tipsCompItem;
	}

	/**
	 * This function will set up Piecework Wage Item
	 * 
	 * @return
	 */
	protected List<CompensationItem> setUpSalaryWage() {
		// SETUP
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		// Create Hourly Compensation Item
		CompensationItem salaryCompItem = new CompensationItem();
		salaryCompItem.setCompensationType(CompensationType.Salary);
		salaryCompItem.setRate(new BigDecimal("1200.50"));
		salaryCompItem.setName("Salary");
		salaryCompItem.setFixedRate(true);
		compensationItemList.add(salaryCompItem);

		return compensationItemList;
	}

	/**
	 * This function will set up PieceWork Wage Item
	 * 
	 * @return
	 */
	protected List<CompensationItem> setUpPieceworkWage() {
		// SETUP
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		// Create Hourly Compensation Item
		CompensationItem pieceworkWage = new CompensationItem();
		pieceworkWage.setCompensationType(CompensationType.Piecework);
		pieceworkWage.setRate(new BigDecimal("12.50"));
		pieceworkWage.setName("Piecework");
		pieceworkWage.setFixedRate(true);
		compensationItemList.add(pieceworkWage);

		return compensationItemList;
	}

	protected CompensationItem[] addWageItem(CompensationType type, boolean fixed, String name, BigDecimal rate)
			throws Exception {

		List<CompensationItem> compensationItems = new ArrayList<CompensationItem>();

		CompensationItem wageitem = new CompensationItem();
		wageitem.setCompensationType(type);
		wageitem.setRate(rate);
		wageitem.setName(name);
		wageitem.setFixedRate(fixed);
		compensationItems.add(wageitem);
		CompensationItem[] wageItems = new CompensationItem[compensationItems.size()];
		wageItems = compensationItems.toArray(wageItems);
		SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[wageItems.length];
		for (int i = 0; i < compensationItemRequests.length; i++) {
			compensationItemRequests[i] = new SaveCompensationItemRequest(wageItems[i], (null == wageItems[i]
					.getObjectHandle()) ? SaveCompensationItemOperation.ADD : SaveCompensationItemOperation.UPDATE);
		}
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, compensationItemRequests);
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		return updatedWageItems;

	}

	protected void validateWageItem(CompensationItem[] compensationItem) throws Exception {

		List<CompensationItem> compensationItems = Arrays.asList(compensationItem);

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItems));

	}

	/**
	 * This function will set up Bonus Wage Item
	 * 
	 * @return
	 */
	protected List<CompensationItem> setUpBonusWage(boolean fixedRate) {
		// SETUP
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		// Create Hourly Compensation Item
		CompensationItem bonus = new CompensationItem();
		bonus.setCompensationType(CompensationType.Bonus);
		bonus.setFixedRate(fixedRate);

		if (fixedRate) {
			bonus.setName("Fixed Bonus");
			bonus.setRate(new BigDecimal("122.50"));
		} else {
			bonus.setName("Varied Bonus");
			bonus.setRate(null);
		}
		compensationItemList.add(bonus);

		return compensationItemList;
	}

	protected List<CompensationItem> modifyCompensationRate(List<CompensationItem> compensationItem, boolean variable,
			BigDecimal rate) {
		List<CompensationItem> modifiedCompensatioItem = compensationItem;

		for (CompensationItem comp : modifiedCompensatioItem) {
			// Always set the rate
			comp.setRate(rate);
//			if (variable) {
//				comp.setRate(null);
//			}
//			if (rate != null) {
//				comp.setRate(rate);
//			}
		}

		return compensationItem;

	}

}
