package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * Tests Piecework handling in the CompensationService.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class PieceworkCompensationTest extends CompensationTestBase {

	/**
	 * <P>
	 * Test Adding Piecework Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Piecework Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Piecework Wage Item is Saved</LI>
	 */
	@Test
	public void testAddPieceworkWageItem() throws Exception {
		List<CompensationItem> compensationItemList = setUpPieceworkWage();
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));
	}

	/**
	 * <P>
	 * Test Deleting Piecework Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Piecework Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved Piecework</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Piecework Wage Item is Saved</LI>
	 * 
	 */
	@Test
	public void testRemovePieceworkWageItem() throws Exception {
		List<CompensationItem> compensationItemList = setUpPieceworkWage();
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Piecework)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Piecework Wage Item not deleted");
		}
	}

	/**
	 * <P>
	 * Test Updating Piecework Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Piecework Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Piecework Wage Item is Saved</LI>
	 * 
	 */
	@Test
	public void testUpdatePieceworkWageItem() throws Exception {
		List<CompensationItem> compensationItemList = setUpPieceworkWage();
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update the Piecework wage rate to a new value
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Piecework)) {
				BigDecimal rate = compItem.getRate();
				if (rate.equals(new BigDecimal("12.50"))) {
					compItem.setRate(new BigDecimal("15.97"));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));
				} else
					AssertJUnit.fail("Piecework Wage Item not saved as expected");
			}
		}
		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Piecework has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Piecework.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(new BigDecimal("15.97"))) {
					AssertJUnit.fail("Piecework Wage Item not updated");
				}
			} else
				AssertJUnit.fail("Piecework Comp Item not saved");
		}
	}

	/**
	 * <P>
	 * Test Updating Piecework Wage Item to zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Piecework Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Piecework Wage Item is Saved</LI>
	 * 
	 */

	@Test
	public void testUpdatePieceworkWageItemAsZero() throws Exception {
		List<CompensationItem> compensationItemList = setUpPieceworkWage();

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update the Piecework wage rate to a new value
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("12.50");
		// set to 2 decimal places
		bigDecimal = bigDecimal.setScale(2);
		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Piecework)) {
				BigDecimal rate = compItem.getRate();
				if (rate.equals(new BigDecimal("12.50"))) {
					compItem.setRate(new BigDecimal("0"));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));
				} else
					AssertJUnit.fail("Piecework Wage Item not saved as expected");
			}
		}
		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Piecework has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Piecework.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				BigDecimal zero = new BigDecimal("0.00");
				// set to 2 decimal places
				zero = zero.setScale(2);
				if (!value.equals(zero)) {
					AssertJUnit.fail("Piecework Wage Item not updated");
				}
			} else
				AssertJUnit.fail("Piecework Comp Item not saved");
		}

	}

	/**
	 * <P>
	 * Test creating duplicate PieceWork Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 PieceWorkwage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddDuplicatePieceworkWageWithSameName() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpPieceworkWage();
		// create duplicate Piecework wage with same name and rate
		CompensationItem pieceworkCompItem = new CompensationItem();
		pieceworkCompItem.setCompensationType(CompensationType.Piecework);
		pieceworkCompItem.setRate(new BigDecimal("1200.43"));
		pieceworkCompItem.setName("Piecework");
		pieceworkCompItem.setFixedRate(true);
		compensationItemList.add(pieceworkCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Piecework.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);
	}

	/**
	 * <P>
	 * Test creating duplicate Piecework Wage Item with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Piecework wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddDuplicatePieceworkWageWithUniqueNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpPieceworkWage();
		// create duplicate Piecework wage with same name and rate
		CompensationItem pieceworkCompItem = new CompensationItem();
		pieceworkCompItem.setCompensationType(CompensationType.Piecework);
		pieceworkCompItem.setRate(new BigDecimal("1200.43"));
		pieceworkCompItem.setName("Piecework 45");
		pieceworkCompItem.setFixedRate(true);
		compensationItemList.add(pieceworkCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Piecework.equals(compItem.getCompensationType())) {
				if (compItem.getName().equals("Piecework 45")) {
					count++;
				} else if (compItem.getName().equals("Piecework")) {
					count++;
				}
			}
		}
		AssertJUnit.assertEquals(2, count);
	}

	/**
	 * <P>
	 * Test updating duplicate Piecework Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Piecework wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update one Piecework item with different rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Verify the correct Piecework wage has been updated</LI>
	 * 
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify updated and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testUpdateDuplicatePieceworkWageWithSameNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpPieceworkWage();
		// create duplicate Piecework wage with same name and rate
		CompensationItem pieceworkCompItem = new CompensationItem();
		pieceworkCompItem.setCompensationType(CompensationType.Piecework);
		pieceworkCompItem.setRate(new BigDecimal("1200.43"));
		pieceworkCompItem.setName("Piecework");
		pieceworkCompItem.setFixedRate(true);
		compensationItemList.add(pieceworkCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		Long id = null;
		// Update the rate of the first Piecework wage item returned by the
		// Service
		updatedWageItems[0].setRate(new BigDecimal("3000.01"));
		id = updatedWageItems[0].getObjectHandle().getId();
		compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
				updatedWageItems[0]));

		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that correct Piecework has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Piecework.equals(cItem.getCompensationType())) {
				if (cItem.getObjectHandle().getId() == id) {
					BigDecimal value = cItem.getRate();
					if (!value.equals(new BigDecimal("3000.01"))) {
						AssertJUnit.fail("Piecework Wage Item not updated");
					}
				}
			} else {
				AssertJUnit.fail("Piecework Comp Item not saved");
			}
		}
	}

	/**
	 * <P>
	 * Test updating duplicate Piecework Wage Item with unique names to
	 * Piecework wage items with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Piecework wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update one Piecework item to the same name as the other</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Verify the correct Piecework wage has been updated</LI>
	 * 
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify updated and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testUpdateDuplicatePieceworkWageWithUniqueNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpPieceworkWage();
		// create duplicate Piecework wage with same name and rate
		CompensationItem pieceworkCompItem = new CompensationItem();
		pieceworkCompItem.setCompensationType(CompensationType.Piecework);
		pieceworkCompItem.setRate(new BigDecimal("1200.43"));
		pieceworkCompItem.setName("Piecework 2");
		pieceworkCompItem.setFixedRate(true);
		compensationItemList.add(pieceworkCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Setup for Updates
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		Long id = null;
		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getName().equals("Piecework 2")) {
				id = compItem.getObjectHandle().getId();
				compItem.setName("Piecework");
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			}
		}
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else {
			AssertJUnit.fail("No Compensation Items to Update");
		}

		// Verify that correct Piecework has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Piecework.equals(cItem.getCompensationType())) {
				if (cItem.getObjectHandle().getId() == id) {
					String name = cItem.getName();
					if (!name.equals("Piecework")) {
						AssertJUnit.fail("Piecework Wage Item not updated");
					}
				}
			} else {
				AssertJUnit.fail("Piecework Comp Item not saved");
			}
		}
	}

	/**
	 * <P>
	 * Test deleting Piecework Wage Item which has the same name as another
	 * Piecework wage item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Piecework wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Delete one Piecework wage item</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Verify the correct Piecework wage has been delete/LI>
	 * 
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify that expected Piecework wage has been deleted</LI>
	 * 
	 */
	@Test
	public void testRemoveDuplicatePieceworkWageWithSameNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpPieceworkWage();
		// create duplicate Piecework wage with same name and rate
		CompensationItem pieceworkCompItem = new CompensationItem();
		pieceworkCompItem.setCompensationType(CompensationType.Piecework);
		pieceworkCompItem.setRate(new BigDecimal("1200.43"));
		pieceworkCompItem.setName("Piecework");
		pieceworkCompItem.setFixedRate(true);
		compensationItemList.add(pieceworkCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		Long id = null;
		// Delete first Piecework wage item returned by the Service
		id = updatedWageItems[0].getObjectHandle().getId();
		compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE,
				updatedWageItems[0]));

		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that correct Piecework has been deleted
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Piecework.equals(cItem.getCompensationType())) {
				if (cItem.getObjectHandle().getId() == id) {
					AssertJUnit.fail("Piecework Wage Item not deleted");
				}
			} else {
				AssertJUnit.fail("Piecework Comp Item not saved");
			}
		}
	}
}
