package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * Tests Tips handling in the CompensationService.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class TipsCompensationTest extends CompensationTestBase {

	// Happy path
	/**
	 * <P>
	 * Test Adding Tips Wage Item with varied rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Tips Wage with varied rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Tips Wage Item is Saved</LI>
	 */
	@Test
	public void testAddVariedCashTips() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(true, false); // set 2nd
																// parameter to
																// true to make
																// it fixed
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<CompensationItem> modifiedWageItems = modifyCompensationRate(compensationItemList, true, null);

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, modifiedWageItems));

	}

	// test to verify the add fix cash tips
	/**
	 * <P>
	 * Test Adding CashTips Wage Item with fixed rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up CashTips Wage with fixed rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item is Saved</LI>
	 */
	@Test
	public void testAddFixedCashTips() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(true, true);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test Adding PaycheckTips Wage Item with varied rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up PaycheckTips Wage with varied rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item is Saved</LI>
	 */
	@Test
	public void testAddVariedPaycheckTips() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(false, false);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<CompensationItem> modifiedWageItems = modifyCompensationRate(compensationItemList, true, null);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, modifiedWageItems));

	}

	// test to verify the add fixed paycheck tips
	/**
	 * <P>
	 * Test Adding PaycheckTips Wage Item with fixed rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up PaycheckTips Wage with fixed rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item is Saved</LI>
	 */
	@Test
	public void testAddTipsWithNegativeFixedPaycheckRate() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(false, true);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));
	}

	// test to add verify the negative fixed cash rate
	/**
	 * <P>
	 * Test Adding CashTips Wage Item with fixed rate as negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up CashTips Wage with fixed rate as negative rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item gives validation exception and not Saved</LI>
	 */
	@Test
	public void testAddNegativeFixedCashTips() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;

		CompensationItem[] initialWageItems = sWageService.getCompensationItems(sWorkerId);

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(true, true);
		wageItem.setRate(new BigDecimal("-20"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		try {
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.ADD, wageItems));

		} catch (ValidationException ve) {
			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		validateWageItem(initialWageItems);

	}

	/**
	 * <P>
	 * Test Adding PaycheckTips Wage Item with varied rate as negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up PaycheckTips Wage with varied rate as negative rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item gives validation exception and not Saved</LI>
	 */
	@Test
	public void testAddNegativeVariedPaycheckTips() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;

		CompensationItem[] initialWageItems = sWageService.getCompensationItems(sWorkerId);

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem wageItem = setUpTipsWage(false, false);
		wageItem.setRate(new BigDecimal("-20"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		try {
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.ADD, wageItems));
		} catch (ValidationException ve) {
			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		validateWageItem(initialWageItems);

	}

	/**
	 * <P>
	 * Test Adding PaycheckTips Wage Item with fixed rate as negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up PaycheckTips Wage with fixed rate as negative rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item gives validation exception and not Saved</LI>
	 */
	@Test
	public void testAddNegativeFixedPaycheckTips() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;

		CompensationItem[] initialWageItems = sWageService.getCompensationItems(sWorkerId);

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem wageItem = setUpTipsWage(false, true);
		wageItem.setRate(new BigDecimal("-20"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		try {
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.ADD, wageItems));
		} catch (ValidationException ve) {
			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		validateWageItem(initialWageItems);

	}

	// Zero Rate
	/**
	 * <P>
	 * Test Adding CashTips Wage Item with varied rate as zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up CashTips Wage with varied rate as zero rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item gives validation exception and not Saved</LI>
	 */
	@Test
	public void testAddVariedZeroCashTips() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(true, false);
		wageItem.setRate(new BigDecimal("0"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<CompensationItem> modifiedWageItems = modifyCompensationRate(compensationItemList, true, new BigDecimal("0"));
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, modifiedWageItems));

	}

	/**
	 * <P>
	 * Test Adding CashTips Wage Item with fixed rate as zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up CashTips Wage with fixed rate as zero rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item gives validation exception and not Saved</LI>
	 */
	@Test
	public void testAddFixedZeroCashTips() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(true, true);
		wageItem.setRate(new BigDecimal("0"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test Adding PaycheckTips Wage Item with varied rate as zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up PaycheckTips Wage with varied rate as zero rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item gives validation exception and not Saved</LI>
	 */
	@Test
	public void testAddVariedZeroPaycheckTips() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(false, false);
		wageItem.setRate(new BigDecimal("0"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<CompensationItem> modifiedWageItems = modifyCompensationRate(compensationItemList, true, new BigDecimal("0"));

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, modifiedWageItems));

	}

	/**
	 * <P>
	 * Test Adding PaycheckTips Wage Item with fixed rate as negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up PaycheckTips Wage with fixed rate as negative rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item gives validation exception and not Saved</LI>
	 */
	@Test
	public void testAddFixedZeroPaycheckTips() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(false, true);
		wageItem.setRate(new BigDecimal("0"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	// Delete
	/**
	 * <P>
	 * Test Deleting VariedCashTip Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up VariedCashTip Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved VariedCashTip</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify VariedCashTip Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteVariedCashTip() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(true, false);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);

		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Tips Wage Item not deleted");
		}

	}

	/**
	 * <P>
	 * Test Deleting FixedCashTip Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedCashTip Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved FixedCashTip</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedCashTip Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteFixedCashTip() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(true, true);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Tips Wage Item not deleted");
		}

	}

	/**
	 * <P>
	 * Test Deleting VariedPaycheckTip Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up VariedPaycheckTip Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved FixedPaycheckTip</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify VariedPaycheckTip Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteVariedPaycheckTip() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(false, false);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Tips Wage Item not deleted");
		}

	}

	/**
	 * <P>
	 * Test Deleting FixedPaycheckTip Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedPaycheckTip Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved FixedPaycheckTip</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedPaycheckTip Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteFixedPaycheckTips() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(false, true);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Tips Wage Item not deleted");
		}

	}

	// Update
	/**
	 * <P>
	 * Test Updating varied CashTips Wage Item to fixed
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new fixed rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateCashTipsVariedToFixed() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update

		// Update the varied Tips wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Tips cannot have a rate");
				}
				compItem.setFixedRate(true);
				compItem.setRate(new BigDecimal("155.97"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Tips has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.TipsCash.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(new BigDecimal("155.97"))) {
					AssertJUnit.fail("Varied Wage Item not updated to Fixed");
				}
			} else
				AssertJUnit.fail("Tips Comp Item not saved");
		}
	}

	/**
	 * <P>
	 * Test Updating varied PaycheckTips Wage Item to fixed
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied PaycheckTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new fixed rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdatePaycheckVariedToFixed() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied tips wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Tips cannot have a rate");
				}
				compItem.setFixedRate(true);
				compItem.setRate(new BigDecimal("155.97"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		BigDecimal bigdecimal = new BigDecimal("155.97");
		bigdecimal = bigdecimal.setScale(2);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.TipsPaycheck.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(bigdecimal)) {
					AssertJUnit.fail("Varied Wage Item not updated to Fixed");
				}
			} else
				AssertJUnit.fail("Tips Comp Item not saved");
		}
	}

	/**
	 * <P>
	 * Test Updating fixed CashTips Wage Item to negative rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e -20</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedCashTipsWithNegativeRate() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, true);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {
				compItem.setRate(new BigDecimal("-155.97"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it

		try {
			if (compItemList.size() > 0) {

				SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
				newCompItems = compItemList.toArray(newCompItems);
				sWageService.saveCompensationItems(sWorkerId, newCompItems);

			} else {
				AssertJUnit.fail("No Compensation Items to Update");
			}
		} catch (ValidationException ve) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		modifyCompensationRate(Arrays.asList(updatedWageItems), false, new BigDecimal("122.50"));

		validateWageItem(updatedWageItems);
		// Cleanup the compensation item(s)
		updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		compItemList.clear();
		for (CompensationItem cItem : updatedWageItems) {
			compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, cItem));
		}
		SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
		newCompItems = compItemList.toArray(newCompItems);
		sWageService.saveCompensationItems(sWorkerId, newCompItems);
		getUserTransaction().commit();

	}

	/**
	 * <P>
	 * Test Updating varied CashTips Wage Item to negative rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e -20</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedCashTipsWithNegativeRate() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;
		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Tips cannot have a rate");
				}

				compItem.setRate(new BigDecimal("-155.97"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it

		try {
			if (compItemList.size() > 0) {

				SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
				newCompItems = compItemList.toArray(newCompItems);
				sWageService.saveCompensationItems(sWorkerId, newCompItems);

			} else {
				AssertJUnit.fail("No Compensation Items to Update");
			}
		} catch (ValidationException ve) {
			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		modifyCompensationRate(Arrays.asList(updatedWageItems), true, null);
		validateWageItem(updatedWageItems);
		// Cleanup the compensation item(s)
		updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		compItemList.clear();
		for (CompensationItem cItem : updatedWageItems) {
			compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, cItem));
		}
		SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
		newCompItems = compItemList.toArray(newCompItems);
		sWageService.saveCompensationItems(sWorkerId, newCompItems);
		getUserTransaction().commit();

	}

	/**
	 * <P>
	 * Test Updating fixed CashTips Wage Item to zero rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedCashTipsWithZeroRate() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, true);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {

				compItem.setRate(new BigDecimal("0"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

	}

	/**
	 * <P>
	 * Test Updating varied CashTips Wage Item to zero rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateCashVariedTipsWithZero() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Tips cannot have a rate");
				}

				compItem.setRate(new BigDecimal("0"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Tips Items to Update");

		modifyCompensationRate(Arrays.asList(updatedWageItems), true, null);

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test Updating varied Paycheck Wage Item to zero rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied Paycheck Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Paycheck Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdatePaycheckVaried() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Tips cannot have a rate");
				}

				compItem.setRate(new BigDecimal("13.2"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Tips Items to Update");

		modifyCompensationRate(Arrays.asList(updatedWageItems), true, null);

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test Updating varied CashTips Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateCashVariedTips() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Tips cannot have a rate");
				}

				compItem.setRate(new BigDecimal("14.22"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Tips Items to Update");

		modifyCompensationRate(Arrays.asList(updatedWageItems), true, null);

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test Updating Fixed Paycheck Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdatePaycheckFixed() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, true);
		compensationItemList.add(tipsComp);
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {

				compItem.setRate(new BigDecimal("34.5"));

				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Tips Items to Update");

		CompensationItem[] newUpdatedWageItems = sWageService.getCompensationItems(sWorkerId);
		modifyCompensationRate(Arrays.asList(updatedWageItems), false, new BigDecimal("34.5"));

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(newUpdatedWageItems, Arrays
				.asList(updatedWageItems)));

	}

	/**
	 * <P>
	 * Test Updating varied CashTips Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateCashFixedTips() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, true);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {

				compItem.setRate(new BigDecimal("12.5"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Tips Items to Update");

		CompensationItem[] newUpdatedWageItems = sWageService.getCompensationItems(sWorkerId);

		modifyCompensationRate(Arrays.asList(updatedWageItems), false, new BigDecimal("12.5"));

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(newUpdatedWageItems, Arrays
				.asList(updatedWageItems)));

	}

	/**
	 * <P>
	 * Test Updating fixed PaycheckTips Wage Item to negative rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed PaycheckTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e -20</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedPaycheckTipsWithNegativeRate() throws Exception {
		getUserTransaction().commit();
		// Setup

		boolean validationExceptionThrown = false;
		// CompensationItem[] compenItem =
		// addWageItem(CompensationType.TipsPaycheck, true, "Fixed Tips", new
		// BigDecimal(20));

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, true);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				compItem.setRate(new BigDecimal("-155.00"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		try {

			// If the list has values , then call operation to update it
			if (compItemList.size() > 0) {
				SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
				newCompItems = compItemList.toArray(newCompItems);

				sWageService.saveCompensationItems(sWorkerId, newCompItems);
			} else
				AssertJUnit.fail("No Compensation Items to Update");
		} catch (ValidationException ve) {
			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		modifyCompensationRate(Arrays.asList(updatedWageItems), false, new BigDecimal("122.50"));
		validateWageItem(updatedWageItems);
		// Cleanup the compensation item(s)
		updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		compItemList.clear();
		for (CompensationItem cItem : updatedWageItems) {
			compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, cItem));
		}
		SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
		newCompItems = compItemList.toArray(newCompItems);
		sWageService.saveCompensationItems(sWorkerId, newCompItems);
		getUserTransaction().commit();

	}

	/**
	 * <P>
	 * Test Updating varied PaycheckTips Wage Item to negative rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied PaycheckTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e -20</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedPaycheckTipsWithNegativeRate() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;
		// CompensationItem[] compenItem =
		// addWageItem(CompensationType.TipsPaycheck, false, "Varied Tips", new
		// BigDecimal(20));

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Tips cannot have a rate");
				}

				compItem.setRate(new BigDecimal("-155.00"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		try {
			// If the list has values , then call operation to update it
			if (compItemList.size() > 0) {
				SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
				newCompItems = compItemList.toArray(newCompItems);

				sWageService.saveCompensationItems(sWorkerId, newCompItems);
			} else
				AssertJUnit.fail("No Compensation Items to Update");
		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		modifyCompensationRate(Arrays.asList(updatedWageItems), true, null);
		validateWageItem(updatedWageItems);
		// Cleanup the compensation item(s)
		updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		compItemList.clear();
		for (CompensationItem cItem : updatedWageItems) {
			compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, cItem));
		}
		SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
		newCompItems = compItemList.toArray(newCompItems);
		sWageService.saveCompensationItems(sWorkerId, newCompItems);
		getUserTransaction().commit();

	}

	/**
	 * <P>
	 * Test Updating fixed PaycheckTips Wage Item to zero rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed PaycheckTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedPaycheckTipsWithZero() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, true);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal1 = new BigDecimal("0.00");
		bigDecimal1 = bigDecimal1.setScale(2);

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				compItem.setRate(bigDecimal1);
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		BigDecimal bigDecimal = new BigDecimal("0.00");
		bigDecimal = bigDecimal.setScale(2);

		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.TipsPaycheck.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(bigDecimal)) {
					AssertJUnit.fail("Wage Item not updated ");
				}
			} else
				AssertJUnit.fail("Tips Comp Item not saved");
		}

	}

	/**
	 * <P>
	 * Test Updating varied PaycheckTips Wage Item to zero rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied PaycheckTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedPaycheckTipsWithZeroRate() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Tips cannot have a rate");
				}

				compItem.setRate(new BigDecimal("0.00"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Tips has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		verifyCompensationItems(updatedWageItems, Arrays.asList(workerWageItemsNew));
	}

	/**
	 * <P>
	 * Test Updating fixed PaycheckTips Wage Item to varied rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed PaycheckTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to varied rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify PaycheckTips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedPayCheckTipsToVaried() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, true);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				compItem.setFixedRate(false);
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.TipsPaycheck.equals(cItem.getCompensationType())) {
				boolean fixed = cItem.isFixedRate();
				if (fixed) {
					AssertJUnit.fail("Varied Wage Item not updated to Fixed");
				}
			} else
				AssertJUnit.fail("Tips Comp Item not saved");
		}

	}

	/**
	 * <P>
	 * Test Updating fixed CashTips Wage Item to varied rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to varied rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateixedCashTipsWageItemToVaried() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, true);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {
				compItem.setFixedRate(false);
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.TipsCash.equals(cItem.getCompensationType())) {
				boolean fixed = cItem.isFixedRate();
				if (fixed) {
					AssertJUnit.fail("Varied Wage Item not updated to Fixed");
				}
			} else
				AssertJUnit.fail("Tips Comp Item not saved");
		}

	}

	/**
	 * <P>
	 * Test Updating fixed PaycheckTips Wage Item to CashTips
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed PaycheckTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item to CashTips</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Tips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedPayCheckTipsToCash() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, true);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				compItem.setCompensationType(CompensationType.TipsCash);
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {

			if (!CompensationType.TipsCash.equals(cItem.getCompensationType())) {
				AssertJUnit.fail("Tips type do not match");
			}
		}
	}

	/**
	 * <P>
	 * Test Updating varied PaycheckTips Wage Item to CashTips
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied PaycheckTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item to CashTips</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Tips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedPayCheckTipsToCash() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(false, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsPaycheck)) {
				compItem.setCompensationType(CompensationType.TipsCash);
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {

			if (!CompensationType.TipsCash.equals(cItem.getCompensationType())) {
				AssertJUnit.fail("Tips type do not match");
			}
		}

	}

	/**
	 * <P>
	 * Test Updating fixed CashTips Wage Item to PaycheckTips
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item to PaycheckTips</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Tips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedCashTipsToPayCheck() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, true);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getName().equals("Fixed Tips")) {
				compItem.setCompensationType(CompensationType.TipsPaycheck);
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {

			if (!CompensationType.TipsPaycheck.equals(cItem.getCompensationType())) {
				AssertJUnit.fail("Tips type do not match");
			}
		}

	}

	/**
	 * <P>
	 * Test Updating varied CashTips Wage Item to PaycheckTips
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up varied CashTips Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item to PaycheckTips</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Tips Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedTipsFromCashToPayCheck() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem tipsComp = setUpTipsWage(true, false);
		compensationItemList.add(tipsComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.TipsCash)) {
				compItem.setCompensationType(CompensationType.TipsPaycheck);
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {

			if (!CompensationType.TipsPaycheck.equals(cItem.getCompensationType())) {
				AssertJUnit.fail("Tips type do not match");
			}
		}

	}

	/**
	 * <P>
	 * Test creating duplicate FixedCashTips Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 FixedCashTips wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedCashTips wage item throws validation exception</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateFixedCashTipsWithSameName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem1 = new CompensationItem();
		wageItem1.setFixedRate(true);
		wageItem1.setCompensationType(CompensationType.TipsCash);
		wageItem1.setRate(new BigDecimal("211"));
		wageItem1.setName("Tips");

		CompensationItem wageItem2 = new CompensationItem();
		wageItem2.setFixedRate(true);
		wageItem2.setCompensationType(CompensationType.TipsCash);
		wageItem2.setRate(new BigDecimal("112"));
		wageItem2.setName("Tips");

		compensationItemList.add(wageItem1);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.TipsCash.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate FixedPaychekTips Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 FixedCashTips wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedCashTips wage item throws validation exception</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateFixedPaycheckTipsWithSameName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem1 = new CompensationItem();
		wageItem1.setFixedRate(true);
		wageItem1.setCompensationType(CompensationType.TipsPaycheck);
		wageItem1.setRate(new BigDecimal("211"));
		wageItem1.setName("Tips");

		CompensationItem wageItem2 = new CompensationItem();
		wageItem2.setFixedRate(true);
		wageItem2.setCompensationType(CompensationType.TipsPaycheck);
		wageItem2.setRate(new BigDecimal("112"));
		wageItem2.setName("Tips");

		compensationItemList.add(wageItem1);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.TipsPaycheck.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate VariedCashTips Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 VariedCashTips wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify VariedCashTips wage item throws validation exception</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateVariedCashTipsWithSameName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem1 = new CompensationItem();
		wageItem1.setFixedRate(false);
		wageItem1.setCompensationType(CompensationType.TipsCash);
		wageItem1.setRate(new BigDecimal("211"));
		wageItem1.setName("Tips");

		CompensationItem wageItem2 = new CompensationItem();
		wageItem2.setFixedRate(false);
		wageItem2.setCompensationType(CompensationType.TipsCash);
		wageItem2.setRate(new BigDecimal("112"));
		wageItem2.setName("Tips");

		compensationItemList.add(wageItem1);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.TipsCash.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate VariedPaycheckTips Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 VariedPaycheckTips wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify VariedPaycheckTips wage item throws validation exception</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateVariedPaycheckTipsWithSameName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem1 = new CompensationItem();
		wageItem1.setFixedRate(false);
		wageItem1.setCompensationType(CompensationType.TipsPaycheck);
		wageItem1.setRate(new BigDecimal("211"));
		wageItem1.setName("Tips");

		CompensationItem wageItem2 = new CompensationItem();
		wageItem2.setFixedRate(false);
		wageItem2.setCompensationType(CompensationType.TipsPaycheck);
		wageItem2.setRate(new BigDecimal("112"));
		wageItem2.setName("Tips");

		compensationItemList.add(wageItem1);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.TipsPaycheck.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate FixedCashTips Wage Item with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 FixedCashTips wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateFixedCashTipsWithUniqueName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem tipsCompensation1 = new CompensationItem();
		tipsCompensation1.setRate(new BigDecimal("122.50"));
		tipsCompensation1.setFixedRate(true);
		tipsCompensation1.setCompensationType(CompensationType.TipsCash);
		tipsCompensation1.setName("Tips 1");

		CompensationItem tipsCompensation2 = new CompensationItem();
		tipsCompensation2.setRate(new BigDecimal("122.50"));
		tipsCompensation2.setFixedRate(true);
		tipsCompensation2.setCompensationType(CompensationType.TipsCash);
		tipsCompensation2.setName("Tips 2");

		compensationItemList.add(tipsCompensation1);
		compensationItemList.add(tipsCompensation2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.TipsCash.equals(compItem.getCompensationType())) {
				if (compItem.getName().equals("Tips 2")) {
					count++;
				} else if (compItem.getName().equals("Tips 1")) {
					count++;
				}
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate VariedCashTips Wage Item with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 VariedCashTips wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateVariedCashTipsWithUniqueName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem tipsCompensation1 = new CompensationItem();
		tipsCompensation1.setRate(new BigDecimal("122.50"));
		tipsCompensation1.setFixedRate(true);
		tipsCompensation1.setCompensationType(CompensationType.TipsCash);
		tipsCompensation1.setName("Tips 1");

		CompensationItem tipsCompensation2 = new CompensationItem();
		tipsCompensation2.setRate(new BigDecimal("122.50"));
		tipsCompensation2.setFixedRate(true);
		tipsCompensation2.setCompensationType(CompensationType.TipsCash);
		tipsCompensation2.setName("Tips 2");

		compensationItemList.add(tipsCompensation1);
		compensationItemList.add(tipsCompensation2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.TipsCash.equals(compItem.getCompensationType())) {
				if (compItem.getName().equals("Tips 2")) {
					count++;
				} else if (compItem.getName().equals("Tips 1")) {
					count++;
				}
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate FixedCashTips Wage Item with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 FixedCashTips wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateFixedPaycheckTipsWithUniqueName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem tipsCompensation1 = new CompensationItem();
		tipsCompensation1.setRate(new BigDecimal("122.50"));
		tipsCompensation1.setFixedRate(true);
		tipsCompensation1.setCompensationType(CompensationType.TipsPaycheck);
		tipsCompensation1.setName("Tips 1");

		CompensationItem tipsCompensation2 = new CompensationItem();
		tipsCompensation2.setRate(new BigDecimal("122.50"));
		tipsCompensation2.setFixedRate(true);
		tipsCompensation2.setCompensationType(CompensationType.TipsPaycheck);
		tipsCompensation2.setName("Tips 2");

		compensationItemList.add(tipsCompensation1);
		compensationItemList.add(tipsCompensation2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.TipsPaycheck.equals(compItem.getCompensationType())) {
				if (compItem.getName().equals("Tips 2")) {
					count++;
				} else if (compItem.getName().equals("Tips 1")) {
					count++;
				}
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate VariedPaycheckTips Wage Item with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 VariedCashTips wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateVariedPaycheckTipsWithUniqueName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem tipsCompensation1 = new CompensationItem();
		tipsCompensation1.setRate(new BigDecimal("122.50"));
		tipsCompensation1.setFixedRate(true);
		tipsCompensation1.setCompensationType(CompensationType.TipsPaycheck);
		tipsCompensation1.setName("Tips 1");

		CompensationItem tipsCompensation2 = new CompensationItem();
		tipsCompensation2.setRate(new BigDecimal("122.50"));
		tipsCompensation2.setFixedRate(true);
		tipsCompensation2.setCompensationType(CompensationType.TipsPaycheck);
		tipsCompensation2.setName("Tips 2");

		compensationItemList.add(tipsCompensation1);
		compensationItemList.add(tipsCompensation2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.TipsPaycheck.equals(compItem.getCompensationType())) {
				if (compItem.getName().equals("Tips 2")) {
					count++;
				} else if (compItem.getName().equals("Tips 1")) {
					count++;
				}
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

}
