package com.intuit.sb.payroll.qa.services.employer;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.services.taxpayments.FutaAmountType;
import com.intuit.sb.payroll.services.taxpayments.FutaPaymentLiability;
import com.intuit.sb.payroll.services.taxpayments.FutaPaymentsAndLiabilities;
import com.intuit.sb.payroll.services.taxpayments.PaymentPeriod;
import com.intuit.sb.payroll.services.taxpayments.TaxPaymentService;
import com.intuit.sb.payroll.services.taxpayments.impl.DateRangeMapper;

public class FUTATaxPaymentsTest extends EmployerServiceTestBase{
	private static TaxPaymentService sTaxPaymentService = null;
	
	@BeforeClass
	public void initializeQTDTaxPaymentsTestClass() throws Exception{
		
		sTaxPaymentService = qaBootstrap.getEjbFromClass(TaxPaymentService.class);		
				
	}
	
	/**
     * <P>Test Adding FUTA Tax Payments by TaxPaymentService</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a FUTA payment for Q2 and call updateFutaPaymentsAndLiabilitiesYtd of TaxPaymentService</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that the amount of FUTA payment saved is correct</LI>
     * </UL>
     */
	@Test(enabled=true)
	public void testAddFUTAPayment() throws Exception {
		
		FutaPaymentsAndLiabilities futaPayments = createFUTAPayment(new BigDecimal("100.00"), PtdTypeEnum.QUARTER2, 2009);		
		// create and save FutaP&L in Q2
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();		
		futaPaymentsAndLiabilities.add(futaPayments);
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		//get FUTA payments saved
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
		int total = PtdUtils.getQuartersBeforeCurrentQuarter().size();
		// retrieval will expect "total" elements, one saved in Q2 and an EMPTY FutaP&L for every other prior quarter
		Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.size(), total, "Failed");
		//Assert the values of each FUTA payment and liability
		for(int i=0;i<total;i++){
			if(i==1){//for QTR2
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(1).getFutaPayment().getAmount(),
						new BigDecimal("100.00"),"QTR2 FUTA payment is changed");
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(1).getFutaLiability().getAmount(),
						new BigDecimal("0.00"),"QTR2 FUTA liability is changed");
			}
			else{
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(i).getFutaLiability().getAmount(),
						new BigDecimal("0.00"),"QTR"+i+" "+"FUTA Liability is changed");
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(i).getFutaPayment().getAmount(),
								new BigDecimal("0.00"),"QTR"+i+" "+"FUTA payment is changed");
			}
		}
	}
	
	/**
     * <P>Test Adding FUTA Tax Liabilities by TaxPaymentService</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a FUTA liability for Q2 and call updateFutaPaymentsAndLiabilitiesYtd of TaxPaymentService</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that the amount of FUTA liability saved is correct</LI>
     * </UL>
     */
	@Test
	public void testAddFUTALiability() throws Exception {
		FutaPaymentsAndLiabilities futaLiabilities = createFUTALiability(new BigDecimal("100.00"), PtdTypeEnum.QUARTER2, 2009);
		// create and save FutaP&L in Q2
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();		
		futaPaymentsAndLiabilities.add(futaLiabilities);
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		//get FUTA liability added
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
		int total = PtdUtils.getQuartersBeforeCurrentQuarter().size();
		// retrieval will expect "total" elements, one saved in Q2 and an EMPTY FutaP&L for every other prior quarter
		Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.size(), total, "Failed");
		//Assert the values of each FUTA payment and liability
		for(int i=0;i<total;i++){
			if(i==1){//for QTR2
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(1).getFutaLiability().getAmount(),
						new BigDecimal("100.00"),"QTR2 FUTA liability is changed");
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(1).getFutaPayment().getAmount(),
						new BigDecimal("0.00"),"QTR2 FUTA payment is changed");
			}
			else{
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(i).getFutaLiability().getAmount(),
						new BigDecimal("0.00"),"QTR"+i+" "+"FUTA Liability is changed");
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(i).getFutaPayment().getAmount(),
								new BigDecimal("0.00"),"QTR"+i+" "+"FUTA payment is changed");
			}
		}		
	}
	
	/**
     * <P>Test Adding FUTA Tax Payments and liabilities by TaxPaymentService</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a FUTA payment and liability for Q2 and call updateFutaPaymentsAndLiabilitiesYtd of TaxPaymentService</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that the amount of FUTA payment and liability saved is correct</LI>
     * </UL>
     */
	@Test
	public void testAddFUTAPaymentAndLiability() throws Exception {
		//add FUTA  Payment
		FutaPaymentsAndLiabilities futaPayments = createFUTAPayment(new BigDecimal("100.00"), PtdTypeEnum.QUARTER2, 2009);		
		//add FUTA  Liability
		FutaPaymentsAndLiabilities futaLiabilities = createFUTALiability(new BigDecimal("120.00"), PtdTypeEnum.QUARTER2, 2009);
		
		// create and save FutaP&L in Q2
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();		
		futaPaymentsAndLiabilities.add(futaPayments);
		futaPaymentsAndLiabilities.add(futaLiabilities);
		
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
		int total = PtdUtils.getQuartersBeforeCurrentQuarter().size();
		// retrieval will expect "total" elements, one saved in Q1 and an EMPTY FutaP&L for every other prior quarter
		Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.size(), total, "Failed");
		//Assert the values of each FUTA payment and liability
		for(int i=0;i<total;i++){
			if(i==1){//for QTR2
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(1).getFutaLiability().getAmount(),
						new BigDecimal("120.00"),"QTR2 FUTA liability is changed");
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(1).getFutaPayment().getAmount(),
						new BigDecimal("100.00"),"QTR2 FUTA payment is changed");
			}
			else{
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(i).getFutaLiability().getAmount(),
						new BigDecimal("0.00"),"QTR"+i+" "+"FUTA Liability is changed");
				Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.get(i).getFutaPayment().getAmount(),
								new BigDecimal("0.00"),"QTR"+i+" "+"FUTA payment is changed");
			}
		}
	}
	
	/**
     * <P>Test Adding FUTA Tax Payments in Q4 and get FUTA payments in Q1 of next year</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a FUTA payments for Q1,Q2,Q3 and call updateFutaPaymentsAndLiabilitiesYtd of TaxPaymentService</LI>
     * <LI>change system date to Q1 of Next year</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * <LI>reset back the system date</LI>
     * </OL>
     * <UL>
     * <LI>Verify that no FUTA payments are retrieved</LI>
     * </UL>
     */
	@Test
	public void testAddFUTAPaymentsInQ4AndGetInNextYear() throws Exception {
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();
		// create FutaP&L in Q4	
		FutaPaymentsAndLiabilities futaPayments = createFUTAPayment(new BigDecimal("10.00"), PtdTypeEnum.QUARTER1, 2009);					
		futaPaymentsAndLiabilities.add(futaPayments);
		futaPayments = createFUTAPayment(new BigDecimal("100.00"), PtdTypeEnum.QUARTER2, 2009);
		futaPaymentsAndLiabilities.add(futaPayments);
		futaPayments = createFUTAPayment(new BigDecimal("1000.00"), PtdTypeEnum.QUARTER3, 2009);
		futaPaymentsAndLiabilities.add(futaPayments);
		// save FutaP&L in Q4
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		Date nowDate = cal.getTime();
		cal.set(Calendar.YEAR,2010);
		cal.set(Calendar.MONTH, Calendar.JANUARY);
		cal.set(Calendar.DAY_OF_MONTH, 10);			//	January 10th of next year
		if(changeSystemDate(cal.getTime())) {		
			// if system date could be changed to Q1 of next year run the test
			List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
			changeSystemDate(nowDate); //reset system time back
			Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.size(),0, "Failed");	
		} else {
			// system date could not be changed - dummy assert
			Assert.assertTrue(true);
		}
	}
	
	/**
     * <P>Test getFutaPaymentsAndLiabilitiesYtd() in Q1</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>change system date to Q1 of current year</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * <LI>reset back the system date</LI>
     * </OL>
     * <UL>
     * <LI>Verify that no FUTA payments are retrieved</LI>
     * </UL>
     */
	@Test
	public void testGetFUTAPayementsInQ1() throws Exception {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		Date nowDate = cal.getTime();
		cal.set(Calendar.MONTH, Calendar.JANUARY);
		cal.set(Calendar.DAY_OF_MONTH, 10);			//	Jan 10th of current year
		if(changeSystemDate(cal.getTime())) {		
			// if system date could be changed to Q1 run the test
			List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
			changeSystemDate(nowDate); //reset system time back
			Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.size(), 0, "Failed");	
		} else {
			// system date could not be changed - dummy assert
			Assert.assertTrue(true);
		}
	}
	
	/**
     * <P>Test getFutaPaymentsAndLiabilitiesYtd() in Q2</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>change system date to Q2 of current year</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * <LI>reset back the system date</LI>
     * </OL>
     * <UL>
     * <LI>Verify that one FUTA payment is retrieved</LI>
     * </UL>
     */
	@Test(enabled=true)
	public void testGetFUTAPayementsInQ2() throws Exception {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		Date nowDate = cal.getTime();
		cal.set(Calendar.MONTH, Calendar.APRIL);
		cal.set(Calendar.DAY_OF_MONTH, 10);			//	April 10th of current year
		if(changeSystemDate(cal.getTime())) {		
			// if system date could be changed to Q1 run the test
			List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
			changeSystemDate(nowDate); //reset system time back
			Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.size(), 1, "Failed");	
		} else {
			// system date could not be changed - dummy assert
			Assert.assertTrue(true);
		}
	}
	
	/**
     * <P>Test getFutaPaymentsAndLiabilitiesYtd() in Q3</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>change system date to Q3 of current year</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * <LI>reset back the system date</LI>
     * </OL>
     * <UL>
     * <LI>Verify that two FUTA payments are retrieved</LI>
     * </UL>
     */
	@Test
	public void testGetFUTAPayementsInQ3() throws Exception {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		Date nowDate = cal.getTime();
		cal.set(Calendar.MONTH, Calendar.JULY);
		cal.set(Calendar.DAY_OF_MONTH, 10);			//	July 10th of current year
		if(changeSystemDate(cal.getTime())) {		
			// if system date could be changed to Q1 run the test
			List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
			changeSystemDate(nowDate); //reset system time back
			Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.size(), 2, "Failed");	
		} else {
			// system date could not be changed - dummy assert
			Assert.assertTrue(true);
		}
	}
	
	/**
     * <P>Test getFutaPaymentsAndLiabilitiesYtd() in Q4</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>change system date to Q4 of current year</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * <LI>reset back the system date</LI>
     * </OL>
     * <UL>
     * <LI>Verify that three FUTA payments are retrieved</LI>
     * </UL>
     */
	@Test(enabled=true)
	public void testGetFUTAPayementsInQ4() throws Exception {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		Date nowDate = cal.getTime();
		cal.set(Calendar.MONTH, Calendar.OCTOBER);
		cal.set(Calendar.DAY_OF_MONTH, 10);			//	October 10th of current year
		if(changeSystemDate(cal.getTime())) {		
			// if system date could be changed to Q1 run the test
			List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
			changeSystemDate(nowDate); //reset system time back
			Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.size(), 3, "Failed");	
		} else {
			// system date could not be changed - dummy assert
			Assert.assertTrue(true);
		}
	}
	
	/**
     * <P>Test Adding FUTA Tax Payments of future QTR(next year) by TaxPaymentService</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a FUTA payment for Q4 next year and call updateFutaPaymentsAndLiabilitiesYtd of TaxPaymentService</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that FUTA payments retrieved doesn't contain the future payment just added</LI>
     * </UL>
     */
	@Test(enabled=true)
	public void testAddFUTAPaymentOfFutureQTR() throws ValidationException,ServiceFailureException{
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		int year = cal.get(Calendar.YEAR);
		FutaPaymentLiability futaPayment = new FutaPaymentLiability();
		futaPayment.setFutaAmountType(FutaAmountType.FUTAPAYMENT);
		futaPayment.setAmount(new BigDecimal("100"));
		FutaPaymentsAndLiabilities futaPayments = new FutaPaymentsAndLiabilities();
		futaPayments.setFutaPayment(futaPayment);
		DateRange computedRangeForQ4 = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER4, year+1);
		PaymentPeriod quarterPaymentPeriod = DateRangeMapper. mapDateRangeToPaymentPeriod(computedRangeForQ4);
		futaPayments.setQuarterPaymentPeriod(quarterPaymentPeriod);
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();
		futaPaymentsAndLiabilities.add(futaPayments);
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		
		
		int total = PtdUtils.getQuartersBeforeCurrentQuarter().size();
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = 
			sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
		// retrieval will expect "total" elements, one FutaP&L for every other prior quarter (saved one will not be found since it is in Q4)
		Assert.assertEquals(futaPaymentsAndLiabilitiesSaved.size(), total, "Failed");
		
	}
	
	/**
     * <P>Test Adding FUTA Tax Payment and liability for Q2 and updating them</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a FUTA payment and liability for Q2 and call updateFutaPaymentsAndLiabilitiesYtd of TaxPaymentService</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * <LI>update the values of liability and payment calling updateFutaPaymentsAndLiabilitiesYtd()</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that both FUTA payment and liability are updated appropriately after retrieved</LI>
     * </UL>
     */
	@Test
	public void testUpdateFUTAPaymentsAndLiabilities() throws ValidationException, ServiceFailureException{
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();	
		FutaPaymentsAndLiabilities futaPayments = createFUTAPayment(new BigDecimal("100.00"), PtdTypeEnum.QUARTER2, 2009);
		futaPaymentsAndLiabilities.add(futaPayments);
		//add FUTA  Liability
		FutaPaymentsAndLiabilities futaLiabilities = createFUTALiability(new BigDecimal("120.00"), PtdTypeEnum.QUARTER2, 2009);
		futaPaymentsAndLiabilities.add(futaLiabilities);
		//save FutaP&L in Q2		
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		
		//get FUTA payments saved
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesSaved = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
		//Edit and Update FUTA Payment and Liabilities
		futaPaymentsAndLiabilitiesSaved.get(1).getFutaPayment().setAmount(new BigDecimal("50.00"));
		futaPaymentsAndLiabilitiesSaved.get(1).getFutaLiability().setAmount(new BigDecimal("200.00"));
		
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilitiesSaved);
		//get FUTA payments Updated
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesUpdated = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
		int total = PtdUtils.getQuartersBeforeCurrentQuarter().size();
		// retrieval will expect "total" elements, one saved in Q2 and an EMPTY FutaP&L for every other prior quarter
		Assert.assertEquals(futaPaymentsAndLiabilitiesUpdated.size(), total, "Failed");
		//Assert the values of each FUTA payment and liability
		for(int i=0;i<total;i++){
			if(i==1){//for QTR2
				Assert.assertEquals(futaPaymentsAndLiabilitiesUpdated.get(1).getFutaPayment().getAmount(),
						new BigDecimal("50.00"),"QTR2 FUTA payment is changed");
				Assert.assertEquals(futaPaymentsAndLiabilitiesUpdated.get(1).getFutaLiability().getAmount(),
						new BigDecimal("200.00"),"QTR2 FUTA liability is changed");
			}
			else{
				Assert.assertEquals(futaPaymentsAndLiabilitiesUpdated.get(i).getFutaLiability().getAmount(),
						new BigDecimal("0.00"),"QTR"+i+" "+"FUTA Liability is changed");
				Assert.assertEquals(futaPaymentsAndLiabilitiesUpdated.get(i).getFutaPayment().getAmount(),
								new BigDecimal("0.00"),"QTR"+i+" "+"FUTA payment is changed");
			}
		}
	}
	
	/**
     * <P>Test removing FUTA Tax Payment and liabilities</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a FUTA payment and liability for Q2 and call updateFutaPaymentsAndLiabilitiesYtd of TaxPaymentService</LI>
     * <LI>call removeFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * <LI>call getFutaPaymentsAndLiabilitiesYtd() of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that all the FUTA payments and liabilities retrieved are of value zero</LI>
     * <LI>Verify that size of FUTA payments and liabilities retrieved is QTR-1</LI>
     </LI>
     * </UL>
     */
	@Test
	public void testRemoveFUTAPayments() throws ValidationException,ServiceFailureException{
		//add FUTA  Payment
		FutaPaymentsAndLiabilities futaPayments = createFUTAPayment(new BigDecimal("100.00"), PtdTypeEnum.QUARTER2, 2009);		
		//add FUTA  Liability
		FutaPaymentsAndLiabilities futaLiabilities = createFUTALiability(new BigDecimal("120.00"), PtdTypeEnum.QUARTER2, 2009);
		
		// create and save FutaP&L in Q2
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();		
		futaPaymentsAndLiabilities.add(futaPayments);
		futaPaymentsAndLiabilities.add(futaLiabilities);		
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		
		//Remove FUTA payments and liabilities
		sTaxPaymentService.removeFutaPaymentsAndLiabilitiesYtd();
		//get FUTA payments
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilitiesUpdated = sTaxPaymentService.getFutaPaymentsAndLiabilitiesYtd();
		int total = PtdUtils.getQuartersBeforeCurrentQuarter().size();
		// retrieval will expect "total" elements, one saved in Q2 and an EMPTY FutaP&L for every other prior quarter
		Assert.assertEquals(futaPaymentsAndLiabilitiesUpdated.size(), total, "Failed");	
		for(int i=0;i<total;i++){			
				Assert.assertEquals(futaPaymentsAndLiabilitiesUpdated.get(i).getFutaLiability().getAmount(),
						new BigDecimal("0.00"),"QTR"+i+" "+"FUTA Liability is changed");
				Assert.assertEquals(futaPaymentsAndLiabilitiesUpdated.get(i).getFutaPayment().getAmount(),
								new BigDecimal("0.00"),"QTR"+i+" "+"FUTA payment is changed");
			
		}
	}
	
	/**
     * <P>Test Adding multiple FUTA Tax Payments of Q2</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create two FUTA payments for Q2 and call updateFutaPaymentsAndLiabilitiesYtd of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that validation exception is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions= ValidationException.class)
	public void testAddMultipleFUTAPaymentsOfSameQTR() throws ValidationException,ServiceFailureException{
		FutaPaymentsAndLiabilities futaPayments = createFUTAPayment(new BigDecimal("100.00"), PtdTypeEnum.QUARTER2, 2009);		
		// create and save FutaP&L in Q2
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();		
		futaPaymentsAndLiabilities.add(futaPayments);
		futaPaymentsAndLiabilities.add(futaPayments);
		
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);
		
	}
	
	/**
     * <P>Test Adding multiple FUTA Tax Liabilities of Q2</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create two FUTA Liabilities for Q2 and call updateFutaPaymentsAndLiabilitiesYtd of TaxPaymentService</LI>
     * </OL>
     * <UL>
     * <LI>Verify that validation exception is thrown</LI>
     * </UL>
     */
	@Test(expectedExceptions= ValidationException.class)
	public void testAddMultipleFUTALiabilitiesOfSameQTR() throws ValidationException,ServiceFailureException{
		
		FutaPaymentsAndLiabilities futaLiabilities = createFUTALiability(new BigDecimal("100.00"), PtdTypeEnum.QUARTER2, 2009);		
		// create and save FutaP&L in Q2
		List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities = new ArrayList<FutaPaymentsAndLiabilities>();		
		futaPaymentsAndLiabilities.add(futaLiabilities);
		futaPaymentsAndLiabilities.add(futaLiabilities);
		sTaxPaymentService.updateFutaPaymentsAndLiabilitiesYtd(futaPaymentsAndLiabilities);		
	}
	
	/**
	 * Changes the System date to the date given
	 * @param Date
	 * @return Boolean	 
	 */
	public Boolean changeSystemDate(Date date) {
		String s = null;
		boolean done = true;
		try {
            Process p = null;
            // run the Windows command
            StringBuilder str = new StringBuilder();
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);            
            str.append((cal.get(2)+1) + "-");
	        str.append(cal.get(5) + "-");
	        str.append(cal.get(1));
			
			if(System.getProperty("os.name").toUpperCase().contains("WINDOWS")) {
				p = Runtime.getRuntime().exec("cmd /c date" + " " + str.toString());
			}
			else{
				done = false;
			}
            
			BufferedReader stdError = new BufferedReader(new InputStreamReader(p.getErrorStream()));
                                   
            // read any errors from the attempted command

            System.out.println("Here is the standard error of the command (if any):\n");
            while ((s = stdError.readLine()) != null) {
                System.out.println(s);
            }            
           
        } catch (IOException e) {
            System.out.println("[IOException]. Printing Stack Trace");
            e.printStackTrace();            
        }
		return done;
	}
	
	/**
	 * Creates a FUTA Payment
	 * @param BigDecimal,PtdTypeEnum,int
	 * @return FutaPaymentsAndLiabilities
	 * 
	 * @throws ValidationException
	 */
	public FutaPaymentsAndLiabilities createFUTAPayment(BigDecimal amount, PtdTypeEnum quarter, int year)throws ValidationException{
		FutaPaymentLiability futaPayment = new FutaPaymentLiability();
		futaPayment.setFutaAmountType(FutaAmountType.FUTAPAYMENT);
		futaPayment.setAmount(amount);
		FutaPaymentsAndLiabilities futaPayments = new FutaPaymentsAndLiabilities();
		futaPayments.setFutaPayment(futaPayment);
		DateRange computedRangeForQ2 = PtdUtils.getDateRangeForPeriodType(quarter, year);
		PaymentPeriod quarterPaymentPeriod = DateRangeMapper. mapDateRangeToPaymentPeriod(computedRangeForQ2);
		futaPayments.setQuarterPaymentPeriod(quarterPaymentPeriod);		
		return futaPayments;		
	}
	
	/**
	 * Creates a FUTA Liability
	 * @param BigDecimal,PtdTypeEnum,int
	 * @return FutaPaymentsAndLiabilities
	 * 
	 * @throws ValidationException
	 */
	public FutaPaymentsAndLiabilities createFUTALiability(BigDecimal amount, PtdTypeEnum quarter, int year)throws ValidationException{
		FutaPaymentLiability futaLiability = new FutaPaymentLiability();
		futaLiability.setFutaAmountType(FutaAmountType.FUTALIABILITY);
		futaLiability.setAmount(amount);
		FutaPaymentsAndLiabilities futaLiabilities = new FutaPaymentsAndLiabilities();
		futaLiabilities.setFutaLiability(futaLiability);
		DateRange computedRangeForQ2 = PtdUtils.getDateRangeForPeriodType(quarter, year);
		PaymentPeriod quarterPaymentPeriod = DateRangeMapper. mapDateRangeToPaymentPeriod(computedRangeForQ2);
		futaLiabilities.setQuarterPaymentPeriod(quarterPaymentPeriod);		
		return futaLiabilities;		
	}
	
	
	
	
}
