package com.intuit.sb.payroll.qa.services.pto;

import java.math.BigDecimal;
import java.util.Calendar;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.services.pto.AccrualStartType;
import com.intuit.sb.payroll.services.pto.PTOAccrualType;
import com.intuit.sb.payroll.services.pto.PTOInfo;
import com.intuit.sb.payroll.services.pto.PTOService;
import com.intuit.sb.payroll.services.pto.PTOType;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.PtoBaseRateType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;

public class GetPTOByNameAndTypeTest extends WorkerServiceTestBase
{
	private static PTOService sPTOService;
	
	private static Worker newWorker;
	private static Worker worker;
	private static long sWorkerId;
	
	@BeforeClass
	public static void initializeGetPTOByNameAndTypeTestClass()
	{
		sPTOService = qaBootstrap.getEjbFromClass(PTOService.class);
	}
	
	public void setUpWorker() throws Exception
	{
		newWorker = createWorker(); // create a worker with minimum data

		// Test SaveWorker with no compensations
		worker = sWorkerService.saveWorker(newWorker);
		sWorkerId = worker.getObjectHandle().getId();
		
		// save salary compensation item
		CompensationItem[] inputCompensations = new CompensationItem[1];
		inputCompensations[0] = new CompensationItem(null, new BigDecimal(10000), 
				0, CompensationType.Salary, "salary",true);
		inputCompensations[0].setPtoBaseRateType(PtoBaseRateType.PTO);
        // Test inserting compensations
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, inputCompensations));
	}
	
	public void tearDownWorker() throws Exception
	{
		cleanUpEIWorker(worker.getObjectHandle().getId());
	}
	
	private PTOInfo createPTOInfo(PTOType type) throws ValidationException, ServiceFailureException
	{
		PTOInfo ptoInfo = new PTOInfo();
		ptoInfo.setWorkerId(sWorkerId);
		ptoInfo.setName("Floating goliday");
		ptoInfo.setPtoType(type);
		ptoInfo.setAccrualRate(new Double("2.35"));
		ptoInfo.setPtoAccrualType(PTOAccrualType.EACH_PAYCHECK);
		ptoInfo.setAllowCarryOver(true);
		ptoInfo.setLimitAccrual(true);
		ptoInfo.setMaximumAccrual(new Double("200.00"));
		ptoInfo.setAccruedHours(new Double("5.40"));
		ptoInfo.setDisbursedHoursYTD(new Double("0.00"));
		
		Calendar now = DateUtil.getNowCalendar();
		ptoInfo.setAccrualStartDate(now.getTime());
		ptoInfo.setAccrualStartType(AccrualStartType.IMMEDIATELY);
		
		return ptoInfo;
	}
	
	private boolean comparePTOInfo(PTOInfo ptoInfo, PTOInfo resultPTOInfo)
	{
		if(ptoInfo.getAccrualRate().doubleValue() == resultPTOInfo.getAccrualRate().doubleValue() 
				&& ptoInfo.getAccrualStartDate().equals(resultPTOInfo.getAccrualStartDate())
				&& ptoInfo.getAccrualStartType().equals(resultPTOInfo.getAccrualStartType())
				&& ptoInfo.getAccruedHours().doubleValue() == resultPTOInfo.getAccruedHours().doubleValue()
				&& ptoInfo.getDisbursedHoursYTD().doubleValue() == resultPTOInfo.getDisbursedHoursYTD().doubleValue()
				&& ptoInfo.getMaximumAccrual().doubleValue() == resultPTOInfo.getMaximumAccrual().doubleValue()
				&& ptoInfo.getName().equals(resultPTOInfo.getName())
				&& ptoInfo.getPtoAccrualType().equals(resultPTOInfo.getPtoAccrualType())
				&& ptoInfo.getPtoType().equals(resultPTOInfo.getPtoType())
				&& ptoInfo.getWorkerId().longValue() == resultPTOInfo.getWorkerId().longValue())
		{
			return true;
		}
		
		return false;
	}
	
	private boolean comparePTOInfo(PTOInfo[] expectedPTOInfo, PTOInfo[] resultPTOInfo)
	{
		if(expectedPTOInfo.length != resultPTOInfo.length)
		{
			return false;
		}
		else
		{
			for(int count = 0; count < expectedPTOInfo.length; count++)
			{
				PTOInfo ptoInfo1 = expectedPTOInfo[count];
				PTOInfo ptoInfo2 = resultPTOInfo[count];
				
				if(ptoInfo1.getAccrualRate().doubleValue() == ptoInfo2.getAccrualRate().doubleValue() 
						&& ptoInfo1.getAccrualStartDate().equals(ptoInfo2.getAccrualStartDate())
						&& ptoInfo1.getAccrualStartType().equals(ptoInfo2.getAccrualStartType())
						&& ptoInfo1.getAccruedHours().doubleValue() == ptoInfo2.getAccruedHours().doubleValue()
						&& ptoInfo1.getDisbursedHoursYTD().doubleValue() == ptoInfo2.getDisbursedHoursYTD().doubleValue()
						&& ptoInfo1.getMaximumAccrual().doubleValue() == ptoInfo2.getMaximumAccrual().doubleValue()
						&& ptoInfo1.getName().equals(ptoInfo2.getName())
						&& ptoInfo1.getPtoAccrualType().equals(ptoInfo2.getPtoAccrualType())
						&& ptoInfo1.getPtoType().equals(ptoInfo2.getPtoType())
						&& ptoInfo1.getWorkerId().longValue() == ptoInfo2.getWorkerId().longValue())
				{
					return true;
				}
			}
		}
		
		return false;
	}
	
	/**
     * <P>Test getPTOByNameAndType() with  null worker ID</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Pass a null worker ID to the API</LI>
     * <LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * @throws Exception
     * </UL>
     */
	@Test(expectedExceptions=ServiceFailureException.class)
	public void testGetPTOWithNullWorkerId() throws Exception
	{
		sPTOService.getPTOByNameAndType(null, "floating holiday", PTOType.VACATION);
	}
	
	/**
     * <P>Test getPTOByNameAndType() with  all valid details - Happy Path</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker with compensation items and Save worker</LI>
     * <LI>Create PTOInfo(in this case, SICK)</LI>
     * <LI>Save the PTOInfo</LI>
     * <LI>make the getPTOByNameAndType() API call with sWorkerId, PTO Name and PTO Type</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PTOInfo is saved on the worker correctly</LI>
     * @throws Exception
     * </UL>
     */
	@Test
	public void testGetPTOByNameAndType() throws Exception
	{
			setUpWorker();
			PTOInfo ptoInfo = createPTOInfo(PTOType.SICK_TIME);
			sPTOService.savePTO(ptoInfo);
			PTOInfo resultPTOInfo = sPTOService.getPTOByNameAndType(sWorkerId, ptoInfo.getName(), ptoInfo.getPtoType());
			AssertJUnit.assertTrue(comparePTOInfo(ptoInfo,resultPTOInfo));
			tearDownWorker();
	}
	
	/**
     * <P>Test getPTOByNameAndType() with  null PTO Name</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Pass a null PTO name to the API</LI>
     * <LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * @throws Exception
     * </UL>
     */
	@Test(expectedExceptions=ServiceFailureException.class)
	public void testGetPTOWithNullPTOName() throws Exception
	{
		sPTOService.getPTOByNameAndType(sWorkerId, null, PTOType.VACATION);
		AssertJUnit.fail("ServiceFailureException not thrown");
	}
	
	/**
     * <P>Test getPTOByNameAndType() with  empty("") PTO Name</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Pass an empty("") PTO name to the API</LI>
     * <LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown with a message that says "ptoName is a required value"</LI>
     * @throws Exception
     * </UL>
     */
	@Test(expectedExceptions=ServiceFailureException.class)
	public void testEmptyPTOName() throws Exception
	{
		sPTOService.getPTOByNameAndType(sWorkerId, "", PTOType.VACATION);
	}
	
	/**
     * <P>Test getPTOByNameAndType() with  null PTOType</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Pass a null PTOType to the API</LI>
     * <LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown </LI>
     * @throws ValidationException, ServiceFailureException
     * </UL>
     */
	@Test(expectedExceptions=ServiceFailureException.class)
	public void testNullPTOType() throws Exception
	{
		setUpWorker();
		sPTOService.getPTOByNameAndType(sWorkerId, "floating goliday", null);
		tearDownWorker();
	}
	
	/**
     * <P>Test getPTOByNameAndType() by passing invalid PTOType</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker with compensation items and Save worker</LI>
     * <LI>Create PTOInfo(in this case, SICK)</LI>
     * <LI>Save the PTOInfo</LI>
     * <LI>make the getPTOByNameAndType() API call with sWorkerId, PTO Name and VACATION PTO Type</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a null PTOInfo is returned.</LI>
     * @throws Exception
     * </UL>
     */
	@Test
	public void testInvalidPTOType() throws Exception
	{
		setUpWorker();
		PTOInfo ptoInfo = createPTOInfo(PTOType.VACATION);
		sPTOService.savePTO(ptoInfo);
		ptoInfo = sPTOService.getPTOByNameAndType(sWorkerId, "floating holiday", PTOType.SICK_TIME);
		AssertJUnit.assertEquals("Expect null when PTO doesn't exist", null, ptoInfo);
		tearDownWorker();
	}
	
	/**
     * <P>Test getAllPTOInfo() with null workerID</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>make the getAllPTOInfo() API call with null workerId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since workerID is null</LI>
     * @throws Exception
     * </UL>
     */
	@Test(expectedExceptions=ServiceFailureException.class)
	public void testGetAllPTOInfoWithNullWorkerId() throws Exception
	{
		sPTOService.getAllPTOInfo(null);
	}
	
	/**
     * <P>Test getAllPTOInfo() with  all valid details - Happy Path</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker with compensation items and Save worker</LI>
     * <LI>Create PTOInfo(in this case, SICK and VACATION)</LI>
     * <LI>Save the PTOInfos</LI>
     * <LI>make the getAllPTOInfo() API call with sWorkerId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PTOInfo is returned correctly for the workerID passed</LI>
     * @throws Exception
     * </UL>
     */
	@Test
	public void testGetAllPTOInfoByWorkerId() throws Exception
	{
		setUpWorker();
		PTOInfo[] expectedPTOInfo = new PTOInfo[2];
		//Create PTOInfo for sWorkerId
		PTOInfo ptoInfo = createPTOInfo(PTOType.SICK_TIME);
		sPTOService.savePTO(ptoInfo);
		expectedPTOInfo[0] = ptoInfo;
		
		ptoInfo = createPTOInfo(PTOType.VACATION);
		sPTOService.savePTO(ptoInfo);
		expectedPTOInfo[1] = ptoInfo;
		
		PTOInfo[] resultPTOInfo = sPTOService.getAllPTOInfo(sWorkerId);
		AssertJUnit.assertTrue(comparePTOInfo(expectedPTOInfo,resultPTOInfo));
		tearDownWorker();
	}
}
