package com.intuit.sb.payroll.qa.services.taxes;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.worker.Worker;

/**
 * <P>
 * The <CODE>GetEmployeeWithholdingsExceptionsTest</CODE> class contains tests for testing the Exceptions that can be
 * thrown by getEmployeeWithholdings() in TaxWithholdingsService
 * </P>
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 * 
 */
public class GetEmployeeWithholdingsExceptionsTest extends EntityUtilsTestBase {

	/**
	 * <P>Test calling getEmployeeWithholdings() with a null worker id</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsService.getEmployeeWithholdings() with a null worker id</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWorkerId() throws Exception {
    	// Exercise
		sTaxWithholdingsService.getEmployeeWithholdings(null);
	}

	/**
	 * <P>Test calling getEmployeeWithholdings() with a non-existent worker id</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsService.getEmployeeWithholdings() with a non-existent worker id</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ServiceFailureException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testNonExistentWorkerId() throws Exception {
    	// Exercise
		sTaxWithholdingsService.getEmployeeWithholdings(7654321L);
	}

	/**
	 * <P>Test calling getEmployeeWithholdings() with a deleted worker id</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker</LI>
	 * <LI>Delete the worker</LI>
	 * <LI>Call TaxWithholdingsService.getEmployeeWithholdings() with the deleted worker's id</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ServiceFailureException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testDeletedWorkerId() throws Exception {
		// Setup
		// Create a worker
		Worker savedWorker = null;
		Worker worker = createDtoWorker();
		worker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
		worker.setWorkLocation(getNamedWorkLocation(sMountainViewLocation));
	    savedWorker = sWorkerService.saveWorker(worker);
	    Long workerId = savedWorker.getObjectHandle().getId();
	    // Now delete the worker
		cleanUpEIWorker(workerId);
	    
    	// Exercise
		sTaxWithholdingsService.getEmployeeWithholdings(workerId);
	}

}