package com.intuit.sb.payroll.qa.services.worker;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;


/**
 *  <P>The <CODE>DeleteWorkerExceptionsTest</CODE> class contains tests for testing exceptions that can be thrown
 *     by deleteWorker() in WorkerService</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class DeleteWorkerExceptionsTest extends EntityUtilsTestBase
{

    /**
     * <P>Test calling deleteWorker() with a null workerId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IWorkerService.deleteWorker()with a null workerId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.deleteWorker"}, expectedExceptions=ValidationException.class)
    public void testDeleteWorkerWithNullWorkerId() throws Exception
    {
    	// Exercise
    	sWorkerService.deleteWorker(null);
    }

    /**
     * <P>Test calling deleteWorker() with a deleted worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Delete the worker</LI>
     * <LI>Call IWorkerService.deleteWorker()with the deleted worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.deleteWorker"}, expectedExceptions=ServiceFailureException.class)
    public void testDeleteDeletedWorker() throws Exception
    {
    	// Setup
    	Long workerId = createPayrollReadyWorker();
    	sWorkerService.deleteWorker(workerId);
    	// Exercise
    	sWorkerService.deleteWorker(workerId);
    }

}
