package com.intuit.sb.payroll.qa.services.worker;

import java.util.Date;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.worker.Worker;


/**
 *  <P>The <CODE>HireExistingWorkerTest</CODE> class contains tests for testing hireExistingWorker() in WorkerService</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class HireExistingWorkerTest extends EntityUtilsTestBase
{
    /**
     * <P>Test re-hiring a terminated worker on the day of termination</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Call IWorkerService.hireExistingWorker() with the worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify expected results</LI>
     * </UL>
     */
    @Test(groups = {"worker.hireExistingWorker"})
    public void testRehiringTerminatedWorkerOnTerminationDate() throws Exception
    {
    	// Setup
    	Long workerId = createPayrollReadyWorker();
    	Worker worker = sWorkerService.getWorkerById(workerId);
    	worker = sWorkerService.terminateWorker(worker, DateUtil.getNowDate());
    	Date origHireDate = worker.getHireDate();
    	// Exercise
    	worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId(), DateUtil.getNowDate());
    	AssertJUnit.assertTrue("Worker should have been marked active!", worker.isActive());
    	AssertJUnit.assertEquals("Worker's employment status was not set to EMPLOYED!",
    			EmploymentStatusEnum.EMPLOYED, worker.getEmploymentStatus());
    	AssertJUnit.assertTrue("Hire date mismatch!", DateUtil.isSameDay(origHireDate, worker.getHireDate()));
    }

    /**
     * <P>Test re-hiring a terminated worker with a date after the termination date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Call IWorkerService.hireExistingWorker() with the worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify expected results</LI>
     * </UL>
     */
    @Test(enabled=false, groups = {"worker.hireExistingWorker"})
    public void testRehiringTerminatedWorkerOnDateAfterTerminationDate() throws Exception
    {
    	// Setup
    	Long workerId = createPayrollReadyWorker();
    	Worker worker = sWorkerService.getWorkerById(workerId);
    	Date today = DateUtil.getNowDate();
    	worker = sWorkerService.terminateWorker(worker, DateUtil.getNextDate(today, -7));
    	// Exercise
    	worker = sWorkerService.hireExistingWorker(worker.getObjectHandle().getId(), today);
    	AssertJUnit.assertTrue("Worker should have been marked active!", worker.isActive());
    	AssertJUnit.assertEquals("Worker's employment status was not set to EMPLOYED!",
    			EmploymentStatusEnum.EMPLOYED, worker.getEmploymentStatus());
    	AssertJUnit.assertTrue("Hire date mismatch!", DateUtil.isSameDay(today, worker.getHireDate()));
    }

}
