package com.intuit.sb.payroll.qa.services.worker;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.worker.Worker;


/**
 *  <P>The <CODE>TerminateWorkerTest</CODE> class contains tests for testing terminateWorker() in WorkerService</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class TerminateWorkerTest extends EntityUtilsTestBase
{
    /**
     * <P>Test calling terminateWorker() with a payroll ready worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Call IWorkerService.terminateWorker()with the worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify expected results</LI>
     * </UL>
     */
    @Test(groups = {"worker.terminateWorker"})
    public void testDeletedWorker() throws Exception
    {
    	// Setup
    	Long workerId = createPayrollReadyWorker();
    	Worker worker = sWorkerService.getWorkerById(workerId);
    	// Exercise
    	worker = sWorkerService.terminateWorker(worker, DateUtil.getNowDate());
    	AssertJUnit.assertFalse("Worker should have been marked inactive!", worker.isActive());
    	AssertJUnit.assertEquals("Worker's employment status was not set to TERMINATED!",
    			EmploymentStatusEnum.TERMINATED, worker.getEmploymentStatus());
    }

}
