package com.intuit.sb.payroll.qa.services.worker;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingInfoRequest;
import com.intuit.sb.payroll.services.taxes.WithholdingInfoOperation;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;

/**
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class WorkerServiceWorkflowTest extends WorkerServiceTestBase {
	/**
	 * <P>
	 * Test to save a Worker without work location with WithholdingInfo
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create new Worker with no WorkLocation</LI>
	 * <LI>Create WithholdingInformation</LI>
	 * <LI>Save updated Worker</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that Workflow exception is thrown</LI>
	 * </UL>
	 *
	 * @throws Exception
	 */
	@Test (expectedExceptions=WorkflowException.class)
	public void saveWorkerWithholdingNoWorkLocation() throws Exception {
		Worker eiWorker = null;
		Worker savedWorker = null;
		WithholdingInformation withholdings = null;
		Long uniqueId = null;
			
		// Save a new Worker with no Worklocation
		eiWorker = createDtoWorker();
		savedWorker = sWorkerService.saveWorker(eiWorker);
		uniqueId = savedWorker.getObjectHandle().getId();
		
		// Add WithholdingInfo
		withholdings = createEIWithholdingInformation(savedWorker);
		SaveWithholdingInfoRequest request = new SaveWithholdingInfoRequest();
		request.setWithholdingInfo(withholdings);
		request.setWorkerId(uniqueId);
		request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
		sTaxWithholdingsService.saveWithholdingInformation(request);
		// No need to cleanup the worker since the entire transaction is rolled back
	}
	
	/**
	 * <P>
	 * Test to save a Worker without hire date with WithholdingInfo
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create new Worker with no HireDate</LI>
	 * <LI>Create WithholdingInformation</LI>
	 * <LI>Save updated Worker</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that Workflow exception is thrown</LI>
	 * </UL>
	 *
	 * @throws Exception
	 */
	@Test (expectedExceptions=WorkflowException.class)
	public void saveWorkerWithholdingNoHireDate() throws Exception {
		Worker eiWorker = null;
		Worker savedWorker = null;
		WithholdingInformation withholdings = null;
		Long uniqueId = null;
			
		// Save a new Worker with no Hire Date
		eiWorker = createDtoWorker();
		eiWorker.setHireDate(null);
		savedWorker = sWorkerService.saveWorker(eiWorker);
		uniqueId = savedWorker.getObjectHandle().getId();
		
		// Add WithholdingInfo
		withholdings = createEIWithholdingInformation(savedWorker);
		SaveWithholdingInfoRequest request = new SaveWithholdingInfoRequest();
		request.setWithholdingInfo(withholdings);
		request.setWorkerId(uniqueId);
		request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
		sTaxWithholdingsService.saveWithholdingInformation(request);
		// No need to cleanup the worker since the entire transaction is rolled back
	}
	
	/**
	 * <P>
	 * Test to save a Worker without WorkerType with WithholdingInfo
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create new Worker with no WorkerType</LI>
	 * <LI>Create WithholdingInformation</LI>
	 * <LI>Save updated Worker</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that Workflow exception is thrown</LI>
	 * </UL>
	 *
	 * @throws Exception
	 */
	@Test (expectedExceptions=WorkflowException.class)
	public void saveWorkerWithholdingNoWorkerType() throws Exception {
		Worker eiWorker = null;
		Worker savedWorker = null;
		WithholdingInformation withholdings = null;
		Long uniqueId = null;
		
		// Save a new Worker with no WorkerType
		eiWorker = createDtoWorker();
		eiWorker.setWorkerType(null);
		savedWorker = sWorkerService.saveWorker(eiWorker);
		uniqueId = savedWorker.getObjectHandle().getId();
		
		// Add WithholdingInfo
		withholdings = createEIWithholdingInformation(savedWorker);
		SaveWithholdingInfoRequest request = new SaveWithholdingInfoRequest();
		request.setWithholdingInfo(withholdings);
		request.setWorkerId(uniqueId);
		request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
		sTaxWithholdingsService.saveWithholdingInformation(request);
		// No need to cleanup the worker since the entire transaction is rolled back
	}
}
