package com.intuit.sb.payroll.qa.services.workerpayments;

import java.util.List;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class GetNextScheduledPayrollRunInfoExceptionsTest extends WorkerPaymentServiceTestBase {
	PayrollScheduleComponent pScheduleComponent;
	
	@BeforeMethod
	public void initializeGetNextScheduledPayrollRunInfoExceptionsTest()
	{
		pScheduleComponent = ServiceLocator.getInstance().getService(
                PayrollScheduleComponent.JNDI_NAME);
	}

    /**
     * <P>Test ServiceFailureException caused by no Payroll Schedule</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's getNextScheduledPayrollRunInfo() when no Payroll Schedule created</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class, enabled=false)
    public void testNoPayrollSchedule() throws Exception {
    	// Setup
		List<PayrollSchedule>  payrollScheduleList = entityService.findAll(PayrollSchedule.class);
		for (PayrollSchedule payrollScheduleToDelete : payrollScheduleList) {
			pScheduleComponent.deletePayrollSchedule(payrollScheduleToDelete);
		}
     	// Exercise
    	@SuppressWarnings("unused")
        PayrollRunInfo payrollRunInfo = sWorkerPayService.getNextScheduledPayrollRunInfo();
    	// Cleanup - The entire transaction should be rolled back, so the deleted PayrollSchedules shouldn't really
    	//           be deleted
    }

}
