package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.CompensationUtils;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.WageService;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckLineItem;
import com.intuit.sb.payroll.services.workerpayments.PaycheckStatus;
import com.intuit.sb.payroll.services.workerpayments.PayrollItemCategoryType;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */

public class GetPaycheckByIdTest extends WorkerPaymentServiceTestBase {
    private static WageService sWageService;
	private static Long sReadyForPayWorkerId1 = null;
	private static int sNumberOfPayrollReadyWorkers = 1;
	private String mHourlyRate = "10.00";
	private String mOvertimeRate = "1.5";
	private String mBonusAmount = "1000.00";
	private double mHourlyHours = 80.0;
	private double mOvertimeHours = 8.0;
	private static PayrollRunSummary sPayrollSummary = null;

	@BeforeClass
	public static void setUpGetPaycheckByIdTest() {
		sWageService = qaBootstrap.getEjbFromClass(WageService.class);
		try {
			getUserTransaction().begin();
			sReadyForPayWorkerId1 = createMVPayrollReadyWorker("Paul");
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpGetPaycheckByIdTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownGetPaycheckByIdTest() {
		try {
			getUserTransaction().begin();
			if (null != sReadyForPayWorkerId1) {
				cleanUpEIWorker(sReadyForPayWorkerId1);
				sReadyForPayWorkerId1 = null;
			}
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownGetPaycheckByIdTest(): " + t.getMessage());
		}
	}

	private PayrollRunSummary createAPayrollRun() throws Exception {
		Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
		workerIds[0] = sReadyForPayWorkerId1;
		CompensationItem[] wageItems = sWageService.getCompensationItems(sReadyForPayWorkerId1);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		sWageService.saveCompensationItems(sReadyForPayWorkerId1,
		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
		                                                                    updatedWageItems));
		PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		return payrollSummary;
	}

	private void cleanUpPayrollRun() throws ValidationException, ServiceFailureException {
		if (null != sPayrollSummary) {
			Long payrollRunId = sPayrollSummary.getPayrollRunId();
			cleanUpPayrollRunItems(payrollRunId);
		}
	}

	/**
	 * <P>
	 * Test getting a paycheck in a PayrollRun
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a payroll run</LI>
	 * <LI>Call WorkerPaymentService's getPaycheckById() with a valid
	 * paycheckId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the PaycheckDetails information returned for the paycheck
	 * with the given paycheckId</LI>
	 * </UL>
	 */
	@Test
	public void testGetPaycheckById() throws Exception {
		try {
			// Setup
			sPayrollSummary = createAPayrollRun();

			// Exercise
			Long paycheckId = sPayrollSummary.getPaycheckSummaries()[0].getPaycheckId();
			PaycheckDetails paycheckDetails = sWorkerPayService.getPaycheckById(paycheckId);

			// Verify
			CompensationItem[] wageItems = sWageService.getCompensationItems(sReadyForPayWorkerId1);
			
			com.intuit.sb.payroll.entity.model.Worker entityWorker = sWorkerEAO.findById(sReadyForPayWorkerId1);
			
			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sPaycheckTipsWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sPaycheckTipsWageDescStr,
			                                                          CompensationType.TipsPaycheck),
			                                     QboeLawIdConstants.sPaycheckTipsLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("286.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("77.96"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			
			fixupSourcePayrollItemDetailId(expectedPaycheckLineItems, entityWorker);
			
			PaycheckDetails expectedPaycheckDetails = new PaycheckDetails(sReadyForPayWorkerId1,
			                                                              new BigDecimal("1920.00"),
			                                                              new BigDecimal("1393.80"),
			                                                              expectedPaycheckLineItems);
			// Fix up the expected paycheckId, CheckNumber, paycheckStatus and PaycheckDistributionMethod
			expectedPaycheckDetails.setPaycheckId(paycheckDetails.getPaycheckId());
			expectedPaycheckDetails.setCheckNumber(null);
			expectedPaycheckDetails.setPaycheckStatus(PaycheckStatus.COMMITTED);
			Worker worker = sWorkerService.getWorkerById(sReadyForPayWorkerId1);
			expectedPaycheckDetails.setDistributionMethod(worker.getPaycheckDistributionMethod());
			AssertJUnit.assertTrue("Expected PaycheckDetails mismatch!", verifyPaycheckDetails(expectedPaycheckDetails,
			                                                                      paycheckDetails));
		} finally {
			// Tear down
			cleanUpPayrollRun();
		}
	}

}
