package com.intuit.sb.payroll.qa.services.workerpayments;


import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunResponse;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunState;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
* Tests the getPayrollRunById operation of the WorkerPaymentService.
*
* $Author$
* $Revision$
* $DateTime$
*
*/

public class GetPayrollRunByIdTest extends WorkerPaymentServiceTestBase {
    private static Long sReadyForPayWorkerId = null;
	private static int numberOfPayrollReadyWorkers = 1;
	
	@BeforeClass
	public static void setUpGetPayrollRunByIdTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createPayrollReadyWorker();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpGetPayrollRunByIdTest(): " + t.getMessage());
        }
	}
	
	@AfterClass
	public static void tearDownGetPayrollRunByIdTest() {
        try {
            getUserTransaction().begin();
            if (null != sReadyForPayWorkerId) {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            }  catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownGetPayrollRunByIdTest(): " + t.getMessage());
        }
	}
	
	//Utility method to compare the PayrollRunSummary and PayrollRunResponse
	private boolean compare(PayrollRunResponse payrollResponse, PayrollRunSummary payrollRunSummary) {
		//Check if the PayrollRun Ids are same
		if (!payrollResponse.getPayrollRunId().equals(payrollRunSummary.getPayrollRunId())) {
			return false;
		}
		
		//Check PayrollRunState
		if (!payrollResponse.getPayrollRunState().equals(payrollRunSummary.getPayrollRunState())) {
			return false;
		}
		
		//Check PayrollRunPeriod
        // Had to compare the dates using String, otherwise the comparison will fail due to discrepancy in milliseconds
		if (!payrollResponse.getPayrollRunPeriod().getFrequency().equals(payrollRunSummary.getPayrollPeriod().getFrequency()) ||
				!payrollResponse.getPayrollRunPeriod().getPayDate().toString().equals(payrollRunSummary.getPayrollPeriod().getPayDate().toString()) ||
				!payrollResponse.getPayrollRunPeriod().getPayrollRunDate().toString().equals(payrollRunSummary.getPayrollPeriod().getPayrollRunDate().toString()) ||
				!payrollResponse.getPayrollRunPeriod().getPeriodEndDate().toString().equals(payrollRunSummary.getPayrollPeriod().getPeriodEndDate().toString()) ||
				!payrollResponse.getPayrollRunPeriod().getPeriodStartDate().toString().equals(payrollRunSummary.getPayrollPeriod().getPeriodStartDate().toString())) {
			return false;
		}
		
		PaycheckDetails[] paycheckDetails = payrollResponse.getPaycheckDetails();
		PaycheckSummary[] paycheckSummary = payrollRunSummary.getPaycheckSummaries();
		
		//Basic check
		if (paycheckDetails == null) {
			return false;
		}
		
		//Basic check
		if (paycheckDetails.length != paycheckSummary.length) {
			return false;
		}
		
		for (int count = 0;count < paycheckDetails.length;count++) {
			//Check for PaycheckDistributionMethod
			if (paycheckDetails[count].getDistributionMethod() == null) {
				return false;
			}
			
			//Check for grossTotal
			if (paycheckDetails[count].getGrossTotal() == null) {
				return false;
			}
			
			//Check for netTotal
			if (paycheckDetails[count].getNetTotal() == null) {
				return false;
			}
			
			//Check for PaycheckId
			if (paycheckDetails[count].getPaycheckId() == null) {
				return false;
			}
			
			//Check for workerId
			if (paycheckDetails[count].getWorkerId() == null) {
				return false;
			}
		}
		
		return true;
	}
	
    /**
     * <P>Test ValidationException caused by null payrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's getPayrollRunById() with a null payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since the payrollRunId is null</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testNullGetPayrollRunById() throws Exception {
		//Exercise
		sWorkerPayService.getPayrollRunById(null);
		//Teardown - Implicit tear down in tearDownClass()
	}
	
    /**
     * <P>Test whether PayrollRunNotFoundException is thrown and null is returned by invalid payrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's getPayrollRunById() with an invalid payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown and null is returned since the payrollRunId is invalid</LI>
     * </UL>
     */
	@Test
	public void testInvalidGetPayrollRunById() throws Exception {
		//Setup - Implicit Setup in setUpClass()
		PayrollRunResponse nullResponse = null;
		try {
			//Exercise - Pass sWorkerId, which is invalid from the API point of view, to the 
			//getPayrollRunById() API  
			nullResponse = sWorkerPayService.getPayrollRunById(sWorkerId);
		} catch(ServiceFailureException e) { //Verify
			AssertJUnit.assertNull(nullResponse);			
		}
		
		//Tear down - Implicit tear down in tearDownClass()
	}
	
    /**
     * <P>Test whether correct PayrollRunResponse object is returned by a valid payrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's getPayrollRunById() with a valid payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a correct PayrollRunResponse object is returned since the payrollRunId is valid</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testValidSavedGetPayrollRunById() throws Exception {
		PayrollRunSummary payrollRunSummary = null;
		try {
			//Setup
		    Long[] workerIds = new Long[numberOfPayrollReadyWorkers];
	    	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			payrollRunSummary = sWorkerPayService.saveDraftPayrollRun(payrollRunSaveRequest);
			
			//Exercise
			PayrollRunResponse payrollResponse = sWorkerPayService.getPayrollRunById(payrollRunSummary.getPayrollRunId());
			
			//Verify
			AssertJUnit.assertTrue(compare(payrollResponse,payrollRunSummary));
		} finally {
	        //Tear down - Implicit tear down in tearDownClass()
			if (payrollRunSummary != null) {
		    	cleanUpPayrollRunItems(new Long(payrollRunSummary.getPayrollRunId()));
			}
		}
	}
	
    /**
     * <P>Test whether correct PayrollRunResponse object is returned by a valid payrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's getPayrollRunById() with a valid payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a correct PayrollRunResponse object is returned since the payrollRunId is valid</LI>
     * </UL>
     */
	@Test
	public void testValidCommitedGetPayrollRunById() throws Exception {
		PayrollRunSummary payrollRunSummary = null;
		try {
			//Setup
		    Long[] workerIds = new Long[numberOfPayrollReadyWorkers];
	    	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
			payrollRunSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
			
			//Exercise
			PayrollRunResponse payrollResponse = sWorkerPayService.getPayrollRunById(payrollRunSummary.getPayrollRunId());
			
			//Verify
			AssertJUnit.assertTrue(compare(payrollResponse,payrollRunSummary));
		} finally {
	        //Tear down - Implicit tear down in tearDownClass()
			if (payrollRunSummary != null) {
		    	cleanUpPayrollRunItems(new Long(payrollRunSummary.getPayrollRunId()));
			}
		}
	}
	
	/**
	 * Tim's temporary test driver...
	 * @throws Exception
	 */
	@Test
	public void testGetNextPayrollRun() throws Exception {
		PayrollRunSummary payrollRunSummary = null;
		try {
			//Setup
		    Long[] workerIds = new Long[numberOfPayrollReadyWorkers];
	    	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
			payrollRunSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
			
			//Exercise
			PayrollRunResponse payrollResponse = sWorkerPayService.getPayrollRunById(new Long(0));
			
			//Verify
			AssertJUnit.assertEquals(PayrollRunState.DRAFT, payrollResponse.getPayrollRunState());
		} catch(Exception e) {
			sLogger.error("Error!", e);
		} finally {
		    //Tear down - Implicit tear down in tearDownClass()
			if (payrollRunSummary != null) {
		    	cleanUpPayrollRunItems(new Long(payrollRunSummary.getPayrollRunId()));
			}
		}
	}
	
	@Test
	public void testGetPayrollRunCSVById() throws Exception {
		PayrollRunSummary payrollRunSummary = null;
		try {
			//Setup
		    Long[] workerIds = new Long[numberOfPayrollReadyWorkers];
	    	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
			payrollRunSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
			
			//Exercise
			String payrollResponse = sWorkerPayService.getPayrollRunCSVById(payrollRunSummary.getPayrollRunId());
			
			//Verify
			AssertJUnit.assertNotNull("CSV was null", payrollResponse);
		} finally {
	        //Tear down - Implicit tear down in tearDownClass()
			if (payrollRunSummary != null) {
		    	cleanUpPayrollRunItems(new Long(payrollRunSummary.getPayrollRunId()));
			}
		}
	}
}
