/**
 * 
 */
package com.intuit.sb.payroll.qa.services.workerpayments;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class GetPayrollRunSummaryByIdTest extends WorkerPaymentServiceTestBase
{
    private static Long sReadyForPayWorkerId = null;
    private static Long sPayrollRunId = null;
	private static int numberOfPayrollReadyWorkers = 1;
	
	@BeforeClass
	public static void setUpGetPayrollRunSummaryByIdTest()
	{
        try
        {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createPayrollReadyWorker();
            //Create a dummy Id for testing purpose
            sPayrollRunId = new Long("923478346");
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
            AssertJUnit.fail("Error in setUpGetPayrollRunSummaryByIdTest(): " + t.getMessage());
        }
	}
	
	@AfterClass
	public static void tearDownGetPayrollRunSummaryByIdTest()
	{
        try
        {
            getUserTransaction().begin();
            if (null != sReadyForPayWorkerId)
            {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
            AssertJUnit.fail("Error in tearDownGetPayrollRunSummaryByIdTest(): " + t.getMessage());
        }
	}
	
	private boolean compare(PayrollRunSummary payrollSummary, PayrollRunSummary payrollRunSummary)
	{
		//Compare PayrollRun Id
		if(!payrollSummary.getPayrollRunId().equals(payrollRunSummary.getPayrollRunId()))
		{
			return false;
		}
		
		//Compare PayrollRunState
		if(!payrollSummary.getPayrollRunState().equals(payrollRunSummary.getPayrollRunState()))
		{
			return false;
		}
		
		//Compare total amount paid to employees
		if(!payrollSummary.getTotalEmployeeGross().equals(payrollRunSummary.getTotalEmployeeGross()))
		{
			return false;
		}
		
		//Compare total taxes paid by employees
		if(!payrollSummary.getTotalEmployeePaidTax().equals(payrollRunSummary.getTotalEmployeePaidTax()))
		{
			return false;
		}
		
		//Compare total taxes paid by employer
		if(!payrollSummary.getTotalEmployerPaidTax().equals(payrollRunSummary.getTotalEmployerPaidTax()))
		{
			return false;
		}
		
		//Compare PayrollRunPeriod
        // Had to compare the dates using String, otherwise the comparison will fail due to discrepancy in milliseconds
		if(!payrollSummary.getPayrollPeriod().getFrequency().equals(payrollRunSummary.getPayrollPeriod().getFrequency()) ||
				!payrollSummary.getPayrollPeriod().getPayDate().toString().equals(payrollRunSummary.getPayrollPeriod().getPayDate().toString()) ||
				!payrollSummary.getPayrollPeriod().getPayrollRunDate().toString().equals(payrollRunSummary.getPayrollPeriod().getPayrollRunDate().toString()) ||
				!payrollSummary.getPayrollPeriod().getPeriodEndDate().toString().equals(payrollRunSummary.getPayrollPeriod().getPeriodEndDate().toString()) ||
				!payrollSummary.getPayrollPeriod().getPeriodStartDate().toString().equals(payrollRunSummary.getPayrollPeriod().getPeriodStartDate().toString()))
		{
			return false;
		}
		
		//Everything is true
		return true;
	}
	
    /**
     * <P>Test ValidationException caused by null payrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IPayEmployeeService's getPayrollRunSummaryById() with a null payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since the payrollRunId is null</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testNullGetPayrollRunSummaryById() throws Exception
	{
		//Exercise
		sWorkerPayService.getPayrollRunSummaryById(null);
		//Teardown - Implicit tear down in tearDownClass()
	}
	
    /**
     * <P>Test whether PayrollRunNotFoundException is thrown and null is returned by invalid payrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IPayEmployeeService's getPayrollRunSummaryById() with an invalid payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown and null is returned since the payrollRunId is invalid</LI>
     * </UL>
     */
	@Test
	public void testInvalidGetPayrollRunSummaryById() throws Exception
	{
		//Setup - Implicit Setup in setUpClass()
		PayrollRunSummary nullSummary = null;
		try
		{
			//Exercise - Pass sWorkerId, which is invalid from the API point of view, to the 
			//getPayrollRunSummaryById() API  
			nullSummary = sWorkerPayService.getPayrollRunSummaryById(sWorkerId);
		}
		//Verify
		catch(ServiceFailureException e)
		{
			AssertJUnit.assertNull(nullSummary);			
		}
		
		//Tear down - Implicit tear down in tearDownClass()
	}
	
    /**
     * <P>Test whether correct PayrollRunSummary object is returned by a valid payrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IPayEmployeeService's getPayrollRunSummaryById() with a valid payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a correct PayrollRunSummary object is returned since the payrollRunId is valid</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testValidSavedGetPayrollRunSummaryById() throws Exception
	{
		PayrollRunSummary payrollRunSummary = null;
		try
		{
			//Setup
		    Long[] workerIds = new Long[numberOfPayrollReadyWorkers];
	    	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,sPayrollRunId);
			payrollRunSummary = sWorkerPayService.saveDraftPayrollRun(payrollRunSaveRequest);
			
			//Exercise
			PayrollRunSummary retrievedPayrollRunSummary = sWorkerPayService.getPayrollRunSummaryById(payrollRunSummary.getPayrollRunId());
			
			//Verify
			AssertJUnit.assertTrue(compare(retrievedPayrollRunSummary,payrollRunSummary));
		}
		//Tear down - Implicit tear down in tearDownClass()
		finally
		{
			if(payrollRunSummary != null)
			{
		    	cleanUpPayrollRunItems(new Long(payrollRunSummary.getPayrollRunId()));
			}
		}
	}
	
    /**
     * <P>Test whether correct PayrollRunSummary object is returned by a valid payrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IPayEmployeeService's getPayrollRunSummaryById() with a valid payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a correct PayrollRunSummary object is returned since the payrollRunId is valid</LI>
     * </UL>
     */
	@Test
	public void testValidCommitedGetPayrollRunSummaryById() throws Exception
	{
		PayrollRunSummary payrollRunSummary = null;
		try
		{
			//Setup
		    Long[] workerIds = new Long[numberOfPayrollReadyWorkers];
	    	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,sPayrollRunId);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
			payrollRunSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
			
			//Exercise
			PayrollRunSummary retrievedPayrollRunSummary = sWorkerPayService.getPayrollRunSummaryById(payrollRunSummary.getPayrollRunId());
			
			//Verify
			AssertJUnit.assertTrue(compare(retrievedPayrollRunSummary,payrollRunSummary));
		}
		//Tear down - Implicit tear down in tearDownClass()
		finally
		{
			if(payrollRunSummary != null)
			{
		    	cleanUpPayrollRunItems(new Long(payrollRunSummary.getPayrollRunId()));
			}
		}
	}
}
