package com.intuit.sb.payroll.qa.services.workerpayments;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.PaycheckStatus;
import com.intuit.sb.payroll.services.workerpayments.PaycheckUpdateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunUpdateRequest;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class UpdatePayrollRunExceptionsTest extends WorkerPaymentServiceTestBase {
    private static Long sReadyForPayWorkerId = null;
	private static int sNumberOfPayrollReadyWorkers = 1;
	private static PayrollRunSummary sPayrollSummary = null;

    @BeforeClass
    public static void setUpUpdatePayrollRunExceptionsTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createMVPayrollReadyWorker("Paul");
            sPayrollSummary = createAPayrollRun();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpUpdatePayrollRunExceptionsTest(): " + t.getMessage());
        }
    }

    @AfterClass
    public static void tearDownUpdatePayrollRunExceptionsTest() {
        try {
            getUserTransaction().begin();
            if (null != sPayrollSummary) {
            	Long payrollRunId = sPayrollSummary.getPayrollRunId();
            	cleanUpPayrollRunItems(payrollRunId);
            }
            if (null != sReadyForPayWorkerId) {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownUpdatePayrollRunExceptionsTest(): " + t.getMessage());
        }
    }

    private static PayrollRunSummary createAPayrollRun() throws Exception {
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
    	return payrollSummary;
    }
    
    /**
     * <P>Test ServiceFailureException caused by null PayrollRunUpdateRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with a null PayrollRunUpdateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PayrollRunUpdateRequest object is null</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullPayrollRunUpdateRequest() throws Exception {
    	// Exercise
        sWorkerPayService.updatePayrollRun(null);
    }

    /**
     * <P>Test ServiceFailureException caused by null PaycheckUpdateRequest array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with a null PaycheckUpdateRequest array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PaycheckUpdateRequest array is null</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullPaycheckUpdateRequestArray() throws Exception {
    	// Setup
    	PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
    	payrollRunUpdateRequest.setPayrollRunId(sPayrollSummary.getPayrollRunId());
    	// Exercise
        sWorkerPayService.updatePayrollRun(payrollRunUpdateRequest);
    }

    /**
     * <P>Test ServiceFailureException caused by null PaycheckUpdateRequest array element</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with a null PaycheckUpdateRequest array element</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PaycheckUpdateRequest array element is null</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullPaycheckUpdateRequestElement() throws Exception {
    	// Setup
    	PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
    	payrollRunUpdateRequest.setPayrollRunId(sPayrollSummary.getPayrollRunId());
    	PaycheckUpdateRequest[] paycheckUpdateRequests = new PaycheckUpdateRequest[2];
    	paycheckUpdateRequests[0] = new PaycheckUpdateRequest();
    	paycheckUpdateRequests[0].setCheckNumber("101");
    	paycheckUpdateRequests[0].setPaycheckId(sPayrollSummary.getPaycheckSummaries()[0].getPaycheckId());
    	paycheckUpdateRequests[0].setPaycheckStatus(PaycheckStatus.COMPLETE);
    	paycheckUpdateRequests[1] = null;
    	payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
    	// Exercise
        sWorkerPayService.updatePayrollRun(payrollRunUpdateRequest);
    }

    /**
     * <P>Test ServiceFailureException caused by uncommitted PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with an uncommitted PayrollRun</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PayrollRun is not committed</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testUnCommittedPayrollRun() throws Exception {
    	// Setup
    	PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
    	payrollRunUpdateRequest.setPayrollRunId(0L);
    	PaycheckUpdateRequest[] paycheckUpdateRequests = new PaycheckUpdateRequest[1];
    	paycheckUpdateRequests[0] = new PaycheckUpdateRequest();
    	paycheckUpdateRequests[0].setCheckNumber("101");
    	paycheckUpdateRequests[0].setPaycheckId(sPayrollSummary.getPaycheckSummaries()[0].getPaycheckId());
    	paycheckUpdateRequests[0].setPaycheckStatus(PaycheckStatus.COMPLETE);
    	payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
    	// Exercise
        sWorkerPayService.updatePayrollRun(payrollRunUpdateRequest);
    }

    /**
     * <P>Test ValidationException caused by null PaycheckId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with null PaycheckId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since the PaycheckId is null</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullPaycheckId() throws Exception {
    	// Setup
    	PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
    	payrollRunUpdateRequest.setPayrollRunId(sPayrollSummary.getPayrollRunId());
    	PaycheckUpdateRequest[] paycheckUpdateRequests = new PaycheckUpdateRequest[1];
    	paycheckUpdateRequests[0] = new PaycheckUpdateRequest();
    	paycheckUpdateRequests[0].setCheckNumber("101");
    	paycheckUpdateRequests[0].setPaycheckId(null);
    	paycheckUpdateRequests[0].setPaycheckStatus(PaycheckStatus.COMPLETE);
    	payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
    	// Exercise
        sWorkerPayService.updatePayrollRun(payrollRunUpdateRequest);
    }

    /**
     * <P>Test ServiceFailureException caused by null PaycheckStatus</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with null PaycheckStatus</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PaycheckStatus is null</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullPaycheckStatus() throws Exception {
    	// Setup
    	PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
    	payrollRunUpdateRequest.setPayrollRunId(sPayrollSummary.getPayrollRunId());
    	PaycheckUpdateRequest[] paycheckUpdateRequests = new PaycheckUpdateRequest[1];
    	paycheckUpdateRequests[0] = new PaycheckUpdateRequest();
    	paycheckUpdateRequests[0].setCheckNumber("101");
    	paycheckUpdateRequests[0].setPaycheckId(sPayrollSummary.getPaycheckSummaries()[0].getPaycheckId());
    	paycheckUpdateRequests[0].setPaycheckStatus(null);
    	payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
    	// Exercise
        sWorkerPayService.updatePayrollRun(payrollRunUpdateRequest);
    }

    /**
     * <P>Test ServiceFailureException caused by non-existent PayrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with a non-existent PayrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PayrollRunId is non-existent</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNonExistentPayrollRunId() throws Exception {
    	// Setup
    	PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
    	payrollRunUpdateRequest.setPayrollRunId(756988425L);
    	PaycheckUpdateRequest[] paycheckUpdateRequests = new PaycheckUpdateRequest[1];
    	paycheckUpdateRequests[0] = new PaycheckUpdateRequest();
    	paycheckUpdateRequests[0].setCheckNumber("101");
    	paycheckUpdateRequests[0].setPaycheckId(sPayrollSummary.getPaycheckSummaries()[0].getPaycheckId());
    	paycheckUpdateRequests[0].setPaycheckStatus(PaycheckStatus.COMPLETE);
    	payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
    	// Exercise
        sWorkerPayService.updatePayrollRun(payrollRunUpdateRequest);
    }

    /**
     * <P>Test ValidationException caused by non-existent PaycheckId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with a non-existent PaycheckId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since the PaycheckId is non-existent</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNonExistentPaycheckId() throws Exception {
    	// Setup
    	PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
    	payrollRunUpdateRequest.setPayrollRunId(sPayrollSummary.getPayrollRunId());
    	PaycheckUpdateRequest[] paycheckUpdateRequests = new PaycheckUpdateRequest[1];
    	paycheckUpdateRequests[0] = new PaycheckUpdateRequest();
    	paycheckUpdateRequests[0].setCheckNumber("101");
    	paycheckUpdateRequests[0].setPaycheckId(861594215L);
    	paycheckUpdateRequests[0].setPaycheckStatus(PaycheckStatus.COMPLETE);
    	payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
    	// Exercise
        sWorkerPayService.updatePayrollRun(payrollRunUpdateRequest);
    }

}
