package com.intuit.sb.payroll.qa.services.workerpayments;

import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.PaycheckStatus;
import com.intuit.sb.payroll.services.workerpayments.PaycheckSummary;
import com.intuit.sb.payroll.services.workerpayments.PaycheckUpdateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunUpdateRequest;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class UpdatePayrollRunTest extends WorkerPaymentServiceTestBase {
    private static final String TEST_MEMO = "testMemo";
	private static Long sReadyForPayWorkerId1 = null;
    private static Long sReadyForPayWorkerId2 = null;
	private static int sNumberOfPayrollReadyWorkers = 2;
	private static PayrollRunSummary sPayrollSummary = null;

    @BeforeClass
    public static void setUpUpdatePayrollRunTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId1 = createMVPayrollReadyWorker("Paul");
            sReadyForPayWorkerId2 = createMVPayrollReadyWorker("Simon");
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpUpdatePayrollRunTest(): " + t.getMessage());
        }
    }

    @AfterClass
    public static void tearDownUpdatePayrollRunTest() {
        try {
            getUserTransaction().begin();
            if (null != sReadyForPayWorkerId1) {
                cleanUpEIWorker(sReadyForPayWorkerId1);
                sReadyForPayWorkerId1 = null;
            }
            if (null != sReadyForPayWorkerId2) {
                cleanUpEIWorker(sReadyForPayWorkerId2);
                sReadyForPayWorkerId2 = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownUpdatePayrollRunTest(): " + t.getMessage());
        }
    }

    private PayrollRunSummary createAPayrollRun() throws Exception {
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId1;
    	workerIds[1] = sReadyForPayWorkerId2;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
    	return payrollSummary;
    }
    
    private void cleanUpPayrollRun() throws ValidationException, ServiceFailureException {
        if (null != sPayrollSummary) {
        	Long payrollRunId = sPayrollSummary.getPayrollRunId();
        	cleanUpPayrollRunItems(payrollRunId);
        }
    }
    
    private boolean verifyUpdate(PaycheckUpdateRequest[] paycheckUpdateRequest, PaycheckSummary[] updatedSummaries) {
    	boolean result = true;
    	for (PaycheckUpdateRequest updateRequest : paycheckUpdateRequest) {
        	for (PaycheckSummary paycheckSummary : updatedSummaries) {
        		if (updateRequest.getPaycheckId().equals(paycheckSummary.getPaycheckId())) {
        			boolean checkNumberEquals = false;
        			if (null == updateRequest.getCheckNumber()) {
        				checkNumberEquals = (null == paycheckSummary.getCheckNumber());
        			} else {
        				checkNumberEquals = updateRequest.getCheckNumber().equals(paycheckSummary.getCheckNumber());
        			}
        			boolean memoEquals = false;
        			if (paycheckSummary.getMemo().equals(TEST_MEMO)) {
        				memoEquals = true;
        			}
        			result = result && (checkNumberEquals) && (memoEquals) &&
        					 updateRequest.getPaycheckStatus().equals(paycheckSummary.getPaycheckStatus());
            		if (!result) {
            			break;
            		}
            		break;
        		}
    		}
    		if (!result) {
    			break;
    		}
    	}
    	return result;
    }
    
    /**
     * <P>Test updating all paychecks in the PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a valid PayrollRunUpdateRequest object</LI>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with a valid PayrollRunUpdateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paychecks in the payroll run are updated</LI>
     * </UL>
     */
    @Test
    public void testUpdateAllPaychecksInPayrollRun() throws Exception {
    	try {
        	//Setup
            sPayrollSummary = createAPayrollRun();
        	PayrollRunUpdateRequest updateRequest = new PayrollRunUpdateRequest();
        	updateRequest.setPayrollRunId(sPayrollSummary.getPayrollRunId());
        	List<PaycheckUpdateRequest> paycheckUpdates = new ArrayList<PaycheckUpdateRequest>();
        	int i = 1;
        	PaycheckSummary[] paycheckSummaries = sPayrollSummary.getPaycheckSummaries();
        	for (PaycheckSummary paycheckSummary : paycheckSummaries) {
        		PaycheckUpdateRequest paycheckUpdate = new PaycheckUpdateRequest();
        		paycheckUpdate.setCheckNumber("123"+(i++));
        		paycheckUpdate.setMemo(TEST_MEMO);
        		paycheckUpdate.setPaycheckId(paycheckSummary.getPaycheckId());
        		paycheckUpdate.setPaycheckStatus(PaycheckStatus.COMPLETE);
        		paycheckUpdates.add(paycheckUpdate);
			}
        	PaycheckUpdateRequest[] paycheckUpdatesArray = paycheckUpdates.toArray(new PaycheckUpdateRequest[paycheckUpdates.size()]);
        	updateRequest.setPaycheckUpdateRequests(paycheckUpdatesArray);
        	
        	//Exercise
        	PayrollRunSummary payrollSummary = sWorkerPayService.updatePayrollRun(updateRequest);
        	
        	// Verify
        	PaycheckSummary[] updatedSummaries = payrollSummary.getPaycheckSummaries();
        	AssertJUnit.assertTrue("Expected updates mismatch!", verifyUpdate(paycheckUpdatesArray, updatedSummaries));
    	} finally {
        	//Tear down
    		cleanUpPayrollRun();
    	}
    }
    
    /**
     * <P>Test updating one paycheck in the PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a valid PayrollRunUpdateRequest object</LI>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with a valid PayrollRunUpdateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck in the payroll run is updated</LI>
     * </UL>
     */
    @Test
    public void testUpdateOnePaychecksInPayrollRun() throws Exception {
    	try {
        	//Setup
            sPayrollSummary = createAPayrollRun();
        	PayrollRunUpdateRequest updateRequest = new PayrollRunUpdateRequest();
        	updateRequest.setPayrollRunId(sPayrollSummary.getPayrollRunId());
        	PaycheckUpdateRequest[] paycheckUpdatesArray = new PaycheckUpdateRequest[1];
        	PaycheckSummary[] paycheckSummaries = sPayrollSummary.getPaycheckSummaries();
    		PaycheckUpdateRequest paycheckUpdate = new PaycheckUpdateRequest();
    		paycheckUpdate.setCheckNumber("123");
    		paycheckUpdate.setMemo(TEST_MEMO);
    		paycheckUpdate.setPaycheckId(paycheckSummaries[paycheckSummaries.length-1].getPaycheckId());
    		paycheckUpdate.setPaycheckStatus(PaycheckStatus.COMPLETE);
    		paycheckUpdatesArray[0] = paycheckUpdate;
        	updateRequest.setPaycheckUpdateRequests(paycheckUpdatesArray);
        	
        	//Exercise
        	PayrollRunSummary payrollSummary = sWorkerPayService.updatePayrollRun(updateRequest);
        	
        	// Verify
        	PaycheckSummary[] updatedSummaries = payrollSummary.getPaycheckSummaries();
        	AssertJUnit.assertTrue("Expected updates mismatch!", verifyUpdate(paycheckUpdatesArray, updatedSummaries));
    	} finally {
        	//Tear down
    		cleanUpPayrollRun();
    	}
    }
    
    /**
     * <P>Test resetting status of one paycheck in the PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a valid PayrollRunUpdateRequest object</LI>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with a valid PayrollRunUpdateRequest object</LI>
     * <LI>Setup a valid PayrollRunUpdateRequest object with different values</LI>
     * <LI>Call WorkerPaymentService's updatePayrollRun() with a valid PayrollRunUpdateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck in the payroll run is updated</LI>
     * </UL>
     */
    @Test
    public void testResetOnePaychecksInPayrollRun() throws Exception {
    	try {
        	//Setup
            sPayrollSummary = createAPayrollRun();
        	PayrollRunUpdateRequest updateRequest = new PayrollRunUpdateRequest();
        	updateRequest.setPayrollRunId(sPayrollSummary.getPayrollRunId());
        	PaycheckUpdateRequest[] paycheckUpdatesArray = new PaycheckUpdateRequest[1];
        	PaycheckSummary[] paycheckSummaries = sPayrollSummary.getPaycheckSummaries();
    		PaycheckUpdateRequest paycheckUpdate = new PaycheckUpdateRequest();
    		paycheckUpdate.setCheckNumber("123");
    		paycheckUpdate.setMemo(TEST_MEMO);
    		paycheckUpdate.setPaycheckId(paycheckSummaries[paycheckSummaries.length-1].getPaycheckId());
    		paycheckUpdate.setPaycheckStatus(PaycheckStatus.COMPLETE);
    		paycheckUpdatesArray[0] = paycheckUpdate;
        	updateRequest.setPaycheckUpdateRequests(paycheckUpdatesArray);
        	
        	//Exercise
        	PayrollRunSummary payrollSummary = sWorkerPayService.updatePayrollRun(updateRequest);
        	
        	// Verify
        	PaycheckSummary[] updatedSummaries = payrollSummary.getPaycheckSummaries();
        	AssertJUnit.assertTrue("Expected updates mismatch!", verifyUpdate(paycheckUpdatesArray, updatedSummaries));
        	
        	// Now update with different values
        	payrollSummary = sWorkerPayService.getPayrollRunSummaryById(sPayrollSummary.getPayrollRunId());
        	paycheckSummaries = payrollSummary.getPaycheckSummaries();
    		paycheckUpdate.setCheckNumber(null);
    		paycheckUpdate.setPaycheckId(paycheckSummaries[paycheckSummaries.length-1].getPaycheckId());
    		paycheckUpdate.setPaycheckStatus(PaycheckStatus.NOT_SET);
    		paycheckUpdatesArray[0] = paycheckUpdate;
        	updateRequest.setPaycheckUpdateRequests(paycheckUpdatesArray);
        	
        	//Exercise
        	payrollSummary = sWorkerPayService.updatePayrollRun(updateRequest);
        	
        	// Verify
        	updatedSummaries = payrollSummary.getPaycheckSummaries();
        	AssertJUnit.assertTrue("Expected updates mismatch!", verifyUpdate(paycheckUpdatesArray, updatedSummaries));
    	} finally {
        	//Tear down
    		cleanUpPayrollRun();
    	}
    }
    
}
