package com.intuit.sb.payroll.qa.services.workerpayments;


import java.util.Calendar;

import javax.transaction.Status;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.PayrollScheduleDateUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.services.workerpayments.PayrollSchedule;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;
import com.intuit.sb.payroll.services.workerpayments.impl.PayrollFrequencyMapper;


/**
 * <P>The <CODE>UpdatePayrollScheduleTest</CODE> class is the test class for testing the UpdatePayrollSchedule API.</p>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class UpdatePayrollScheduleTest extends WorkerPaymentServiceTestBase
{

    @AfterMethod
    public void tearDown() throws Exception
    {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE)
        {
        	BasicTestBase.deleteAllPayrollSchedule();
        }
        super.tearDown();
    }
    
    @BeforeMethod
    public void testInit() throws Exception {
        sDefaultPayrollDayOffset = 3;
        sDefaultPayrollRunDayOffset = 1;
        sDefaultPayPeriodEndDayOfMonth1 = 0;
        sDefaultPayPeriodEndDayOfMonth2 = 0;
    	PayrollScheduleTestUtils.createDefaultSchedule();
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly in happy path secnario.
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1 </LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2 </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test(enabled=false, groups={"broken"})
    public void updateFrequencySemiMonthly() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 7;
        sDefaultPayPeriodEndDayOfMonth2 = 14;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        // Exercise and Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.SEMIMONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthly in happy path secnario.
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1 </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyMonthly() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 7;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        // Exercise and Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.MONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthlySameDay in Negative 
     * scenario which throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1, which is same to second date </LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2, which is same to second date </LI>
     * <LI>Finally setting the Frequency to Semi monthly  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void updateFrequencySemiMonthlySameDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 14;
        sDefaultPayPeriodEndDayOfMonth2 = 14;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        pSch.setFrequency(PayrollScheduleFrequency.SEMIMONTHLY);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthlySameDay in positive 
     * scenario which is not bother about the setting the sDeafultPayPeriodEndDayOfMonth2 parameter
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1, which is same to second date </LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2, which is same to second date but should not have impact</LI>
     * <LI>Finally setting the Frequency to monthly and calling the updatePayrollschedule API </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyMonthlySameDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 14;
        sDefaultPayPeriodEndDayOfMonth2 = 14;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        // Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.MONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly when first date is greater then second Date  
     * it should throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1, it is greater then the second date </LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2, it is lesser  then the first date </LI>
     * <LI>Finally setting the Frequency to Semi monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    //@Test(enabled=false, expectedExceptions = ValidationException.class)
    public void updateFrequencySemiMonthlyEndDayNegative() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 27;
        sDefaultPayPeriodEndDayOfMonth2 = 7;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        pSch.setFrequency(PayrollScheduleFrequency.SEMIMONTHLY);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly when second date is in between the last day and 28  
     * it should throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2, which is 29 falls in between 28 and last day </LI>
     * <LI>Finally setting the Frequency to Semi monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void updateFrequencySemiMonthlySecondLastDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 28;
        sDefaultPayPeriodEndDayOfMonth2 = 29;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        pSch.setFrequency(PayrollScheduleFrequency.SEMIMONTHLY);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly in happy path secnario when second date is 28 and first date is 27
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1 </LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2 </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test(enabled=false, groups={"broken"})
    public void updateFrequencySemiMonthlyBoundryLast() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 27;
        sDefaultPayPeriodEndDayOfMonth2 = 28;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2); 
        // Exercise and Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.SEMIMONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly in happy path scenario when second date is 2 and first date is 1
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1 </LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2 </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three parameters. </L1>
     * <LI>These three parameter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test(enabled=false, groups={"broken"})
    public void updateFrequencySemiMonthlyBoundryFirst() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 1;
        sDefaultPayPeriodEndDayOfMonth2 = 2;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);  
        // Exercise and Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.SEMIMONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly in happy path scenario when second date is 28 and first date is 1
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1 </LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2 </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test(enabled=false, groups={"broken"})
    public void updateFrequencySemiMonthlyBoundryExtremeinMonth() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 1;
        sDefaultPayPeriodEndDayOfMonth2 = 28;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);  
        // Exercise and Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.SEMIMONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly when second date is in invallid   
     * it should throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2, which is 45 and invalid </LI>
     * <LI>Finally setting the Frequency to Semi monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test (expectedExceptions = ValidationException.class)
    public void updateFrequencySemiMonthlyBoundryExtremeBeyond() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 15;
        sDefaultPayPeriodEndDayOfMonth2 = 45;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        pSch.setFrequency(PayrollScheduleFrequency.SEMIMONTHLY);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly when first date is in invallid   
     * it should throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1, which is invalid</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2</LI>
     * <LI>Finally setting the Frequency to Semi monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test (expectedExceptions = ValidationException.class)
    public void updateFrequencySemiMonthlyBoundryExtremeNegative() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = -1;
        sDefaultPayPeriodEndDayOfMonth2 = 25;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        pSch.setFrequency(PayrollScheduleFrequency.SEMIMONTHLY);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthly in happy path secnario for first date
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1 </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyMonthlyBoundryFirst() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 1;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        // Exercise and Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.MONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthly in happy path scenario for last date as 28
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1 </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three parameters. </L1>
     * <LI>These three parameter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyMonthlyBoundryLast() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 28;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.MONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthly when first date is in invallid   
     * it should throws the exception InvalidPayrollScheduleException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1, which is invalid</LI>
     * <LI>Finally setting the Frequency to monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void updateFrequencyMonthlyExtreme() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 32;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setFrequency(PayrollScheduleFrequency.MONTHLY);
        // verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthly when first date is in invallid but falls betweel 28 and last day
     * it should throws the exception InvalidPayrollScheduleException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1, which is invalid</LI>
     * <LI>Finally setting the Frequency to Semi monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void updateFrequencyMonthlyExtremeinMonth() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 29;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setFrequency(PayrollScheduleFrequency.MONTHLY);
        // verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthly when first date is in invallid and zero
     * it should throws the exception InvalidPayrollScheduleException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1, which is invalid</LI>
     * <LI>Finally setting the Frequency to monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void updateFrequencyMonthlyLowest() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 0;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setFrequency(PayrollScheduleFrequency.MONTHLY);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthly when first date is in invallid and -1
     * it should throws the exception InvalidPayrollScheduleException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1, which is invalid</LI>
     * <LI>Finally setting the Frequency to monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void updateFrequencyMonthlyNeagtive() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = -1;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setFrequency(PayrollScheduleFrequency.MONTHLY);
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyNull
     * it should throws the exception ServiceFailureException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Setting the Frequency to null</LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ServiceFailureException.class)
    public void updateFrequencyNull() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        pSch.setFrequency(null);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyNull
     * it should throws the exception ServiceFailureException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with null result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ServiceFailureException.class)
    public void updatePayrollScheduleAsNull() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(null);
        
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyNotSet
     * it should throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Setting the Frequency to null</LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void updateFrequencyNotSet() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        pSch.setFrequency(null);
        pSch.setFrequency(PayrollScheduleFrequency.NOT_SET);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthly in happy path secnario for LAST_DAY_OF_MONTH
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1 </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyMonthlyLastDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = PayrollSchedule.LAST_DAY_OF_MONTH;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.MONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly in happy path secnario when second date is LAST_DAY_OF_MONTH and first date is 26
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1 </LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2 </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date which is null in this case and payrollfrequency</LI>
     * </UL>
     */
    @Test(enabled=false, groups={"broken"})
    public void updateFrequencySemiMonthlyLastDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth1 = 26;
        sDefaultPayPeriodEndDayOfMonth2 = PayrollSchedule.LAST_DAY_OF_MONTH;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, null, PayrollScheduleFrequency.SEMIMONTHLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyWeeklyLeapYear when End date is LeapYear, in happy path secnario, this will check whether leap year has been taken care or not 
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value near to Leap yr time frame </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyWeeklyLeapYear() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.MARCH, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.WEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyBiWeeklyLeapYear when End date is LeapYear, in happy path secnario, this will check whether leap year has been taken care or not 
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value near to Leap yr time frame </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyBiWeeklyLeapYear() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.MARCH, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyWeeklyNotLeapYear when End date is Not LeapYear, in happy path secnario, we are trying same day in non leap yr 
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value near to Leap yr time frame </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyWeeklyNotLeapYear() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2009, Calendar.MARCH, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.WEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyBiWeekly when End date is Not LeapYear, in happy path secnario, we are trying same day in non leap yr 
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value near to Leap yr time frame </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyBiWeeklyNotLeapYear() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2009, Calendar.MARCH, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyWeekly when End date is Monday in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on Monday  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyWeeklySetingEndDateMonDay() throws Exception
    {
        // Setup
        createDefaultPayrollSchedule();
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.WEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyWeekly when End date is Tuesday in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on Tuesday  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyWeeklySetingEndDatTuesDay() throws Exception
    {
        // Setup
        createDefaultPayrollSchedule();
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.APRIL, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.WEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyWeekly when End date is WednesDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on WednesDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyWeeklySetingEndDateWednesDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.OCTOBER, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.WEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyWeekly when End date is thursDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on thursDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyWeeklySetingEndDateThursDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.MAY, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.WEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyWeekly when End date is FriDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on FriDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyWeeklySetingEndDateFriDay() throws Exception
    {
        // Setup
        createDefaultPayrollSchedule();
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.FEBRUARY, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.WEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyWeekly when End date is SatDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on SatDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyWeeklySetingEndDateSatDay() throws Exception
    {
        // Setup
        createDefaultPayrollSchedule();
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.NOVEMBER, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.WEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyWeekly when End date is SunDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on SunDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyWeeklySetingEndDateSunDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2009, Calendar.FEBRUARY, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.WEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyBiWeekly when End date is MonDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on MonDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyBiWeeklySetingEndDateMonDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyBiWeekly when End date is TuesDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on TuesDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyBiWeeklySetingEndDatTuesDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.APRIL, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyBiWeekly when End date is WednesDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on WednesDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyBiWeeklySetingEndDateWednesDay() throws Exception
    {
        // Setup
        createDefaultPayrollSchedule();
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.OCTOBER, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyBiWeekly when End date is ThursDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on ThursDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyBiWeeklySetingEndDateThursDay() throws Exception
    {
        // Setup
        createDefaultPayrollSchedule();
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.MAY, 1);
        // Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyBiWeekly when End date is FriDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on FriDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyBiWeeklySetingEndDateFriDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.FEBRUARY, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyBiWeekly when End date is SaturDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining End Date value which falls on SaturDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1>Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI>These three paremeter will consist of the payrollschedule,</L1>
     * <L1>date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyBiWeeklySetingEndDateSatDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.NOVEMBER, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyBiWeekly when End date is SunDay in happy path secnario  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI> Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI> Defining End Date value which falls on SunDay  </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Exercise and Verification both are taken care in exercisePayrollSchedule </L1>
     * <L1> Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI> These three paremeter will consist of the payrollschedule,</L1>
     * <L1> date and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updateFrequencyBiWeeklySetingEndDateSunDay() throws Exception
    {
        // Setup
        createDefaultPayrollSchedule();
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2009, Calendar.FEBRUARY, 1);
        // Exercise with Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updatePeriodEndDateNull
     * it should throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Setting the End date to null</LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */

    @Test(expectedExceptions = ValidationException.class)
    public void updatePeriodEndDateNull() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        pSch.setPeriodEndDate(null);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updatePeriodStartDateNull when Start date is null it should not give any impact   
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI> Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI> Defining and setting start Date value as null  </LI>
     * <LI> setting through updatePayrollSchedule </LI>
     * </OL>
     * 
     * <UL>
     * <L1> Verification is taken care in verifyPayrollSchedule </L1>
     * <L1> Calling the function exercisePayrollSchedule with three paraemeters. </L1>
     * <LI> These three paremeter will consist of the payrollschedule,</L1>
     * <L1> date which is null and payrollfrequency</LI>
     * </UL>
     */
    @Test
    public void updatePeriodStartDateNull() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        pSch.setPeriodStartDate(null);
        Calendar pDate = DateUtil.createCalendar(2009, Calendar.FEBRUARY, 1);
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
        // Verify
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateWhenPayDayOffsetEarlyThenEndDate1, this we are setting through off set and pay date  
     * it should throws the exception ValidationException  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI> Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI> Defining and setting end Date  </LI>
     * <LI> defining and setting sDefaultPayrollDayOffset to -5, which is not allowed as it can't be negative</LI>
     * <LI> defining and setting sDefaultPayrollRunDay to 1</LI>
     * </OL>
     * 
     * <UL>
     * </L1>calling the updatePayrollschedule API with these settings result a exception.</L1>
     * </UL>
     */
    // Don't know why the following restriction exists:
    //        "payrollSchedule.getPayrollRunDayOffset() + payrollSchedule.getPayDayOffset() < -3"
    // But this restriction is now commented out, so this test is being commented out
//    @Test(expectedExceptions = ValidationException.class)
//    public void updateWhenPayDayOffsetEarlyThenEndDate1() throws Exception
//    {
//        // Setup
//        pSch = sWorkerPayService.getPayrollSchedule();
//        Calendar pDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 1);
//        sDefaultPayrollDayOffset = -5;
//        pSch.setPayrollPayDateOffset(sDefaultPayrollDayOffset);
//        sDefaultPayrollRunDayOffset = 1;
//        pSch.setPayrollRunDateOffset(sDefaultPayrollRunDayOffset);
//        pSch.setPeriodEndDate(pDate.getTime());
//        // Verify
//        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
//    }
    
    /**
     * <P>
     * Test to verify the Default updateWhenPayDayOffsetAndEndDateSame, this we are setting through off set and pay date  
     * it should throws the exception ValidationException  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI> Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI> Defining and setting end Date  </LI>
     * <LI> defining and setting sDefaultPayrollDayOffset to -1, which is not allowed as it can't be negative</LI>
     * <LI> defining and setting sDefaultPayrollRunDay to 1</LI>
     * </OL>
     * 
     * <UL>
     * </L1>calling the updatePayrollschedule API with these settings result a exception.</L1>
     * </UL>
     */
    @Test(enabled=false, expectedExceptions = ValidationException.class)
    public void updateWhenPayDayOffsetAndEndDateSame() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 1);
        sDefaultPayrollDayOffset = -1;
        pSch.setPayrollPayDateOffset(sDefaultPayrollDayOffset);
        sDefaultPayrollRunDayOffset = 1;
        pSch.setPayrollRunDateOffset(sDefaultPayrollRunDayOffset);
        pSch.setPeriodEndDate(pDate.getTime());
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateWhenPayDayEarlyThenEndDate1, this we are setting through off set and pay date  
     * it should throws the exception ValidationException  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI> Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI> Defining and setting end Date  </LI>
     * <LI> defining and setting sDefaultPayrollDayOffset to 1</LI>
     * <LI> defining and setting sDefaultPayrollRunDay to -5, which result as -4 throws error</LI>
     * </OL>
     * 
     * <UL>
     * </L1>calling the updatePayrollschedule API with these settings result a exception.</L1>
     * </UL>
     */
    // Don't know why the following restriction exists:
    //        "payrollSchedule.getPayrollRunDayOffset() + payrollSchedule.getPayDayOffset() < -3"
    // But this restriction is now commented out, so this test is being commented out
//    @Test(expectedExceptions = ValidationException.class)
//    public void updateWhenPayDayEarlyThenEndDate1() throws Exception
//    {
//        // Setup
//        pSch = sWorkerPayService.getPayrollSchedule();
//        Calendar pDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 1);
//        sDefaultPayrollDayOffset = 1;
//        pSch.setPayrollPayDateOffset(sDefaultPayrollDayOffset);
//        sDefaultPayrollRunDayOffset = -5;
//        pSch.setPayrollRunDateOffset(sDefaultPayrollRunDayOffset);
//        pSch.setPeriodEndDate(pDate.getTime());
//        // Verify
//        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
//    }
    
    /**
     * <P>
     * Test to verify the Default updateWhenPayDayEarlyThenEndDate2, this we are setting through off set and pay date  
     * it should throws the exception InvalidPayrollScheduleException  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI> Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI> Defining and setting end Date  </LI>
     * <LI> defining and setting sDefaultPayrollDayOffset to 1</LI>
     * <LI> defining and setting sDefaultPayrollRunDay to -4, which result as -3 throws error now but will pass after changes</LI>
     * </OL>
     * 
     * <UL>
     * </L1>calling the updatePayrollschedule API with these settings result a exception.</L1>
     * </UL>
     */
    @Test
    public void updateWhenPayDayEarlyThenEndDate2() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 1);
        sDefaultPayrollDayOffset = 1;
        pSch.setPayrollPayDateOffset(sDefaultPayrollDayOffset);
        sDefaultPayrollRunDayOffset = -4;
        pSch.setPayrollRunDateOffset(sDefaultPayrollRunDayOffset);
        pSch.setPeriodEndDate(pDate.getTime());
        // Exercise
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateWhenPayDayAndEndOffsetSamewithBig, this we are setting through off set and pay date  
     * it should throws the exception InvalidPayrollScheduleException  
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI> Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI> Defining and setting end Date  </LI>
     * <LI> defining and setting sDefaultPayrollDayOffset to 11</LI>
     * <LI> defining and setting sDefaultPayrollRunDay to 11, which result as 22 </LI>
     * </OL>
     * 
     * <UL>
     * </L1>calling the updatePayrollschedule API with these settings result a exception.</L1>
     * </UL>
     */
    @Test
    public void updateWhenPayDayAndEndOffsetSamewithBig() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.createCalendar(2008, Calendar.DECEMBER, 1);
        sDefaultPayrollDayOffset = 11;
        pSch.setPayrollPayDateOffset(sDefaultPayrollDayOffset);
        sDefaultPayrollRunDayOffset = 11;
        pSch.setPayrollRunDateOffset(sDefaultPayrollRunDayOffset);
        pSch.setPeriodEndDate(pDate.getTime());
        // Exercise
        exercisePayrollSchedule(pSch, pDate, PayrollScheduleFrequency.BIWEEKLY);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly when Only One Day is provided 
     * it should throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth1, it should require second date </LI>
     * <LI>Finally setting the Frequency to Semi monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void updateFrequencySemiMonthlyOnlyDay1() throws Exception
    {
    	pSch = sWorkerPayService.getPayrollSchedule();
        // Setup
        sDefaultPayPeriodEndDayOfMonth1 = 16;
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setFrequency(PayrollScheduleFrequency.SEMIMONTHLY);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencySemiMonthly when Only second Day is provided 
     * it should throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2, it should require first date </LI>
     * <LI>Finally setting the Frequency to Semi monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test (expectedExceptions = ValidationException.class)
    public void updateFrequencySemiMonthlyOnlyDay2() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth2 = 25;
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        pSch.setFrequency(PayrollScheduleFrequency.SEMIMONTHLY);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
        AssertJUnit.fail("PayPeriodEndDay1 must be between 1 and 28 inclusive or Last Day of Month");
       
    }
    
    /**
     * <P>
     * Test to verify the Default updateFrequencyMonthly when Only second Day is provided 
     * it should throws the exception ValidationException
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IWorkerPaymentsService's getPayrollSchedule() to get the Payroll schedule</LI>
     * <LI>Defining value and Setting the sDeafultPayPeriodEndDayOfMonth2, it should require first date </LI>
     * <LI>Finally setting the Frequency to monthly </LI>
     * </OL>
     * 
     * <UL>
     * <L1> calling the updatePayrollschedule API with these settings result a exception. </L1>
     * </UL>
     */
    @Test (expectedExceptions = ValidationException.class)
    public void updateFrequencyMonthlyWithDay() throws Exception
    {
        // Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        sDefaultPayPeriodEndDayOfMonth2 = 29;
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        pSch.setFrequency(PayrollScheduleFrequency.MONTHLY);
        // Verify
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
   
    /**
     *  <P>Returns void and call the verify function </P>
     *  
     * This method returns void but internally call the verifyPayrollSchedule which will verify all the parameters 
     * The method will Set all the paremeter and then update the payrollSchedule API
     * There is one logic added, in which checking few combination of sDefaultPayrollDayOffset and sDefaultPayrollRunDay
     * This actually help us in generating other possible valid combination for them, these are static variable and can keeping the valuse
     * Finally we are calling the verify function which verify the updated valuse with calculated values
     * 
     * @param pSch The PayrollSchedule to verify
     * @param calendar Calendar calendar date   
     * @param pSchFrequency PayrollScheduleFrequency Bi-weekly, Weekly
     *         
     */
    private void exercisePayrollSchedule(PayrollSchedule pSch, Calendar calendar, PayrollScheduleFrequency pSchFrequency)
        throws Exception
    {
        if ((pSchFrequency == PayrollScheduleFrequency.SEMIMONTHLY ||
        	pSchFrequency == PayrollScheduleFrequency.MONTHLY) && (calendar == null))
        {
            pSch.setFrequency(pSchFrequency);
            pSch = sWorkerPayService.updatePayrollSchedule(pSch);
            verifyPayrollSchedule(pSch, calendar, pSchFrequency);
        }
        else
        {
            int arrayDefaultOffset[] = new int[] { 0, 2 };
            int arrayDefaultRunDay[] = new int[] { -3, 0 };
            for (int i = 0; i < arrayDefaultOffset.length; i++)
            {
                for (int j = 0; j < arrayDefaultRunDay.length; j++)
                {
                    sDefaultPayrollDayOffset = arrayDefaultOffset[i];
                    pSch.setPayrollPayDateOffset(sDefaultPayrollDayOffset);
                    sDefaultPayrollRunDayOffset = arrayDefaultRunDay[j];
                    pSch.setPayrollRunDateOffset(sDefaultPayrollRunDayOffset);
                    pSch.setPeriodEndDate(calendar.getTime());
                    pSch.setFrequency(pSchFrequency);
                    pSch = sWorkerPayService.updatePayrollSchedule(pSch);
                    // Actual Verification starts here
                    verifyPayrollSchedule(pSch, calendar, pSchFrequency);
                    
                }
            }
        }
    }
    
    /**
     *  <P>Returns void and Verify all the payrollSchedule  </P>
     *     
     * This method returns Calendar date which will help us in determining the Start date 
     * Boolean parameter is just to setting function global, right now we require the true value only
     * Last it has a logic in which start date falls in saturday or sunday we will move to next day
     * 
     * @param pSch The PayrollSchedule to verify
     * @param pDate Calendar calendar date 
     * @param pSchFrequency PayrollScheduleFrequency Bi-weekly, Weekly
     */
    protected void verifyPayrollSchedule(PayrollSchedule pSch, Calendar pDate, PayrollScheduleFrequency pSchFrequency)
    {
        Calendar lDate = null;

        Calendar payDate;    
        if (pSchFrequency == PayrollScheduleFrequency.SEMIMONTHLY ||
        	pSchFrequency == PayrollScheduleFrequency.MONTHLY && pDate == null)
        {
            AssertJUnit.assertEquals("Expected Payroll End Date", pDate, lDate);
            AssertJUnit.assertEquals("Expected Payroll Default Period End Day of month1", sDefaultPayPeriodEndDayOfMonth1, pSch
                                .getPayPeriodEndDayofMonth1());
            if(pSchFrequency.name().equals("Semimonthly"))
            AssertJUnit.assertEquals("Expected Payroll Default Period End Day of month2", sDefaultPayPeriodEndDayOfMonth2, pSch
                                .getPayPeriodEndDayofMonth2());
        }
        else
        {
            lDate = DateUtil.getNowCalendar();
            lDate.setTime(pSch.getPeriodEndDate());
            // This is for verifying the End Date
            payDate = DateUtil.getNowCalendar();
            payDate.setTime(pDate.getTime());
            AssertJUnit.assertEquals("Expected Payroll End Date", true, DateUtil.isSameDay(pDate, lDate));
            // This for verifying the PayDay
            lDate.add(Calendar.DAY_OF_YEAR, pSch.getPayrollPayDateOffset() + pSch.getPayrollRunDateOffset());
            payDate.add(Calendar.DAY_OF_YEAR, sDefaultPayrollDayOffset + sDefaultPayrollRunDayOffset);
            AssertJUnit.assertTrue("Expected Payroll Pay Date", DateUtil.isSameDay(payDate, lDate));
            // This is for verifying the Start Date
            pDate = getCalculatedStartDate(pDate, pSchFrequency, false);
            Calendar actual = DateUtil.getNowCalendar();
            actual.setTime(pSch.getPeriodStartDate());
            AssertJUnit.assertEquals("Expected Payroll Start Date", pDate, actual);
                       
        }
        AssertJUnit.assertEquals("Expected Payroll Frequency Weekly", pSchFrequency, pSch.getFrequency());
        AssertJUnit.assertEquals("Expected Payroll Day", sDefaultPayrollDayOffset, pSch.getPayrollPayDateOffset());
        AssertJUnit.assertEquals("Expected Pay Day", sDefaultPayrollRunDayOffset, pSch.getPayrollRunDateOffset());
    }
    
    /**
     *  <P>Returns Calendar and calculate the start date  </P>
     *     
     * This method calculate the date (Start & End) and return back 
     *     
     * @param calendar Calendar passing the date 
     * @param pSchFrequency PayrollScheduleFrequency Bi-weekly, Weekly
     * @param next boolean this will decide traverse forward or backward  
     * @return A Calendar containing the calculated Start Date
     * 
     */
    protected static Calendar getCalculatedStartDate(Calendar calendar, PayrollScheduleFrequency pSchFrequency,
                                                         boolean next)
    {
        if (next)
            return PayrollScheduleDateUtils.getPayPeriodEndDate(calendar, PayrollFrequencyMapper.mapFromEIToDEntity(pSchFrequency));
        else
            return PayrollScheduleDateUtils.getPayPeriodStartDate(calendar, PayrollFrequencyMapper.mapFromEIToDEntity(pSchFrequency));
    }
    
    /**
     * <P>Set the current time as the default payroll Schedule  </P>
     * 
     * This method will set up the Default Payrollschedule using the current time
     * All the values are the default values and this will be called each time we are executing the test
     * we are calling this afterclass also. It will set all the default values in the PayrollSchedule
     *  
     */
    protected static void createDefaultPayrollSchedule() throws Exception
    {
        sDefaultPayrollDayOffset = -1;
        sDefaultPayrollRunDayOffset = -2;
        sDefaultPayPeriodEndDayOfMonth1 = 0;
        sDefaultPayPeriodEndDayOfMonth2 = 0;

        pSch = sWorkerPayService.getPayrollSchedule();
        pSch.setFrequency(PayrollScheduleFrequency.BIWEEKLY);
        pSch.setPayrollPayDateOffset(sDefaultPayrollDayOffset);
        pSch.setPayrollRunDateOffset(sDefaultPayrollRunDayOffset);
        pSch.setPayPeriodEndDayofMonth1(sDefaultPayPeriodEndDayOfMonth1);
        pSch.setPayPeriodEndDayofMonth2(sDefaultPayPeriodEndDayOfMonth2);
        Calendar pDate = DateUtil.getNowCalendar();
        //To avoid Thursday and Friday
        pDate.add(Calendar.DAY_OF_YEAR, sDefaultPayrollDayOffset);
        pDate = DateUtil.getNextWeekDayOfWeek(pDate, Calendar.FRIDAY);
        pDate.add(Calendar.DAY_OF_YEAR, -(sDefaultPayrollDayOffset+sDefaultPayrollRunDayOffset));
        pSch.setPeriodEndDate(pDate.getTime());
        pSch = sWorkerPayService.updatePayrollSchedule(pSch);
    }
}
