package com.intuit.sb.payroll.qa.services.workerpayments;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 * <P>
 * The <CODE>VoidPaychecksExceptionsTest</CODE> class contains tests for testing the
 * exceptions thrown by voidPaychecks() in WorkerPaymentService
 * </P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class VoidPaychecksExceptionsTest extends WorkerPaymentServiceTestBase {

    /**
     * <P>Test ValidationException caused by null PaycheckIds array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's createPayrollRun() with a null PaycheckIds array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullPaycheckIds() throws Exception {
        sWorkerPayService.voidPaychecks(null);
    }

    /**
     * <P>Test ValidationException caused by empty PaycheckIds array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's createPayrollRun() with an empty PaycheckIds array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testEmptyPaycheckIds() throws Exception {
        sWorkerPayService.voidPaychecks(new Long[0]);
    }

    /**
     * <P>Test ValidationException caused by null PaycheckIds array element</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's createPayrollRun() with a null PaycheckIds array element</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullPaycheckIdsElement() throws Exception {
    	Long[] paycheckIds = new Long[1];
    	paycheckIds[0] = null;
        sWorkerPayService.voidPaychecks(paycheckIds);
    }

}
