package com.intuit.sb.payroll.testtools;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.naming.InitialContext;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.LineBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableModel;
import javax.transaction.Status;
import javax.transaction.UserTransaction;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.jboss.deployers.spi.DeploymentException;
import org.jboss.embedded.Bootstrap;
import org.jboss.virtual.plugins.context.vfs.AssembledContextFactory;
import org.jboss.virtual.plugins.context.vfs.AssembledDirectory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.common.JNDITestUtil;
import com.intuit.sb.payroll.capability.common.PayrollTestBase;
import com.intuit.sb.payroll.capability.common.WorkLocationAndWorkerDataPopulator;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.impl.EmployerSeedData;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerTaxWithholdingRules;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.employer.EmployerEAO;
import com.intuit.sb.payroll.entity.model.ComplianceDoc;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.rules.RulesBridge;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;


/**
 * A Swing based java application for viewing the Tax rules on the Gemini project.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class TaxRulesViewer extends JFrame
{

    private JFrame jFrame = null;  //  @jve:decl-index=0:visual-constraint="6,3"

    private JPanel jContentPane = null;

    private JMenuBar jJMenuBar = null;

    private JMenu fileMenu = null;

    private JMenu editMenu = null;

    private JMenu helpMenu = null;

    private JMenuItem exitMenuItem = null;

    private JMenuItem aboutMenuItem = null;

    private JMenuItem cutMenuItem = null;

    private JMenuItem copyMenuItem = null;

    private JMenuItem pasteMenuItem = null;

    private JDialog aboutDialog = null;  //  @jve:decl-index=0:visual-constraint="376,609"

    private JPanel aboutContentPane = null;

    private JTabbedPane jTabs = null;

    private JPanel panel1 = null;

    private JPanel panel2 = null;

    private JLabel cityLabel = null;

    private JTextField cityField = null;

    private JLabel stateLabel = null;

    private JComboBox stateField = null;

    private JLabel zipLabel = null;

    private JTextField zipField = null;

    private JLabel countryLabel = null;

    private JLabel legalLabel = null;

    private JComboBox legalField = null;

    private JLabel industryLabel = null;

    private JComboBox industryField = null;

    private JButton showTaxes = null;
    
    private JPanel employerInfoPanel = null;
    
    static final long serialVersionUID = 12345678L;

    private JLabel addressLabel = null;

    private JTextField addressField = null;

    private JButton validateAddress = null;

    private JLabel addressIsValidLabel = null;

    private JTextField addressIsValid = null;

    private JLabel aboutVersion = null;

    private JLabel tbdLabel = null;

    private JLabel contributorLabel = null;

    private JLabel contributor1 = null;
    
    private String appName = "TaxRulesViewer";  //  @jve:decl-index=0:

    private JComboBox countryField = null;

    private JPanel addressPanel = null;

    /****************/
    private static final Class<TaxRulesViewer> THIS = TaxRulesViewer.class;
    private static Logger sLogger = LogManager.getLogger(THIS);
    private static final List<URL> jeeArchiveURLs = new ArrayList<URL>();
    private static List<URL> jeeDeployedURLs = new ArrayList<URL>();
    private static InitialContext jndi;

    private static final int maxNumOfHrEmployees = 3;
    
    private String[] stateAbbreviations = {"AL", "AK", "AS", "AZ", "AR", "CA", "CO", "CT", "DE", "DC", "FM", "FL", "GA",
                                           "GU", "HI", "ID", "IL", "IN", "IA", "KS", "KY", "LA", "ME", "MH", "MD", "MA",
                                           "MI", "MN", "MS", "MO", "MT", "NE", "NV", "NH", "NJ", "NM", "NY", "NC", "ND",
                                           "MP", "OH", "OK", "OR", "PW", "PA", "PR", "RI", "SC", "SD", "TN", "TX", "UT",
                                           "VT", "VI", "VA", "WA", "WV", "WI", "WY"
    };

    private String[] mStates = {"Alabama", "Alaska", "American Samoa", "Arizona", "Arkansas", "California", "Colorado",
                                "Connecticut", "Delaware", "District of Columbia", "Federated States of Micronesia",
                                "Florida", "Georgia", "Guam", "Hawaii", "Idaho", "Illinois", "Indiana", "Iowa", "Kansas",
                                "Kentucky", "Louisiana", "Maine", "Marshall Islands", "Maryland", "Massachusetts",
                                "Michigan", "Minnesota", "Mississippi", "Missouri", "Montana", "Nebraska", "Nevada",
                                "New Hampshire", "New Jersey", "New Mexico", "New York", "North Carolina", "North Dakota",
                                "Northern Mariana Islands", "Ohio", "Oklahoma", "Oregon", "Palau", "Pennsylvania",
                                "Puerto Rico", "Rhode Island", "South Carolina", "South Dakota", "Tennessee", "Texas",
                                "Utah", "Vermont", "Virgin Islands", "Virginia", "Washington", "West Virginia",
                                "Wisconsin", "Wyoming"
    };

    private String[] mCountries = {"United States", "Canada"};

    private Worker mWorker = null;  //  @jve:decl-index=0:
    
    private Long mWorkList[] = null;  //  @jve:decl-index=0:
    
    private int numOfWorkers = 0;
    
    private int currentWorker = 0;
    
    private Employer mEmployer = null;  //  @jve:decl-index=0:
    
    private USPostalAddress mWorkAddress = null;  //  @jve:decl-index=0:

    private WorkLocation mWorkLocation = null;  
    
    private USPostalAddress mHomeAddress = null;
    
    private RelationshipToOwnerEnum mRelationToOwner = RelationshipToOwnerEnum.UNRELATED;  //  @jve:decl-index=0:
    
    private Date mBirthDate = null;  //  @jve:decl-index=0:
    
    private String mWithholdingFormList[] = null;  //  @jve:decl-index=0:
    
    //private WithholdingForm mWithholdingForm = null;
    
    private int currentWithholdingForm = 0;
    
    private JPanel personalInfoPanel = null;

    private JLabel relationshipLabel = null;

    private JLabel dateOfBirthLabel = null;

    private JComboBox relationshipField = null;

    private JTextField DateOfBirthField = null;

    private JLabel dateOfBirthFormatLabel = null;

    private JScrollPane taxesScrollPane = null;

    private JTable taxesTable = null;

    private JLabel applicableTaxesLabel = null;

    private JPanel homeAddressPanel = null;

    private JLabel homeAddressLabel = null;

    private JLabel homeCityLabel = null;

    private JLabel homeStateLabel = null;

    private JLabel homeZipLabel = null;

    private JLabel homeCountryLabel = null;

    private JLabel homeAddressIsValidLabel = null;

    private JTextField homeAddressField = null;

    private JTextField homeCityField = null;

    private JComboBox homeStateField = null;

    private JTextField homeZipField = null;

    private JComboBox homeCountryField = null;

    private JTextField homeAddressIsValid = null;

    private JButton homeValidateAddress = null;

    private JLabel withholdingFormsLabel = null;

    private JList withholdingFormsList = null;

    private JLabel withholdingFormLabel2 = null;

    private JComboBox workerComboBox = null;

    private JLabel workerLabel = null;

    private JCheckBox IsNonProfitCheckBox = null;

    private JPanel withholdingInfoPanel = null;

    private JLabel startDateLabel = null;

    private JTextField startDateField = null;

    private JLabel endDateLabel = null;

    private JTextField endDateField = null;

    private JLabel formDateLabel = null;

    private JTextField formDateField = null;

    private JLabel startDateFormatLabel = null;

    private JLabel endDateFormatLabel = null;

    private JLabel formDateFormatLabel = null;

    private JCheckBox isDerivedCheckBox = null;

    private JCheckBox IsExemptCheckBox = null;

    private JCheckBox isSourceDocOnFileCheckBox = null;

    private JComboBox withholdingStatusComboBox = null;

    private JLabel withholdingStatusLabel = null;

    private JLabel allowancesLabel = null;

    private JLabel additionalWithholdingAmountLabel = null;

    private JTextField allowancesTextField = null;

    private JTextField additionalWithholdingAmountTextField = null;

    private void initEnv()
    {
        //TODO  Init the environment
    }

    private void setUpEmployer(Worker worker) {
        EmployerEAO employerEAO = PayrollTestBase.getQaBootstrap().getEjbFromClass(EmployerEAO.class);
        mEmployer = employerEAO.findCurrentEmployer();
    }
    
    private void setUpWorkAddress(Worker worker) {
		WorkLocation workLocation = worker.getWorkLocation();
        if (workLocation != null)
        {
        	GlobalPostalAddress taxAddressEntity = workLocation.getTaxResidenceAddress();
    		mWorkAddress = AddressConvertor.getUSFromGlobal(
    				taxAddressEntity, new USPostalAddressImpl());
        }
    }
    
    private void setUpBirthDates(Worker worker) {
        mBirthDate = worker.getDateOfBirth();
    }

    /****************/
    
    /**
     * constructor 
     * 
     */
    public TaxRulesViewer() {
        super();
        bootstrap();
        BasicTestBase.initializeEAOs();
        initialize();
    }

    private static UserTransaction getUserTransaction()
    {
        UserTransaction tx = null;
        try {
            tx = (UserTransaction)jndi.lookup("UserTransaction");
        }
        catch (Exception ex)
        {
            throw new RuntimeException("Error in look up transcation in jndi", ex);
        }
        return tx;
        
    }

    private void setUpWorkerInfo(int whichOne) {
        mWorker = BasicTestBase.getEntityService().findById(Worker.class, mWorkList[whichOne]);
        mWorkLocation = mWorker.getWorkLocation();
        // set up mHomeAddress
       	GlobalPostalAddress homeAddressEntity = mWorker.getTaxResidenceAddress();
   		mHomeAddress = AddressConvertor.getUSFromGlobal(
				homeAddressEntity, new USPostalAddressImpl());
    }
    
    private void setUpRequiredData() {
        try
        {
            getUserTransaction().begin();
            List<Jurisdiction>  jurisdictions = BasicTestBase.getEntityService().findAll(Jurisdiction.class);
            // Seed the data if necessary
            if (jurisdictions == null || jurisdictions.size() == 0)
            {
            	EmployerSeedData.setupEmployerDataInSystem();
            }
            if (mWorker == null)
            {
                List<Employee>  workerList = BasicTestBase.getEntityService().findAll(Employee.class);
                if (workerList == null || workerList.size() == 0)
                {
                    WorkLocationAndWorkerDataPopulator workLocationAndWorkerDataPopulator = new WorkLocationAndWorkerDataPopulator(maxNumOfHrEmployees);
                    workLocationAndWorkerDataPopulator.populateWorkLocAndWorkers();
                    workerList = BasicTestBase.getEntityService().findAll(Employee.class);
                }
                numOfWorkers = workerList.size();
                if (numOfWorkers == 0)
                {
                    showUnexpectedErrorBox("No usable Employee found in the database");
                }
                // Now create and populate the employee picklist
                mWorkList = new Long[numOfWorkers];
                int idx = 0;
                for (int i = 0; i < workerList.size(); i++)
                {
                    Worker worker = workerList.get(i);
	                // Only use workers that has the WorkLocation set
	                WorkLocation workLocation = worker.getWorkLocation();
	                if (workLocation != null)
	                {
	                    mWorkList[idx++] = worker.getId();
	                    String name = worker.getPersonName().getFirstName();
	                    if (worker.getPersonName().getLastName() != null)
	                    {
	                        name = name + " " + worker.getPersonName().getLastName();
	                    }
	                    if (worker.getPersonName().getSuffix() != null)
	                    {
	                        name = name  + " " + worker.getPersonName().getSuffix();
	                    }
	                    getWorkerComboBox().addItem(makeObj(name));
	                }
                }
            }
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
        	t.printStackTrace();
            sLogger.error(t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
            showUnexpectedErrorBox(t.getMessage());
        }
    }
    
    private void loadDataAndUpdateForm()
    {
        boolean transactionStarted = false;
        try
        {
            if (getUserTransaction().getStatus() != Status.STATUS_ACTIVE)
            {
                getUserTransaction().begin();
                transactionStarted = true;
            }
            setUpWorkerInfo(currentWorker);
            setUpEmployer(mWorker);
            setUpWorkAddress(mWorker);
            setUpBirthDates(mWorker);
            updateTaxesTabData();
            if (transactionStarted)
            {
                getUserTransaction().commit();
            }
        }
        catch (Exception e)
        {
            try
            {
                if (transactionStarted)
                {
                    getUserTransaction().rollback();
                }
            }
            catch (Exception ex)
            {
            }
            showUnexpectedErrorBox(e.getMessage());
        }
    }
    
    private void updateTaxesTabData()
    {

        getAddressField().setText(mWorkAddress.getAddressLine1());
        getCityField().setText(mWorkAddress.getCity());
        getStateField().setSelectedIndex(getStateIndex(mWorkAddress.getRegionOrState()));
        getZipField().setText(mWorkAddress.getZipOrPostalCode());
        getCountryField().setSelectedIndex(getCountryIndex(mWorkAddress.getCountry()));
        // Update the personal info section
        mRelationToOwner = mWorker.getRelationshipToOwner();
        getRelationshipField().setSelectedIndex(getRelationIndex(mRelationToOwner.toString()));
//        getDateOfBirthField().setText(mBirthDate.format("M/d/yyyy"));
        SimpleDateFormat formatter = new SimpleDateFormat("M/d/yyyy");
        getDateOfBirthField().setText(formatter.format(mBirthDate));
        // Update the Employer Info section
        getLegalField().setSelectedIndex(getLegalStructureIndex(mEmployer.getLegalStructure().name()));
        getIndustryField().setSelectedIndex(getIndustryIndex(mEmployer.getIndustryType().name()));
        isNonProfitCheckBox().setSelected(mEmployer.isNonProfit());
        // Update the Home address section
        getHomeAddressField().setText(mHomeAddress.getAddressLine1());
        getHomeCityField().setText(mHomeAddress.getCity());
        getHomeStateField().setSelectedIndex(getStateIndex(mHomeAddress.getRegionOrState()));
        getHomeZipField().setText(mHomeAddress.getZipOrPostalCode());
        getHomeCountryField().setSelectedIndex(getCountryIndex(mHomeAddress.getCountry()));
   }
    
    private void updateWithholdingInfoSection()
    {
//        Employee employee = mWorker.getEmployee();
//        PrEmployee prEmployee = employee.<PrEmployee>getExtension(PrEmployee.class);
//        WithholdingForm withholdingForm = mEntityService.<WithholdingForm>findById(WithholdingForm.class, mWithholdingFormList[currentWithholdingForm]);
    }
    
    private int getLegalStructureIndex(String legalStructure)
    {
        int i = 0;
        for (; i < LegalStructureEnum.values().length; i++)
        {
            if (legalStructure.equals(LegalStructureEnum.values()[i].name()))
                break;
        }
        return i;
    }
    
    private int getIndustryIndex(String industry)
    {
        int i = 0;
        for (; i < IndustryTypeEnum.values().length; i++)
        {
            if (industry.equals(IndustryTypeEnum.values()[i].name()))
                break;
        }
        return i;
    }
    
    private int getStateIndex(String state)
    {
        int i = 0;
        for (; i < stateAbbreviations.length; i++)
        {
            if (state.equals(stateAbbreviations[i]))
                break;
        }
        return i;
    }
    
    private int getCountryIndex(String country)
    {
        int i = 0;
        // Only search when country has a value, otherwise default to the first element (which is United States)
        if (null != country)
        {
            for (; i < mCountries.length; i++)
            {
                if (country.equals(mCountries[i]))
                    break;
            }
        }
        return i;
    }
    
    private int getRelationIndex(String relation)
    {
        int i = 0;
        for (; i < IndustryTypeEnum.values().length; i++)
        {
            if (relation.equals(RelationshipToOwnerEnum.values()[i].name()))
                break;
        }
        return i;
    }
    
	/**
	 * Adds a jar from the file system to the list of jars to deploy to the JEE container.
	 * @param jarFile the file path.
	 */
	public static void addDeployJarFromFileSystem(String jarFile)
	{
		try
		{
			File newfile = new File(jarFile);
			Boolean exists = newfile.exists();
			if (exists)
			{
				String path = newfile.getAbsolutePath();
			    URL url = new URL("file:/" + path);
			    
			    if (null != url)
			    {
			        jeeArchiveURLs.add(url);
			    }
			}
		}
		catch(Exception ex)
		{
		    //do nothing. If the URL is not valid, the jar won't be deployed	
		}
	}

    /**
     * Set up the environment
     * 
     */
    private void initialize() {
        this.setSize(new Dimension(320, 27));  
        this.setTitle(appName);
        this.initEnv();
        this.setUpRequiredData();
    }
    
    protected void finalize() throws Throwable {
        classTearDown();
        super.finalize();
    }

    public static void classTearDown() throws Exception
    {
        sLogger.debug("afterClass");
        try
        {
            for (URL jeeArchiveURL : jeeDeployedURLs)
            {
                Bootstrap.getInstance().undeploy(jeeArchiveURL);
            }
        }
        catch (DeploymentException e)
        {
            throw new InfrastructureException("Unable to undeploy", e);
        }
        if (System.getProperty("shutdown.embedded.jboss") != null) Bootstrap.getInstance().shutdown();
    }
    
    /**
     * Deploy the archives specified in calls to {@code addDeployJarFromClasspath}.
     * @throws PayrollServiceFailureException if it's unable to deploy an archive.
     */
    synchronized protected static void deploy()
    {
        try
        {
            for (URL jeeArchiveURL : jeeArchiveURLs)
            {
                //if the jar is already deployed, skip
                if ( ! jeeDeployedURLs.contains(jeeArchiveURL) )
                {
                    Bootstrap.getInstance().deploy(jeeArchiveURL);
                    jeeDeployedURLs.add(jeeArchiveURL);
                }
            }
        }
        catch (DeploymentException e)
        {
            throw new InfrastructureException("Unable to deploy", e);
        }
    }

    synchronized protected static void deploy(String resource)
    {
        try
        {
            URL url = THIS.getClassLoader().getResource(resource);
            //if the jar is already deployed, skip
            if ( ! jeeDeployedURLs.contains(url) )
            {
                Bootstrap.getInstance().deploy(url);
                jeeDeployedURLs.add(url);
            }
        }
        catch (DeploymentException e)
        {
            throw new InfrastructureException("Unable to deploy", e);
        }
    }
    
    synchronized private static void bootstrap()
    {
        sLogger.debug("startContainer");
        if (Bootstrap.getInstance().isStarted()) return;

        try
        {
            Bootstrap.getInstance().bootstrap();
            ServiceLocator.setEmbedded(true);
            startContainer();
        }
        catch (Exception error)
        {
            sLogger.error(error);
            try
            {
                classTearDown();
            }
            catch (Exception e)
            {
                sLogger.error(e);
            }
            throw new RuntimeException("Failed to bootstrap", error);
        }
    }

	synchronized public static String getVersionString(String filename) {
		String version = "";
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document dom = db.parse(filename);
			Element docElement = dom.getDocumentElement();
			NodeList nl = docElement.getElementsByTagName("parent");
			if (null != nl && nl.getLength() == 1) {
				Element element = (Element)nl.item(0);
				NodeList nodeList = element.getElementsByTagName("version");
				if (null != nodeList && nodeList.getLength() == 1) {
					Element versionElement = (Element)nodeList.item(0);
					version = versionElement.getFirstChild().getNodeValue();
				}
			}
		} catch(ParserConfigurationException pce) {
    		pce.printStackTrace();
    	} catch(SAXException se) {
    		se.printStackTrace();
    	} catch(IOException ioe) {
    		ioe.printStackTrace();
    	}
		return version;
	}
	
    synchronized public static void startContainer() throws Exception
    {
		AssembledDirectory commonServiceJar =  AssembledContextFactory.getInstance().create("sb-payroll-common-employee.jar");
		AssembledDirectory capabilityUsJar = AssembledContextFactory.getInstance().create("sb-payroll-capability-us.jar");
		AssembledDirectory capabilityIntlJar = AssembledContextFactory.getInstance().create("sb-payroll-capability-intl.jar");
		AssembledDirectory compositeUsJar = AssembledContextFactory.getInstance().create("sb-payroll-composed-us.jar");
        
    	String [] beanIncludes = {"**/*Bean.class"};
    	
		capabilityUsJar.addResources(EmployerComponent.class, beanIncludes, null);
		capabilityIntlJar.addResources(PaycheckCalculator.class, beanIncludes, null);
    	commonServiceJar.addResources(RulesBridge.class, beanIncludes, null);
    	//if run from composite, this class will exist
    	Class<?> companyClz = null;
    	try {
    	    companyClz = Class.forName("com.intuit.sb.payroll.services.common.BankAccount");
  		  compositeUsJar.addResources(companyClz, beanIncludes, null);
    	}
    	catch (ClassNotFoundException ex) {
    	    //do nothing, if capability project, BankAccount can not be found.
    	}
    
    	try {
    	    //deploy entity jar
    		String filename = "../PayrollEntityService/target/sb-payroll-entity-us-" +
								getVersionString("pom.xml") + ".jar";
			addDeployJarFromFileSystem(filename);
    		deploy();
    		Bootstrap bootStrap = Bootstrap.getInstance();
    		bootStrap.deploy(capabilityIntlJar);
    		bootStrap.deploy(capabilityUsJar);
    	    //Test jar for TaskSubsystemTest
    		bootStrap.deploy(compositeUsJar);
    		bootStrap.deploy(commonServiceJar);
    	} catch (Exception e) {
    	    e.printStackTrace();
    	}
	
        // Create InitialContext from jndi.properties
        jndi = new InitialContext();
        
        sLogger.debug(JNDITestUtil.listJNDITree(null));     
    }
    
    /**
     * This method initializes jTabs	
     * 	
     * @return javax.swing.JTabbedPane	
     */
    private JTabbedPane getJTabs()
    {
        if (jTabs == null)
        {
            jTabs = new JTabbedPane();
            jTabs.setBounds(new Rectangle(0, 0, 934, 552));  
            jTabs.addTab("Taxes", null, getPanel1(), null);  
            jTabs.addTab("Tab 2", null, getPanel2(), null);  
        }
        return jTabs;
    }

    /**
     * This method initializes panel1	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getPanel1()
    {
        if (panel1 == null)
        {
            workerLabel = new JLabel();
            workerLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            workerLabel.setBounds(new Rectangle(15, 10, 104, 24));  
            workerLabel.setText("Employee:");  
            withholdingFormLabel2 = new JLabel();
            withholdingFormLabel2.setBounds(new Rectangle(345, 256, 86, 16));  
            withholdingFormLabel2.setText("Forms:");  
            withholdingFormsLabel = new JLabel();
            withholdingFormsLabel.setBounds(new Rectangle(345, 242, 86, 16));  
            withholdingFormsLabel.setText("Withholding");  
            applicableTaxesLabel = new JLabel();
            applicableTaxesLabel.setBounds(new Rectangle(15, 239, 110, 17));  
            applicableTaxesLabel.setText("Applicable Taxes:");  
            addressIsValidLabel = new JLabel();
            addressIsValidLabel.setText("Is valid:");  
            addressIsValidLabel.setBounds(new Rectangle(11, 154, 90, 26));  
            addressIsValidLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            addressLabel = new JLabel();
            addressLabel.setText("Street Address:");  
            addressLabel.setBounds(new Rectangle(12, 18, 90, 23));  
            addressLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            industryLabel = new JLabel();
            industryLabel.setText("Industry:");  
            industryLabel.setBounds(new Rectangle(12, 42, 90, 26));  
            industryLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            legalLabel = new JLabel();
            legalLabel.setText("Legal Structure:");  
            legalLabel.setBounds(new Rectangle(10, 15, 92, 26));  
            legalLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            countryLabel = new JLabel();
            countryLabel.setText("Country:");  
            countryLabel.setBounds(new Rectangle(11, 124, 90, 26));  
            countryLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            zipLabel = new JLabel();
            zipLabel.setText("Zip:");  
            zipLabel.setBounds(new Rectangle(12, 96, 90, 26));  
            zipLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            stateLabel = new JLabel();
            stateLabel.setText("State:");  
            stateLabel.setBounds(new Rectangle(12, 69, 90, 26));  
            stateLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            cityLabel = new JLabel();
            cityLabel.setText("City:");  
            cityLabel.setBounds(new Rectangle(12, 42, 90, 26));  
            cityLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            panel1 = new JPanel();
            panel1.setLayout(null);  
            panel1.add(getShowTaxes(), null);  
            panel1.add(getEmployerInfoPanel(), null);  
            panel1.add(getAddressPanel(), null);  
            panel1.add(getPersonalInfoPanel(), null);  
            panel1.add(getTaxesScrollPane(), null);  
            panel1.add(applicableTaxesLabel, null);  
            panel1.add(getHomeAddressPanel(), null);  
            panel1.add(withholdingFormsLabel, null);  
            panel1.add(getWithholdingFormsList(), null);  
            panel1.add(withholdingFormLabel2, null);  
            panel1.add(getWorkerComboBox(), null);  
            panel1.add(workerLabel, null);  
            panel1.add(getWithholdingInfoPanel(), null);  
        }
        return panel1;
    }

    /**
     * This method initializes panel2	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getPanel2()
    {
        if (panel2 == null)
        {
            tbdLabel = new JLabel();
            tbdLabel.setBounds(new Rectangle(164, 176, 444, 43));  
            tbdLabel.setText("For future expansion");  
            tbdLabel.setFont(new Font(getName(), Font.BOLD, 24));
            tbdLabel.setHorizontalAlignment(SwingConstants.CENTER);
            panel2 = new JPanel();
            panel2.setLayout(null);  
            panel2.add(tbdLabel, null);  
        }
        return panel2;
    }

    /**
     * This method initializes cityField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getCityField()
    {
        if (cityField == null)
        {
            cityField = new JTextField();
            cityField.setBounds(new Rectangle(110, 42, 170, 26));  
            cityField.setText("");  
        }
        return cityField;
    }

    private Object makeObj(final String item)  {
        return new Object() { public String toString() { return item; } };
      }

    /**
     * This method initializes stateField	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getStateField()
    {
        if (stateField == null)
        {
            stateField = new JComboBox();
            stateField.setBounds(new Rectangle(110, 69, 170, 26));  
            stateField.setBackground(Color.white);
            for (int i = 0; i < mStates.length; i++) {
                stateField.addItem(makeObj(mStates[i]));
            }
        }
        return stateField;
    }

    /**
     * This method initializes zipField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getZipField()
    {
        if (zipField == null)
        {
            zipField = new JTextField();
            zipField.setBounds(new Rectangle(110, 96, 170, 26));  
            zipField.setText("");  
        }
        return zipField;
    }

    /**
     * This method initializes legalField	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getLegalField()
    {
        if (legalField == null)
        {
            legalField = new JComboBox();
            legalField.setBounds(new Rectangle(110, 15, 170, 26));  
            legalField.setBackground(Color.white);
            LegalStructureEnum[] legalStructures = LegalStructureEnum.values();
            for (int i = 0; i < legalStructures.length; i++) {
                legalField.addItem(makeObj(legalStructures[i].toString()));
            }
        }
        return legalField;
    }

    /**
     * This method initializes industryField	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getIndustryField()
    {
        if (industryField == null)
        {
            industryField = new JComboBox();
            industryField.setBounds(new Rectangle(110, 42, 170, 26));  
            industryField.setBackground(Color.white);
            IndustryTypeEnum[] industries = IndustryTypeEnum.values();
            for (int i = 0; i < industries.length; i++) {
                industryField.addItem(makeObj(industries[i].toString()));
            }
        }
        return industryField;
    }

    /**
     * This method maps a Country name to its ISO country code	
     * 	
     * @return a String representing an ISO Country code	
     */
    private String getISOCountryCode(JComboBox comboBox) {
    	String isoCode = "US";
    	if (comboBox.getSelectedItem().toString().compareTo("Canada") == 0) {
    		isoCode = "CA";
    	}
    	return isoCode;
    }
    
    private void updateAddressInfo(USPostalAddress origAddress, USPostalAddress address)
    {
        if (origAddress != null)
        {
            origAddress.setAddressLine1(address.getAddressLine1());
            origAddress.setAddressLine2(address.getAddressLine2());
            origAddress.setAddressLine3(address.getAddressLine3());
            origAddress.setCity(address.getCity());
            origAddress.setCountry(address.getCountry());
            origAddress.setCountryISOCode(address.getCountryISOCode());
            origAddress.setCounty(address.getCounty());
            origAddress.setRegionOrState(address.getRegionOrState());
            origAddress.setZipOrPostalCode(address.getZipOrPostalCode());
        }
        else
        {
            showUnexpectedErrorBox("PhysicalAddress is unexpectedly null!");
        }
    }
    
    private void setUpTestData(Employer org, USPostalAddress workAddress, USPostalAddress homeAddress,
                               Date dateOfBirth, RelationshipToOwnerEnum relationship)
    {
        EmployerEAO employerEAO = PayrollTestBase.getQaBootstrap().getEjbFromClass(EmployerEAO.class);
        Employer employer = employerEAO.findCurrentEmployer();
        // Update the organization information
        employer.setIndustryType(org.getIndustryType());
        employer.setLegalName(org.getLegalName());
//        employer.setOrganizationId(org.getOrganizationId());
//        employer.setOrganizationTaxId(org.getOrganizationTaxId());
        employer.setCompanyName(org.getCompanyName());
        employer.setDbaName(org.getDbaName());
        employer.setLegalStructure(org.getLegalStructure());
        employer.setNonProfit(org.isNonProfit());
        
        setUpWorkerInfo(currentWorker);
        
        //Update the work location detail
        //This is tricky, watch carefully
        //First Get the Original Reference that we want to update
        GlobalPostalAddress destWorkAddressEntity = mWorkLocation.getTaxResidenceAddress();
        //Create a Proxy for the update
        USPostalAddress destWorkAddressProxy = AddressConvertor.getUSFromGlobal(
        		destWorkAddressEntity, new USPostalAddressImpl());
        //Get the Updated Destination Proxy
        updateAddressInfo(destWorkAddressProxy, workAddress);        
        //Set the Destination to the new Value
        AddressConvertor.getGlobalFromUS(destWorkAddressProxy, destWorkAddressEntity);

        // Update the home address contact detail
        //This is again tricky, watch Carefully
        //First Get the Original Reference that we want to update
        GlobalPostalAddress destHomeAddressEntity = mWorker.getTaxResidenceAddress(); 
        //Create a Proxy for the update
        USPostalAddress destHomeAddressProxy = AddressConvertor.getUSFromGlobal(
        		destHomeAddressEntity, new USPostalAddressImpl());
        updateAddressInfo(destHomeAddressProxy, homeAddress);
        //Set the Destination to the new Value. Its not the Assignment, its the Side Effect
       AddressConvertor.getGlobalFromUS(
        		destHomeAddressProxy, destHomeAddressEntity);

        // Update the employee relationship to the employer
        mWorker.setRelationshipToOwner(relationship);

        // Update the date of birth
        mWorker.setDateOfBirth(dateOfBirth);
        
        //Save the mWorker
        
        // ensure that mWorkLocation has the most up to date information
		mWorkLocation = mWorker.getWorkLocation();
    }
    
    
   
    private void performGetApplicableTaxesAndForms(Employer org, USPostalAddress address, USPostalAddress homeAddress,
                                                   Date dateOfBirth, RelationshipToOwnerEnum relationship)
   {
//        setUpApplyApplicableTaxes();
       boolean transactionStarted = false;
       try {
           if (getUserTransaction().getStatus() != Status.STATUS_ACTIVE)
           {
               getUserTransaction().begin();
               transactionStarted = true;
           }
           mWorker = BasicTestBase.getEntityService().findById(Worker.class, mWorker.getId());
           setUpTestData(org, address, homeAddress, dateOfBirth, relationship);
           TaxComponent taxComponent = PayrollTestBase.getQaBootstrap().getEjbFromClass(TaxComponent.class);
           mWorker = taxComponent.applyApplicableTaxes(mWorker);
           
           // Get the applicable taxes
           
           List<Law> computedTaxes = taxComponent.determineApplicableTaxes(mWorker);
           MyTableModel myTableModel = (MyTableModel)getTaxesTable().getModel();
           myTableModel.init(computedTaxes.size());
           for (int row = 0; row < computedTaxes.size(); row++)
           {
               Law tax = computedTaxes.get(row);
               myTableModel.setValueAt(tax.getDescription(), row, 0);
               myTableModel.setValueAt(tax.getLawId(), row, 1);
               myTableModel.setValueAt(tax.getSourceEntity().toString(), row, 2);
           }

           // Get the withholding forms
           List<ComplianceDoc> computedWithholdingForms = WorkerTaxWithholdingRules.getWithholdingFormsApplicableToWorker(mWorker);
           mWithholdingFormList = new String[computedWithholdingForms.size()];
           for (int i = 0; i < computedWithholdingForms.size(); i++)
           {
        	   ComplianceDoc form = computedWithholdingForms.get(i);
               mWithholdingFormList[i] = form.getDocId().name();
               ((DefaultListModel)getWithholdingFormsList().getModel()).addElement(form.getName());
           }
           // Set the default selected item to the first one
           currentWithholdingForm = 0;
           getWithholdingFormsList().setSelectedIndex(currentWithholdingForm);
           updateWithholdingInfoSection();
           // save worker
           if (transactionStarted)
           {
               getUserTransaction().commit();
           }
       } catch (Exception ex) {
           try
           {
               if (transactionStarted)
               {
                   getUserTransaction().rollback();
               }
           }
           catch (Exception e)
           {
           }
           if (ex.getMessage().contains("Unsupported Jurisdiction"))
           {
               showMessageBox();
           }
           else
           {
               showUnexpectedErrorBox(ex.getMessage());
           }
       }
   }

    private void showUnexpectedErrorBox(String msg)
    {
        JPanel aContentPane = new JPanel();
        aContentPane.setLayout(null);
        aContentPane.setPreferredSize(new Dimension(400, 200));

        JTextArea message = new JTextArea();
        message.setBounds(new Rectangle(5, 5, 390, 190));
        message.setText(msg);
        message.setLineWrap(true);
        message.setEditable(false);
        message.setBackground(aContentPane.getBackground());
        aContentPane.add(message, null);

        JDialog aDialog = new JDialog(getJFrame(), true);
        aDialog.setTitle("Exception");
        aDialog.setResizable(false);
        aDialog.setContentPane(aContentPane);
        
        aDialog.pack();
        Point loc = getJFrame().getLocation();
        Dimension d = getJFrame().getSize();
        loc.translate((d.width-aDialog.getSize().width)/2, (d.height-aDialog.getSize().height)/2);
        aDialog.setLocation(loc);
        aDialog.setVisible(true);
    }
    
    private void showMessageBox() {
        JLabel message = new JLabel();
        message.setBounds(new Rectangle(60, 68, 180, 18));
        message.setText("Not a supported jurisdiction");
        message.setHorizontalAlignment(SwingConstants.CENTER);
        
        JPanel aContentPane = new JPanel();
        aContentPane.setLayout(null);
        aContentPane.setPreferredSize(new Dimension(300, 150));
        aContentPane.add(message, null);

        JDialog aDialog = new JDialog(getJFrame(), true);
        aDialog.setTitle("Info");
        aDialog.setResizable(false);
        aDialog.setContentPane(aContentPane);
        
        aDialog.pack();
        Point loc = getJFrame().getLocation();
        Dimension d = getJFrame().getSize();
        loc.translate((d.width-aDialog.getSize().width)/2, (d.height-aDialog.getSize().height)/2);
        aDialog.setLocation(loc);
        aDialog.setVisible(true);
    }
    
    private void updatePrimarySubdiv() {
        LegalStructureEnum legalStructureType = LegalStructureEnum.valueOf(getLegalField().getSelectedItem().toString());
        mEmployer.setLegalStructure(legalStructureType);
        IndustryTypeEnum industryType = IndustryTypeEnum.valueOf(industryField.getSelectedItem().toString());
        mEmployer.setIndustryType(industryType);
        mEmployer.setNonProfit(isNonProfitCheckBox().isSelected());
    }
    
    private void updateWorkPhysicalAddress() {
        mWorkAddress.setAddressLine1(getAddressField().getText());        
        mWorkAddress.setCity(getCityField().getText());
        mWorkAddress.setRegionOrState(stateAbbreviations[getStateField().getSelectedIndex()]);
        mWorkAddress.setZipOrPostalCode(getZipField().getText());
        mWorkAddress.setCountryISOCode(getISOCountryCode(getCountryField()));
        mWorkAddress.setCountry(getCountryField().getSelectedItem().toString());
        //Get the Global Address From the Physical Address
        
    }
    
    private void updateHomePhysicalAddress() {
        mHomeAddress.setAddressLine1(getHomeAddressField().getText());        
        mHomeAddress.setCity(getHomeCityField().getText());
        mHomeAddress.setRegionOrState(stateAbbreviations[getHomeStateField().getSelectedIndex()]);
        mHomeAddress.setZipOrPostalCode(getHomeZipField().getText());
        mHomeAddress.setCountryISOCode(getISOCountryCode(getCountryField()));
        mHomeAddress.setCountry(getCountryField().getSelectedItem().toString());
    }
    
    @SuppressWarnings("deprecation")
    private void updateBirthDate() {
       try
       {
           String theDate = getDateOfBirthField().getText();
           mBirthDate = new Date(Date.parse(theDate));
       }
       catch (Exception e)
       {
           showUnexpectedErrorBox("Error in parsing the birthdate!  Please enter a date in the correct format.");
       }
    }
    
    private void updateRelationship() {
        mRelationToOwner = RelationshipToOwnerEnum.valueOf(getRelationshipField().getSelectedItem().toString());
    }
    
    private void clearApplicableTaxesAndWithholdingForms()
    {
        // Clear the existing table first
        TableModel tableModel = getTaxesTable().getModel();
        if (tableModel != null)
        {
            MyTableModel myTableModel = (MyTableModel)tableModel;
            myTableModel.clear();
        }
        // Clear the existing list first
        ListModel listModel = getWithholdingFormsList().getModel();
        if (listModel != null) {
            ((DefaultListModel)listModel).clear();
        }
    }
    
    /**
     * This method initializes showTaxes	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getShowTaxes()
    {
        if (showTaxes == null)
        {
            showTaxes = new JButton();
            showTaxes.setBounds(new Rectangle(95, 492, 230, 26));  
            showTaxes.setText("Show Taxes & Withholding forms");  
            showTaxes.setToolTipText("Click to see a list of applicable taxes and withholding forms for the displayed address");
            showTaxes.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent e)
                {
                    clearApplicableTaxesAndWithholdingForms();
                    updateRelationship();
                    updatePrimarySubdiv();
                    updateWorkPhysicalAddress();
                    updateHomePhysicalAddress();
                    updateBirthDate();
                    performGetApplicableTaxesAndForms(mEmployer, mWorkAddress, mHomeAddress, mBirthDate,
                                                      mRelationToOwner);
                }
            });
        }
        return showTaxes;
    }

    /**
     * This method initializes employerInfoPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getEmployerInfoPanel()
    {
        if (employerInfoPanel == null)
        {
            employerInfoPanel = new JPanel();
            employerInfoPanel.setLayout(null);  
            employerInfoPanel.setBounds(new Rectangle(620, 125, 295, 102));  
            employerInfoPanel.setBorder(BorderFactory.createCompoundBorder(
                                            BorderFactory.createTitledBorder(" Employer Info: "),
                                            BorderFactory.createEmptyBorder(1,1,1,1)));
            employerInfoPanel.add(legalLabel, null);  
            employerInfoPanel.add(getLegalField(), null);  
            employerInfoPanel.add(industryLabel, null);  
            employerInfoPanel.add(getIndustryField(), null);  
            employerInfoPanel.add(isNonProfitCheckBox(), null);  
        }
        return employerInfoPanel;
    }

    /**
     * This method initializes addressField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getAddressField()
    {
        if (addressField == null)
        {
            addressField = new JTextField();
            addressField.setBounds(new Rectangle(110, 15, 170, 26));  
            addressField.setText("");  
            addressField.requestFocusInWindow();
        }
        return addressField;
    }

    /**
     * This method initializes validateAddress	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getValidateAddress()
    {
        if (validateAddress == null)
        {
            validateAddress = new JButton();
            validateAddress.setText("Validate");  
            validateAddress.setBounds(new Rectangle(200, 155, 80, 26));  
            validateAddress.setToolTipText("Click to verify whether or not the displayed address is valid");
            validateAddress.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent e)
                {
                    // TODO: Replace the following line of code with actual code to validate the address
                    addressIsValid.setText("unknown");
                }
            });
        }
        return validateAddress;
    }

    /**
     * This method initializes addressIsValid	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getAddressIsValid()
    {
        if (addressIsValid == null)
        {
            addressIsValid = new JTextField();
            addressIsValid.setEnabled(false);
            addressIsValid.setBounds(new Rectangle(110, 154, 80, 26));  
            addressIsValid.setBackground(new Color(250, 250, 220));
        }
        return addressIsValid;
    }

    /**
     * This method initializes countryField	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getCountryField()
    {
        if (countryField == null)
        {
            countryField = new JComboBox();
            countryField.setBounds(new Rectangle(110, 123, 170, 26));  
            countryField.setBackground(Color.white);
            for (int i = 0; i < mCountries.length; i++) {
                countryField.addItem(makeObj(mCountries[i]));
            }
        }
        return countryField;
    }

    /**
     * This method initializes addressPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getAddressPanel()
    {
        if (addressPanel == null)
        {
            addressPanel = new JPanel();
            addressPanel.setLayout(null);  
            addressPanel.setBounds(new Rectangle(15, 38, 295, 189));  
            addressPanel.setBorder(BorderFactory.createCompoundBorder(
                                       BorderFactory.createTitledBorder(" Work Location: "),
                                       BorderFactory.createEmptyBorder(1,1,1,1)));
            addressPanel.add(addressLabel, null);  
            addressPanel.add(getAddressField(), null);  
            addressPanel.add(cityLabel, null);  
            addressPanel.add(getCityField(), null);  
            addressPanel.add(stateLabel, null);  
            addressPanel.add(getStateField(), null);  
            addressPanel.add(zipLabel, null);  
            addressPanel.add(getZipField(), null);  
            addressPanel.add(countryLabel, null);  
            addressPanel.add(getCountryField(), null);  
            addressPanel.add(addressIsValidLabel, null);  
            addressPanel.add(getAddressIsValid(), null);  
            addressPanel.add(getValidateAddress(), null);  
        }
        return addressPanel;
    }

    /**
     * This method initializes personalInfoPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getPersonalInfoPanel()
    {
        if (personalInfoPanel == null)
        {
            dateOfBirthFormatLabel = new JLabel();
            dateOfBirthFormatLabel.setBounds(new Rectangle(192, 42, 80, 26));  
            dateOfBirthFormatLabel.setText("(mm/dd/yyyy)");  
            dateOfBirthLabel = new JLabel();
            dateOfBirthLabel.setBounds(new Rectangle(10, 42, 80, 26));  
            dateOfBirthLabel.setText("Date of birth:");  
            dateOfBirthLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            relationshipLabel = new JLabel();
            relationshipLabel.setBounds(new Rectangle(10, 16, 80, 26));  
            relationshipLabel.setText("Relationship:");  
            relationshipLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            personalInfoPanel = new JPanel();
            personalInfoPanel.setLayout(null);  
            personalInfoPanel.setBounds(new Rectangle(620, 38, 295, 80));  
            personalInfoPanel.setBorder(BorderFactory.createCompoundBorder(
                                        BorderFactory.createTitledBorder(" Personal Info: "),
                                        BorderFactory.createEmptyBorder(1,1,1,1)));
            personalInfoPanel.add(relationshipLabel, null);  
            personalInfoPanel.add(dateOfBirthLabel, null);  
            personalInfoPanel.add(getRelationshipField(), null);  
            personalInfoPanel.add(getDateOfBirthField(), null);  
            personalInfoPanel.add(dateOfBirthFormatLabel, null);  
        }
        return personalInfoPanel;
    }

    /**
     * This method initializes relationshipField	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getRelationshipField()
    {
        if (relationshipField == null)
        {
            relationshipField = new JComboBox();
            relationshipField.setBounds(new Rectangle(96, 15, 184, 26));  
            relationshipField.setBackground(Color.white);
            RelationshipToOwnerEnum[] relations = RelationshipToOwnerEnum.values();
            for (int i = 0; i < relations.length; i++) {
                relationshipField.addItem(makeObj(relations[i].toString()));
            }
        }
        return relationshipField;
    }

    /**
     * This method initializes DateOfBirthField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getDateOfBirthField()
    {
        if (DateOfBirthField == null)
        {
            DateOfBirthField = new JTextField();
            DateOfBirthField.setBounds(new Rectangle(96, 42, 90, 26));  
            DateOfBirthField.setText("");  
        }
        return DateOfBirthField;
    }

    /**
     * This method initializes taxesScrollPane	
     * 	
     * @return javax.swing.JScrollPane	
     */
    private JScrollPane getTaxesScrollPane()
    {
        if (taxesScrollPane == null)
        {
            taxesScrollPane = new JScrollPane();
            taxesScrollPane.setBounds(new Rectangle(15, 256, 322, 227));  
            taxesScrollPane.setViewportView(getTaxesTable());  
        }
        return taxesScrollPane;
    }

    /**
     * This method initializes taxesTable	
     * 	
     * @return javax.swing.JTable	
     */
    private JTable getTaxesTable()
    {
        if (taxesTable == null)
        {
            taxesTable = new JTable(new MyTableModel());
            taxesTable.setBorder(new LineBorder(Color.black));
            taxesTable.getColumnModel().getColumn(0).setPreferredWidth(240);
            taxesTable.getColumnModel().getColumn(1).setPreferredWidth(46);
            taxesTable.getColumnModel().getColumn(2).setPreferredWidth(80);
            taxesTable.setShowHorizontalLines(false);
            taxesTable.setEnabled(false);
            //Set up tool tips for the LawId column.
            DefaultTableCellRenderer renderer =
                    new DefaultTableCellRenderer();
            renderer.setToolTipText("This is the lawId value from law.h in QuickBooks");
            taxesTable.getColumnModel().getColumn(1).setCellRenderer(renderer);
        }
        return taxesTable;
    }

    class MyTableModel extends AbstractTableModel
    {
        static final long serialVersionUID = 124356798012L;
        private String[] columnNames = {"Tax item", "LawId", "Paid by"};
        private Object [][] data = {{"", "", ""}};
        
        public void init(int size)
        {
            data = new Object[size][3];
        }
        
        public void clear()
        {
            data = null;
            fireTableDataChanged();
        }
        
        public int getColumnCount()
        {
            return columnNames.length;
        }
        
        public int getRowCount()
        {
            return ((data != null) ? data.length : 0);
        }
        
        public String getColumnName(int col)
        {
            return columnNames[col];
        }
        
        public Object getValueAt(int row, int col)
        {
            return data[row][col];
        }
        
        public void setValueAt(Object aValue, int row, int col)
        {
            data[row][col] = aValue;
            fireTableCellUpdated(row, col);
        }
        
    }
    
    /**
     * This method initializes homeAddressPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getHomeAddressPanel()
    {
        if (homeAddressPanel == null)
        {
            homeAddressIsValidLabel = new JLabel();
            homeAddressIsValidLabel.setBounds(new Rectangle(12, 154, 90, 26));  
            homeAddressIsValidLabel.setText("Is valid:");  
            homeAddressIsValidLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            homeCountryLabel = new JLabel();
            homeCountryLabel.setBounds(new Rectangle(12, 124, 90, 26));  
            homeCountryLabel.setText("Country:");  
            homeCountryLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            homeZipLabel = new JLabel();
            homeZipLabel.setBounds(new Rectangle(12, 96, 90, 26));  
            homeZipLabel.setText("Zip:");  
            homeZipLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            homeStateLabel = new JLabel();
            homeStateLabel.setBounds(new Rectangle(12, 69, 90, 26));  
            homeStateLabel.setText("State:");  
            homeStateLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            homeCityLabel = new JLabel();
            homeCityLabel.setBounds(new Rectangle(12, 42, 90, 26));  
            homeCityLabel.setText("City:");  
            homeCityLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            homeAddressLabel = new JLabel();
            homeAddressLabel.setBounds(new Rectangle(12, 18, 90, 26));  
            homeAddressLabel.setText("Street Address:");  
            homeAddressLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            homeAddressPanel = new JPanel();
            homeAddressPanel.setLayout(null);  
            homeAddressPanel.setBounds(new Rectangle(320, 38, 295, 189));  
            homeAddressPanel.setBorder(BorderFactory.createCompoundBorder(
                                       BorderFactory.createTitledBorder(" Home Address: "),
                                       BorderFactory.createEmptyBorder(1,1,1,1)));
            homeAddressPanel.add(homeAddressLabel, null);  
            homeAddressPanel.add(homeCityLabel, null);  
            homeAddressPanel.add(homeStateLabel, null);  
            homeAddressPanel.add(homeZipLabel, null);  
            homeAddressPanel.add(homeCountryLabel, null);  
            homeAddressPanel.add(homeAddressIsValidLabel, null);  
            homeAddressPanel.add(getHomeAddressField(), null);  
            homeAddressPanel.add(getHomeCityField(), null);  
            homeAddressPanel.add(getHomeStateField(), null);  
            homeAddressPanel.add(getHomeZipField(), null);  
            homeAddressPanel.add(getHomeCountryField(), null);  
            homeAddressPanel.add(getHomeAddressIsValid(), null);  
            homeAddressPanel.add(getHomeValidateAddress(), null);  
        }
        return homeAddressPanel;
    }

    /**
     * This method initializes homeAddressField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getHomeAddressField()
    {
        if (homeAddressField == null)
        {
            homeAddressField = new JTextField();
            homeAddressField.setBounds(new Rectangle(110, 15, 170, 26));  
            homeAddressField.setText("");  
        }
        return homeAddressField;
    }

    /**
     * This method initializes homeCityField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getHomeCityField()
    {
        if (homeCityField == null)
        {
            homeCityField = new JTextField();
            homeCityField.setBounds(new Rectangle(110, 42, 170, 26));  
            homeCityField.setText("");  
        }
        return homeCityField;
    }

    /**
     * This method initializes homeStateField	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getHomeStateField()
    {
        if (homeStateField == null)
        {
            homeStateField = new JComboBox();
            homeStateField.setBounds(new Rectangle(110, 69, 170, 26));  
            homeStateField.setBackground(Color.white);  
            for (int i = 0; i < mStates.length; i++) {
                homeStateField.addItem(makeObj(mStates[i]));
            }
        }
        return homeStateField;
    }

    /**
     * This method initializes homeZipField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getHomeZipField()
    {
        if (homeZipField == null)
        {
            homeZipField = new JTextField();
            homeZipField.setBounds(new Rectangle(110, 96, 170, 26));  
            homeZipField.setText("");  
        }
        return homeZipField;
    }

    /**
     * This method initializes homeCountryField	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getHomeCountryField()
    {
        if (homeCountryField == null)
        {
            homeCountryField = new JComboBox();
            homeCountryField.setBounds(new Rectangle(110, 123, 170, 26));  
            homeCountryField.setBackground(Color.white);  
            for (int i = 0; i < mCountries.length; i++) {
                homeCountryField.addItem(makeObj(mCountries[i]));
            }
        }
        return homeCountryField;
    }

    /**
     * This method initializes homeAddressIsValid	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getHomeAddressIsValid()
    {
        if (homeAddressIsValid == null)
        {
            homeAddressIsValid = new JTextField();
            homeAddressIsValid.setBounds(new Rectangle(110, 154, 80, 26));  
            homeAddressIsValid.setEnabled(false);  
            homeAddressIsValid.setBackground(new Color(250, 250, 220));  
        }
        return homeAddressIsValid;
    }

    /**
     * This method initializes homeValidateAddress	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getHomeValidateAddress()
    {
        if (homeValidateAddress == null)
        {
            homeValidateAddress = new JButton();
            homeValidateAddress.setBounds(new Rectangle(200, 155, 80, 26));  
            homeValidateAddress.setText("Validate");  
            homeValidateAddress.setToolTipText("Click to verify whether or not the displayed address is valid");  
            homeValidateAddress.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent e)
                {
                    // TODO: Replace the following line of code with actual code to validate the address
                    homeAddressIsValid.setText("unknown");
                }
            });
        }
        return homeValidateAddress;
    }

    /**
     * This method initializes withholdingFormsList	
     * 	
     * @return javax.swing.JList	
     */
    private JList getWithholdingFormsList()
    {
        if (withholdingFormsList == null)
        {
            DefaultListModel listModel = new DefaultListModel();
            withholdingFormsList = new JList(listModel);
            withholdingFormsList.setBorder(new LineBorder(Color.black));
            withholdingFormsList.setBounds(new Rectangle(345, 272, 86, 108));
            withholdingFormsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            // Set the font to the same font used in the taxesTable
            withholdingFormsList.setFont(new Font("Dialog", 0, 12));
            withholdingFormsList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
            {
                public void valueChanged(javax.swing.event.ListSelectionEvent e)
                {
                    if (e.getValueIsAdjusting() == false)
                    {
                        currentWithholdingForm = withholdingFormsList.getSelectedIndex();
                        updateWithholdingInfoSection();
                    }
                }
            });
        }
        return withholdingFormsList;
    }

    /**
     * This method initializes workerComboBox	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getWorkerComboBox()
    {
        if (workerComboBox == null)
        {
            workerComboBox = new JComboBox();
            workerComboBox.setBounds(new Rectangle(125, 10, 184, 26));  
            workerComboBox.setBackground(Color.white);
            workerComboBox.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent e)
                {
                    JComboBox cb = (JComboBox)e.getSource();
                    currentWorker = cb.getSelectedIndex();
                    clearApplicableTaxesAndWithholdingForms();
                    loadDataAndUpdateForm();
                    performGetApplicableTaxesAndForms(mEmployer, mWorkAddress, mHomeAddress, mBirthDate,
                                                      mRelationToOwner);
                }
            });
        }
        return workerComboBox;
    }

    /**
     * This method initializes IsNonProfitCheckBox	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox isNonProfitCheckBox()
    {
        if (IsNonProfitCheckBox == null)
        {
            IsNonProfitCheckBox = new JCheckBox();
            IsNonProfitCheckBox.setBounds(new Rectangle(106, 74, 108, 23));  
            IsNonProfitCheckBox.setText("Is Non Profit");  
        }
        return IsNonProfitCheckBox;
    }

    /**
     * This method initializes withholdingInfoPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getWithholdingInfoPanel()
    {
        if (withholdingInfoPanel == null)
        {
            additionalWithholdingAmountLabel = new JLabel();
            additionalWithholdingAmountLabel.setBounds(new Rectangle(12, 190, 120, 26));  
            additionalWithholdingAmountLabel.setText("Extra Withholding:");  
            additionalWithholdingAmountLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            allowancesLabel = new JLabel();
            allowancesLabel.setBounds(new Rectangle(12, 160, 120, 26));  
            allowancesLabel.setText("Allowances:");  
            allowancesLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            withholdingStatusLabel = new JLabel();
            withholdingStatusLabel.setBounds(new Rectangle(12, 130, 120, 26));  
            withholdingStatusLabel.setText("Withholding Status:");  
            withholdingStatusLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            formDateFormatLabel = new JLabel();
            formDateFormatLabel.setBounds(new Rectangle(216, 80, 80, 26));  
            formDateFormatLabel.setText("(mm/dd/yyyy)");  
            endDateFormatLabel = new JLabel();
            endDateFormatLabel.setBounds(new Rectangle(216, 50, 80, 26));  
            endDateFormatLabel.setText("(mm/dd/yyyy)");  
            startDateFormatLabel = new JLabel();
            startDateFormatLabel.setBounds(new Rectangle(216, 20, 80, 26));  
            startDateFormatLabel.setText("(mm/dd/yyyy)");  
            formDateLabel = new JLabel();
            formDateLabel.setBounds(new Rectangle(12, 80, 100, 26));  
            formDateLabel.setText("FormDate:");  
            formDateLabel.setHorizontalAlignment(SwingConstants.RIGHT);  
            endDateLabel = new JLabel();
            endDateLabel.setBounds(new Rectangle(12, 50, 100, 26));  
            endDateLabel.setText("EndDate:");  
            endDateLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            startDateLabel = new JLabel();
            startDateLabel.setText("StartDate:");  
            startDateLabel.setBounds(new Rectangle(12, 20, 100, 26));  
            startDateLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            withholdingInfoPanel = new JPanel();
            withholdingInfoPanel.setLayout(null);  
            withholdingInfoPanel.setBounds(new Rectangle(440, 256, 472, 226));  
            withholdingInfoPanel.setBorder(BorderFactory.createCompoundBorder(
                                           BorderFactory.createTitledBorder(" Withholding Info: "),
                                           BorderFactory.createEmptyBorder(1,1,1,1)));
            withholdingInfoPanel.add(startDateLabel, null);  
            withholdingInfoPanel.add(getStartDateField(), null);  
            withholdingInfoPanel.add(endDateLabel, null);  
            withholdingInfoPanel.add(getEndDateField(), null);  
            withholdingInfoPanel.add(formDateLabel, null);  
            withholdingInfoPanel.add(getFormDateField(), null);  
            withholdingInfoPanel.add(startDateFormatLabel, null);  
            withholdingInfoPanel.add(endDateFormatLabel, null);  
            withholdingInfoPanel.add(formDateFormatLabel, null);  
            withholdingInfoPanel.add(getIsDerivedCheckBox(), null);  
            withholdingInfoPanel.add(getIsExemptCheckBox(), null);  
            withholdingInfoPanel.add(getIsSourceDocOnFileCheckBox(), null);  
            withholdingInfoPanel.add(getWithholdingStatusComboBox(), null);  
            withholdingInfoPanel.add(withholdingStatusLabel, null);  
            withholdingInfoPanel.add(allowancesLabel, null);  
            withholdingInfoPanel.add(additionalWithholdingAmountLabel, null);  
            withholdingInfoPanel.add(getAllowancesTextField(), null);  
            withholdingInfoPanel.add(getAdditionalWithholdingAmountTextField(), null);  
        }
        return withholdingInfoPanel;
    }

    /**
     * This method initializes startDateField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getStartDateField()
    {
        if (startDateField == null)
        {
            startDateField = new JTextField();
            startDateField.setBounds(new Rectangle(120, 20, 90, 26));  
            startDateField.setText("");  
        }
        return startDateField;
    }

    /**
     * This method initializes endDateField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getEndDateField()
    {
        if (endDateField == null)
        {
            endDateField = new JTextField();
            endDateField.setBounds(new Rectangle(120, 50, 90, 26));  
            endDateField.setText("");  
        }
        return endDateField;
    }

    /**
     * This method initializes formDateField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getFormDateField()
    {
        if (formDateField == null)
        {
            formDateField = new JTextField();
            formDateField.setBounds(new Rectangle(120, 80, 90, 26));  
            formDateField.setText("");  
        }
        return formDateField;
    }

    /**
     * This method initializes isDerivedCheckBox	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getIsDerivedCheckBox()
    {
        if (isDerivedCheckBox == null)
        {
            isDerivedCheckBox = new JCheckBox();
            isDerivedCheckBox.setBounds(new Rectangle(320, 20, 140, 26));  
            isDerivedCheckBox.setText("IsDerived");  
        }
        return isDerivedCheckBox;
    }

    /**
     * This method initializes IsExemptCheckBox	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getIsExemptCheckBox()
    {
        if (IsExemptCheckBox == null)
        {
            IsExemptCheckBox = new JCheckBox();
            IsExemptCheckBox.setBounds(new Rectangle(320, 50, 140, 26));  
            IsExemptCheckBox.setText("IsExempt");  
        }
        return IsExemptCheckBox;
    }

    /**
     * This method initializes isSourceDocOnFileCheckBox	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getIsSourceDocOnFileCheckBox()
    {
        if (isSourceDocOnFileCheckBox == null)
        {
            isSourceDocOnFileCheckBox = new JCheckBox();
            isSourceDocOnFileCheckBox.setBounds(new Rectangle(320, 80, 140, 26));  
            isSourceDocOnFileCheckBox.setText("IsSourceDocOnFile");  
        }
        return isSourceDocOnFileCheckBox;
    }

    /**
     * This method initializes withholdingStatusComboBox	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getWithholdingStatusComboBox()
    {
        if (withholdingStatusComboBox == null)
        {
            withholdingStatusComboBox = new JComboBox();
            withholdingStatusComboBox.setBounds(new Rectangle(140, 130, 230, 26));  
            withholdingStatusComboBox.setBackground(Color.white);  
            W4WithholdingStatusEnum[] withholdingStatuses = W4WithholdingStatusEnum.values();
            for (int i = 0; i < withholdingStatuses.length; i++) {
                withholdingStatusComboBox.addItem(makeObj(withholdingStatuses[i].toString()));
            }
        }
        return withholdingStatusComboBox;
    }

    /**
     * This method initializes allowancesTextField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getAllowancesTextField()
    {
        if (allowancesTextField == null)
        {
            allowancesTextField = new JTextField();
            allowancesTextField.setBounds(new Rectangle(140, 160, 90, 26));  
            allowancesTextField.setText("1");  
        }
        return allowancesTextField;
    }

    /**
     * This method initializes additionalWithholdingAmountTextField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getAdditionalWithholdingAmountTextField()
    {
        if (additionalWithholdingAmountTextField == null)
        {
            additionalWithholdingAmountTextField = new JTextField();
            additionalWithholdingAmountTextField.setBounds(new Rectangle(140, 190, 90, 26));  
            additionalWithholdingAmountTextField.setText("0");  
        }
        return additionalWithholdingAmountTextField;
    }

    /**
     * @param args
     */
    public static void main(String[] args)
    {

        SwingUtilities.invokeLater(new Runnable()
        {

            public void run()
            {
                TaxRulesViewer application = new TaxRulesViewer();
                application.getJFrame().setVisible(true);
            }
        });
    }

    /**
     * This method initializes jFrame
     * 
     * @return javax.swing.JFrame
     */
    private JFrame getJFrame()
    {
        if (jFrame == null)
        {
            JFrame.setDefaultLookAndFeelDecorated(false);
            jFrame = new JFrame();
            jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            jFrame.setJMenuBar(getJJMenuBar());
            jFrame.setSize(940, 600);
            jFrame.setLocationRelativeTo(null);
            jFrame.setContentPane(getJContentPane());
            jFrame.setTitle(appName);
            jFrame.setResizable(false);
            jFrame.addWindowListener(new java.awt.event.WindowAdapter()
            {
                public void windowActivated(java.awt.event.WindowEvent e)
                {
                    getAddressField().requestFocusInWindow();
                }
            });
        }
        return jFrame;
    }

    /**
     * This method initializes jContentPane
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getJContentPane()
    {
        if (jContentPane == null)
        {
            jContentPane = new JPanel();
            jContentPane.setLayout(null);
            jContentPane.add(getJTabs(), null);  
        }
        return jContentPane;
    }

    /**
     * This method initializes jJMenuBar	
     * 	
     * @return javax.swing.JMenuBar	
     */
    private JMenuBar getJJMenuBar()
    {
        if (jJMenuBar == null)
        {
            jJMenuBar = new JMenuBar();
            jJMenuBar.add(getFileMenu());
            jJMenuBar.add(getEditMenu());
            jJMenuBar.add(getHelpMenu());
        }
        return jJMenuBar;
    }

    /**
     * This method initializes jMenu	
     * 	
     * @return javax.swing.JMenu	
     */
    private JMenu getFileMenu()
    {
        if (fileMenu == null)
        {
            fileMenu = new JMenu();
            fileMenu.setText("File");
            fileMenu.add(getExitMenuItem());
        }
        return fileMenu;
    }

    /**
     * This method initializes jMenu	
     * 	
     * @return javax.swing.JMenu	
     */
    private JMenu getEditMenu()
    {
        if (editMenu == null)
        {
            editMenu = new JMenu();
            editMenu.setText("Edit");
            editMenu.add(getCutMenuItem());
            editMenu.add(getCopyMenuItem());
            editMenu.add(getPasteMenuItem());
            editMenu.setEnabled(false);
        }
        return editMenu;
    }

    /**
     * This method initializes jMenu	
     * 	
     * @return javax.swing.JMenu	
     */
    private JMenu getHelpMenu()
    {
        if (helpMenu == null)
        {
            helpMenu = new JMenu();
            helpMenu.setText("Help");
            helpMenu.add(getAboutMenuItem());
        }
        return helpMenu;
    }

    /**
     * This method initializes jMenuItem	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getExitMenuItem()
    {
        if (exitMenuItem == null)
        {
            exitMenuItem = new JMenuItem();
            exitMenuItem.setText("Exit");
            exitMenuItem.addActionListener(new ActionListener()
            {

                public void actionPerformed(ActionEvent e)
                {
                    System.exit(0);
                }
            });
        }
        return exitMenuItem;
    }

    /**
     * This method initializes jMenuItem	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getAboutMenuItem()
    {
        if (aboutMenuItem == null)
        {
            aboutMenuItem = new JMenuItem();
            aboutMenuItem.setText("About");
            aboutMenuItem.addActionListener(new ActionListener()
            {

                public void actionPerformed(ActionEvent e)
                {
                    JDialog aboutDialog = getAboutDialog();
                    aboutDialog.pack();
                    Point loc = getJFrame().getLocation();
                    Dimension d = getJFrame().getSize();
                    loc.translate((d.width-aboutDialog.getSize().width)/2, (d.height-aboutDialog.getSize().height)/2);
                    aboutDialog.setLocation(loc);
                    aboutDialog.setVisible(true);
                }
            });
        }
        return aboutMenuItem;
    }

    /**
     * This method initializes aboutDialog	
     * 	
     * @return javax.swing.JDialog
     */
    private JDialog getAboutDialog()
    {
        if (aboutDialog == null)
        {
            aboutDialog = new JDialog(getJFrame(), true);
            aboutDialog.setTitle("About " + getJFrame().getTitle());
            aboutDialog.setResizable(false);
            aboutDialog.setContentPane(getAboutContentPane());
        }
        return aboutDialog;
    }

    /**
     * This method initializes aboutContentPane
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getAboutContentPane()
    {
        if (aboutContentPane == null)
        {
            contributor1 = new JLabel();
            contributor1.setBounds(new Rectangle(60, 68, 180, 18));  
            contributor1.setText("Peter Szeto");  
            contributor1.setHorizontalAlignment(SwingConstants.CENTER);
            contributorLabel = new JLabel();
            contributorLabel.setBounds(new Rectangle(60, 50, 180, 18));  
            contributorLabel.setText("Contributors:");  
            contributorLabel.setHorizontalAlignment(SwingConstants.CENTER);
            aboutVersion = new JLabel();
            aboutVersion.setBounds(new Rectangle(60, 20, 180, 18));  
            aboutVersion.setText("Version 0.41");  
            aboutVersion.setHorizontalAlignment(SwingConstants.CENTER);
            aboutContentPane = new JPanel();
            aboutContentPane.setLayout(null);
            aboutContentPane.setPreferredSize(new Dimension(300, 150));
            aboutContentPane.add(aboutVersion, null);  
            aboutContentPane.add(contributorLabel, null);  
            aboutContentPane.add(contributor1, null);  
        }
        return aboutContentPane;
    }

    /**
     * This method initializes jMenuItem	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getCutMenuItem()
    {
        if (cutMenuItem == null)
        {
            cutMenuItem = new JMenuItem();
            cutMenuItem.setText("Cut");
            cutMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, Event.CTRL_MASK, true));
        }
        return cutMenuItem;
    }

    /**
     * This method initializes jMenuItem	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getCopyMenuItem()
    {
        if (copyMenuItem == null)
        {
            copyMenuItem = new JMenuItem();
            copyMenuItem.setText("Copy");
            copyMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, Event.CTRL_MASK, true));
        }
        return copyMenuItem;
    }

    /**
     * This method initializes jMenuItem	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getPasteMenuItem()
    {
        if (pasteMenuItem == null)
        {
            pasteMenuItem = new JMenuItem();
            pasteMenuItem.setText("Paste");
            pasteMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, Event.CTRL_MASK, true));
        }
        return pasteMenuItem;
    }

    
}  //  @jve:decl-index=0:visual-constraint="17,625"
