package com.intuit.sb.payroll.tools;

import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Map;

import javax.transaction.UserTransaction;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.common.PayrollTestBase;
import com.intuit.sb.payroll.capability.common.WorkLocationAndWorkerDataPopulator;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.impl.EmployerSeedData;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.employer.EmployerEAO;
import com.intuit.sb.payroll.entity.employer.WorkLocationEAO;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.taxauthority.JurisdictionEAO;
import com.intuit.sb.payroll.services.common.EntityTestBaseHelper;
import com.intuit.sb.payroll.services.taxes.TaxWithholdingsService;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;
import com.intuit.sb.payroll.services.wages.WageService;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerService;
import com.intuit.sb.payroll.util.PrincipalUtil;

/**
 * This utility file will seed Payroll related global data into repository
 *
 * $Author: amalik $ $Revision: #1 $ $DateTime: 2008/01/01 11:26:23 $
 */
@SuppressWarnings("unused")
public class CreateNewEmployer {

	/*
	 * if perfTest is true, populdate workers for all employers otherwise, only
	 * populate workers for test users "peter"/"paul"/"mary"
	 */
	private boolean perfTest = false;
	private static final String PASSWORD = "password";
	private UserTransaction tx;

	private String[] usernames = { "intuituser1", "paul", "peter", "mary", "ptstestuserone", "ptstestusertwo" };

	/* Username and password for PTS Test User (there will be two of them)
	 * ptstestuserone and ptstestusertwo
	 * Passwords for these will be simply "ptstestuser"
	 * Equivalent users will be created for BHP
	 */
	private static final String PTS_TEST_USER_PREFIX   = "ptstestuser";
	private static final String PTS_TEST_USER_PASSWORD = "intuitptstest";

	/* the realm ID of paul_gemini@intuit.com in ptc server */
	private String paulRealmId ="103340352";
	/* the realm ID of peter_gemini@intuit.com in ptc server */
	private String peterReamId="103301051";
	/* the realm ID of mary_gemini@intuit.com in ptc server */
	private String maryRealmId="103301058";

	private int noOfCompanies = 0;



	/**
	 * This utility method will perform these steps
	 *
	 * 1. Update security policy file with the new user data, generates unique
	 * realmId and AuthId for the user 2. Seed Compliance data for the realmId
	 * generated in the step 1 ( temp step only for Alpha) 3. Seed Employer
	 * related data
	 *
	 * @param directory
	 *            location containings Employer data
	 * @param noOfEmployers
	 *            number of Employers to create
	 */
	public void createNewUser(int noOfEmployers) throws Exception {

		/* seed data in BHPIAMMock */
		createNewMockUsers(noOfEmployers);

		/* seed data for using IAM + ptc server */
		seedEmployerRelatedDataForIAM();
	}

	/*
	 * this helper method populates users in BHPIAMMock
	 */
	private void createNewMockUsers(int noOfEmployers) throws Exception
	{
		tx = PayrollTestBase.getUserTransaction();

		SecurityPolicyHelper secHelper = new SecurityPolicyHelper();
		String password = "";

		/* seed data in BHPIAMMock */
		for (String username : usernames) {
			try {
				//seed data for intuituser1, intuituser2, etc
				if (qualifiedUser(username)) {
					String indexStr = username.substring(username.length() - 1);
					int index = Integer.valueOf(indexStr);
					String stripedUser = username.substring(0, username.length() - 1);

					for (int i = index; i <= noOfEmployers; i++) {
						username = stripedUser + Integer.toString(i);
						// for intuituser$i, set password to password$i
						//seedEmployerRelatedDataForIAMMock(secHelper, username, PASSWORD + Integer.toString(i));
						seedEmployerRelatedDataForIAMMock(secHelper, username, username);
					}
					continue;
				}


				// for other users(peter, paul, mary), set password to the same as the user name
				// for "ptstestuser*", password is "ptstestuser"
				password = username;
				if (username.startsWith(PTS_TEST_USER_PREFIX)) {
					password = PTS_TEST_USER_PASSWORD;
				}
				seedEmployerRelatedDataForIAMMock(secHelper, username, password);
			} catch (Exception e) {
				e.printStackTrace();
				throw e;
			}
		}
		//save to mock_data.xml
		secHelper.saveSecurityPolicy();
	}

	/*
	 * this helper method populdates Data for users "peter/may/paul_gemini@intuit.com" in ptc server
	 */
	private void seedEmployerRelatedDataForIAM()throws Exception
	{
		seedEmployerRelatedDataForIAM("peter_gemini@intuit.com", "103301051");
		seedEmployerRelatedDataForIAM("mary_gemini@intuit.com", "103301058");
		seedEmployerRelatedDataForIAM("paul_gemini@intuit.com", "103340352");
	}

	/**
	 * Create 100 Employers
	 *
	 * @param directory
	 *            location containings Employer data
	 * @throws Exception
	 */
	public void createNewUser() throws Exception {
		createNewUser(100);
	}

	private boolean qualifiedUser(String userName) {
		if (userName.startsWith("intuituser"))
			return true;
		else
			return false;
	}

	private int calculateNoOfEmployeesToPopulate(String userName)
	{
		if ( "peter_gemini@intuit.com".equals(userName) || "paul_gemini@intuit.com".equals(userName)||
			    "mary".equals(userName) || "paul".equals(userName)) {
			return 3;
		}
		if ("peter".equals(userName)) {
			return 10;
		}
		if (!perfTest)
		{
			if(qualifiedUser(userName))
			{
				String userNumberStr = userName.substring(userName.length()-1, userName.length());
				int userNumber =Integer.parseInt(userNumberStr);
				/* for intuituser6 ... intuituser10, this function returns 0, don't populate employee, populate worklocations */
				if(userNumber > 5)
				{
					return 0;
				}
				else
				{
					/* for intuituser1 ... intuituser5, this function returns -1, don't populate employees or work locations */
					return -1;
				}
			}
			else
			{
				/* populate work locations but no employees for all other employers */
				return 0;
			}
		}
		else
		{
			/*
			 * populate workers for perf test, for first hundred companies,
			 * populate 3 users, for the next hundred companies,populate 4
			 * users, for companies from 200-300, populate 5 users, and so
			 * on
			 */
			System.out.println("Perf: Seed work locations and workers for user " + userName + " in Realm " + PrincipalUtil.getRealmId());
			return 3 + (noOfCompanies / 100) % 7;
		}
	}

	private boolean seedEmployerRelatedData(String userName) throws Exception
	{
		//don't seed data for intuituser if perfTest is not true
		if(userName.startsWith("intuituser") && ! perfTest )
		{
			return true;
		}


		tx.begin();
		//create the employer for this user
		EmployerEAO employerEAO = ServiceLocator.getInstance().getService(EmployerEAO.JNDI_NAME);
		List<Employer> employerList = employerEAO.findAll();

		//If there are no employers in the DB, create one.
		if(employerList == null || employerList.size() == 0)
		{
			EmployerComponent employerComp = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
			Employer employer = new Employer();
			employer.setCompanyName(userName);
			employer.setDbaName(userName);
			employerComp.createEmployer(employer);
		}

		WorkLocationEAO workLocationEAO = ServiceLocator.getInstance().getService(WorkLocationEAO.JNDI_NAME);
		List<WorkLocation> workLocations = workLocationEAO.findAll();
		tx.commit();

		if (workLocations.size() > 0) {
			return true;
		}
		System.out.println("Seeding employer data for user " + userName + " in Realm " + PrincipalUtil.getRealmId());

		tx.begin();
		seedJurisdicationData();

		/*
		 * for IAMMock, seed workers for peter, mary and paul
		 * for IAM, seed workers for peter_gemini@intuit.com and paul_gemini@intuit.com, but not for mary_gemini@intuit.com
		 */
		int noOfEmployees = calculateNoOfEmployeesToPopulate(userName);
		seedCompanyWorkLocsAndWorkers(noOfEmployees);
		if ( perfTest )
		{
			PayrollScheduleTestUtils.createDefaultSchedule();
		}
		tx.commit();

		return false;

	}

	private void seedEmployerRelatedDataForIAMMock(SecurityPolicyHelper secHelper, String userName, String password)
			throws Exception {

		noOfCompanies++;

		secHelper.saveUserPropertyFile(userName, password);

		Map<String, String> secContext = secHelper.updateSecurityPolicy(userName);
		//set realm id
		List<String> realmList = new ArrayList<String>();
		String nextRealm = secContext.get(SecurityPolicyHelper.ELEM_REALM_ID);
		realmList.add(nextRealm);
		PrincipalUtil.setPrincipal(userName, nextRealm, realmList);

		boolean workLocationsPreExisted = seedEmployerRelatedData(userName);

		if ( workLocationsPreExisted )
		{
			return;
		}
	}

	private void seedEmployerRelatedDataForIAM(String userName, String realmId)
	throws Exception {
		// set realm id
		List<String> realmList = new ArrayList<String>();
		realmList.add(realmId);
		PrincipalUtil.setPrincipal(userName, realmId, realmList);

		seedEmployerRelatedData(userName);
	}

	private void seedJurisdicationData() throws Exception {

		// jndi lookup for JurisdiationEAO
		JurisdictionEAO jDao = ServiceLocator.getInstance().getService(JurisdictionEAO.JNDI_NAME);

		List<Jurisdiction> jurisdictions = jDao.findAll();

		// Seed the data if necessary
		if (jurisdictions == null || jurisdictions.size() == 0) {
        	EmployerSeedData.setupEmployerDataInSystem();
		}
		BasicTestBase.initializeEAOs();
		BasicTestBase.setUpEmployer();
	}

	/*
	 * Create new Employer
	 */

	private void seedCompanyWorkLocs() {
		WorkLocationAndWorkerDataPopulator workLocationAndWorkerDataPopulator = new WorkLocationAndWorkerDataPopulator(
				3);
		workLocationAndWorkerDataPopulator.populateWorkLocs();
	}

	private void seedCompanyWorkLocsAndWorkers(int numOfWorkers) throws Exception {
		WorkerService workerService = ServiceLocator.getInstance().getService(WorkerService.JNDI_NAME);
		TaxWithholdingsService taxWithholdingsService = ServiceLocator.getInstance().getService(
				TaxWithholdingsService.JNDI_NAME);
		WageService wageService = ServiceLocator.getInstance().getService(WageService.JNDI_NAME);

		if ( numOfWorkers < 0 )
		{
			return;
		}
		seedCompanyWorkLocs();

		for (int i = 0; i < numOfWorkers; i++) {
			EntityTestBaseHelper.PayrollReadyWorker payrollReadyWorker = EntityTestBaseHelper
					.createPayrollReadyWorkerInMemory();
			Worker worker = payrollReadyWorker.getWorker();
			switch (i) {
			case 0:
				setUpCuriousGeorge(workerService, taxWithholdingsService, wageService, worker);
				break;
			case 1:
				setUpKingKong(workerService, taxWithholdingsService, wageService, worker);
				break;
			case 2:
				setUpKingLouie(workerService, taxWithholdingsService, wageService, worker);
				break;
			default:
				setUpCuriousGeorge(workerService, taxWithholdingsService, wageService, worker);
			}
		}

	}

	// Hourly with OT and DT.
	private void setUpCuriousGeorge(WorkerService workerService, TaxWithholdingsService taxWithholdingsService,
			WageService wageService, Worker worker)
			throws ValidationException, WorkflowException, ServiceFailureException {
		worker.setFirstName("Travis");
		worker.setLastName("O'Hearn");
		worker.setJobTitle("Bartender");
		Calendar nowCal = DateUtil.getNowCalendar();
		nowCal.add(Calendar.MONTH, -1);
		worker.setHireDate(nowCal.getTime());
		Calendar jamesBondsDoB = new GregorianCalendar();
		jamesBondsDoB.set(1921, 11, 11); // http://en.wikipedia.org/wiki/James_Bond_(character)#Birth_year_debate
		worker.setBirthDate(jamesBondsDoB.getTime());
		worker.setAvatar("assets/avatars/sil01.swf");
		Worker savedWorker = workerService.saveWorker(worker);

		CompensationItem[] compensationItems = new CompensationItem[3];
		compensationItems[0] = new CompensationItem(null, new BigDecimal("13.00"), 0,
				CompensationType.Hourly, "Hourly", true);
		compensationItems[1] = new CompensationItem(null, new BigDecimal("1.5"), 0,
				CompensationType.Overtime, "Overtime", compensationItems[0], true);
		compensationItems[2] = new CompensationItem(null, new BigDecimal("2.0"), 0,
				CompensationType.Doubletime, "Doubletime", compensationItems[0], true);
		SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[compensationItems.length];
		for (int j = 0; j < compensationItemRequests.length; j++) {
			compensationItemRequests[j] = new SaveCompensationItemRequest(compensationItems[j],
					SaveCompensationItemOperation.ADD);
		}
		wageService.saveCompensationItems(savedWorker.getObjectHandle().getId(), compensationItemRequests);
	}

	//Salary and Bonus
	private void setUpKingKong(WorkerService workerService, TaxWithholdingsService taxWithholdingsService,
			WageService wageService, Worker worker)
			throws ValidationException, WorkflowException, ServiceFailureException {
		worker.setFirstName("Susan");
		worker.setLastName("Smith");
		worker.setJobTitle("Assistant");
		Calendar nowCal = DateUtil.getNowCalendar();
		nowCal.add(Calendar.MONTH, -1);
		worker.setHireDate(nowCal.getTime());
		Calendar jamesBondsDoB = new GregorianCalendar();
		jamesBondsDoB.set(1921, 11, 11); // http://en.wikipedia.org/wiki/James_Bond_(character)#Birth_year_debate
		worker.setBirthDate(jamesBondsDoB.getTime());
		worker.setAvatar("assets/avatars/sil02.swf");
        worker.setHireDate(DateUtil.createDate(2006, 0, 1));
		Worker savedWorker = workerService.saveWorker(worker);

		CompensationItem[] compensationItems = new CompensationItem[2];
		compensationItems[0] = new CompensationItem(null, new BigDecimal("80000.00"), 0,
				CompensationType.Salary, "Salary", true);
		compensationItems[1] = new CompensationItem(null, new BigDecimal("0.0"), 0,
				CompensationType.Bonus, "Bonus", false);
		SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[compensationItems.length];
		for (int j = 0; j < compensationItemRequests.length; j++) {
			compensationItemRequests[j] = new SaveCompensationItemRequest(compensationItems[j],
					SaveCompensationItemOperation.ADD);
		}
		wageService.saveCompensationItems(savedWorker.getObjectHandle().getId(), compensationItemRequests);
	}

	//Commission and Tips
	private void setUpKingLouie(WorkerService workerService, TaxWithholdingsService taxWithholdingsService,
			WageService wageService, Worker worker)
			throws ValidationException, WorkflowException, ServiceFailureException {
		worker.setFirstName("Robert");
		worker.setLastName("Jones");
		worker.setJobTitle("Gardener");
		Calendar nowCal = DateUtil.getNowCalendar();
		nowCal.add(Calendar.MONTH, -1);
		worker.setHireDate(nowCal.getTime());
		Calendar jamesBondsDoB = new GregorianCalendar();
		jamesBondsDoB.set(1921, 11, 11); // http://en.wikipedia.org/wiki/James_Bond_(character)#Birth_year_debate
		worker.setBirthDate(jamesBondsDoB.getTime());
		worker.setAvatar("assets/avatars/sil03.swf");
		Worker savedWorker = workerService.saveWorker(worker);

		CompensationItem[] compensationItems = new CompensationItem[2];
		compensationItems[0] = new CompensationItem(null, new BigDecimal("0.00"), 0,
				CompensationType.Commission, "Commission", false);
		compensationItems[1] = new CompensationItem(null, new BigDecimal("0.0"), 0,
				CompensationType.TipsPaycheck, "Tips", false);
		SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[compensationItems.length];
		for (int j = 0; j < compensationItemRequests.length; j++) {
			compensationItemRequests[j] = new SaveCompensationItemRequest(compensationItems[j],
					SaveCompensationItemOperation.ADD);
		}
		wageService.saveCompensationItems(savedWorker.getObjectHandle().getId(), compensationItemRequests);
	}

	public void setPerfTest(boolean perfTest) {
		this.perfTest = perfTest;
	}

	@SuppressWarnings("unchecked")
	public static void main(String[] args) {
		try {
			CreateNewEmployer seedDataPopulator = new CreateNewEmployer();

			try {
				Class cl = Class.forName("com.intuit.sb.payroll.capability.common.PayrollEmbeddedContainer");
				Method mth = cl.getMethod("startContainer");
				mth.invoke(null);
				//PayrollEmbeddedContainerTestBase.startContainer();
			} catch (Exception ex) {
				throw new RuntimeException("Can not bootstrap embedded container", ex);
			}

			if (args.length > 0) {
				// To populate seed data for performance test, Usage: java
				// CreateNewEmployer 2000 perf
				if (args.length > 1) {
					seedDataPopulator.setPerfTest(true);
				}
				seedDataPopulator.createNewUser(Integer.parseInt(args[0]));
			} else {
				seedDataPopulator.createNewUser(10);
			}
			Class ecl = Class.forName("com.intuit.sb.payroll.capability.common.EmbeddedContainerTestBase");
			Method mth = ecl.getMethod("classTearDown");
			mth.invoke(null);
			//EmbeddedContainerTestBase.classTearDown();
			System.exit(0);
		} catch (Exception ex) {
			throw new RuntimeException("seed data population fails", ex);
		}
	}

}
