package com.intuit.sb.payroll.tools;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class SecurityPolicyHelper {
	private static final String ELEM_REALMS = "Realms";
	private static final String ELEM_REALM_ROLES = "RealmRoles";
	private static final String ELEM_REALM_ROLE_ID = "RealmRole_ID";
	public static final String ELEM_REALM_ID = "Realm_ID";
	private static final String ELEM_IDENTITY_REALM_INFO = "IdentityRealmInfo";
	private static final String ELEM_LOGIN_NAME = "LoginName";
	public static final String ELEM_AUTH_ID = "Auth_ID";
	private static final String ELEM_IDENTITIES = "Identities";

	private Document     securityPolicyDoc = null;
	private final static String securityPolicyFile = "test//sample//sample_mock_data.xml";
	private final String outputPolicyFile = "test//sample//mock_data.xml";
	private final String outputuserPropertyFile = "test//sample//users.properties";
	public static String newline = System.getProperty("line.separator");


	public SecurityPolicyHelper()
	{
		this(securityPolicyFile);
	}

	public SecurityPolicyHelper(String secPolicyFile)
	{
		try {

			securityPolicyDoc = loadSecurityPolicy(secPolicyFile);

		} catch (Exception e) {

			e.printStackTrace();
		}
	}

	/**
	 * Returns the Document representation for the mock_data.xml file
	 * @param fileName
	 * @return org.dom.Document for the xml file
	 * @throws Exception
	 */

	private static Document loadSecurityPolicy(String fileName) throws Exception
	{
		if (fileName == null)
		{
			throw new Exception("File name is null");
		}
		File xmlFile = new File(fileName);

        try
        {
            // Get Document Builder Factory
            DocumentBuilderFactory factory =
            				DocumentBuilderFactory.newInstance();

            // Obtain a document builder object
            DocumentBuilder builder = factory.newDocumentBuilder();

            // Parse the document
            Document doc = builder.parse(xmlFile);

            return doc;
		}catch (Exception e)
		{
			throw e;
		}
	}

	/*
	 * 1. Load security Policy file mock_data.xml
	 *
	 * 2. Calculate available realm Id
	 * 3. Create New Realm Id element
	 *
	 * 4. Create New Realm Role
	 * 		calculate avl realm Id
	 * 		assign some role and permissions to this realmRole
	 *
	 * 5. Create new Identity on this username
	 *
	 */
	public Map<String,String> updateSecurityPolicy(String userName)
	{
		Map<String,String> map = new HashMap<String, String>();
		String realmID = appendNewRealm();
		String realmRole = appendNewRealmRole(realmID);
		String authId = appendNewIdentity(userName, realmID,realmRole);
		map.put(ELEM_REALM_ID, realmID);
		map.put(ELEM_AUTH_ID, authId);

		return map;
	}

	public void saveSecurityPolicy()
	{

		try {
			Transformer transformer = TransformerFactory.newInstance().newTransformer();
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");

			StreamResult result = new StreamResult(new File(outputPolicyFile));
			DOMSource source = new DOMSource(securityPolicyDoc);
			transformer.transform(source, result);
		} catch (TransformerConfigurationException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (TransformerFactoryConfigurationError e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		}

	}

	public void saveUserPropertyFile(String userName, String password)
	{
		 StringBuffer contents = new StringBuffer();
   	     BufferedReader input = null;
   	     BufferedWriter output = null;
		 try
		 {
			 File userFile = new File(outputuserPropertyFile);
			 if (userFile.exists())
			 {
				 input = new BufferedReader( new FileReader(outputuserPropertyFile) );

			      String line = null;
			      while (( line = input.readLine()) != null)
			      {
			        contents.append(line).append(newline);
			      }
			 }

 		      contents.append(userName).append("=").append(password);

		      output = new BufferedWriter( new FileWriter(outputuserPropertyFile) );
		      output.write( contents.toString() );
		    }
		    catch (FileNotFoundException ex) {
		      ex.printStackTrace();
		    }
		    catch (IOException ex){
		      ex.printStackTrace();
		    }
		    finally {
		      try {
		        if (input!= null) {
		            input.close();
		        }
		        if (output != null) {
		            output.close();
		        }
		      }
		      catch (IOException ex) {
		        ex.printStackTrace();
		      }
		    }


	}

	private String appendNewIdentity(String userName, String realmId, String realmRoleId)
	{
		Node identities = securityPolicyDoc.getElementsByTagName(ELEM_IDENTITIES).item(0);
		NodeList identityList = identities.getChildNodes();
		String authId = findAvailableId(identityList, ELEM_AUTH_ID);
		Node newIdentity = createIdentityNode(identityList, realmId, realmRoleId, authId, userName);
		identities.appendChild(newIdentity);
		return authId;
	}

	/**
	 * Creates a anew Identity Node for the username, authId, realmId and realmRoleId
	 *
	 * @param identityList
	 * @param realmId
	 * @param realmRoleId
	 * @param authId
	 * @param userName
	 * @return  Identity Node that can be appended as a child of identities in mock_data.xml
	 */
	private Node createIdentityNode(NodeList identityList, String realmId,
					String realmRoleId, String authId, String userName)
	{
		for (int i=0; i < identityList.getLength(); i++)
		{
			Node node = identityList.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE)
			{
				Node newNode = node.cloneNode(true);
				NodeList childList = newNode.getChildNodes();
				for  (int j = 0; j < childList.getLength(); j++ )
				{
					Node atrnode = childList.item(j);
					if (atrnode.getNodeType() != Node.ELEMENT_NODE)
						continue;

					if (atrnode.getNodeName().equals(ELEM_AUTH_ID))
					{
						atrnode.setTextContent(authId);
					}
					else if (atrnode.getNodeName().equals(ELEM_LOGIN_NAME))
					{
						atrnode.setTextContent(userName);
					}
					else if (atrnode.getNodeName().equals(ELEM_IDENTITY_REALM_INFO))
					{
						NodeList IdRealmList = atrnode.getChildNodes();
						for (int k = 0; k < IdRealmList.getLength(); k++)
						{
							Node idNode = IdRealmList.item(k);
							if (idNode.getNodeName().equals(ELEM_REALM_ID))
							{
								idNode.setTextContent(realmId);
							}
							else if (idNode.getNodeName().equals(ELEM_REALM_ROLE_ID))
							{
								idNode.setTextContent(realmRoleId);
							}
						}
					}
				}
				return newNode;
			}
		}
		return null;
	}

	private String appendNewRealmRole(String realmId)
	{
		NodeList realmRoleList = securityPolicyDoc.getElementsByTagName(ELEM_REALM_ROLES);
		Node realmRole = realmRoleList.item(0);
		NodeList realmRoleChild = realmRole.getChildNodes();
		String realmRoleId = findAvailableId(realmRoleChild, ELEM_REALM_ROLE_ID);
		Node newRealmRole = createRealmNode(realmRoleChild, realmId, realmRoleId);
		realmRole.appendChild(newRealmRole);

		return realmRoleId;
	}



	private String appendNewRealm()
	{
		NodeList realmsNodeList = securityPolicyDoc.getElementsByTagName(ELEM_REALMS);
		Node realmsNode =  realmsNodeList.item(0);
		NodeList realmList = realmsNode.getChildNodes();
		String realmId = findAvailableId(realmList, ELEM_REALM_ID);
		Node newRealm = createRealmNode(realmList, realmId, null);

		realmsNode.appendChild(newRealm);

		return realmId;
	}

	private Node createRealmNode(NodeList nodeList, String realmId,
							String realmRoleId)
	{
		for (int i=0; i < nodeList.getLength(); i++)
		{
			Node node = nodeList.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE)
			{
				Node newNode = node.cloneNode(true);
				NodeList childList = newNode.getChildNodes();
				for  (int j = 0; j < childList.getLength(); j++ )
				{
					Node atrnode = childList.item(j);
					if (atrnode.getNodeType() != Node.ELEMENT_NODE)
						continue;

					if (atrnode.getNodeName().equals(ELEM_REALM_ID))
					{
						atrnode.setTextContent(realmId);
						if (realmRoleId == null) break;
					}
					else if (atrnode.getNodeName().equals(ELEM_REALM_ROLE_ID))
					{
						atrnode.setTextContent(realmRoleId);
					}
				}
				return newNode;
			}
		}
		return null;
	}

	private String findAvailableId(NodeList nodeList, String attribute)
	{
		int max = 0;
		for (int i=0; i < nodeList.getLength(); i++)
		{
			Node node = nodeList.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE)
			{
				NodeList map = node.getChildNodes();
				for (int j = 0; j < map.getLength(); j++)
				{
					Node n = map.item(j);
					if (n.getNodeName().equals(attribute))
					{
						int val = Integer.valueOf(n.getTextContent());
						if (max < val)
							max = val;

						break;
					}
				}
			}
		}
		max = max+1;
		return String.valueOf(max);
	}



}
