package com.intuit.sb.payroll.performance.tools;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * this class returns an array of users with corresponding realms, auth IDs,  configured, currently, 
 * in mock_data.xml
 */
public class RealmsReader {
	public static final String ELEM_REALM_ID = "Realm_ID";
	private static final String ELEM_IDENTITY_REALM_INFO = "IdentityRealmInfo";
	private static final String ELEM_LOGIN_NAME = "LoginName";
	public static final String ELEM_AUTH_ID = "Auth_ID";
	private static final String ELEM_IDENTITIES = "Identities";
	
	private Document     securityPolicyDoc = null;
	
	public RealmsReader(String secPolicyFile) 
	{
		try {
			
			securityPolicyDoc = loadSecurityPolicy(secPolicyFile);
			
		} catch (Exception e) {
			
			e.printStackTrace();
		}
	}

	/**
	 * Returns the Document representation for the mock_data.xml file
	 * @param fileName
	 * @return org.dom.Document for the xml file
	 * @throws Exception
	 */
	
	private Document loadSecurityPolicy(String fileName) throws Exception
	{
		if (fileName == null)
		{
			throw new Exception("File name is null");
		}
		File xmlFile = new File(fileName);
		 
        try 
        {
            // Get Document Builder Factory
            DocumentBuilderFactory factory = 
            				DocumentBuilderFactory.newInstance();

            // Obtain a document builder object
            DocumentBuilder builder = factory.newDocumentBuilder();
        
            // Parse the document
            Document doc = builder.parse(xmlFile);
            
            return doc;
		}catch (Exception e) 
		{
			throw e;
		}
	}

	/**
	 * Get usernames(LoginName) together with corresponding auth_ids and realm_ids from Identities node
	 * in mock_data.xml
	 */
	public List<RealmUser> getIdentities() 
	{
		List<RealmUser> userList = new ArrayList<RealmUser>();
		
		//Get Identities Node in mock_data.xml
		Node identities = securityPolicyDoc.getElementsByTagName(ELEM_IDENTITIES).item(0);
		NodeList identityList = identities.getChildNodes();
		for (int i=0; i < identityList.getLength(); i++)
		{
		    String loginName = null;
		    String authId = null;
		    String realmId = null;
		    
			Node node = identityList.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE)
			{
				NodeList map = node.getChildNodes();
				for (int j = 0; j < map.getLength(); j++)
				{ 
					Node n = map.item(j);
					if (n.getNodeName().equals(ELEM_LOGIN_NAME))
					{
						loginName = n.getTextContent();
						
					}
					if (n.getNodeName().equals(ELEM_AUTH_ID))
					{
						authId = n.getTextContent();
						
					}
					if (n.getNodeName().equals(ELEM_IDENTITY_REALM_INFO))
					{
						NodeList realmMap = n.getChildNodes();
						for ( int k = 0; k < realmMap.getLength(); k++ )
						{
							Node realmNode = realmMap.item(k);
							if ( realmNode.getNodeName() == ELEM_REALM_ID )
							{
								realmId = realmNode.getTextContent();
							}
						}
						
					}
				}
			}
			if ( loginName != null)
			{
				RealmUser userInfo = new RealmUser();
				userInfo.setLoginName(loginName);
				userInfo.setAuthId(authId);
				userInfo.setRealmId(realmId);
				userList.add(userInfo);
			}
		}
		
		return userList;
	}
}
