package com.intuit.sb.payroll.util;

import java.text.MessageFormat;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.common.ExceptionUtils;

/**
 * Locates objects in the directory. This is an implementation of the standard JEE
 * service locator pattern. Interestingly, even with EJB3 dependency injection
 * we still find ourselves directly accessing JNDI and so still need this pattern.
 * <p>
 * This is a NON-CACHING implementation at this time. We'll likely need to revisit
 * that in order to improve performance later.
 */
public class RemoteServiceLocator
{
    static private Logger logger = Logger.getLogger(RemoteServiceLocator.class);

    static private InitialContext initialContext;
	static private final RemoteServiceLocator serviceLocator = new RemoteServiceLocator();;

	private static Properties env;
	
	
	/*
	 * Private construction for singleton
	 */
	private RemoteServiceLocator()
	{
		try
		{
			// Get the default JNDI context as configured in jndi.properties.
			// If a remote context is required it should be configured in jndi.properties.
			  env = new Properties();
			  env.setProperty(Context.SECURITY_PRINCIPAL, "paul");
			  env.setProperty(Context.SECURITY_CREDENTIALS, "paul");
			  env.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.security.jndi.JndiLoginInitialContextFactory");
			  initialContext = new InitialContext(env);
		}
		catch (NamingException ne)
		{
            String mesg = MessageFormat.format("Unable to access JNDI: {0}", new Object[] {ExceptionUtils.throwableMessageChain(ne)});
            logger.fatal(mesg);
            throw new RuntimeException(mesg);
		}
	}
	
	/**
	 * Gets the singleton {@code ServiceLocator}.
	 * @return the service locator.
	 */
	static public RemoteServiceLocator getInstance()
	{
		return serviceLocator;
	}
	
	/**
	 * Get the service from your own jndi binding, not <ear name>/EJB-NAME/remote or local
	 * @param <T>
	 * @param serviceName
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public <T> T getServiceFromJndi(String serviceName)
	{
		try
		{
			String lookupString = serviceName;
			Object o = initialContext.lookup(lookupString);
			T service = (T)o; 
			return service;
		}
		catch (NamingException ne)
		{
            String mesg = MessageFormat.format("Unable to retrieve service ''{0}'' from JNDI: {1}",
            		new Object[] {serviceName, ExceptionUtils.throwableMessageChain(ne)});
            logger.error(mesg);
            throw new RuntimeException(mesg);
		}
	}

	public static Properties getEnv() {
		return env;
	}

	public static void setEnv(Properties env) throws Exception {
		RemoteServiceLocator.env = env;
		initialContext = new InitialContext(env);
	}
	
	public static InitialContext getInitialContext() {
		return initialContext;
	} 	

}
