/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.ScopedDepthFirstDecorator.java
 * 
 * Created: Oct 24, 2011 4:36:01 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query;

import com.intuit.sb.services.query.parser.*;

/**
 *
 * @author gkesler
 */
public final class ScopedDepthFirstDecorator extends DepthFirstDecorator {
    protected final ScopeStrategy strategy;
    
    public ScopedDepthFirstDecorator (SimpleQueryParserVisitor delegate) {
        this(delegate, defaultStrategy);
    }

    public ScopedDepthFirstDecorator (ScopedSimpleQueryVisitor delegate) {
        this(delegate, scopedStrategy);
    }

    private ScopedDepthFirstDecorator (SimpleQueryParserVisitor delegate, ScopeStrategy strategy) {
        super(delegate);
        
        this.strategy = strategy;
    } 
    
    @Override
    protected <T extends SimpleNode> Object visitSubtree(T node, Object data) {
        //return super.visitSubtree(node, data);
        Object localData = null;
        try {
            return localData = super.visitSubtree(node, data);
        } finally {
            // give the delegate a chance to close the scope by
            // calling its visit(SimpleNode, Object) or leave(SimpleNode, Object) method
            leaveSubtree(node, localData);
        }
    }
   
    private <T extends SimpleNode> void leaveSubtree (T node, Object data) {
        strategy.leave(delegate, node, data);
    }
    
    private interface ScopeStrategy<T extends SimpleQueryParserVisitor> {
        void leave (T delegate, SimpleNode node, Object data);
    }
    
    private static final ScopeStrategy defaultStrategy = 
            new ScopeStrategy<SimpleQueryParserVisitor>() {
        public void leave(SimpleQueryParserVisitor delegate, SimpleNode node, Object data) {
            // Note! it is assumed that under normal circumstances the SimpleQueryVisitors
            // reserve method Object visit (SimpleNode node, Object data) to do something special
            // since an AST shall not contain nodes, which are instances of SimpleNode
            delegate.visit(node, data);
        }
    };
    
    private static final ScopeStrategy scopedStrategy = 
            new ScopeStrategy<ScopedSimpleQueryVisitor>() {
        public void leave(ScopedSimpleQueryVisitor delegate, SimpleNode node, Object data) {
            delegate.leave(node, data);
        }
    };
}
