/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.validator.outer.BeanInfoProvider.java
 * 
 * Created: Nov 10, 2011 11:20:35 AM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

import com.intuit.sb.services.query.parser.ASTPathExpression;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 *
 * @author gkesler
 */
public class BeanInfoProvider implements MetadataProvider {
    private final BeanInfo beanInfo;
    private final Class contextType;
    private final Context context;
    private final MetadataProviderFactory factory;
    private final Map<String, Property> propertiesMap = new ConcurrentHashMap<String, Property>();

    public BeanInfoProvider (MetadataProviderFactory factory, Context context, Class contextType) {
        try {
            this.beanInfo = Introspector.getBeanInfo(contextType, Object.class);
            this.contextType = contextType;
            this.context = context;
            this.factory = factory;
        } catch (IntrospectionException e) {
            throw new ExceptionInInitializerError(e);
        }
    }
    
    public void init () {
        for (PropertyDescriptor desc: beanInfo.getPropertyDescriptors()) {
            Method readMethod = desc.getReadMethod();
            Property property = AbstractContext.createProperty(context, desc.getName(), desc.getPropertyType(), desc.getReadMethod(), true, true);
            propertiesMap.put(property.getName(), property);
        }
    }

    public String getName() {
        return beanInfo.getBeanDescriptor().getName();
    }

    public Class getType() {
        return beanInfo.getBeanDescriptor().getBeanClass();
    }

    public Property lookupProperty(String name) {
        return propertiesMap.get(Introspector.decapitalize(name));
    }

    public Map<String, Property> getAllProperties() {
        return propertiesMap;
    }

    public PathBuilder createPathBuilder() {
        return new PathBuilder() {
            Context currentContext = context;

            public void addPathElement(String name) {
                currentContext = currentContext.lookupProperty(name);
            }

            public void addAnyPathElement() {
                currentContext = AbstractContext.createWildcardProperty(currentContext, currentContext.getName());
            }

            public void addIdPathElement(String idValue) {
                currentContext = getCustomFieldContext().lookupIdProperty(idValue);
            }

            public void addNamePathElement(String nameValue) {
                currentContext = getCustomFieldContext().lookupNameProperty(nameValue);
            }

            public Property evaluate(ASTPathExpression path) {
                return (Property)currentContext;
            }
            
            private final Context getCustomFieldContext () {
                return currentContext.lookupProperty(CustomFieldProperty.CONTEXT_NAME);
            }
        };
    }

    public MetadataProviderFactory getFactory() {
        return factory;
    }
}
