/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.Context.java
 * 
 * Created: Jul 12, 2011 5:37:10 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

import java.util.Collection;

/**
 *
 * @author gkesler
 */
public interface Context {
    /**
     * Retrieves name for this context.
     * 
     * @return context name
     */
    String getName();
    /**
     * Retrieves all path elements of this context
     * 
     * @return an array of all path elements starting from root context
     */
    Context[] getFullPath ();
    /**
     * Retrieves root context for this path
     * 
     * @return root context
     */
    RootContext getRootContext ();
    /**
     * Retrieves immediate parent context for this path
     * 
     * @return parent context
     */
    Context getParentContext ();
    /**
     * Creates a path builder instance that evaluates (looks up) the path expression.
     * The result will contain either single-valued property descriptor
     * or a Context object in case of collection-valued one
     * 
     * @return path builder
     */
    PathBuilder createPathBuilder ();
    /**
     * Looks up a property that is an immediate child of
     * this context
     * 
     * @param name  property name to lookupProperty
     * @return corresponding property
     */
    Property lookupProperty (String name);
    /**
     * Looks up a property that is an immediate child of
     * this context
     * 
     * @param idValue  property name to lookupProperty
     * @return corresponding property
     */
    Property lookupIdProperty (String idValue);
    /**
     * Looks up a property that is an immediate child of
     * this context
     * 
     * @param nameValue  property name to lookupProperty
     * @return corresponding property
     */
    Property lookupNameProperty (String nameValue);
    /**
     * Obtains a collection of all properties defined in this context
     * 
     * @return collection of all properties
     */
    Collection<Property> getAllProperties ();
    
    /**
     * Strategy interface to be used when populating properties
     * collection in getAllProperties call
     */
    interface PropertyFilter {
        boolean accept (Property property);
    }
    
    /**
     * Sets current property filter to affect on the collection
     * of properties to be returned by getProperties call.
     * <code>null</code> value drops the filter.
     * 
     * @param filter current filter
     * 
     */
    void setPropertyFilter (PropertyFilter filter);
    /**
     * Obtains a collection of properties defined in this context
     * filtered according to the current PropertyFilter
     * 
     * @return collection of all properties
     */
    Collection<Property> getProperties ();
    /**
     * 
     * @return 
     */
    MetadataProvider getMetadataProvider ();
}
