/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.PersistentProperty.java
 * 
 * Created: Jul 14, 2011 2:30:01 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

/**
 *
 * @author gkesler
 */
public class PersistentProperty extends AbstractContext implements Property {
    protected final String propertyName;
    protected final boolean queryable;
    protected final boolean sortable;
    
    public PersistentProperty (Context parent, String propertyName, Class propertyType, boolean queryable, boolean sortable) {
        super(parent, propertyType, parent.getMetadataProvider().getFactory());
        
        this.propertyName = propertyName;
        this.queryable = queryable;
        this.sortable = sortable;
    }
    
    public Class getType() {
        return metadataProvider.getType();
    }

    public boolean validateValue(Object value) {
        // only NULL value is allowed for persistent property
        // if real value comparision is rrequired, the path
        // MUST evaluatePath to a single property
        return value == null; 
    }

    @Override
    public String getName() {
        return propertyName;
    }

    public <T> T accept(PropertyVisitor<T> visitor, T data) {
        return visitor.visit(this, data);
    }

    public boolean isQueryable() {
        return queryable;
    }

    public Property lookupIdProperty(String idValue) {
        throw generateException(idValue);
    }

    public Property lookupNameProperty(String nameValue) {
        throw generateException(nameValue);
    }

	public boolean isSortable() {
		return sortable;
	}
}
