/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.Property.java
 * 
 * Created: Jul 12, 2011 5:44:38 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

/**
 *
 * @author gkesler
 */
public interface Property extends Context {
    /**
     * Obtains type of this property, i.e. java.lang.String, java.util.Date, etc.
     * 
     * @return type of property
     */
    Class getType ();
    /**
     * Validates the provided value to have or to be cast/converted to the type
     * of this property
     * 
     * @param value value to validate
     * @return <code>true</code> if the provided value is compatible with this
     *      property type
     */
    boolean validateValue (Object value);
    /**
     * Returns <code>true</code> if this property can be queried
     * 
     * @return queryable status
     */
    boolean isQueryable ();
    /**
     * Returns <code>true</code> if this property can be used for sorting
     * 
     * @return sortable status
     */
    boolean isSortable ();
    /**
     * 
     * @param <T>
     * @param visitor
     * @param data
     * @return 
     */
    <T> T accept (PropertyVisitor<T> visitor, T data);
}
