/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.SimpleQueryValidatorTest.java
 * 
 * Created: Jul 14, 2011 11:47:20 AM
 * Author: gkesler
 */
package com.intuit.sb.services.query;

import java.util.HashMap;
import java.util.Map;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.intuit.sb.services.query.validator.ValidatorException;
import com.intuit.sb.services.query.validator.context.ContextFactory;

/**
 *
 * @author gkesler
 */
public class SimpleQueryValidatorTest {
    
    public SimpleQueryValidatorTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
    
    private Map<String, Object> createProperties() {
        return new HashMap<String, Object>() {
            {
                put(ContextFactory.CONTEXT_USER_ID, System.getProperty("user.name"));
                put(ContextFactory.CONTEXT_APP_ID, "SimpleQuery-console");
                put(ContextFactory.CONTEXT_PREFIX, "com.intuit.schema.finance.v3");
                put(ContextFactory.PAGE_MAX_LIMIT, Integer.valueOf(1024));
            }
        };
    }
    
    private final void validatorTest (String resourceName) throws Exception {
        System.out.println("resourceName=" + resourceName);
        SimpleQuery parser = Utils.parse(resourceName, createProperties());
        parser.validate();
        
        DumpVisitor.dump(parser.getRootNode(), "-->");
    }
    
    // TODO add test methods here.
    // The methods must be annotated with annotation @Test. For example:
   
    @Test
    public void testAggregateValidator () throws Exception {
        System.out.println("testAggregateValidator");
        validatorTest("/aggregate.sq");
    }
    
    @Test
    public void testBasicValidator () throws Exception {
        System.out.println("testBasicValidator");
        validatorTest("/basicCustomerQuery.sq");
    }
        
    @Test
    public void testValidator () throws Exception {
        System.out.println("testValidator");
        validatorTest("/customerQuery.sq");
    }
        
    @Test
    public void testValidatorWithCustomFields () throws Exception {
        System.out.println("testValidatorWithCustomFields");
        validatorTest("/customerQueryWithCustomFields.sq");
    }
    
    @Test(expected = ValidatorException.class)
    public void testNegativeValidatorProperty () throws Exception {
        System.out.println("testNegativeValidatorProperty");
        validatorTest("/invalidCustomerProperty.sq");
    }
    
    @Test(expected = ValidatorException.class)
    public void testNegativeValidatorValue () throws Exception {
        System.out.println("testNegativeValidatorValue");
        validatorTest("/invalidPropertyValue.sq");
    }
    
    @Test(expected = ValidatorException.class)
    public void testNegativeValidatorComposite () throws Exception {
        System.out.println("testNegativeValidatorComposite");
        validatorTest("/invalidCompositeExpression.sq");
    }
}
