/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.test.SimpleQueryResource.java
 * 
 * Created: Dec 5, 2011 1:42:44 PM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.test;

import com.intuit.sb.datasync.query.ws.rs.resteasy.SyntaxMode;
import com.intuit.sb.services.query.SimpleQuery;
import com.intuit.sb.services.query.generator.SimpleQueryGeneratorDelegate;
import com.intuit.sb.services.query.generator.tuple.ExpressionTuple;
import com.intuit.sb.services.query.generator.tuple.OrderByTuple;
import com.intuit.sb.services.query.generator.tuple.PaginationTuple;
import com.intuit.sb.services.query.validator.context.Property;
import com.intuit.sb.services.query.validator.context.RootContext;
import java.util.ArrayList;
import java.util.List;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

/**
 *
 * @author gkesler
 */
@Path("/v2/my-realm-id")
public class SimpleQueryResource {
    @GET
    @Path("invoice")
    @Produces(MediaType.TEXT_PLAIN)
    public String getInvoiceQuery (@QueryParam("q") SimpleQuery query) {
        SimpleQueryGeneratorDelegate generator = createGenerator();
        query.generate(generator);
        return String.valueOf(generator);
    }
    
    @GET
    @Produces(MediaType.TEXT_PLAIN)
    public String getQuery (@QueryParam("q") @SyntaxMode(com.intuit.sb.services.query.SimpleQuery.SyntaxMode.EXTENDED) SimpleQuery query) {
        SimpleQueryGeneratorDelegate generator = createGenerator();
        query.generate(generator);
        return String.valueOf(generator);
    }
    
    @POST
    @Path("invoice")
    @Consumes(MediaType.TEXT_PLAIN)
    @Produces(MediaType.TEXT_PLAIN)
    public String postInvoiceQuery (SimpleQuery query) {
        SimpleQueryGeneratorDelegate generator = createGenerator();
        query.generate(generator);
        return String.valueOf(generator);
    }
    
    private SimpleQueryGeneratorDelegate createGenerator () {
        return new SimpleQueryGeneratorDelegate() {
            final List<RootContext> contexts = new ArrayList<RootContext>();
            final List<Property> projection = new ArrayList<Property>();
            final List<ExpressionTuple> filters = new ArrayList<ExpressionTuple>();
            final List<OrderByTuple> orderSpec = new ArrayList<OrderByTuple>();
            final List<PaginationTuple> paginationSpec = new ArrayList<PaginationTuple>();
            
            public void addRootContext(RootContext context) {
                contexts.add(context);
            }

            public void addToProjection(Property path) {
                projection.add(path);
            }

            public void addFilter(ExpressionTuple tuple) {
                filters.add(tuple);
            }

            public void addOrderSpec(OrderByTuple tuple) {
                orderSpec.add(tuple);
            }

            public void addPaginationItem(PaginationTuple tuple) {
                paginationSpec.add(tuple);
            }

            @Override
            public String toString() {
                return "{" + "contexts=" + contexts + 
                            ", projection=" + projection + 
                            ", filters=" + filters + 
                            ", orderSpec=" + orderSpec + 
                            ", paginationSpec=" + paginationSpec + 
                        '}';
            }
        };
    }
}
