/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.Class.java
 * 
 * Created: Nov 16, 2011 10:35:56 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query;

import com.intuit.sb.services.query.generator.SimpleQueryGeneratorDelegate;
import com.intuit.sb.services.query.generator.tuple.AndTuple;
import com.intuit.sb.services.query.generator.tuple.AscTuple;
import com.intuit.sb.services.query.generator.tuple.BetweenTuple;
import com.intuit.sb.services.query.generator.tuple.BinaryTuple;
import com.intuit.sb.services.query.generator.tuple.ConditionalTuple;
import com.intuit.sb.services.query.generator.tuple.ContainsTuple;
import com.intuit.sb.services.query.generator.tuple.DescTuple;
import com.intuit.sb.services.query.generator.tuple.EqTuple;
import com.intuit.sb.services.query.generator.tuple.ExpressionTuple;
import com.intuit.sb.services.query.generator.tuple.FilterTuple;
import com.intuit.sb.services.query.generator.tuple.GtTuple;
import com.intuit.sb.services.query.generator.tuple.GteTuple;
import com.intuit.sb.services.query.generator.tuple.InTuple;
import com.intuit.sb.services.query.generator.tuple.IteratorTuple;
import com.intuit.sb.services.query.generator.tuple.LikeTuple;
import com.intuit.sb.services.query.generator.tuple.MaxResultsTuple;
import com.intuit.sb.services.query.generator.tuple.LtTuple;
import com.intuit.sb.services.query.generator.tuple.LteTuple;
import com.intuit.sb.services.query.generator.tuple.NEqTuple;
import com.intuit.sb.services.query.generator.tuple.NotTuple;
import com.intuit.sb.services.query.generator.tuple.OrTuple;
import com.intuit.sb.services.query.generator.tuple.OrderByTuple;
import com.intuit.sb.services.query.generator.tuple.PaginationTuple;
import com.intuit.sb.services.query.generator.tuple.StartPositionTuple;
import com.intuit.sb.services.query.generator.tuple.TupleVisitor;
import com.intuit.sb.services.query.generator.tuple.UnaryTuple;
import com.intuit.sb.services.query.validator.context.Property;
import com.intuit.sb.services.query.validator.context.RootContext;

/** 
 *
 * @author gkesler
 */
public final class DumpGeneratorDelegate implements SimpleQueryGeneratorDelegate {
    public void addRootContext(RootContext context) {
        System.out.println("generated --> RootContext: " + context.getName());
    }

    public void addFilter(ExpressionTuple tuple) {
        System.out.println("generated --> addFilter: " + tuple);
        tuple.accept(expressionDump, Integer.valueOf(2));
    }

    public void addOrderSpec(OrderByTuple tuple) {
        System.out.println("generated --> addOrderSpec: " + tuple);
    }

    public void addPaginationItem(PaginationTuple tuple) {
        System.out.println("generated --> addPaginationItem: " + tuple);
    }

    public void addToProjection(Property path) {
        System.out.println("generated --> addToProjection: " + path);
    }
    private final TupleVisitor<Integer> expressionDump = new TupleVisitor<Integer>() {

        private final Integer visitBinary(BinaryTuple tuple, Integer data, String name) {
            traceChangeOfOrder(tuple, tuple.getLeftOperand(), data);
            return visitUnary(tuple, data, name);
        }

        private final Integer visitUnary(UnaryTuple tuple, Integer data, String name) {
            traceIndent(data.intValue(), name);
            traceChangeOfOrder(tuple, tuple.getRightOperand(), data);
            return data;
        }

        private final void traceChangeOfOrder(ConditionalTuple condition, ExpressionTuple operand, Integer data) {
            final boolean changeOfOrder = operand.getPrecedence().compareTo(condition.getPrecedence()) < 0;
            int indent = data.intValue();
            if (changeOfOrder) {
                traceIndent(indent++, "(");
            }
            operand.accept(this, Integer.valueOf(indent));
            if (changeOfOrder) {
                traceIndent(--indent, ")");
            }
        }

        private final Integer visitFilter(FilterTuple tuple, Integer indent) {
            traceIndent(indent.intValue(), String.valueOf(tuple));
            return indent;
        }

        private final void traceIndent(int indent, String s) {
            StringBuilder builder = new StringBuilder("generated --> ");
            while (indent-- > 0) {
                builder.append(' ');
            }
            builder.append(s);
            ASTUtils.trace(builder.toString());
        }

        public Integer visit(AndTuple tuple, Integer data) {
            return visitBinary(tuple, Integer.valueOf(data), "AND");
        }

        public Integer visit(AscTuple tuple, Integer data) {
            throw new UnsupportedOperationException("Not supported yet.");
        }

        public Integer visit(BetweenTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(ContainsTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(DescTuple tuple, Integer data) {
            throw new UnsupportedOperationException("Not supported yet.");
        }

        public Integer visit(EqTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(GtTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(GteTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(InTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(IteratorTuple tuple, Integer data) {
            throw new UnsupportedOperationException("Not supported yet.");
        }

        public Integer visit(LikeTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(MaxResultsTuple tuple, Integer data) {
            throw new UnsupportedOperationException("Not supported yet.");
        }

        public Integer visit(LtTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(LteTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(NEqTuple tuple, Integer data) {
            return visitFilter(tuple, data);
        }

        public Integer visit(NotTuple tuple, Integer data) {
            return visitUnary(tuple, data, "NOT");
        }

        public Integer visit(OrTuple tuple, Integer data) {
            return visitBinary(tuple, data, "OR");
        }

        public Integer visit(StartPositionTuple tuple, Integer data) {
            throw new UnsupportedOperationException("Not supported yet.");
        }
    };    
}
