/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.ws.rs.ContextMapContextResolver.java
 * 
 * Created: Nov 30, 2011 3:18:25 PM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.ws.rs;

import javax.servlet.ServletContext;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.ext.ContextResolver;
import javax.ws.rs.ext.Provider;
import org.apache.log4j.Logger;

/**
 *
 * @author gkesler
 */
@Provider
@Produces(MediaType.WILDCARD)
public class ContextMapContextResolver implements ContextResolver<ContextMap> {
    private static final Logger logger = Logger.getLogger(ContextMapContextResolver.class);

    @Context
    private ServletContext servletContext;
    
    private volatile ContextMap contextMap = null;
    
    public ContextMap getContext(Class<?> type) {
        if (type.isAssignableFrom(ContextMap.class)) {
            return getContextMap();
        }
        
        // not mine
        return null;
    }

    ContextMap getContextMap () {
        // Note! Double-checked locking has been fixed in Java 5
        ContextMap result = contextMap;
        if (result == null) {
            synchronized (this) {
                result = contextMap;
                if (result == null) {
                    result = contextMap = new ContextMap(servletContext);
                }
            }            
        }
        
        return result;
    }
    
    synchronized void setContextMap (ContextMap contextMap) {
        this.contextMap = contextMap;
    }
    
    ServletContext getServletContext() {
        return servletContext;
    }

    void setServletContext(ServletContext servletContext) {
        this.servletContext = servletContext;
    }    
}
