/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.rest.ConverterHelper.java
 * 
 * Created: Nov 30, 2011 12:36:16 AM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.ws.rs;

import java.io.IOException;
import java.io.Reader;
import java.lang.annotation.Annotation;
import java.util.Arrays;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.ext.ContextResolver;
import javax.ws.rs.ext.Providers;

import org.apache.log4j.Logger;

import com.intuit.sb.services.query.SimpleQuery;
import com.intuit.sb.services.query.SimpleQueryContext;
import com.intuit.sb.services.query.parser.ParseException;
import com.intuit.sb.services.query.validator.ValidatorException;


/**
 *
 * @author gkesler
 */
public final class ConverterHelper {
    private static final Logger logger = Logger.getLogger(ConverterHelper.class);
    
    private ConverterHelper () {
    }
    
    public static SimpleQuery readFrom (Reader stream, Providers providers, 
            Annotation[] annotations, MediaType mediaType) throws IOException, WebApplicationException {
        try {        	        	        	
        	
        	//Convert Stream to String for logging the request body
        	String s  = convertStreamToString(stream);
        	
        	logger.info("Request body: "+ s);
        	
        	if(s==null || s.equals("")) return null;
        	
            ContextResolver<SimpleQueryContext> contextResolver = providers.getContextResolver(SimpleQueryContext.class, mediaType);
            logger.debug("contextResolver=" + contextResolver);

            SimpleQueryContext context = contextResolver.getContext(SimpleQueryContext.class);
            logger.debug("context=" + context);
            
            SimpleQuery result = context.newSimpleQuery(s);
            logger.debug("result=" + result);
            
            configure(result, annotations, mediaType);
            
            result.parse();
            result.validate();

            return result;
        } catch (InstantiationException e) {
            throw new AssertionError(e);
        } catch (IllegalAccessException e) {
            throw new AssertionError(e);
        } catch (ParseException e) {
            throw new WebApplicationException(e);
        } catch (ValidatorException e) {
            throw new WebApplicationException(e);
        }
    }
    
    private static void configure (SimpleQuery target, Annotation[] annotations, MediaType mediaType) 
            throws InstantiationException, IllegalAccessException {
        logger.debug("annotations=" + Arrays.toString(annotations));
        for (Annotation annotation: annotations) {
            Configurator configurator = annotation.annotationType().getAnnotation(Configurator.class);
            if (configurator != null) {
                logger.debug("configurator=" + configurator);
                ConfiguratorProcessor processor = configurator.value().newInstance();
                logger.debug("processor=" + processor);
                
                processor.configure(target, annotation, annotations, mediaType);
            }
        }
    }
    
    public static String convertStreamToString(Reader r) throws IOException { 
    	if(r!=null)
    	{
	        StringBuilder sb = new StringBuilder();
	        int i;      
	        
			while ((i =  r.read()) != -1) {
			  sb.append((char)i);
			}
			
	        return sb.toString();
    	}
    	
    	return null;
      }
}
