/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.rest.SimpleQueryContextResolver.java
 * 
 * Created: Nov 28, 2011 2:46:39 PM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.ws.rs;

import com.intuit.sb.services.query.SimpleQueryContext;
import com.intuit.sb.services.query.validator.context.ContextFactory;
import com.intuit.spc.bhp.serviceapi.authentication.IBHPPrincipal;
import java.util.Map;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.core.UriInfo;
import javax.ws.rs.ext.ContextResolver;
import javax.ws.rs.ext.Provider;
import javax.ws.rs.ext.Providers;
import org.apache.log4j.Logger;
import org.jboss.resteasy.spi.NotFoundException;

/**
 *
 * @author gkesler
 */
@Provider
@Produces(MediaType.WILDCARD)
public class SimpleQueryContextResolver implements ContextResolver<SimpleQueryContext>, SimpleQueryConstants {
    private static final Logger logger = Logger.getLogger(SimpleQueryContextResolver.class);

    @Context
    private Providers providers;
    
    @Context
    private UriInfo uriInfo;
    
    @Context
    private SecurityContext securityContext;
    
    public SimpleQueryContext getContext(Class<?> type) {
        if (type.isAssignableFrom(SimpleQueryContext.class)) {
            logger.debug("getContext:" + type);
            final ContextMap contextMap = getContextMap();
            
            Map<String, Object> contextProperties = contextMap.getContextProps();
            logger.debug("contextProperties:" + contextProperties);

            ContextMap.ContextDesc contextDesc = contextMap.getContextDesc(uriInfo);
            logger.debug("contextDesc:" + contextDesc);
            
            if (contextDesc == null) {
            	// No context, resource not found
            	throw new NotFoundException("Resource not found", Response.serverError() 
                        .status(javax.ws.rs.core.Response.Status.NOT_FOUND) 
                        .entity("Not Found") 
                        .build());
            }
            contextProperties.put(ContextFactory.CONTEXT_PREFIX, contextDesc.prefix);
            contextProperties.put(ContextFactory.CONTEXT_NAME, contextDesc.name);
            
            final IBHPPrincipal userPrincipal = (IBHPPrincipal)securityContext.getUserPrincipal();
            if (userPrincipal != null) {
                contextProperties.put(ContextFactory.CONTEXT_USER_ID, userPrincipal.getIdentity().getLoginName());
                contextProperties.put(ContextFactory.CONTEXT_APP_ID, userPrincipal.getCurrentOfferingId());
            } else {
                logger.warn("FIXME: userPrincipal is not available");
            }

            return new SimpleQueryContext(contextProperties);
        }
        
        // not applicable for other type
        return null;
    }

    UriInfo getUriInfo() {
        return uriInfo;
    }

    void setUriInfo(UriInfo uriInfo) {
        this.uriInfo = uriInfo;
    }

    ContextMap getContextMap() {
        ContextResolver<ContextMap> contextResolver = 
                providers.getContextResolver(ContextMap.class, MediaType.WILDCARD_TYPE);
        return contextResolver.getContext(ContextMap.class);
    }
    
    SecurityContext getSecurityContext() {
        return securityContext;
    }

    void setSecurityContext(SecurityContext securityContext) {
        this.securityContext = securityContext;
    }

    Providers getProviders() {
        return providers;
    }

    void setProviders(Providers providers) {
        this.providers = providers;
    }
}
