/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.rest.SimpleQueryContextResolverTest.java
 * 
 * Created: Nov 28, 2011 6:34:09 PM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.ws.rs;

import com.intuit.sb.services.query.SimpleQueryContext;
import com.intuit.spc.bhp.serviceapi.authentication.IBHPPrincipal;
import com.intuit.spc.bhp.serviceapi.authentication.ILoginSourceName;
import com.intuit.spc.bhp.serviceapi.entity.ICredential;
import com.intuit.spc.bhp.serviceapi.entity.IIdentity;
import com.intuit.spc.bhp.serviceapi.entity.IIdentity.SecurityLevel;
import com.intuit.spc.bhp.serviceapi.entity.IIdentity.UserType;
import com.intuit.spc.bhp.serviceapi.entity.IdentityLifecycleStatus;
import com.intuit.spc.bhp.serviceapi.entity.OfferingRole;
import com.intuit.spc.bhp.serviceapi.entity.OfferingWrapper;
import com.intuit.spc.bhp.serviceapi.entity.Permission;
import com.intuit.spc.bhp.serviceapi.entity.RealmRole;
import java.io.InputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.security.Principal;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.servlet.RequestDispatcher;
import javax.servlet.Servlet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.PathSegment;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.core.UriInfo;
import javax.ws.rs.ext.ContextResolver;
import javax.ws.rs.ext.ExceptionMapper;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Providers;
import org.apache.log4j.BasicConfigurator;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author gkesler
 */
public class SimpleQueryContextResolverTest {
    private ServletContext servletContext;
    private UriInfo uriInfo;
    private SecurityContext securityContext;
    
    public SimpleQueryContextResolverTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
        BasicConfigurator.configure();
        
        servletContext = new ServletContextStub() {
            Map<String, String> initParams = new HashMap<String, String>() {
                {
                    put(SimpleQueryConstants.CLASS_BASES, "v2:com.intuit.sb.cdm.v2.CdmBase;v1:com.intuit.sb.cdm.basetypes.CdmBase");
                }
            };

            public String getInitParameter(String string) {
                return initParams.get(string);
            }
        };
        
        uriInfo = new UriInfoStub() {
            public String getPath() {
                return "/v2/12345678/invoice";
            }
        };
        
        securityContext = new SecurityContext() {
            public Principal getUserPrincipal() {
                return new IBHPPrincipalStub() {
                    public IIdentity getIdentity() {
                        return new IIdentityStub() {
                            public String getLoginName() {
                                return "gkesler@sbcglobal.net";
                            }
                        };
                    }

                    public String getCurrentOfferingId() {
                        return "Greg's test";
                    }
                };
            }

            public boolean isUserInRole(String string) {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public boolean isSecure() {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public String getAuthenticationScheme() {
                throw new UnsupportedOperationException("Not supported yet.");
            }
        };
    }
    
    @After
    public void tearDown() {
    }
    
    // TODO add test methods here.
    // The methods must be annotated with annotation @Test. For example:
    //
    @Test
    public void testScanContexts() {
        System.out.println("testScanContexts");
        
        System.setProperty(SimpleQueryConstants.CLASS_BASES, "v2:com.intuit.sb.cdm.v2.CdmBase;v1:com.intuit.sb.cdm.basetypes.CdmBase");
        ContextMapContextResolver contextResolver = new ContextMapContextResolver();
        contextResolver.setServletContext(servletContext);
        
        ContextMap contextMap = contextResolver.getContext(ContextMap.class);        
        Assert.assertEquals("Customer contexts", new ContextMap.ContextDesc("com.intuit.sb.cdm.v2.Customer"), contextMap.getContextDesc("/customer/v2/12345"));
        Assert.assertEquals("Invoice contexts", new ContextMap.ContextDesc("com.intuit.sb.cdm.v2.Invoice"), contextMap.getContextDesc("/v2/12345/invoice"));
        Assert.assertEquals("Item contexts", new ContextMap.ContextDesc("com.intuit.sb.cdm.v2.Item"), contextMap.getContextDesc("/v2/12345/item"));
    }
    
    @Test
    public void testGetContext() {
        System.out.println("testGetContext");
        
        SimpleQueryContextResolver contextResolver = new SimpleQueryContextResolver();
        contextResolver.setProviders(new Providers() {
            final Map<Class, ContextResolver> contextResolvers = new HashMap<Class, ContextResolver>();
            {
                ContextMapContextResolver o = new ContextMapContextResolver();
                o.setServletContext(servletContext);
                contextResolvers.put(ContextMap.class, o);
            }

            public <T> MessageBodyReader<T> getMessageBodyReader(Class<T> type, Type type1, Annotation[] antns, MediaType mt) {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public <T> MessageBodyWriter<T> getMessageBodyWriter(Class<T> type, Type type1, Annotation[] antns, MediaType mt) {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public <T extends Throwable> ExceptionMapper<T> getExceptionMapper(Class<T> type) {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public <T> ContextResolver<T> getContextResolver(Class<T> type, MediaType mt) {
                return contextResolvers.get(type);
            }
        });
        contextResolver.setUriInfo(uriInfo);
        contextResolver.setSecurityContext(securityContext);
        
        SimpleQueryContext context = contextResolver.getContext(SimpleQueryContext.class);
        Assert.assertNotNull(context);
        System.out.println(String.valueOf(context));
    }
}
