/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.rest.SimpleQueryResourceTest.java
 * 
 * Created: Nov 29, 2011 1:45:21 PM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.ws.rs;

import com.intuit.sb.datasync.query.ws.rs.resteasy.SimpleQueryParamUnmarshaller;
import com.intuit.sb.datasync.query.ws.rs.resteasy.SimpleQueryStringConverter;
import com.intuit.spc.bhp.serviceapi.authentication.IBHPPrincipal;
import com.intuit.spc.bhp.serviceapi.authentication.ILoginSourceName;
import com.intuit.spc.bhp.serviceapi.entity.ICredential;
import com.intuit.spc.bhp.serviceapi.entity.IIdentity;
import com.intuit.spc.bhp.serviceapi.entity.IdentityLifecycleStatus;
import com.intuit.spc.bhp.serviceapi.entity.OfferingRole;
import com.intuit.spc.bhp.serviceapi.entity.OfferingWrapper;
import com.intuit.spc.bhp.serviceapi.entity.Permission;
import com.intuit.spc.bhp.serviceapi.entity.RealmRole;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLEncoder;
import java.security.Principal;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;
import javax.servlet.RequestDispatcher;
import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.SecurityContext;
import junit.framework.Assert;
import org.apache.log4j.BasicConfigurator;
import org.jboss.resteasy.core.Dispatcher;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.jboss.resteasy.mock.*;
import org.jboss.resteasy.plugins.server.resourcefactory.POJOResourceFactory;

/**
 *
 * @author gkesler
 */
public class SimpleQueryResourceTest {
    private Dispatcher dispatcher;
    final Hashtable<String, String> params = new Hashtable<String, String>() {
        {
            put(SimpleQueryConstants.CLASS_BASES, "v2:com.intuit.sb.cdm.v2.CdmBase;v1:com.intuit.sb.cdm.basetypes.CdmBase");
        }
    };
    
    public SimpleQueryResourceTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
        BasicConfigurator.configure();
        
        dispatcher = MockDispatcherFactory.createDispatcher();
        
        // Contexts
        dispatcher.getDefaultContextObjects().put(ServletConfig.class, new ServletConfig() {
            public String getServletName() {
                return "Resteasy";
            }

            public ServletContext getServletContext() {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public String getInitParameter(String name) {
                return params.get(name);
            }

            public Enumeration getInitParameterNames() {
                return params.keys();
            }
        });
        dispatcher.getDefaultContextObjects().put(ServletContext.class, new ServletContextStub() {
            public String getInitParameter(String name) {
                return params.get(name);
            }

            public Enumeration getInitParameterNames() {
                return params.keys();
            }
        });
        dispatcher.getDefaultContextObjects().put(SecurityContext.class, new SecurityContext() {
            public Principal getUserPrincipal() {
                return new IBHPPrincipalStub() {
                    public IIdentity getIdentity() {
                        return new IIdentityStub() {
                            public String getLoginName() {
                                return "gkesler@sbcglobal.net";
                            }
                        };
                    }

                    public String getCurrentOfferingId() {
                        return "Greg's test";
                    }
                };
            }

            public boolean isUserInRole(String string) {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public boolean isSecure() {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public String getAuthenticationScheme() {
                throw new UnsupportedOperationException("Not supported yet.");
            }
        });
        
        // Providers
        dispatcher.getProviderFactory().addContextResolver(ContextMapContextResolver.class);
        dispatcher.getProviderFactory().addContextResolver(SimpleQueryContextResolver.class);
        dispatcher.getProviderFactory().addMessageBodyReader(SimpleQueryMessageBodyReader.class);
//        dispatcher.getProviderFactory().addStringConverter(SimpleQueryStringConverter.class);
        dispatcher.getProviderFactory().addStringParameterUnmarshaller(SimpleQueryParamUnmarshaller.class);
        
        // IMPORTANT! Resources are added last
        dispatcher.getRegistry().addResourceFactory(new POJOResourceFactory(SimpleQueryResource.class));
    }
    
    @After
    public void tearDown() {
    }
    
    // TODO add test methods here.
    // The methods must be annotated with annotation @Test. For example:
    //
    @Test
    public void testPostInvoiceQuery() throws URISyntaxException, IOException {
        System.out.print("testPostInvoiceQuery");
        
        final String invoiceQuery = 
                "SELECT Header.status, Header.customerId " +
                "WHERE Header.txnDate = '2011-07-18' " +
                "AND Header.totalAmt >= '1500.50' " +
                "ORDER BY Header.customerId ASC " +
                "MAXRESULTS 50";
        
        MockHttpRequest request = MockHttpRequest.post("/v2/my-realm-id/invoice");
        request.contentType(MediaType.TEXT_PLAIN_TYPE);
        request.content(invoiceQuery.getBytes());
        
        MockHttpResponse response = new MockHttpResponse();
        
        dispatcher.invoke(request, response);
        
        Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatus());
        
        String responseContent = response.getContentAsString();
        Assert.assertNotNull(responseContent != null);
        Assert.assertTrue(responseContent.length() > 0);
        
        System.out.println("Response:");
        System.out.println(responseContent);
        
        // Send one more time
        System.out.println("testPostInvoiceQuery #2");
        request = MockHttpRequest.post("/v2/my-realm-id/invoice");
        request.contentType(MediaType.TEXT_PLAIN_TYPE);
        request.content(invoiceQuery.getBytes());
        
        response = new MockHttpResponse();
        
        dispatcher.invoke(request, response);
    }
    
    @Test
    public void testGetInvoiceQuery() throws URISyntaxException, IOException {
        System.out.println("testGetInvoiceQuery");
        
        final String invoiceQuery = 
                "SELECT Header.status, Header.customerId " +
                "WHERE Header.txnDate = '2011-07-18' " +
                "AND Header.totalAmt >= '1500.50' " +
                "ORDER BY Header.customerId ASC " +
                "MAXRESULTS 50";
                       
        MockHttpRequest request = MockHttpRequest.get("/v2/my-realm-id/invoice?q=" + URLEncoder.encode(invoiceQuery, "US-ASCII"));        
        MockHttpResponse response = new MockHttpResponse();
        
        dispatcher.invoke(request, response);
        
        Assert.assertEquals(HttpServletResponse.SC_OK, response.getStatus());
        
        String responseContent = response.getContentAsString();
        Assert.assertNotNull(responseContent != null);
        Assert.assertTrue(responseContent.length() > 0);
        
        System.out.println("Response:");
        System.out.println(responseContent);
    }
}
