/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.ws.rs.UriTemplateTest.java
 * 
 * Created: Dec 1, 2011 4:51:40 PM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.ws.rs.utils;

import junit.framework.Assert;
import org.apache.log4j.BasicConfigurator;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author gkesler
 */
public class UriTemplateTest {
    
    public UriTemplateTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
        BasicConfigurator.configure();
    }
    
    @After
    public void tearDown() {
    }
    // TODO add test methods here.
    // The methods must be annotated with annotation @Test. For example:
    //
    
    private UriTemplate compile (String template) throws ParseException {
        UriTemplate uriTemplate = UriTemplate.compile(template);
//        ((SimpleNode)uriTemplate.getRootNode()).dump("-->");
        System.out.println("AST:");
        DumpVisitor.dump(uriTemplate, "-->");
        
        System.out.println("Pattern:" + uriTemplate.getPatern());
        System.out.println("Rules:");
        for (UriTemplate.Rule rule: uriTemplate.getRules()) {
            System.out.println(rule);
        }
        
        return uriTemplate;
    }
    
    @Test
    public void testSimpleRule() throws Exception {
        System.out.println("testSimpleRule");
        
        final String refactor = "{version: v.*}/{realmId}/{entity}";   
        UriTemplate pattern = compile(refactor);
        
        UriTemplate.Match match = pattern.matches("/v2/12345/invoice");
        assertNotNull(match);
        
        assertEquals("v2", match.matcher.group(match.rule.groups.get("version")));
        assertEquals("invoice", match.matcher.group(match.rule.groups.get("entity")));
    }
    
    @Test
    public void testSimpleRuleLeadingSlash() throws Exception {
        System.out.println("testSimpleRuleLeadingSlash");
        
        final String refactor = "/{version: v.*}/{realmId}/{entity}";        
        compile(refactor);
    }
    
    @Test
    public void testSimpleRuleTrailingSlash() throws Exception {
        System.out.println("testSimpleRuleTrailingSlash");
        
        final String refactor = "{version: v.*}/{realmId}/{entity}/";        
        compile(refactor);
    }
    
    @Test
    public void testCompositeRule() throws Exception {
        System.out.println("testCompositeRule");
        
        final String refactor = "{version: v.*}/{realmId}/{entity}|{entity}/{realmId}/{version: v.*}";
        UriTemplate template = compile(refactor);
        
        UriTemplate.Match match = template.matches("/customer/12345/v2");
        assertNotNull(match);
        assertEquals("customer", match.group("entity"));
        assertEquals("v2", match.group("version"));
    }
    
    @Test
    public void testMixedSegment() throws Exception {
        System.out.println("testMixedSegment");
        
        final String refactor = "v2/aa{realmId}bb{entity}/my-realm-id/invoice";
        compile(refactor);
    }
    
    @Test
    public void testMatrixParam() throws Exception {
        System.out.println("testMatrixParam");
        
        final String refactor = "library/book;name=EJB 3.0;author=Bill Burke";
        compile(refactor);
    }
    
    @Test
    public void testIncompletePath() throws Exception {
        System.out.println("testIncompletePath");
        
        final String refactor = "/{version: v.*}/{realmId}/{entity}|/{version: v.*}/{realmId}";
        UriTemplate template = compile(refactor);
        
        UriTemplate.Match match = template.matches("/v2/123456/customer");
        assertNotNull(match);
        assertEquals("v2", match.group("version"));
        assertEquals("customer", match.group("entity"));
        
        match = template.matches("/v2/12345");
        assertNotNull(match);
        assertEquals("v2", match.group("version"));
        assertNull(match.group("entity"));
    } 
}
